<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('../include/miphpf/Init.php');
	require_once('../include/sc/Init.php');
	
	scAdminAuth::singleton()->checkAuthorization();
	
	class scAdministratorHomeView extends miView {
		/**
		 * Chance in percents to run the garbage collector
		 */
		const GC_PROBABILITY = 5;
		
		/**
		 * Remove files after 30 days
		 */
		const STALE_SECONDS = 2592000;
		
		const RECURSIVE_DELETE_REMOVE_TEMP = false;
		const RECURSIVE_DELETE_FAILURE = 0;
		const RECURSIVE_DELETE_ALL_DELETED = 1;
		const RECURSIVE_DELETE_SOME_DELETED = 2;
		
		public function getControllerCommands()
		{
			// Randomly, with specified probablity, remove old temp files
			if (rand(0, 100) < self::GC_PROBABILITY) {
				register_shutdown_function(array($this, 'collectGarbage'));
			}
			
			$t = new scTemplateParser();
			$t->readTemplate('admin/index.tmpl');
			$t->addSectionInfo($this->getDashboardSection());
			$this->assignMessages($t);
			$html = $t->templateParse();
			
			return array(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html));
		}
		
		/**
		 * Removes stale files
		 */
		public function collectGarbage()
		{
			$dir = scPaths::getFilenamePath('', scPaths::RESOURCE_TEMP_IMAGE);
			$files = scandir($dir);
			foreach ($files as $file)
				$this->removeStaleFile($dir . $file);
			
			$dir = scPaths::getFilenamePath(scPaths::EGOODS_TEMP, scPaths::RESOURCE_BASE_EGOOD);
			self::recursiveDeleteStaleFiles($dir, self::RECURSIVE_DELETE_REMOVE_TEMP);
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		protected function getDashboardSection()
		{
			$section = new miTemplateParserSectionInfo('dashboard', 1);
			
			$columnsCount = 2;
			
			$columns = new miTemplateParserSectionInfo('columns', $columnsCount);
			$section->addSubsection($columns);
			
			$columnBoxes = array();
			for ($columnNumber = 1; $columnNumber <= $columnsCount; $columnNumber++) {
				$boxes = $this->getDashboarBoxesForColumn($columnNumber);
				
				$boxesHtml = array();
				foreach ($boxes as $box) {
					$boxesHtml[] = $box->getDashboardBoxHtml();
				}
				$values = array('%%HTML_BOX%%' => $boxesHtml);
				$columnBoxes[] = new miTemplateParserSectionInfo('boxes', count($boxesHtml), $values);
			}
			$columns->addSubsectionsArray('boxes', $columnBoxes);
			
			return $section;
		}
		
		/**
		 * return IDashboardBox[]
		 */
		protected function getDashboarBoxesForColumn($column)
		{
			if ($column == 1) {
				return array(
					IDashboardBox::DASHBOARD_BOX_PRODUCT_REPORT => new scDashboardBoxProductReport(),
					IDashboardBox::DASHBOARD_BOX_CATEGORY_REPORT => new scDashboardBoxCategoryReport(),
				);
			}
			if ($column == 2) {
				return array(
					IDashboardBox::DASHBOARD_BOX_RECENT_ORDERS => new scDashboardBoxRecentOrders(),
					IDashboardBox::DASHBOARD_BOX_VIEWS_VISITS_REPORT => new scDashboardBoxViewsVisitsReport(),
				);
			}
			return array();
		}
		
		/**
		 * Removes temp file if older than self::STALE_SECONDS, which is 30 days
		 * 
		 * @param string $file
		 */
		protected function removeStaleFile($file)
		{
			if (!is_file($file))
				return;
			
			if (@filemtime($file) < (time() - self::STALE_SECONDS))
				@unlink($file);
		}
		
		
		/**
		 * Walks the passed $dir recursively and deletes stale files.
		 * @param string $dir
		 * @param bool $removePassedDir Should $dir be removed as well
		 * @return int
		 */
		protected static function recursiveDeleteStaleFiles($dir, $removePassedDir = true)
	    {
	    	if (!is_dir($dir)) {
				if (@filemtime($dir) >= (time() - self::STALE_SECONDS)) {
					return self::RECURSIVE_DELETE_SOME_DELETED; // Success. No need to delete this file.
				}
				if (unlink($dir)) {
					return self::RECURSIVE_DELETE_ALL_DELETED; // Success. File was deleted.
				}
				return self::RECURSIVE_DELETE_FAILURE;// Failure.
			}
			
			$deletedItems = self::RECURSIVE_DELETE_ALL_DELETED;
			$d = dir($dir);
			while (false !== ($entry = $d->read())) {
				if (in_array($entry, array('.', '..', '.svn'))) {
					continue;
				}
				
				$fullEntry = $dir . '/' . $entry;
				$result = self::recursiveDeleteStaleFiles($fullEntry);
				if ($result == self::RECURSIVE_DELETE_ALL_DELETED) {
					continue;
				} else if ($result == self::RECURSIVE_DELETE_SOME_DELETED) {
					// We'll store "SOME DELETED" so we can return it later
					$deletedItems = self::RECURSIVE_DELETE_SOME_DELETED;
					continue; // The dir won't be removed as some of its contents are still in there.
				} else {
					return self::RECURSIVE_DELETE_FAILURE;
				}
			}
			$d->close();
			
			// We should remove $dir only if all files in it were deleted.
			if ($removePassedDir && ($deletedItems == self::RECURSIVE_DELETE_ALL_DELETED)) {
				rmdir($dir);
			}
			return $deletedItems;
	    }
		
	    protected function assignMessages($t)
		{
			$problems = array();
			
			$messages = array();
			scInstallChecks::checkSystemRequirements($messages);
			if (!empty($messages))
				$problems[] = _t('System requirements');
			
			$messages = array();
			scInstallChecks::checkLicense($messages);
			if (!empty($messages))
				$problems[] = _t('License');
			
			$messages = array();
			scInstallChecks::checkDirectoryPermissions($messages);
			if (!empty($messages))
				$problems[] = _t('Directory permissions');
			
			$count = empty($problems) ? 0 : 1;
			$section = new miTemplateParserSectionInfo('messages', $count,
				array('%%PROBLEMS%%' => miI18N::htmlEscape(implode(', ', $problems))));
			$t->addSectionInfo($section);
		}
	}
	
	$page = new scAdminDisplayController(new scAdministratorHomeView);
	$page->showPage();
?>