<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	abstract class scSoapApiServiceImpl {
		protected $_localSyncInfo = null;
		protected $_cmsLayoutId = null;
		protected $_apiAccountPrivs;
		
		abstract protected function setSyncId($operationResult, $operation);
		
		/**
		 * @return miDomainObject
		 */
		abstract protected function createDomainObject();
		
		public function doModifyOperation($entityName, $operations, $allowPartialSuccess)
		{
			if (!is_array($operations))
				$operations = array($operations);
			
			$result = array();
			$entityElement = $this->getEntityElement($entityName);
			$entityIdElement = $this->getEntityIdElement($entityName);
			$operationResultClass = $this->getOperationResultClass($entityName);
			
			foreach ($operations as $operation) {
				$operationName = $operation->Operation;
				$entity = $operation->{$entityElement};
				
				$operationResult = new $operationResultClass;
				$operationResult->Operation = $operationName;
				
				try {
					miStaticDBUtil::startTransaction();
					
					$this->setSyncId($operationResult, $operation);
					switch ($operationName) {
						case scSoapApiOperationType::OP_INSERT:
							$operationResult->{$entityIdElement} = $this->doInsert($entityName, $entity);
							break;
						
						case scSoapApiOperationType::OP_UPDATE:
							$operationResult->{$entityIdElement} = $this->doUpdate($entityName, $entity);
							break;
						
						case scSoapApiOperationType::OP_DELETE:
							$operationResult->{$entityIdElement} = $this->doDelete($entityName, $entity);
							break;
						
						default:
							throw new miException(_i('Invalid operation.'));
					}
					miStaticDBUtil::commit();
					
					$operationResult->OperationStatusCode = scSoapApiOperationStatusCode::SUCCESS;
				} catch (Exception $exception) {
					miStaticDBUtil::rollback();
					$operationResult->OperationStatusCode = scSoapApiOperationStatusCode::EXCEPTION;
					$operationResult->OperationStatusMessage = $exception->getMessage();
				}
				
				$result[] = $operationResult;
			}
			
			return $result;
		}
		
		/**
		 * Perform get operations
		 * @param $selector
		 * @return array
		 */
		public function doGetOperation($entityName, $selector)
		{
			$results = array();
			
			$entityIds = $selector->{$this->getEntityIdElement($entityName)};
			if (empty($entityIds))
				return array();
			
			// TODO: Check for non-existing ids
			$recordset = $this->getRecordset();
			$recordset->addFilter(new miSqlFilterIn($recordset->getPrimaryKey(), $entityIds));
			$records = $recordset->getRecords();
			
			$localIds = miUtil::getKeyValueArray($records, $recordset->getPrimaryKey(), $recordset->getPrimaryKey());
			$syncInfos = $this->getSyncImpl($entityName)->getSyncInfos($localIds);
			
			foreach ($records as $record) {
				$item = $this->getItemDataFromRecord($record);
				if (isset($syncInfos[$record[$recordset->getPrimaryKey()]])) {
					$item->{$entityName . 'SyncInfo'} = $syncInfos[$record[$recordset->getPrimaryKey()]];
				}
				$results[] = $item;
			}
			return $results;
		}
		
		public function setApiAccountPrivs($privs)
		{
			$this->_apiAccountPrivs = $privs;
		}
		
		protected function doInsert($entityName, $item)
		{
			$domainObject = $this->createDomainObject();
			$this->setItemDataToDomainObject($domainObject, $item, true);
			$itemId = $domainObject->insert();
			
			if (isset($item->{$entityName . 'SyncInfo'})) {
				$xmlForeignId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_FOREIGN_ID);
				$this->getSyncImpl($entityName)->insertSyncInfo($itemId, $item->{$entityName . 'SyncInfo'}->{$xmlForeignId}, $item->{$entityName . 'SyncInfo'}->HashCode);
			}
			return $itemId;
		}
		
		protected function doUpdate($entityName, $item)
		{
			$xmlLocalId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_LOCAL_ID);
			if (!isset($item->{$xmlLocalId}))
				throw new miException(sprintf(_i('%s is required.'), $xmlLocalId));
			
			$domainObject = $this->createDomainObject();
			$domainObject->read($item->{$xmlLocalId});
			$this->setItemDataToDomainObject($domainObject, $item, false);
			$itemId = $domainObject->update();
			
			if (isset($item->{$entityName . 'SyncInfo'})) {
				$xmlForeignId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_FOREIGN_ID);
				$this->getSyncImpl($entityName)->updateSyncInfo($itemId, $item->{$entityName . 'SyncInfo'}->{$xmlForeignId}, $item->{$entityName . 'SyncInfo'}->HashCode);
			}
			return $itemId;
		}
		
		protected function doDelete($entityName, $item)
		{
			$xmlLocalId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_LOCAL_ID);
			if (!isset($item->{$xmlLocalId}))
				throw new miException(sprintf(_i('%s is required.'), $xmlLocalId));
			
			$domainObject = $this->createDomainObject();
			$domainObject->delete($item->{$xmlLocalId});
			
			$this->getSyncImpl($entityName)->deleteSyncInfo($item->{$xmlLocalId});
			return $item->{$xmlLocalId};
		}
		
		/**
		 * Return the local id from a soap record key
		 * @return int
		 */
		public static function getLocalId($entityName, $itemKey)
		{
			$xmlLocalId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_LOCAL_ID);
			if (isset($itemKey->{$xmlLocalId}))
				return (int)$itemKey->{$xmlLocalId};
			
			$xmlForeignId = scSoapApiMapping::getEntityField($entityName, scSoapApiMapping::XML_FOREIGN_ID);
			if (isset($itemKey->{$xmlForeignId})) {
				// Get the local id from the sync id
				if ($itemKey->{$xmlForeignId} === '')
					return 0;
				
				$op = new scSoapApiSyncImpl($entityName);
				return $op->getLocalIdFromSync($itemKey->{$xmlForeignId});
			}
			
			return null;
		}
		
		/**
		 * Returns language id by code
		 * @param string $languageCode
		 * @return int
		 */
		protected function getLanguageByCode($languageCode)
		{
			static $allLanguageCodes = null;
			
			if ($allLanguageCodes === null)
				$allLanguageCodes = miUtil::getDBArray('SELECT LanguageID,LanguageCode FROM Languages', 'LanguageCode', 'LanguageID'); // FIXME: move to another class
			
			if (!isset($allLanguageCodes[$languageCode]))
				throw new miException(sprintf(_i('Unknown language code "%s".'), $languageCode));
			
			return $allLanguageCodes[$languageCode];
		}
		
		/**
		 * Returns the language code by id
		 * @param int $langId
		 * @return string
		 */
		protected function getLanguageCodeById($langId)
		{
			static $allLanguageCodes = null;
			
			if ($allLanguageCodes === null)
				$allLanguageCodes = miUtil::getDBArray('SELECT LanguageID,LanguageCode FROM Languages', 'LanguageID', 'LanguageCode'); // FIXME: move to another class
			
			if (!isset($allLanguageCodes[$langId]))
				throw new miException(sprintf(_i('Unknown language id "%s".'), $langId));
			
			return $allLanguageCodes[$langId];
		}
		
		/**
		 * Frees a sort position by moving the item that's taken it and all items after it one ahead.
		 * @param string $table
		 * @param string $sortColumn
		 * @param string|false $additonalClause
		 */
		protected function freeSort($table, $sortColumn, $sortValueToFree, $additionalClause = false)
		{
			// Check if the sort value is taken
			$sql = 'SELECT `' . sql_escape_string($sortColumn) . '` FROM `' . sql_escape_string($table) . '`';
			$sql .= ' WHERE (`' . sql_escape_string($sortColumn) . '` = ' . intval($sortValueToFree) . ')';
			if ($additionalClause) {
				$sql .= ' AND (' . $additionalClause  . ')';
			}
			$sql .= ' LIMIT 1';
			$rows = miStaticDBUtil::execSelect($sql);
			if (!isset($rows[0])) {
				return;
			}
			
			// It's taken - move every item forward
			$sql = 'UPDATE `' . sql_escape_string($table) . '` SET `' . sql_escape_string($sortColumn) . '` = `' . sql_escape_string($sortColumn) . '` + 1';
			$sql .= ' WHERE (`' . sql_escape_string($sortColumn) . '` >= ' . intval($sortValueToFree) . ')';
			if ($additionalClause) {
				$sql .= ' AND (' . $additionalClause  . ')';
			}
			$sql .= ' ORDER BY `' . sql_escape_string($sortColumn) . '` DESC';
			miStaticDBUtil::execSQL($sql);
		}
		
		/**
		 * Converts SOAP multi-language var to format suitable for scSqlRecordML
		 * 
		 * @param $soapObject
		 * @param string $name
		 * @return array
		 */
		protected function xmlToMl($soapObject, $name = false)
		{
			if ($name !== false) {
				if (!isset($soapObject->{$name}))
					return;
				$xml = $soapObject->{$name};
			} else
				$xml = $soapObject;
			
			if (!is_array($xml))
				$xml = array($xml);
				
			$var = array();
			foreach ($xml as $xmlRec) {
				$value = (string)$xmlRec->_;
				$lang = (string)$xmlRec->lang;
				
				$languageId = $this->getLanguageByCode($lang);
				if (isset($var[$languageId]))
					throw new miException(sprintf(_i('Duplicate language code "%s".'), $lang));
				
				$var[$languageId] = $value;
			}
			
			return $var;
		}
		
		/**
		 * Converts multi-language var from scSqlRecordML format to SOAP
		 * @param array $mlValues
		 * @return array
		 */
		protected function mlToXml($mlValues)
		{
			$result = array();
			foreach ($mlValues as $langId => $mlValue) {
				$result[] = array(
					'_' => $mlValue,
					'lang' => $this->getLanguageCodeById($langId)
				);
			}
			return $result;
		}
		
		/**
		 * Finds the next available sort value.
		 * @param string $table
		 * @param string $sortColumn
		 * @param string|false $additonalClause
		 * @return int
		 */
		protected function getNextSort($table, $sortColumn, $additonalClause = false)
		{
			$sql = 'SELECT MAX(' . $sortColumn . ') AS N FROM ' . $table;
			if ($additonalClause) {
				$sql .= ' WHERE ' . $additonalClause;
			}
			$rows = miStaticDBUtil::execSelect($sql);
			if (!isset($rows[0]) || !isset($rows[0]['N'])) {
				return 1;
			}
			return $rows[0]['N'] + 1;
		}
		
		/**
		 * Returns the default CMSLayoutID
		 * @return int
		 */
		protected function getDefaultCmsLayoutId()
		{
			if (is_null($this->_cmsLayoutId)) {
				$record = new miSqlRecord('CMSLayouts', 'CMSLayoutID');
				$record->read('CMSLayoutIsDefault', 1);
				$this->_cmsLayoutId = $record->getPK();
			}
			return $this->_cmsLayoutId;
		}
		
		/**
		 * @var scSoapApiImage $image
		 */
		protected function processImage($image)
		{
			if (!isset($image->ImageUri) && !isset($image->ImageEncoded)) {
				throw new miException(_i('No image to process: specify ImageUri or ImageEncoded.'));
			}
			
			if (isset($image->ImageUri) && isset($image->ImageEncoded)) {
				throw new miException(_i('Specify either ImageUri or ImageEncoded.'));
			}
			
			if (isset($image->ImageUri)) {
				if (!scValidatorHttp::isValid($image->ImageUri)) {
					throw new miException(sprintf(_i('Invalid url "%s".'), $image->ImageUri));
				}
				
				@$image = file_get_contents($image->ImageUri);
				if ($image === false) {
					throw new miException(sprintf(_i('Cannot read from location "%s".'), $image->ImageUri));
				}
			} else {
				$image = @base64_decode($image->ImageEncoded);
				if ($image === false) {
					throw new miException(_i('Invalid image base64-encoded content.'));
				}
			}
			
			$imageName = md5(uniqid(null, true));
			$imagePathFile = scPaths::getItemPath($imageName, scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_FILE);
			$imagePathWeb = scPaths::getItemPath($imageName, scPaths::RESOURCE_TEMP_IMAGE, scPaths::PATH_WEB);
			if (@file_put_contents($imagePathFile, $image) === false) {
				throw new miException(sprintf(_i('Cannot write file "%s".'), $imagePathFile));
			}
			
			return $imagePathWeb;
		}
		
		protected function getEntityElement($entityName)
		{
			$entity = scSoapApiMapping::getEntity($entityName);
			return $entity[scSoapApiMapping::XML_ELEMENT];
		}
		
		protected function getEntityIdElement($entityName)
		{
			$entity = scSoapApiMapping::getEntity($entityName);
			return $entity[scSoapApiMapping::XML_LOCAL_ID];
		}
		
		protected function getOperationResultClass($entityName)
		{
			$entity = scSoapApiMapping::getEntity($entityName);
			return $entity[scSoapApiMapping::CLASS_OPERATION_RESULT];
		}
		
		/**
		 * @return scSoapApiSyncImpl
		 */
		protected function getSyncImpl($entityName)
		{
			return new scSoapApiSyncImpl($entityName);
		}
	}
?>