<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSoapApiSyncImpl {
		
		protected $_entityName = null;
		
		protected $_syncInfoHashes = null;
		protected $_syncInfoLocalIds = null;
		
		public function __construct($entityName)
		{
			$this->_entityName = $entityName;
		}
		
		protected function getSyncElementId()
		{
			return scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::XML_FOREIGN_ID);
		}
		
		protected function getSyncItemHandler()
		{
			$entity = scSoapApiMapping::getEntity($this->_entityName);
			return $entity[scSoapApiMapping::HANDLER];
		}
		
		/**
		 * Remove sync info for non-existing records
		 */
		public function cleanupSyncInfo()
		{
			$table = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::TABLE_NAME);
			$tableId = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::TABLE_PK);
			$sql = 'DELETE SyncItems FROM SyncItems LEFT JOIN ' . $table . ' ON SyncItems.SyncItemLocalID=' . $table . '.' . $tableId .' WHERE ' .
				$table . '.' . $tableId . ' IS NULL AND SyncItems.SyncItemHandler=?';
			miStaticDBUtil::execSQL($sql, array($this->getSyncItemHandler()));
		}
		
		/**
		 * Creates scSyncTransactionInfo objects that will be inserted into the database
		 * 
		 * @param array $syncOperations
		 * @return scSyncTransactionInfo[]
		 */
		public function createSyncTransactionInfos($syncOperations)
		{
			if (!is_array($syncOperations))
				$syncOperations = array($syncOperations);
			$syncElementId = $this->getSyncElementId();
			$syncItemHandler = $this->getSyncItemHandler();
			
			$transactionInfos = array();
			foreach ($syncOperations as $syncOperation) {
				$transactionInfo = new scSyncTransactionInfo();
				$transactionInfo->set(scSyncTransactionInfo::COLUMN_HANDLER, $syncItemHandler);
				$transactionInfo->set(scSyncTransactionInfo::COLUMN_SYNC_ID, (string)$syncOperation->{$syncElementId});
				$transactionInfo->set(scSyncTransactionInfo::COLUMN_HASH, (string)$syncOperation->HashCode);
				$transactionInfos[] = $transactionInfo;
			}
			return $transactionInfos;
		}
		
		/**
		 * Return sync info result objects, which will be returned to the client
		 *
		 * @param array $syncOperations
		 * @return array
		 */
		public function createSyncInfoResults($syncOperations)
		{
			$results = array();
			foreach ($syncOperations as $syncOperation) {
				$operation = scSyncTransactionResultOperationType::getSoapApiOperationType($syncOperation['SyncTransactionResultOperation']);
				$localId = $syncOperation['SyncTransactionResultLocalID'];
				$foreignId = $syncOperation['SyncTransactionResultForeignID'];
				$hash = $syncOperation['SyncTransactionResultHashCode'];
				
				$results[] = $this->getSyncInfoResultObj($operation, $localId, $foreignId, $hash);
			}
			return $results;
		}
		
		/**
		 * Perform get sync operation
		 * @param array $syncOperations
		 * @param boolean $returnNonSynced
		 * @return array
		 */
		public function doGetSyncOperation($syncOperations, $returnNonSynced)
		{
			$this->cleanupSyncInfo();
			
			if (!is_array($syncOperations))
				$syncOperations = array($syncOperations);
			
			$results = array();
			
			$currentHashes = $this->getSyncInfoHashes();
			$localIds = $this->getSyncInfoLocalIds();
			$syncElementId = $this->getSyncElementId();
			
			$foundIds = array();
			foreach ($syncOperations as $syncOperation) {
				$syncId = (string)$syncOperation->{$syncElementId};
				$hashCode = (string)$syncOperation->HashCode;
				
				if (!isset($currentHashes[$syncId])) {
					// Unknown id - new item
					$results[] = $this->getSyncInfoResultObj(scSoapApiOperationType::OP_INSERT, null, $syncId, $hashCode);
					continue;
				}
				
				if ($currentHashes[$syncId] != $hashCode) {
					// Different hash - updated item
					$results[] = $this->getSyncInfoResultObj(scSoapApiOperationType::OP_UPDATE, $localIds[$syncId], $syncId, $currentHashes[$syncId]);
				}
				
				$foundIds[] = $syncId;
			}
			
			// Deleted sync ids are ids that we have in the db but were not passed to the service
			$deletedSyncIds = array_diff(array_keys($currentHashes), $foundIds);
			
			foreach ($deletedSyncIds as $deletedSyncId) {
				$results[] = $this->getSyncInfoResultObj(scSoapApiOperationType::OP_DELETE, $localIds[$deletedSyncId], $deletedSyncId, $currentHashes[$deletedSyncId]);
			}
			
			// Return items without sync info
			if ($returnNonSynced) {
				$results = array_merge($results, $this->getNotSyncedItems());
			}
			
			return $results;
		}
		
		public function getSyncInfos($localIds)
		{
			$handler = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::HANDLER);
			
			$recordset = new miSqlRecordset('SyncItems');
			$recordset->addFilter(new miSqlFilterEqual('SyncItemHandler', $handler));
			$recordset->addFilter(new miSqlFilterIn('SyncItemLocalID', $localIds));
			$recordset->getSelectQuery();
			$recordset = $recordset->getAllRecords();
			
			$syncInfos = array();
			$className = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::CLASS_SYNC_INFO);
			$foreignIdField = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::XML_FOREIGN_ID);
			foreach ($recordset as $row) {
				$syncInfo = new $className;
				$syncInfo->{$foreignIdField} = $row['SyncItemForeignID'];
				$syncInfo->HashCode = $row['SyncItemHashCode'];
				$syncInfos[$row['SyncItemLocalID']] = $syncInfo;
			}
			return $syncInfos;
		}
		
		/**
		 * Return items with no sync info
		 * 
		 * @return array
		 */
		protected function getNotSyncedItems()
		{
			$localItemIds = $this->getSyncInfoLocalIds();
			$table = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::TABLE_NAME);
			$primaryKey = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::TABLE_PK);
			$recordset = new miSqlRecordset($table);
			$recordset->addFilter(new miSqlFilterNotIn($primaryKey, $localItemIds));
			$recordset->setSelectFields(array($primaryKey));
			$records = $recordset->getAllRecords();
			
			$results = array();
			foreach ($records as $record) {
				$results[] = $this->getSyncInfoResultObj(scSoapApiOperationType::OP_DELETE, $record[$primaryKey], null, null);
			}
			
			return $results;
		}
		
		/**
		 * Retrieves the hashes for the current sync handler
		 * @return array array(ForeignID => HashCode)
		 */
		protected function getSyncInfoHashes()
		{
			if ($this->_syncInfoHashes == null)
				$this->readSyncInfoFromDb();
			return $this->_syncInfoHashes;
		}
		
		/**
		 * Returns the mapping between foreign id and local id
		 * @return array array(ForeignID => LocalID)
		 */
		protected function getSyncInfoLocalIds()
		{
			if ($this->_syncInfoHashes == null)
				$this->readSyncInfoFromDb();
			return $this->_syncInfoLocalIds;
		}
		
		
		/**
		 * Return sync info object to be returned to client
		 * 
		 * @param string $entity
		 * @param string $operation
		 * @param int $localId
		 * @param string $syncId
		 * @param string $hashCode
		 */
		protected function getSyncInfoResultObj($operation, $localId, $syncId, $hashCode)
		{
			$syncInfoResult = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::CLASS_SYNC_INFO_RESULT);
			$localIdElement = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::XML_LOCAL_ID);
			$foreignIdElement = scSoapApiMapping::getEntityField($this->_entityName, scSoapApiMapping::XML_FOREIGN_ID);
			
			$result = new $syncInfoResult;
			$result->Operation = $operation;
			if ($localId !== null)
				$result->{$localIdElement} = $localId;
			if ($syncId !== null)
				$result->{$foreignIdElement} = $syncId;
			if ($hashCode !== null)
				$result->HashCode = $hashCode;
			return $result;
		}
		
		/**
		 * Reads sync info from the database
		 */
		protected function readSyncInfoFromDb()
		{
			$recordset = new miSqlRecordset('SyncItems');
			$recordset->addFilter(new miSqlFilterEqual('SyncItemHandler', $this->getSyncItemHandler()));
			$recordset->setSelectFields(array('SyncItemForeignID', 'SyncItemLocalID', 'SyncItemHashCode'));
			$localSyncInfo = $recordset->getAllRecords();
			
			$this->_syncInfoHashes = miUtil::getKeyValueArray($localSyncInfo, 'SyncItemForeignID', 'SyncItemHashCode');
			$this->_syncInfoLocalIds = miUtil::getKeyValueArray($localSyncInfo, 'SyncItemForeignID', 'SyncItemLocalID');
		}
		
		/**
		 * Insert sync info
		 */
		public function insertSyncInfo($localId, $syncId, $hashCode)
		{
			// Remove old sync info
			$this->deleteSyncInfo($localId);
			miStaticDBUtil::execDelete('SyncItems', array('SyncItemHandler', 'SyncItemForeignID'), array($this->getSyncItemHandler(), $syncId));
			
			return miStaticDBUtil::execInsert('SyncItems', array(
				'SyncItemHandler' => $this->getSyncItemHandler(),
				'SyncItemLocalID' => $localId,
				'SyncItemForeignID' => $syncId,
				'SyncItemHashCode' => $hashCode
			));
		}
		
		public function updateSyncInfo($localId, $syncId, $hashCode)
		{
			$this->insertSyncInfo($localId, $syncId, $hashCode);
		}
		
		public function deleteSyncInfo($localId)
		{
			return miStaticDBUtil::execDelete('SyncItems', array('SyncItemHandler', 'SyncItemLocalID'), array($this->getSyncItemHandler(), $localId));
		}
		
		public function getLocalIdFromSync($syncId)
		{
			$sql = 'SELECT SyncItemLocalID FROM SyncItems WHERE SyncItemForeignID=? AND SyncItemHandler=?';
			$result = miStaticDBUtil::execSelect($sql, array($syncId, $this->getSyncItemHandler()));
			if (count($result) == 0)
				throw new Exception(sprintf(_i('Invalid sync id "%s", handler "%s".'), $syncId, $this->getSyncItemHandler()));
			
			return $result[0]['SyncItemLocalID'];
		}
	}
	
	class scSyncTransactionInfo extends miDefaultDomainObject {
		
		const COLUMN_HANDLER = 'SyncTransactionItemHandler';
		const COLUMN_SYNC_ID = 'SyncTransactionItemForeignID';
		const COLUMN_HASH = 'SyncTransactionItemHashCode';
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('SyncTransactionItems', 'SyncTransactionItemID'));
		}
	}
?>