<?php
	/**
	 * Proxy Database Util Class
	 *
	 * @copyright Copyright (c) 2003-2006 Mirchev Ideas Ltd. All rights reserved.
	 * @package MIPHPF
	 */

	/**
	 */
	require_once(dirname(__FILE__) . '/' . 'DBUtilImpl.php');
	
	
	/**
	 * A proxy class that has static methods to access the preconfigured database connection
	 * The database connection is configured using the following settings:
	 * MI_DEFAULT_DB_HOST, MI_DEFAULT_DB_USER, MI_DEFAULT_DB_PASS and MI_DEFAULT_DB_NAME
	 *
	 * @copyright Copyright (c) 2003-2006 Mirchev Ideas Ltd. All rights reserved.
	 * @package MIPHPF
	 */
	class miStaticDBUtil {
		
		
		/**
		 * Returns an object of class miDBUtilImpl
		 *
		 * Object of class miDBUtilImpl or its subclasses is needed to handle
		 * the connection between database and miStaticDBUtil class
		 * 
		 * @access public
		 * @return miDBUtilImpl object of class miDBUtilImpl or its subclasses
		 */
		public static function getDBUtil()
		{
			static $dBUtil = null;
			
			if ($dBUtil != null)
				return $dBUtil;
			
			$settings = miSettings::singleton();
			$dBUtil = new miDBUtilImpl($settings->get('MI_DEFAULT_DB_HOST'), $settings->get('MI_DEFAULT_DB_USER'),
				$settings->get('MI_DEFAULT_DB_PASS'), $settings->get('MI_DEFAULT_DB_NAME'));
			return $dBUtil;
		}
		
		/**
		 * Ping the database and attempt to reconnect if necessary
		 * 
		 * @access public
		 * @return false on failure, true on success
		 * @throws miDBException
		 */
		public static function ping()
		{
			return miStaticDBUtil::getDBUtil()->ping();
		}
		
		/**
		 * Executes a SQL query
		 * 
		 * @access public
		 * @param string $query query string
		 * @param array $params positional query params (optional)
		 * @return resource Resource identifier or true depending on the query
		 * @throws miDBException
		 */
		public static function &execSQL($query, $params = array())
		{
			return miStaticDBUtil::getDBUtil()->execSQL($query, $params);
		}
		
		
		/**
		 * Executes a SQL insert query
		 * 
		 * @access public
		 * @param string $query query string
		 * @return int the ID generated for an AUTO_INCREMENT column by the previous INSERT query
		 * or 0 if the previous query does not generate an AUTO_INCREMENT value
		 * @throws miDBException
		 */
		public static function execSQLInsert($query)
		{
			return miStaticDBUtil::getDBUtil()->execSQLInsert($query);
		}
		
		
		/**
		 * Perform multi value lists insert
		 * 
		 * @param string $table
		 * @param array $columns array with column names
		 * @param array $valueLists array of arrays with values
		 * @param string|false $updateColumn column to be used with ON DUPLICATE KEY UPDATE
		 * @return unknown_type
		 */
		public static function multiValueListsInsert($table, $columns, $valueLists, $updateColumn = false)
		{
			return miStaticDBUtil::getDBUtil()->multiValueListsInsert($table, $columns, $valueLists, $updateColumn);
		}
		
		/**
		 * Returns an array with associative arrays from the result
		 * 
		 * @access public
		 * @param string $query query string
		 * @param array $params positional query params (optional)
		 * @return array array with associative arrays from the result
		 * @throws miDBException
		 */
		public static function execSelect($query, $params = array())
		{
			return miStaticDBUtil::getDBUtil()->execSelect($query, $params);
		}
		
		/**
		 * Returns the query with replaced positional params.
		 * 
		 * @param string $query query string
		 * @param array $params positional query params (optional)
		 * @return string
		 * @throws miDBException
		 */
		public static function getParsedQuery($query, $params = array())
		{
			return miStaticDBUtil::getDBUtil()->getParsedQuery($query, $params);
		}
		
		/**
		 * Returns the string so it is safe to use in a LIKE expression.
		 *
		 * @param string $string
		 * @return string
		 */
		public static function escapeStringForLike($string)
		{
			return miStaticDBUtil::getDBUtil()->escapeStringForLike($string);
		}
		
		
		/**
		 * Executes the select query and also returns the selected column names in the $fieldsArray
		 * 
		 * @access public
		 * @param string $query query string
		 * @param array $fieldsArray array for names of the fields
		 * @return array array with associative arrays from the result
		 * @throws miDBException
		 */
		public static function execSelectAndGetFields($query, &$fieldsArray)
		{
			return miStaticDBUtil::getDBUtil()->execSelectAndGetFields($query, $fieldsArray);
		}
		
		
		/**
		 * Executes a SQL insert query from an associative array.
		 * 
		 * Associative array is representing one record where keys 
		 * are the table fields names.
		 * 
		 * @access public
		 * @param string $table table name
		 * @param array $values associative arrays with data for inserting
		 * @param int $flags
		 * @return int the ID generated for an AUTO_INCREMENT column by the previous INSERT query
		 * or 0 if the previous query does not generate an AUTO_INCREMENT value
		 * @throws miDBException
		 */
		public static function execInsert($table, $values, $flags = 0)
		{
			return miStaticDBUtil::getDBUtil()->execInsert($table, $values, $flags);
		}
		
		
		/**
		 * Performs a multi-row SQL insert query
		 * 
		 * Associative array is representing one record where keys 
		 * are the table fields names.
		 * 
		 * @access public
		 * @param string $table table name
		 * @param array $values array of associative arrays with data for inserting
		 * @param int $flags
		 * @throws miDBException
		 */
		public static function execMultiInsert($table, $multiValues, $flags = 0)
		{
			return miStaticDBUtil::getDBUtil()->execMultiInsert($table, $multiValues, $flags);
		}
		
		
		/**
		 * Executes a SQL update query from an array for a specify record key
		 * 
		 * @access public
		 * @param string $table database table name
		 * @param array $values associative arrays with data for updating
		 * @param string $key name of the key field
		 * @param int $keyval value of the key field
		 * @return void
		 * @throws miDBException
		 */
		public static function execUpdate($table, $values, $key, $keyval)
		{
			miStaticDBUtil::getDBUtil()->execUpdate($table, $values, $key, $keyval);
		}
		
		
		/**
		 * Executes a SQL delete query for a specify record key
		 * 
		 * @access public
		 * @param string $table database table name
		 * @param string $key name of the key field
		 * @param int $keyval value of the key field
		 * @return void
		 * @throws miDBException
		 */
		public static function execDelete($table, $key, $keyval)
		{
			miStaticDBUtil::getDBUtil()->execDelete($table, $key, $keyval);
		}
		
		
		/**
		 * Returns an array with names of the fields in the specified table.
		 * 
		 * @access public
		 * @param string $table database table name
		 * @return array array with names of the fields
		 * @throws miDBException
		 */
		public static function getTableFields($table)
		{
			return miStaticDBUtil::getDBUtil()->getTableFields($table);
		}
		
		/**
		 * Starts a transaction
		 * You can start more than one transaction, but this will only increase transaction level counter
		 * Subtransactions are not passed to the database
		 * 
		 * @return void
		 */
		public static function startTransaction()
		{
			miStaticDBUtil::getDBUtil()->startTransaction();
		}
		
		/**
		 * Commits the current transaction
		 * 
		 * @return void
		 */
		public static function commit()
		{
			miStaticDBUtil::getDBUtil()->commit();
		}
		
		/**
		 * Rollbacks the current transaction
		 * 
		 * @return void
		 */
		public static function rollback()
		{
			miStaticDBUtil::getDBUtil()->rollback();
		}
	}
?>