<?php
	/**
	 * SQL Record Class
	 *
	 * @copyright Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 * @package MIPHPF
	 */
	
	/**
	 * If you delete all rows from $this->_table you will still have a row in the join table.
	 * Also you must add the readonly property to primary key
	 * $_dataFields = array(
	 * 		array(
	 *			'field' => 'miBaseTextWidget',
	 *			'data' => 'CustomUrlID',
	 *			'properties' => array('readOnly' => 1)
     * 		)
     * )
	 */
	if (class_exists('miStaticDBUtil', false) == false) {
		require_once(dirname(__FILE__) . '/../database/StaticDBUtil.php');
	}
	
	/**
	 * Handle and manage the inserting, updating, deleting and reading of data from the
	 * database. This class works only with one record row specified by a complex table primary key or
	 * unique fields.
	 *
	 * @copyright Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 * @package MIPHPF
	 */
	class miSqlRecordComplexKey extends miSqlRecord {
		
		const KEY_SEPARATOR = '|';
		const PK_NAME = 'ID';
		
		protected $_joinConditions = array();
		protected $_tableFields = null;
		
		/**
		 * miSqlRecordComplexKey constructor. It takes two parameters table names and 
		 * table primary keys
		 * 
		 * Example:
		 * <code>
		 * <?php
		 * $record = new miSqlRecord('tableName', array('tablePrimaryKey1', 'tablePrimaryKey2'));
		 * ?>
		 * </code>
		 * 
		 * @access public
		 * @param array $table database table name
		 * @param array $primaryKey database complex table primary key
		 */
		public function __construct($table, $primaryKeyFields)
		{
			$this->_table = $table;
			
			$primaryKeyFields = array_values($primaryKeyFields);
			$this->_primaryKey = array_combine($primaryKeyFields, $primaryKeyFields);
		}
		
		/**
		 * Read a record using the specified primary key value
		 *
		 * Example:
		 * <code>
		 * <?php
		 * $record = new miSqlRecord('tableName', array('tablePrimaryKey1', 'tablePrimaryKey2'));
		 * $record->readPK($value);
		 * ?>
		 * </code>
		 * @param mixed $value primary key value
		 * @return void
		 * @throws miDBException
		 */
		public function readPK($value)
		{
			$primaryKey = array();
			foreach ($this->_primaryKey as $field)
				$primaryKey[] = $this->_table . '.' . $field;
			
			$this->read($primaryKey, $value);
		}
		
		/**
		 * Gets the primary key column of the table that this SQLRecord uses
		 * 
		 * @access public
		 * @return string the primary key column name
		 */
		public function getPrimaryKeyColumn()
		{
			return self::PK_NAME;
		}
		
		/**
		 * Gets the value of the primary key
		 * 
		 * @access public
		 * @return mixed the primary key value
		 */
		public function getPK()
		{
			return implode(self::KEY_SEPARATOR, array_intersect_key($this->_row, $this->_primaryKey));
		}
		
		/**
		 * Returns true if the primary key is set
		 *  
		 * @return boolean
		 */
		public function hasPK()
		{
			foreach ($this->_primaryKey as $pk) {
				if (empty($this->_row[$pk])) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * Insert a record in the db table.
		 */
		public function insert()
		{
			miStaticDBUtil::execInsert($this->_table, $this->_row);
			return $this->getPK();
		}
		
		/**
		 * Read a record where the specified field represented by $key
		 * is equal to the value represented by $value
		 * If there are is than one record matching the first one will be read in
		 *
		 * Example:
		 * <code>
		 * <?php
		 * $record = new miSqlRecord('tableName', array('tablePrimaryKey1', 'tablePrimaryKey2'));
		 * $record->read(array('uniqueField1', 'uniqueField2'), array('value1', 'value2'));
		 * $record->read('uniqueField1|uniqueField2', 'value1|value2');
		 * ?>
		 * </code>
		 * @param string $key the name of the field
		 * @param string $value the value 
		 * @return void
		 * @throws miDBException
		 */
		public function read($key, $value)
		{
			if (!is_array($key))
				$key = explode(self::KEY_SEPARATOR, $key);
			
			if (!is_array($value))
				$value = explode(self::KEY_SEPARATOR, $value);
			
			$key = array_values($key);
			$value = array_values($value);
			
			if (count($key) != count($value))
				throw new miException('Key and value counts don\'t match.');
			
			$query = 'SELECT * FROM ' . $this->_table . $this->getJoinConditionsClause();
			
			$keysArray = array();
			foreach ($key as $keyId => $keyName)
				$keysArray[] = $keyName . $this->getFilterByValueSql($value[$keyId]);
			$query .= ' WHERE ' . implode(' AND ', $keysArray);
			
			$rows = miStaticDBUtil::execSelect($query);
			if (count($rows) < 1)
				throw new miDBException(sprintf('Record not found (Table "%s").', $this->_table), miDBException::EXCEPTION_RECORD_NOT_FOUND);
			
			$this->_row = $rows[0];
		}
		
		/**
		 * Update a record in the db table
		 *
		 * @access public
		 * @return PK the primary key of the updated record
		 * @throws miDBException
		 */
		public function update()
		{
			$tableFields = $this->getTableFields();
			miStaticDBUtil::execUpdate($this->_table, array_intersect_key($this->_row, $tableFields), $this->_primaryKey, array_intersect_key($this->_row, $this->_primaryKey));
			return $this->getPK();
		}
		
		/**
		 * Delete a record in the db table
		 *
		 * @access public
		 * @return PK the primary key of the deleted record
		 * @throws miDBException
		 */
		public function delete()
		{
			if (isset($this->_row[self::PK_NAME])) {
				$pkValues = explode(self::KEY_SEPARATOR, $this->_row[self::PK_NAME]);
				$pkValues = array_combine($this->_primaryKey, $pkValues);
				$this->_row = array_merge($this->_row, $pkValues);
			}
			
			$pkValues = array();
			foreach ($this->_primaryKey as $pkField) {
				if (!isset($this->_row[$pkField])) {
					throw new miDBException('Cannot delete: primary key not set');
				}
				$pkValues[$pkField] = $this->_row[$pkField];
			}
			
			miStaticDBUtil::execDelete($this->_table, $this->_primaryKey, array_intersect_key($this->_row, $this->_primaryKey));
			return $this->getPK();
		}
		
		/**
		 * Adds join condition
		 * 
		 * @param string $joinType the type of the join, eg. INNER, LEFT
		 * @param string $joinTable the table to be joined
		 * @param string $joinCondition the condition of the join
		 */
		public function addJoinCondition($joinType, $joinTable, $joinCondition, $joinTableAlias = '')
		{
			$this->_joinConditions[] = array($joinType, $joinTable, $joinCondition, $joinTableAlias);
		}
		
		protected function getFilterByValueSql($value)
		{
			if (is_null($value))
				return ' IS NULL';
			if ($value === false)
				return ' =FALSE';
			return ' ="' . sql_escape_string($value) . '"';
		}
		
		/**
		 * Return the join conditions sql
		 *
		 * @access protected
		 * @return string the join conditions sql
		 */
		protected function getJoinConditionsClause()
		{
			$sql = '';
			foreach ($this->_joinConditions as $joinCondition) {
				$sql .= ' ' . $joinCondition[0] . ' JOIN ' .
					$joinCondition[1] . (!empty($joinCondition[3]) ? ' AS ' . $joinCondition[3] : '') . ' ' .
					$joinCondition[2];
			}
			return $sql;
		}
		
		protected function getTableFields()
		{
			if ($this->_tableFields === null) {
				$tableFields = miStaticDBUtil::getTableFields($this->_table);
				$this->_tableFields = array_combine($tableFields, $tableFields);
			}
			
			return $this->_tableFields;
		}
	}
?>