<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCustomerDomainObject extends scDomainObject {
		const GENERATE_PASSWORD_LENGTH = 12;
		
		protected $_deactivatedDiscounts = array();
		protected $_deactivatedCoupons = array();
		protected $_addresses = null;
		protected $_billingAddressId = null;
		protected $_shippingAddressId = null;
		
		/**
		 * @param miSqlRecord $record
		 */
		public function __construct($record = null)
		{
			if (is_null($record)) {
				$record = new miSqlRecord('Customers', 'CustomerID');
			}
			parent::__construct($record);
		}
		
		public function set($name, $value)
		{
			// Hash the password
			if ('CustomerPasswordHashMode' == $name)
				return;
			
			if ('CustomerPassword' == $name) {
				$hashMode = scSettings::getSettingValue(scSettings::SETTING_PASSWORD_HASH_MODE);
				$this->_record->set('CustomerPasswordHashMode', $hashMode);
				
				$auth = scCustomerAuth::singleton();
				$value = $auth->hashPassword($value);
			}
			$this->_record->set($name, $value);
		}
		
		/**
		 * @return miSqlRecord
		 */
		public function getRecord()
		{
			return $this->_record;
		}
		
		/**
		 * @param mixed $value
		 * @param string $field
		 */
		public function read($value, $field = 'CustomerID')
		{
			$this->_record->read($field, $value);
		}
		
		public function update()
		{
			$customerId = parent::update();
			
			miStaticDBUtil::execUpdate('SyncItems', array('SyncItemHashCode' => ''),
				array('SyncItemHandler', 'SyncItemLocalID'), array('scXmlSyncCustomers', $customerId));
			
			return $customerId;
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedDiscounts()
		{
			return $this->_deactivatedDiscounts;
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedCoupons()
		{
			return $this->_deactivatedCoupons;
		}
		
		public function setRow($data)
		{
			$fields = miStaticDBUtil::getTableFields($this->getTableName());
			foreach ($fields as $field) {
				if (isset($data[$field])) {
					$this->set($field, $data[$field]);
				}
			}
		}
		
		public function validateOnInsert()
		{
			$record = new miSqlRecord('Customers', 'CustomerID');
			try {
				$record->read('CustomerEmail', $this->_record->get('CustomerEmail'));
			} catch (miException $exception) {
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return true;
				}
				throw $exception;
			}
			throw new miException(_t('This email is already registered.'));
		}
		
		public function validateOnUpdate()
		{
			$record = new miSqlRecord('Customers', 'CustomerID');
			try {
				$record->read('CustomerEmail', $this->_record->get('CustomerEmail'));
			} catch (miException $exception) {
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					return true;
				}
				throw $exception;
			}
			if ($record->get('CustomerID') == $this->_record->get('CustomerID')) {
				// this is our email
				return true;
			}
			throw new miException(_t('This email is already registered.'));
		}
		
		public function &getAddresses()
		{
			$this->loadAddresses();
			return $this->_addresses;
		}
		
		public function getBillingAddress()
		{
			$addressId = $this->getBillingAddressId();
			if (is_null($addressId)) {
				return array();
			}
			$adresses = $this->getAddresses();
			return $adresses[$this->_billingAddressId];
		}
		
		public function getShippingAddress()
		{
			$addressId = $this->getShippingAddressId();
			if (is_null($addressId)) {
				return array();
			}
			$adresses = $this->getAddresses();
			return $adresses[$this->_shippingAddressId];
		}
		
		public function getBillingAddressId()
		{
			$this->loadAddresses();
			return $this->_billingAddressId;
		}
		
		public function getShippingAddressId()
		{
			$this->loadAddresses();
			return $this->_shippingAddressId;
		}
		
		/**
		 * Returns billing and shipping address suitable for cart / order
		 *
		 * @return array
		 */
		public function getBillingAndShippingAddresses()
		{
			$data = array();
			
			$customerAddresses = $this->getAddresses();
			$billingAddressId = $this->getBillingAddressId();
			$shippingAddressId = $this->getShippingAddressId();
			
			if (isset($customerAddresses[$billingAddressId])) {
				$data = array_merge($data, scCustomerAddress::getMappedAddressFields(scCustomerAddress::BILLING, $customerAddresses[$billingAddressId]));
			}
			if (isset($customerAddresses[$shippingAddressId])) {
				$data = array_merge($data, scCustomerAddress::getMappedAddressFields(scCustomerAddress::SHIPPING, $customerAddresses[$shippingAddressId]));
			}
			$data['CustomerShipToBillingAddress'] = ($billingAddressId == $shippingAddressId ? 1 : 0);
			return $data;
		}
		
		protected function onDelete($pkValue)
		{
			$this->deactivateDiscounts($pkValue);
		}
		
		protected function deactivateDiscounts($customerIds)
		{
			foreach ($customerIds as $customerId) {
				$discounts = scDiscountService::validateDiscounts(
					'Discounts', 'DiscountID', 'DiscountCustomers', 'CustomerID', 'DiscountIsActive', $customerId);
				$this->addDeactivatedDiscounts($discounts);
				
				$coupons = scDiscountService::validateDiscounts(
					'Coupons', 'CouponID', 'CouponCustomers', 'CustomerID', 'CouponIsActive', $customerId);
				$this->addDeactivatedCoupons($coupons);
			}
		}
		
		/**
		 * @param array $discounts
		 */
		protected function addDeactivatedDiscounts($discounts)
		{
			$this->_deactivatedDiscounts = array_merge($this->_deactivatedDiscounts, $discounts);
		}
		
		/**
		 * @param array $coupons
		 */
		protected function addDeactivatedCoupons($coupons)
		{
			$this->_deactivatedCoupons = array_merge($this->_deactivatedCoupons, $coupons);
		}
		
		protected function loadAddresses()
		{
			if (!is_null($this->_addresses)) {
				return;
			}
			
			$recordset = new miSqlRecordset('CustomerAddresses');
			$recordset->addFilter(new miSqlFilterEqual('CustomerID', $this->get('CustomerID')));
			$this->_addresses = $recordset->getAllRecords();
			$this->_addresses = miUtil::getArrayWithKeys($this->_addresses, 'CustomerAddressID');
			
			foreach ($this->_addresses as $addressId => $address) {
				if ($address['CustomerAddressIsDefaultBilling']) {
					$this->_billingAddressId = $addressId;
				}
				if ($address['CustomerAddressIsDefaultShipping']) {
					$this->_shippingAddressId = $addressId;
				}
			}
			
			if (count($this->_addresses) > 0) {
				if (is_null($this->_billingAddressId)) {
					$this->_billingAddressId = key($this->_addresses);
				}
				if (is_null($this->_shippingAddressId)) {
					$this->_shippingAddressId = key($this->_addresses);
				}
			}
		}
		
		/**
		 * Returns the customer id
		 * If the customer doesn't exists creates new customer
		 *
		 * @param string $email
		 * @param scOrder $order
		 * @return int|false the customer id, or false if not found
		 */
		public static function getCustomerIdForNewOrder($email, $order)
		{
			// Check if customer with the given email exists
			$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $email, false);
			
			$record = new scCustomerDomainObject();
			if ($customerId !== false)
				$record->read($customerId);
			else {
				$record->set('CustomerEmail', $email);
				
				// Generate random password no longer than GENERATE_PASSWORD_LENGTH
				$password = md5(uniqid(rand(), true));
				if (strlen($password) > self::GENERATE_PASSWORD_LENGTH)
					$password = substr($password, 0, self::GENERATE_PASSWORD_LENGTH);
					
				$record->set('CustomerPassword', $password);
				$record->set('CustomerStatus', scCustomerStatuses::CUSTOMER_STATUS_ACTIVE);
				$record->set('CustomerGroupID', scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CUSTOMER_GROUP));
			}
			
			// If it is existing customer - update
			if ($customerId !== false) {
				$record->update();
				return $customerId;
			}
			
			// If it is new customer - insert
			$customerId = $record->insert();
			
			if ($order != null) {
				// FIXME: This shouldn't be here
				$address = new scCustomerAddress();
				$address->set('CustomerID', $customerId);
				$address->set('CustomerAddressFirstName', $order->get('OrderBillingFirstName'));
				$address->set('CustomerAddressLastName', $order->get('OrderBillingLastName'));
				$address->set('CustomerAddressCompany', $order->get('OrderBillingCompany'));
				$address->set('CustomerAddressPhone', $order->get('OrderBillingPhone'));
				$address->set('CustomerAddressFax', $order->get('OrderBillingFax'));
				$address->set('CustomerAddressLine1', $order->get('OrderBillingAddress1'));
				$address->set('CustomerAddressLine2', $order->get('OrderBillingAddress2'));
				$address->set('CustomerAddressCity', $order->get('OrderBillingCity'));
//				var_dump($record->get('CustomerBillingCountry'));die;
				$address->set('CountryID', scCountry::getCountryIdByCode2($order->get('OrderBillingCountry')));
				$address->set('StateID', scCountry::getStateId($record->get('CustomerBillingCountry'), $order->get('OrderBillingState')));
				$address->set('CustomerAddressZip', $order->get('OrderBillingZip'));
				$address->set('CustomerAddressIsDefaultBilling', 1);
				$address->set('CustomerAddressIsDefaultShipping', $order->isShippingAddressSameAsBilling());
				$address->insert();
				
				if (!$order->isShippingAddressSameAsBilling()) {
					$address = new scCustomerAddress();
					$address->set('CustomerID', $customerId);
					$address->set('CustomerAddressFirstName', $order->get('OrderShippingFirstName'));
					$address->set('CustomerAddressLastName', $order->get('OrderShippingLastName'));
					$address->set('CustomerAddressCompany', $order->get('OrderShippingCompany'));
					$address->set('CustomerAddressPhone', $order->get('OrderShippingPhone'));
					$address->set('CustomerAddressFax', $order->get('OrderShippingFax'));
					$address->set('CustomerAddressLine1', $order->get('OrderShippingAddress1'));
					$address->set('CustomerAddressLine2', $order->get('OrderShippingAddress2'));
					$address->set('CustomerAddressCity', $order->get('OrderShippingCity'));
					$address->set('CountryID', scCountry::getCountryIdByCode2($order->get('OrderShippingCountry')));
					$address->set('StateID', scCountry::getStateId($record->get('CustomerShippingCountry'), $order->get('OrderShippingState')));
					$address->set('CustomerAddressZip', $order->get('OrderShippingZip'));
					$address->set('CustomerAddressIsDefaultBilling', 0);
					$address->set('CustomerAddressIsDefaultShipping', 1);
					$address->insert();
				}
			}
			
			// Send notification email
			$notifications = new scEmailNotifications();
			$notifications->sendSignupNotification($customerId);
			return $customerId;
		}
	}
?>