<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductDomainObject extends scDomainObject implements IStockable {
		
		/**
		 * @var array
		 */
		protected $_deactivatedDiscounts = array();
		
		/**
		 * @var array
		 */
		protected $_deactivatedCoupons = array();
		
		/**
		 * @var scManufacturerDomainObject
		 */
		protected $_manufacturer = false;
		
		/**
		 * @var scProductSL[]
		 */
		protected $_bundleProducts = null;
		
		/**
		 * When updating products a recursion may occur.
		 * Help keep track of such an event to prevent it.
		 */
		protected static $_bundleUpdateProducts = array();
		
		protected static $_cache = array();
		
		public function __construct()
		{
			$recordMlFields = array('ProductName', 'ProductDescription', 'ProductDetailedDescription',
				'ProductPageTitle', 'ProductMetaKeywords', 'ProductMetaDescription');
			$recordLanguages = array_keys(scMultiLanguage::getActiveLanguageCodes());
			
			parent::__construct(new scSqlRecordML('Products', 'ProductID', $recordMlFields, $recordLanguages));
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedDiscounts()
		{
			return $this->_deactivatedDiscounts;
		}
		
		/**
		 * @return array
		 */
		public function getDeactivatedCoupons()
		{
			return $this->_deactivatedCoupons;
		}
		
		/**
		 * @return int
		 */
		public function insert()
		{
			if ($this->has('ManufacturerID') and ($this->get('ManufacturerID') == 0))
				$this->set('ManufacturerID', null);
			if (!$this->has('ProductCreatedTimestamp'))
				$this->set('ProductCreatedTimestamp', time());
			
			// Get sort number
			if (scSettings::getSettingValue(scSettings::SETTING_NEW_PRODUCTS_INFRONT_OF_CATEGORY_LISTING) == 1) {
				$newSortId = 1;
				$sql = 'UPDATE Products SET ProductSortInCategory = ProductSortInCategory + 1 WHERE CategoryID = ? ORDER BY ProductSortInCategory DESC';
				miStaticDBUtil::execSQL($sql, array($this->get('CategoryID')));
			} else {
				$newSortId = scProductSortService::getMaxSortInCategory($this->get('CategoryID')) + 1;
			}
			$this->set('ProductSortInCategory', $newSortId);
			
			return parent::insert();
		}
		
		/**
		 * @return int
		 */
		public function update()
		{
			if (in_array($this->get('ProductID'), self::$_bundleUpdateProducts)) {
				throw new scBundleRecursionException(_i('Product data may not be set to parent products due to recursive dependency.'));
			}
			
			if ($this->has('ManufacturerID') and ($this->get('ManufacturerID') == 0))
				$this->set('ManufacturerID', null);
			
			// Check the default category has changed
			if (!$this->isProductDefaultCategory($this->get('CategoryID'))) {
				// Get max sort number
				$newSortId = scProductSortService::getMaxSortInCategory($this->get('CategoryID')) + 1;
				$this->set('ProductSortInCategory', $newSortId);
			}
			
			// If this is a bundle, update its data from the subproducts
			$this->retrieveBundleData();
			
			// Update bundles that this product is contained in
			$bundles = $this->getParentBundles();
			self::$_bundleUpdateProducts[$this->get('ProductID')] = $this->get('ProductID');
			foreach ($bundles as $bundleProduct) {
				try {
					$bundleProduct->retrieveBundleData();
					$bundleProduct->update();
				} catch (scBundleRecursionException $e) {
					// If an exception of this type is thrown
					// just go on allowing the product to be saved.
					// The only reason this would happen is wrong data
					// and the only chance to fix the data is saving the product.
				}
			}
			unset(self::$_bundleUpdateProducts[$this->get('ProductID')]);
			
			// Store the return value
			$return = parent::update();
			
			// Update the cache. Do it after successfully updating.
			if (isset(self::$_cache[$this->get('ProductID')])) {
				self::$_cache[$this->get('ProductID')] = $this->getRow();
			}
			
			return $return;
		}
		
		public function read($pkValue)
		{
			if (isset(self::$_cache[$pkValue])) {
				$this->_record->setRow(self::$_cache[$pkValue]);
				return;
			}
			
			parent::read($pkValue);
			self::$_cache[$pkValue] = $this->_record->getRow();
		}
		
		public function retrieveBundleData()
		{
			if (!$this->get('ProductIsBundle')) {
				return;
			}
			$this->getBundleProducts(true); // Force reading subproducts
			$this->retrieveBundleWeight();
			$this->retrieveBundleQuantity();
		}
		
		/**
		 * @param int $productId
		 * @return array
		 */
		public function getDeletableProductEgoods($productId)
		{
			$productEgoodsRecordset = new miSqlRecordset('ProductEgoods');
			$productEgoodsRecordset->addFilter(new miSqlFilterEqual('ProductID', intval($productId)));
			$egoods = $productEgoodsRecordset->getAllRecords();
			// We have all the egoods of this product.
			// Now we need to return only the deletable ones.
			$egoodDomainObject = new scProductEgood();
			return $egoodDomainObject->getDeletableEgoodsFromArray($egoods);
		}
		
		/**
		 * @return bool
		 */
		public function hasVariants()
		{
			$query = 'SELECT COUNT(ProductVariantID) AS N FROM ProductVariants WHERE ProductID = ?';
			$result = miStaticDBUtil::execSelect($query, array($this->get('ProductID')));
			if (!isset($result[0]) || !isset($result[0]['N'])) {
				return false;
			}
			return (bool)(int)$result[0]['N'];
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getDefaultVariant()
		{
			$query = 'SELECT * FROM ProductVariants WHERE ProductID = ? ORDER BY ProductVariantIsDefault DESC LIMIT 1';
			$productVariantData = miStaticDBUtil::execSelect($query, array($this->get('ProductID')));
			if (!isset($productVariantData[0])) {
				throw new miException(_i('Unable to find default product variant.'));
			}
			
			$productVariant = new scProductVariant();
			$productVariant->setRow($productVariantData[0]);
			return $productVariant;
		}
		
		public function getQuantityInStock()
		{
			return $this->get('ProductQuantity');
		}
		
		public function reduceQuantityInStock($itemsRemoved)
		{
			$newQuantity = $this->get('ProductQuantity') - $itemsRemoved;
			$this->setQuantityInStock($newQuantity);
		}
		
		public function setQuantityInStock($newQuantityInStock)
		{
			$this->set('ProductQuantity', $newQuantityInStock);
			$this->update();
		}
		
		public function isOutOfStock($quantity = 1)
		{
			if ((int)$this->getQuantityInStock() >= (int)$quantity)
				return false;
			
			$outOfStockHandling = scSettings::getSettingValue(scSettings::SETTING_OUT_OF_STOCK_HANDLING);
			if (($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_TAG) or ($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_HIDE))
				return true;
			return false;
		}
		
		public function getSkuCode()
		{
			return $this->get('ProductCode');
		}
		
		/**
		 * @return scWeight
		 */
		public function getWeight()
		{
			return new scWeight($this->get('ProductWeight'), scSettings::getSettingValue(scSettings::SETTING_LOCALIZATION_WEIGHT_UNIT));
		}
		
		/**
		 * @return bool
		 */
		public function isEgood()
		{
			return (bool)$this->get('ProductIsEgood');
		}
		
		/**
		 * @return int or null
		 */
		public function getManufacturerId()
		{
			$id = $this->get('ManufacturerID');
			if (empty($id)) {
				return null;
			}
			return $id;
		}
		
		/**
		 * @return scManufacturerDomainObject or null
		 */
		public function getManufacturer()
		{
			if ($this->_manufacturer === false) {
				$this->_manufacturer = null;
				$id = $this->getManufacturerId();
				if (!is_null($id)) {
					$this->_manufacturer = scManufacturersCache::getDomainObject($id);
				}
			}
			return $this->_manufacturer;
		}
		
		/**
		 * Checks if any of the IDs in $unwantedIds is a subproduct.
		 * Thows an exception if that is so.
		 * @throw miException
		 */
		public function checkBundleRecursion($unwantedIds)
		{
			$bundleProducts = $this->getBundleProducts();
			$unwantedIds[] = $this->get('ProductID');
			
			$bundledIds = array();
			foreach ($bundleProducts as $bundleProduct) {
				$bundledIds[] = $bundleProduct->get('ProductID');
			}
			$foundUnwanted = array_intersect($unwantedIds, $bundledIds);
			if (count($foundUnwanted) > 0) {
				$unwanted = new scProductSL();
				$unwanted->read(reset($foundUnwanted));
				throw new scBundleRecursionException(sprintf(_t('Product "%s" found as bundled product. This would cause recursive dependency.'), $unwanted->getName()));
			}
			
			foreach ($bundleProducts as $bundleProduct) {
				if (!$bundleProduct->get('ProductIsBundle')) {
					continue;
				}
				$bundleProduct->checkBundleRecursion($unwantedIds);
			}
		}
		
		public function getBundleProductsData()
		{
			if (!$this->get('ProductIsBundle')) {
				return array();
			}
			
			$recordset = new miSqlRecordset('BundleProducts');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $this->get('ProductID')));
			$recordset->setOrder('BundleProductSort');
			return miUtil::getArrayWithKeys($recordset->getAllRecords(), 'SubProductID');
		}
		
		/**
		 * @return scProductSL[]
		 */
		public function getBundleProducts($forceRead = false)
		{
			if ($forceRead || is_null($this->_bundleProducts)) {
				$productIds = $this->getBundleProductsData();
				
				$this->_bundleProducts = array();
				foreach ($productIds as $productId => $bundleProductData) {
					$product = new scBundledProduct();
					$product->read($productId);
					$product->setInBundleQuantity($bundleProductData['BundleProductQuantity']);
					$this->_bundleProducts[] = $product;
				}
			}
			
			return $this->_bundleProducts;
		}
		
		/**
		 * Returns all the bundles that this product is a part of.
		 * @return scProductDomainObject[]
		 */
		public function getParentBundles()
		{
			$recordset = new miSqlRecordset('BundleProducts');
			$recordset->setSelectFields(array(
				'DISTINCT(ProductID)',
			));
			$recordset->addFilter(new miSqlFilterEqual('SubProductID', $this->get('ProductID')));
			
			if (!$recordset->getRecordsCount()) {
				return array();
			}
			
			$productIds = miUtil::getKeyValueArray($recordset->getAllRecords(), 'ProductID', 'ProductID');
			$products = array();
			foreach ($productIds as $productId) {
				// Check for invalid bundling of itself
				if ($productId == $this->get('ProductID'))
					continue;
				
				$product = new scProductDomainObject();
				$product->read($productId);
				$products[] = $product;
			}
			return $products;
		}
		
		protected function onDelete($pkValue)
		{
			$this->deleteRatingActions($pkValue);
			$this->deleteProductImages($pkValue);
			$this->deleteProductEgoods($pkValue);
			$this->deleteVariants($pkValue);
			
			$this->deactivateDiscounts($pkValue);
		}
		
		/**
		 * Deletes all rating actions that have ProductRatingVotes or ProductReviews about the deleted products
		 */
		protected function deleteRatingActions($pkValue)
		{
			$sql = 'SELECT RatingActionID FROM ProductRatingVotes WHERE ProductID IN (' . implode(', ', $pkValue) . ')';
			$raIds1 = miUtil::getDBArray($sql, 'RatingActionID', 'RatingActionID');
			
			$sql = 'SELECT RatingActionID FROM ProductReviews WHERE ProductID IN (' . implode(', ', $pkValue) . ')';
			$raIds2 = miUtil::getDBArray($sql, 'RatingActionID', 'RatingActionID');
			
			$raIds = array_unique(array_merge($raIds1, $raIds2));
			if (empty($raIds)) {
				return;
			}
			
			$deleteQuery = 'DELETE FROM RatingActions WHERE RatingActionID IN (' . implode(', ', $raIds) . ')';
			miStaticDBUtil::execSQL($deleteQuery);
		}
		
		/**
		 * @param array $productId
		 */
		protected function deleteProductImages($productIds)
		{
			// Delete the image files
			$sql = 'SELECT ProductImageID FROM ProductImages WHERE ' .
				$this->getPKName() . ' IN ("' . implode('", "', $productIds) . '")';
			$imageIds = miUtil::getDBArray($sql, 'ProductImageID', 'ProductImageID');
			$images = new scProductImageDomainObject();
			$images->delete($imageIds);
		}
		
		/**
		 * @param array $productId
		 */
		protected function deleteProductEgoods($productIds)
		{
			// Delete the e-good files
			$sql = 'SELECT ProductEgoodID FROM ProductEgoods WHERE ' .
				$this->getPKName() . ' IN ("' . implode('", "', $productIds) . '")';
			$egoodIds = miUtil::getDBArray($sql, 'ProductEgoodID', 'ProductEgoodID');
			$egoods = new scProductEgood();
			$egoods->delete($egoodIds);
		}
		
		/**
		 * @param array $productIds
		 */
		protected function deactivateDiscounts($productIds)
		{
			foreach ($productIds as $productId) {
				$discounts = scDiscountService::validateDiscounts(
					'Discounts', 'DiscountID', 'DiscountProducts', 'ProductID', 'DiscountIsActive', $productId);
				$this->addDeactivatedDiscounts($discounts);
				
				$coupons = scDiscountService::validateDiscounts(
					'Coupons', 'CouponID', 'CouponProducts', 'ProductID', 'CouponIsActive', $productId);
				$this->addDeactivatedCoupons($coupons);
			}
		}
		
		/**
		 * Delete all variants for this product
		 * @param array $productIds
		 */
		protected function deleteVariants($productIds)
		{
			$sql = 'SELECT ProductVariantID FROM ProductVariants WHERE ProductID IN ("' . implode('", "', $productIds) . '")';
			$variantIds = miUtil::getDBArray($sql, 'ProductVariantID', 'ProductVariantID');
			if (empty($variantIds))
				return;
			
			$productVariant = new scProductVariant();
			$productVariant->delete($variantIds);
		}
		
		/**
		 * @param array $discounts
		 */
		protected function addDeactivatedDiscounts($discounts)
		{
			$this->_deactivatedDiscounts = array_merge($this->_deactivatedDiscounts, $discounts);
		}
		
		/**
		 * @param array $coupons
		 */
		protected function addDeactivatedCoupons($coupons)
		{
			$this->_deactivatedCoupons = array_merge($this->_deactivatedCoupons, $coupons);
		}
		
		/**
		 * Check if $categoryId is the product category
		 * @param int $categoryId
		 */
		protected function isProductDefaultCategory($categoryId)
		{
			$result = miStaticDBUtil::execSelect('SELECT ProductID FROM Products WHERE ProductID = ? AND CategoryID = ?', 
						array($this->get('ProductID'), $categoryId));
			return empty($result) ? false : true;
		}
		
		protected function retrieveBundleWeight()
		{
			$subProducts = $this->getBundleProducts();
			if (empty($subProducts)) {
				return;
			}
			
			$productWeight = 0;
			foreach ($subProducts as $subProduct) {
				$productWeight += $subProduct->getWeight()->getWeight();
			}
			$this->set('ProductWeight', $productWeight);
		}
		
		protected function retrieveBundleQuantity()
		{
			if (!$this->get('ProductBundleAutoQuantity')) {
				return;
			}
			
			$subProducts = $this->getBundleProducts();
			if (empty($subProducts)) {
				return;
			}
			
			$productQuantity = PHP_INT_MAX;
			foreach ($subProducts as $subProduct) {
				// If somehow we have bundle product with zero quantity treat it as 1
				$inBundleQuantity = $subProduct->getInBundleQuantity();
				if ($inBundleQuantity == 0)
					$inBundleQuantity = 1;
				
				$maxQuantityOfSub = floor($subProduct->getQuantityInStock() / $inBundleQuantity);
				$productQuantity = min($productQuantity, $maxQuantityOfSub);
			}
			$this->set('ProductQuantity', $productQuantity);
		}
		
		/*protected function retrieveBundlePrices()
		{
			$subProducts = $this->getBundleProducts();
			if (empty($subProducts)) {
				return;
			}
			
			$price = 0;
			foreach ($subProducts as $subProduct) {
				$priceOfSub = $subProduct->get('ProductPrice');
				$price += $priceOfSub * $subProduct->getInBundleQuantity();
			}
			$this->set('ProductPrice', $price);
		}
		
		protected function retrieveBundleActivity()
		{
			$subProducts = $this->getBundleProducts();
			if (empty($subProducts)) {
				return;
			}
			
			foreach ($subProducts as $subProduct) {
				if (!$subProduct->get('ProductIsActive')) {
					$this->set('ProductIsActive', false);
					return;
				}
			}
		}*/
	}
	
	class scBundleRecursionException extends miException {
		
	}
?>