<?php
	/**
	 * Captcha Image Generator Class
	 * 
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class generate captcha images
	 */
	class scCaptchaImage {
		
		/**
		 * Configuration
		 * @var Array
		 */
		protected $_config = array(
			'imageWidth' => 200,
			'imageHeight' => 70,
			'imageOffsetX' => 20,
			'imageOffsetY' => 45,
			'backgroundColor' => array(255, 255, 255),
			'fontColor' => array(0, 0, 0),
			'randomColor' => true,
			'overwriteOffset' => -4, // 0 means 'No', -1 means '1px' and so on...
			'fontPath' => '/fonts/',
			'xPeriod' => 15,
			'xAmplitude' => 12,
			'maxRotation' => 10,
			'applyFilters' => array(
				// IMG_FILTER_GRAYSCALE, // Converts the image into grayscale. 
				// IMG_FILTER_EDGEDETECT, // Uses edge detection to highlight the edges in the image. 
				// IMG_FILTER_EMBOSS, // Embosses the image.
				// IMG_FILTER_GAUSSIAN_BLUR, // Blurs the image using the Gaussian method. 
				// IMG_FILTER_SELECTIVE_BLUR, // Blurs the image.
				// IMG_FILTER_MEAN_REMOVAL, // Uses mean removal to achieve a "sketchy" effect. 
			),
		);
		
		/**
		 * Fonts
		 * @var Array
		 */
		protected $_fonts = array(
			'Tuffy' => array(
				'minSpacing' => 0,
				'maxSpacing' => 3,
				'minSize' => 27,
				'maxSize' => 28,
				'font' => 'Tuffy_Bold.ttf',
			)
		);
		
		private $_image = null;
		private $_bgColor = null;
		private $_fgColor = null;
		
		/**
		 * Contruct scCaptchaImage object
		 * @param $config Array CaptchaSecurity settings, overwrites the default one if set
		 * @param $fonts Array Available fonts
		 * @return void
		 */
		public function __construct(array $config = array(), array $fonts = array()) 
		{
			$this->_config = array_merge($this->_config, $config);
			$this->_fonts = array_merge($this->_fonts, $fonts);
		}
		
		/**
		 * @return void
		 */
		public function generateImage($text)
		{
			$this->allocateImage();
			$this->fillWithBackground();
			$this->printText($text);
			$this->applyFilters();
			$this->addWave();
			$this->showImage();
            $this->destroyImage();
		}
		
		/*
		 * Apply filters to the image
		 */
		protected function applyFilters()
		{
			foreach ($this->_config['applyFilters'] as $filter) {
				imagefilter($this->_image, $filter);
			}
		}
		
		/*
		 * Print the text to the image.
		 */
		protected function printText($text)
		{
			$font = $this->useRandomFont();
			$fontFile = dirname(__FILE__) . $this->_config['fontPath'] . $font['font'];
			$x = $this->_config['imageOffsetX'];
			$y = $this->_config['imageOffsetY'];
			
			for ($i = 0, $l = strlen($text); $i < $l; $i++) {
				
				$fontSize = mt_rand($font['minSize'], $font['maxSize']);
				$fontSpacing = mt_rand($font['minSpacing'], $font['maxSpacing']);
				$angle = mt_rand(($this->_config['maxRotation'] * -1), $this->_config['maxRotation']);
				$letter = $text[$i];
				
				$this->randomFgColor();
				$coordinates = imagettftext(
					$this->_image,
					$fontSize,
					$angle,
					$x,
					$y,
					$this->_fgColor,
					$fontFile,
					$letter
				);
				
				$x += ($coordinates[2] - $x) + $fontSpacing;
				
				if ($this->_config['overwriteOffset'] < 0) {
					$x += $this->_config['overwriteOffset'];
				}
			}
		}
		
		
		/**
		 * Adds a wavelike effect to further skew the image
		 */
		protected function addWave()
		{
	       	$xPeriod = $this->_config['xPeriod'] * rand(1,3);
	        $ratio = rand(0, 100);
	        for ($i = 0; $i < $this->_config['imageWidth']; $i++) {
	        	imagecopy(
	        		$this->_image,
	        		$this->_image,
	        		$i - 1,
	        		sin( ($ratio + $i) / $xPeriod ) * $this->_config['xAmplitude'],
	        		$i,
	        		0,
	        		1,
	        		$this->_config['imageHeight']
	        	);
	        }
		}
		
		
		/**
		 * Build the image and output it
		 */
		protected function showImage()
		{
            header("Content-type: image/png");
            imagepng($this->_image);
		}
		
		/**
		 * Create image resource
		 * @return void
		 */
		protected function allocateImage()
		{
			if (is_resource($this->_image)) {
				// if the image is already generated destroy it and re-generate it
				$this->destroyImage();
			}
			
			$this->_image = imagecreatetruecolor($this->_config['imageWidth'], $this->_config['imageHeight']);
		}
		
		protected function fillWithBackground()
		{
			$this->_bgColor = imagecolorallocate(
				$this->_image,
				$this->_config['backgroundColor'][0],
				$this->_config['backgroundColor'][1],
				$this->_config['backgroundColor'][2]
			);
			
			imagefilledrectangle(
				$this->_image,
				0,
				0,
				$this->_config['imageWidth'],
				$this->_config['imageHeight'],
				$this->_bgColor
			);
		}
		
		/**
		 * TODO: Improve color generation
		 * Do not allow generated color to be equal to background one, max 200 to so that it's not white
		 * @return void
		 */
		protected function randomFgColor()
		{
			if ($this->_config['randomColor']) {
				$this->_config['fontColor'][0] = mt_rand(0, 200);
				$this->_config['fontColor'][1] = mt_rand(0, 200);
				$this->_config['fontColor'][2] = mt_rand(0, 200);
			}
			
			$this->_fgColor = imagecolorallocate(
				$this->_image,
				$this->_config['fontColor'][0],
				$this->_config['fontColor'][1],
				$this->_config['fontColor'][2]
			);
		}
		
		/**
		 * Pick up a random font
		 * @return Array
		 */
		protected function useRandomFont()
		{
			$font = array_rand($this->_fonts);
			
			return $this->_fonts[$font];
		}
		
		/**
		 * Remove the image from memory
		 * @return void
		 */
		private function destroyImage()
		{
			if (is_resource($this->_image)) {
				imagedestroy($this->_image);
			}
		}
		
		/**
		 * Remove the image from memory
		 * @return void
		 */
		public function __destruction()
		{
			$this->destroyImage();
		}
	}
?>