<?php
	/**
	 * Image Handler
	 * 
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scImageHandler {
		const JPEG_QUALITY = 80;
		
		/**
		 * Returns array with width and height
		 * 
		 * @param string $imageFilename
		 * @return array($width, $height)
		 */
		public static function getImageSize($imageFilename)
		{
			list($width, $height) = @getimagesize($imageFilename);
			return array($width, $height);
		}
		
		public static function getImageWidth($source)
		{
			list($width) = @getimagesize($source);
			return $width;
		}
		
		public static function getImageHeight($source)
		{
			list($width, $height) = @getimagesize($source);
			return $height;
		}
		
		/**
		 * Resize image $source into $dest, having maximum size of $maxWidth by $maxHeight, keeping the aspect ratio
		 * 
		 * @throws scImageException
		 */
		public static function resizeImage($source, $dest, $maxWidth = 0, $maxHeight = 0)
		{
			$image = $resized = null;
			
			try {
			    list($origWidth, $origHeight, $imagetype) = @getimagesize($source);
	
				// if image dimensions could not be taken, bail out		    
			    if (!$origWidth || !$origHeight)
					throw new scImageException(sprintf(_t('File "%s" is not an image.'), $source), scImageException::EXCEPTION_FILE_NOT_AN_IMAGE);
				
			    // If the uploaded image fits don't resize the image
			    if (($origWidth <= $maxWidth) and ($origHeight <= $maxHeight)) {
			    	if ($source !== $dest) {
				    	$copyResult = copy($source, $dest);
				    	if ($copyResult != true) {
				    		throw new scImageException(_t('Image copying failed.'), scImageException::EXCEPTION_IMAGE_WRITE_ERROR);
				    	}
			    	}
			    	@chmod($dest, 0644);
			    	return;
			    }
			    
				list($width, $height) = self::getImageFitSize($origWidth, $origHeight, $maxWidth, $maxHeight);
				
			    // If it's GIF create palette-based image, otherwise true color
			    // PNGs can be either true color or palette, so use true color
			    if (function_exists('imagecreatetruecolor') and ($imagetype != IMAGETYPE_GIF))
			        $resized = imagecreatetruecolor($width, $height); // GD2 only
			    else
			        $resized = imagecreate($width, $height);
			    
			    switch ($imagetype) {
			        case IMAGETYPE_GIF:
			            $image = imagecreatefromgif($source);
						break;
						
					case IMAGETYPE_JPEG:
						$image = imagecreatefromjpeg($source);
						break;
					
					case IMAGETYPE_PNG:
						$image = imagecreatefrompng($source);
						break;
					
					default:
						throw new scImageException(_t('Invalid image format.'), scImageException::EXCEPTION_INVALID_IMAGE_FORMAT);
			    }
				
			    // Preserve transparency for PNGs
			    if ($imagetype == IMAGETYPE_PNG) {
			    	imagealphablending($resized, false);
			    	imagesavealpha($resized, true);
			    }
			    
			    // For transparent GIFs preserve the transparent color
			    if ($imagetype == IMAGETYPE_GIF) {
					$colorTransparent = imagecolortransparent($image);
					if ($colorTransparent >= 0) {
						imagepalettecopy($resized, $image);
						imagefill($resized, 0, 0, $colorTransparent);
						imagecolortransparent($resized, $colorTransparent);
					}
			    }
			    
			    if (function_exists('imagecopyresampled')) {
			        $resizeResult = @imagecopyresampled($resized, $image, 0, 0, 0, 0, $width, $height, $origWidth, $origHeight); // GD2 only
			    } else {
			        $resizeResult = @imagecopyresized($resized, $image, 0, 0, 0, 0, $width, $height, $origWidth, $origHeight);
			    }
			    
				if (!$resizeResult) {
					throw new scImageException(_t('Image resizing failed.'), scImageException::EXCEPTION_IMAGE_RESIZE_ERROR);
				}
				
				$destinationDir = dirname($dest);
				if (!file_exists($destinationDir)) {
					$oldumask = umask(0);
					// Create the directory recursively
					if (false === @mkdir($destinationDir, 0777, true)) {
						umask($oldumask);
						throw new miException(sprintf(_t('The destination directory "%s" cannot be created.'), $destinationDir));
					}
					umask($oldumask);
				} else if (!is_dir($destinationDir)) {
					throw new miException(sprintf(_t('"%s" is not a directory.'), $destinationDir));
				} else if (!is_writable($destinationDir) && (false === @chmod($destinationDir, 0777))) {
					throw new miException(sprintf(_t('"%s" is not writable.'), $destinationDir));
				}
				
				switch ($imagetype) {
			        case IMAGETYPE_GIF:
						$res = imagegif($resized, $dest);
						break;
					
					case IMAGETYPE_JPEG:
						$res = imagejpeg($resized, $dest, self::JPEG_QUALITY);
						break;
					
					case IMAGETYPE_PNG:
						$res = imagepng($resized, $dest);
						break;
					
					default:
						throw new scImageException(_t('Invalid image format.'), scImageException::EXCEPTION_INVALID_IMAGE_FORMAT);
			    }
				if (!$res)
					throw new scImageException(_t('Image write error.'), scImageException::EXCEPTION_IMAGE_WRITE_ERROR);
				
				// Try chmod and ignore any errors
				@chmod($dest, 0644);
				
			} catch (Exception $exception) {
				// Cleanup
				if ($image !== null)
					imagedestroy($image);
				
				if ($resized !== null)
					imagedestroy($resized);
				
				throw $exception;
			}
			
			imagedestroy($image);
			imagedestroy($resized);
		}
		
		/**
		 * Returns the maximum width and height for the image that will fit into $maxWidth and $maxHeight, keeping the aspect ratio
		 * 
		 * @return array($width, $height) where $width and $height are integers
		 */
		public static function getImageFitSize($originalWidth, $originalHeight, $maxWidth, $maxHeight)
		{
			$width = $originalWidth;
		    $height = $originalHeight;
		
			// Taller
			if ($maxHeight && ($height > $maxHeight)) {
			    $width = ($maxHeight / $height) * $width;
			    $height = $maxHeight;
			}
		
		    // Wider
		    if ($maxWidth && ($width > $maxWidth)) {
		        $height = ($maxWidth / $width) * $height;
		        $width = $maxWidth;
		    }
		    
		    if ($width == 0)
		    	$width = $maxWidth;
		    if ($height == 0)
		    	$height = $maxHeight;
		    
		    return array(intval($width), intval($height));
		}
		
		/**
		 * Returns the correct extension based on the type of the image
		 * @param string $imageFilename
		 * @return string the extension
		 * @throws miException if image is not gif/png/jpg
		 */
		public static function getImageExtension($imageFilename)
		{
			list(,, $imagetype) = @getimagesize($imageFilename);
			switch ($imagetype) {
				case IMAGETYPE_GIF: return '.gif';
				case IMAGETYPE_PNG: return '.png';
				case IMAGETYPE_JPEG: return '.jpg';
			}
			throw new miException(_i('Unknown image type.'));
		}
	}
?>