<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSeo {
		
		const TITLE = 'Title';
		const KEYWORDS = 'Keywords';
		const DESCRIPTION = 'Description';
		const BODY_CLASSES = 'BodyClass';
		
		protected $_description = array();
		protected $_keywords = array();
		protected $_pageTitle = '';
		protected $_bodyClasses = array();
		protected $_pageName = '';
		
		/**
		 * Create scSeo object
		 * 
		 * @param string $title the title of the page
		 */
		public function __construct()
		{
			$this->_pageTitle = scSettings::getSettingValue(scSettings::SETTING_SEO_SITE_TITLE);
			if (empty($this->_pageName)) {
				$this->_pageName = _t('Customer Area');
			}
			$this->initMetaData();
		}
		
		public function getMetaData()
		{
			// Clean up empty values before implode
			$this->_description = array_filter($this->_description);
			$this->_keywords = array_filter($this->_keywords);
			$this->_bodyClasses = array_unique($this->_bodyClasses);
			
			$result[self::TITLE] = $this->getPageTitle();
			$result[self::BODY_CLASSES] = implode(' ', $this->_bodyClasses);
			$result[self::DESCRIPTION] = implode(' ', array_reverse($this->_description));
			$result[self::KEYWORDS] = implode(',', array_reverse($this->_keywords));
			
			return $result;
		}
		
		public function isMultilanguage()
		{
			return false;
		}
		
		/**
		 * Return the params needed to build the current URL.
		 */
		public function getUrlParams()
		{
			throw new miException(_i('Not implemented.')); // FIXME
		}
		
		protected function getPageTitle()
		{
			$t = new scTemplateParser();
			$t->setContents($this->_pageTitle);
			$this->assignPageTitleVars($t);
			return $t->templateParse();
		}
		
		protected function assignPageTitleVars(miTemplateParser $t)
		{
			$t->assign('%%COMPANY_NAME%%', scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME));
			$t->assign('%%PAGE_NAME%%', $this->_pageName);
		}
		
		protected function initMetaData()
		{
			$this->_description[] = scSettings::getSettingValue(scSettings::SETTING_SEO_SITE_DESCRIPTION);
			$this->_keywords[] = scSettings::getSettingValue(scSettings::SETTING_SEO_SITE_KEYWORDS);
			
			$this->addBodyClasses();
		}
		
		protected function addMetaData($metaData)
		{
			foreach ($metaData as $record) {
				if (!empty($record['Description'])) {
					$this->_description[] = $record['Description'];
				}
				if (!empty($record['Keywords'])) {
					$this->_keywords[] = $record['Keywords'];
				}
				if (!empty($record['Title'])) {
					$this->_pageTitle = $record['Title'];
				}
				if (!empty($record['PageName'])) {
					$this->_pageName = $record['PageName'];
				}
				if (!empty($record['CMSLayoutID'])) {
					$layoutCssClassNames = explode(' ', trim(scLayout::getLayoutCssClassNames($record['CMSLayoutID'])));
					$layoutCssClassNames = array_filter($layoutCssClassNames);
					foreach ($layoutCssClassNames as $layoutCssClassName)
						$this->_bodyClasses[] = 'layout-' . $layoutCssClassName;
				}
			}
		}
		
		protected function addBodyClasses()
		{
		}
	}
	
	class scDynamicPageSeo extends scSeo {
		
		protected $_path;
		
		public function __construct($pagePath, $pageName = false)
		{
			$this->_path = $pagePath;
			if ($pageName !== false) {
				$this->_pageName = $pageName;
			}
			parent::__construct();
		}
		
		public function getPagePath()
		{
			return $this->_path;
		}
		
		public function isMultilanguage()
		{
			$pages = scFriendlyUrl::getLinkableDynamicPages();
			return in_array($this->_path, $pages);
		}
		
		public function getUrlParams()
		{
			$pages = scFriendlyUrl::getLinkableDynamicPages();
			$pageType = array_search($this->_path, $pages);
			
			return array(
				scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_DYNAMIC_PAGE,
				scSeoUrlTypes::PAGE_ID => $pageType,
			);
		}
		
		protected function initMetaData()
		{
			parent::initMetaData();
			
			$seo = miStaticDBUtil::execSelect(
				'SELECT CMSDynamicPageSEODescription AS Description, CMSDynamicPageSEOKeywords AS Keywords, CMSDynamicPageSEOTitle AS Title, CMSDynamicPageTitle AS PageName, CMSLayoutID FROM CMSDynamicPages ' .
				'INNER JOIN CMSDynamicPagesML USING(CMSDynamicPageID) WHERE CMSDynamicPagePath = ? AND LanguageID = ?',
				array($this->_path, scMultiLanguage::getCurrentLanguageId())
			);
			$this->addMetaData($seo);
		}
		
		protected function addBodyClasses()
		{
			parent::addBodyClasses();
			$this->_bodyClasses[] = 'dynamic-page';
			$this->_bodyClasses[] = 'dynamic-page-' . basename($this->_path, '.php');
		}
	}
	
	class scRichTextPageSeo extends scSeo {
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$seo = miStaticDBUtil::execSelect(
				'SELECT CMSRichTextPageSEODescription AS Description, CMSRichTextPageSEOKeywords AS Keywords, CMSRichTextPageSEOTitle AS Title, CMSRichTextPageTitle AS PageName, CMSLayoutID FROM CMSRichTextPagesML ' .
				'LEFT JOIN CMSRichTextPages USING(CMSRichTextPageID) ' .
				'WHERE CMSRichTextPagesML.CMSRichTextPageID = ? AND LanguageID = ?',
				array(intval(miGetParamDefault('PageID', 0)), scMultiLanguage::getCurrentLanguageId())
			);
			$this->addMetaData($seo);
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('PageID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_RICH_TEXT_PAGE,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('PageID')),
				);
			}
		}
	}
	
	class scFormSeo extends scDynamicPageSeo {
		
		public function __construct()
		{
			parent::__construct('form.php');
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			if (miHasParam('FormID')) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT CMSFormSEOTitle AS Title, CMSFormTitle AS PageName FROM CMSFormsML ' .
					'WHERE CMSFormID = ? AND LanguageID = ?',
					array(intval(miGetParamDefault('FormID', 0)), scMultiLanguage::getCurrentLanguageId())
				);
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('FormID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_DYNAMIC_FORM,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('FormID')),
				);
			}
		}
	}
	
	class scListingSeo extends scDynamicPageSeo {
		
		protected $_listingPageNumber = null;
		
		protected $_listingTotalPages = null;
		
		public function setListingPageInfo($pageNum, $totalPages)
		{
			$this->_listingPageNumber = $pageNum;
			$this->_listingTotalPages = $totalPages;
		}
		
		protected function assignPageTitleVars(miTemplateParser $t)
		{
			parent::assignPageTitleVars($t);
			
			// Init the variables
			$t->assign('%%PAGE_NUMBER%%', '');
			$t->assign('%%FMT_PAGE_NUMBER%%', '');
			
			if (!is_null($this->_listingPageNumber)) {
				$t->assign('%%PAGE_NUMBER%%', $this->_listingPageNumber);
			}
			if (!is_null($this->_listingTotalPages) && ($this->_listingPageNumber > 1)) {
				$pageNumber = sprintf(_t('(page %s of %s)'), $this->_listingPageNumber, $this->_listingTotalPages);
				$t->assign('%%FMT_PAGE_NUMBER%%', $pageNumber);
			}
		}
	}
	
	class scCategorySeo extends scListingSeo {
		
		public function __construct()
		{
			parent::__construct('categories.php', _t('Categories'));
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			if (miHasParam('CategoryID')) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT CategorySEODescription AS Description, CategorySEOKeywords AS Keywords, CategorySEOTitle AS Title, CategoryName AS PageName, CMSLayoutID FROM CategoriesML ' .
					'LEFT JOIN Categories USING(CategoryID) ' .
					'WHERE CategoriesML.CategoryID = ? AND LanguageID = ?',
					array(intval(miGetParamDefault('CategoryID', 0)), scMultiLanguage::getCurrentLanguageId())
				);
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('CategoryID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_CATEGORY,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('CategoryID')),
				);
			}
		}
	}
	
	class scManufacturerSeo extends scListingSeo {
		
		public function __construct()
		{
			parent::__construct('manufacturers.php', _t('Manufacturers'));
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			if (miHasParam('ManufacturerID')) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT ManufacturerSEOTitle AS Title, ManufacturerName AS PageName FROM ManufacturersML ' .
					'WHERE ManufacturerID = ? AND LanguageID = ?',
					array(intval(miGetParamDefault('ManufacturerID', 0)), scMultiLanguage::getCurrentLanguageId())
				);
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('ManufacturerID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_MANUFACTURER,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('ManufacturerID')),
				);
			}
			return array(
				scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_DYNAMIC_PAGE,
				scSeoUrlTypes::PAGE_ID => scFriendlyUrl::DYN_PAGE_MANUFACTURERS,
			);
		}
	}
	
	class scGallerySeo extends scListingSeo {
		
		public function __construct()
		{
			parent::__construct('gallery.php', _t('Gallery'));
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			if (miHasParam('GalleryID')) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT GalleryTitle AS PageName, GallerySEOTitle AS Title FROM GalleriesML ' .
					'WHERE GalleryID = ? AND LanguageID = ?',
					array(intval(miGetParamDefault('GalleryID', 0)), scMultiLanguage::getCurrentLanguageId())
				);
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('GalleryID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_GALLERY,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('GalleryID')),
				);
			}
		}
	}
	
	class scProductSeo extends scDynamicPageSeo {
		
		const CATEGORY_BRANCH_SEPARATOR = ' - ';
		
		/**
		 * @var IProductListingItem
		 */
		protected $_listingItem = null;
		
		public function __construct()
		{
			parent::__construct('product.php');
		}
		
		/**
		 * @return IProductListingItem
		 */
		protected function getListingItem()
		{
			if (!is_null($this->_listingItem)) {
				return $this->_listingItem;
			}
			
			try {
				if (miHasParam('ProductVariantID')) {
					$this->_listingItem = new scProductVariant;
					$this->_listingItem->read(miGetParam('ProductVariantID'));
					return $this->_listingItem;
				}
				
				if (miHasParam('ProductID')) {
					$this->_listingItem = new scProductSL();
					$this->_listingItem->read(miGetParam('ProductID'));
					return $this->_listingItem;
				}
			} catch (miException $e) {
				$this->_listingItem = false;
				return false;
			}
			
			return false;
		}
		
		public function initMetaData()
		{
			parent::initMetaData();
			
			$listingItem = $this->getListingItem();
			if ($listingItem !== false) {
				$seo = miStaticDBUtil::execSelect(
					'SELECT ProductMetaDescription AS Description, ProductMetaKeywords AS Keywords, ProductPageTitle AS Title FROM ProductsML ' .
					'WHERE ProductID = ? AND LanguageID = ?',
					array($listingItem->getProductId(), scMultiLanguage::getCurrentLanguageId())
				);
				$seo[0]['PageName'] = $listingItem->getName();
				$this->addMetaData($seo);
			}
		}
		
		public function isMultilanguage()
		{
			return true;
		}
		
		public function getUrlParams()
		{
			if (miHasParam('ProductVariantID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_PRODUCT_VARIANT,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('ProductVariantID')),
				);
			}
			if (miHasParam('ProductID')) {
				return array(
					scSeoUrlTypes::PAGE_TYPE => scFriendlyUrl::TYPE_PRODUCT,
					scSeoUrlTypes::PAGE_ID => intval(miGetParam('ProductID')),
				);
			}
		}
		
		protected function assignPageTitleVars(miTemplateParser $t)
		{
			parent::assignPageTitleVars($t);
			
			$listingItem = $this->getListingItem();
			if ($listingItem === false) {
				return;
			}
			$categoryBranch = scCategories::getCategoryFullBranch($listingItem->getCategoryId());
			
			$t->assign('%%CATEGORY_NAME%%', end($categoryBranch));
			$t->assign('%%CATEGORY_BRANCH%%', implode(self::CATEGORY_BRANCH_SEPARATOR, $categoryBranch));
		}
	}
?>