<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCart {
		
		/**
		 * @var array
		 */
		protected $_cartItems = array();
		
		/**
		 * @var array
		 */
		protected $_customerInfo = array();
		
		/**
		 * @var scOrderTotal
		 */
		protected $_orderTotal;
		
		/**
		 * @var scCurrency
		 */
		protected $_currentCurrency;
		
		/**
		 * @var int
		 */
		protected $_hashSeed = 0;
		
		/**
		 * @var array
		 */
		protected $_coupon = null;
		
		/**
		 * @var int|null
		 */
		protected $_freeShippingMethodId = null;
		
		/**
		 * Constructs the Cart
		 * Fills in the cart items from session
		 */
		public function __construct()
		{
			$this->_orderTotal = new scOrderTotal($this);
			$this->initCustomerInfo();
		}
		
		public function initCustomerInfo()
		{
			$this->_customerInfo = array(
				'CustomerEmail' => '',
				'ShippingModuleAndMethod' => 0,
				'PaymentModuleID' => 0,
				
				'CustomerBillingFirstName' => '',
				'CustomerBillingLastName' => '',
				'CustomerBillingCompany' => '',
				'CustomerBillingPhone' => '',
				'CustomerBillingFax' => '',
				'CustomerBillingAddress1' => '',
				'CustomerBillingAddress2' => '',
				'CustomerBillingCity' => '',
				'CustomerBillingCityID' => '',
				'CustomerBillingState' => '',
				'CustomerBillingCountry' => '',
				'CustomerBillingZip' => '',
				'CustomerBillingVatNumber' => '',
				
				'CustomerShipToBillingAddress' => true,
				'CustomerShippingFirstName' => '',
				'CustomerShippingLastName' => '',
				'CustomerShippingCompany' => '',
				'CustomerShippingPhone' => '',
				'CustomerShippingFax' => '',
				'CustomerShippingAddress1' => '',
				'CustomerShippingAddress2' => '',
				'CustomerShippingCity' => '',
				'CustomerShippingCityID' => '',
				'CustomerShippingState' => '',
				'CustomerShippingCountry' => '',
				'CustomerShippingZip' => '',
				'CustomerShippingVatNumber' => '',
			);
		}
		
		public function __destruct()
		{
			$_SESSION['Cart'] = serialize($this);
		}
		
		/**
		 * Returns the Cart object
		 * There is only one Cart object - Singleton pattern
		 *
		 * @return scCart the cart object
		 */
		public static function getCart()
		{
			static $cart = null;
			
			if ($cart == null) {
				if (isset($_SESSION['Cart']))
					$cart = unserialize($_SESSION['Cart']);
				else
					$cart = new scCart;
				$cart->setCurrentCurrency(scCurrencyUtil::getCurrentCurrency());
			}
			
			return $cart;
		}
		
		/**
		 * Update the hash seed effectively changing the cart hash
		 */
		public function incHashSeed()
		{
			$this->_hashSeed++;
		}
		
		/**
		 * Returns a hash of the cart state
		 *
		 * @return string
		 */
		public function getCartHash()
		{
			return md5(serialize($this->_cartItems) . serialize($this->_customerInfo) .
				serialize($this->_currentCurrency->getCurrencyId()) . serialize($this->_coupon) .
				serialize($this->_freeShippingMethodId) . $this->_hashSeed);
		}
		
		/**
		 * Returns the items in the cart
		 *
		 * @return array the items
		 */
		public function getItems()
		{
			return $this->_cartItems;
		}
		
		/**
		 * Returns item with the given $cartItemCode
		 * If the item is not found returns null
		 *
		 * @param string $cartItemCode
		 * @return scCartItem|null the cart item object, or null
		 */
		public function getItem($cartItemHash)
		{
			if (isset($this->_cartItems[$cartItemHash]))
				return $this->_cartItems[$cartItemHash];
			
			return null;
		}
		
		/**
		 * Returns the number of items in the cart
		 *
		 * @return int the number of items
		 */
		public function getItemsCount()
		{
			return count($this->_cartItems);
		}
		
		/**
		 * Adds new item into the cart
		 * If an item with the same cartItemCode already exists its quantity will be increased
		 *
		 * @param scCartItem $cartItem
		 */
		public function addItem(scCartItem $cartItem)
		{
			$hash = $cartItem->getHash();
			if (!array_key_exists($hash, $this->_cartItems))
				$this->_cartItems[$hash] = $cartItem;
			else {
				$quantity = $this->_cartItems[$hash]->getQuantity() + $cartItem->getQuantity();
				$this->_cartItems[$hash]->setQuantity($quantity);
			}
		}
		
		/**
		 * Removes item from the cart
		 *
		 * @param string $cartItemCode
		 * @return void
		 */
		public function removeItem($cartItemHash)
		{
			if (array_key_exists($cartItemHash, $this->_cartItems))
				unset($this->_cartItems[$cartItemHash]);
			if (empty($this->_cartItems)) {
				$this->removePaymentAndShipping();
			}
		}
		
		/**
		 * Removes all items from the cart
		 */
		public function removeAllItems()
		{
			$this->_cartItems = array();
			$this->removeCoupon();
			$this->removePaymentAndShipping();
		}
		
		/**
		 * Returns TRUE if all cart items are intangible and FALSE if there is at least one tangible.
		 * @return bool
		 */
		public function isAllIntangible()
		{
			foreach ($this->_cartItems as $cartItem) {
				if ($cartItem->isTangible()) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * Returns TRUE if all cart items are tangible and FALSE if there is at least one intangible.
		 * @return bool
		 */
		public function isAllTangible()
		{
			foreach ($this->_cartItems as $cartItem) {
				if (!$cartItem->isTangible()) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * Sets the currency that will be used for computing the cart amounts
		 * It could be the same as the base currency
		 *
		 * @param scCurrency $currency
		 */
		public function setCurrentCurrency(scCurrency $currency)
		{
			$this->_currentCurrency = $currency;
		}
		
		/**
		 * Return the current currency
		 *
		 * @return scCurrency
		 */
		public function getCurrentCurrency()
		{
			return $this->_currentCurrency;
		}
		
		/**
		 * Returns the order total object
		 *
		 * @access public
		 * @return scOrderTotal the order total lines
		 */
		public function getOrderTotal()
		{
			return $this->_orderTotal;
		}
		
		/**
		 * Returns the subtotal - the sum of all items in the cart
		 * Any options will be applied too
		 * 
		 * @return scMultiCurrencyAmount the subtotal
		 */
		public function getSubtotal()
		{
			$subtotal = scMultiCurrencyAmount::getZero();
			foreach ($this->_cartItems as $cartItem) {
				$subtotal = scMultiCurrencyAmountMath::add($subtotal, $cartItem->getTotal());
			}
			return $subtotal;
		}
		
		/**
		 * Returns the declared value for shipping in the asked currency
		 *
		 * @return float
		 */
		public function getDeclaredValueForShipping(scCurrency $currency)
		{
			return $this->_orderTotal->getDeclaredValueForShipping($currency);
		}
		
		/**
		 * Returns the customer who is using the cart
		 *
		 * @return Customer the customer
		 */
		public function getCustomerInfo()
		{
			return $this->_customerInfo;
		}
		
		/**
		 * Sets the customer info
		 *
		 * @param array $customerInfo
		 */
		public function setCustomerInfo($customerInfo)
		{
			$this->_customerInfo = array_merge($this->_customerInfo, $customerInfo);
		}
		
		/**
		 * Set customer info field
		 * 
		 * @param string $fieldName
		 * @param string $fieldValue
		 */
		public function setCustomerInfoField($fieldName, $fieldValue)
		{
			$this->_customerInfo[$fieldName] = $fieldValue;
		}
		
		/**
		 * Returns the shipping address
		 *
		 * @return scAddress the shipping address
		 */
		public function getShippingAddress()
		{
			$address = new scAddress();
			if (isset($this->_customerInfo['CustomerShipToBillingAddress']) && $this->_customerInfo['CustomerShipToBillingAddress']) {
				$address->setAddressLine1($this->_customerInfo['CustomerBillingAddress1']);
				$address->setAddressLine2($this->_customerInfo['CustomerBillingAddress2']);
				$address->setCity($this->_customerInfo['CustomerBillingCity']);
				$address->setCityId($this->_customerInfo['CustomerBillingCityID']);
				$address->setStateId($this->_customerInfo['CustomerBillingState']);
				$address->setPostalCode($this->_customerInfo['CustomerBillingZip']);
				$address->setCountryId($this->_customerInfo['CustomerBillingCountry']);
			} else if (isset($this->_customerInfo['CustomerShippingAddress1'])) { // Check if customer info is set
				$address->setAddressLine1($this->_customerInfo['CustomerShippingAddress1']);
				$address->setAddressLine2($this->_customerInfo['CustomerShippingAddress2']);
				$address->setCity($this->_customerInfo['CustomerShippingCity']);
				$address->setCityId($this->_customerInfo['CustomerShippingCityID']);
				$address->setStateId($this->_customerInfo['CustomerShippingState']);
				$address->setPostalCode($this->_customerInfo['CustomerShippingZip']);
				$address->setCountryId($this->_customerInfo['CustomerShippingCountry']);
			}
			return $address;
		}
		
		/**
		 * Refreshes the customer address and email from the data in the database
		 */
		public function refreshCustomerInfo($customer)
		{
			if (!$customer)
				return;
			
			$customerInfo = array_merge($customer->getRow(), $customer->getBillingAndShippingAddresses());
			$this->setCustomerInfo($customerInfo);
			
			$this->checkMinimumPoints();
			$this->resetWholesalePrices();
			$this->applyPriceModifications();
		}
		
		/**
		 * Returns the shipping module id
		 *
		 * @return int the shipping module id
		 */
		public function getShippingModuleId()
		{
			if (isset($this->_customerInfo['ShippingModuleAndMethod']))
				return current(explode('-', $this->_customerInfo['ShippingModuleAndMethod']));
			return 0;
		}
		
		/**
		 * Returns the shipping method
		 *
		 * @return int the shipping method
		 */
		public function getShippingMethod()
		{
			if (isset($this->_customerInfo['ShippingModuleAndMethod'])) {
				$shippingMethod = explode('-', $this->_customerInfo['ShippingModuleAndMethod']);
				return next($shippingMethod);
			}
			return 0;
		}
		
		/**
		 * Returns the payment method
		 *
		 * @return int the PaymentModuleID, or 0 if unknown
		 */
		public function getPaymentMethod()
		{
			if (isset($this->_customerInfo['PaymentModuleID']))
				return $this->_customerInfo['PaymentModuleID'];
			return 0;
		}
		
		/**
		 * Returns the total weight of all items in the cart
		 *
		 * @param int $weightUnits the weight units
		 * @return float the total weight
		 */
		public function getTotalWeight($weightUnits)
		{
			$weight = 0;
			foreach ($this->_cartItems as $item) {
				if ($item->isTangible())
					$weight += $item->getQuantity() * $item->getWeight()->getWeightInUnits($weightUnits);
			}
			return $weight;
		}
		
		/**
		 * Applies a coupon to the cart items
		 * Throws exception to indicate error condition
		 *
		 * @param string $couponCode
		 * @return void
		 * @throws miException
		 */
		public function applyCoupon($couponCode)
		{
			// Check if coupon can be used
			if (!$this->canApplyCoupon())
				throw new miException(_t('You already used a coupon.'));
				
			// Returns the coupon info. If error occurs throws exception
			$coupon = $this->getCouponInfo($couponCode);
			
			if (!scCouponsContainer::singleton()->isCouponValid($coupon, $this))
				throw new miException(_t('Not all conditions for this coupon are met.'));
				
			$this->_coupon = $coupon;
			$this->applyPriceModifications();
		}
		
		/**
		 * Checks if coupon can be applied
		 * If coupon module is not active trows exception
		 *
		 * @return bool
		 * @throws miException
		 */
		public function canApplyCoupon()
		{
			$moduleId = scModulesFactory::getModuleIdByName('com.summercart.ordertotal.coupon');
			$couponsModule = scModulesFactory::getImplementation($moduleId, 'IOrderTotalModule');
			
			if ($couponsModule->isActive()) {
				if (scCart::getCart()->hasCoupon()) {
					if (scCouponsContainer::singleton()->isCouponValid(scCart::getCart()->getCoupon(), $this))
						return false;		// If the coupon is valid no other coupon can be used
					else {
						// Coupon is not valid remove it from the cart and indicate that the user can use another
						scCart::getCart()->removeCoupon();
						return true;
					}
				}
				else
					return true;
			}
			else
				throw new miException(_t('Coupon module is not active.'));
		}
		
		/**
		 * Checks if coupon is already applyed
		 *
		 * @return bool
		 */
		public function hasCoupon()
		{
			return !is_null($this->_coupon);
		}
		
		/**
		 * Removes the applyed coupon
		 *
		 * @return void
		 */
		public function removeCoupon()
		{
			$this->_coupon = null;
			$this->checkMinimumPoints();
			$this->resetWholesalePrices();
			$this->applyPriceModifications();
		}
		
		/**
		 * Returns TRUE if at least one item has a price that differs from its base price.
		 * 
		 * @return bool
		 */
		public function hasPerItemDiscount()
		{
			foreach ($this->_cartItems as $cartItem) {
				$baseAmount = $cartItem->getPrice(scCartItem::PRICE_TYPE_OPTIONS_MODIFIED)->getAmount();
				$finalAmount = $cartItem->getPrice(scCartItem::PRICE_TYPE_MODIFIED)->getAmount();
					
				if (!$baseAmount->equals($finalAmount)) {
					return true;
				}
			}
			return false;
		}
		
		public function removePaymentAndShipping()
		{
			$this->_customerInfo['ShippingModuleAndMethod'] = '';
			$this->_customerInfo['PaymentModuleID'] = 0;
		}
		
		/**
		 * Increases the usage count of the coupon, and removes it from the cart
		 * This function can be called multiple times. It will increase the usage count only once
		 */
		public function markCouponUsed()
		{
			if ($this->_coupon == null)
				return;
			
			miStaticDBUtil::execSQL('UPDATE Coupons SET CouponUsageCount = CouponUsageCount + 1 WHERE CouponCode = "' . sql_escape_string($this->_coupon['CouponCode']) . '"');
			$this->removeCoupon();
		}
		
		/**
		 * Get coupon information. If coupon code is not valid throws exception
		 *
		 * @param string $couponCode
		 * @return array
		 * @throws miException
		 */
		protected function getCouponInfo($couponCode)
		{
			return scCouponsContainer::singleton()->getCoupon($couponCode);
		}
		
		/**
		 * Get used coupon
		 *
		 * @return void
		 */
		public function getCoupon()
		{
			return $this->_coupon;
		}
		
		public function setFreeShippingMethodId($shippingMethod)
		{
			$this->_freeShippingMethodId = $shippingMethod;
		}
		
		public function getFreeShippingMethodId()
		{
			return $this->_freeShippingMethodId;
		}
		
		/*
		 * Get the total quantity of the products in the cart
		 */
		public function getTotalQuantity()
		{
			$totalQuantity = 0;
			foreach ($this->getItems() as $item) {
				$totalQuantity += $item->getQuantity();
			}
			
			return $totalQuantity;
		}
		
		public function applyPriceModifications()
		{
			foreach ($this->_cartItems as $cartItem) {
				$cartItem->applyPriceModification();
			}
		}
		
		public function resetWholesalePrices()
		{
			foreach ($this->_cartItems as $cartItem) {
				$cartItem->setPrice(ICartItem::PRICE_TYPE_WHOLESALE, null);
			}
		}
		
		/**
		 * Checks if the custom has enough points
		 * Throws miException if not
		 * 
		 * @param int $requiredPoints additional required points, beyond the points already required for items in cart
		 * @param string $addedItemName
		 * @throws miException
		 */
		public function checkMinimumPoints($requiredPoints = 0, $addedItemName = '')
		{
			$customer = $this->getCustomerAuth()->getCustomer();
			if (($customer === false) && ($requiredPoints > 0)) {
				throw new miException(sprintf(_t('"%s" is only available for registered customers.'), $addedItemName));
			}
			foreach ($this->_cartItems as $cartItem) {
				if ($cartItem->getBonusPointsMode() != scBonusPoints::PRODUCT_POINTS_ONLY) {
					continue;
				}
				if ($customer === false) {
					throw new miException(sprintf(_t('"%s" is only available for registered customers.'), $cartItem->getName()));
				}
				$requiredPoints += $cartItem->getPriceInPoints() * $cartItem->getQuantity();
			}
			if ($requiredPoints <= 0) {
				return;
			}
			if (!$this->hasCustomerPoints($customer, $requiredPoints)) {
				throw new miException(_t('You have insufficient bonus points.'));
			}
		}
		
		/**
		 * @return scCustomerAuth
		 */
		protected function getCustomerAuth()
		{
			return scCustomerAuth::singleton();
		}
		
		/**
		 * @return bool
		 */
		protected function hasCustomerPoints(scCustomerDomainObject $customer, $requiredPoints)
		{
			if ($customer->get('CustomerBonusPoints') < $requiredPoints) {
				return false;
			}
			return true;
		}
	}
?>