<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Each item in the Cart is subclass of scCartItem
	 */
	class scCartItem implements ICartItem {
		
		/**
		 * @var scCartItemInfo
		 */
		protected $_info = null;
		
		/**
		 * @var int
		 */
		protected $_quantity = 0;
		
		/**
		 * @var scWeight
		 */
		protected $_weightObj = null;
		
		/**
		 * @var string
		 */
		protected $_cartItemCode = '';
		
		protected $_hash = null;
		
		/**
		 * All the prices of the cart item, keyed by PRICE_TYPE_*.
		 * @var scPrice[]
		 */
		protected $_prices = array(
			self::PRICE_TYPE_BASE => null,
			self::PRICE_TYPE_WHOLESALE => null,
			self::PRICE_TYPE_OPTIONS_MODIFIED => null,
			self::PRICE_TYPE_MODIFIED => null,
		);
		
		/**
		 * @var int
		 */
		protected $_orderItemBonusPoints = 0;
		
		public function __construct(scCartItemInfo $cartItemInfo)
		{
			$this->_info = $cartItemInfo;
			
			$listingItem = $cartItemInfo->getListingItem();
			
			$this->setCartItemCode($listingItem->getProductId());
			$this->setWeight($listingItem->getWeight());
			$this->setPrice(self::PRICE_TYPE_BASE, $listingItem->getPrice());
			$this->setQuantity($cartItemInfo->getQuantity());
			
			if (is_null($this->_hash))
				$this->rehash();
		}
		
		public function setQuantity($quantity)
		{
			$this->_quantity = $quantity;
		}
		
		public function getQuantity()
		{
			return $this->_quantity;
		}
		
		/**
		 * Set the price object with the item price.
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @param scPrice $priceObj
		 */
		public function setPrice($priceType, scPrice $price = null)
		{
			$this->_prices[$priceType] = $price;
		}
		
		/**
		 * Gets the price object with the item price
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @return scPrice
		 */
		public function getPrice($priceType = self::PRICE_TYPE_MODIFIED)
		{
			return $this->_prices[$priceType];
		}
		
		/**
		 * @return scMultiCurrencyAmount
		 */
		public function getTotal()
		{
			$amount = $this->getPrice()->getAmount();
			return scMultiCurrencyAmountMath::multiply($amount, $this->_quantity);
		}
		
		/**
		 * Sets the unique code of the cart item
		 * If two items have the same code they are considered the same
		 */
		public function setCartItemCode($cartItemCode)
		{
			$this->_cartItemCode = $cartItemCode;
			$this->rehash();
		}
		
		/**
		 * Retrieves the cart item code
		 */
		public function getCartItemCode()
		{
			return $this->_cartItemCode;
		}
		
		/**
		 * Should return true if the item is tangible.
		 */
		public function isTangible()
		{
			return false;
		}
		
		public function setWeight(scWeight $weightObj)
		{
			$this->_weightObj = $weightObj;
		}
		
		public function getWeight()
		{
			if ($this->_weightObj == null)
				throw new miException(_t('Retrieving weight of intangible item.'));
			return $this->_weightObj;
		}
		
		/**
		 * Returns the hash for the cart item - used for item comparisons
		 */
		public function getHash()
		{
			return $this->_hash;
		}
		
		/**
		 * @return array
		 */
		public function getImageInfo()
		{
			return new scProductNoImageImageInfo();
		}
		
		/**
		 * Return the mode for paying with bonus points
		 * @return int
		 */
		public function getBonusPointsMode()
		{
			return scBonusPoints::PRODUCT_MONEY_ONLY;
		}
		
		/**
		 * Returns the number of points that the product costs.
		 * Note that points will be returned no matter the value of getBonusPointsMode()
		 * @return int
		 */
		public function getPriceInPoints()
		{
			return 0;
		}
		
		/**
		 * Get the points conversion of rate - how much money for 1 point
		 * @return float rounded to 4 digits after the decimal separator
		 */
		public function getPointsRate()
		{
			return 1;
		}
		
		/**
		 * Get the name of the cart item.
		 */
		public function getName()
		{
			throw new miException(_i('Not implemeneted.'));
		}
		
		/**
		 * @param int $bonusPoints
		 */
		public function setOrderItemBonusPoints($bonusPoints = 0)
		{
			$this->_orderItemBonusPoints = $bonusPoints;
		}
		
		/**
		 * @return int
		 */
		public function getOrderItemBonusPoints()
		{
			return $this->_orderItemBonusPoints;
		}
		
		/**
		 * Reinitializes the hash for the cart item
		 */
		protected function rehash()
		{
			if (empty($this->_cartItemCode))
				$this->_hash = null;
			
			$this->_hash = md5($this->serialize());
		}
		
		/**
		 * Override when there are other variables on which the object comparison depends
		 *
		 * @return string
		 */
		protected function serialize()
		{
			return $this->_cartItemCode;
		}
	}
	
	/**
	 * A product in the cart
	 */
	class scCartItemProduct extends scCartItem {
		
		/**
		 * @var int
		 */
		protected $_productId;
		
		/**
		 * @var array
		 */
		protected $_productInfo;
		
		/**
		 * Array with the product options selected
		 *
		 * @var array
		 */
		protected $_options = array();
		
		/**
		 * Contains the individual/local options
		 * @var array
		 */
		protected $_localOptions = array();
		
		public function __construct(scCartItemInfo $cartItemInfo)
		{
			$this->_productId = $cartItemInfo->getListingItem()->getProductId();
			$this->_productInfo = $cartItemInfo->getProduct()->getRow();
			
			$this->addOptions($cartItemInfo->getOptionsInfo()->getOptions());
			$this->addLocalOptions($cartItemInfo->getOptionsInfo()->getLocalOptions());
			
			parent::__construct($cartItemInfo);
		}
		
		/**
		 * Set the price object with the item price.
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @param scPrice $priceObj
		 */
		public function setPrice($priceType, scPrice $price = null)
		{
			switch ($priceType) {
				case self::PRICE_TYPE_BASE:
					$this->_prices[self::PRICE_TYPE_BASE] = $price;
					$this->_prices[self::PRICE_TYPE_WHOLESALE] = null;
					$this->_prices[self::PRICE_TYPE_OPTIONS_MODIFIED] = null;
					$this->_prices[self::PRICE_TYPE_MODIFIED] = null;
					break;
					
				case self::PRICE_TYPE_WHOLESALE:
					$this->_prices[self::PRICE_TYPE_WHOLESALE] = $price;
					$this->_prices[self::PRICE_TYPE_OPTIONS_MODIFIED] = null;
					$this->_prices[self::PRICE_TYPE_MODIFIED] = null;
					break;
					
				case self::PRICE_TYPE_OPTIONS_MODIFIED:
					$this->_prices[self::PRICE_TYPE_OPTIONS_MODIFIED] = $price;
					$this->_prices[self::PRICE_TYPE_MODIFIED] = null;
					break;
					
				case self::PRICE_TYPE_MODIFIED:
					$this->_prices[self::PRICE_TYPE_MODIFIED] = $price;
					break;
				
				default:
					throw new miException(sprintf(_i('Unknown price type "%s".'), $priceType));
			}
		}
		
		/**
		 * Gets the price object with the item price
		 * 
		 * @param int $priceType PRICE_TYPE_*
		 * @return scPrice
		 */
		public function getPrice($priceType = self::PRICE_TYPE_MODIFIED)
		{
			switch ($priceType) {
				case self::PRICE_TYPE_BASE:
					if (!is_null($this->_prices[self::PRICE_TYPE_BASE]))
						return $this->_prices[self::PRICE_TYPE_BASE];
					$listingItem = $this->_info->getListingItem();
					$basePrice = $listingItem->getPrice();
					$this->setPrice(self::PRICE_TYPE_BASE, $basePrice);
					return $basePrice;
					
				case self::PRICE_TYPE_WHOLESALE:
					if (!is_null($this->_prices[self::PRICE_TYPE_WHOLESALE]))
						return $this->_prices[self::PRICE_TYPE_WHOLESALE];
					$listingItem = $this->_info->getListingItem();
					$wholesalePrice = scWholesalePrices::getListingItemWholesalePrice($listingItem, $this->_quantity);
					$this->setPrice(self::PRICE_TYPE_WHOLESALE, $wholesalePrice);
					return $wholesalePrice;
					
				case self::PRICE_TYPE_OPTIONS_MODIFIED:
					if (!is_null($this->_prices[self::PRICE_TYPE_OPTIONS_MODIFIED]))
						return $this->_prices[self::PRICE_TYPE_OPTIONS_MODIFIED];
					$wholesalePrice = $this->getPrice(self::PRICE_TYPE_WHOLESALE);
					$priceWithOptions = $this->getOptionsModifiedPrice($wholesalePrice);
					$this->setPrice(self::PRICE_TYPE_OPTIONS_MODIFIED, $priceWithOptions);
					return $priceWithOptions;
					
				case self::PRICE_TYPE_MODIFIED:
					
					if (!is_null($this->_prices[self::PRICE_TYPE_MODIFIED]))
						return $this->_prices[self::PRICE_TYPE_MODIFIED];
					$priceWithOptions = $this->getPrice(self::PRICE_TYPE_OPTIONS_MODIFIED);
					// Set it the current value to prevent recursion as applyPriceModification
					// calls modules which may need the current value.
					$this->_prices[self::PRICE_TYPE_MODIFIED] = $priceWithOptions;
					
					$this->applyPriceModification();
					return $this->_prices[self::PRICE_TYPE_MODIFIED];
					
				default:
					throw new miException(sprintf(_i('Unknown price type "%s".'), $priceType));
			}
		}
		
		/**
		 * Apply the price modifications
		 */
		public function applyPriceModification()
		{
			$priceWithOptions = $this->getPrice(self::PRICE_TYPE_OPTIONS_MODIFIED);
			$this->_prices[self::PRICE_TYPE_MODIFIED] = $priceWithOptions;
			
 			// Calculate the price modifications
 			$priceModifications = $this->getCartItemPriceModifications();
			$modifiedAmount = clone $priceWithOptions->getAmount();
 			
 			// Apply the price modification
 			foreach ($priceModifications as $priceModification) {
 				$modifiedAmount = scMultiCurrencyAmountMath::add($modifiedAmount, $priceModification->getAmount());
 			}
 			
			$this->setPrice(self::PRICE_TYPE_MODIFIED, scPrice::factory($modifiedAmount));
		}
		
		/**
		 * Check if the customer has enough points to the new quantity
		 * @param int $forQuantity new quantity to be set
		 * @throw miException
		 */
		public function checkPointsForNewQuantity($forQuantity)
		{
			// Check if this is a points-only item.
			if ($this->getBonusPointsMode() != scBonusPoints::PRODUCT_POINTS_ONLY) {
				return;
			}
			$addedQuantity = $forQuantity - $this->getQuantity();
			$addedPoints = $this->getPriceInPoints() * $addedQuantity;
			$this->getCart()->checkMinimumPoints($addedPoints, $this->getName());
		}
		
		/**
		 * Sets the quantity
		 * Checks that the product is not "out of stock" and there is enough quantity
		 */
		public function setQuantity($quantity)
		{
			$this->checkIsQuantityAvailable($quantity);
			// Reset the wholesale price so it is calculated based on the new quantity.
			$this->setPrice(self::PRICE_TYPE_WHOLESALE, null);
			parent::setQuantity($quantity);
			
			$this->applyPriceModification();
		}
		
		protected function checkIsQuantityAvailable($quantity)
		{
			// Find all currently added quantity of this product (ignore the options)
			$currentQuantity = 0;
			$items = $this->getCart()->getItems();
			foreach ($items as $item) {
				if ($item instanceof scCartItemProduct) {
					if ($item->getHash() == $this->getHash()) {
						// Skip this item. Its quantity should be considered $quantity.
						continue;
					}
					if ($item->getProductId() == $this->getProductId()) {
						$currentQuantity += $item->getQuantity();
					}
				}
			}
			
			$product = new scProductSL();
			$product->read($this->_productId);
			if ($product->isOutOfStock($currentQuantity + $quantity)) {
				if ($product->get('ProductQuantity') > 0)
					throw new miException(_t(sprintf(_t('There are only %d "%s" left.'), $product->get('ProductQuantity'), $product->getName())));
				else
					throw new miException(sprintf(_t('"%s" is out of stock.'), $product->getName()));
			}
			
			$this->checkPointsForNewQuantity($quantity);
		}
		
		/**
		 * Set the product id
		 *
		 * @param int $productId the product id
		 */
		public function setProductId($productId)
		{
			$this->_productId = $productId;
		}
		
		public function setProductInfoField($field, $value)
		{
			$this->_productInfo[$field] = $value;
		}
		
		public function getName()
		{
			return $this->getProductListingItem()->getName();
		}
		
		/**
		 * Get the product id
		 *
		 * @return int the product id
		 */
		public function getProductId()
		{
			return $this->_productId;
		}
		
		/**
		 * Returns the product info
		 *
		 * @return array an array with product info
		 */
		public function getProductInfo()
		{
			return $this->_productInfo;
		}
		
		/**
		 * @return scProductSL
		 */
		public function getProductListingItem()
		{
			return $this->_info->getListingItem();
		}
		
		/**
		 * @return bool
		 */
		public function isTangible()
		{
			return !(bool)$this->_productInfo['ProductIsEgood'];
		}
		
		/**
		 * Return the price object
		 *
		 * @param boolean $withOptions including the options price changes, defaults to true
		 * @return scPrice
		 * @deprecated Use getPrice() instead.
		 */
		public function getPriceObj($withOptions = true)
		{
			if (!$withOptions) {
				return $this->getPrice(self::PRICE_TYPE_WHOLESALE);
			}
			return $this->getPrice();
		}
		
		/**
		 * Returns the base price without any modifications.
		 * Optionally may apply options' modifications as they are not IProductPriceModifier.
		 * Note: Returns a copy!
		 * @return scPrice
		 * @deprecated Use getPrice() instead.
		 */
		public function getBasePrice()
		{
			return clone $this->getPrice(self::PRICE_TYPE_BASE);
		}
		
		/**
		 * Add product option
		 *
		 * @param array(scProductOption) $options
		 * @return void
		 */
		public function addOptions($options)
		{
			foreach ($options as $option) {
				$optionId = $option->getOptionId();
				if (array_key_exists($optionId, $this->_options))
					return;
				
				$this->_options[$optionId] = $option;
			}
			
			$this->rehash();
		}
		
		/**
		 * Add local option
		 *
		 * @param array(scProductOption) $options
		 * @return void
		 */
		public function addLocalOptions($options)
		{
			foreach ($options as $option) {
				$optionId = $option->getOptionId();
				if (array_key_exists($optionId, $this->_localOptions))
					return;
					
				$this->_localOptions[$optionId] = $option;
			}
			
			$this->rehash();
		}
		
		/**
		 * Return the product options selected
		 *
		 * @return array
		 */
		public function getOptions()
		{
			return $this->_options;
		}
		
		/**
		 * Return the local options selected
		 *
		 * @return array
		 */
		public function getLocalOptions()
		{
			return $this->_localOptions;
		}
		
		public function getEgoods()
		{
			$recordset = new miSqlRecordset('ProductEgoods');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $this->_productId));
			return $recordset->getAllRecords();
		}
		
		/**
		 * @return scProductImageInfo
		 */
		public function getImageInfo()
		{
			return scProductImage::getImage($this->getProductId());
		}
		
		/**
		 * @return string
		 */
		public function getProductCode()
		{
			return $this->_productInfo['ProductCode'];
		}
		
		/**
		 * Return the mode for paying with bonus points
		 * @return int
		 */
		public function getBonusPointsMode()
		{
			return $this->_productInfo['ProductBonusPointsMode'];
		}
		
		/**
		 * Returns the number of points that the product costs.
		 * Note that points will be returned no matter the value of getBonusPointsMode().
		 * @return int
		 */
		public function getPriceInPoints()
		{
			$points = (int)$this->_productInfo['ProductBonusPointsPrice'];
			if ($points > 0) {
				return $points;
			}
			
			return scBonusPoints::getPricePoints($this->getPrice());
		}
		
		/**
		 * Get the points conversion of rate - how much money for 1 point
		 * @return float rounded to 4 digits after the decimal separator
		 */
		public function getPointsRate()
		{
			$points = (int)$this->_productInfo['ProductBonusPointsPrice'];
			if ($points > 0) {
				return round($this->getPrice()->getAmount()->get() / $points, 4);
			}
			return scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_CONVERSION_RATE);
		}
		
		/**
		 * @return scProductPriceModification
		 */
		protected function getCartItemPriceModifications()
		{
			return scProductPrice::getCartItemPriceModifications($this);
		}
		
		/**
		 * Applies options' price modifications to a scPrice object.
		 * Note: Returns a copy!
		 * @return scPrice
		 */
		protected function getOptionsModifiedPrice(scPrice $priceObj)
		{
			$productPrice = $priceObj->getAmount();
			$modifiedAmount = clone $productPrice;
			foreach ($this->_options as $option) {
				$priceChange = $option->getOptionPriceChange($productPrice);
				$modifiedAmount = scMultiCurrencyAmountMath::add($modifiedAmount, $priceChange);
			}
			foreach ($this->_localOptions as $option) {
				$priceChange = $option->getOptionPriceChange($productPrice);
				$modifiedAmount = scMultiCurrencyAmountMath::add($modifiedAmount, $priceChange);
			}
			return scPrice::factory($modifiedAmount);
		}
		
		/**
		 * @return scCart
		 */
		protected function getCart()
		{
			return scCart::getCart();
		}
		
		/**
		 * To determine the product hash we need to take in mind that identical products can have different options selected
		 * which we will have to consider as different products in the shopping cart
		 */
		protected function serialize()
		{
			return $this->_cartItemCode . '-' .
				serialize($this->_options) . '-' .
				serialize($this->_localOptions);
		}
	}
	
	interface ICartItemProductVariant extends ICartItem {
		
		/**
		 * @return scProductVariant
		 */
		public function getProductVariant();
	}
	
	class scCartItemProductVariant extends scCartItemProduct implements ICartItemProductVariant {
		
		/**
		 * @var scProductVariant
		 */
		protected $_productVariant = null;
		
		public function __construct(scCartItemInfo $cartItemInfo)
		{
			$this->_productVariant = $cartItemInfo->getProductVariant();
			
			parent::__construct($cartItemInfo);
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getProductVariant()
		{
			return $this->_productVariant;
		}
		
		/**
		 * @return array
		 */
		public function getImageInfo()
		{
			return $this->_productVariant->getImageInfo();
		}
		
		/**
		 * @return string
		 */
		public function getProductCode()
		{
			return $this->_productVariant->get('ProductVariantCode');
		}
		
		/**
		 * @return int
		 */
		public function getPriceInPoints()
		{
			if ((int)$this->_productVariant->get('ProductVariantBonusPointsPrice') > 0) {
				return (int)$this->_productVariant->get('ProductVariantBonusPointsPrice');
			}
			return scBonusPoints::getPricePoints($this->getPrice());
		}
		
		protected function checkIsQuantityAvailable($quantity)
		{
			if ($this->_productVariant->isOutOfStock($quantity)) {
				if ($this->_productVariant->getQuantityInStock() > 0)
					throw new miException(_t(sprintf(_t('There are only %d "%s" left.'), $this->_productVariant->getQuantityInStock(), $this->getFullName())));
				else
					throw new miException(sprintf(_t('"%s" is out of stock.'), $this->getFullName()));
			}
			
			$this->checkPointsForNewQuantity($quantity);
		}
		
		protected function getFullName()
		{
			$optionsByType = $this->_productVariant->getSelectedOptionsDetails();
			$nameOptions = array();
			foreach ($optionsByType as $type => $options) {
				foreach ($options as $option) {
					$nameOptions[] = ($type == scProductVariantOption::TYPE_LOCAL ? $option['LocalOptionName'] : $option['OptionName']);
				}
			}
			
			return $this->_productVariant->getName() . ' (' . implode(', ', $nameOptions) . ')';
		}
	}
	
	interface ICartItemProductBundle extends ICartItem {
		
		/**
		 * @return ICartItem[]
		 */
		public function getBundledItems();
	}
	
	class scCartItemProductBundle extends scCartItemProduct implements ICartItemProductBundle {
		
		/**
		 * @var scCartItemInfo
		 */
		protected $_cartItemInfo;
		
		protected $_bundledItems = array();
		
		public function __construct(scCartItemInfo $cartItemInfo)
		{
			$bundleProduct = $cartItemInfo->getProduct();
			if (!$bundleProduct->get('ProductIsBundle')) {
				throw new miException(_i('Product is not a bundle.'));
			}
			
			$this->_cartItemInfo = $cartItemInfo;
			
			$this->createBundledItems();
			
			parent::__construct($cartItemInfo);
		}
		
		public function getBundledItems()
		{
			return $this->_bundledItems;
		}
		
		public function setQuantity($quantity)
		{
			parent::setQuantity($quantity);
			
			foreach ($this->_bundledItems as $item) {
				$listingItem = $item->getProductListingItem();
				if ($listingItem instanceof scProductVariant) {
					$product = $listingItem->getProductSL();
				} else {
					$product = $listingItem;
				}
				if (!($product instanceof scBundledProduct)) {
					throw new miException(_i('A bundled item\'s product must be an scBundledProduct.'));
				}
				$item->setQuantity($quantity * $product->getInBundleQuantity());
			}
		}
		
		protected function createBundledItems()
		{
			$oldServiceFactory = scCartItemService::singleton()->getFactory();
			scCartItemService::singleton()->setFactory(scBundlesCartItemServiceFactory::singleton());
			
			$productsInBundle = $this->_cartItemInfo->getProduct()->getBundleProducts();
			$bundleOptions = $this->_cartItemInfo->getOptionsInfo();
			foreach ($productsInBundle as $product) {
				if (!($product instanceof scBundledProduct)) {
					throw new miException(_i('scBundledProduct expected.'));
				}
				$productId = $product->getProductId();
				
				$subItemInfo = new scCartItemInfo();
				$subItemInfo->setProduct($product);
				$subItemInfo->setQuantity($this->_cartItemInfo->getQuantity() * $product->getInBundleQuantity());
				
				$optionsInfo = $bundleOptions->getProductOptionsInfo($productId);
				$subItemInfo->setOptionsInfo($optionsInfo);
				
				$variant = scCartItemService::singleton()->getProductVariantByOptions($productId, $optionsInfo);
				if (!is_null($variant)) {
					$subItemInfo->setProductVariant($variant);
				}
				
				$this->_bundledItems[$productId] = scCartItemService::singleton()->getCartItem($subItemInfo);
			}
			
			scCartItemService::singleton()->setFactory($oldServiceFactory);
		}
		
		/**
		 * Hashes of two bundles should differ when the subitems of the two bundles are different.
		 */
		protected function serialize()
		{
			$subsHashes = array();
			foreach ($this->_bundledItems as $subItem) {
				$subsHashes[] = $subItem->getHash();
			}
			
			return parent::serialize() . '-' . implode('-', $subsHashes);
		}
	}
	
	class scCartItemInfo {
		
		/**
		 * @var scProductSL
		 */
		protected $_product = null;
		
		/**
		 * @var scProductVariant
		 */
		protected $_productVariant = null;
		
		/**
		 * @var int
		 */
		protected $_quantity = 0;
		
		/**
		 * @var scCartItemOptionsInfo
		 */
		protected $_optionsInfo = null;
		
		public function setProduct(scProductSL $product)
		{
			$this->_product = $product;
		}
		
		/**
		 * @return scProductSL
		 */
		public function getProduct()
		{
			return $this->_product;
		}
		
		public function setProductVariant(scProductVariant $productVariant)
		{
			$this->_productVariant = $productVariant;
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getProductVariant()
		{
			return $this->_productVariant;
		}
		
		public function setQuantity($quantity)
		{
			$this->_quantity = (int)$quantity;
		}
		
		/**
		 * @return int
		 */
		public function getQuantity()
		{
			return $this->_quantity;
		}
		
		public function setOptionsInfo(scCartItemOptionsInfo $info)
		{
			$this->_optionsInfo = $info;
		}
		
		/**
		 * @return scCartItemOptionsInfo
		 */
		public function getOptionsInfo()
		{
			return $this->_optionsInfo;
		}
		
		/**
		 * @return IProductListingItem
		 */
		public function getListingItem()
		{
			if (!is_null($this->_productVariant)) {
				return $this->_productVariant;
			}
			return $this->_product;
		}
	}
?>