<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCartItemOptions {
		
		public static function getVariantOptions(scCartItemOptionsInfo $optionsInfo)
		{
			$variantOptions = array();
			
			foreach ($optionsInfo->getOptions() as $option) {
				$optionGroup = $option->getOptionGroup();
				if ($optionGroup->isVariant()) {
					$variantOptions[] = new scProductVariantOption($option->getOptionId(), scProductVariantOption::TYPE_GLOBAL);
				}
			}
			
			foreach ($optionsInfo->getLocalOptions() as $option) {
				$optionGroup = $option->getOptionGroup();
				if ($optionGroup->isVariant()) {
					$variantOptions[] = new scProductVariantOption($option->getOptionId(), scProductVariantOption::TYPE_LOCAL);
				}
			}
			
			return $variantOptions;
		}
	}
	
	interface ICartItemOptionsRetriever {
		
		public function __construct(scCartItemOptionsRetrieverInfo $info);
		
		public function getCartItemOptions();
	}
	
	class scCartItemProductOptionsRetriever implements ICartItemOptionsRetriever {
		
		/**
		 * @var scCartItemOptionsRetrieverInfo
		 */
		protected $_info = null;
		
		/**
		 * @var array
		 */
		protected $_result = null;
		
		public function __construct(scCartItemOptionsRetrieverInfo $info)
		{
			$this->_info = $info;
		}
		
		/**
		 * Returns an scCartItemOptionsInfo object.
		 * Uses different options retrievers for local and global options.
		 * 
		 * @return scCartItemOptionsInfo
		 */
		public function getCartItemOptions()
		{
			if (is_null($this->_result)) {
				$this->_result = new scCartItemOptionsInfo();
				
				$globalOptions = new scCartItemOptionsRetriever($this->_info);
				$globalOptions = $globalOptions->getCartItemOptions();
				$this->_result->setOptions($globalOptions);
				
				$localOptions = new scCartItemLocalOptionsRetriever($this->_info);
				$localOptions = $localOptions->getCartItemOptions();
				$this->_result->setLocalOptions($localOptions);
			}
			
			return $this->_result;
		}
	}
	
	class scCartItemBundleOptionsRetriever implements ICartItemOptionsRetriever {
		
		/**
		 * @var scCartItemOptionsRetrieverInfo
		 */
		protected $_info = null;
		
		/**
		 * @var array
		 */
		protected $_result = null;
		
		protected $_splitBundleOptions = null;
		
		public function __construct(scCartItemOptionsRetrieverInfo $info)
		{
			if (!$info->getProduct()->get('ProductIsBundle')) {
				throw new miException(_i('A product bundle expected.'));
			}
			
			$this->_info = $info;
		}
		
		public function getCartItemOptions()
		{
			if (is_null($this->_result)) {
				$this->_result = new scCartItemBundleOptionsInfo();
				$this->processCartItemOptions();
				$this->processBundledItemsOptions();
			}
			
			return $this->_result;
		}
		
		protected function processCartItemOptions()
		{
			$cartItemOptions = $this->getProductOptions($this->_info->getProduct()->getProductId());
			
			$cartItemOptionsInfo = new scCartItemOptionsRetrieverInfo();
			$cartItemOptionsInfo->setProduct($this->_info->getProduct());
			$cartItemOptionsInfo->setSelectedOptions($cartItemOptions);
			
			$cartItemOptionsRetriever = new scCartItemProductOptionsRetriever($cartItemOptionsInfo);
			$options = $cartItemOptionsRetriever->getCartItemOptions();
			$this->_result->setLocalOptions($options->getLocalOptions());
			$this->_result->setOptions($options->getOptions());
		}
		
		protected function processBundledItemsOptions()
		{
			$bundleProducts = $this->_info->getProduct()->getBundleProducts();
			foreach ($bundleProducts as $product) {
				$productId = $product->getPK();
				$options = $this->getProductOptions($productId);
				
				$subItemOptionsInfo = new scCartItemOptionsRetrieverInfo();
				$subItemOptionsInfo->setProduct($product);
				$subItemOptionsInfo->setSelectedOptions($options);
				
				$options = scCartItemService::singleton()->getCartItemOptions($subItemOptionsInfo);
				$this->_result->setProductOptionsInfo($productId, $options);
			}
		}
		
		protected function getProductOptions($productId)
		{
			if (is_null($this->_splitBundleOptions)) { // Process only once
				$bundleId = $this->_info->getProduct()->getPK(); // The bundle product's ID
				$bundleOptions = $this->_info->getSelectedOptions(); // All the selected options
				
				foreach ($bundleOptions as $optionType => $options) {
					foreach ($options as $optionGroupId => $option) {
						$exploded = explode('-', $optionGroupId);
						if (count($exploded) == 1) {
							// This is an option of the bundle product
							$this->_splitBundleOptions[$bundleId][$optionType][$optionGroupId] = $option;
							continue;
						}
						
						$bundledProductId = array_shift($exploded);
						$optionGroupId = implode('-', $exploded);
						// This is an option of a sub product
						$this->_splitBundleOptions[$bundledProductId][$optionType][$optionGroupId] = $option;
					}
				}
			}
			
			if (!isset($this->_splitBundleOptions[$productId])) {
				return array();
			}
			
			return $this->_splitBundleOptions[$productId];
		}
	}
	
	/**
	 * Retrieves global options
	 */
	class scCartItemOptionsRetriever implements ICartItemOptionsRetriever {
		
		/**
		 * @var scCartItemOptionsRetrieverInfo
		 */
		protected $_info;
		
		protected $_options = array();
		
		public function __construct(scCartItemOptionsRetrieverInfo $info)
		{
			$this->_info = $info;
			
			$selectedOptions = $info->getSelectedOptions();
			if (!is_array($selectedOptions)) {
				// Make sure the correct data is passed.
				throw new miException(_i('Not an array of selected options.'));
			}
			if (isset($selectedOptions['ProductOptions'])) {
				// The retriever of global options does not care about LocalOptions
				// so set only the globals here.
				$this->_options = $selectedOptions['ProductOptions'];
			}
		}
		
		/**
		 * @return scProductOption[]
		 */
		public function getCartItemOptions()
		{
			// Image upload and text options are stored in session.
			// Only the IDs of the options remain in the _options array for processing.
			$this->handleImageUploadOptions();
			$this->handleUserTextOptions();
			
			// Validate the product options and remove unknown/duplicate options
			$this->validate();
			
			// No options provided, try to get the default options for this item
			if (count($this->_options) == 0)
				$this->useDefault();
			
			// The item has no options
			if (count($this->_options) == 0)
				return array();
			
			// Get detailed data on the _options (needed for creation of scProductOption objects).
			$optionsInfo = $this->getOptionsInfo();
			
			// Add the product options
			$productOptions = array();
			foreach ($this->_options as $optionId) {
				// Check option id for consistency
				if (!ctype_digit($optionId) && !is_int($optionId))
					throw new miException(_t('Invalid product option.'));
				
				// Get an instance of scProductOption
				$productOptions[] = $this->getOption($optionId, $optionsInfo[$optionId]);
			}
			
			return $productOptions;
		}
		
		protected function getOptionsInfo()
		{
			return scProductOptions::getOptionsInfo($this->_options);
		}
		
		/**
		 * @return scProductOption
		 */
		protected function getOption($optionId, $optionInfo)
		{
			return new scProductOption($optionId, $optionInfo);
		}
		
		/**
		 * Validates the product options and removes unknown/dublicate options
		 */
		protected function validate()
		{
			$productId = (int)$this->_info->getProduct()->getPK();
			
			// if options is set but not an array or no options provided apply default options
			if (!is_array($this->_options) or count($this->_options) == 0) 
				return array();
			
			$optionIds = array();
			foreach ($this->_options as $optionGroupId => $values) {
				if (!is_array($values)) {
					throw new miException(sprintf(_i('Array of selected option values expected. "%s" given.'), gettype($values)));
				}
				foreach ($values as $option) {
					if ($option instanceof scFile) {
						// file uploads cannot be validated
						continue;
					}
					// Check if the $option is actually an integer.
					if ((string)(int)$option != (string)$option) {
						throw new miException(_i('Option ID expected.'));
					}
					$optionIds[] = (int)$option;
				}
			}
			
			$this->_options = $this->getValidProductOptionsIds($optionIds);
		}
		
		protected function getValidProductOptionsIds($optionIds)
		{
			$productId = (int)$this->_info->getProduct()->getPK();
			$sql = 'SELECT Options.OptionID ' .
				'FROM Options LEFT JOIN ProductOptionGroups ON ProductOptionGroups.OptionGroupID = Options.OptionGroupID ' .
				'WHERE ProductOptionGroups.ProductID = ' . $productId . ' AND Options.OptionID IN ("' . implode('","', $optionIds) . '")';
			return miUtil::getDBArray($sql, 'OptionID', 'OptionID');
		}
		
		/**
		 * @return array
		 */
		protected function useDefault()
		{
			$productId = (int)$this->_info->getProduct()->getPK();
			
			$userTypes = scProductOptions::getUserValueGroupTypes();
			
			$recordset = new miSqlRecordset('ProductOptionGroups');
			$recordset->addJoinCondition('INNER', 'OptionGroups', 'ON ProductOptionGroups.OptionGroupID = OptionGroups.OptionGroupID');
			$recordset->addJoinCondition('INNER', 'Options', 'ON ProductOptionGroups.OptionGroupID = Options.OptionGroupID');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->addFilter(new miSqlFilterNotIn('OptionGroups.OptionGroupType', $userTypes));
			$recordset->setGroupBy('ProductOptionGroups.OptionGroupID');
			$recordset->setSelectFields(array(
				'ProductOptionGroups.OptionGroupID',
				'OptionID'
			));
			$defaultOptions = $recordset->getAllRecords();
			
			$this->_options = miUtil::getKeyValueArray($defaultOptions, 'OptionGroupID', 'OptionID');
			
			if (is_null($this->_info->getProductVariant())) {
				return;
			}
			
			$variant = $this->_info->getProductVariant();
			$options = $variant->getSelectedOptionsArray();
			$options = $options[scProductVariantOption::TYPE_GLOBAL];
			$this->_options = $options + $this->_options;
		}
		
		public function handleImageUploadOptions()
		{
			if (empty($_FILES) || empty($this->_options)) {
				// Either no file has been uploaded ot it cannot be attached to an option as such is not passed.
				return;
			}
			
			foreach ($this->_options as $optionGroupId => $optionGroup) {
				foreach ($optionGroup as $optionId => $file) {
					if (!($file instanceof scUploadedFile)) {
						continue;
					}
					
					$this->checkIfOptionRequired($file, $optionGroupId, $optionId);
					
					if ($file->getErrorCode() == UPLOAD_ERR_NO_FILE)
						continue;
					
					if ($file->getErrorCode() != UPLOAD_ERR_OK)
						throw new miException(scFilesUtil::getUploadErrorDescription($file['error']));
					
					if (@getimagesize($file->getTempName()) === false)
						throw new miException(_t('Only image uploads are accepted.'));
					
					// generate some random file name and upload file to temp files location
					$randomFilename = 'order_item_image_' . md5(time() . $file->getTempName());
					$tempFilename = scPaths::getItemFilename($randomFilename, scPaths::RESOURCE_TEMP_IMAGE);
					$tempFilePath = scPaths::getItemPath($randomFilename, scPaths::RESOURCE_TEMP_IMAGE);
					
					$file->move($tempFilePath);
					
					$this->_options[$optionGroupId][] = $optionId;
					
					$this->setSessionTempUserFileName($optionGroupId, $optionId, $tempFilename);
					continue;
				}
			}
		}
		
		protected function checkIfOptionRequired(scUploadedFile $file, $optionGroupId, $optionId)
		{
			try {
				// validate optionid / option group and get required value
				$optionSql = 'SELECT OptionRequired AS Required FROM Options ' .
					'INNER JOIN OptionGroups USING(OptionGroupID) ' .
					'WHERE OptionGroupType = ? AND OptionID = ? AND OptionGroups.OptionGroupID = ?';
				
				$option = scUtil::getDbFirstRow($optionSql, array(scProductOptions::GROUP_TYPE_USER_IMAGE, $optionId, $optionGroupId));
			} catch (miDBException $e) {
				if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw new miException(_t('Invalid product option.'));
				}
				
				throw $e;
			}
			
			if ($option['Required'] && ($file->getErrorCode() == UPLOAD_ERR_NO_FILE))
				throw new miException(_t('Please upload image.'));
		}
		
		protected function setSessionTempUserFileName($optionGroupId, $optionId, $tempFilename)
		{
			// later if the order is finished, we'll use the temp filepath to retrieve the file and move it to a permanent location
			scProductOptions::setSessionTempUserFileName('ProductOptions', $optionGroupId, $optionId, $tempFilename);
		}
		
		protected function handleUserTextOptions()
		{
			foreach ($this->_options as $optionGroupId => $optionTexts) {
				foreach ($optionTexts as $optionId => $optionText) {
					// validate optionid / option group and get required value
					$optionSql = 'SELECT OptionRequired AS Required FROM Options ' .
						'INNER JOIN OptionGroups USING(OptionGroupID) ' .
						'WHERE OptionGroupType = ? AND OptionID = ? AND OptionGroups.OptionGroupID = ?';
					
					$option = miStaticDBUtil::execSelect($optionSql, array(scProductOptions::GROUP_TYPE_USER_TEXT, $optionId, $optionGroupId));
					if (count($option) == 0) {
						continue;
					}
					$option = $option[0];
					
					if ($option['Required'] && empty($optionText))
						throw new miException(_t('Please enter text.'));
					
					$this->_options[$optionGroupId][$optionId] = $optionId;
					
					scProductOptions::setSessionTempUserText('ProductOptions', $optionGroupId, $optionId, $optionText);
				}
			}
		}
	}
	
	/**
	 * Retrieves local options
	 */
	class scCartItemLocalOptionsRetriever extends scCartItemOptionsRetriever {
		
		public function __construct(scCartItemOptionsRetrieverInfo $info)
		{
			$this->_info = $info;
			
			$selectedOptions = $info->getSelectedOptions();
			if (!is_array($selectedOptions)) {
				// Make sure the correct data is passed.
				throw new miException(_i('Not an array of selected local options.'));
			}
			if (isset($selectedOptions['LocalOptions'])) {
				// The retriever of local options does not care about global options
				// so set only the locals here.
				$this->_options = $selectedOptions['LocalOptions'];
			}
		}
		
		protected function getOptionsInfo()
		{
			return scLocalOptions::getOptionsInfo($this->_options);
		}
		
		/**
		 * @return scLocalOption
		 */
		protected function getOption($optionId, $optionInfo)
		{
			return new scLocalOption($optionId, $optionInfo);
		}
		
		protected function getValidProductOptionsIds($optionIds)
		{
			$productId = (int)$this->_info->getProduct()->getPK();
			$sql = 'SELECT LocalOptions.LocalOptionID'.
				' FROM LocalOptions LEFT JOIN LocalOptionGroups ON LocalOptionGroups.LocalOptionGroupID = LocalOptions.LocalOptionGroupID'.
				' WHERE LocalOptionGroups.ProductID = ' . (int)$productId . ' AND LocalOptions.LocalOptionID IN ("' . implode('","', $optionIds) . '")';
			return miUtil::getDBArray($sql, 'LocalOptionID', 'LocalOptionID');
		}
		
		/**
		 * @return array
		 */
		protected function useDefault()
		{
			$productId = (int)$this->_info->getProduct()->getPK();
			
			$userTypes = scProductOptions::getUserValueGroupTypes();
			
			$recordset = new miSqlRecordset('LocalOptionGroups');
			$recordset->addJoinCondition('INNER', 'LocalOptions', 'USING(LocalOptionGroupID)');
			$recordset->addFilter(new miSqlFilterEqual('ProductID', $productId));
			$recordset->addFilter(new miSqlFilterNotIn('LocalOptionGroups.LocalOptionGroupType', $userTypes));
			$recordset->setGroupBy('LocalOptionGroups.LocalOptionGroupID');
			$recordset->setSelectFields(array(
				'LocalOptionGroups.LocalOptionGroupID',
				'LocalOptionID'
			));
			$defaultOptions = $recordset->getAllRecords();
			
			$this->_options = miUtil::getKeyValueArray($defaultOptions, 'LocalOptionGroupID', 'LocalOptionID');
			
			if (is_null($this->_info->getProductVariant())) {
				return;
			}
			
			$variant = $this->_info->getProductVariant();
			$options = $variant->getSelectedOptionsArray();
			$options = $options[scProductVariantOption::TYPE_LOCAL];
			$this->_options = $options + $this->_options;
		}
		
		protected function checkIfOptionRequired(scUploadedFile $file, $optionGroupId, $optionId)
		{
			try {
				// validate optionid / option group and get required value
				$optionSql = 'SELECT LocalOptionRequired AS Required FROM LocalOptions ' .
					'INNER JOIN LocalOptionGroups USING(LocalOptionGroupID) ' .
					'WHERE LocalOptionGroupType = ? AND LocalOptionID = ? AND LocalOptionGroups.LocalOptionGroupID = ?';
				
				$option = scUtil::getDbFirstRow($optionSql, array(scProductOptions::GROUP_TYPE_USER_IMAGE, $optionId, $optionGroupId));
			} catch (miDBException $e) {
				if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw new miException(_t('Invalid product option.'));
				}
				
				throw $e;
			}
			
			if ($option['Required'] && ($file->getErrorCode() == UPLOAD_ERR_NO_FILE))
				throw new miException(_t('Please upload image.'));
		}
		
		protected function setSessionTempUserFileName($optionGroupId, $optionId, $tempFilename)
		{
			// later if the order is finished, we'll use the temp filepath to retrieve the file and move it to a permanent location
			scProductOptions::setSessionTempUserFileName('LocalOptions', $optionGroupId, $optionId, $tempFilename);
		}
		
		protected function handleUserTextOptions()
		{
			foreach ($this->_options as $optionGroupId => $optionTexts) {
				foreach ($optionTexts as $optionId => $optionText) {
					// validate optionid / option group and get required value
					$optionSql = 'SELECT LocalOptionRequired AS Required FROM LocalOptions ' .
						'INNER JOIN LocalOptionGroups USING(LocalOptionGroupID) ' .
						'WHERE LocalOptionGroupType = ? AND LocalOptionID = ? AND LocalOptionGroups.LocalOptionGroupID = ?';
					
					$option = miStaticDBUtil::execSelect($optionSql, array(scProductOptions::GROUP_TYPE_USER_TEXT, $optionId, $optionGroupId));
					if (count($option) == 0) {
						continue;
					}
					$option = $option[0];
					
					if ($option['Required'] && empty($optionText))
						throw new miException(_t('Please enter text.'));
					
					$this->_options[$optionGroupId][$optionId] = $optionId;
					
					scProductOptions::setSessionTempUserText('LocalOptions', $optionGroupId, $optionId, $optionText);
				}
			}
		}
	}
	
	/**
	 * Request data for ICartItemOptionsRetriever.
	 */
	class scCartItemOptionsRetrieverInfo {
		
		/**
		 * @var scProductSL
		 */
		protected $_product;
		
		/**
		 * @var scProductVariant
		 */
		protected $_productVariant = null;
		
		/**
		 * @var array
		 */
		protected $_selectedOptions;
		
		public function setProduct(scProductSL $product)
		{
			$this->_product = $product;
		}
		
		/**
		 * @return scProductSL
		 */
		public function getProduct()
		{
			return $this->_product;
		}
		
		public function setProductVariant(scProductVariant $variant = null)
		{
			$this->_productVariant = $variant;
		}
		
		/**
		 * @return scProductVariant
		 */
		public function getProductVariant()
		{
			return $this->_productVariant;
		}
		
		public function setSelectedOptions($optionsArray)
		{
			// Check for the right data format.
			if (!is_array($optionsArray)) {
				throw new miException(_i('Array of selected options grouped by type expected.'));
			}
			foreach ($optionsArray as $optionType => $optionGroups) {
				if (!in_array($optionType, array('LocalOptions', 'ProductOptions'))) {
					throw new miException(_i('ProductOptions and LocalOptions expected.'));
				}
				if (!is_array($optionGroups)) {
					throw new miException(_i('Array of selected options expected.'));
				}
				foreach ($optionGroups as $optionGroupId => $values) {
					if ((string)(int)$optionGroupId != (string)$optionGroupId) {
						if ((count(explode('-', $optionGroupId)) == 1) || (!$this->_product->get('ProductIsBundle'))) {
							throw new miException(_i('Integer option group ID expected.'));
						}
					}
					if (!is_array($values)) {
						throw new miException(_i('Array of selected option values expected.'));
					}
				}
			}
			
			$this->_selectedOptions = $optionsArray;
		}
		
		/**
		 * @return array
		 */
		public function getSelectedOptions()
		{
			return $this->_selectedOptions;
		}
	}
	
	/**
	 * The result of calling scCartItemProductOptionsRetriever's getCartItemOptions()
	 */
	class scCartItemOptionsInfo {
		
		/**
		 * @var scProductOption[]
		 */
		protected $_options = array();
		
		/**
		 * @var scLocalOption[]
		 */
		protected $_localOptions = array();
		
		/**
		 * @param scProductOption[] $options
		 */
		public function setOptions($options)
		{
			$this->_options = $options;
		}
		
		/**
		 * @return scProductOption[]
		 */
		public function getOptions()
		{
			return $this->_options;
		}
		
		/**
		 * @param scLocalOption[] $options
		 */
		public function setLocalOptions($options)
		{
			$this->_localOptions = $options;
		}
		
		/**
		 * @return scLocalOption[]
		 */
		public function getLocalOptions()
		{
			return $this->_localOptions;
		}
	}
	
	/**
	 * The result of calling scCartItemBundleOptionsRetriever's getCartItemOptions()
	 */
	class scCartItemBundleOptionsInfo extends scCartItemOptionsInfo {
		
		protected $_productOptionsInfo = array();
		
		public function setProductOptionsInfo($productId, scCartItemOptionsInfo $optionsInfo)
		{
			$this->_productOptionsInfo[$productId] = $optionsInfo;
		}
		
		public function getProductOptionsInfo($productId)
		{
			return $this->_productOptionsInfo[$productId];
		}
	}
?>