<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCartItemService {
		
		/**
		 * @var scCartItemServiceFactory
		 */
		protected $_factory = null;
		
		/**
		 * @return scCartItemService
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scCartItemService();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		public function setFactory(scCartItemServiceFactory $factory)
		{
			$this->_factory = $factory;
		}
		
		/**
		 * @return scCartItemServiceFactory
		 */
		public function getFactory()
		{
			if (is_null($this->_factory)) {
				$this->_factory = scCartItemServiceFactory::singleton();
			}
			return $this->_factory;
		}
		
		/**
		 * Depending on the cart item info (request) return an different implementation of cart item.
		 * @param scCartItemInfo $cartItemInfo
		 * @return scCartItemProduct
		 */
		public function getCartItem(scCartItemInfo $cartItemInfo)
		{
			if ($cartItemInfo->getProductVariant() instanceof scProductVariant) {
				return new scCartItemProductVariant($cartItemInfo);
			}
			
			if ($cartItemInfo->getProduct()->get('ProductIsBundle')) {
				// Note: A bundle cart item contains cart items in itself.
				// They are also created through this or a sub of this method.
				return new scCartItemProductBundle($cartItemInfo);
			}
			
			// The simpliest of cart items
			return new scCartItemProduct($cartItemInfo);
		}
		
		/**
		 * Creates scCartItemProduct object
		 * The parameters are sanitized first
		 * 
		 * @param string $productId the product id
		 * @param string $quantity the quantity
		 * @param array $options
		 * @return scCartItemProduct
		 */
		public function createCartItemProduct($productId, $quantity, $options = array(), $productVariantId = 0)
		{
			// Make sure quantity is a positive number
			if ((!is_int($quantity) and !ctype_digit($quantity)) or ($quantity <= 0))
				$quantity = 1;
			
			// Make sure a product has been selected
			if (!is_int($productId) and !ctype_digit($productId)) {
				throw new miException(_t('Invalid product.'));
			}
			
			// Get the product
			$product = $this->getProduct($productId);
			if (!$product->get('ProductIsActive')) {
				throw new miException(sprintf(_t('"%s" is currently not available.'), $product->getName()));
			}
			$productVariant = $this->getProductVariant($productVariantId); // May be null if $productVariantId = 0
			
			// Process selected options and get them in the format the cart item needs them.
			$cartItemOptionsInfo = new scCartItemOptionsRetrieverInfo();
			$cartItemOptionsInfo->setProduct($product);
			$cartItemOptionsInfo->setProductVariant($productVariant);
			$cartItemOptionsInfo->setSelectedOptions($options);
			$optionsInfo = $this->getCartItemOptions($cartItemOptionsInfo);
			
			if (is_null($productVariant)) {
				// Get a product variant
				$productVariant = $this->getProductVariantByOptions($productId, $optionsInfo, $productVariantId);
			}
			
			// Prepare cart item info
			$cartItemInfo = $this->createCartItemInfo();
			$cartItemInfo->setQuantity($quantity);
			$cartItemInfo->setProduct($product);
			$cartItemInfo->setOptionsInfo($optionsInfo);
			if (!is_null($productVariant)) {
				$cartItemInfo->setProductVariant($productVariant);
			}
			
			$cartItem = $this->getCartItem($cartItemInfo);
			
			return $cartItem;
		}
		
		/**
		 * Depending on the selected options returns a product variant if such is available.
		 * If the $productVariantId is specfied the $optionsInfo is ignored and the specified by
		 * $productVariantId variant is returned.
		 * @return scProductVariant
		 */
		public function getProductVariantByOptions($productId, scCartItemOptionsInfo $optionsInfo, $productVariantId = 0)
		{
			if ($productVariantId > 0) {
				$productVariant = $this->createProductVariant();
				$productVariant->read($productVariantId);
				return $productVariant;
			}
			
			$variantOptions = scCartItemOptions::getVariantOptions($optionsInfo);
			if (empty($variantOptions)) {
				return null;
			}
			
			$productVariant = $this->createProductVariant();
			$productVariant->set('ProductID', $productId);
			$productVariant->setOptions($variantOptions);
			$productVariantId = $productVariant->readByOptions();
			if (is_null($productVariantId)) {
				return null;
			}
			
			if ($productVariant->getProductId() != $productId) {
				// The passed variant id is the id of a variant that does not
				// belong to the product identified by $productId.
				throw new miException(_t('Invalid product variant.'));
			}
			
			return $productVariant;
		}
		
		/**
		 * @param string $param
		 * @return mixed
		 */
		public function getOptionsParam($param)
		{
			$options = miGetParamDefault($param, array());
			if (!is_array($options)) {
				$options = array();
			}
			$this->prepareOptionsArray($options);
			
			$files = $this->getFilesArray($param);
			$this->prepareOptionsArray($files);
			
			$result['LocalOptions'] = $options['LocalOptions'] + $files['LocalOptions'];
			$result['ProductOptions'] = $options['ProductOptions'] + $files['ProductOptions'];
			
			return $result;
		}
		
		protected function prepareOptionsArray(&$optionsArray)
		{
			if (!isset($optionsArray['ProductOptions'])) {
				$optionsArray['ProductOptions'] = array();
			}
			if (!isset($optionsArray['LocalOptions'])) {
				$optionsArray['LocalOptions'] = array();
			}
		}
		
		/**
		 * @param string $inputName
		 * @return array
		 */
		protected function getFilesArray($inputName)
		{
			$result = array();
			
			if (empty($_FILES[$inputName]) || !is_array($_FILES[$inputName])) {
				return array();
			}
			
			foreach ($_FILES[$inputName] as $name => $values) {
				foreach ($values as $optionType => $options) {
					foreach ($options as $optionGroupId => $fileData) {
						foreach ($fileData as $optionId => $fileValue) {
							$uniqueKey = $optionType . '|' . $optionGroupId . '|' . $optionId;
							$result[$uniqueKey]['OptionType'] = $optionType;
							$result[$uniqueKey]['OptionGroupID'] = $optionGroupId;
							$result[$uniqueKey]['OptionID'] = $optionId;
							$result[$uniqueKey][$name] = $fileValue;
						}
					}
				}
			}
			
			if (empty($result)) {
				return array();
			}
			
			$return = array();
			foreach ($result as $optionFileData) {
				$file = new scUploadedFile();
				$file->setFileData($optionFileData);
				$return[$optionFileData['OptionType']][$optionFileData['OptionGroupID']][$optionFileData['OptionID']] = $file;
			}
			return $return;
		}
		
		/**
		 * Returns an scCartItemOptionsInfo object.
		 * Uses different options retrievers for bundles and regular products.
		 * 
		 * @param scCartItemOptionsRetrieverInfo $optionsRetrieverInfo
		 * @return scCartItemOptionsInfo
		 */
		public function getCartItemOptions(scCartItemOptionsRetrieverInfo $cartItemOptionsInfo)
		{
			if ($cartItemOptionsInfo->getProduct()->get('ProductIsBundle')) {
				$retriever = new scCartItemBundleOptionsRetriever($cartItemOptionsInfo);
			} else {
				$retriever = new scCartItemProductOptionsRetriever($cartItemOptionsInfo);
			}
			
			// Calling $retriever's getCartItemOptions() returns a scCartItemOptionsInfo.
			return $retriever->getCartItemOptions();
		}
		
		/**
		 * @return scProductSL
		 */
		protected function getProduct($productId)
		{
			try {
				$product = new scProductSL();
				$product->read($productId);
				return $product;
			} catch(miDBException $exception) {
				if ($exception->getDbCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw new miException(_t('Invalid product.'));
				}
				
				throw $exception; // Rethrow
			}
		}
		
		/**
		 * @return scProductVariant
		 */
		protected function getProductVariant($productVariantId)
		{
			if ($productVariantId == 0) {
				return null;
			}
			
			try {
				$productVariant = $this->createProductVariant();
				$productVariant->read($productVariantId);
				return $productVariant;
			} catch(miDBException $exception) {
				if ($exception->getDbCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw new miException(_t('Invalid product variant.'));
				}
				
				throw $exception; // Rethrow
			}
		}
		
		/**
		 * @return scCartItemInfo
		 */
		protected function createCartItemInfo()
		{
			return new scCartItemInfo();
		}
		
		/**
		 * @return scProductVariant
		 */
		protected function createProductVariant()
		{
			if (is_null($this->getFactory())) {
				throw new miException(_i('scCartItemServiceFactory needed.'));
			}
			
			return $this->_factory->createProductVariant();
		}
	}
	
	class scCartItemServiceFactory {
		
		/**
		 * @return scCartItemServiceFactory
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scCartItemServiceFactory();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		/**
		 * @return scProductVariant
		 */
		public function createProductVariant()
		{
			return new scProductVariant();
		}
	}
	
	class scBundlesCartItemServiceFactory extends scCartItemServiceFactory {
		
		/**
		 * @return scBundlesCartItemServiceFactory
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scBundlesCartItemServiceFactory();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		/**
		 * @return scProductVariant
		 */
		public function createProductVariant()
		{
			return new scBundledProductVariant();
		}
	}
?>