<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderTotal {
		
		/**
		 * @var scCart
		 */
		protected $_cart;
		
		/**
		 * @var string
		 */
		protected $_cartHash;
		
		/**
		 * @var array
		 */
		protected $_orderTotalLines;
		
		public function __construct(scCart $cart)
		{
			$this->_cart = $cart;
		}
		
		/**
		 * Computes the order total lines
		 * 
		 * @param int $type (optional) the type of the order total module
		 */
		protected function computeOrderTotalLines($type = false)
		{
			$orderTotalModules = $this->getActiveOrderTotalModules($type);
			
			// Remove old order total lines
			if ($type === false) {
				$this->_orderTotalLines = array();
			} else {
				// Remove the order total lines from the specified type
				foreach ($this->_orderTotalLines as $key => $orderTotalLine) {
					if ($orderTotalLine->getType() == $type) {
						unset($this->_orderTotalLines[$key]);
					}
				}
			}
			
			$this->processOrderTotalModules($orderTotalModules);
		}
		
		protected function processOrderTotalModules($orderTotalModules)
		{
			foreach ($orderTotalModules as $orderTotalModule) {
				try {
					$result = $orderTotalModule->getOrderTotalLines($this->_cart);
					$this->_orderTotalLines = array_merge($this->_orderTotalLines, $result);
				} catch (Exception $exception) {
					scLog::singleton()->warning(sprintf(_t('Order total module exception: "%s".'), $exception->getMessage()));
					
					// Clear the cart
//					scCart::getCart()->initCustomerInfo();
//					scCart::getCart()->removeAllItems();
					
					// Rethrow to let the customer know what is going on
					throw $exception;
				}
			}
		}
		
		/**
		 * Returns an array with the active order total modules
		 * @see scModulesFactory::getActiveOrderTotalModules()
		 */
		protected function getActiveOrderTotalModules($type = false)
		{
			return scModulesFactory::getActiveOrderTotalModules($type);
		}
		
		/**
		 * Return the sum of the order total lines
		 * Order total lines if type ORDER_TOTAL_TYPE_SUBTOTAL are ignored
		 * 
		 * @return scMultiCurrencyAmount the sum
		 * @throws Exception an exception from order total module
		 */
		protected function getTotalLinesSum()
		{
			$sum = scMultiCurrencyAmount::getZero();
			$orderTotalLines = $this->getOrderTotalLines();
			foreach ($orderTotalLines as $orderTotalLine) {
				if ($orderTotalLine->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL)
					continue;
				
				$sum = scMultiCurrencyAmountMath::add($sum, $orderTotalLine->getAmount());
			}
			return $sum;
		}
		
		/**
		 * Returns the order total lines
		 * 
		 * @access public
		 * @return array the order total lines
		 * @throws Exception an exception from order total module
		 */
		public function getOrderTotalLines()
		{
			if ($this->_cart->getCartHash() != $this->_cartHash) {
				$this->_cartHash = $this->_cart->getCartHash();
				$this->computeOrderTotalLines();
			}
			return $this->_orderTotalLines;
		}
		
		/**
		 * Returns the currently available order total lines.
		 * No computing is done when calling this method.
		 * NOTE: Those lines may be obsolete or incomplete!
		 * @return array
		 */
		public function getCurrentOrderTotalLines()
		{
			if (!empty($this->_orderTotalLines)) {
				return $this->_orderTotalLines;
			}
			return array();
		}
		
		/**
		 * Deletes order total lines of $type
		 * 
		 * @param int $type
		 */
		public function deleteOrderTotalLinesByType($type)
		{
			$lines = $this->getOrderTotalLines();
			
			$newLines = array();
			foreach ($lines as $line)
				if ($line->getType() != $type)
					$newLines[] = $line;
			$this->_orderTotalLines = $newLines;
		}
		
		/**
		 * Returns the value of the items for shipping purposes
		 * The value that we calculate here is the sum of all order total modules before the shipping order total module (excluding points)
		 * 
		 * @param scCurrency $currency
		 * @return float
		 */
		public function getDeclaredValueForShipping(scCurrency $currency)
		{
			$totalLines = $this->getCurrentOrderTotalLines();
			
			$total = scMultiCurrencyAmount::getZero();
			foreach ($totalLines as $line) {
				// get all total lines before shipping
				if ($line->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) {
					break;
				}
				// ignore the bonus points total line
				if ($line->getType() != IOrderTotalModule::ORDER_TOTAL_TYPE_POINTS) {
					$total = scMultiCurrencyAmountMath::add($total, $line->getAmount());
				}
			}
			
			$price = scPrice::factory($total);
			return $price->getAmountInCurrency($currency);
		}
		
		/**
		 * @return scMultiCurrencyAmount
		 */
		public function getGrandTotalAmount()
		{
			$cartItems = $this->_cart->getItems();
			
			$subtotal = scMultiCurrencyAmount::getZero();
			foreach ($cartItems as $cartItem) {
				$subtotal = scMultiCurrencyAmountMath::add($subtotal, $cartItem->getTotal());
			}
			$totalLinesSum = $this->getTotalLinesSum();
			
			return scMultiCurrencyAmountMath::add($subtotal, $totalLinesSum);
		}
		
		/**
		 * Return the grand total in the current currency
		 * 
		 * @return float the grand total
		 * @throws Exception an exception from order total module
		 */
		public function getGrandTotalInCurrency($currency)
		{
			$price = scPrice::factory($this->getGrandTotalAmount());
			return $price->getAmountInCurrency($currency);
		}
		
		/**
		 * Return the grand total in the current currency
		 * 
		 * @return float the grand total
		 * @throws Exception an exception from order total module
		 */
		public function getCurrentCurencyGrandTotal()
		{
			return $this->getGrandTotalInCurrency(scCurrencyUtil::getCurrentCurrency());
		}
		
		/**
		 * Returns the cart grand total in base currency
		 * 
		 * @return float the grand total
		 * @throws Exception an exception from order total module
		 */
		public function getBaseCurrencyGrandTotal()
		{
			return $this->getGrandTotalInCurrency(scCurrencyUtil::getBaseCurrency());
		}
		
		/**
		 * Calculate the sum of order total lines, eg Tax, Shipping, Discounts, etc
		 * 
		 * @param int $orderTotalModuleType the type of the order total lines
		 * @param boolean $recalculate if the tax should be recalculated, default false (optiona)
		 * @return scMultiCurrencyAmount the tax
		 */
		public function getSummedOrderTotalLinesAmount($orderTotalModuleType, $recalculate = false)
		{
			if ($recalculate)
				$this->computeOrderTotalLines($orderTotalModuleType);
			
			$amount = scMultiCurrencyAmount::getZero();
			foreach ($this->_orderTotalLines as $orderTotalLine) {
				if ($orderTotalLine->getType() == $orderTotalModuleType) {
					$amount = scMultiCurrencyAmountMath::add($amount, $orderTotalLine->getAmount());
				}
			}
			return $amount;
		}
		
		/**
		 * Set the shipping amount, overriding the shipping module calculations
		 */
		public function setShippingAmount($shippingAmount)
		{
			foreach ($this->_orderTotalLines as $orderTotalLine) {
				if ($orderTotalLine->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) {
					$orderTotalLine->setAmount($shippingAmount);
					return;
				}
			}
			
			// No shipping line has been found - add one
			$this->_orderTotalLines[] = new scOrderTotalLine(_t('Shipping'), $shippingAmount, IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING);
		}
	}
?>