<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSkinSettingsUtil {
		
		const SETTING_TYPE_TEXT = 1;
		const SETTING_TYPE_CHECKBOX = 2;
		const SETTING_TYPE_IMAGE = 3;
		
		const IMAGE_DEFAULT = 1;
		const IMAGE_USERFILE = 2;
		
		protected static $_settingTypeValues = array(
			self::SETTING_TYPE_TEXT => 'text',
			self::SETTING_TYPE_CHECKBOX => 'checkbox',
			self::SETTING_TYPE_IMAGE => 'image',
		);
		
		protected static $_settingTypeWidgets = array(
			self::SETTING_TYPE_TEXT => 'miTextWidget',
			self::SETTING_TYPE_CHECKBOX => 'miCheckboxWidget',
			self::SETTING_TYPE_IMAGE => 'scImageUploadWidget',
		);
		
		protected static $_settintTypeViewMappers = array(
			self::SETTING_TYPE_IMAGE => 'scViewMapperUploadImage',
		);
		
		protected static $_translations = array();
		
		/**
		 * @param string $value
		 * @return int
		 */
		public static function getTypeIdByValue($value)
		{
			$typeId = array_search($value, self::$_settingTypeValues);
			if ($typeId === false) {
				$msg = sprintf(_i('Unknown setting type "%s".'), $value);
				throw new scSkinConfigurationException($msg, scSkinConfigurationException::UNKNOWN_SETTING_TYPE);
			}
			return $typeId;
		}
		
		/**
		 * @param int $typeId
		 * @return string
		 */
		public static function getWidgetClassForType($typeId)
		{
			if (!isset(self::$_settingTypeWidgets[$typeId])) {
				$msg = sprintf(_i('Unknown setting type "%s".'), $typeId);
				throw new scSkinConfigurationException($msg, scSkinConfigurationException::UNKNOWN_SETTING_TYPE);
			}
			return self::$_settingTypeWidgets[$typeId];
		}
		
		public static function getSkinSettingDefaultValue(scSkinDomainObject $skin, $settingName)
		{
			$xml = self::getSkinXml($skin);
			$settings = self::getSkinSettingsFromXml($xml);
			if (!isset($settings[$settingName])) {
				$msg = sprintf(_i('Unknown skin setting "%s".'), $settingName);
				throw new scSkinConfigurationException($msg, scSkinConfigurationException::UNKNOWN_SETTING);
			}
			
			return $settings[$settingName]['SkinSettingValue'];
		}
		
		public static function getSkinSettingAttribute($skin, $settingName, $attributeName)
		{
			$xml = self::getSkinXml($skin);
			foreach ($xml->Settings->Setting as $setting) {
				if ($setting['name'] == $settingName) {
					if (!isset($setting[$attributeName])) {
						$msg = sprintf(_i('Unknown skin setting attribute "%s" for setting "%s".'), $attributeName, $settingName);
						throw new scSkinConfigurationException($msg, scSkinConfigurationException::UNKNOWN_SETTING_ATTRIBUTE);
					}
					
					return $setting[$attributeName];
				}
			}
			
			// Since a the code has reached that point, the setting could not be found.
			$msg = sprintf(_i('Unknown skin setting "%s".'), $settingName);
			throw new scSkinConfigurationException($msg, scSkinConfigurationException::UNKNOWN_SETTING);
		}
		
		public static function getXmlSettingValue(SimpleXMLElement $setting)
		{
			$settingType = (string)$setting['type'];
			$settingType = self::getTypeIdByValue($settingType);
			
			switch ($settingType) {
				case self::SETTING_TYPE_IMAGE:
					return scPaths::RESOURCE_SKIN_DEFAULT_IMAGE . ':' . (string)$setting;
				case self::SETTING_TYPE_CHECKBOX:
					return ((string)$setting == 'true' ? 1 : 0);
				default:
					return (string)$setting;
			}
		}
		
		public static function getSkinSettingValueForType($skinLocation, $settingValue, $settingType)
		{
			switch ($settingType) {
				case self::SETTING_TYPE_CHECKBOX:
					return (int)(bool)$settingValue;
					
				case self::SETTING_TYPE_IMAGE:
					$value = explode(':', $settingValue);
					$resourceType = array_shift($value);
					$value = implode(':', $value);
					if ($resourceType == scPaths::RESOURCE_SKIN_DEFAULT_IMAGE) {
						$value = 'skins/' . $skinLocation . '/' . $value;
					}
					return scPaths::getAbsoluteUrl($value);
					
				default:
					return $settingValue;
			}
		}
		
		/**
		 * @param scSkinDomainObject $skin
		 * @return SimpleXMLElement
		 */
		public static function getSkinXml(scSkinDomainObject $skin)
		{
			$filePath = scSkins::getSkinsRoot() . $skin->get('SkinLocation') . '/skin.xml';
			
			if (file_exists($filePath) == false) {
				return false;
			}
			
			$skinData = file_get_contents($filePath);
			if ($skinData == false) {
				return false;
			}
			
			return new SimpleXMLElement($skinData);
		}
		
		public static function getSkinSettingDisplayName(scSkinDomainObject $skin, $settingName)
		{
			$translations = self::getDisplayNameTranslations($skin);
			if (isset($translations[$settingName])) {
				return $translations[$settingName];
			}
			
			return self::getSkinSettingAttribute($skin, $settingName, 'displayName');
		}
		
		public static function getSkinSettingsFromXml(SimpleXMLElement $xml)
		{
			$settingsArray = array();
			foreach ($xml->Settings->Setting as $setting) {
				$data = array();
				$data['SkinSettingType'] = scSkinSettingsUtil::getTypeIdByValue($setting['type']);
				$data['SkinSettingName'] = (string)$setting['name'];
				$data['SkinSettingValue'] = scSkinSettingsUtil::getXmlSettingValue($setting);
				
				$settingsArray[] = $data;
			}
			$settingsArray = miUtil::getArrayWithKeys($settingsArray, 'SkinSettingName');
			
			return $settingsArray;
		}
		
		public static function upgradeSkinSettings($skinId, $currentSettings, $upgradeSettings)
		{
			try {
				miStaticDBUtil::startTransaction();
				// Get setting names
				$currentNames = array_keys($currentSettings);
				$upgradeNames = array_keys($upgradeSettings);
				
				// Settings that are no longer supported by the skin
				$toDelete = array_diff($currentNames, $upgradeNames);
				// Newly added settings (exist in xml but don't in db)
				$toInsert = array_diff($upgradeNames, $currentNames);
				
				// To check for differences in types
				$toCompare = array_intersect($currentNames, $upgradeNames);
				
				foreach ($toCompare as $settingName) {
					if ($currentSettings[$settingName]['SkinSettingType'] != $upgradeSettings[$settingName]['SkinSettingType']) {
						// Setting type in the xml differs from the one in the db.
						// That means that the value in the db may be incompatible with the new type.
						
						$toDelete[] = $settingName; // Delete the old setting
						$toInsert[] = $settingName; // Insert the new one
					}
				}
				
				self::deleteSkinSettings($skinId, $toDelete);
				
				$createSettings = array();
				foreach ($toInsert as $settingName) {
					$createSettings[] = $upgradeSettings[$settingName];
				}
				self::createSkinSettings($skinId, $createSettings);
				
			} catch (miException $e) {
				miStaticDBUtil::rollback();
				throw $e;
			}
			miStaticDBUtil::commit();
		}
		
		public static function getWidgets(scSkinDomainObject $skin, miWebForm $webForm)
		{
			$widgets = array();
			
			$settings = $skin->getSettings();
			foreach ($settings as $setting) {
				$widgetClass = self::getWidgetClassForType($setting['SkinSettingType']);
				$widget = new $widgetClass($webForm);
				$widget->setFieldName($setting['SkinSettingName']);
				$widget->setProperty('displayName', self::getSkinSettingDisplayName($skin, $setting['SkinSettingName']));
				switch ($setting['SkinSettingType']) {
					case scSkinSettingsUtil::SETTING_TYPE_IMAGE:
						$widget->setProperty('resource', scPaths::RESOURCE_SKIN_UPLOADED_IMAGE);
						$widget->setProperty('noThumb', true);
						break;
				}
				
				$widgets[] = $widget;
			}
			
			return $widgets;
		}
		
		public static function deleteSkinSettings($skinId, array $settingNames)
		{
			if (empty($settingNames))
				return;
			foreach ($settingNames as $settingName) {
				miStaticDBUtil::execDelete('SkinSettings', array('SkinID', 'SkinSettingName'), array($skinId, $settingName));
			}
		}
		
		public static function createSkinSettings($skinId, array $settings)
		{
			if (empty($settings))
				return;
			foreach ($settings as $settingData) {
				$setting = new scSkinSettingDomainObject();
				$setting->setRow($settingData);
				$setting->set('SkinID', $skinId);
				$setting->insert();
			}
		}
		
		public static function getSkinSettings($skinId)
		{
			static $skinSettings = null;
			
			if ($skinSettings == null) {
				$recordset = new miSqlRecordset('SkinSettings');
				$recordset->addFilter(new miSqlFilterEqual('SkinID', (int)$skinId));
				$settings = $recordset->getAllRecords();
				$skinSettings = miUtil::getArrayWithKeys($settings, 'SkinSettingName');
			}
			
			return $skinSettings;
		}
		
		/**
		 * @param scSkinDomainObject $skin
		 * @return array
		 */
		protected static function getDisplayNameTranslations(scSkinDomainObject $skin)
		{
			if (isset(self::$_translations[$skin->get('SkinID')])) {
				return self::$_translations[$skin->get('SkinID')];
			}
			
			self::$_translations[$skin->get('SkinID')] = array();
			
			$langLocation = scSkins::getSkinsRoot() . $skin->get('SkinLocation') . '/lang';
			if (!is_dir($langLocation)) {
				return array();
			}
			
			$langFile = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$langFile = $langLocation . '/' . $langFile . '.xml';
			
			if (!file_exists($langFile)) {
				return array();
			}
			
			$data = file_get_contents($langFile);
			if ($data == false) {
				return array();
			}
			
			$xml = new SimpleXMLElement($data);
			
			if (!isset($xml->SettingDisplayName)) {
				return array();
			}
			
			foreach ($xml->SettingDisplayName as $displayName) {
				$setting = (string)$displayName['name'];
				self::$_translations[$skin->get('SkinID')][$setting] = (string)$displayName;
			}
			return self::$_translations[$skin->get('SkinID')];
		}
	}
	
	class scSkinSettingDomainObject extends miDefaultDomainObject {
		
		public function __construct()
		{
			parent::__construct(new miSqlRecord('SkinSettings', 'SkinSettingID'));
		}
	}
	
	class scSkinConfigurationException extends miException {
		
		const UNKNOWN_SETTING = 1;
		const UNKNOWN_SETTING_ATTRIBUTE = 2;
		const UNKNOWN_SETTING_TYPE = 3;
	}
?>