<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCurrencyUtil {
		static protected $currencyCode;
		
		static protected $_activeCurrencies = null;
		static protected $_activeCurrencyCodes = null;
		
		/**
		 * A background function wrapper for the updateBaseRates
		 */
		public static function updateInBackground()
		{
			ignore_user_abort(true);
			scSettings::setSettingValue(scSettings::SETTING_LAST_CURRENCYRATE_UPDATE, time());
			self::updateBaseRates(); 
		}
		
		/**
		 * Updates the rates for currencies against the base currency
		 */
		public static function updateBaseRates()
		{
			$rates = self::getNewQuotes();
			$currencies = self::getCurrencyRows();
			
			$baseCurrencyCode = self::getBaseCurrencyCode();
			foreach ($currencies as $currency) {
				
				$rate = self::getRate($baseCurrencyCode, $currency['CurrencyCode'], $rates);
				if ($rate === false) {
					// Don't log warning if we can't find a currency rate
//					$msg = sprintf(_t('Cannot update rate for %s to %s.'), $baseCurrencyCode, $currency['CurrencyCode']);
//					scLog::singleton()->warning($msg, scLog::CATEGORY_CURRENCY_RATES);
					continue;
				}
				
				miStaticDBUtil::execUpdate('Currencies', array('CurrencyRate' => $rate, 'CurrencyRateLastUpdated' => time()), 'CurrencyID', $currency['CurrencyID']);
				if ($currency['CurrencyAllowFixedPrices']) {
					self::updateProductPrices($currency['CurrencyCode'], $rate);
				}
			}
		}
		
		public static function validateCurrencyCode($currencyCode)
		{
			if ((strlen($currencyCode) == 3) && (ctype_alnum($currencyCode)))
				return true;
			
			throw new miException(sprintf(_t('Currency code "%s" is invalid.'), $currencyCode));
		}
		
		public static function getCurrenciesWithFixedPrices()
		{
			static $currenciesWithFixedPrices;
			
			if (is_null($currenciesWithFixedPrices)) {
				$currenciesWithFixedPrices = miStaticDBUtil::execSelect(
					'SELECT * FROM Currencies WHERE CurrencyAllowFixedPrices AND CurrencyID != ?',
					array(scCurrencyUtil::getBaseCurrency()->getCurrencyId())
				);
				foreach ($currenciesWithFixedPrices as $currency) {
					self::validateCurrencyCode($currency['CurrencyCode']);				
				}
			}
			
			return $currenciesWithFixedPrices;
		}
		
		public static function updateProductPrices($currencyCode, $currencyRate, $productId = null)
		{
			self::validateCurrencyCode($currencyCode);
			
			$productIdFilter = '';
			if (!empty($productId)) {
				$productIdFilter = ' AND ProductID = ' . intval($productId);
			}
			
			miStaticDBUtil::execSQL(sprintf(
				'UPDATE Products SET ProductPrice%s = (ProductPrice * %f) WHERE ProductPrice%sAuto' . $productIdFilter,
				$currencyCode, $currencyRate, $currencyCode
			));
		}
		
		public static function getProductPriceColumnName()
		{
			$currency = self::getCurrentCurrency()->getCurrencyRow();
			try {
				self::validateCurrencyCode($currency['CurrencyCode']);
				
				if ($currency['CurrencyAllowFixedPrices']) {
					return 'ProductPrice' . $currency['CurrencyCode'];
				}
			} catch (miException $exception) {
				scLog::singleton()->error($exception->getMessage(), scLog::CATEGORY_NONE);
			}
			return 'ProductPrice';
		}
		
		/**
		 * Returns the exchange rate between two currencies
		 * 
		 * @param string $baseCurrencyCode
		 * @param string $quoteCurrencyCode
		 * @param array array of scCurrencyRate objects
		 * @return float|false the rate
		 */
		protected static function getRate($baseCurrencyCode, $quoteCurrencyCode, $quotes)
		{
			if ($baseCurrencyCode == $quoteCurrencyCode)
				return 1;
			
			// First check if we have the pair among the quotes
			$rate = self::getDirectRate($baseCurrencyCode, $quoteCurrencyCode, $quotes);
			if ($rate !== false)
				return $rate;
			
			// Compute cross rate
			// Attempt to find suitable currency to be used for the cross rate
			foreach ($quotes as $quote) {
				if ($quote->getBaseCurrency() == $baseCurrencyCode) {
					$crossrate = self::getDirectRate($quote->getQuoteCurrency(), $quoteCurrencyCode, $quotes);
					if ($crossrate !== false) {
						return $crossrate * $quote->getRate();
					}
				}
				if ($quote->getQuoteCurrency() == $baseCurrencyCode) {
					$crossrate = self::getDirectRate($quote->getBaseCurrency(), $quoteCurrencyCode, $quotes);
					if ($crossrate !== false) {
						return $crossrate / $quote->getRate();
					}
				}
			}
			
			return false;
		}
		
		protected static function getDirectRate($baseCurrencyCode, $quoteCurrencyCode, $quotes)
		{
			foreach ($quotes as $quote) {
				if (($quote->getBaseCurrency() == $baseCurrencyCode) and ($quote->getQuoteCurrency() == $quoteCurrencyCode))
					return $quote->getRate();
				
				// Try the pair with the currencies reversed
				if (($quote->getBaseCurrency() == $quoteCurrencyCode) and ($quote->getQuoteCurrency() == $baseCurrencyCode))
					return 1 / $quote->getRate();
			}
			return false;
		}
		
		/**
		 * Returns the new rates from all currency rate modules
		 * The rates are ordered module by module
		 * 
		 * @return array array of scCurrencyRate objects
		 */
		protected static function getNewQuotes()
		{
			$sql = 'SELECT ModuleID FROM CurrencyRateModules WHERE CUrrencyRateModuleIsActive != 0 ORDER BY CurrencyRateModuleSort';
			$currencyRateModuleIds = miUtil::getDBArray($sql, 'ModuleID', 'ModuleID');
			
			$rates = array();
			foreach ($currencyRateModuleIds as $moduleId) {
				try {
					$module = scModulesFactory::getImplementation($moduleId, 'ICurrencyRates');
					$rates = array_merge($rates, $module->getCurrencyRates());
				} catch (Exception $exception) {
					scLog::singleton()->error(sprintf(_t('Error from module "%s": %s.'), get_class($module),$exception->getMessage()), scLog::CATEGORY_CURRENCY_RATES);
				}
			}
			return $rates;
		}
		
		/**
		 * Returns if the currency code exists
		 */
		public static function isCurrencyCodeExists($currencyCode)
		{
			$sql = 'SELECT COUNT(*) AS N FROM Currencies WHERE CurrencyCode=?';
			$rows = miStaticDBUtil::execSelect($sql, array($currencyCode));
			return $rows[0]['N'] > 0;
		}
		
		/**
		 * @return scCurrency
		 */
		public static function getBaseCurrency()
		{
			$defaultCurrencyId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY);
			return new scCurrency($defaultCurrencyId);
		}
		
		public static function setBaseCurrency($currencyId)
		{
			scSettings::setSettingValue(scSettings::SETTING_DEFAULT_CURRENCY, $currencyId);
			self::$currencyCode = null;
		}
		
		public static function getBaseCurrencyCode()
		{
			if (self::$currencyCode === null) {
				$sql = 'SELECT CurrencyCode FROM Currencies WHERE CurrencyID=?';
				$rows = miStaticDBUtil::execSelect($sql, array(scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY)));
				if (count($rows) == 0)
					throw new miException(_t('Cannot find default currency.'));
				self::$currencyCode = $rows[0]['CurrencyCode'];
			}
			return self::$currencyCode;
		}
		
		public static function getBaseCurrencySymbols()
		{
			$defaultCurrencyId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY);
			return self::getCurrencySymbols($defaultCurrencyId);
		}
		
		public static function getCurrencySymbols($currencyId)
		{
			$sql = 'SELECT CurrencySymbolLeft, CurrencySymbolRight FROM CurrenciesML WHERE CurrencyID=? AND LanguageID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($currencyId, scMultiLanguage::getCurrentLanguageId()));
			if (count($rows) == 0) {
				$rows = miStaticDBUtil::execSelect($sql, array($currencyId, scMultiLanguage::getDefaultLanguageId()));
				if (count($rows) == 0) {
					throw new miException(_t('Cannot find default currency.'));
				}
			}
			return array($rows[0]['CurrencySymbolLeft'], $rows[0]['CurrencySymbolRight']);
		}
		
		/**
		 * Return the currencies
		 * 
		 * @return array the currencies
		 */
		protected static function getCurrencyRows()
		{
			$sql = 'SELECT * FROM Currencies';
			return miStaticDBUtil::execSelect($sql);
		}
		
		protected static function loadCurrencies()
		{
			if (!is_null(self::$_activeCurrencies)) {
				return;
			}
			
			$languageId = scMultiLanguage::getCurrentLanguageId();
			$sql = 'SELECT Currencies.CurrencyID, CurrencyCode, CurrencyAllowFixedPrices, COALESCE(CurrencyName, CurrencyCode) AS CurrencyName' .
				' FROM Currencies' .
				' LEFT JOIN CurrenciesML ON Currencies.CurrencyID=CurrenciesML.CurrencyID AND LanguageID=' . $languageId .
				' WHERE CurrencyIsActive' .
				' AND (CurrencyRateLastUpdated > ' . (time() - (scSettings::getSettingValue(scSettings::SETTING_CURRENCY_RATE_EXPIRE_DAYS) * 24*60*60)) . ')' .
				' ORDER BY CurrencySort';
			$currencies = miStaticDBUtil::execSelect($sql);
			
			self::$_activeCurrencies = array();
			self::$_activeCurrencyCodes = array();
			foreach ($currencies as $row) {
				self::$_activeCurrencies[$row['CurrencyID']] = $row['CurrencyName'];
				self::$_activeCurrencyCodes[$row['CurrencyID']] = $row['CurrencyCode'];
			}
		}
		
		/**
		 * Return active currencies
		 * @return array array(CurrencyId => CurrencyName)
		 */
		public static function getActiveCurrencies()
		{
			self::loadCurrencies();
			return self::$_activeCurrencies;
		}
		
		/**
		 * Return the active currency codes
		 * 
		 * @return array array(CurrencyId => CurrencyCode)
		 */
		public static function getActiveCurrencyCodes()
		{
			self::loadCurrencies();
			return self::$_activeCurrencyCodes;
		}
		
		/**
		 * @return scCurrency
		 */
		public static function getCurrentCurrency()
		{
			static $currency = null;
			
			if ($currency == null) {
				try {
					$currency = new scCurrency(self::getCurrentCurrencyId());
				} catch (miDBException $exception) {
					if ($exception->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND)
						throw $exception;
					
					$_SESSION['CurrencyID'] = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY);
					$currency = new scCurrency($_SESSION['CurrencyID']);
				}
			}
			
			return $currency;
		}
		
		public static function getCurrentCurrencyId()
		{
			if (miGetParamDefault('CurrencyID', '') != '')
				$_SESSION['CurrencyID'] = miGetParamDefault('CurrencyID', '');
			
			if (!isset($_SESSION['CurrencyID']))
				$_SESSION['CurrencyID'] = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_CURRENCY);
			
			return $_SESSION['CurrencyID'];
		}
		
		
		/**
		 * Convert $amount from $fromCurrency to current currency
		 * 
		 * @param float $amount
		 * @param string $fromCurrencyCode
		 * @param scCurrency $toCurrency
		 */
		public static function convert($amount, $fromCurrencyCode, scCurrency $toCurrency)
		{
			if ($toCurrency->getCurrencyCode() == $fromCurrencyCode) {
				return $amount;
			}
			
			if (self::getBaseCurrencyCode() == $fromCurrencyCode) {
				return $toCurrency->getCurrencyRate() * $amount;
			}
			
			$fromCurrency = new scCurrency($fromCurrencyCode);
			return ($toCurrency->getCurrencyRate() * $amount) / $fromCurrency->getCurrencyRate();
		}
		
		/**
		 * @param float $baseCurrencyAmount
		 * @return float
		 */
		public static function getCurrentCurrencyAmount($baseCurrencyAmount)
		{
			if (self::getBaseCurrencyCode() == self::getCurrentCurrency()->getCurrencyCode()) {
				return $baseCurrencyAmount;
			}
			
			return self::convert($baseCurrencyAmount, self::getBaseCurrencyCode(), self::getCurrentCurrency());
		}
	}
?>