<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scMultiCurrencyAmount {
		
		/**
		 * @var float[]
		 */
		protected $_amounts = array();
		
		/**
		 * @var scCurrency[]
		 */
		protected $_currencies = array();
		
		/**
		 * Returns new object with the amounts in the base and current currencies set to 0
		 * 
		 * @param float $baseCurrencyAmount
		 * @param float $currentCurrencyAmount
		 * @return scMultiCurrencyAmount
		 */
		public static function getZero()
		{
			$amount = new scMultiCurrencyAmount();
			$amount->set(scCurrencyUtil::getBaseCurrency(), 0);
			$amount->set(scCurrencyUtil::getCurrentCurrency(), 0);
			return $amount;
		}
		
		/**
		 * Returns new object
		 * 
		 * @param float $baseCurrencyAmount
		 * @param float $currentCurrencyAmount
		 * @return scMultiCurrencyAmount
		 */
		public static function factory()
		{
			$amount = new scMultiCurrencyAmount();
			return $amount;
		}
		
		/**
		 * Returns new object with the amount set in the base currency
		 * 
		 * @param float $baseCurrencyAmount
		 * @return scMultiCurrencyAmount
		 */
		public static function factoryBaseCurrency($baseCurrencyAmount)
		{
			$amount = new scMultiCurrencyAmount();
			$amount->set(scCurrencyUtil::getBaseCurrency(), $baseCurrencyAmount);
			return $amount;
		}
		
		protected function __construct($baseCurrencyAmount = null)
		{
			if (!is_null($baseCurrencyAmount)) {
				$this->set(scCurrencyUtil::getBaseCurrency(), $baseCurrencyAmount);
			}
		}
		
		/**
		 * Set the amount in the specified currency
		 * 
		 * @param scCurrency $currency
		 * @param float $amount
		 */
		public function set(scCurrency $currency, $amount)
		{
			$this->_amounts[$currency->getCurrencyCode()] = $amount;
			$this->_currencies[$currency->getCurrencyCode()] = $currency;
		}
		
		/**
		 * Return the amount in the specified currency
		 * 
		 * @param scCurrency $currency if null uses the base currency
		 * @return string
		 */
		public function get(scCurrency $currency = null)
		{
			if (is_null($currency)) {
				$currency = scCurrencyUtil::getBaseCurrency();
			}
			$currencyCode = $currency->getCurrencyCode();
			
			// Check if an amount is set for that currency.
			if (isset($this->_amounts[$currencyCode])) {
				return $this->formatAmountToCurrency($this->_amounts[$currencyCode], $currency);
			}
			
			// There is no amount set for that currency.
			// Check if there is an amount in the base currency.
			$baseCurrencyCode = scCurrencyUtil::getBaseCurrencyCode();
			if (isset($this->_amounts[$baseCurrencyCode])) {
				$this->_amounts[$currencyCode] = scCurrencyUtil::convert($this->_amounts[$baseCurrencyCode], $baseCurrencyCode, $currency);
				$this->_currencies[$currencyCode] = $currency;
				return $this->formatAmountToCurrency($this->_amounts[$currencyCode], $currency);
			}
			
			// There may be amount set in some other currency.
			// For example the object may have been constructed and only given an amount in a non-default currency.
			// This amount may be used for cross conversion.
			if (!empty($this->_amounts)) {
				// There is an amount set in some currency.
				// Since we've already checked for the base currency and there is no amount for it
				// it doesn't matter which of the other amounts (in other currencies) we'll use.
				// So we'll use the first in the _amounts array as it, most probably, is the first set.
				$amount = reset($this->_amounts);
				$fromCurrencyCode = key($this->_amounts);
				if (!isset($this->_currencies[$fromCurrencyCode])) {
					// This is wrong data. If correctly used, the scMultiCurrencyAmount object
					// should contain references to all currencies for which there is an amount set in it.
					throw new miException(sprintf(_i('Amount is set for currency "%s" but the currency object is not set.'), $currencyCode));
				}
				$this->_amounts[$currencyCode] = scCurrencyUtil::convert($amount, $fromCurrencyCode, $currency);
				$this->_currencies[$currencyCode] = $currency;
				return $this->formatAmountToCurrency($this->_amounts[$currencyCode], $currency);
			}
			
			throw new miException(sprintf(_i('Amount is not set in currency "%s".'), $currencyCode));
		}
		
		/**
		 * Return the currencies this amount is defined in
		 * 
		 * @return scCurrency[]
		 */
		public function getCurrencies()
		{
			return $this->_currencies;
		}
		
		/**
		 * Returns TRUE if the two amounts have the same values for all their currencies.
		 * 
		 * @param scMultiCurrencyAmount $compareTo
		 * @return bool
		 */
		public function equals(scMultiCurrencyAmount $compareTo)
		{
			$currencies = array_merge($this->getCurrencies(), $compareTo->getCurrencies());
			foreach ($currencies as $currency) {
				if ($this->get($currency) != $compareTo->get($currency)) {
					return false;
				}
			}
			return true;
		}
		
		/**
		 * Return the amount formatted according to currency
		 * It is expected that the amount is already in the specified currency
		 * 
		 * @param float $amount
		 * @param scCurrency $currency
		 * @return string
		 */
		protected function formatAmountToCurrency($amount, scCurrency $currency)
		{
			return number_format($amount, $currency->getCurrencyNumDecimals(), '.', '');
		}
	}
?>