<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scMultiCurrencyAmountMath {
		
		/**
		 * Add one amount to another.
		 * A new instance of scMultiCurrencyAmount is returned.
		 * 
		 * @param scMultiCurrencyAmount $addend1
		 * @param scMultiCurrencyAmount $addend2
		 * @return scMultiCurrencyAmount
		 */
		public static function add(scMultiCurrencyAmount $addend1, scMultiCurrencyAmount $addend2)
		{
			$currencies = self::getCalculationCurrencies($addend1, $addend2);
			$sum = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$sum->set($currency, $addend1->get($currency) + $addend2->get($currency));
			}
			return $sum;
		}
		
		/**
		 * Substract one amount from another.
		 * A new instance of scMultiCurrencyAmount is returned.
		 * 
		 * @param scMultiCurrencyAmount $minuend
		 * @param scMultiCurrencyAmount $subtrahend
		 * @return scMultiCurrencyAmount
		 */
		public static function subtract(scMultiCurrencyAmount $minuend, scMultiCurrencyAmount $subtrahend)
		{
			$currencies = self::getCalculationCurrencies($minuend, $subtrahend);
			$difference = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$difference->set($currency, $minuend->get($currency) - $subtrahend->get($currency));
			}
			return $difference;
		}
		
		/**
		 * Multiply $multiplicand by $multiplier.
		 * A new instance of scMultiCurrencyAmount is returned as product.
		 * If the specified $multiplier is a number all currency amounts are multiplied by it.
		 * If the specified $multiplier is an instance of scMultiCurrencyAmount
		 * each currency amount in the $multiplicand is multiplied by its corresponding currency amount in the $multiplier.
		 * 
		 * Example:
		 * <code>
		 * // Example data:
		 * // - Current currency = BGN
		 * // - Base currency = EUR
		 * // - Rate: 1 EUR = 2 BGN
		 * 
		 * $amount = scMultiCurrencyAmount::factory();
		 * $amount->set(scCurrencyUtil::getBaseCurrency(), 1);
		 * $amount->set(scCurrencyUtil::getCurrentCurrency(), 2);
		 * $multiplier = 2;
		 * var_dump(scMultiCurrencyAmountMath::multiply($amount, $multiplier));
		 * // Result: 2 EUR, 4 BGN
		 * 
		 * $amount = scMultiCurrencyAmount::factory();
		 * $amount->set(scCurrencyUtil::getBaseCurrency(), 1);
		 * $amount->set(scCurrencyUtil::getCurrentCurrency(), 2);
		 * $multiplier = scMultiCurrencyAmount::factory();
		 * $multiplier->set(scCurrencyUtil::getBaseCurrency(), 20);
		 * $multiplier->set(scCurrencyUtil::getCurrentCurrency(), 40.1);
		 * var_dump(scMultiCurrencyAmountMath::multiply($amount, $multiplier));
		 * // Result: 20 EUR, 80.2 BGN
		 * </code>
		 * 
		 * @param scMultiCurrencyAmount $multiplicand
		 * @param float|scMultiCurrencyAmount $multiplier
		 * @return scMultiCurrencyAmount
		 */
		public static function multiply(scMultiCurrencyAmount $multiplicand, $multiplier)
		{
			if ($multiplier instanceof scMultiCurrencyAmount) {
				return self::multiplyByMultiCurrencyAmount($multiplicand, $multiplier);
			}
			return self::multiplyByNumber($multiplicand, $multiplier);
		}
		
		/**
		 * Divide by $divisor.
		 * A new instance of scMultiCurrencyAmount is returned as quotient.
		 * 
		 * Example:
		 * <code>
		 * // Example data:
		 * // - Current currency = BGN
		 * // - Base currency = EUR
		 * // - Rate: 1 EUR = 2 BGN
		 * 
		 * $amount = scMultiCurrencyAmount::factory();
		 * $amount->set(scCurrencyUtil::getBaseCurrency(), 20);
		 * $amount->set(scCurrencyUtil::getCurrentCurrency(), 80.2);
		 * $divisor = 2;
		 * var_dump(scMultiCurrencyAmount::divide($amount, $divisor));
		 * // Result: 10 EUR, 40.1 BGN
		 * 
		 * $amount = scMultiCurrencyAmount::factory();
		 * $amount->set(scCurrencyUtil::getBaseCurrency(), 20);
		 * $amount->set(scCurrencyUtil::getCurrentCurrency(), 80.2);
		 * $divisor = scMultiCurrencyAmount::factory();
		 * $divisor->set(scCurrencyUtil::getBaseCurrency(), 20);
		 * $divisor->set(scCurrencyUtil::getCurrentCurrency(), 40.1);
		 * var_dump(scMultiCurrencyAmount::divide($amount, $divisor));
		 * // Result: 1 EUR, 2 BGN
		 * </code>
		 * 
		 * @param scMultiCurrencyAmount $dividend
		 * @param float|scMultiCurrencyAmount $divisor
		 * @return scMultiCurrencyAmount
		 */
		public static function divide(scMultiCurrencyAmount $dividend, $divisor)
		{
			if ($divisor instanceof scMultiCurrencyAmount) {
				return self::divideByMultiCurrencyAmount($dividend, $divisor);
			}
			return self::divideByNumber($dividend, $divisor);
		}
		
		/**
		 * Return abs() of the currency amount
		 * 
		 * @param scMultiCurrencyAmount $amount
		 * @return scMultiCurrencyAmount
		 */
		public static function abs($amount)
		{
			$currencies = $amount->getCurrencies();
			$result = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$result->set($currency, abs($amount->get($currency)));
			}
			return $result;
		}
		
		/**
		 * Returns an array of all the currencies in the two amounts
		 * for which there is a set value in at least one of the amounts.
		 * 
		 * @param scMultiCurrencyAmount $amount1
		 * @param scMultiCurrencyAmount $amount2
		 * @return scCurrency[]
		 */
		protected static function getCalculationCurrencies(scMultiCurrencyAmount $amount1, scMultiCurrencyAmount $amount2 = null)
		{
			$currencies = array();
			$baseCurrency = scCurrencyUtil::getBaseCurrency();
			$currencies[$baseCurrency->getCurrencyCode()] = $baseCurrency;
			$currentCurrency = scCurrencyUtil::getCurrentCurrency();
			$currencies[$currentCurrency->getCurrencyCode()] = $currentCurrency;
			// http://www.php.net/array_merge: If the input arrays have the same string keys,
			// then the later value for that key will overwrite the previous one.
			$currencies = array_merge($currencies, $amount1->getCurrencies());
			if (!is_null($amount2)) {
				$currencies = array_merge($currencies, $amount2->getCurrencies());
			}
			return $currencies;
		}
		
		/**
		 * @param scMultiCurrencyAmount $multiplicand
		 * @param float $multiplier
		 * @return scMultiCurrencyAmount
		 */
		protected static function multiplyByNumber(scMultiCurrencyAmount $multiplicand, $multiplier)
		{
			$currencies = self::getCalculationCurrencies($multiplicand);
			$product = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$multiplied = $multiplicand->get($currency) * $multiplier;
				$product->set($currency, $multiplied);
			}
			return $product;
		}
		
		/**
		 * @param scMultiCurrencyAmount $multiplicand
		 * @param scMultiCurrencyAmount $multiplier
		 * @return scMultiCurrencyAmount
		 */
		protected static function multiplyByMultiCurrencyAmount(scMultiCurrencyAmount $multiplicand, scMultiCurrencyAmount $multiplier)
		{
			$currencies = self::getCalculationCurrencies($multiplicand, $multiplier);
			$product = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$multiplied = $multiplicand->get($currency) * $multiplier->get($currency);
				$product->set($currency, $multiplied);
			}
			return $product;
		}
		
		/**
		 * @param scMultiCurrencyAmount $dividend
		 * @param float $divisor
		 * @return scMultiCurrencyAmount
		 */
		protected static function divideByNumber(scMultiCurrencyAmount $dividend, $divisor)
		{
			return self::multiply($dividend, (1 / $divisor));
		}
		
		/**
		 * @param scMultiCurrencyAmount $dividend
		 * @param float $divisor
		 * @return scMultiCurrencyAmount
		 */
		protected static function divideByMultiCurrencyAmount(scMultiCurrencyAmount $dividend, scMultiCurrencyAmount $divisor)
		{
			$currencies = self::getCalculationCurrencies($dividend, $divisor);
			$quotient = scMultiCurrencyAmount::factory();
			foreach ($currencies as $currency) {
				$divided = $dividend->get($currency) / $divisor->get($currency);
				$quotient->set($currency, $divided);
			}
			return $quotient;
		}
	}
?>