<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scBonusPoints {
		
		const ACTION_ASSIGN = 1;
		const ACTION_SPEND = 2;
		
		const TRANSACTION_STATUS_PENDING = 0;
		const TRANSACTION_STATUS_APPROVED = 1;
		const TRANSACTION_STATUS_VOID = 2;
		
		const LIMIT_PERIOD_DAY = 1;
		const LIMIT_PERIOD_WEEK = 2;
		const LIMIT_PERIOD_MONTH = 3;
		const LIMIT_PERIOD_ALLTIME = 4;
		const LIMIT_PERIOD_7_DAYS = 5;
		const LIMIT_PERIOD_30_DAYS = 6;
		
		const PRODUCT_MONEY_ONLY = 0;
		const PRODUCT_POINTS_AND_MONEY = 1;
		const PRODUCT_POINTS_ONLY = 2;
		
		const ASSIGNMENT_REASON_PURCHASE = 1;
		const ASSIGNMENT_REASON_REFERRAL = 2;
		const ASSIGNMENT_REASON_REVIEW = 3;
		const ASSIGNMENT_REASON_VOTE = 4;
		const ASSIGNMENT_REASON_SPEND = 5;
		const ASSIGNMENT_REASON_ADMIN_ACTION = 6;
		const ASSIGNMENT_REASON_RETURN = 7;
		
		const SPENT_REASON_SPEND = 8;
		const SPENT_REASON_DEASSIGN = 9;
		
		const REFERRER_PARAM = 'PointsReferrer';
		
		public static function getAllProductCustomerPointModes()
		{
			return array(
				self::PRODUCT_MONEY_ONLY => _t('Money only'),
				self::PRODUCT_POINTS_AND_MONEY => _t('Points and money'),
				self::PRODUCT_POINTS_ONLY => _t('Points only'),
			);
		}
		
		public static function getAllActionTypes()
		{
			return array(
				self::ACTION_ASSIGN => _t('Earn'),
				self::ACTION_SPEND => _t('Spend'),
			);
		}
		
		public static function getAllActionStatuses()
		{
			return array(
				self::TRANSACTION_STATUS_PENDING => _t('Pending'),
				self::TRANSACTION_STATUS_APPROVED => _t('Approved'),
				self::TRANSACTION_STATUS_VOID => _t('Void'),
			);
		}
		
		public static function getAllLimitPeriods()
		{
			return array(
				self::LIMIT_PERIOD_DAY => _t('Day'),
				self::LIMIT_PERIOD_WEEK => _t('Week'),
				self::LIMIT_PERIOD_MONTH => _t('Month'),
				self::LIMIT_PERIOD_ALLTIME => _t('All time'),
				self::LIMIT_PERIOD_7_DAYS => _t('7 days'),
				self::LIMIT_PERIOD_30_DAYS => _t('30 days'),
			);
		}
		
		/**
		 * Return how many points will be required to cover $price
		 * 
		 * @param scPrice $price
		 * @return int number of points
		 */
		public static function getPricePoints(scPrice $price)
		{
			$bonusPointsRate = scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_CONVERSION_RATE);
			return (int)round($price->getAmount()->get() / $bonusPointsRate);
		}
		
		/**
		 * Checks if the customer has enough points
		 * 
		 * @param int $customerId
		 * @param int $neededPoints
		 * @return bool true if the customer has enough points
		 */
		public static function hasBonusPoints($customerId, $neededPoints)
		{
			$sql = 'SELECT CustomerBonusPoints FROM Customers WHERE CustomerID = ?';
			$points = miStaticDBUtil::execSelect($sql, array($customerId));
			if (!isset($points[0]) || !isset($points[0]['CustomerBonusPoints'])) {
				return false;
			}
			return ($points[0]['CustomerBonusPoints'] >= $neededPoints);
		}
		
		/**
		 * @return bool
		 */
		public static function isOrderStatusAssign($orderStatus)
		{
			$assignPointsStatuses = explode(',', scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_ORDER_STATUSES));
			return in_array($orderStatus, $assignPointsStatuses);
		}
		
		public static function spendBonusPoints($customerId, $spentPoints, $reason, $description = '', $identifier = null)
		{
			if (($customerId <= 0) || ($customerId == ICheckoutModule::GUEST_USER_ID)) {
				// Not a customer that can spend points.
				return;
			}
			
			$spentPoints = (int)$spentPoints;
			if ($spentPoints <= 0) {
				// Nothing to spend.
				return;
			}
			
			// Check if the customer has enough points.
			if (!self::hasBonusPoints($customerId, $spentPoints)) {
				throw new miException(_i('Insufficient points.'));
			}
			
			// Create a transaction
			scBonusPointsTransaction::create(self::ACTION_SPEND, $spentPoints, $identifier, $customerId, $description, $reason);
		}
		
		public static function assignBonusPoints($customerId, $reason, $description = '', $identifier = null, $spentAmount = '')
		{
			if (($customerId <= 0) || ($customerId == ICheckoutModule::GUEST_USER_ID)) {
				// Not a customer that can spend points.
				return;
			}
			
			// Get the number of times the operation has already been done.
			$recordset = scBonusPointsTransaction::getRecordset(self::ACTION_ASSIGN, $identifier, $customerId, $reason);
			$count = $recordset->getRecordsCount();
			
			// TODO: Currently no contra-transactions are created. When they are the check has to be
			// redone so it checks the count of transactions and contra-transaction matches. 
			if ($count) {
				return;
			}
			
			$assignedPoints = self::getAssignmentPointsForReason($customerId, $reason, $spentAmount);
			if ($assignedPoints <= 0) {
				return;
			}
			
			// Create a transaction
			scBonusPointsTransaction::create(self::ACTION_ASSIGN, $assignedPoints, $identifier, $customerId, $description, $reason);
		}
		
		public static function deassignEarnedPoints($identifier, $customerId)
		{
			if (($customerId <= 0) || ($customerId == ICheckoutModule::GUEST_USER_ID)) {
				// Not a customer that can spend points.
				return;
			}
			
			// FIXME: we should create new transaction with SPENT_REASON_DEASSIGN, not delete existing one
			
			// Get assigned points
			$assignmentReasons = array(
				self::ASSIGNMENT_REASON_PURCHASE,
				self::ASSIGNMENT_REASON_SPEND,
			);
			
			$sql = 'SELECT SUM(BonusPointsAmount) AS BonusPointsAmount FROM BonusPointsTransactions' .
				' WHERE BonusPointsTransactionType = ' . self::ACTION_ASSIGN .
				' AND BonusPointsTransactionReason IN (' . implode(', ', $assignmentReasons) . ')' .
				' AND BonusPointsTransactionIdentifier = ' . $identifier .
				' AND BonusPointsTransactionStatus = ' . self::TRANSACTION_STATUS_APPROVED;
			$points = miStaticDBUtil::execSelect($sql);	// FIXME: add params
			
			if (isset($points[0]) && isset($points[0]['BonusPointsAmount']) && ($points[0]['BonusPointsAmount'] > 0)) {
				// Remove them from customer's account
				miStaticDBUtil::execSQL('UPDATE Customers SET CustomerBonusPoints = CustomerBonusPoints - ? WHERE CustomerID = ?', array($points[0]['BonusPointsAmount'], $customerId));
			}
			
			// Remove earned transactions
			$sql = 'DELETE FROM BonusPointsTransactions' .
				' WHERE BonusPointsTransactionType = ' . self::ACTION_ASSIGN .
				' AND BonusPointsTransactionReason IN (' . implode(', ', $assignmentReasons) . ')' .
				' AND BonusPointsTransactionIdentifier = ' . $identifier;
			miStaticDBUtil::execSQL($sql);
		}
		
		public static function returnSpentPoints($identifier, $customerId, $spentPoints)
		{
			if (($customerId <= 0) || ($customerId == ICheckoutModule::GUEST_USER_ID)) {
				// Not a customer that can spend points.
				return;
			}
			
			$spentPoints = (int)$spentPoints;
			if ($spentPoints <= 0) {
				// Nothing to spend.
				return;
			}
			
			// Create auto approved assign transaction
			scBonusPointsTransaction::create(self::ACTION_ASSIGN, $spentPoints, $identifier, $customerId, _t('Return points.'), self::ASSIGNMENT_REASON_RETURN, true);
		}
		
		public static function getReferringUrl($url, $referredEmail)
		{
			$referrer = scCustomerAuth::singleton()->getCustomer();
			if ($referrer === false) {
				return $url;
			}
			
			$code = base64_encode($referrer->get($referrer->getPKName()) . '-' . $referredEmail);
			return scUrlUtil::addQueryParams($url, self::REFERRER_PARAM . '=' . $code);
		}
		
		/**
		 * Return who refered this user, based on the passed hash
		 * 
		 * @param string $hash
		 * @return array($referrerId, $referrerEmail)
		 */
		public static function decodeReferringHash($hash)
		{
			$hash = base64_decode($hash);
			$separatorPos = strpos($hash, '-');
			
			return array(
				substr($hash, 0, $separatorPos),
				substr($hash, $separatorPos + 1),
			);
		}
		
		public static function isValidReferringHash($hash)
		{
			list($referrerId, $referredEmail) = self::decodeReferringHash($hash);
			if (is_numeric($referrerId) && miValidatorEmail::isValid($referredEmail)) {
				return true;
			}
			return false;
		}
		
		public static function setNewCustomerReferrer(scCustomerDomainObject $newCustomer)
		{
			if (!isset($_COOKIE[self::REFERRER_PARAM])) {
				return;
			}
			if (!self::isValidReferringHash($_COOKIE[self::REFERRER_PARAM])) {
				return;
			}
			list($referrerId, $referredEmail) = self::decodeReferringHash($_COOKIE[self::REFERRER_PARAM]);
			$referrerId = (int)$referrerId;
			if ($referrerId <= 0) {
				return;
			}
			
			try {
				$referrer = new scCustomerDomainObject();
				$referrer->read($referrerId);
				
				$newCustomer->set('CustomerReferrerID', $referrerId);
			} catch (miException $e) {
			}
		}
		
		public static function handleBonusPointsReferral()
		{
			$pointsReferrer = miGetParamDefault(self::REFERRER_PARAM, '');
			if ($pointsReferrer != '') {
				$cookieLife = scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REFERRAL_COOKIE_LIFE_IN_DAYS);
				setcookie(self::REFERRER_PARAM, $pointsReferrer, time() + $cookieLife * 24 * 60 * 60, '/');			
			}
		}
		
		protected static function getAssignmentPointsForReason($customerId, $reason, $spentAmount = '')
		{
			switch ($reason) {
				case self::ASSIGNMENT_REASON_PURCHASE:
					$points = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_PURCHASE_EARN);
					$maxUsage = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_PURCHASE_LIMIT);
					$usagePeriod = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_PURCHASE_LIMIT_PERIOD);
					break;
				
				case self::ASSIGNMENT_REASON_REFERRAL:
					$points = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REFERRAL_EARN);
					$maxUsage = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REFERRAL_LIMIT);
					$usagePeriod = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REFERRAL_LIMIT_PERIOD);
					break;
				
				case self::ASSIGNMENT_REASON_REVIEW:
					$points = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REVIEW_EARN);
					$maxUsage = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REVIEW_LIMIT);
					$usagePeriod = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_REVIEW_LIMIT_PERIOD);
					break;
				
				case self::ASSIGNMENT_REASON_VOTE:
					$points = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_VOTE_EARN);
					$maxUsage = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_VOTE_LIMIT);
					$usagePeriod = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_VOTE_LIMIT_PERIOD);
					break;
				
				case self::ASSIGNMENT_REASON_SPEND:
					$points = (int)scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_SPEND_EARN);
					$perAmount = scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_SPEND_AMOUNT);
					return floor($spentAmount / $perAmount) * $points;
				
				default:
					throw new miException(_i('Unknown points assignment reason.'));
			}
			
			if ($maxUsage == 0) {
				return $points;
			}
			
			switch ($usagePeriod) {
				case self::LIMIT_PERIOD_DAY:
					$periodStart = gmmktime(0, 0, 0, gmdate('n'), gmdate('j'), gmdate('Y'));
					break;
				
				case self::LIMIT_PERIOD_WEEK:
					$dayOfWeek = gmdate('N');
					$periodStart = gmmktime(0, 0, 0, gmdate('n'), gmdate('j') - $dayOfWeek + 1, gmdate('Y'));
					break;
				
				case self::LIMIT_PERIOD_MONTH:
					$periodStart = gmmktime(0, 0, 0, gmdate('n'), 1, gmdate('Y'));
					break;
				
				case self::LIMIT_PERIOD_ALLTIME:
					$periodStart = gmmktime(0, 0, 0, 1, 1, 1970);
					break;
				
				case self::LIMIT_PERIOD_7_DAYS:
					$periodStart = gmmktime(0, 0, 0, gmdate('n'), gmdate('j') - 7, gmdate('Y'));
					break;
				
				case self::LIMIT_PERIOD_30_DAYS:
					$periodStart = gmmktime(0, 0, 0, gmdate('n'), gmdate('j') - 30, gmdate('Y'));
					break;
			}
			
			$sql = 'SELECT COUNT(*) AS Assignments FROM BonusPointsTransactions' .
				' WHERE CustomerID = ? AND BonusPointsTransactionTime >= ? AND BonusPointsTransactionType = ? AND BonusPointsTransactionReason = ?';
			$result = miStaticDBUtil::execSelect($sql, array($customerId, $periodStart, self::ACTION_ASSIGN, $reason));
			if ($result[0]['Assignments'] < $maxUsage) {
				return $points;
			}
			return 0;
		}
		
		public static function recalculateCurrentBalance($customerId)
		{
			$sql = 'SELECT '.
						'SUM(IF(BonusPointsTransactionType = ' . self::ACTION_ASSIGN . ', BonusPointsAmount, 0 - BonusPointsAmount)) as CurrentBalance '.
					'FROM BonusPointsTransactions '.
					'WHERE CustomerID = ? AND BonusPointsTransactionStatus = ?';
			$res = miStaticDBUtil::execSelect($sql, array($customerId, scBonusPoints::TRANSACTION_STATUS_APPROVED));
			return $res[0]['CurrentBalance'];
		}
	}
?>