<?php
	/**
	 * Discounts class
	 * 
	 * Copyright (c) 2009,2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class for Discounts management
	 */
	class scDiscount {
		
		const TYPE_AMOUNT = 0;
		const TYPE_PERCENTAGE = 1;
		const TYPE_FREE_SHIPPING = 2;
		
		/**
		 * Applies to
		 */
		//const APPLY_MODE_PER_PRODUCT = 0;
		const APPLY_MODE_PER_QUANTITY = 1;
		const APPLY_MODE_TO_SUBTOTAL = 2;
		
		const IMPLEMENTATION_ORDER_TOTAL = 0; // IOrderTotalModule
		const IMPLEMENTATION_PRODUCT_PRICE_MODIFIER = 1; // IProductPriceModifier
		
		protected $_discountIdFieldName;
		
		protected $_discountId;
		
		public static function getDiscountTypes()
		{
			return array(
				self::TYPE_AMOUNT => '$',
				self::TYPE_PERCENTAGE => '%',
				self::TYPE_FREE_SHIPPING => _t('Free Shipping'),
			);
		}
		
		/**
		 * 
		 * @return array
		 */
		public static function getAppliesToTypes()
		{
			return array(
				//self::APPLY_MODE_PER_PRODUCT => _t('Per product'),
				self::APPLY_MODE_PER_QUANTITY => _t('Per product quantity'),
				self::APPLY_MODE_TO_SUBTOTAL => _t('To subtotal'),
			);
		}
		
		/**
		 * Returns the active shipping modules in scGroupableSelectWidget format.
		 * @return array
		 */
		public static function getShippingMethodsForDropdown()
		{
			$shippingModules = scModulesFactory::getActiveShippingModules();
			
			$return = array();
			foreach ($shippingModules as $shippingModuleId => $shippingModule) {
				$options = array();
				
				$return[$shippingModuleId][scGroupableSelectWidget::GROUP_LABEL] = $shippingModule['ShippingModuleDisplayName'];
				
				$systemShippingMethods = $shippingModule['object']->getShippingMethods();
				foreach ($shippingModule['ShippingModuleActiveMethods'] as $shippingMethodCode => $shippingMethod) {
					$shippingMethodId = $shippingMethod['ShippingMethodID'];
					$systemName = $systemShippingMethods[$shippingMethod['ShippingMethodCode']];
					// By default set the system name
					$options[$shippingMethodId] = $systemName;
					if (!empty($shippingMethod['ShippingMethodDisplayName'])) {
						$options[$shippingMethodId] = $shippingMethod['ShippingMethodDisplayName'];
					}
				}
				
				$return[$shippingModuleId][scGroupableSelectWidget::GROUP_OPTIONS] = $options;
			}
			return $return;
		}
		
		/**
		 * @param scCart $cart
		 * @param scDiscountCalculator $calculator
		 * @param scMultiCurrencyAmount $subTotal
		 * @return array
		 */
		public static function getFreeShippingMethods(scCart $cart, $calculator, scMultiCurrencyAmount $subTotal, $discounts)
		{
			$cartItems = $cart->getItems();
			
			$applicableFreeShippings = array();
			foreach ($cartItems as $item) {
				$applicableFreeShippings[$item->getHash()] = $calculator->getApplicableFreeShippings($item->getProductListingItem(), $subTotal, $discounts);
				if (empty($applicableFreeShippings[$item->getHash()])) {
					// If there are no applicable shipping methods for a product going further is useless.
					return array();
				}
			}
			
			if (count($applicableFreeShippings) != count($cartItems)) {
				throw new miException(_i('Applicable shippings do not match cart items.'));
			}
			
			// Get all shipping methods that may be used fot the customer's shipping zone.
			$shippingsForZone = self::getShippingsForZone($cart);
			
			// Get all shipping methods that may be applied to all products and are active.
			foreach ($applicableFreeShippings as $current) {
				$shippingsForZone = array_intersect($shippingsForZone, $current);
				if (empty($shippingsForZone)) {
					// If there are no matching shipping methods going further is useless.
					return array();
				}
			}
			
			return $shippingsForZone;
		}
		
		/**
		 * @param scAddress $shippingAddress
		 * @return array Array of shipping method IDs that are active for the shipping zone.
		 */
		protected static function getShippingsForZone(scCart $cart)
		{
			$shippingAddress = $cart->getShippingAddress();
			$activeShippingModules = scModulesFactory::getActiveShippingModulesForZone($shippingAddress->getCountryId(), $shippingAddress->getStateId());
			$shippingsForZone = array();
			foreach ($activeShippingModules as $activeShippingModule) {
				foreach ($activeShippingModule['ShippingModuleActiveMethods'] as $methodCode => $method) {
					if (!isset($activeShippingModule['ShippingModuleActiveMethods'][$methodCode]))
						continue;
					$shippingsForZone[] = $method['ShippingMethodID'];
				}
			}
			
			return $shippingsForZone;
		}
		
		/**
		 * scDiscount constructor.
		 * 
		 * @access public
		 * @param string $discountIdFieldName discount id field name
		 * @param string $discountId discount ID
		 */
		public function __construct($discountIdFieldName, $discountId)
		{
			$this->_discountIdFieldName = $discountIdFieldName;
			$this->_discountId = $discountId;
		}
		
		/**
		 * Set discount categories field multi select settings
		 *
		 * @access public
		 * @param pointer $field form field object
		 * @param string $table discount table name
		 */
		public function setDiscountCategoriesFieldSettings(&$field, $table)
		{
			$field->setMode(scDoubleListBoxWidget::MODE_COPY_VALUES);
			
			$dummyForm = new miWebForm(array());
			$categorySelectWidget = new scCategorySelect($dummyForm, array());
			$categoriesArray = $categorySelectWidget->getCategoriesTree(0);
			unset($categoriesArray[0]);		// Remove the top level category
			$field->setOptions($categoriesArray);			
			$field->setSelectedItemsTextFilter(create_function('$name', 'return preg_replace("/.*▢/", "", $name);'));
			
			$field->setLeftToRightJsHandler('SC.Discounts.copySelected');
			$field->setRightToLeftJsHandler('SC.Discounts.removeSelected');
		}
		
		/**
		 * Set discount customer groups field multi select settings
		 *
		 * @access public
		 * @param pointer $field form field object
		 * @param string $table discount table name
		 */
		public function setDiscountCustomerGroupsFieldSettings(&$field, $table)
		{
			$field->setMode(scDoubleListBoxWidget::MODE_COPY_VALUES);
			
			$sql = 'SELECT CustomerGroupID, CustomerGroupName FROM CustomerGroupsML WHERE LanguageID = ' . scMultiLanguage::getCurrentLanguageId();
			$customerGroupsArray = miUtil::getDBArray($sql, 'CustomerGroupID', 'CustomerGroupName');
			$field->setOptions($customerGroupsArray);			
			
			$field->setLeftToRightJsHandler('SC.Discounts.copySelected');
			$field->setRightToLeftJsHandler('SC.Discounts.removeSelected');
		}
	}
?>