<?php
	/**
	 * Email class
	 * 
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	if (!defined('PHP_EOL')) {
		switch (strtoupper(substr(PHP_OS, 0, 3))) {
			// Windows
			case 'WIN':
				define('PHP_EOL', "\r\n");
				break;
			// Mac
			case 'DAR':
				define('PHP_EOL', "\r");
				break;
			// Unix
			default:
				define('PHP_EOL', "\n");
		}
	}
	
	/**
	 * Class send emails
	 */

	class scEmail {
		
		// Method Of Sending Emails
		const METHOD_SENDING_EMAILS_VIA_PHP = 0;
		const METHOD_SENDING_EMAILS_VIA_SMTP_SERVER = 1;
		
		// SMTP Settings
		const SMTP_DEFAULT_PORT = 25;
		const SMTP_TIMEOUT = 10; // seconds
		const SMTP_KEEP_ALIVE = false;
		
		// SMTP Security
		const SMTP_SECURITY_NONE = 0;
		const SMTP_SECURITY_SSL = 1;
		const SMTP_SECURITY_TLS = 2;
		
		/**
		 * @var SMTP
		 */
		protected $_smtpInstance = null;
		
		/**
		 * Charset of the message
		 * 
		 * @access protected
		 */
		protected $_charset;
		
		/**
		 * @var scEmailAddress
		 */
		protected $_mailFrom = null;
		
		/**
		 * Receivers emails
		 * 
		 * @var scEmailAddress[]
		 */
		protected $_mailsTo = array();
		
		/**
		 * @var scEmailAddress[]
		 */
		protected $_mailsCc = null;
		
		/**
		 * @var scEmailAddress[]
		 */
		protected $_mailsBcc = null;
		
		/**
		 * Subject of the message
		 * 
		 * @access protected
		 */
		protected $_subject;
		
		/**
		 * Body of the message
		 * 
		 * @access protected
		 */
		protected $_message;
		
		
		/**
		 * scEmail constructor
		 */
		function __construct()
		{
			$this->_charset = 'utf-8';
		}
		
		/**
		 * Send email
		 * TODO: Use charset and encoding
		 * 
		 * @return false on failure, true on success
		 */
		public function sendEmail()
		{
			return mail(implode(',', $this->_mailsTo), $this->_subject, $this->_message);
		}
		
		/**
		 * Send email with HTML message
		 *
		 * @param string|array $to
		 * @param string $subject
		 * @param string $htmlMessage
		 */
		public function sendHtmlEmail($htmlMessage, $additionalHeaders = array(), $attachments = array())
		{
			$additionalHeaders['From'] = $this->getSenderEmailAddress()->__toString();
			
			$replyToAddress = scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NOTIFICATIONS);
			$replyToName = scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME);
			$replyTo = new scEmailAddress($replyToAddress, $replyToName);
			$additionalHeaders['Reply-To'] = $replyTo->__toString();
			
			if (!empty($this->_mailsCc)) {
				$additionalHeaders['Cc'] = $to = $this->getEmailAddressesAsString($this->_mailsCc);
			}
			if (!empty($this->_mailsBcc)) {
				$additionalHeaders['Bcc'] = $to = $this->getEmailAddressesAsString($this->_mailsBcc);
			}
			
			list($body, $headers) = $this->composeMessage($htmlMessage, $additionalHeaders, $attachments);
			
			return $this->doSendMail($body, $headers, scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NOTIFICATIONS));
		}
		
		/**
		 * @param scEmailAddress[] $mailsTo
		 */
		public function setTo($mailsTo)
		{
			if (!is_array($mailsTo)) {
				$mailsTo = array($mailsTo);
			}
			
			foreach ($mailsTo as $mail) {
				if (!($mail instanceof scEmailAddress)) {
					throw new miException('scEmailAddress expected.');
				}
				$this->_mailsTo[] = $mail;
			}
		}
		
		/**
		 * @param scEmailAddress[] $mailsCc
		 */
		public function setCc($mailsCc)
		{
			if (!is_array($mailsCc)) {
				$mailsCc = array($mailsCc);
			}
			
			foreach ($mailsCc as $mail) {
				if (!($mail instanceof scEmailAddress)) {
					throw new miException('scEmailAddress expected.');
				}
				$this->_mailsCc[] = $mail;
			}
		}
		
		/**
		 * @param scEmailAddress[] $mailsBcc
		 */
		public function setBcc($mailsBcc)
		{
			if (!is_array($mailsBcc)) {
				$mailsBcc = array($mailsBcc);
			}
			
			foreach ($mailsBcc as $mail) {
				if (!($mail instanceof scEmailAddress)) {
					throw new miException('scEmailAddress expected.');
				}
				$this->_mailsBcc[] = $mail;
			}
		}
		
		public function setSubject($subject)
		{
			$this->_subject = $subject;
		}
		
		public function setFrom(scEmailAddress $mailFrom)
		{
			$this->_mailFrom = $mailFrom;
		}
		
		/**
		 * This function decides if to use SMTP or PHP's mail() function 
		 * based on the SETTING_EMAIL_METHOD_OF_SENDING setting value
		 * 
		 * @param string $subject
		 * @param string $body
		 * @param string $headers
		 */
		protected function doSendMail($body, $headers)
		{
			if (scSettings::getSettingValue(scSettings::SETTING_EMAIL_METHOD_OF_SENDING) == self::METHOD_SENDING_EMAILS_VIA_PHP) {
				return $this->doSendPhpMail($body, $headers);
			} else {
				try {
					return $this->doSendSmtpMail($body, $headers);
				} catch (scSmtpException $e) {
					scLog::singleton()->info(sprintf(_i('SMTP Error: %s'), $e->getMessage()), scLog::CATEGORY_NONE);
					return false;
				}
			}
		}
		
		/**
		 * Sends mail via SMTP
		 * 
		 * @param string $body
		 * @param string $headers
		 */
		protected function doSendSmtpMail($body, $headers)
		{
			if (!$this->smtpConnect()) {
				$error = $this->_smtpInstance->getError();
				throw new scSmtpException(sprintf(_i('Could not connect to SMTP host: %s.'), scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_HOST)));
			}
			
			// The SMTP->Mail() setter expects an email only. No name.
			$from = $this->getSenderEmailAddress()->getAddress();
			if (!$this->_smtpInstance->Mail($from)) {
				$error = $this->_smtpInstance->getError();
				throw new scSmtpException(sprintf(_i('From header "%s" not accepted: %s (%s: %s).'), $from, $error['error'], $error['smtp_code'], $error['smtp_msg']));
			}
			
			// Attempt to attach all recipients
			foreach ($this->_mailsTo as $email) {
				// The SMTP->Recipient() setter expects an email only. No name.
				$email = $email->getAddress();
				if (!$this->_smtpInstance->Recipient($email)) {
					$error = $this->_smtpInstance->getError();
					throw new scSmtpException(sprintf(_i('Could not add email "%s": %s (%s: %s).'), $email, $error['error'], $error['smtp_code'], $error['smtp_msg']));
				}
			}
			
			$headers .= $this->_smtpInstance->CRLF . 'Subject: ' . self::encodeHeader('', $this->_subject);
			if (!$this->_smtpInstance->Data($headers . $this->_smtpInstance->CRLF . $this->_smtpInstance->CRLF . $body)) {
				$error = $this->_smtpInstance->getError();
				throw new scSmtpException(sprintf(_i('Data was not accepeted: %s (%s: %s).'), $error['error'], $error['smtp_code'], $error['smtp_msg']));
			}
			if (self::SMTP_KEEP_ALIVE) {
				$this->_smtpInstance->Reset();
			}
			return true;
		}
		
		/**
		 * @return scEmailAddress
		 */
		protected function getSenderEmailAddress()
		{
			if (!($this->_mailFrom instanceof scEmailAddress)) {
				$address = scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NOTIFICATIONS);
				$name = scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME);
				return new scEmailAddress($address, $name);
			}
			return $this->_mailFrom;
		}
		
		/**
		 * Connect to the SMTP server and 
		 * authenticate before it if neccessary
		 * 
		 * @return boolean true or throws a scSmtpException 
		 */
		protected function smtpConnect()
		{
			if (is_null($this->_smtpInstance)) {
				require_once(dirname(__FILE__) . '/SMTP.php');
				$this->_smtpInstance = new SMTP();
			}
			// set debug level 0-off 2-all
			$this->_smtpInstance->do_debug = 0;
			
			// shortcuts
			$smtpHost = scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_HOST);
			$smtpPort = scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_PORT);
			if (empty($smtpPort)) {
				$smtpPort = self::SMTP_DEFAULT_PORT;
			}
			
			$smtpUsername = scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_USERNAME);
			$smtpPassword = scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_PASSWORD);
			$smtpSecurity = scSettings::getSettingValue(scSettings::SETTING_EMAIL_SMTP_SECURITY);
			$authenticate = scSettings::getSettingValue(scSettings::SETTING_EMAIL_USE_SMTP_AUTHENTICATION);
			
			$connection = $this->_smtpInstance->Connected();
			
			// Retry while there is no connection
			try {
				
				$tls = ($smtpSecurity == self::SMTP_SECURITY_TLS);
				$ssl = ($smtpSecurity == self::SMTP_SECURITY_SSL);
		
				if ($this->_smtpInstance->Connect(($ssl ? 'ssl://':'') . $smtpHost, $smtpPort, self::SMTP_TIMEOUT)) {
		
					$this->_smtpInstance->Hello($this->getServerHostname());
		
					if ($tls) {
						if (!$this->_smtpInstance->StartTLS()) {
							$error = $this->_smtpInstance->getError();
							throw new scSmtpException(sprintf(_i('STARTTLS not accepted from server: %s (%s: %s).'), $error['error'], $error['smtp_code'], $error['smtp_msg']));
						}
		
						// We must resend HELO after tls negotiation
						$this->_smtpInstance->Hello($hello);
					}
		
					$connection = true;
					if ($authenticate) {
						if (!$this->_smtpInstance->Authenticate($smtpUsername, $smtpPassword)) {
							$error = $this->_smtpInstance->getError();
							throw new scSmtpException(sprintf(_i('Could not authenticate: %s (%s: %s).'), $error['error'], $error['smtp_code'], $error['smtp_msg']));
						}
					}
				}
				if (!$connection) {
					$error = $this->_smtpInstance->getError();
					throw new scSmtpException(sprintf(_i('Could not connect to SMTP host "%s" via port %d: %s (%s: %s).'), $smtpHost, $smtpPort, @$error['error'], @$error['smtp_code'], @$error['smtp_msg']));
				}
			} catch (scSmtpException $e) {
				$this->_smtpInstance->Reset();
				throw $e;
			}
			return true;
		}
		
		/**
		 * Determines the hostname for the SMTP HELLO command
		 * 
		 * @return string
		 */
		protected function getServerHostname()
		{
			if (isset($_SERVER['SERVER_NAME'])) {
				$result = $_SERVER['SERVER_NAME'];
			} else {
				$result = scPaths::getHttpHost();
			}
			return $result;
		}
		
		/**
		 * Sends mail via PHP's mail() function
		 * 
		 * @param string $to
		 * @param string $subject
		 * @param string $body
		 * @param string $headers
		 */
		protected function doSendPhpMail($body, $headers)
		{
			$to = $this->getEmailAddressesAsString($this->_mailsTo);
			$subject = self::encodeHeader('', $this->_subject);
			
			return mail($to, $subject, $body, $headers);
		}
		
		/**
		 * Composes the MIME message
		 *
		 */
		protected function composeMessage($htmlMessage, $headers = array(), $attachments = array())
		{
			$headers = $headers + array(
				'X-Mailer' => 'Summer Cart ' . file_get_contents(dirname(__FILE__) . '/../../../VERSION'),
				'MIME-Version' => '1.0'
			);
			
			$parts = array();
			
			$params = array(
				'contentType' => 'text/html',
				'charset' => $this->_charset
			);
			$parts[] = new scEmailPart($htmlMessage, $params);
			
			if (!empty($attachments)) {
				foreach ($attachments as $attachment) {
					if (!($attachment instanceof scEmailPart)) {
						continue;
					}
					$parts[] = $attachment;
				}
			}
			
			list($body, $headers) = $this->getMultiPartBodyAndHeaders($parts, $headers);
			
			return array($body, $headers);
		}
		
		/**
		 * @param array $parts
		 * @param array $headers
		 * @return array
		 */
		protected function getMultipartBodyAndHeaders($parts, $headers)
		{
			if (count($parts) == 1) {
				return $this->getSinglePartBodyAndHeaders($parts[0], $headers);
			}
			
			$boundary = $this->generateMultiPartBoundary();
			
			$headers['Content-Type'] = 'multipart/mixed; boundary="' . $boundary . '"';
			
			$stringParts = array();
			$stringParts[] = _i('This is a message with multiple parts in MIME format.');
			foreach ($parts as $part) {
				list($partBody, $partHeaders) = $this->getSinglePartBodyAndHeaders($part);
				
				$stringParts[] = '--' . $boundary;
				$stringParts[] = $partHeaders . PHP_EOL . PHP_EOL . $partBody;
			}
			$stringParts[] = '--' . $boundary . '--';
			
			$body = implode(PHP_EOL, $stringParts);
			$headers = $this->encodeHeaders($headers);
			
			return array($body, $headers);
		}
		
		/**
		 * @param scEmailPart $part
		 * @param array $headers
		 * @return array
		 */
		protected function getSinglePartBodyAndHeaders(scEmailPart $part, $headers = array())
		{
			list($partHeaders, $body) = $part->encode();
			$headers += $partHeaders;
			$headers = $this->encodeHeaders($headers);
			return array($body, $headers);
		}
		
		protected function generateMultiPartBoundary()
		{
			return md5(uniqid(null, true));
		}
		
		/**
		 * @param scEmailAddress[] $inputEmails
		 */
		protected function getEmailAddressesAsString($inputEmails)
		{
			$outputEmails = array();
			foreach ($inputEmails as $mail) {
				$outputEmails[] = $mail->__toString();
			}
			return implode(',', $outputEmails);
		}
		
		protected function encodeHeaders($headers)
		{
			$alreadyEncodedHeaders = array('From', 'Reply-To', 'Cc', 'Bcc', 'Content-Type');	// TODO: Correctly encode all addr-spec headers according to RFC 2047
			
			$encoded = array();
			foreach ($headers as $headerName => $headerValue) {
				if (in_array($headerName, $alreadyEncodedHeaders))
					$encoded[] = $headerName . ': ' . $headerValue;
				else
					$encoded[] = self::encodeHeader($headerName, $headerValue);
			}
			return implode(PHP_EOL, $encoded);
		}
		
		protected function encodeRecipientsHeader($header)
		{
			preg_match_all('/([^,]+)<.+>(?:,)?/uU', $header, $matches);
			foreach ($matches[1] as $key => $encodable) {
				// $encodable is the name preceding the email in <>, eg. "Encodable Name <test@test.com>"
				$encodable = trim($encodable);
				$encoded = str_replace($encodable, self::encodeHeader('', $encodable), $matches[0][$key]);
				$header = str_replace($matches[0][$key], $encoded, $header);
			}
			return $header;
		}
		
		public static function encodeHeader($headerName, $headerValue)
		{
			// If ASCII only - don't encode it
			if (scUtf8::isAsciiPrint($headerValue) and (strlen($headerName) + strlen($headerValue) <= 74)) {
				if ($headerName != '')
					return $headerName . ': ' . $headerValue;
				else
					return $headerValue;
			}
			
			return self::utf8EncodeMimeHeader($headerName, $headerValue);
		}
		
		/**
		 * Encode mime header
		 * 
		 * @param string $headerName if the name is not empty it will be prepended to the encoded header
		 * @param string $headerValue the header value
		 * @param string $chunkEnd Each chunk's suffix
		 * @return string the encoded header
		 */
		public static function utf8EncodeMimeHeader($headerName, $headerValue, $chunkEnd = null)
		{
			if (is_null($chunkEnd)) {
				$chunkEnd = PHP_EOL . ' ';
			}
			
			if ($headerName != '')
				$headerName .= ': ';
			$headerSize = ($headerName == '') ? 0 : strlen($headerName);
			
			// Max encoded characters: 60 - length_of_header_and_colon
			// Then converted to base64 number of characters using 4/3 ratio
			$max = intval(((60 - $headerSize) / 4) * 3);	
			$pos = 0;
			$len = strlen($headerValue);
			
			$buffer = '';
			while ($pos < $len) {
				
				// Check if this is the last part
				if ($pos + $max >= $len) {
					$buffer .= base64_encode(substr($headerValue, $pos));
					break;
				}
				
				// Don't split multibyte chars
				$skipbytes = 0;
				while ((ord($headerValue[$pos + $max + $skipbytes]) & 0xC0) == 0x80) 
					$skipbytes--;
				
				$buffer .= base64_encode(substr($headerValue, $pos, $max + $skipbytes)) . '?=' . $chunkEnd . '=?UTF-8?B?';
				$pos += $max + $skipbytes;
				
				$max = 45; // Max encoded characters for the remaining lines: 60
			}
			
			return $headerName . '=?UTF-8?B?' . $buffer . '?=';
		}
	}
	
	/**
	 * Class representing email MIME part
	 */
	class scEmailPart {
		
		protected $_body;
		protected $_params;
		
		public function __construct($body, $params = array())
		{
			$this->_body = $body;
			$this->_params = $params;
		}
		
		public function encode()
		{
			$headers = array();
			
			$headers['Content-Transfer-Encoding'] = 'base64';
			$headers['Content-Type'] = $this->_params['contentType'] . '; charset=' . $this->_params['charset'];
			$body = rtrim(chunk_split(base64_encode($this->_body), 76, PHP_EOL));
			
			return array($headers, $body);
		}
	}
	
	class scEmailAttachment extends scEmailPart {
		
		/**
		 * @var scUploadedFile
		 */
		protected $_uploadedFile;
		
		public function __construct(scUploadedFile $file)
		{
			$this->_uploadedFile = $file;
		}
	
		public function encode()
		{
			$headers = array();
			
			$headers['Content-Transfer-Encoding'] = 'base64';
			$headers['Content-Disposition'] = 'attachment';
			$headers['Content-Type'] = $this->_uploadedFile->getType() . '; name="' . scEmail::utf8EncodeMimeHeader('', $this->_uploadedFile->getName(), '') . '"';
			$body = rtrim(chunk_split(base64_encode($this->_uploadedFile->getContents()), 76, PHP_EOL));
			
			return array($headers, $body);
		}
	}
	
	class scEmailAddress {
		
		/**
		 * @var string
		 */
		protected $_address;
		
		/**
		 * @var string
		 */
		protected $_name;
		
		public function __construct($address, $name = '')
		{
			$this->_address = $address;
			$this->_name = $name;
		}
		
		public function getAddress()
		{
			return $this->_address;
		}
		
		public function getName()
		{
			return $this->_name;
		}
		
		public function __toString()
		{
			if ($this->_name == '') {
				return $this->_address;
			}
			return '"' . scEmail::encodeHeader('', $this->_name) . '" <' . $this->_address . '>';
		}
	}
	
	class scSmtpException extends miException {}
?>