<?php
	/**
	 * Email Notifications Class
	 * 
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Class generate notification emails
	 */

	class scEmailNotifications {
		
		/**
		 * Send to friend
		 *
		 * @param string $mailTo receiver email address
		 * @param string $senderName sender name
		 * @param string $description sender description
		 * @return false on failure, true on success
		 * @throws miException if $productId is not found
		 */
		public function sendToFriend($customerEmail, $customerName, $friendEmail, $emailNotes, $productId)
		{	
			$productInfo = $this->getProductInfo($productId);
			
			// Construct the e-mail subject
			$subject = _t('Information for product') . ' ' . $productInfo['ProductName'];
			
			$productUrl = scBonusPoints::getReferringUrl($productInfo['ProductUrl'], $friendEmail);
			
			// Parse the e-mail and send it to the requested e-mail
			$parser = new scTemplateParser();
			$parser->readClientAreaTemplate('customer/emails/send_to_friend.tmpl');
			$parser->assign('%%CUSTOMEREMAIL%%', miI18N::htmlEscape($customerEmail));
			$parser->assign('%%CUSTOMERNAME%%', miI18N::htmlEscape($customerName));
			$parser->assign('%%EMAILNOTES%%', miI18N::htmlEscape($emailNotes));
			$parser->assign('%%PRODUCTURL%%', miI18N::htmlEscape($productUrl));
			$parser->assign('%%PRODUCTNAME%%', miI18N::htmlEscape($productInfo['ProductName']));
			$html = $parser->templateParse();
			
			$email = new scEmail();
			$email->setFrom(new scEmailAddress($customerEmail));
			$email->setTo(new scEmailAddress($friendEmail));
			$email->setSubject($subject);
			return $email->sendHtmlEmail($html);
		}
		
		/**
		 * Returns the product info
		 * 
		 * @param int $productId
		 * @return array
		 * @throws miException if $productId is not found
		 */
		protected function getProductInfo($productId)
		{
			// Get the product display name
			$productName = miStaticDBUtil::execSelect(
				'SELECT ProductName FROM ProductsML WHERE ProductID = ? AND LanguageID = ? LIMIT 1',
				array($productId, scMultiLanguage::getCurrentLanguageId())
			);
			if (empty($productName))
				throw new miException(sprintf(_t('Unknown product id "%s".'), $productId));
			
			$productName = reset($productName);
			$productName = $productName['ProductName'];
			
			// Get the product details url
			$defaultName = $productName;
			if (scMultiLanguage::getDefaultLanguageId() != scMultiLanguage::getCurrentLanguageId()) {
				$defaultName = miStaticDBUtil::execSelect(
					'SELECT ProductName FROM ProductsML WHERE ProductID = ? AND LanguageID = ? LIMIT 1',
					array($productId, scMultiLanguage::getDefaultLanguageId())
				);
				$defaultName = reset($defaultName);
				$defaultName = $defaultName['ProductName'];
			}
			$productUrl = scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_PRODUCT, $productId, $defaultName), scUrlUtil::URL_TYPE_ABSOLUTE);
				
			return array('ProductUrl' => $productUrl, 'ProductName' => $productName);
		}
		
		/**
		 * Send signup notification email
		 *
		 * @param int $customerId new customer ID
		 * @return false on failure, true on success
		 */
		public function sendSignupNotification($customerId)
		{	
			// Get customer data
			$customer = $this->getProfileData($customerId);
			
			// Generate email template for signup notification
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/emails/signup_notification.tmpl');
			$t->assignArray(miUIUtil::templetizeRows($customer));
			
			$section = new miTemplateParserSectionInfo('password', 1);
			if ($customer['CustomerPasswordHashMode'] != scAuth::HASH_MODE_NONE) {
				$section ->setIterationsCount(0);
			}
			$t->addSectionInfo($section);
			
			$email = new scEmail();
			$email->setTo(new scEmailAddress($customer['CustomerEmail']));
			$email->setSubject(_t('Signup notification'));
			return $email->sendHtmlEmail($t->templateParse());
		}
		
	    /**
		 * Send forgot password via email
		 *
		 * @param int $customerId customer ID
		 * @return false on failure, true on success
		 */
		public function sendForgotPass($customerId)
		{
			// Get customer date
			$customer = $this->getProfileData($customerId);
			
			// Generate email template signup notification
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/emails/forgot_pass.tmpl');
			$t->assignArray(miUIUtil::templetizeRows($customer));
			
			if ($customer['CustomerPasswordHashMode'] != scAuth::HASH_MODE_NONE) {
				$resetPassUrl = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_FORGOT_PASS);
				$params = 'action=resetPass&CustomerID=' . $customer['CustomerID'] .
					'&secret=' . md5($customer['CustomerPassword']);
				$resetPassUrl = scUrlUtil::addQueryParams($resetPassUrl, $params);
				$resetPassUrl = scUrlUtil::convertUrl($resetPassUrl, scUrlUtil::URL_TYPE_ABSOLUTE);
				$t->assign('%%RESET_PASS_URL%%', $resetPassUrl);
				
				$t->addSectionInfo(new miTemplateParserSectionInfo('hashed', 1));
				$t->addSectionInfo(new miTemplateParserSectionInfo('plaintext', 0));
			} else {
				$t->addSectionInfo(new miTemplateParserSectionInfo('hashed', 0));
				$t->addSectionInfo(new miTemplateParserSectionInfo('plaintext', 1));
			}
			
			$email = new scEmail();
			$email->setTo(new scEmailAddress($customer['CustomerEmail']));
			$email->setSubject(_t('Forgotten Password'));
			return $email->sendHtmlEmail($t->templateParse());
		}
		
		/**
		 * Sends email notification when form is submitted
		 * 
		 * @param string $to send notification to email
		 * @param string $formName the form name
		 * @param array $data the submitted data
		 */
		public function sendFormEmail($to, $formName, $items, $formFields, $data)
		{
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/emails/form.tmpl');
			$t->assign('%%FORMNAME%%', miI18N::htmlEscape($formName));
			
			$dataKeys = array_keys($data);
			$itemNames = array();
			$itemDisplayValues = array();
			$attachmentFiles = array();
			foreach ($dataKeys as $dataKey) {
				if (substr($dataKey, 0, strlen(scFormItem::CUSTOM_HTML_FIELD_PREFIX)) == scFormItem::CUSTOM_HTML_FIELD_PREFIX) {
					$itemNames[] = $dataKey;
					$itemDisplayValues[] = $data[$dataKey];
					continue;
				}
				if ($formFields[$dataKey] instanceof scFileUploadWidget) {
					$attachmentFiles = $attachmentFiles + $formFields[$dataKey]->getData();
					continue;
				}
				$itemNames[] = miI18N::htmlEscape($items[substr($dataKey, 4)]['CMSFormItemName']);
				$controlVars = $formFields[$dataKey]->getControl();
				$itemDisplayValues[] = $controlVars[$dataKey];
			}
			
			$attachments = array();
			foreach ($attachmentFiles as $attachmentFile) {
				$attachments[] = new scEmailAttachment($attachmentFile);
			}
			
			$values = array('%%ITEMDISPLAYVALUE%%' => $itemDisplayValues, '%%ITEMNAME%%' => $itemNames);
			$section = new miTemplateParserSectionInfo('items', count($itemDisplayValues), $values);
			$t->setSectionInfos(array($section));
			
			$email = new scEmail();
			$email->setTo(new scEmailAddress($to));
			$email->setSubject(sprintf(_t('Form: %s'), $formName));
			return $email->sendHtmlEmail($t->templateParse(), array(), $attachments);
		}
		
		/**
		 * Send new order notification
		 * 
		 * @param int $orderId
		 * @throws miException if the email is missing and it's not the guest customer
		 */
		public function sendNewOrderEmail($orderId)
		{
			scOrdersCache::fullLoadOrderData($orderId);
			$order = scOrdersCache::getOrder($orderId);
			
			$to = $this->getCustomerEmailFromOrder($order);
			
			// If the customer email is empty, and it's not the guest customer - throw exception
			if (empty($to) && ($order->get('CustomerID') != ICheckoutModule::GUEST_USER_ID)) {
				throw new miException(_t('Cannot send new order notification - customer email is missing.'));
			}
			
			$administrators = $this->getOrderDepartmentEmails();
			
			if (empty($to) && empty($administrators)) {
				throw new miException(_t('Cannot send new order notification - customer and administrator emails are missing.'));
			}
			
			if (empty($to)) {
				$to = $administrators;
				$bcc = array();
			} else {
				$bcc = $administrators;
			}
			
			$notificationLines[] = '<strong>' . _t('Thank you for ordering!') . '</strong>';
			$notificationLines[] = _t('Your order will be fulfilled as soon as possible.');
			return $this->sendOrderNotification($order, $to, $notificationLines, $bcc);
		}
		
		public function sendOrderNotification(scOrder $order, $to, $notificationLines, $bcc = array(), $subject = null)
		{
			if (is_null($subject)) {
				$subject = _t('New order #') . $order->getPK();
			}
			
			$productsSection = array();
			$linesSection = array();
			$shipToBillingSection = array();
			$shippingDiffersFromBillingSection = array();
			
			$orderData = $order->getRow();
			
			$templateVars = scEmailOrdersUI::singleton()->getOrderInfoVars($order);
			$orderData = array_merge($orderData, $templateVars);
			$productsSection[0] = scEmailOrdersUI::singleton()->getOrderItemsSection($order, scOrdersUI::ORDER_ITEMS_WITHOUT_IMAGES, $order->getEgoods());
			$linesSection[0] = scEmailOrdersUI::singleton()->getOrderTotalLinesSection($order);
			$shipToBillingSection[0] = new miTemplateParserSectionInfo('shipToBillingAddress', $order->get('OrderShipToBillingAddress'));
			$shippingDiffersFromBillingSection[0] = new miTemplateParserSectionInfo('shippingDiffersFromBilling', (int)!$order->get('OrderShipToBillingAddress'));
			
			// Set section orders
			$ordersSection = new miTemplateParserSectionInfo();
			$ordersSection->setSectionInfo('orders', 1, miUIUtil::templetizeRows(miUIUtil::transposeRows(array($order))));
			$ordersSection->addSubsectionsArray('products', $productsSection);
			$ordersSection->addSubsectionsArray('lines', $linesSection);
			$ordersSection->addSubsectionsArray('shipToBillingAddress', $shipToBillingSection);
			$ordersSection->addSubsectionsArray('shippingDiffersFromBilling', $shippingDiffersFromBillingSection);
			
			$template = new scTemplateParser();
			// Get the store owner info
			$template->assign('%%COMPANYNAME%%', scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME));
			// Show page
			$template->setSectionInfos(array($ordersSection));
			$template->readClientAreaTemplate('customer/emails/new_order.tmpl');
			
			$template->assignArray(array(
				'%%ORDERPAGEURL%%' => scPaths::getSecureUrl('view_order.php?OrderID=' . $order->getPK()),
				'%%EMAIL%%' => scSettings::getSettingValue(scSettings::SETTING_EMAIL_HELP_SUPPORT),
				'%%COMPANYNAME%%' => scSettings::getSettingValue(scSettings::SETTING_COMPANY_NAME),
				'%%COMPANYPHONE%%' => scSettings::getSettingValue(scSettings::SETTING_COMPANY_PHONE),
				'%%STORE_ROOT_URL%%' => 'http://' . scPaths::getHttpHost() . scPaths::getStoreWebPath()
			));
			$template->assignArray(miUIUtil::templetizeRows($orderData));
			
			$notificationLines = array('HTML_NOTIFICATIONLINE' => $notificationLines);
			$notificationLines = new miTemplateParserSectionInfo('notificationLines', count($notificationLines), miUIUtil::templetizeRows($notificationLines));
			$template->addSectionInfo($notificationLines);
			
			// Send the notification email
			$email = new scEmail();
			$email->setTo($to);
			$email->setBcc($bcc);
			$email->setSubject($subject);
			return $email->sendHtmlEmail($template->templateParse());
		}
		
		public function sendOrderEgoods($orderId)
		{
			$order = new scOrder($orderId);
			if (!$order->hasEgoods()) {
				throw new miException(_t('No E-Goods to send.'));
			}
			
			$to = $this->getCustomerEmailFromOrder($order);
			
			// If the customer email is empty, and it's not the guest customer - throw exception
			if (empty($to))
				throw new miException(_t('Cannot deliver E-Goods - customer email is missing.'));
			
			$subject = sprintf(_t('E-Goods delivery for order #%s'), $orderId);
			
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/emails/order_egoods.tmpl');
			$t->assign('%%ORDERID%%', miI18N::htmlEscape($orderId));
			$t->assign('%%ACCESSKEY%%', miI18N::htmlEscape($order->getAccessKey()));
			$t->assign('%%CARTURL%%', miI18N::htmlEscape(scPaths::getSecureUrl()));
			$t->assignArray(miUIUtil::templetizeRows($order->getRow()));
			
			// Send the notification email
			$email = new scEmail();
			$email->setTo($to);
			$email->setSubject($subject);
			return $email->sendHtmlEmail($t->templateParse());
		}
		
		/**
		 * @param scOrder $order
		 * @return scEmailAddress
		 */
		public function getCustomerEmailFromOrder(scOrder &$order)
		{
			$email = $order->get('OrderCustomerEmail');
			if (empty($email)) {
				return null;
			}
			
			$name = $order->get('OrderBillingFirstName') .' '. $order->get('OrderBillingLastName');
			return new scEmailAddress($email, $name);
		}
		
		/**
		 * Returns array with the emails of the order department
		 * @return scEmailAddress[]
		 */
		public function getOrderDepartmentEmails()
		{
			$emails = scSettings::getSettingValue(scSettings::SETTING_EMAIL_ORDER_DEPARTMENT);
			$administrators = array_map('trim', explode(',', $emails));
			
			$emailAddresses = array();
			foreach ($administrators as $key => $administrator) {
				if (!empty($administrator))
					$emailAddresses[] = new scEmailAddress($administrator);
			}
			return $emailAddresses;
		}
		
		/**
		 * Sends administrative notes about given order to administrators (and eventualy customer who created the order)
		 * 
		 * @param string $note
		 * @param int $orderId
		 * @param bool $sendToCustomer
		 */
		public function sendOrderNote($note, $orderId, $sendToCustomer)
		{
			$administrators = $this->getOrderDepartmentEmails();
			
			$order = new miSqlRecord('Orders', 'OrderID');
			$order->readPK($orderId);
				
			// Check if the customer has email
			$validCustomer = $sendToCustomer && ($order->get('OrderCustomerEmail') != '');
			
			$email = new scEmail();
			
			if ($validCustomer) {
				$customerEmail = $order->get('OrderCustomerEmail');
				$customerName = $order->get('OrderBillingFirstName') . ' ' . $order->get('OrderBillingLastName');
				$email->setTo(new scEmailAddress($customerEmail, $customerName));
				
				if (!empty($administrators)) {
					$email->setBcc($administrators);
				}
			} else {
				if (empty($administrators))
					return true;	// not a valid customer and no administrators - do nothing
				
				$email->setTo($administrators);
			}
			
			$t = new scTemplateParser();
			$t->readClientAreaTemplate('customer/emails/order_note.tmpl');
			$t->assign('%%ORDERID%%', miI18N::htmlEscape($orderId));
			$t->assign('%%NOTE%%', miI18N::htmlEscape($note));
			$t->assignArray(miUIUtil::templetizeRows($order->getRow()));
			
			// Send the notification email
			$email->setSubject(_t('Administrative note for order #') . $orderId);
			return $email->sendHtmlEmail($t->templateParse());
		}
		
		/**
		 * Get profile data
		 *
		 * @param int $customerId the customer ID
		 * @return array customer profile data
		 */
		protected function getProfileData($customerId)
		{	
			// Get customer data
			$customer = new scCustomerDomainObject();
			$customer->read($customerId);
			$customerDataArray = $customer->getRow();
			
			// Get billing address
			$address = $customer->getBillingAddress();
			$fields = scCustomerAddress::getAddressFields(scCustomerAddress::BILLING);
			foreach ($address as $addressField => $data) {
				$mappedField = array_search($addressField, $fields);
				if ($mappedField === false) {
					continue;
				}
				$customerDataArray[$mappedField] = $data;
			}
			
			// Get shipping address
			$address = $customer->getShippingAddress();
			$fields = scCustomerAddress::getAddressFields(scCustomerAddress::SHIPPING);
			foreach ($address as $addressField => $data) {
				$mappedField = array_search($addressField, $fields);
				if ($mappedField === false) {
					continue;
				}
				$customerDataArray[$mappedField] = $data;
			}
			
			return $customerDataArray;
		}
		
		public function sendStatusChangeNotification(scOrder $order, $newOrderStatus)
		{
			if (!scOrderStatuses::singleton()->shouldSendNotification($newOrderStatus)) {
				return;
			}
			
			$notifications = new scEmailNotifications();
			$to = $this->getCustomerEmailFromOrder($order);
			if (empty($to)) {
				return;
			}
			
			$administrators = $this->getOrderDepartmentEmails();
			$notificationLines = array(
				sprintf(_t('Order status changed to "%s".'), scOrderStatuses::singleton()->getOrderStatusName($newOrderStatus)),
			);
			$subject = sprintf(_t('Order #%s status changed'), $order->getPK());
			$this->sendOrderNotification($order, $to, $notificationLines, $administrators, $subject);
		}
	}
?>