<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scFilesUtil {
		
		public static function getUploadErrorDescription($error)
		{
			if (!defined('UPLOAD_ERR_NO_TMP_DIR'))	// From 5.0.3
				define('UPLOAD_ERR_NO_TMP_DIR', 6);
			if (!defined('UPLOAD_ERR_CANT_WRITE'))	// From 5.1.0
				define('UPLOAD_ERR_CANT_WRITE', 7);
			if (!defined('UPLOAD_ERR_EXTENSION'))	// From 5.2.0
				define('UPLOAD_ERR_EXTENSION', 8);
			
			switch ($error) {
				case UPLOAD_ERR_OK:
					return _t('No error.');
				case UPLOAD_ERR_INI_SIZE:
					return _t('File size exceeds the upload_max_filesize directive in php.ini.');
				case UPLOAD_ERR_FORM_SIZE:
					return _t('The uploaded file exceeds the maximum upload file size.');
				case UPLOAD_ERR_PARTIAL:
					return _t('The uploaded file was only partially uploaded.');
				case UPLOAD_ERR_NO_FILE:
					return '';
				case UPLOAD_ERR_NO_TMP_DIR:
					return _t('Missing temporary folder.');
				case UPLOAD_ERR_CANT_WRITE:
					return _t('Failed to write file to disk.');
				case UPLOAD_ERR_EXTENSION:
					return _t('File upload stopped by extension.');
			}
			
			return _t('Unknown error.');
		}
		
		/**
		 * @param string $string
		 * @return string
		 */
		public static function getFilenameFromString($string)
		{
	    	// Remove the file's extension to get the filename only
			$string = explode('.', $string);
			$extension = array_pop($string);
			$string = implode('.', $string);
			// Remove unwanted characters from the file's name and add back the extension
			return scFilesUtil::escapeStringForFilename($string) . '.' . $extension;
		}
		
		/**
		 * @param string $string
		 * @return string
		 */
		public static function escapeStringForFilename($string)
		{
		    $except = array('\\', '/', ':', '*', '?', '"', '<', '>', '|', '.');
		    return str_replace($except, '', $string);
		}
		
		/**
		 * Try to find available file name, by adding _1, _2, and so on, up to 5000
		 * 
		 * @param string $string
		 * @param callback $isAvailableCallback
		 * @return string
		 */
		public static function generateFilenameFromString($string, $isAvailableCallback)
		{
			// Get the extension out of the filename.
			// Suppose it is the last string after a .
			// E.g. file.name.string would get .string as $extension.
			$string = explode('.', $string);
			$extension = '';
			if (count($string) > 1) {
				$extension = array_pop($string);
			}
			$string = implode('.', $string);
			
			// Separate the filename on _ and get the last part.
			// Suppose it is a file.name_1.string
			$exploded = explode('_', $string);
			// Get the number (1) out of the string.
			$number = '';
			if (count($exploded) > 1) {
				$number = array_pop($exploded);
			}
			// Check if the number is an empty string.
			// e.g. file_name_.ext
			// If it is not an empty string check if it is a number.
			// It would be "name" in "file_name.ext".
			if ($number == '') {
				$number = 1;
			} else if (!is_numeric($number)) {
				$exploded[] = $number;
				$number = 1;
			}
			$string = implode('_', $exploded);
			$string = array(self::escapeStringForFilename($string));
			do {
				$filename = implode('_', $string);
				// Call the callback function to check if the filename is avaialable.
				$isAvailable = call_user_func_array($isAvailableCallback, array($filename, $filename . '.' . $extension));
				// If available, return
				if ($isAvailable) {
					if ($extension != '') {
						$extension = '.' . $extension;
					}
					return $filename . $extension;
				}
				// The filename is already in use.
				// If a number has not been recognized in it, add one.
				// Otherwise increase the current one.
				if (!isset($string[1])) {
					$string[1] = $number;
				} else {
					$string[1] += 1;
				}
			} while ($string[1] < $number + 5000); // ...this is way too much
			
			// We've tested 5000 filenames and they were all busy.
			throw new miException(_i('File name already exists.'));
		}
		
		/**
		 * Moves a directory recursively.
		 * @param string $source
		 * @param string $target
		 * @return bool
		 */
		public static function recursiveMove($source, $target)
	    {
			if (!is_dir($source)) {
				copy($source, $target);
				unlink($source);
				return true;
			}
			
			if (!mkdir($target)) {
				return false;
			}
			
			$d = dir($source);
			while (false !== ($entry = $d->read())) {
				if ($entry == '.' || $entry == '..') {
					continue;
				}
				
				$fullEntry = $source . '/' . $entry;           
				if (is_dir($fullEntry)) {
					if (!self::recursiveMove($fullEntry, $target .'/'. $entry )) {
						return false;
					}
					continue;
				}
				copy($fullEntry, $target .'/'. $entry);
				unlink($fullEntry);
			}
			$d->close();
			rmdir($source);
			return true;
	    }
	    
		/**
		 * Copies a file or a directory recursively.
		 * @param string $source
		 * @param string $target
		 * @return bool
		 */
		public static function recursiveCopy($source, $target)
	    {
			if (!is_dir($source)) {
				$sourceDir = dirname($source);
				$targetDir = dirname($target);
				if (!is_dir($targetDir) && !self::recursiveCopy($sourceDir, $targetDir)) {
					return false;
				}
				if (!copy($source, $target)) {
					return false;
				}
				return true;
			}
			
			if (is_dir($target)) {
				return false;
			}
			
			if (!mkdir($target)) {
				return false;
			}
			
			$d = dir($source);
			while (false !== ($entry = $d->read())) {
				if (($entry == '.') || ($entry == '..')) {
					continue;
				}
				
				$fullEntry = $source . '/' . $entry;           
				if (is_dir($fullEntry)) {
					if (!self::recursiveCopy($fullEntry, $target .'/'. $entry )) {
						return false;
					}
					continue;
				}
				copy($fullEntry, $target .'/'. $entry);
			}
			$d->close();
			return true;
	    }
	    
	    public static function getMaxUploadSize()
		{
			$postMaxSize = self::getBytes(ini_get('post_max_size'));
			$uploadMaxFilesize = self::getBytes(ini_get('upload_max_filesize'));
			
			return min($postMaxSize, $uploadMaxFilesize);
		}
		
		protected static function getBytes($val)
		{
			$val = trim($val);
			$unit = strtolower(substr($val, -1));
			switch ($unit) {
				case 'g':
					return $val * 1073741824;
				case 'm':
					return $val * 1048576;
				case 'k':
					return $val * 1024;
			}
			return (int)$val;
		}
		
		public static function bytesToShorthand($val)
		{
			if ($val % 1073741824 == 0)
				return ($val / 1073741824) . ' ' . _t('GB');
			if ($val % 1048576 == 0)
				return ($val / 1048576) . ' ' . _t('MB');
			if ($val % 1024 == 0)
				return ($val / 1024) . ' ' . _t('KB');
			return $val;
		}
	}
?>