<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scInterfaceTexts {
		
		const TEXT_TYPE_CODE = 1;
		const TEXT_TYPE_TEMPLATE = 2;
		
		/**
		 * Array with all the translated texts
		 * 
		 * @var array
		 */
		protected $_translatedTexts = array();
		
		/**
		 * New text hashes are saved in this array.
		 * Their translations will be saved when the script finishes execution.
		 * 
		 * @var array
		 */
		protected $_newTexts = array();
		
		/**
		 * Return the one and only instance of the object
		 * 
		 * @return scInterfaceTexts
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scInterfaceTexts();
			}
			return $instance;
		}
		
		/**
		 * Return $text translated into the current language
		 * 
		 * @param string $text
		 * @return string the translated text
		 */
		public static function getTranslatedText($text)
		{
			return self::singleton()->getText($text);
		}
		
		/**
		 * Return $text translated into the specific language
		 * 
		 * @param string $text
		 * @param string $languageId
		 * @return string the translated text
		 */
		public static function getTranslatedTextInLanguage($text, $languageId)
		{
			if ($languageId == scMultiLanguage::getCurrentLanguageId())
				return self::singleton()->getText($text);
			
			return self::singleton()->getTextInLanguage($text, $languageId);
		}
		
		protected function __construct()
		{
			register_shutdown_function(array($this, 'saveTexts'));
			
			$this->loadTexts();
		}
		
		/**
		 * Ensures that the object cannot be cloned
		 */
		protected function __clone() {}
		
		/**
		 * Save the translated texts information in the cache
		 */
		public function saveTexts()
		{
			// We can't close the connection, but at least flush the contents to the browser
			while (ob_get_level() > 0) {
				if (!@ob_end_flush())
					break;
			}
			
			$page = scPaths::getCurrentScriptPathName();
			
			try {
				$multiValues = array();
				foreach ($this->_newTexts as $textHash) {
					if (!array_key_exists($textHash, $this->_translatedTexts))
						continue;
					
					list(, $textId) = $this->_translatedTexts[$textHash];
					$multiValues[] = array('TextsCachePagePath' => $page, 'TextID' => $textId);
				}
				miStaticDBUtil::execMultiInsert('TextsCache', $multiValues, miDBUtilImpl::FLAG_IGNORE);
			}
			catch (miException $exception)
			{
				scLog::singleton()->log(scLog::SEVERITY_WARNING, sprintf(_i('Cannot insert into TextsCache: %s'), $exception->getMessage()), scLog::CATEGORY_NONE);
			}
		}
			
		protected function getText($text)
		{
			$textHash = sprintf('%u', crc32($text));
			
			if (array_key_exists($textHash, $this->_translatedTexts)) {
				list($text, ) = $this->_translatedTexts[$textHash];
				return $text;
			}
			
			// No text found in the cache - read from the database and mark to be saved
			$textInfo = miStaticDBUtil::execSelect(
				'SELECT TextID, TextTranslated FROM TextsTranslated WHERE TextIsTranslated = 1 AND TextOriginalHash = ? AND LanguageID = ?',
				array($textHash, scMultiLanguage::getCurrentLanguageId())
			);
			
			if (count($textInfo) == 0)
				return $text;
			
			$textInfo = reset($textInfo);
			$this->_translatedTexts[$textHash] = array($textInfo['TextTranslated'], $textInfo['TextID']);
			$this->_newTexts[$textHash] = $textHash;
			
			return $textInfo['TextTranslated'];
		}
		
		protected function getTextInLanguage($text, $languageId)
		{
			$textHash = sprintf('%u', crc32($text));
			
			$textInfo = miStaticDBUtil::execSelect(
				'SELECT TextID, TextTranslated FROM TextsTranslated WHERE TextIsTranslated = 1 AND TextOriginalHash = ? AND LanguageID = ?',
				array($textHash, $languageId)
			);
			
			if (count($textInfo) == 0)
				return $text;
			
			return $textInfo[0]['TextTranslated'];
		}
		
		/**
		 * Load the translated texts information in the cache
		 */
		protected function loadTexts()
		{
			$texts = miStaticDBUtil::execSelect(
				'SELECT *,TextsTranslated.TextID AS TextID FROM TextsCache INNER JOIN TextsTranslated USING(TextID) WHERE TextIsTranslated = 1 AND TextsCachePagePath = ? AND LanguageID = ?',
				array(scPaths::getCurrentScriptPathName(), scMultiLanguage::getCurrentLanguageId())
			);
			
			foreach ($texts as $text)
				$this->_translatedTexts[$text['TextOriginalHash']] = array($text['TextTranslated'], $text['TextID']);
		}
	}
?>