<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scMultiLanguage {
		
		/**
		 * The one and only instance of the object
		 * 
		 * @var scMultiLanguage $_instance
		 */
		protected static $_instance = null;
		
		/**
		 * Array with all active languages
		 * 
		 * @var array
		 */
		protected $_languages = array();
		
		/**
		 * Array with all active language names
		 * array(LanguageID => LanguageName)
		 * 
		 * @var array
		 */
		protected $_languageNames = array();
		
		/**
		 * Array with all active language codes
		 * array(LanguageCode => LanguageID)
		 * 
		 * @var array
		 */
		protected $_languageCodes = array();
		
		/**
		 * Array with the directions of the languages
		 * array(LanguageCode => LanguageDirection)
		 */
		protected $_languageDirections = array();
		
		/**
		 * The current language code
		 */
		protected $_currentLanguageCode = 'EN';
		
		/**
		 * 
		 */
		protected function __construct()
		{
			$languagesRs = new miSqlRecordset('Languages');
			$languagesRs->addFilter(new miSqlFilterEqual('LanguageIsActive', 1));
			$languagesRs->setOrder('LanguageSort');
			$this->_languages = $languagesRs->getAllRecords();
			
			// Load all available languages
			foreach ($this->_languages as $row) {
				$this->_languageCodes[$row['LanguageCode']] = $row['LanguageID'];
				$this->_languageNames[$row['LanguageID']] = $row['LanguageName'];
				$this->_languageDirections[$row['LanguageCode']] = $row['LanguageDirection'];
			}
			
			$this->initCurrentLanguage();
		}
		
		/**
		 * Set the current language code for the viewing user
		 */
		protected function initCurrentLanguage()
		{
			if ((miGetParamDefault('UILanguage', '') != '') and ($this->isValidLanguageCode(miGetParamDefault('UILanguage', ''))))
				$_SESSION['UILanguage'] = miGetParamDefault('UILanguage', '');
			
			if (isset($_SESSION['UILanguage'])) {
				$this->_currentLanguageCode = $_SESSION['UILanguage'];
				return;
			}
			
			// Set the default language code
			// NOTE: Uses scUtil::getDbValue() instead of scSettings::getSettingValue() becuase the scSettings class depends on the current language code
			$defaultLanguageId = scUtil::getDbValue('Settings', 'SettingValue', 'SettingName', scSettings::SETTING_DEFAULT_LANGUAGE, '');
			$languageCodes = array_flip($this->_languageCodes);
			if (isset($languageCodes[$defaultLanguageId]))
				$_SESSION['UILanguage'] = $this->_currentLanguageCode = $languageCodes[$defaultLanguageId];
		}
		
		/**
		 * Ensures that the object cannot be cloned
		 */
		protected function __clone() {}
		
		/**
		 * Return the one and only instance of the object
		 * 
		 * @return scMultiLanguage
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scMultiLanguage();
				
			return self::$_instance;
		}
		
		/**
		 * Returns the current language code for the viewing user
		 */
		public static function getCurrentLanguageCode()
		{
			return self::singleton()->_currentLanguageCode;
		}
		
		/**
		 * Return the id of the currently selected language
		 */
		public static function getCurrentLanguageId()
		{
			return self::singleton()->getLanguageIdByCode(self::getCurrentLanguageCode());
		}
		
		public static function getCurrentLanguageDirection()
		{
			$multilanguage = self::singleton();
			if (!isset($multilanguage->_languageDirections[strtoupper($multilanguage->_currentLanguageCode)]))
				return scLanguageDomainObject::LANGUAGE_DIRECTION_LTR;
			
			return $multilanguage->_languageDirections[strtoupper($multilanguage->_currentLanguageCode)];
		}
		
		/**
		 * Returns the default language id
		 * 
		 * @return int
		 * @throws miException
		 */
		public static function getDefaultLanguageId()
		{
			$defaultLanguageId = scSettings::getSettingValue(scSettings::SETTING_DEFAULT_LANGUAGE);
			if (!self::singleton()->isValidLanguageId($defaultLanguageId))
				throw new miException(_t('Default language is not set!'));
			
			return $defaultLanguageId;
		}
		
		/**
		 * @return string
		 * @throws miException
		 */
		public static function getDefaultLanguageCode()
		{
			$defaultLanguageId = self::getDefaultLanguageId();
			return scMultiLanguage::singleton()->getLanguageCodeById($defaultLanguageId);
		}
		
		public static function getActiveLanguageCodes()
		{
			return array_flip(self::singleton()->getLanguageCodes());
		}
		
		/**
		 * Returns array with the active language names
		 * 
		 * @return array the languageNames
		 */
		public static function getActiveLanguageNames()
		{
			return self::singleton()->getLanguageNames();
		}
		
		/**
		 * Return all active language codes (as keys) and their corresponding language ids (as values)
		 * 
		 * @return array
		 */
		public function getLanguageCodes()
		{
			return $this->_languageCodes;
		}
		
		/**
		 * Return all active language names (as values) and their corresponding language ids (as keys)
		 * 
		 * @return array
		 */
		public function getLanguageNames()
		{
			return $this->_languageNames;
		}
		
		/**
		 * Set the current language to $languageId
		 * @param int $languageId
		 */
		public function setCurrentLanguageId($languageId)
		{
			$languageCode = array_search($languageId, $this->_languageCodes);
			if ($languageCode === false)
				return;
			
			$_SESSION['UILanguage'] = $languageCode;
			$this->_currentLanguageCode = $languageCode;
		}
		
		/**
		 * Set the current language to $languageCode
		 * @param string $languageCode
		 */
		public function setCurrentLanguageCode($languageCode)
		{
			if (!array_key_exists($languageCode, $this->_languageCodes))
				return;
			
			$_SESSION['UILanguage'] = $languageCode;
			$this->_currentLanguageCode = $languageCode;
		}
		
		/**
		 * Gets the language id by code. If not found the return the language id of the first language in DB
		 * 
		 * @param string $languageCode
		 * @return int the language id
		 */
		public function getLanguageIdByCode($languageCode)
		{
			$languageCode = strtoupper($languageCode);
			if (array_key_exists($languageCode, $this->_languageCodes))
				return $this->_languageCodes[$languageCode];
			
			if (count($this->_languageCodes) == 0)
				throw new miException(sprintf('Language code "%s" not found.', $languageCode));

			return reset($this->_languageCodes);
		}
		
		/**
		 * @param int $languageId
		 * @return string the language code
		 */
		public function getLanguageCodeById($languageId)
		{
			return array_search($languageId, $this->_languageCodes);
		}
		
		/**
		 * Checks if given language id is valid - if language with this id exists
		 * 
		 * @return bool
		 */
		public function isValidLanguageId($languageId)
		{
			return in_array($languageId, $this->_languageCodes);
		}
		
		/**
		 * Checks if language with this code exists
		 * 
		 * @return bool
		 */
		public function isValidLanguageCode($languageCode)
		{
			return isset($this->_languageCodes[$languageCode]);
		}
		
		
		/**
		 * Returns HTML with the language selection options
		 * 
		 * @return string the html
		 */
		public function getUILanguageSelectionHtml()
		{
			$result = '';
			$curLanguageCode = self::getCurrentLanguageCode();
			foreach ($this->_languages as $language) {
				$selected = ($language['LanguageCode'] == $curLanguageCode) ? ' selected="selected"' : '';
				$result .= '<option value="' . miI18N::htmlEscape($language['LanguageCode']) . '"' . $selected . '>' .
					miI18N::htmlEscape($language['LanguageName']) . '</option>';
			}
			return $result;
		}
		
		/**
		 * Returns array with the active languages
		 * 
		 * @return array the languages rows
		 */
		public function getActiveLanguages()
		{
			return $this->_languages;
		}
	}
?>