<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scXliffExport {
		/**
		 * Return XLIFF file contents
		 * 
		 * @param $body
		 * @param $languageCode
		 * @return string
		 */
		protected function getXliffContents($body, $languageCode)
		{
			$definitions = array(
				array(
					scXml::TAG => 'xliff',
					scXml::ATTRIBUTES => array(
						'version' => '1.2',
						'xmlns' => 'urn:oasis:names:tc:xliff:document:1.2',
						'xmlns:xsi' => 'http://www.w3.org/2001/XMLSchema-instance',
						'xsi:schemaLocation' => 'urn:oasis:names:tc:xliff:document:1.2 xliff-core-1.2-strict.xsd',
					),
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'file',
							scXml::ATTRIBUTES => array(
								'original' => $this->getOriginalName(),
								'source-language' => 'EN',
								'target-language' => $languageCode,
								'datatype' => 'php',
								'date' => scDateTime::timestampToIso8601(time()),
								'product-name' => 'Summer Cart',
								'product-version' => scVersion::getVersion(),
							),
							scXml::CHILDREN => array(
								array(
									scXml::TAG => 'body',
									scXml::CHILDREN => $body
								)
							)
						)
					)
				),
			);
			
			return "<?xml version='1.0' encoding='utf-8'?>" . "\n" . scXml::xmlWriter($definitions);
		}
		
		/**
		 * Return the "original" xliff attribute. It is the class name, with the scXliffExport stripped
		 * @return string
		 */
		protected function getOriginalName()
		{
			$className = get_class($this);
			return substr($className, 13);
		}
		
		/**
		 * Return the 2-letter language code from $languageId
		 * @param int $languageId
		 * @return string
		 */
		protected function getLanguageCode($languageId)
		{
			return scUtil::getDbValue('Languages', 'LanguageCode', 'LanguageID', $languageId, '');
		}
		
		/**
		 * Return text array in the following format:
		 * array(
		 * 		array('id' => '', 'source' => '', 'target' => ''),
		 * 		array('id' => '', 'source' => '', 'target' => '')
		 * )
		 * 
		 * @param int $sourceLanguageId
		 * @param int $targetLanguageId
		 * @param string $keyColumnName
		 * @param string $textColumnName
		 * @param $rows rows returned from miSqlRecordset with multilanguage data
		 * @return array
		 */
		protected function getTextForTranslation($sourceLanguageId, $targetLanguageId, $keyColumnName, $textColumnName, $rows)
		{
			$sourceTexts = array();
			$targetTexts = array();
			foreach ($rows as $value) {
				if ($value['LanguageID'] == $sourceLanguageId)
					$sourceTexts[$value[$keyColumnName]] = $value[$textColumnName];
				if ($value['LanguageID'] == $targetLanguageId)
					$targetTexts[$value[$keyColumnName]] = $value[$textColumnName];
			}
			
			$result = array();
			foreach ($sourceTexts as $key => $value) {
				$value = $value;
				if (trim($value) == '')
					continue;
				
				$target = isset($targetTexts[$key]) ? $targetTexts[$key] : '';
				$result[] = array(
					'source' => $value,
					'target' => $target,
					'id' => $key
				);
			}
			return $result;
		}
		
		/**
		 * Return xliff body for $texts
		 * 
		 * The format of $texts is:
		 * array(
		 * 		array('id' => '', 'source' => '', 'target' => ''),
		 * 		array('id' => '', 'source' => '', 'target' => '')
		 * )
		 * 
		 * If target is empty, the translation is not available
		 * 
		 * @param array $texts
		 * @return array xml definitions
		 */
		protected function getXliffBody($texts)
		{
			$body = array();
			foreach ($texts as $text) {
				$attributes = array('id' => $text['id']);
				
				$transUnit = array();
				$transUnit[] = array(
					'tag' => 'source',
					'data' => $text['source']
				);
				if ($text['target'] != '') {
					$transUnit[] = array(
						'tag' => 'target',
						'data' => $text['target'],
					);
					$attributes['approved'] = 'yes';
				}
				
				$body[] = array(
					'tag' => 'trans-unit',
					scXml::ATTRIBUTES => $attributes,
					scXml::CHILDREN => $transUnit
				);
			}
			return $body;
		}
		
		/**
		 * Perform export operation
		 * @param int $languageId
		 * @return string
		 */
		public function export($languageId)
		{
			$texts = $this->getTextsFromDb($languageId);
			
			$body = $this->getXliffBody($texts);
			
			$languageCode = $this->getLanguageCode($languageId);
			return $this->getXliffContents($body, $languageCode);
		}
		
		protected function checkXliffXml($xml, $languageCode, $xliffFilename)
		{
			if (count($xml->file) != 1)
				throw new Exception(sprintf('XLIFF file %s contains more than one <file> element.', $xliffFilename));
			
			$xliffFileElement = $xml->file;
			$sourceLanguage = strtoupper($xliffFileElement->attributes()->{'source-language'});
			if ($sourceLanguage != 'EN')
				throw new Exception('source-language must be EN.');
			$targetLanguage = strtoupper($xliffFileElement->attributes()->{'target-language'});
			if ($targetLanguage != $languageCode)
				throw new Exception(sprintf('target-language must be %s. It is %s.', $languageCode, $targetLanguage));
		}
		
/*		
		protected function getTextsFromXliff($xliffFilename, $languageCode)
		{
			$xml = simplexml_load_file($xliffFilename);
			
			$this->checkXliffXml($xml, $languageCode);
			
			$xliffFileElement = $xml->file;
			$result = array();
			foreach ($xliffFileElement->body->{'trans-unit'} as $transUnit) {
				
				$textOrig = (string)$transUnit->source;
				$textTrans = (string)$transUnit->target;
				
				$result[] = array(
					'source' => (string)$textOrig,
					'target' => (string)$textTrans,
					'id' => (string)$transUnit->attributes()->id
				);
			}

			return $result;
		}*/
		
		/**
		 * Merge new texts (as existing in the database) into an existing Xliff file
		 * Used to supply new texts for translations
		 * 
		 * @param int $languageId
		 * @param string $xliffFilename
		 * @param bool $overwriteFile
		 */
		public function merge($languageId, $xliffFilename, $overwriteFile)
		{
			$languageCode = $this->getLanguageCode($languageId);
			
			// Get the current translations from the XLIFF file
			$path = '../../../../../sc-util/html/langs/summer_cart_ui/';
			$xliffFilename = $path . strtolower($languageCode) . '/' . $xliffFilename;
			
			// Get the updated texts from the DB
			$textsFromDb = $this->getTextsFromDb($languageId);
			$textsFromDb = miUtil::getArrayWithKeys($textsFromDb, 'source');	// The source should be unique
			
			// Perform the merge
			
			// Find all texts that have no translations - they will be in $textsFromDb after the loop
			if (!file_exists($xliffFilename))
				throw new miException(sprintf('File %s does not exists.', $xliffFilename));
			$xml = simplexml_load_file($xliffFilename);
			$this->checkXliffXml($xml, $languageCode, $xliffFilename);
			
			$xliffFileElement = $xml->file;
			foreach ($xliffFileElement->body->{'trans-unit'} as $transUnit) {
				$textOrig = (string)$transUnit->source;
				
				if (isset($textsFromDb[$textOrig]))
					unset($textsFromDb[$textOrig]);
			}
			
			// Add all texts in $textsFromDb to the end of XLIFF file
			
			// Get DOM from SimpleXML and find the <file> element
			$dom = dom_import_simplexml($xml)->ownerDocument;
			$dom->formatOutput = true;
			$dom->preserveWhiteSpace = true;
			$fileElement = $dom->getElementsByTagName('body')->item(0);
			
			// In $newdoc build DOM with all new trans-unit nodes
			$body = $this->getXliffBody($textsFromDb);
			$appendChildXliff = $this->getXliffContents($body, $languageCode);
			$newdoc = new DOMDocument;
			$newdoc->formatOutput = true;
			$newdoc->preserveWhiteSpace = true;
			$newdoc->loadXML($appendChildXliff);
			
			$elems = $newdoc->getElementsByTagName('trans-unit');
			foreach ($elems as $elem) {
				$elem = $dom->importNode($elem, true);
				$fileElement->appendChild($elem);
			}
			
			$xmlText = $dom->saveXML();
			$xmlText = str_replace('</trans-unit><trans-unit', "</trans-unit>\n   <trans-unit", $xmlText); // Fix formatting
			$xmlText = str_replace('<?xml version="1.0" encoding="utf-8"?>', "<?xml version='1.0' encoding='utf-8'?>", $xmlText);
			
			if ($overwriteFile) {
				file_put_contents($xliffFilename, $xmlText);
				echo 'XLIFF file ' . $xliffFilename . ' overwritten.<br/>';
				return;
			}
			
			echo $xmlText;
		}
		
		/**
		 * Return the language id for English language
		 * @return int
		 */
		protected static function getEnglishLanguageId()
		{
			static $langId = null;
			
			if ($langId === null)
				$langId = scUtil::getDbValue('Languages', 'LanguageID', 'LanguageCode', 'EN');
			
			return $langId;
		}
	}

	class scXliffExportUserInterfaceTexts extends scXliffExport {
//		public function export($languageId)
//		{
//			$body = $this->getXliffBodyTexts($languageId);
//			
//			$languageCode = $this->getLanguageCode($languageId);
//			return $this->getXliffContents($body, $languageCode);
//		}
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,Texts.TextOriginalHash,TextTranslated FROM Texts LEFT JOIN TextsTranslated USING(TextID) WHERE LanguageID=? OR LanguageID=? ORDER BY TextOriginal';
			$textData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'TextOriginalHash', 'TextTranslated', $textData);
		}
		
		/**
		 * Return the XLIFF body XML definitions for User Interface Texts
		 * 
		 * @param int $languageId
		 * @return array xml definitions
		 */
/*		protected function getXliffBodyTexts($languageId)
		{
			$sql = 'SELECT Texts.*, TextsTranslated.TextTranslated, TextsTranslated.TextIsTranslated FROM Texts LEFT JOIN TextsTranslated ON Texts.TextID=TextsTranslated.TextID AND LanguageID=?';
			$texts = miStaticDBUtil::execSelect($sql, array($languageId));
			
			$body = array();
			foreach ($texts as $text) {
				$transUnit = array();
				$transUnit[] = array(
					'tag' => 'source',
					'data' => $text['TextOriginal']
				);
				
				if ($text['TextTranslated'] !== null) {
					$transUnit[] = array(
						'tag' => 'target',
						'data' => $text['TextTranslated'],
//						scXml::ATTRIBUTES => array('state' => ($text['TextIsTranslated'] != 0) ? 'translated' : 'needs-translation')
					);
				}
				
				$attributes = array('id' => $text['TextOriginalHash']);
				if ($text['TextIsTranslated']) {
					$attributes['approved'] = 'yes';
				}
				$body[] = array(
					'tag' => 'trans-unit',
					scXml::ATTRIBUTES => $attributes,
					scXml::CHILDREN => $transUnit
				);
			}
			return $body;
		} */
	}
	
	/**
	 * Export helps
	 */
	class scXliffExportHelps extends scXliffExport {
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,Helps.HelpID,HelpText FROM Helps JOIN HelpsML USING(HelpID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'HelpID', 'HelpText', $helpsData);
		}
	}
	
	/**
	 * Export quick links
	 */
	class scXliffExportQuickLinks extends scXliffExport {
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,AdminQuickLinks.AdminQuickLinkID,AdminQuickLinkText FROM AdminQuickLinks JOIN AdminQuickLinksML USING(AdminQuickLinkID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'AdminQuickLinkID', 'AdminQuickLinkText', $helpsData);
		}
	}
	
	/**
	 * Export country names
	 */
	class scXliffExportCountries extends scXliffExport {
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,Countries.CountryID,CountryName FROM Countries JOIN CountriesML USING(CountryID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'CountryID', 'CountryName', $helpsData);
		}
	}
	
	/**
	 * Export state names
	 */
	class scXliffExportStates extends scXliffExport {
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,States.StateID,StateName FROM States JOIN StatesML USING(StateID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'StateID', 'StateName', $helpsData);
		}
	}
	
	/**
	 * Export currencies
	 */
	class scXliffExportCurrencies extends scXliffExport {
		
		/**
		 * Export currency names
		 * @param $languageId
		 * @return string
		 */
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,Currencies.CurrencyID,CurrencyName FROM Currencies JOIN CurrenciesML USING(CurrencyID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'CurrencyID', 'CurrencyName', $helpsData);
		}
	}
	
	/**
	 * Export order statuses
	 */
	class scXliffExportOrderStatuses extends scXliffExport {
		
		protected function getTextsFromDb($languageId)
		{
			$sql = 'SELECT LanguageID,OrderStatuses.OrderStatusID,OrderStatusName FROM OrderStatuses JOIN OrderStatusesML USING(OrderStatusID) WHERE LanguageID=? OR LanguageID=?';
			$helpsData = miStaticDBUtil::execSelect($sql, array(self::getEnglishLanguageId(), $languageId));
			
			return $this->getTextForTranslation(self::getEnglishLanguageId(), $languageId, 'OrderStatusID', 'OrderStatusName', $helpsData);
		}
	}
?>