<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Imports translation file in XLIFF format
	 */
	abstract class scXliffImport {
		protected $xliffBasename = '';
		protected $textsToBeTranslatedCount = array();
		protected $unusedTranslations = array();
		protected $missingTranslations = array();
		protected $translatedCount = array();
		
		/**
		 * Import Xliff file
		 * @param string $xliffFilename
		 * @param string $expectedTargetLanguageCode expected target language code
		 */
		public function importLangFromXliffFile($xliffFilename, $expectedTargetLanguageCode)
		{
			$xml = simplexml_load_file($xliffFilename);
			$this->importLangFromXliff($xml, $expectedTargetLanguageCode);
		}
		
		/**
		 * Import Xliff file
		 * @param string $xliffString
		 * @param string $expectedTargetLanguageCode expected target language code
		 */
		public function importLangFromXliffString($xliffString, $expectedTargetLanguageCode)
		{
			$xml = simplexml_load_string($xliffString);
			$this->importLangFromXliff($xml, $expectedTargetLanguageCode);
		}
		
		/**
		 * Import Xliff file
		 * @param SimpleXMLElement $xml
		 * @param string $expectedTargetLanguageCode expected target language code
		 */
		protected function importLangFromXliff(SimpleXMLElement $xml, $expectedTargetLanguageCode)
		{
			if (count($xml->file) != 1)
				throw new miException(_i('Expected only one <file> element in a XLIFF file.'));
			
			foreach ($xml->file as $xliffFileElement) {
				$sourceLanguageCode = (string)$xliffFileElement['source-language'];
				$targetLanguageCode = (string)$xliffFileElement['target-language'];
				if (strtoupper($sourceLanguageCode) != 'EN')
					throw new miException(_i('Only EN is supported as source language.'));
				if (strtoupper($targetLanguageCode) != strtoupper($expectedTargetLanguageCode))
					throw new miException(sprintf(_t('Expected target language code "%s", but found "%s".'), $expectedTargetLanguageCode, $targetLanguageCode));
				
				$this->importFileElement($xliffFileElement);
			}
		}
		
		protected function getLanguageIdFromCode($languageCode)
		{
			$sql = 'SELECT LanguageID FROM Languages WHERE LanguageCode=?';
			$lang = miStaticDBUtil::execSelect($sql, array($languageCode));
			if (empty($lang))
				throw new miException(sprintf(_t('Could not find langauge with language code %s.'), $languageCode));
			
			return $lang[0]['LanguageID'];
		}
		
		protected function getItemsFromDb($sql, $keyField, $valueField)
		{
			$rows = miStaticDBUtil::execSelect($sql);
			$keyedArray = array();
			
			foreach ($rows as $row) {
				if (isset($keyedArray[$row[$keyField]]))
					$keyedArray[$row[$keyField]][] = $row[$valueField];
				else
					$keyedArray[$row[$keyField]] = array($row[$valueField]);
			}
			return $keyedArray;
		}
		
		protected function importFileElement(SimpleXMLElement $xliffFileElement)
		{
			$targetLanguageCode = (string)$xliffFileElement['target-language'];
			$targetLanguageId = $this->getLanguageIdFromCode($targetLanguageCode);
			
			$currentItems = $this->getAllCurrentItems();
			$this->textsToBeTranslatedCount[$targetLanguageCode] = count($currentItems);
			
			$this->deleteOldItems($targetLanguageId);
			
			$valueLists = array();
			foreach ($xliffFileElement->body->{'trans-unit'} as $transUnit) {
				
				$attributes = $transUnit->attributes();
				$isApproved = ($attributes['approved'] == 'yes');
				
				if (!isset($transUnit->target))
					continue;
				
				$textOrig = (string)$transUnit->source;
				$textTrans = (string)$transUnit->target;
				
				if (!isset($currentItems[$textOrig])) {
					$this->unusedTranslations[$targetLanguageCode][] = $textOrig;
					continue;
				}
				
				foreach ($currentItems[$textOrig] as $currentItem)
					$valueLists[] = array($currentItem, $targetLanguageId, $textTrans);
				unset($currentItems[$textOrig]);	// This will mark duplicate translations as unused items
			}
			
			$this->insertItems($valueLists);
			
			$this->translatedCount[$targetLanguageCode] = count($valueLists);
			
			if (!empty($currentItems)) {
				ksort($currentItems);
				$this->missingTranslations[$targetLanguageCode] = $currentItems;
			}
		}
		
		public function getXliffBasename()
		{
			return $this->xliffBasename;
		}
		
		/**
		 * Returns array with unused translations
		 * @return array
		 */
		public function getUnusedTranslations()
		{
			return $this->unusedTranslations;
		}
		
		/**
		 * Returns array with missing translations
		 * @return array
		 */
		public function getMissingTranslations()
		{
			return $this->missingTranslations;
		}
		
		/**
		 * Return the translated count
		 * @return array array(TargetLanguageCode => Count)
		 */
		public function getTranslatedCount()
		{
			return $this->translatedCount;
		}
		
		/**
		 * Return the number of texts to be translated
		 * @return array array(TargetLanguageCode => Count)
		 */
		public function getTextsToBeTranslatedCount()
		{
			return $this->textsToBeTranslatedCount;
		}
		
		/**
		 * Return the language id for English language
		 * @return int
		 */
		protected static function getEnglishLanguageId()
		{
			static $langId = null;
			
			if ($langId === null)
				$langId = scUtil::getDbValue('Languages', 'LanguageID', 'LanguageCode', 'EN');
			
			return $langId;
		}
		
		abstract protected function getAllCurrentItems();
	}
	
	/**
	 * Import language texts
	 */
	class scXliffImportUserInterfaceTexts extends scXliffImport {
		protected $xliffBasename = 'user_interface.xlf';
		
		protected $_textsOriginalCache = null;
		
		protected function getAllCurrentItems()
		{
			if ($this->_textsOriginalCache === null) {
				$sql = 'SELECT TextOriginal,TextID FROM Texts';
				$this->_textsOriginalCache = miUtil::getDBArray($sql, 'TextOriginal', 'TextID');
			}
			return $this->_textsOriginalCache;
		}
		
		/**
		 * Import XLIFF file element
		 * 
		 * @param SimpleXmlElement $xliffFileElement
		 */
		protected function importFileElement(SimpleXMLElement $xliffFileElement)
		{
			$targetLanguageCode = (string)$xliffFileElement['target-language'];
			$targetLanguageId = $this->getLanguageIdFromCode($targetLanguageCode);
			
			$textsOriginal = $this->getAllCurrentItems();
			$this->textsToBeTranslatedCount[$targetLanguageCode] = count($textsOriginal);
			
			// Remove old translation
			miStaticDBUtil::execSQL('DELETE FROM TextsTranslated WHERE LanguageID=' . $targetLanguageId);
			
			$valueLists = array();
			foreach ($xliffFileElement->body->{'trans-unit'} as $transUnit) {
				
				$attributes = $transUnit->attributes();
				$isApproved = ($attributes['approved'] == 'yes');
				
				if (!isset($transUnit->target))
					continue;
				
				$textOrig = (string)$transUnit->source;
				$textTrans = (string)$transUnit->target;
				
				if (!isset($textsOriginal[$textOrig])) {
					$this->unusedTranslations[$targetLanguageCode][] = $textOrig;
					continue;
				}
				
				$valueLists[] = array($textsOriginal[$textOrig], $targetLanguageId, sprintf('%u', crc32($textOrig)), $textTrans, $isApproved ? 1 : 0);
				unset($textsOriginal[$textOrig]);	// This will mark duplicate translations as unused items
			}
			
			miStaticDBUtil::multiValueListsInsert('TextsTranslated', array('TextID', 'LanguageID', 'TextOriginalHash', 'TextTranslated', 'TextIsTranslated'), $valueLists);
			
			$this->translatedCount[$targetLanguageCode] = count($valueLists);
			
			if (!empty($textsOriginal)) {
				ksort($textsOriginal);
				$this->missingTranslations[$targetLanguageCode] = $textsOriginal;
			}
		}
		
		public static function setupEnglishTranslations()
		{
			miStaticDBUtil::execSQL('DELETE FROM TextsTranslated WHERE LanguageID=' . self::getEnglishLanguageId());
			$sql = 'INSERT INTO TextsTranslated(TextID,LanguageID,TextOriginalHash,TextTranslated,TextIsTranslated) ' .
				'SELECT TextID,' . self::getEnglishLanguageId() . ',TextOriginalHash,TextOriginal,1 FROM Texts';
			miStaticDBUtil::execSQL($sql);
		}
	}
	
	/**
	 * Import currencies
	 */
	class scXliffImportCurrencies extends scXliffImport {
		protected $xliffBasename = 'currencies.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,Currencies.CurrencyID,CurrencyName FROM Currencies JOIN CurrenciesML USING(CurrencyID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'CurrencyName', 'CurrencyID');
		}
		
		protected function deleteOldItems($languageId)
		{
//			$sql = 'DELETE FROM CurrenciesML WHERE LanguageID=?';
//			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('CurrenciesML', array('CurrencyID', 'LanguageID', 'CurrencyName'), $valueLists, 'CurrencyName');
		}
	}
	
	/**
	 * Import helps
	 */
	class scXliffImportHelps extends scXliffImport {
		protected $xliffBasename = 'helps.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,Helps.HelpID,HelpText FROM Helps JOIN HelpsML USING(HelpID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'HelpText', 'HelpID');
		}
		
		protected function deleteOldItems($languageId)
		{
			$sql = 'DELETE FROM HelpsML WHERE LanguageID=?';
			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('HelpsML', array('HelpID', 'LanguageID', 'HelpText'), $valueLists);
		}
	}
	
	/**
	 * Import admin quick links
	 */
	class scXliffImportQuickLinks extends scXliffImport {
		protected $xliffBasename = 'admin_quick_links.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,AdminQuickLinks.AdminQuickLinkID,AdminQuickLinkText FROM AdminQuickLinks JOIN AdminQuickLinksML USING(AdminQuickLinkID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'AdminQuickLinkText', 'AdminQuickLinkID');
		}
		
		protected function deleteOldItems($languageId)
		{
			$sql = 'DELETE FROM AdminQuickLinksML WHERE LanguageID=?';
			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('AdminQuickLinksML', array('AdminQuickLinkID', 'LanguageID', 'AdminQuickLinkText'), $valueLists);
		}
	}
	
	/**
	 * Import countries
	 */
	class scXliffImportCountries extends scXliffImport {
		protected $xliffBasename = 'countries.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,Countries.CountryID,CountryName FROM Countries JOIN CountriesML USING(CountryID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'CountryName', 'CountryID');
		}
		
		protected function deleteOldItems($languageId)
		{
			$sql = 'DELETE FROM CountriesML WHERE LanguageID=?';
			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('CountriesML', array('CountryID', 'LanguageID', 'CountryName'), $valueLists);
		}
	}
	
	/**
	 * Import states
	 */
	class scXliffImportStates extends scXliffImport {
		protected $xliffBasename = 'states.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,States.StateID,StateName FROM States JOIN StatesML USING(StateID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'StateName', 'StateID');
		}
		
		protected function deleteOldItems($languageId)
		{
			$sql = 'DELETE FROM StatesML WHERE LanguageID=?';
			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('StatesML', array('StateID', 'LanguageID', 'StateName'), $valueLists);
		}
	}
	
	/**
	 * Import order statuses
	 */
	class scXliffImportOrderStatuses extends scXliffImport {
		protected $xliffBasename = 'order_statuses.xlf';
		
		protected function getAllCurrentItems()
		{
			$sql = 'SELECT LanguageID,OrderStatuses.OrderStatusID,OrderStatusName FROM OrderStatuses JOIN OrderStatusesML USING(OrderStatusID) WHERE LanguageID=' . self::getEnglishLanguageId();
			return $this->getItemsFromDb($sql, 'OrderStatusName', 'OrderStatusID');
		}
		
		protected function deleteOldItems($languageId)
		{
			$sql = 'DELETE FROM OrderStatusesML WHERE LanguageID=?';
			miStaticDBUtil::execSql($sql, array($languageId));
		}
		
		protected function insertItems($valueLists)
		{
			miStaticDBUtil::multiValueListsInsert('OrderStatusesML', array('OrderStatusID', 'LanguageID', 'OrderStatusName'), $valueLists);
		}
	}
?>