<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scModuleOptions {
		
		const MODULE_OPTION_PREFIX = 'ModuleOption';
		
		const MODULE_OPTIONS_NO_ML = null;
		const MODULE_OPTIONS_INCLUDE_ML = 1;
		const MODULE_OPTIONS_INCLUDE_FULL_ML = 2;
		
		static protected $_onUpdateCallback = array();
		
		/**
		 * Returns the module options for a module
		 * 
		 * @param int $moduleId
		 * @param array $requiredOptions optional array($requiredOption1, $requiredOption2, ...)
		 * @param int $multiLanguage Defines whether ML options should be included or not and if included, in what format.
		 * @return array a module option name/value pairs
		 * @throws scPaymentProcessException if a required option cannot be found
		 */
		public static function getModuleOptions($moduleId, $requiredOptions = array(), $multiLanguage = self::MODULE_OPTIONS_NO_ML)
		{
			$helper = new scModuleOptionsHelper();
			$moduleOptions = $helper->getOptions($moduleId, $multiLanguage);
			
			foreach ($requiredOptions as $optionName)
				if (!isset($moduleOptions[$optionName]))
					throw new scPaymentProcessException(sprintf(_t('Module option "%s" is not set. Please setup "%s" module.'), $optionName, scModuleUtil::getModuleDisplayName($moduleId)));
			
			return $moduleOptions;
		}
		
		/**
		 * Returns the cms box options
		 * 
		 * @param int $cmsBoxId
		 * @param int $multiLanguage {@see scModuleOptions::getModuleOptions()}
		 * @return array a module option name/value pairs
		 */
		public static function getCmsBoxOptions($cmsBoxId, $multiLanguage = self::MODULE_OPTIONS_NO_ML)
		{
			if ($multiLanguage != self::MODULE_OPTIONS_NO_ML) {
				$helper = new scCmsBoxOptionsHelper();
				return $helper->getOptions($cmsBoxId, $multiLanguage);
			}
			
			// Optimization for non-multilanguage cms box options
			static $cachedcmsBoxOptions = null;
			
			if ($cachedcmsBoxOptions === null) {
				$cachedcmsBoxOptions = array();
				$sql = 'SELECT * FROM CMSBoxOptions';
				$rows = miStaticDBUtil::execSelect($sql);
				foreach ($rows as $row) {
					$cachedcmsBoxOptions[$row['CMSBoxID']][$row['CMSBoxOptionName']] = $row['CMSBoxOptionValue'];
				}
			}
			
			if (isset($cachedcmsBoxOptions[$cmsBoxId]))
				return $cachedcmsBoxOptions[$cmsBoxId];
			
			return array();
		}
		
		/**
		 * Return the module options for a shipping  module
		 * 
		 * @param int $shippingModuleId
		 * @return array the module options
		 */
		public static function getShippingModuleOptions($shippingModuleId)
		{
			$helper = new scShippingModuleOptionsHelper();
			return $helper->getOptions($shippingModuleId);
		}
		
		/**
		 * Creates a module option
		 * 
		 * @param int $moduleId
		 * @param string $moduleOptionName
		 * @param string|array $moduleOptionValue string standard options and array for MLs
		 */
		public static function createModuleOption($moduleId, $moduleOptionName, $moduleOptionValue)
		{
			$helper = new scModuleOptionsHelper();
			$helper->createOption($moduleId, $moduleOptionName, $moduleOptionValue);
		}
		
		/**
		 * Creates a module option
		 * 
		 * @param int $cmsBoxId
		 * @param string $optionName
		 * @param string|array $optionValue string standard options and array for MLs
		 */
		public static function createCmsBoxOption($cmsBoxId, $optionName, $optionValue)
		{
			$helper = new scCmsBoxOptionsHelper();
			$helper->createOption($cmsBoxId, $optionName, $optionValue);
		}
		
		/**
		 * Creates a shipping module option
		 * 
		 * @param int $shippingModuleId
		 * @param string $optionName
		 * @param string $optionValue
		 */
		public static function createShippingModuleOption($shippingModuleId, $optionName, $optionValue)
		{
			$helper = new scShippingModuleOptionsHelper();
			$helper->createOption($shippingModuleId, $optionName, $optionValue);
		}
		
		/**
		 * Sets a module option
		 * 
		 * @param int $moduleId
		 * @param string $moduleOptionName
		 * @param string|array $moduleOptionValue
		 */
		public static function setModuleOption($moduleId, $moduleOptionName, $moduleOptionValue)
		{
			$helper = new scModuleOptionsHelper();
			$helper->setOption($moduleId, $moduleOptionName, $moduleOptionValue);
		}
		
		/**
		 * Sets a cms box option
		 * 
		 * @param int $cmsBoxId
		 * @param string $optionName
		 * @param string|array $optionValue
		 */
		public static function setCmsBoxOption($cmsBoxId, $optionName, $optionValue)
		{
			$helper = new scCmsBoxOptionsHelper();
			$helper->setOption($cmsBoxId, $optionName, $optionValue);
		}
		
		/**
		 * Sets a shipping module option
		 * 
		 * @param int $shippingModuleId
		 * @param string $optionName
		 * @param string $optionValue
		 */
		public static function setShippingModuleOption($shippingModuleId, $optionName, $optionValue)
		{
			$helper = new scShippingModuleOptionsHelper();
			$helper->setOption($shippingModuleId, $optionName, $optionValue);
		}
		
		/**
		 * Retrieves a module option
		 * 
		 * @param int $moduleId
		 * @param string $moduleOptionName
		 * @return string
		 */
		public static function getModuleOption($moduleId, $moduleOptionName)
		{
			$sql = 'SELECT ModuleOptionValue FROM ModuleOptions WHERE ModuleID=? AND ModuleOptionName=?';
			$rows = miStaticDBUtil::execSelect($sql, array($moduleId, $moduleOptionName));
			if (count($rows) == 0)
				return false;
			
			return $rows[0]['ModuleOptionValue'];
		}
		
		public static function registerOnUpdateCallback($moduleId, $callback)
		{
			self::$_onUpdateCallback[$moduleId] = $callback;
		}
		
		public static function callOnUpdateCallbacks($moduleId, $optionValues)
		{
			if (isset(self::$_onUpdateCallback[$moduleId]))
				call_user_func(self::$_onUpdateCallback[$moduleId], $optionValues);
		}
	}
	
	abstract class scOptionsHelper {
		
		protected $_table;
		protected $_searchKey;
		protected $_nameKey;
		protected $_valueKey;
		
		public function createOption($searchValue, $optionName, $optionValue)
		{
			$this->createSLOption($searchValue, $optionName, $optionValue);
		}
		
		public function getOptions($searchValue)
		{
			return $this->getSLOptions($searchValue);
		}
		
		public function setOption($searchValue, $optionName, $optionValue)
		{
			// Start transaction
			miStaticDBUtil::startTransaction();
			
			// Delete the old options
			$this->deleteOption($searchValue, $optionName);
			
			$this->createOption($searchValue, $optionName, $optionValue);
			
			// Commit
			miStaticDBUtil::commit();
		}
		
		protected function createSLOption($searchValue, $optionName, $optionValue)
		{
			$values = array(
				$this->_searchKey => $searchValue,
				$this->_nameKey => $optionName,
				$this->_valueKey => $optionValue,
			);
			miStaticDBUtil::execInsert($this->_table, $values);
		}
		
		protected function getSLOptions($searchValue)
		{
			$moduleOptionsRs = new miSqlRecordset($this->_table);
			$moduleOptionsRs->addFilter(new miSqlFilterEqual($this->_searchKey, $searchValue));
			$moduleOptions = $moduleOptionsRs->getAllRecords();
			return miUtil::getKeyValueArray($moduleOptions, $this->_nameKey, $this->_valueKey);
		}
		
		protected function deleteOption($searchValue, $optionName)
		{
			miStaticDBUtil::execDelete($this->_table, array($this->_searchKey, $this->_nameKey), array($searchValue, $optionName));
		}
	}
	
	abstract class scMLOptionsHelper extends scOptionsHelper {
		
		protected $_mlTable;
		
		public function createOption($searchValue, $optionName, $optionValue)
		{
			if (is_array($optionValue)) {
				// Multi-language option
				$this->createMLOption($searchValue, $optionName, $optionValue);
			} else {
				// Non multi-language option
				parent::createOption($searchValue, $optionName, $optionValue);
			}
		}
		
		public function getOptions($searchValue, $multiLanguage = scModuleOptions::MODULE_OPTIONS_NO_ML)
		{
			$options = parent::getOptions($searchValue);
			
			switch ($multiLanguage) {
				case scModuleOptions::MODULE_OPTIONS_INCLUDE_FULL_ML:
					$options = array_merge($options, $this->getMLOptions($searchValue));
					break;
				case scModuleOptions::MODULE_OPTIONS_INCLUDE_ML:
					$options = array_merge($options, $this->getMLOptionsForLanguage($searchValue, scMultiLanguage::getCurrentLanguageId()));
					break;
			}
			
			return $options;
		}
		
		protected function createMLOption($searchValue, $optionName, $optionValue)
		{
			foreach ($optionValue as $languageId => $mlValue) {
				$values = array(
					'LanguageID' => $languageId,
					$this->_searchKey => $searchValue,
					$this->_nameKey => $optionName,
					$this->_valueKey => $mlValue,
				);
				miStaticDBUtil::execInsert($this->_mlTable, $values);
			}
		}
		
		protected function getMLOptionsForLanguage($searchValue, $languageId)
		{
			$moduleOptionsRs = new miSqlRecordset($this->_mlTable);
			$moduleOptionsRs->addFilter(new miSqlFilterEqual($this->_searchKey, $searchValue));
			$moduleOptionsRs->addFilter(new miSqlFilterEqual('LanguageID', $languageId));
			$moduleOptions = $moduleOptionsRs->getAllRecords();
			return miUtil::getKeyValueArray($moduleOptions, $this->_nameKey, $this->_valueKey);
		}
		
		protected function getMLOptions($searchValue)
		{
			$moduleOptionsRs = new miSqlRecordset($this->_mlTable);
			$moduleOptionsRs->addFilter(new miSqlFilterEqual($this->_searchKey, $searchValue));
			$moduleOptions = $moduleOptionsRs->getAllRecords();
			
			$moduleOptionsML = array();
			foreach ($moduleOptions as $row) {
				if (!isset($moduleOptionsML[$row[$this->_nameKey]])) {
					$moduleOptionsML[$row[$this->_nameKey]] = array();
				}
				
				$moduleOptionsML[$row[$this->_nameKey]][$row['LanguageID']] = $row[$this->_valueKey];
			}
			return $moduleOptionsML;
		}
		
		protected function deleteOption($searchValue, $optionName)
		{
			miStaticDBUtil::execDelete($this->_table, array($this->_searchKey, $this->_nameKey), array($searchValue, $optionName));
			miStaticDBUtil::execDelete($this->_mlTable, array($this->_searchKey, $this->_nameKey), array($searchValue, $optionName));
		}
	}
	
	class scModuleOptionsHelper extends scMLOptionsHelper {
		
		protected $_table = 'ModuleOptions';
		protected $_mlTable = 'ModuleOptionsML';
		protected $_searchKey = 'ModuleID';
		protected $_nameKey = 'ModuleOptionName';
		protected $_valueKey = 'ModuleOptionValue';
	}
	
	class scCmsBoxOptionsHelper extends scMLOptionsHelper {
		
		protected $_table = 'CMSBoxOptions';
		protected $_mlTable = 'CMSBoxOptionsML';
		protected $_searchKey = 'CMSBoxID';
		protected $_nameKey = 'CMSBoxOptionName';
		protected $_valueKey = 'CMSBoxOptionValue';
	}
	
	class scShippingModuleOptionsHelper extends scOptionsHelper {
		
		protected $_table = 'ShippingModuleOptions';
		protected $_searchKey = 'ShippingModuleID';
		protected $_nameKey = 'ShippingModuleOptionName';
		protected $_valueKey = 'ShippingModuleOptionValue';
	}
?>