<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scModuleOptionsInfo {
		protected $_optionFields;
		protected $_templateFile;
		protected $_validators;
		protected $_modelStrategies = array();
		
		/**
		 * Constructs the module options info object
		 *
		 * @param array $optionFields array of scFieldSet and miWidget objects
		 * @param array $validators array of miValidator objects
		 */
		public function __construct($optionFields = array(), $validators = array())
		{
			$this->_optionFields = $optionFields;
			$this->_validators = $validators;
		}
		
		/**
		 * Builds the option fields based on fields description
		 *
		 * @param array $fieldsDescriptions
		 */
		public function buildOptionFields($fieldsDescriptions, miWebForm &$webForm)
		{
			foreach ($fieldsDescriptions as $fieldDescription) {
				if (isset($fieldDescription['fieldset'])) {
					$name = isset($fieldDescription['fieldset']['name']) ? $fieldDescription['fieldset']['name'] : '';
					$description = isset($fieldDescription['fieldset']['description']) ? $fieldDescription['fieldset']['description'] : '';
					$fieldset = new scFieldSet($name, $description);
					$this->buildFieldsetOptions($fieldset, $fieldDescription['fields'], $webForm);
					$this->_optionFields[] = $fieldset;
					continue;
				}
				$this->_optionFields[] = $this->buildOptionField($fieldDescription, $webForm);
			}
		}
		
		protected function buildOptionField($fieldDescription, miWebForm &$webForm)
		{
			$properties = isset($fieldDescription['properties']) ? $fieldDescription['properties'] : array();
			$field = new $fieldDescription['field']($webForm, $properties);
			$field->setFieldName($fieldDescription['data']);
			
			if (isset($fieldDescription['validator'])) {
				if (is_array($fieldDescription['validator'])) {
					foreach ($fieldDescription['validator'] as $validator)
						$this->addValidator(new $validator($webForm, $fieldDescription['data']));
				} else
					$this->addValidator(new $fieldDescription['validator']($webForm, $fieldDescription['data']));
			}
			
			return $field;
		}
		
		protected function buildFieldsetOptions(&$fieldset, $fieldsDescriptions, miWebForm &$webForm)
		{
			foreach ($fieldsDescriptions as $fieldDescription) {
				if (isset($fieldDescription['fieldset'])) {
					$name = isset($fieldDescription['fieldset']['name']) ? $fieldDescription['fieldset']['name'] : '';
					$description = isset($fieldDescription['fieldset']['description']) ? $fieldDescription['fieldset']['description'] : '';
					$fieldset = new scFieldSet($name, $description);
					$this->buildFieldsetOptions($fieldset, $fieldDescription['fields'], $webForm);
					$fieldset->addField($fieldset);
					continue;
				}
				$fieldset->addField($this->buildOptionField($fieldDescription, $webForm));
			}
		}
		
		/**
		 * Adds option field
		 *
		 * @param scFieldSet|miWidget $$optionField
		 */
		public function addOptionField($optionField)
		{
			$this->_optionFields[] = $optionField;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @return array scFieldSet and miWidget objects
		 */
		public function getOptionFields()
		{
			return $this->_optionFields;
		}
		
		/**
		 * Retrieves the option field with name $name
		 *
		 * @param string $name
		 * @return miWidget the field object
		 */
		public function getOptionField($name)
		{
			$result = $this->getOptionFieldRecursive($this->_optionFields, $name);
			if ($result != null)
				return $result;
			throw new miException(sprintf(_t('Unknown option field %s.'), $name));
		}
		
		protected function getOptionFieldRecursive($optionFields, $name)
		{
			foreach ($optionFields as $optionField) {
				if ($optionField instanceof scFieldSet) {
					$result = $this->getOptionFieldRecursive($optionField->getFields(), $name);
					if ($result != null)
						return $result;
					continue;
				}
				if (($optionField instanceof miBaseWidget) and ($optionField->getFieldName() == $name))
					return $optionField;
			}
			return null;
		}
		
		/**
		 * Set the template file
		 *
		 * @param string $templateFile
		 */
		public function setTemplateFile($templateFile)
		{
			$this->_templateFile = $templateFile;
		}
		
		/**
		 * Get the template file
		 *
		 * @return string
		 */
		public function getTemplateFile()
		{
			return $this->_templateFile;
		}
		
		/**
		 * Adds validator
		 *
		 * @param miValidator $validator
		 */
		public function addValidator(miValidator $validator)
		{
			$this->_validators[] = $validator;
		}
		
		/**
		 * Returns the validators
		 *
		 * @return array miValidator objects
		 */
		public function getValidators()
		{
			return $this->_validators;
		}
		
		/**
		 * Set the model strategies
		 * The keys are the field names, the values are the strategy class names
		 *
		 * @param array $modelStrategies
		 */
		public function setModelStrategies($modelStrategies)
		{
			$this->_modelStrategies = $modelStrategies;
		}
		
		/**
		 * Get the module options model strategy
		 * 
		 * @return miViewMapper
		 */
		public function getModelStrategies()
		{
			return $this->_modelStrategies;
		}
	}
?>