<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrderService {
		
		/**
		 * @return scOrderService
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scOrderService();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		/**
		 * Saves the Order Total Lines.
		 * Expects an array of scOrderTotalLine objects.
		 * @param array $orderTotalLines
		 * @param int $orderId
		 */
		public function saveOrderTotalLines($orderTotalLines, $orderId)
		{
			$orderId = (int)$orderId;
			if ($orderId <= 0) {
				throw new miException('Invalid order.');
			}
			
			$sort = 0;
			foreach ($orderTotalLines as $orderTotalLine) {
				if (!($orderTotalLine instanceof scOrderTotalLine)) {
					throw new miException('Invalid order total line.');
				}
				
				$domainObject = new scOrderTotalLineDomainObject();
				$domainObject->set('OrderID', $orderId);
				$domainObject->set('OrderTotalLineSort', $sort++);
				$this->setOrderTotalLineData($orderTotalLine, $domainObject);
				$domainObject->insert();
			}
		}
		
		/**
		 * Inserts new order total line
		 * 
		 * @param int $orderId the order id
		 * @param string $amount the amount
		 * @param string $name the name of the order total line
		 * @param int $orderTotalType a IOrderTotalModule::ORDER_TOTAL_TYPE_* constant
		 */
		public function insertOrderTotalLine($orderId, $amount, $name, $orderTotalType)
		{
			$sql = 'SELECT MAX(OrderTotalLineSort) AS Sort FROM OrderTotalLines WHERE OrderID=?';
			$result = miStaticDBUtil::execSelect($sql, array($orderId));
			$sort = $result[0]['Sort'];
			
			$values = array(
					'OrderID' => $orderId,
					'OrderTotalLineType' => $orderTotalType,
					'OrderTotalLineName' => $name,
					'OrderTotalLineAmount' => $amount,
					'OrderTotalLineSort' => $sort
				);
			miStaticDBUtil::execInsert('OrderTotalLines', $values);
		}
		
		public function deleteOrderTotalLinesByType($orderId, $orderTotalLineType)
		{
			$sql = 'DELETE FROM OrderTotalLines WHERE OrderID=? AND OrderTotalLineType=?';
			miStaticDBUtil::execSQL($sql, array($orderId, $orderTotalLineType));
		}
		
		/**
		 * Updates the order total from OrderTotalLines
		 */
		public function updateOrderTotal($orderId)
		{
			$sql = ' SELECT OrderID, SUM(OrderTotalLineAmount) AS OrderTotal, SUM(OrderTotalLineCustomerCurrencyAmount) AS OrderCustomerCurrencyTotal' .
					' FROM OrderTotalLines WHERE OrderID = ? GROUP BY OrderID';
			$total = miStaticDBUtil::execSelect($sql, array('OrderID' => $orderId));
			$total = $total[0];
			
			$values = array('OrderTotal' => $total['OrderTotal'], 'OrderCustomerCurrencyTotal' => $total['OrderCustomerCurrencyTotal']);
			miStaticDBUtil::execUpdate('Orders', $values, 'OrderID', $orderId);
			scOperationsLog::storeOperation(scOperationsLog::ITEM_TYPE_ORDER, $orderId, scOperationsLog::OPERATION_UPDATE);
		}
		
		/**
		 * Find the shipping module and method code by shipping method name
		 * Returns the shipping module id and the shipping method code
		 * 
		 * @param string $shippingMethodName the shipping method name
		 * @return array array($shippingModuleId,$shippingMethodCode)
		 */
		public function getShippingModuleIdAndMethodCode($shippingMethodName, $countryId = 0, $stateId = 0, $digitalDelivery = 'any')
		{
			if ($countryId or $stateId or ($digitalDelivery != 'any'))
				$shippingModules = scModulesFactory::getActiveShippingModulesForZone($countryId, $stateId, $digitalDelivery);
			else
				$shippingModules = scModulesFactory::getActiveShippingModules();
			
			foreach ($shippingModules as $shippingModule) {
				$shippingModuleId = $shippingModule['ShippingModuleID'];
				$module = $shippingModule['object'];
				
				$shippingMethods = $module->getShippingMethods();
				
				foreach ($shippingMethods as $shippingMethodCode => $shippingMethod) {
					if (!isset($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]))
						continue;
					
					if ($shippingModule['ShippingModuleActiveMethods'][$shippingMethodCode]['ShippingMethodDisplayName'] == $shippingMethodName)
						return array($shippingModuleId, $shippingMethodCode);
				}
			}
			
			// Not found
			return array(0, '');
		}
		
		/**
		 * @var scOrder $order
		 * @var scCartItem[] $orderItems
		 */
		public function saveOrderItems(scOrder $order, $orderItems, $parentOrderItemId = null)
		{
			$currentCurrency = scCurrencyUtil::getCurrentCurrency();
			foreach ($orderItems as $orderItem) {
				if (!($orderItem instanceof scCartItemProduct)) {
					continue;
				}
				
				$info = $orderItem->getProductInfo();
				
				$image = $orderItem->getImageInfo();
				
				$defaultLanguageId = scMultiLanguage::getDefaultLanguageId();
				$product = new scProductSL();
				$product->setLanguageId($defaultLanguageId);
				$product->read($info['ProductID']);
				
				$category = new scCategorySL();
				$category->setLanguageId($defaultLanguageId);
				$category->read($product->get('CategoryID'));
				
				$basePriceAmount = $orderItem->getBasePrice(true)->getAmount();
				$finalPriceAmount = $orderItem->getPriceObj(true)->getAmount();
				$discountAmount = scMultiCurrencyAmountMath::subtract($basePriceAmount, $finalPriceAmount);
				$totalAmount = $orderItem->getTotal();
				
				$orderItemDetails = array(
					'OrderID' => $order->getPK(), 
					'ParentOrderItemID' => $parentOrderItemId, 
					'ProductID' => $info['ProductID'],
					'ProductVariantID' => (($orderItem instanceof ICartItemProductVariant) ? $orderItem->getProductVariant()->get('ProductVariantID') : null),
					'ProductImageID' => $image->getProductImageId(),
					'OrderItemProductName' => $product->get('ProductName'),
					'OrderItemCategoryName' => $category->get('CategoryName'),
					'OrderItemProductCode' => $orderItem->getProductCode(),
					'OrderItemQty' => $orderItem->getQuantity(),
					'OrderItemDiscount' => $discountAmount->get(),
					'OrderItemPrice' => $finalPriceAmount->get(),
					'OrderItemTotal' => $totalAmount->get(),
					'OrderItemCustomerCurrencyDiscount' => $discountAmount->get($currentCurrency),
					'OrderItemCustomerCurrencyPrice' => $finalPriceAmount->get($currentCurrency),
					'OrderItemCustomerCurrencyTotal' => $totalAmount->get($currentCurrency),
					'OrderItemBonusPointsSpent' => $orderItem->getOrderItemBonusPoints()
				);
				$orderItemId = miStaticDBUtil::execInsert('OrderItems', $orderItemDetails);
				
				// Insert options
				$this->saveOrderItemOptions($orderItem, $orderItemId);
				
				// Insert local options
				$this->saveOrderItemLocalOptions($orderItem, $orderItemId);
				
				if (!$orderItem->isTangible()) {
					// The item is intangible. Store E-Goods.
					$this->saveOrderItemEgoods($orderItem, $orderItemId);
				}
				
				if ($orderItem instanceof ICartItemProductBundle) {
					$this->saveOrderItems($order, $orderItem->getBundledItems(), $orderItemId);
				}
			}
		}
		
		/**
		 * Saves the product options
		 * 
		 * @param scCartItemProduct $orderItem
		 * @param int $orderItemId
		 */
		public function saveOrderItemOptions(scCartItemProduct $orderItem, $orderItemId)
		{
			$orderItemId = (int)$orderItemId;
			if ($orderItemId <= 0) {
				throw new miException('Invalid order item.');
 			}
 			
			$options = $orderItem->getOptions();
			$price = $orderItem->getPriceObj(false)->getAmount();
			foreach ($options as $option) {
				$optionInfo = $option->getOptionInfo($price);
				
				$sql = 'SELECT OptionGroupName, OptionGroupType FROM OptionGroups LEFT JOIN OptionGroupsML USING(OptionGroupID) WHERE OptionGroups.OptionGroupID=? AND LanguageID=?';
				$optionGroupRows = miStaticDBUtil::execSelect($sql, array($optionInfo['OptionGroupID'], scMultiLanguage::getCurrentLanguageId()));
				if (empty($optionGroupRows)) {
					continue;
				}
				
				$optionGroupName = $optionGroupRows[0]['OptionGroupName'];
				$optionGroupType = $optionGroupRows[0]['OptionGroupType'];
				
				$sql = 'SELECT OptionName FROM Options LEFT JOIN OptionsML USING(OptionID) WHERE Options.OptionID=? AND LanguageID=?';
				$optionRows = miStaticDBUtil::execSelect($sql, array($optionInfo['OptionID'], scMultiLanguage::getCurrentLanguageId()));
				if (empty($optionRows)) {
					continue;
				}
				
				$optionName = $optionRows[0]['OptionName'];
				
				$domainObject = new scOrderItemOption();
				$domainObject->set('OrderItemID', $orderItemId);
				$domainObject->set('OrderItemOptionGroupName', $optionGroupName);
				$domainObject->set('OrderItemOptionGroupType', $optionGroupType);
				$domainObject->set('OrderItemOptionName', $optionName);
				$domainObject->set('OrderItemOptionPriceModType', $optionInfo['OptionPriceModType']);
				$domainObject->set('OrderItemOptionPriceModValue', $optionInfo['OptionPriceModValue']);
				$priceChange = $option->getOptionPriceChange($price);
				$domainObject->set('OrderItemOptionPrice', $priceChange->get());
				$domainObject->set('OrderItemOptionCustomerCurrencyPrice', $priceChange->get(scCurrencyUtil::getCurrentCurrency()));
				
				$domainObject->set('OrderItemOptionUserText', '');
				if ($optionGroupType == scProductOptions::GROUP_TYPE_USER_TEXT) {
					$domainObject->set('OrderItemOptionUserText', $optionInfo['OptionUserText']);
				}

				$domainObject->insert(); // the id is needed for the uploaded image
				
				if ($optionGroupType == scProductOptions::GROUP_TYPE_USER_IMAGE) {
					$this->setOptionImageData($option, $domainObject);
					$domainObject->update();
				}
			}
		}
		
		/**
		 * Saves the local options
		 * 
		 * @param scCartItemProduct $orderItem
		 * @param int $orderItemId
		 */
		public function saveOrderItemLocalOptions(scCartItemProduct $orderItem, $orderItemId)
		{
			$orderItemId = (int)$orderItemId;
			if ($orderItemId <= 0) {
				throw new miException('Invalid order item.');
 			}
 			
			$options = $orderItem->getLocalOptions();
			$price = $orderItem->getPriceObj(false)->getAmount();
			foreach ($options as $option) {
				$optionInfo = $option->getOptionInfo($price);
				
				$sql = 'SELECT LocalOptionGroupName, LocalOptionGroupType FROM LocalOptionGroups LEFT JOIN LocalOptionGroupsML USING(LocalOptionGroupID) WHERE LocalOptionGroups.LocalOptionGroupID=? AND LanguageID=?';
				$optionGroupRows = miStaticDBUtil::execSelect($sql, array($optionInfo['OptionGroupID'], scMultiLanguage::getCurrentLanguageId()));
				if (empty($optionGroupRows))
					continue;
				
				$optionGroupName = $optionGroupRows[0]['LocalOptionGroupName'];
				$optionGroupType = $optionGroupRows[0]['LocalOptionGroupType'];
				
				$sql = 'SELECT LocalOptionName FROM LocalOptions LEFT JOIN LocalOptionsML USING(LocalOptionID) WHERE LocalOptions.LocalOptionID=? AND LanguageID=?';
				$optionRows = miStaticDBUtil::execSelect($sql, array($optionInfo['OptionID'], scMultiLanguage::getCurrentLanguageId()));
				if (empty($optionRows))
					continue;
				$optionName = $optionRows[0]['LocalOptionName'];
				
				$domainObject = new scOrderItemLocalOption();
				$domainObject->set('OrderItemID', $orderItemId);
				$domainObject->set('OrderItemLocalOptionGroupName', $optionGroupName);
				$domainObject->set('OrderItemLocalOptionGroupType', $optionGroupType);
				$domainObject->set('OrderItemLocalOptionName', $optionName);
				$domainObject->set('OrderItemLocalOptionPriceModType', $optionInfo['OptionPriceModType']);
				$domainObject->set('OrderItemLocalOptionPriceModValue', $optionInfo['OptionPriceModValue']);
				$priceChange = $option->getOptionPriceChange($price);
				$domainObject->set('OrderItemLocalOptionPrice', $priceChange->get());
				$domainObject->set('OrderItemLocalOptionCustomerCurrencyPrice', $priceChange->get(scCurrencyUtil::getCurrentCurrency()));
				
				$domainObject->set('OrderItemLocalOptionUserText', '');
				if ($optionGroupType == scProductOptions::GROUP_TYPE_USER_TEXT) {
					$domainObject->set('OrderItemLocalOptionUserText', $optionInfo['OptionUserText']);
				}
				
				$domainObject->insert(); // the id is needed for the uploaded image
				
				if ($optionGroupType == scProductOptions::GROUP_TYPE_USER_IMAGE) {
					$this->setLocalOptionImageData($option, $domainObject);
					$domainObject->update();
				}
			}
		}
		
		/**
		 * Saves the e-goods
		 * 
		 * @param scCartItemProduct $orderItem
		 * @param int $orderItemId
		 * @return array
		 */
		public function saveOrderItemEgoods(scCartItemProduct $orderItem, $orderItemId)
		{
			$orderItemId = (int)$orderItemId;
			if ($orderItemId <= 0) {
				throw new miException('Invalid order item.');
 			}
 			
			$orderItemEgoods = $orderItem->getEgoods();
			$egoodIds = array();
			foreach ($orderItemEgoods as $egood) {
				$egoodObject = new scOrderItemEgood();
				$egoodObject->set('OrderItemID', $orderItemId);
				$egoodObject->set('OrderItemEgoodFile', $egood['ProductEgoodFile']);
				$egoodIds[] = $egoodObject->insert();
			}
			
			return $egoodIds;
		}
		
		protected function setOrderTotalLineData(scOrderTotalLine $orderTotalLine, scOrderTotalLineDomainObject $domainObject)
		{
			$domainObject->set('OrderTotalLineType', $orderTotalLine->getType());
			$domainObject->set('OrderTotalLineName', $orderTotalLine->getTranslatedDescription());
			$amount = $orderTotalLine->getAmount();
			$domainObject->set('OrderTotalLineAmount', $amount->get());
			$domainObject->set('OrderTotalLineCustomerCurrencyAmount', $amount->get(scCurrencyUtil::getCurrentCurrency()));
		}
		
		protected function setOptionImageData(scLocalOption $option, scOrderItemLocalOption $domainObject)
		{
			$orderItemOptionId = $domainObject->get($domainObject->getPKName());
			$tempFile = $option->getOptionUploadedFilePath();
			$imagePath = scPaths::getItemPath($orderItemOptionId, scPaths::RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE);
			rename($tempFile, $imagePath);
			list($width, $height) = getimagesize($imagePath);
			$domainObject->set('OrderItemOptionImage', scPaths::getItemPath($orderItemOptionId, scPaths::RESOURCE_ORDER_ITEM_OPTION_USER_IMAGE, scPaths::PATH_WEB));
			$domainObject->set('OrderItemOptionImageWidth', $width);
			$domainObject->set('OrderItemOptionImageHeight', $height);
		}
		
		protected function setLocalOptionImageData(scLocalOption $option, scOrderItemLocalOption $domainObject)
		{
			$orderItemLocalOptionId = $domainObject->get($domainObject->getPKName());
			$tempFile = $option->getOptionUploadedFilePath();
			$imagePath = scPaths::getItemPath($orderItemLocalOptionId, scPaths::RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE);
			rename($tempFile, $imagePath);
			list($width, $height) = getimagesize($imagePath);
			$domainObject->set('OrderItemLocalOptionImage', scPaths::getItemPath($orderItemLocalOptionId, scPaths::RESOURCE_ORDER_ITEM_LOCAL_OPTION_USER_IMAGE, scPaths::PATH_WEB));
			$domainObject->set('OrderItemLocalOptionImageWidth', $width);
			$domainObject->set('OrderItemLocalOptionImageHeight', $height);
		}
	}
?>