<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scOrdersInProgress {
		
		protected $_orderIds = array();
		protected $_lastOrderId;
		protected $_lastOrderCartHash;
		
		/**
		 * Returns the scOrdersInProgress object
		 * There is only one object - Singleton pattern
		 *
		 * @return scOrdersInProgress the object
		 */
		public static function singleton()
		{
			static $obj = null;
			//unset($_SESSION['OrdersInProgress']);
			if ($obj == null)
				if (isset($_SESSION['OrdersInProgress']))
					$obj = unserialize($_SESSION['OrdersInProgress']);
				else
					$obj = new scOrdersInProgress;
			return $obj;
		}
		
		public function __destruct()
		{
			$_SESSION['OrdersInProgress'] = serialize($this);
		}
		
		/**
		 * If the current user has access to the order
		 * 
		 * @param int $orderId
		 * @return boolean true if access allowed
		 */
		public function checkOrderId($orderId)
		{
			return isset($this->_orderIds[$orderId]);
		}
		
		public function addOrderId($orderId)
		{
			$this->_orderIds[$orderId] = $orderId;
		}
		
		/**
		 * Creates an order with the contents of the cart
		 * If the order is already created returns it's orderId
		 * 
		 * @param string $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param int $chekoutModuleId
		 * @return int the newly created order id
		 */
		public function createOrder($payAmount, $payCurrencyCode, scCart $cart, $chekoutModuleId)
		{
			// If the cart hasn't changed and there is an orderId return it
			if (($this->_lastOrderCartHash == $cart->getCartHash()) and $this->_lastOrderId) {
				$this->addOrderId($this->_lastOrderId);
				return $this->_lastOrderId;
			}
			
			miStaticDBUtil::startTransaction();
			
			$customerInfo = $cart->getCustomerInfo();
			
			$currentCurrency = scCurrencyUtil::getCurrentCurrency();
			
			$order = new scOrder();
			$order->set('CheckoutModuleID', $chekoutModuleId);
			$order->set('OrderCheckoutModuleName', $this->getCheckoutModuleName($chekoutModuleId));
			$order->set('OrderTotal', $cart->getOrderTotal()->getBaseCurrencyGrandTotal());
			$order->set('OrderCurrencyCode', scCurrencyUtil::getBaseCurrencyCode());
			$order->set('OrderCustomerCurrencyTotal', $cart->getOrderTotal()->getCurrentCurencyGrandTotal());
			$order->set('OrderCustomerCurrencyCode', $currentCurrency->getCurrencyCode());
			$order->set('OrderTotalPayCurrencyCode', $payCurrencyCode);
			$order->set('OrderTotalPayAmount', $payAmount);
			$order->setCustomerInfo($customerInfo);
			$this->setCustomerReferrerToOrder($order);
			
			$coupon = $cart->getCoupon();
			if (!is_null($coupon)) {
				$order->set('OrderCouponCode', $coupon['CouponCode']);
			} else {
				$order->set('OrderCouponCode', '');
			}
			
			$this->_lastOrderId = $order->insert();
			$this->_lastOrderCartHash = $cart->getCartHash();
			
			scOrderService::singleton()->saveOrderItems($order, $cart->getItems());
			
			// Save order total lines
			$orderTotalLines = $cart->getOrderTotal()->getOrderTotalLines();
			scOrderService::singleton()->saveOrderTotalLines($orderTotalLines, $this->_lastOrderId);
			
			miStaticDBUtil::commit();
			
			$this->addOrderId($this->_lastOrderId);
			
			return $this->_lastOrderId;
		}
		
		/**
		 * Returns array with the first value the shipping module name, the second the shipping method name
		 * 
		 * @param string $shippingModuleId the shipping module id
		 * @param string $shippingMethodCode the shipping method code
		 * @return array array(shipping module name, shipping method name)
		 * @throws miException if the module cannot be found, or the method is unknown
		 */
		public function getShippingModuleAndMethodNames($shippingModuleId, $shippingMethodCode)
		{
			if ($shippingModuleId == 0)
				return array('', '');
			
			$moduleId = scUtil::getDbValue('ShippingModules', 'ModuleID', 'ShippingModuleID', $shippingModuleId);
			if (empty($moduleId))
				throw new miException(sprintf(_t('Cannot find shipping module with ShippingModuleID "%s"!'), $shippingModuleId));
			
			$languageId = scMultiLanguage::getDefaultLanguageId();
			$sql = 'SELECT ShippingModuleDisplayName FROM ShippingModulesML WHERE ShippingModuleID=? AND LanguageID=?';
			$result = miStaticDBUtil::execSelect($sql, array($shippingModuleId, $languageId));
			if (empty($result))
				throw new miException(sprintf(_t('Cannot find shipping module name for ShippingModuleID "%s".'), $shippingModuleId));
			$shippingModuleDisplayName = $result[0]['ShippingModuleDisplayName'];
			
			$sql = 'SELECT ShippingMethodDisplayName FROM ShippingMethodsML' .
				' INNER JOIN ShippingMethods USING(ShippingMethodID) WHERE ShippingModuleID=? AND LanguageID=? AND ShippingMethodCode=?';
			$result = miStaticDBUtil::execSelect($sql, array($shippingModuleId, $languageId, $shippingMethodCode));
			if (!empty($result)) {
				$shippingMethodName = $result[0]['ShippingMethodDisplayName'];
			}
			
			if (!isset($shippingMethodName) || empty($shippingMethodName)) {
				$className = scModulesFactory::getModuleClassName($moduleId, 'IShippingModule');
				$shippingModule =  new $className($moduleId, $shippingModuleId);
				$shippingMethods = $shippingModule->getShippingMethods();
				$shippingMethodName = $shippingMethods[$shippingMethodCode];
			}
			
			if (isset($shippingMethodName))
				return array($shippingModuleDisplayName, $shippingMethodName);
			
			throw new miException(sprintf(_t('Unknown shipping method "%s" for shipping module "%s".'), $shippingMethodCode, $moduleId));
		}
		
		/**
		 * Returns the name of the payment module name
		 * 
		 * @param string $paymentModuleId the PaymentModuleID of the payment module
		 * @return array array(payment module name, payment module integration method)
		 * @throws miException if the payment module cannot be found
		 */
		public function getPaymentModuleName($paymentModuleId)
		{
			if ($paymentModuleId == 0)
				return array('','');
			
			$query = 'SELECT' .
				' IF (PaymentModuleDisplayName <> "", PaymentModuleDisplayName, PaymentModuleName) AS PaymentModuleName,' .
				' PaymentModuleIntegrationMethod' .
				' FROM PaymentModules' .
				' LEFT JOIN PaymentModulesML ON PaymentModules.PaymentModuleID = PaymentModulesML.PaymentModuleID AND LanguageID=?' .
				' WHERE PaymentModules.PaymentModuleID=?';
			
			$languageId = scMultiLanguage::getDefaultLanguageId();
			$rows = miStaticDBUtil::execSelect($query, array($languageId, $paymentModuleId));
			if (empty($rows))
				throw new miException(sprintf(_t('Cannot find payment module with PaymentModuleID "%s".'), $paymentModuleId));
			
			return array($rows[0]['PaymentModuleName'], $rows[0]['PaymentModuleIntegrationMethod']);
		}
		
		protected function getCheckoutModuleName($checkoutModuleId)
		{
			if ($checkoutModuleId == 0)
				return '';
			
			$moduleName = scUtil::getDbValue('CheckoutModules', 'CheckoutModuleName', 'CheckoutModuleID', $checkoutModuleId);
			if ($moduleName)
				return $moduleName;
			
			throw new miException(sprintf(_t('Cannot find checkout module with CheckoutModuleID "%s".'), $checkoutModuleId));
		}
		
		/**
		 * Returns the order id
		 */
		public function getLastOrderId()
		{
			return $this->_lastOrderId;
		}
		
		protected function setCustomerReferrerToOrder(scOrder $order)
		{
			if (!isset($_COOKIE[scBonusPoints::REFERRER_PARAM])) {
				return;
			}
			if (!scBonusPoints::isValidReferringHash($_COOKIE[scBonusPoints::REFERRER_PARAM])) {
				return;
			}
			list($referrerId, $referredEmail) = scBonusPoints::decodeReferringHash($_COOKIE[scBonusPoints::REFERRER_PARAM]);
			$referrerId = (int)$referrerId;
			
			if ($referrerId <= 0) {
				return;
			}
			
			try {
				$referrer = new scCustomerDomainObject();
				$referrer->read($referrerId);
				
				$order->set('OrderCustomerReferringHash', $_COOKIE[scBonusPoints::REFERRER_PARAM]);
			} catch (miException $e) {
			}
		}
	}
?>