<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPaymentForms {
		/**
		 * Number of years to list in the credit card expiration drop-down
		 */
		const NUMBER_OF_YEARS = 15;
		
		/**
		 * Check account types
		 */
		const ACCOUNT_TYPE_CHECKING = 1;
		const ACCOUNT_TYPE_SAVINGS = 2;
		
		const CHECK_TYPE_US = 0;
		const CHECK_TYPE_UK = 1;
		
		static public function getCheckTypes()
		{
			return array(
				self::CHECK_TYPE_US => _t('US Checks'),
				self::CHECK_TYPE_UK => _t('UK Checks')
			);
		}
		
		static public function getCheckAccountTypes()
		{
			return array(
				self::ACCOUNT_TYPE_CHECKING => _t('Checking'),
				self::ACCOUNT_TYPE_SAVINGS => _t('Savings')
			);
		}
		
		static public function getRoutingNumberName($checkType)
		{
			if ($checkType == self::CHECK_TYPE_US)
				return _t('ABA Routing Number:');
			else if ($checkType == self::CHECK_TYPE_UK)
				return _t('Sort Code:');
			else
				throw new miException(sprintf(_i('Unknown check type "%s".'), $checkType));
		}
		
		static public function getCreditCardForm()
		{
			$expMonths = array(0 => _t('Month'), 1 => '01', 2 => '02', 3 => '03', 4 => '04', 5 => '05',
				6 => '06', 7 => '07', 8 => '08', 9 => '09', 10 => '10', 11 => '11', 12 => '12');
			
			$expYears = array(0 => _t('Year'));
			$currentYear = gmdate('Y');
			for ($i = 0; $i < self::NUMBER_OF_YEARS; $i++)
				$expYears[$currentYear+$i] = $currentYear+$i;
			
			$form = new scPaymentModuleWebForm();
			$form->initWidgets(array(
				array(
					'field' => 'miTextWidget',
					'data' => 'NameOnCard'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'CardNumber'
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CardExpirationMonth'
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CardExpirationYear'
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'CardCVV'
				),
			));
			
			$form->getWidget('CardExpirationMonth')->setOptions($expMonths);
			$form->getWidget('CardExpirationYear')->setOptions($expYears);
			
			return $form->parse('modules/payment/creditcard/payment_form.tmpl', true);
		}
		
		/**
		 * Perform MOD 10 check on the card number
		 * Will ignore spaces and dashes
		 * 
		 * @param string $cardNumber
		 * @return boolean
		 */
		static public function isCardNumberValid($cardNumber)
		{
			// strip all spaces and dashes
			$cardNumber = str_replace(array(' ', '-'), '', $cardNumber);
			
			if (!ctype_digit($cardNumber))
				return false;
			
			// According to ISO/IEC 7812-1 the card number is maximum 19 digits
			// Also it must have a parity check digit computed according to the Luhn formula
			if ((strlen($cardNumber) < 12) or (strlen($cardNumber) > 19))
				return false;
			
			// Compute checksum
			$cardNumber = str_pad($cardNumber, 16, '0', STR_PAD_LEFT);
			$checksum = 0;
			for ($i = strlen($cardNumber)-1; $i >= 0; $i--) {
				$digit = ($i % 2) ? $cardNumber[$i] : $cardNumber[$i] * 2;
				if ($digit > 9)
					$digit -= 9;
				$checksum += $digit;
			}
			
			if ($checksum % 10)
				return false;
			
			return true;
		}
		
		/**
		 * Strips all spaces and dashes from the card number
		 * 
		 * @param string $cardNumber
		 */
		static public function normalizeCardNumber($cardNumber)
		{
			return str_replace(array(' ', '-'), '', $cardNumber);
		}

		/**
		 * Checks if the start is this month or in the past
		 * 
		 * @param int $startYear
		 * @param int $startMonth
		 * @return boolean
		 */
		static public function isCardStartValid($startYear, $startMonth)
		{
			if (!$startYear or !$startMonth)
				return false;
			
			if ($startYear > gmdate('Y'))
				return false;
			
			if (($startYear == gmdate('Y')) and ($startMonth > gmdate('n')))
				return false;
			
			return true;
		}
		
		/**
		 * Checks if the expiration is this month, or in the future
		 * 
		 * @param int $expYear
		 * @param int $expMonth
		 * @return boolean
		 */
		static public function isCardExpirationValid($expYear, $expMonth)
		{
			if (!$expYear or !$expMonth)
				return false;
			
			if ($expYear < gmdate('Y'))
				return false;
			
			if (($expYear == gmdate('Y')) and ($expMonth < gmdate('n')))
				return false;
			
			return true;
		}
		
		static public function getCheckForm($acceptedCheckType)
		{
			$form = new scPaymentModuleWebForm();
			$form->initWidgets(array(
				array(
					'field' => 'miSelectWidget',
					'data' => 'AccountType'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'RoutingNumber'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AccountNumber'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'BankName'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'NameOnAccount'
				),
			));
			
			// Display the correct name of the routing number
			$form->addMainPageElements(array('%%ROUTING_NUMBER_NAME%%' => self::getRoutingNumberName($acceptedCheckType)));
			
			$form->getWidget('AccountType')->setOptions(self::getCheckAccountTypes());
			
			return $form->parse('modules/payment/check/payment_form.tmpl', true);
		}
	}
?>
