<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scPaymentProcess {
		const CALL_TYPE_RETURN = 1;
		const CALL_TYPE_RELAYED_RETURN = 2;
		const CALL_TYPE_NOTIFY = 3;
		const CALL_TYPE_DIRECT = 4;
		
		static $_paymentModuleResult = null;
		static $_paymentProcessException = null;
		
		/**
		 * Calls a payment module and handles the result
		 *
		 * @param string $moduleName the module name
		 * @param string $methodName the method to call
		 * @param int $callType one of the CALL_TYPE_* constants
		 * @param string $interface the interface that the module implements, by default IPaymentModule
		 * @return boolean returns true on success
		 */
		public static function callPaymentModule($moduleName, $methodName, $callType, $interface = 'IPaymentModule')
		{
			$moduleId = scModulesFactory::getModuleIdByName($moduleName);
			$paymentModule = scModulesFactory::getImplementation($moduleId, $interface);
			
			try {
				$callPaymentModuleResult = $paymentModule->$methodName(array_merge($_GET, $_POST));
				self::$_paymentModuleResult = $callPaymentModuleResult;
				self::handlePaymentProcessResult($callPaymentModuleResult, $moduleId, $callType);
				return true;
			} catch (miException $exception) {
				self::handlePaymentProcessException($exception, $callType);
				self::$_paymentProcessException = $exception;
				return false;
			}
		}
		
		/**
		 * Returns the exception encountered during the payment process
		 *
		 * @return Exception|null
		 */
		public static function getPaymentProcessException()
		{
			return self::$_paymentProcessException;
		}
		
		/**
		 * Returns the payment module method result
		 *
		 * @return scPaymentProcessResult|null
		 */
		public static function getPaymentModuleResult()
		{
			return self::$_paymentModuleResult;
		}
		
		/**
		 * Handle payment process result
		 *
		 * @param scPaymentProcessResult $callPaymentModuleResult the result of the payment process
		 * @param int $moduleId the ModuleID of the payment module
		 * @param int $callType one of the CALL_TYPE_* constants
		 */
		public static function handlePaymentProcessResult(scPaymentProcessResult $callPaymentModuleResult, $moduleId, $callType)
		{
			try {
				// Check if we do third party redirect
				list($url, $params, $method, $options) = $callPaymentModuleResult->getThirdPartyRedirect();
				if ($url != '') {
					scPaymentProcess::doThirdPartyRedirect($url, $params, $method, $options);
					return;
				}
				
				// We don't do redirect - handle payment result
				$orderId = $callPaymentModuleResult->getOrderId();
				if (!$orderId)	// No order id - we can't process it without order id
					return;
				
				$order = new scOrder();
				$order->read($orderId);
				
				$transactionCode = $callPaymentModuleResult->getTransactionCode();
				
				$paymentNotes = trim($callPaymentModuleResult->getPaymentNotes());
				if ($paymentNotes != '') {
					// If there are old OrderPaymentNotes, append the new notes
					if ($order->get('OrderPaymentNotes') != '')
						$paymentNotes = $order->get('OrderPaymentNotes') . '; ' . $paymentNotes;
					
					$order->set('OrderPaymentNotes', $paymentNotes);
					$order->update();
				}
				
				// Security check - if the orderId and transaction code match allow the user to view the order
				// This will allow viewing the order even if the session has expired
				if (!empty($transactionCode) and scTransaction::transactionExists($orderId, $transactionCode)) {
					scOrdersInProgress::singleton()->addOrderId($orderId);
				}
				
				self::createTransaction(new scOrder($orderId), $moduleId, $callPaymentModuleResult->getTransactionAmount(),
					$callPaymentModuleResult->getTransactionCurrencyCode(),
					$transactionCode, $callPaymentModuleResult->getPaymentStatus(),
					$callPaymentModuleResult->getTransactionNote(), $callPaymentModuleResult->getAllTransactionDetails());
				
				switch ($callType) {
					case self::CALL_TYPE_DIRECT:
					case self::CALL_TYPE_RETURN:
						self::redirectToPaymentResult($orderId, $callPaymentModuleResult->getPaymentNotes());
						break;
					
					case self::CALL_TYPE_RELAYED_RETURN:
						// TODO: V1.1 Implement
						
					case self::CALL_TYPE_NOTIFY:
						break;
				}
				
			} catch (miException $exception) {
				self::handlePaymentProcessException($exception, $callType);
			}
		}
		
		public static function handlePaymentProcessException($exception, $callType)
		{
			$message = _t('Processing payment exception: ') . $exception->getMessage();
			$messageDetailsStr = '';
			if ($exception instanceof scPaymentProcessException) {
				$details = $exception->getDetails();
				if (count($details)) {
					$messageDetails = array();
					foreach ($details as $key => $value)
						$messageDetails[] = $key . '="' . $value . '"';
					$messageDetailsStr = "\n " . _t('Details:') . ' ' . implode('; ', $messageDetails);
				}
			}
			scLog::singleton()->warning($message . $messageDetailsStr, scLog::CATEGORY_PAYMENT);
			
			if ($callType == self::CALL_TYPE_NOTIFY)
				return;
			
			// Inform the user
			self::redirectToPaymentResult(0, $message);
		}
		
		/**
		 * Redirects to third party processor
		 *
		 * @param string $url the url to redirect to
		 * @param array $params the params sent to the url
		 * @param string $method the redirect method - get, post or iframe (optional), post by default
		 * @param array $options redirect options, depend on the method
		 */
		public static function doThirdPartyRedirect($url, $params, $method = 'post', $options = array())
		{
			if ($method == 'post') {
				$redirectingText = _t('Redirecting to payment processor...');
				
				// The doctype is commented because the message will not be centered in the screen otherwise
//				$html = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
				$html = '<html xmlns="http://www.w3.org/1999/xhtml">';
				$html .= '<head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /><title>' . $redirectingText . '</title></head>';
				$html .= '<body onload="document.payment.submit();">';
				$html .= '<table width="100%" height="100%"><tr><td align="center" valign="middle"><h2>' . $redirectingText . '</h2></td></tr></table>';
				$html .= '<form action="' . miI18N::htmlEscape($url) . '" name="payment" method="post">';
				foreach ($params as $name => $param) {
					$html .= '<input type="hidden" name="' . miI18N::htmlEscape($name) . '" value="' . miI18N::htmlEscape($param) . '"></input>';
				}
				$html .= '</form></body></html>';
				echo $html;
				
			} elseif ($method == 'iframe') {
				$page = new scUserDisplayController(array());
				$page->setSeo(new scSeo(_t('Payment Process')));
				echo $page->getHeader();
				$skin = scSkins::singleton()->getCurrentSkin();
				echo $skin->getColumn(scLayout::COLUMN_LEFT, scLayout::AREA_CUSTOMER);
				
				// Build the iframe
				$url .= '?' . http_build_query($params);
				$width = isset($options['iframe_width']) ? $options['iframe_width'] : '100%';
				$height = isset($options['iframe_height']) ? $options['iframe_height'] : '100%';
				
				$html = '<div style="text-align: center">';
				$html .= '<iframe src="' . miI18N::htmlEscape($url) . '" width="' . miI18N::htmlEscape($width) . '" height="' . miI18N::htmlEscape($height) . '" frameborder="0"></iframe>';
				$html .= '</div>';
				echo $html;
				
				echo $skin->getColumn(scLayout::COLUMN_RIGHT, scLayout::AREA_CUSTOMER);
				echo $page->getFooter();
				
			} else {
				ob_end_clean();
				if (!empty($params))
					$url .= '?' . http_build_query($params);
				header('Location: ' . $url);
			}
		}
		
		/**
		 * Creates transaction and updates the order status and the order payment status
		 * Checks for duplicate CAPTURE transactions
		 *
		 * @param scOrder $order
		 * @param int $moduleId the module id of the payment module
		 * @param int $paymentStatus
		 * @param string $note
		 * @param array $transactionDetails
		 * @return int|false the transaction id
		 */
		public static function createTransaction(scOrder $order, $moduleId, $transactionAmount, $transactionCurrencyCode,
			$transactionCode, $paymentStatus, $note = '', $transactionDetails = array())
		{
			// Check for duplicate CAPTURE
			if ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_CAPTURE) {
				if (scTransaction::transactionExists($order->getPK(), $transactionCode, $paymentStatus)) {
					return false;
				}
			}
			
			// Create transaction for all payment statuses except "None", "At Processor" and "Offline"
			$noTransactions = array(scPaymentStatus::PAYMENT_STATUS_NONE, scPaymentStatus::PAYMENT_STATUS_AT_PROCESSOR, scPaymentStatus::PAYMENT_STATUS_OFFLINE);
			if (!in_array($paymentStatus, $noTransactions)) {
				$transaction = new scTransaction($order->getPK(), $moduleId, $transactionAmount, $transactionCurrencyCode,
					$transactionCode, $paymentStatus, $note);
				$transaction->setAllTransactionDetails($transactionDetails);
				$transactionId = $transaction->insertTransaction();
			} else
				$transactionId = false;
			
			
			self::updateOrderPaymentStatus($order, $paymentStatus);
			
			// Change the order status ONLY if it is unfinished, payment failed or payment pending
			$orderStatus = $order->get('OrderStatus');
			$canChageOrderStatuses = array(scOrderStatus::ORDER_STATUS_UNFINISHED,
				scOrderStatus::ORDER_STATUS_PAYMENT_FAILED,
				scOrderStatus::ORDER_STATUS_PAYMENT_PENDING
			);
			if (!in_array($orderStatus, $canChageOrderStatuses))
				return $transactionId;
			
			switch ($paymentStatus) {
				case scPaymentStatus::PAYMENT_STATUS_OFFLINE:
				case scPaymentStatus::PAYMENT_STATUS_AUTH:
				case scPaymentStatus::PAYMENT_STATUS_CAPTURE:
					$order->updateOrderStatus(scOrderStatus::ORDER_STATUS_NEW);
					break;
				
				case scPaymentStatus::PAYMENT_STATUS_DECLINED:
					$order->updateOrderStatus(scOrderStatus::ORDER_STATUS_PAYMENT_FAILED);
					break;
				
				case scPaymentStatus::PAYMENT_STATUS_PENDING:
					$order->updateOrderStatus(scOrderStatus::ORDER_STATUS_PAYMENT_PENDING);
			}
			
			// Set the payment transaction code if the payment status is Auth or Capture
			if (($paymentStatus == scPaymentStatus::PAYMENT_STATUS_AUTH) or ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_CAPTURE)) {
				$order->set('OrderPaymentTransactionNo', $transactionCode);
				$order->update();
			}
			
			return $transactionId;
		}
		
		/**
		 * Update the order payment status
		 * If the status is PAYMENT_STATUS_NONE nothing is updated
		 * If the status is PAYMENT_STATUS_AT_PROCESSOR then checks for newer info in Transactions
		 * 
		 * @param scOrder $order
		 * @param int $paymentStatus
		 */
		protected static function updateOrderPaymentStatus(scOrder $order, $paymentStatus)
		{
			// If the new payment status is PENDING, we can change the OrderPaymentStatus only if
			// the current OrderPaymentStatus is AT_PROCESSOR or NONE
			if (($paymentStatus == scPaymentStatus::PAYMENT_STATUS_PENDING) and 
				($order->get('OrderPaymentStatus') != scPaymentStatus::PAYMENT_STATUS_AT_PROCESSOR) and
				($order->get('OrderPaymentStatus') != scPaymentStatus::PAYMENT_STATUS_NONE))
			{
					return;
			}

			if ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_NONE)
				return;
			
			// If the payment status says at processor, check if we have more recent info
			// eg. from PayPal IPN, Moneybookers notify, etc.
			if ($paymentStatus == scPaymentStatus::PAYMENT_STATUS_AT_PROCESSOR) {
				$sql = 'SELECT TransactionPaymentStatus FROM Transactions WHERE OrderID=? ORDER BY TransactionTimestamp DESC LIMIT 1';
				$result = miStaticDBUtil::execSelect($sql, array($order->getPK()));
				if (!empty($result)) {
					$lastPaymentStatus = $result[0]['TransactionPaymentStatus'];
					
					// If the last payment status is different, then we have more recent info, thus don't update the order payment status
					if ($lastPaymentStatus != $paymentStatus)
						return;
				}
			}
			
			$order->updateOrderPaymentStatus($paymentStatus);
		}
		
		/**
		 * Checks that the payment amount and currency correspond to the order
		 * Precision is 2 digits after the decimal point
		 *
		 * @param int $orderId
		 * @param string $amount
		 * @param string $currencyCode (optional)
		 * @throws scPaymentProcessException
		 */
		public static function checkPaymentToOrder($orderId, $amount, $currencyCode = false)
		{
			$order = new scOrder($orderId);
			
			if ($currencyCode !== false) {
				if ($order->get('OrderTotalPayCurrencyCode') != $currencyCode)
					throw new scPaymentProcessException(_t('Currency code mismatch.'), array(_t('OrderID') => $orderId,
						_t('Received currency') => $currencyCode, _t('Expected currency') => $order->get('OrderTotalPayCurrencyCode')));
			}
			
			// Check the amount - precision is 2 digits after the decimal point
			if (round($order->get('OrderTotalPayAmount'),2) != round($amount,2))
				throw new scPaymentProcessException(_t('Payment amount mismatch.'), array(_t('OrderID') => $orderId,
					_t('Received amount') => $amount, _t('Expected amount') => $order->get('OrderTotalPayAmount')));
		}
		
		/**
		 * Redirects to the payment result page
		 *
		 * @param int $orderId
		 * @param string $transactionCode
		 * @param int $paymentStatus
		 * @param string $message (optional)
		 */
		public static function redirectToPaymentResult($orderId, $message = '')
		{
			if (intval($orderId) == 0) {
				$url = scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART);
				$url = scUrlUtil::addQueryParams($url, 'msg=' . urlencode($message));
				$url = scUrlUtil::convertUrl($url, scUrlUtil::URL_TYPE_ABSOLUTE);
				scUtil::redirect($url);
			}
			
			header('Location: ' . scPaths::getSecureUrl('checkout_final.php') .
				'?OrderID=' . urlencode($orderId) . '&Message=' . urlencode($message));
		}
		
		/**
		 * Checks that all params have been passed from the payment processor
		 *
		 * @param array $params array($paramName => $paramValue, ...)
		 * @param array $paramNames array($expectedParam1, $expectedParam2, ...)
		 * @throws scPaymentProcessException
		 */
		public static function checkParamsExist($params, $paramNames)
		{
			foreach ($paramNames as $paramName)
				if (!isset($params[$paramName]))
					throw new scPaymentProcessException(sprintf(_t('Missing param "%s".'), $paramName));
		}
	}
?>