<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scProductListingService {
		
		/**
		 * Product options are usually not used in listings but may be needed sometimes.
		 */
		const OPTION_INCLUDE_PRODUCT_OPTIONS_SECTION = 1;
		
		/**
		 * Convert product URLs to absolute
		 */
		const OPTION_FORCE_ABSOLUTE_URLS = 2;
		
		/**
		 * Product listings may have modules. Currently only Product Comparison is initialized but that may change.
		 * Loading and using modules may slow down listings.
		 */
		const OPTION_NO_MODULES = 4;
		
		/**
		 * @var bool
		 */
		protected $_forceAbsoluteUrls = false;
		
		/**
		 * @return scProductListingService
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scProductListingService();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		public function setForceAbsoluteUrls($force)
		{
			$this->_forceAbsoluteUrls = $force;
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		public function getSection($sectionName, $listingItems, $viewType = scProductService::VIEW_LIST, $options = 0)
		{
			if ($this->_forceAbsoluteUrls) {
				$options = $options | self::OPTION_FORCE_ABSOLUTE_URLS;
			}
			$commonFields = array_flip(scUtil::getCommonKeysInRow($listingItems));
			
			$this->prepareCache($listingItems);
			
			$displayItems = array();
			foreach ($listingItems as &$listingItem) {
				if (!($listingItem instanceof IProductListingItem)) {
					throw new miException(_i('Each item must implement IProductListingItem interface.'));
				}
				
				$displayItem = array();
				$displayItem = array_intersect_key($listingItem->getRow(), $commonFields);
				$displayItem['ProductID'] = $listingItem->getProductId();
				$displayItem['ProductName'] = $listingItem->getName();
				$displayItem['ProductPrice'] = $listingItem->getPrice()->getAmount()->get();
				$displayItem['ProductUrl'] = $listingItem->getUrl();
				if ($options & self::OPTION_FORCE_ABSOLUTE_URLS) {
					$displayItem['ProductUrl'] = scUrlUtil::convertUrl($displayItem['ProductUrl'], scUrlUtil::URL_TYPE_ABSOLUTE);
				}
				$displayItem['HTML_ProductDescription'] = $listingItem->getDescription();
				$displayItem['CategoryID'] = $listingItem->getCategoryId();
				$displayItem['InBundleQuantity'] = 1;
				if ($listingItem instanceof scBundledProduct) {
					$displayItem['InBundleQuantity'] = $listingItem->getInBundleQuantity();
				}
				$displayItems[] = $displayItem;
			}
			
			if ($viewType == scProductService::VIEW_GRID) {
				return $this->getGridSections($sectionName, $listingItems, $displayItems, $options);
			}
			
			return $this->getListSections($sectionName, $listingItems, $displayItems, $options);
		}
		
		/**
		 * @param IProductListingItem[] $listingItems
		 */
		protected function prepareCache($listingItems)
		{
			// Tell the ProductsXCategories service which products it will need later
			// so it can gather their data all at once.
			// Do it as well for CustomUrls with products, variants and manufacturers.
			// Preare manufacturers cache as well.
			$productIds = array();
			$productVariantIds = array();
			$manufacturerIds = array();
			foreach ($listingItems as $listingItem) {
				$productIds[] = $listingItem->getProductId();
				if ($listingItem instanceof scProductVariant) {
					$productVariantIds[] = $listingItem->getPK();
				}
				if (!is_null($listingItem->getManufacturerId())) {
					$manufacturerIds[] = $listingItem->getManufacturerId();
				}
				unset($listingItem);
			}
			scProductsXCategories::prepareProducts($productIds);
			scCustomUrls::prepareItems(scFriendlyUrl::TYPE_PRODUCT, $productIds);
			if (!empty($productVariantIds)) {
				scCustomUrls::prepareItems(scFriendlyUrl::TYPE_PRODUCT_VARIANT, $productVariantIds);
			}
			if (!empty($manufacturerIds)) {
				scManufacturersCache::prepareItems($manufacturerIds);
			}
		}
		
		/**
		 * @return miTemplateParserSectionInfo
		 */
		protected function getGridSections($sectionName, $listingItems, $displayItems, $options = 0)
		{
			$priceModifications = $this->getPriceModifications($listingItems);
			$columnsNumber = scProductService::getProductGridColumns();
			
			$productIds = array();
			$bundleProductIds = array();
			foreach ($listingItems as $listingItem) {
				$productIds[] = $listingItem->getProductId();
				if ($listingItem instanceof scProductSL) {
					$product = $listingItem;
				} else {
					$product = $listingItem->getProductSL();
				}
				if ($product->get('ProductIsBundle')) {
					$bundleProductIds[] = $listingItem->getProductId();
				}
			}
			$productsHavingOptions = $this->getProductsHavingOptions($productIds, $bundleProductIds);
			
			// Split the products in rows
			$productsCount = count($displayItems);
			$columnWidth = (100.0 / $columnsNumber) . '%';
			$displayItems = array_chunk($displayItems, $columnsNumber);
			$listingItems = array_chunk($listingItems, $columnsNumber);
			
			$comparisonModuleLoaded = false;
			if (!($options & self::OPTION_NO_MODULES)) {
				try {
					// Load the product comparison module if it has not been loaded
					$moduleId = scModulesFactory::getModuleIdByName('com.summercart.productcomparison');
					scModulesFactory::getImplementation($moduleId, 'IProductDetails');
					$comparisonModuleLoaded = true;
				} catch (miException $e) {
					// Do not throw exception if module loading failed. It's a module after all!
				}
			}
			$comparableItemsAvailable = false;
			
			// Collect all the row sections
			$columnSections = array();
			foreach ($displayItems as $rowKey => $row) {
				// Pad each row to specified length
				$row = array_pad($row, $columnsNumber, null);
				
				// Create the column section
				$columnSection = new miTemplateParserSectionInfo(
					$sectionName . 'Row',
					$columnsNumber,
					array('%%COLUMN_WIDTH%%' => array_fill(0, $columnsNumber, $columnWidth))
				);
				
				// Collect the subsections for the column section
				$productSections = array();
				$noProductSections = array();
				foreach ($row as $productKey => $productInfo) {
					if (is_null($productInfo)) {
						$noProductSection = new miTemplateParserSectionInfo('no' . $sectionName, 1);
						$productSection = new miTemplateParserSectionInfo($sectionName, 0);
						$productSections[] = $productSection;
						$noProductSections[] = $noProductSection;
						continue;
					}
					$noProductSection = new miTemplateParserSectionInfo('no' . $sectionName, 0);
					// Set the product section information
					$productSection = new miTemplateParserSectionInfo($sectionName, 1, miUIUtil::templetizeRows(miUIUtil::transposeRows(array($productInfo))));
					
					$listingItem = $listingItems[$rowKey][$productKey];// IProductListingItem
					$productId = $listingItem->getProductId();
					
					// Add the product subsections
					$productPriceModifications = isset($priceModifications[$productId]) ? $priceModifications[$productId] : array();
					$productSection->addSubsection($this->getPriceSection($listingItem, $productPriceModifications));
					
					$imageSection = new miTemplateParserSectionInfo('image');
					$imageData = $listingItem->getImageInfo()->getDataArray('ProductImage');
					if ($this->_forceAbsoluteUrls) {
						$imageData['ProductImage'] = scUrlUtil::convertUrl($imageData['ProductImage'], scUrlUtil::URL_TYPE_ABSOLUTE);
						$imageData['ProductImageThumb'] = scUrlUtil::convertUrl($imageData['ProductImageThumb'], scUrlUtil::URL_TYPE_ABSOLUTE);
					}
					$imageSection->setValues(miUIUtil::templetizeAndEscapeRows($imageData));
					$productSection->addSubsection($imageSection);
					
					list($addToCart, $outOfStock, $viewDetails) = $this->getCartRelatedSections($listingItem, $productsHavingOptions, $options);
					$productSection->addSubsection($addToCart);
					$productSection->addSubsection($outOfStock);
					$productSection->addSubsection($viewDetails);
					
					$productSection->addSubsection(scProductService::getNewProductSection($listingItem));
					$productSection->addSubsection(scProductService::getEgoodProductSection($listingItem));
					
					if (($listingItem instanceof scBundledProduct) && ($listingItem->getInBundleQuantity() > 1)) {
						$productSection->addSubsection(new miTemplateParserSectionInfo('quantityIsMoreThanOne', 1));
					} else {
						$productSection->addSubsection(new miTemplateParserSectionInfo('quantityIsMoreThanOne', 0));
					}
					
					if ($comparisonModuleLoaded && !empty($productInfo['ProductClassID'])) {
						$comparableItemsAvailable = true;
						if (in_array($productId, scProductComparison::singleton()->getProductIds())) {
							$productSection->addSubsection(new miTemplateParserSectionInfo('productIsInCompareList', 1));
							$productSection->addSubsection(new miTemplateParserSectionInfo('productIsComparable', 0));
						} else {
							$productSection->addSubsection(new miTemplateParserSectionInfo('productIsComparable', 1));
							$productSection->addSubsection(new miTemplateParserSectionInfo('productIsInCompareList', 0));
						}
					}
					
					$productSections[] = $productSection;
					$noProductSections[] = $noProductSection;
				}
				
				// Add the row subsections
				$columnSection->addSubsectionsArray('no' . $sectionName, $noProductSections);
				$columnSection->addSubsectionsArray($sectionName, $productSections);
				
				// Collect the row sections
				$columnSections[] = $columnSection;
			}
			
			if ($productsCount > 0) {
				$hasProductsSection = new miTemplateParserSectionInfo('has' . $sectionName, 1);
				// Add the rows section
				$rowSection = new miTemplateParserSectionInfo($sectionName . 'Row', count($columnSections));
				$rowSection->addSubsectionsArray($sectionName . 'Column', $columnSections);
				$hasProductsSection->addSubsection($rowSection);
				if ($comparisonModuleLoaded && $comparableItemsAvailable) {
					$hasProductsSection->addSubsection(new miTemplateParserSectionInfo('hasComparableProducts', 1));
				}
			} else {
				$hasProductsSection = new miTemplateParserSectionInfo('no' . $sectionName, 1);
			}
			
			return $hasProductsSection;
		}
		
		/**
		 * @param string $sectionName
		 * @param IProductListingItem[] $listingItems
		 * @param int $options Bitmask.
		 * @return miTemplateParserSectionInfo
		 */
		protected function getListSections($sectionName, &$listingItems, $displayItems, $options = 0)
		{
			$productsSection = new miTemplateParserSectionInfo();
			$productsSection->setSectionInfo($sectionName, count($displayItems), miUIUtil::templetizeRows(miUIUtil::transposeRows($displayItems)));
			
			$priceModifications = $this->getPriceModifications($listingItems);
			
			$comparisonModuleLoaded = false;
			if (!($options & self::OPTION_NO_MODULES)) {
				try {
					// Load the product comparison module if it has not been loaded
					$moduleId = scModulesFactory::getModuleIdByName('com.summercart.productcomparison');
					scModulesFactory::getImplementation($moduleId, 'IProductDetails');
					$comparisonModuleLoaded = true;
				} catch (miException $e) {
					// Do not throw exception if module loading failed. It's a module after all!
				}
			}
			$comparableItemsAvailable = false;
			
			$productPricesSections = array();
			$productPictureSections = array();
			$isNewProductSections = array();
			$isEgoodProductSections = array();
			$productIsComparableSections = array();
			$productIsInCompareListSections = array();
			$quantityIsMoreThanOneSections = array();
			
			foreach ($listingItems as $listingItem) {
				$productId = $listingItem->getProductId();
				
				$productPriceModifications = isset($priceModifications[$productId]) ? $priceModifications[$productId] : array(); 
				$productPricesSections[] = $this->getPriceSection($listingItem, $productPriceModifications);
				
				$productPictureSection = new miTemplateParserSectionInfo('image');
				$imageData = $listingItem->getImageInfo()->getDataArray('ProductImage');
				if ($this->_forceAbsoluteUrls) {
					$imageData['ProductImage'] = scUrlUtil::convertUrl($imageData['ProductImage'], scUrlUtil::URL_TYPE_ABSOLUTE);
					$imageData['ProductImageThumb'] = scUrlUtil::convertUrl($imageData['ProductImageThumb'], scUrlUtil::URL_TYPE_ABSOLUTE);
				}
				$productPictureSection->setValues(miUIUtil::templetizeAndEscapeRows($imageData));
				$productPictureSections[] = $productPictureSection;
				
				$isNewProductSections[] = scProductService::getNewProductSection($listingItem);
				$isEgoodProductSections[] = scProductService::getEgoodProductSection($listingItem);
				
				if (($listingItem instanceof scBundledProduct) && ($listingItem->getInBundleQuantity() > 1)) {
					$quantityIsMoreThanOneSections[] = new miTemplateParserSectionInfo('quantityIsMoreThanOne', 1);
				} else {
					$quantityIsMoreThanOneSections[] = new miTemplateParserSectionInfo('quantityIsMoreThanOne', 0);
				}
				
				if ($comparisonModuleLoaded && ($listingItem->getProductClassId() > 0)) {
					$comparableItemsAvailable = true;
					if (in_array($productId, scProductComparison::singleton()->getProductIds())) {
						$productIsInCompareListSections[] = new miTemplateParserSectionInfo('productIsInCompareList', 1);
						$productIsComparableSections[] = new miTemplateParserSectionInfo('productIsComparable', 0);
					} else {
						$productIsComparableSections[] = new miTemplateParserSectionInfo('productIsComparable', 1);
						$productIsInCompareListSections[] = new miTemplateParserSectionInfo('productIsInCompareList', 0);
					}
				} else {
					$productIsComparableSections[] = new miTemplateParserSectionInfo('productIsComparable', 0);
					$productIsInCompareListSections[] = new miTemplateParserSectionInfo('productIsInCompareList', 0);
				}
			}
			$productsSection->addSubsectionsArray('price', $productPricesSections);
			$productsSection->addSubsectionsArray('image', $productPictureSections);
			$productsSection->addSubsectionsArray('isNewProduct', $isNewProductSections);
			$productsSection->addSubsectionsArray('isEgoodProduct', $isEgoodProductSections);
			$productsSection->addSubsectionsArray('quantityIsMoreThanOne', $quantityIsMoreThanOneSections);
			if (!($options & self::OPTION_NO_MODULES)) {
				// XXX: This doesn't look very "modular"
				$productsSection->addSubsectionsArray('productIsComparable', $productIsComparableSections);
				$productsSection->addSubsectionsArray('productIsInCompareList', $productIsInCompareListSections);
			}
			
			$this->addCartRelatedListSections($productsSection, $listingItems, $options);
			
			if (count($listingItems) > 0) {
				$hasProductsSection = new miTemplateParserSectionInfo('has' . $sectionName, 1);
				$hasProductsSection->addSubsection($productsSection);
				if ($comparisonModuleLoaded && $comparableItemsAvailable) {
					$hasProductsSection->addSubsection(new miTemplateParserSectionInfo('hasComparableProducts', 1));
				}
			}
			else
				$hasProductsSection = new miTemplateParserSectionInfo('no' . $sectionName, 1);
			
			return $hasProductsSection;
		}
		
		protected function addCartRelatedListSections(miTemplateParserSectionInfo $productsSection, &$listingItems, $settings = 0)
		{
			$addToCartSections = array();
			$outOfStockSections = array();
			$viewDetailsSections = array();
			$hasOptionsSections = array();
			
			$productIds = array();
			$bundleProductIds = array();
			foreach ($listingItems as $listingItem) {
				$productIds[] = $listingItem->getProductId();
				if ($listingItem instanceof scProductSL) {
					$product = $listingItem;
				} else {
					$product = $listingItem->getProductSL();
				}
				if ($product->get('ProductIsBundle')) {
					$bundleProductIds[] = $listingItem->getProductId();
				}
			}
			
			// Get product options (keyed by ProductID)
			$productsHavingOptions = $this->getProductsHavingOptions($productIds, $bundleProductIds);
			foreach ($listingItems as $listingItem) {
				$productId = $listingItem->getProductId();
				
				if ($settings & self::OPTION_INCLUDE_PRODUCT_OPTIONS_SECTION) {
					// Options selection is included in the listing
					if ($listingItem instanceof scProductVariant) {
						$product = $listingItem->getProductSL();
						$productVariantOptions = $listingItem->getSelectedOptionsArray();
					} else {
						$product = $listingItem;
						$productVariantOptions = array();
					}
					
					$hasOptionsSection = scProductService::getProductOptionsSection($product, $productVariantOptions);
					
					$hasOptionsSections[] = $hasOptionsSection;
					$addToCartSections[] = new miTemplateParserSectionInfo('addToCart', $listingItem->isOutOfStock() ? 0 : 1);
					$outOfStockSections[] = new miTemplateParserSectionInfo('outOfStock', $listingItem->isOutOfStock() ? 1 : 0);
					$viewDetailsSections[] = new miTemplateParserSectionInfo('viewDetails', $listingItem->isOutOfStock() ? 1 : 0);
					continue;
				}
				
				list($addToCartSections[], $outOfStockSections[], $viewDetailsSections[]) = $this->getCartRelatedSections($listingItem, $productsHavingOptions, $settings);
			}
			
			$productsSection->addSubsectionsArray('hasOptions', $hasOptionsSections);
			$productsSection->addSubsectionsArray('addToCart', $addToCartSections);
			$productsSection->addSubsectionsArray('outOfStock', $outOfStockSections);
			$productsSection->addSubsectionsArray('viewDetails', $viewDetailsSections);
		}
		
		/**
		 * @return array(addToCart, outOfStock, viewDetails)
		 */
		protected function getCartRelatedSections(IProductListingItem $listingItem, $productsHavingOptions, $settings = 0)
		{
			if ($listingItem->isOutOfStock()) {
				// The item is out of stock
				return array(
					new miTemplateParserSectionInfo('addToCart', 0),
					new miTemplateParserSectionInfo('outOfStock', 1),
					new miTemplateParserSectionInfo('viewDetails', 0),
				);
			}
			if (($settings & self::OPTION_INCLUDE_PRODUCT_OPTIONS_SECTION) || !in_array($listingItem->getProductId(), $productsHavingOptions)) {
				// The item is in stock and has no options or the item has options and a selection should be included
				return array(
					new miTemplateParserSectionInfo('addToCart', 1),
					new miTemplateParserSectionInfo('outOfStock', 0),
					new miTemplateParserSectionInfo('viewDetails', 0),
				);
			}
			// The item is in stock and has options
			return array(
				new miTemplateParserSectionInfo('addToCart', 0),
				new miTemplateParserSectionInfo('outOfStock', 0),
				new miTemplateParserSectionInfo('viewDetails', 1),
			);
		}
		
		protected function getProductsHavingOptions($productIds, $bundleProductIds = null)
		{
			if (empty($productIds)) {
				return array();
			}
			
			$productIds = array_map('intval', $productIds);
			$sql = 'SELECT ProductID FROM ProductOptionGroups WHERE ProductID IN (' . implode(', ', $productIds) . ')
				UNION DISTINCT
				SELECT ProductID FROM LocalOptionGroups WHERE ProductID IN (' . implode(', ', $productIds) . ')';
			$havingOptions = miUtil::getDBArray($sql, 'ProductID', 'ProductID');
			$notHaving = array_diff($productIds, $havingOptions);
			if (empty($notHaving)) {
				return $havingOptions;
			}
			
			if (!is_null($bundleProductIds) && empty($bundleProductIds)) {
				return $havingOptions;
			}
			
			return array_merge($havingOptions, $bundleProductIds);
			// Currently all bundles are considered products with options.
			// TODO: Check if the sub items have options.
		}
		
		/**
		 * @param IProductListingItem[] $listingItems
		 */
		protected function getPriceModifications($listingItems)
		{
			return scProductPrice::getPriceModifications($listingItems);
		}
		
		/**
		 * @param scMultiCurrencyAmount $productPrice
		 * @param array $productPriceModifications
		 * @param int $productId
		 */
		protected function getPriceSection(IProductListingItem $listingItem, $productPriceModifications)
		{
			return scProductPrice::getPriceSection($listingItem, $productPriceModifications);
		}
	}
?>