<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * This is the product price calculator
	 */
	class scProductPrice {
		
		protected static $_productsTaxClasses = array();
		protected static $_productsTaxedPrices = array();
		protected static $_taxZones = array();
		protected static $_taxRates = null;
		
		/**
		 * The available price modification modules
		 * 
		 * @var array $_priceModificationModules
		 */
		protected static $_priceModificationModules = null;
		
		/**
		 * The computed final price, including currency
		 * @var string
		 */
		public static $_finalPrice;
		
		/**
		 * The computed final taxed price, including currency
		 * @var string
		 */
		public static $_finalTaxedPrice;
		
		/**
		 * Prevents object creation
		 */
		protected function __construct()
		{
		}
		
		/**
		 * Returns the product price section - used to display it in the user interface
		 * The second parameter can be retrieved by getSingleProductPriceModifications() method
		 * 
		 * @param scMultiCurrencyAmount $productPrice
		 * @param array $productPriceModifications
		 * @return miTemplateParserSectionInfo
		 */
		public static function getPriceSection(IProductListingItem $listingItem, $productPriceModifications)
		{
			// Name all sections to be available usage of addSubsectionsArray
			$priceSection = new miTemplateParserSectionInfo('price');
			
			$listAmount = $listingItem->getPrice()->getAmount();
			$wholesaleAmount = scWholesalePrices::getListingItemWholesalePrice($listingItem)->getAmount();
			if ((!self::showOriginalPriceSection($productPriceModifications)) || !$listAmount->equals($wholesaleAmount)) {
				$priceSection->addSubsection(self::getModifiedPriceSection($listingItem, $productPriceModifications));
			} else {
				$priceSection->addSubsection(self::getOriginalPriceSection($listingItem, count($productPriceModifications)));
			}
			
			$priceSection->addSubsection(self::getPriceModificationsSection($listingItem, $productPriceModifications));
			return $priceSection;
		}
		
		/**
		 * Returns array with price modifications
		 *
		 * @param array $products
		 * @return array array($productId => array(scProductPriceModification objects), ...)
		 */
		public static function getPriceModifications($products)
		{
			if (count($products) == 0)
				return array();
			
			self::initPriceModules();
			
			$allPriceModifications = array();
			foreach (self::$_priceModificationModules as $module) {
				$priceModifications = $module->getPriceModifications($products);
				foreach ($priceModifications as $priceModification) {
					if (!($priceModification instanceof scProductPriceModification))
						throw new miException('IProductPriceModifier::getPriceModifications must return scProductPriceModification objects.');
					
					$productId = $priceModification->getProductId();
					if (isset($allPriceModifications[$productId]))
						$allPriceModifications[$productId][] = $priceModification;
					else
						$allPriceModifications[$productId] = array($priceModification);
				}
			}
			return $allPriceModifications;
		}
		
		/**
		 * Returns the price rows used to construct the price sections
		 * 
		 * @param IProductListingItem $listingItem
		 * @return array array(scProductPriceModification objects)
		 */
		public static function getSingleProductPriceModifications(IProductListingItem $listingItem)
		{
			self::initPriceModules();
			
			$allPriceModifications = array();
			foreach (self::$_priceModificationModules as $module) {
				$priceModifications = $module->getPriceModifications(array($listingItem));
				foreach ($priceModifications as $priceModification) {
					if (!($priceModification instanceof scProductPriceModification))
						throw new miException('IProductPriceModifier::getPriceModifications must return scProductPriceModification objects.');
					$allPriceModifications[] = $priceModification;
				}
			}
			return $allPriceModifications;
		}
		
		/**
		 * Returns all price modifications for a cart item
		 *
		 * @param scCartItem $cartItem
		 */
		public static function getCartItemPriceModifications(scCartItem $cartItem)
		{
			self::initPriceModules();
			
			$allPriceModifications = array();
			foreach (self::$_priceModificationModules as $module) {
				$priceModifications = $module->getCartPriceModifications($cartItem);
				foreach ($priceModifications as $priceModification) {
					if (!($priceModification instanceof scProductPriceModification))
						throw new miException('IProductPriceModifier::getPriceModifications must return scProductPriceModification objects.');
					$allPriceModifications[] = $priceModification;
				}
			}
			return $allPriceModifications;
		}
		
		/**
		 * Initializes the price modules array, call it first in every public function in the class
		 * 
		 * @return void
		 */
		protected static function initPriceModules()
		{
			if (self::$_priceModificationModules !== null)
				return;
			
			try {
				$priceModificationModules = scModulesFactory::getImplementations('IProductPriceModifier');
				// Use a specific order:
				// First add the Wholesale module.
				// Make the Discounts module second.
				// And after them add all other modules in the order they were returned from DB.
				self::$_priceModificationModules = array(0 => null, 1 => null);
				foreach ($priceModificationModules as $module) {
					if ($module instanceof scProductPriceWholesalePrices) {
						self::$_priceModificationModules[0] = $module;
						continue;
					}
					if ($module instanceof scProductPriceDiscount) {
						self::$_priceModificationModules[1] = $module;
						continue;
					}
					self::$_priceModificationModules[] = $module;
				}
			} catch (miException $exception) {
				scLog::singleton()->error($exception->getMessage(), scLog::CATEGORY_NONE);
				self::$_priceModificationModules = array();
			}
		}
		
		/**
		 * Returns section with the price not modified - used when no modifiers are found
		 * 
		 * @param scMultiCurrencyAmount $productPrice
		 * @return miTemplateParserSectionInfo
		 */
		public static function getOriginalPriceSection(IProductListingItem $listingItem, $productPriceModificationsCount = 0)
		{
			$priceSection = new miTemplateParserSectionInfo('productPrice');
			$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
			$productPrice = $listingItem->getPrice()->getAmount();
			$productId = $listingItem->getProductId();
			if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				// Apply taxes
				$productPrice = self::getTaxedAmount($productPrice, $productId);
			}
			$price = scPrice::factory($productPrice);
			self::$_finalPrice = $price->getDisplayPrice();
			$listPriceElements = array();
			$listPriceElements['%%FINAL_PRICE%%'] = self::$_finalPrice;
			$listPriceElements['%%FINAL_PRICE_FLOAT%%'] = $price->getCurrentCurrencyAmount();
			
			$listPriceSection = new miTemplateParserSectionInfo('listPrice', 0);
			$listTaxedPriceSection = new miTemplateParserSectionInfo('listTaxedPrice', 0);
			
			if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				$listPriceSection->setIterationsCount(1);
				$listPriceSection->addValues($listPriceElements);
				
			} else if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
				// Add subsection for taxed price
				$taxedProductPrice = self::getTaxedAmount($productPrice, $productId);
				$taxedPrice = scPrice::factory($taxedProductPrice);
				self::$_finalTaxedPrice = $taxedPrice->getDisplayPrice();
				$listPriceElements['%%FINAL_TAXED_PRICE%%'] = self::$_finalTaxedPrice;
				$listPriceElements['%%FINAL_TAXED_PRICE_FLOAT%%'] = $taxedPrice->getCurrentCurrencyAmount();
				
				$listTaxedPriceSection->setIterationsCount(1);
				$listTaxedPriceSection->addValues($listPriceElements);
			}
			
			if ($productPriceModificationsCount > 0) {
				$hasPriceModificationsSection = new miTemplateParserSectionInfo('hasPriceModifications', 1);
				$listPriceSection->addSubsection($hasPriceModificationsSection);
				$listTaxedPriceSection->addSubsection($hasPriceModificationsSection);
			}
			$priceSection->addSubsection($listPriceSection);
			$priceSection->addSubsection($listTaxedPriceSection);
			
			return $priceSection;
		}
		
		/**
		 * Retrieves the wholesale price for a variant for Quantity = 1.
		 * @return float or false if there is no wholesale price for the variant
		 * @todo: The value may be saved as the ProductVariantPrice so reading is not done each time it is requested.
		 */
		public static function getProductVariantWholesalePrice($productId, $productVariantId, $customerGroupId)
		{
			static $cache = array();
			
			if (!isset($cache[$productVariantId])) {
				$sql = 'SELECT CustomerGroupID, ProductWholesalePriceAmount FROM ProductWholesalePrices WHERE ProductVariantID = ? AND ProductWholesalePriceQuantity = 1';
				$result = miStaticDBUtil::execSelect($sql, array($productVariantId));
				$cache[$productVariantId] = array();
				foreach ($result as $row) {
					$cache[$productVariantId][$row['CustomerGroupID']] = $row['ProductWholesalePriceAmount'];
				}
			}
			
			if (isset($cache[$productVariantId][$customerGroupId])) {
				return $cache[$productVariantId][$customerGroupId];
			}
			
			return self::getProductWholesalePrice($productId, $customerGroupId);
		}
		
		protected static function getProductWholesalePrice($productId, $customerGroupId)
		{
			static $cache = array();
			
			if (!isset($cache[$productId])) {
				$sql = 'SELECT CustomerGroupID, ProductWholesalePriceAmount FROM ProductWholesalePrices WHERE ProductID = ? AND ProductWholesalePriceQuantity = 1';
				$result = miStaticDBUtil::execSelect($sql, array($productId));
				$cache[$productId] = array();
				foreach ($result as $row) {
					$cache[$productId][$row['CustomerGroupID']] = $row['ProductWholesalePriceAmount'];
				}
			}
			
			if (isset($cache[$productId][$customerGroupId])) {
				return $cache[$productId][$customerGroupId];
			}
			
			return false;
		}
		
		/**
		 * Returns the price section and appropriate subsections with modifiers applyed
		 * 
		 * @param scMultiCurrencyAmount $productPriceAmount
		 * @param array $productPriceModifications array of scProductPriceModification objects
		 * @return miTemplateParserSectionInfo
		 */
		protected static function getModifiedPriceSection(IProductListingItem $listingItem, $productPriceModifications)
		{
			$priceSection = new miTemplateParserSectionInfo('productPriceModified');
			
			$productId = $listingItem->getProductId();
			$listAmount = $listingItem->getPrice()->getAmount();
			$wholesaleAmount = scWholesalePrices::getListingItemWholesalePrice($listingItem)->getAmount();
			
			// Prepare the main section elements - the original product price
			$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
			if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				// Apply taxes
				$listAmount = self::getTaxedAmount($listAmount, $productId);
				$wholesaleAmount = self::getTaxedAmount($wholesaleAmount, $productId);
			}
			$listPrice = scPrice::factory($listAmount);
			
			$mainElements = array('%%LIST_PRICE%%' => $listPrice->getDisplayPrice(), '%%LIST_PRICE_FLOAT%%' => $listPrice->getCurrentCurrencyAmount());
			$priceSection->setValues($mainElements);
			
			$noWholesaleSection = new miTemplateParserSectionInfo('noWholesalePrice', 1);
			$hasWholesaleSection = new miTemplateParserSectionInfo('hasWholesalePrice', 0);
			if (!$listAmount->equals($wholesaleAmount)) {
				$noWholesaleSection->setIterationsCount(0);
				$hasWholesaleSection->setIterationsCount(1);
				$wholesalePrice = scPrice::factory($wholesaleAmount);
				$wholesaleDiscountAmount = scMultiCurrencyAmountMath::subtract($listAmount, $wholesaleAmount);
				$wholesaleDiscountPrice = scPrice::factory($wholesaleDiscountAmount);
				$hasWholesaleSection->setValues(array(
					'%%WHOLESALE_PRICE%%' => $wholesalePrice->getDisplayPrice(),
					'%%WHOLESALE_DISCOUNT%%' => $wholesaleDiscountPrice->getDisplayPrice(),
					'%%WHOLESALE_DISCOUNT_PERCENT%%' => number_format(abs(($wholesaleDiscountAmount->get()) * 100) / $listAmount->get(), 2),
					'%%WHOLESALE_PRICE_FLOAT%%' => $wholesalePrice->getCurrentCurrencyAmount()
				));
			}
			$priceSection->addSubsection($noWholesaleSection);
			$priceSection->addSubsection($hasWholesaleSection);
			
			// Initialize to basic product price
			$finalPriceAmount = clone $wholesaleAmount;
			
			// Apply the price modifiers
			foreach ($productPriceModifications as $productPriceModification) {
				if ($productPriceModification->getDisplayMode() == scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE)
					continue;
				
				$modificationAmount = $productPriceModification->getAmount();
				$modificationPercent = $productPriceModification->getPercent();
				if (scMultiCurrencyAmountMath::add($finalPriceAmount, $productPriceModification->getAmount())->get() < 0) {
					$modificationAmount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $finalPriceAmount);
				}
				$finalPriceAmount = scMultiCurrencyAmountMath::add($finalPriceAmount, $modificationAmount);
			}
			
			// Don't allow negative prices
			if ($finalPriceAmount->get() < 0)
				return self::getOriginalPriceSection($listingItem);
			
			// Check general setting display prices mode and add the modified price to the section
			$displayPricesTaxMode = scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE);
			self::$_finalPrice = scPrice::factory($finalPriceAmount)->getDisplayPrice();
			$salePriceElements = array();
			$salePriceElements['%%FINAL_PRICE%%'] = self::$_finalPrice;
			$salePriceElements['%%FINAL_PRICE_FLOAT%%'] = scPrice::factory($finalPriceAmount)->getCurrentCurrencyAmount();
			if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				$priceSection->addSubsection(new miTemplateParserSectionInfo('salePrice', 1, $salePriceElements));
				$priceSection->addSubsection(new miTemplateParserSectionInfo('saleTaxedPrice', 0));
			} else if ($displayPricesTaxMode == scTax::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX) {
				// Add subsection for final taxed price
				$finalTaxedPriceAmount = self::getTaxedAmount($finalPriceAmount, $productId);
				self::$_finalTaxedPrice = scPrice::factory($finalTaxedPriceAmount)->getDisplayPrice();
				$salePriceElements['%%FINAL_TAXED_PRICE%%'] = self::$_finalTaxedPrice;
				$salePriceElements['%%FINAL_TAXED_PRICE_FLOAT%%'] = scPrice::factory($finalTaxedPriceAmount)->getCurrentCurrencyAmount();
				$priceSection->addSubsection(new miTemplateParserSectionInfo('salePrice', 0));
				$priceSection->addSubsection(new miTemplateParserSectionInfo('saleTaxedPrice', 1, $salePriceElements));
			}
			
			return $priceSection;
		}
		
		/**
		 * Returns scMultiCurrencyAmount $taxedPriceAmount
		 * @param scMultiCurrencyAmount $priceAmount
		 * @param $productId
		 * @return scMultiCurrencyAmount taxed price
		 */
		public static function getTaxedAmount(scMultiCurrencyAmount $priceAmount, $productId)
		{
			$hash = md5($priceAmount->get());
			if (!empty(self::$_productsTaxedPrices[$productId][$hash]))
				return self::$_productsTaxedPrices[$productId][$hash];
			
			if (empty(self::$_productsTaxedPrices[$productId]))
				self::$_productsTaxedPrices[$productId] = array();
			
			$taxClasses = scTax::getProductTaxClasses($productId);
			if (empty($taxClasses)) {
				self::$_productsTaxedPrices[$productId][$hash] = $priceAmount;
				return $priceAmount;
			}
			
			// Initialize to basic product price
			$taxedPriceAmount = clone $priceAmount;
			
			$productPrice = $priceAmount->get();
			foreach ($taxClasses as $taxClassId) {
				$taxInfo = scTax::getTaxInfoByTaxClassId($taxClassId); // this is cached in scTax class
				
				// Check if tax is not includable with display prices
				if (empty($taxInfo['TaxIncludeWithDisplayPrice']))
					continue;
				
				$taxZoneId = self::getTaxZoneId($taxInfo['TaxID']);
				$taxRate = self::getTaxRate($taxZoneId, $taxClassId);
				$modificationAmount = scMultiCurrencyAmountMath::multiply($priceAmount, ($taxRate / 100));
				$taxedPriceAmount = scMultiCurrencyAmountMath::add($taxedPriceAmount, $modificationAmount);
			}
			
			self::$_productsTaxedPrices[$productId][$hash] = $taxedPriceAmount;
			
			return $taxedPriceAmount;
		}
		
		public static function getProductPriceFieldForQuery($fieldName)
		{
			static $taxRate;
			
			if (is_null($taxRate)) {
				$taxRate = self::getFirstApplicableTaxRate();
			}
			
			if ($taxRate > 0) {
				$fieldName = '(' . $fieldName . ' * ' . (($taxRate + 100) / 100) . ')';
			}
			return $fieldName;
		}
		
		protected static function getFirstApplicableTaxRate()
		{
			$customerAddress = scCustomers::getCurrentCustomerShippingAddress();
			
			// Check if the states match
			$sql = 'SELECT TaxRate FROM TaxZoneStates 
					INNER JOIN TaxZones USING(TaxZoneID)
					INNER JOIN TaxRates USING(TaxZoneID)
					WHERE StateID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($customerAddress['CustomerShippingState']));
			if (!empty($rows)) {
				return $rows[0]['TaxRate'];
			}
			
			// Check if countries match
			$sql = 'SELECT TaxRate FROM TaxZoneCountries 
					INNER JOIN TaxZones USING(TaxZoneID)
					INNER JOIN TaxRates USING(TaxZoneID)
					WHERE CountryID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($customerAddress['CustomerShippingCountry']));
			if (!empty($rows)) {
				return $rows[0]['TaxRate'];
			}
			
			return 0;
		}
		
		/**
		 * Returns TaxZoneId, depends from CustomerAddress
		 * If customer is not logged, depends from GuestAddress
		 * @param $productId
		 */
		protected static function getTaxZoneId($taxId)
		{
			if (!isset(self::$_taxZones[$taxId])) {
				self::$_taxZones[$taxId] = scZones::getTaxZoneId($taxId, scCustomers::getCurrentCustomerShippingAddress());
			}
			return self::$_taxZones[$taxId];
		}
		
		/**
		 * Returns TaxRate by TaxZoneId and TaxClassId
		 * @param int $taxZoneId
		 * @param int $taxClassId
		 */
		protected static function getTaxRate($taxZoneId, $taxClassId)
		{
			if (is_null(self::$_taxRates)) {
				self::$_taxRates = array();
				$allRates = scTax::getAllTaxRates();
				foreach ($allRates as $rate) {
					if (!isset(self::$_taxRates[$rate['TaxZoneID']])) {
						self::$_taxRates[$rate['TaxZoneID']] = array();
					}
					self::$_taxRates[$rate['TaxZoneID']][$rate['TaxClassID']] = $rate['TaxRate'];
				}
			}
			if (!isset(self::$_taxRates[$taxZoneId][$taxClassId]))
				return 0;
			
			return self::$_taxRates[$taxZoneId][$taxClassId];
		}
		
		protected static function getPriceModificationsSection(IProductListingItem $listingItem, $productPriceModifications)
		{
			if (empty($productPriceModifications)) {
				return new miTemplateParserSectionInfo('hasPriceModifications', 0);
			}
			$hasPriceModificationsSection = new miTemplateParserSectionInfo('hasPriceModifications', 1);
			
			$wholesaleAmount = scWholesalePrices::getListingItemWholesalePrice($listingItem)->getAmount();
			
			// Apply taxes if necessary
			if (scSettings::getSettingValue(scSettings::SETTING_DISPLAY_PRICES_TAX_MODE) == scTax::DISPLAY_PRICES_TAX_MODE_WITH_TAX) {
				$wholesaleAmount = self::getTaxedAmount($wholesaleAmount, $listingItem->getProductId());
			}
			
			// Initialize to basic product price
			$finalPriceAmount = clone $wholesaleAmount;
			
			$priceModifiersDiscountSubsections = array();
			$priceModifiersProductPriceSubsections = array();
			
			// Apply the price modifiers
			foreach ($productPriceModifications as $productPriceModification) {
				if (!$productPriceModification->getIsVisible())
					continue;
				
				$displayMode = $productPriceModification->getDisplayMode();
				$modificationAmount = $productPriceModification->getAmount();
				$modificationPercent = $productPriceModification->getPercent();
				
				if (($displayMode == scProductPriceModification::DISPLAY_MODE_DISCOUNT) &&
					(scMultiCurrencyAmountMath::add($finalPriceAmount, $productPriceModification->getAmount())->get() < 0)) {
					// Calculate the modification amount using on the final price amount
					$modificationAmount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $finalPriceAmount);
					$finalPriceAmount = scMultiCurrencyAmountMath::add($finalPriceAmount, $modificationAmount);
				}
				
				if (($displayMode == scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE) &&
					(scMultiCurrencyAmountMath::add($wholesaleAmount, $productPriceModification->getAmount())->get() <= 0)) {
					// Calculate the modification amount using the unmodified value in the $wholesaleAmount variable
					$modificationAmount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $wholesaleAmount);
				}
				
				$modifiedPriceAmount = scMultiCurrencyAmount::getZero();
				if ($displayMode == scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE) {
					$modifiedPriceAmount = scMultiCurrencyAmountMath::add($wholesaleAmount, $modificationAmount);
				}
				
				$values = array(
					'Description' => $productPriceModification->getDescription(),
					'Amount' => scPrice::factory(scMultiCurrencyAmountMath::abs($modificationAmount))->getDisplayPrice(),
					'Percent' => number_format(abs($modificationPercent), 2),
					'ModifiedPrice' => scPrice::factory(scMultiCurrencyAmountMath::abs($modifiedPriceAmount))->getDisplayPrice(),
				);
				
				if ($productPriceModification->getDisplayMode() == scProductPriceModification::DISPLAY_MODE_DISCOUNT) {
					$priceModifiersDiscountSubsections[] = new miTemplateParserSectionInfo('displayModeDiscount', 1, miUIUtil::templetizeRows($values));
					$priceModifiersProductPriceSubsections[] = new miTemplateParserSectionInfo('displayModeProductPrice', 0);
					
				} else if ($productPriceModification->getDisplayMode() == scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE) {
					$priceModifiersDiscountSubsections[] = new miTemplateParserSectionInfo('displayModeDiscount', 0);
					$priceModifiersProductPriceSubsections[] = new miTemplateParserSectionInfo('displayModeProductPrice', 1, miUIUtil::templetizeRows($values));
				}
			}
			
			if ($finalPriceAmount->get() < 0) {
				return new miTemplateParserSectionInfo('hasPriceModifications', 0);
			}
			
			$priceModifiersSection = new miTemplateParserSectionInfo('priceModifiers');
			$priceModifiersSection->setIterationsCount(count($priceModifiersDiscountSubsections));
			$priceModifiersSection->addSubsectionsArray('displayModeDiscount', $priceModifiersDiscountSubsections);
			$priceModifiersSection->addSubsectionsArray('displayModeProductPrice', $priceModifiersProductPriceSubsections);
			
			$hasPriceModificationsSection->addSubsection($priceModifiersSection);
			return $hasPriceModificationsSection;
		}
		
		/**
		 * This function determines whether we should show the original price section or the modified price section
		 *
		 * @param array $productPriceModifications
		 * @return bool
		 */
		protected static function showOriginalPriceSection($productPriceModifications)
		{
			foreach ($productPriceModifications as $productPriceModification) {
				if ($productPriceModification->getDisplayMode() == scProductPriceModification::DISPLAY_MODE_DISCOUNT) {
					return false;
				}
			}
			return true;
		}
	}
?>