<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Encapsulates the product functionality (single language use)
	 */
	class scProductSL extends miDefaultDomainObject implements IStockable, IProductListingItem {
		
		protected $_languageId;
		
		/**
		 * @var scProductDomainObject
		 */
		protected $_domainObject = null;
		
		/**
		 * @var scProductImageInfo[]
		 */
		protected $_productImages = null;
		
		protected $_options = null;
		
		/**
		 * @var scPrice
		 */
		protected $_price = null;
		
		/**
		 * Constructs the product object
		 * 
		 * @return void
		 */
		public function __construct()
		{
			$this->_record = new miSqlRecord('Products', 'ProductID');
			$this->_languageId = scMultiLanguage::getCurrentLanguageId();
		}
		
		/**
		 * Set the language id to use
		 * If not set the current language is used
		 * 
		 * @param int $languageId
		 */
		public function setLanguageId($languageId)
		{
			$this->_languageId = $languageId;
		}
		
		/**
		 * Reads the product information from the database
		 * 
		 * @param int $productId
		 * @return void
		 */
		public function read($productId)
		{
			static $cache = array();
			if (!isset($cache[$productId])) {
				$sql = 'SELECT *, Products.ProductID as ProductID FROM Products JOIN ProductsML USING(ProductID) WHERE Products.ProductID=? AND LanguageID=?';
				$rows = miStaticDBUtil::execSelect($sql, array($productId, $this->_languageId));
				if (empty($rows))
					throw new miDBException(_t('Cannot find product.'), miDBException::EXCEPTION_RECORD_NOT_FOUND);
				$cache[$productId] = $rows;
			}
			
			$this->_record->setRow($cache[$productId][0]);
		}
		
		public function getPK()
		{
			return $this->get($this->getPKName());
		}
		
		public function getDescription()
		{
			return ($this->get('ProductDescriptionXHTML') ? $this->get('ProductDescription') : miI18N::htmlEscape($this->get('ProductDescription')));
		}
		
		public function getDetailedDescription()
		{
			return $this->get('ProductDetailedDescriptionXHTML') ? $this->get('ProductDetailedDescription') : miI18N::htmlEscape($this->get('ProductDetailedDescription'));
		}
		
		public function getName()
		{
			return $this->get('ProductName');
		}
		
		public function getUrl()
		{
			return scFriendlyUrl::get(scFriendlyUrl::TYPE_PRODUCT, $this->get('ProductID'), $this->get('ProductName'));
		}
		
		public function getCategoryId()
		{
			return (int)$this->get('CategoryID');
		}
		
		public function getAllCategoryIds()
		{
			return scProductsXCategories::getProductCategories($this->getProductId());
		}
		
		public function getProductClassId()
		{
			return (int)$this->get('ProductClassID');
		}
		
		/**
		 * Returns the product id
		 * 
		 * @return int
		 */
		public function getProductId()
		{
			return (int)$this->get('ProductID');
		}
		
		/**
		 * Return the product price
		 * 
		 * @return scPrice
		 */
		public function getPrice()
		{
			if (is_null($this->_price)) {
				$amount = scMultiCurrencyAmount::factoryBaseCurrency($this->get('ProductPrice'));
				// Add the amount in the current currency so they are both set in the object and they are correctly used in calculations.
				$amount->set(scCurrencyUtil::getCurrentCurrency(), scCurrencyUtil::getCurrentCurrencyAmount($this->get('ProductPrice')));
				
				$currenciesWithFixedPrices = scCurrencyUtil::getCurrenciesWithFixedPrices();
				// Return the price if there are no currencies with fixed prices OR the current product has wholesale prices
				if (empty($currenciesWithFixedPrices) || scWholesalePrices::productHasWholesalePrice($this->getProductId())) {
					$this->_price = scPrice::factory($amount);
					return $this->_price;
				}
				
				// Set the prices for all fixed currencies
				$productData = $this->getRow();
				foreach ($currenciesWithFixedPrices as $currency) {
					$currencyCode = $currency['CurrencyCode'];
					if ($productData['ProductPrice' . $currencyCode . 'Auto']) {
						continue; // no need to set the price if the price mode for this currency is auto
					}
					
					$productPrice = $productData['ProductPrice' . $currencyCode];
					$amount->set(new scCurrency($currencyCode), $productPrice);
				}
				$this->_price = scPrice::factory($amount);
			}
			return $this->_price;
		}
		
		/**
		 * NOTE: No multilanguage data is being set. insert()ing or update()ing is not recommended.
		 * @return scProductDomainObject
		 */
		public function getProductDomainObject()
		{
			if (is_null($this->_domainObject)) {
				$this->_domainObject = new scNoExecProductDomainObject();
				$this->_domainObject->setRow($this->getRow());
			}
			return $this->_domainObject;
		}
		
		public function getQuantityInStock()
		{
			return $this->get('ProductQuantity');
		}
		
		public function reduceQuantityInStock($itemsRemoved)
		{
			$this->getProductDomainObject()->reduceQuantityInStock($itemsRemoved);
			$this->set('ProductQuantity', $this->getProductDomainObject()->get('ProductQuantity'));
		}
		
		public function setQuantityInStock($newQuantityInStock)
		{
			$this->getProductDomainObject()->setQuantityInStock($newQuantityInStock);
			$this->set('ProductQuantity', $this->getProductDomainObject()->get('ProductQuantity'));
		}
		
		public function isOutOfStock($quantity = 1)
		{
			if ((int)$this->getQuantityInStock() >= (int)$quantity)
				return false;
			
			$outOfStockHandling = scSettings::getSettingValue(scSettings::SETTING_OUT_OF_STOCK_HANDLING);
			if (($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_TAG) or ($outOfStockHandling == scProductService::OUT_OF_STOCK_HANDLING_HIDE))
				return true;
			return false;
		}
		
		/**
		 * @return scProductImageInfo
		 */
		public function getImageInfo()
		{
			$productInfo = $this->getRow();
			
			// If we don't have any image info - read it
			if (!isset($productInfo['ProductImage']) and !isset($productInfo['ProductImageThumb'])) {
				// If the product has variants and the default one has an image use it - check only if the Variant info is not already available
				if (!array_key_exists('ProductVariantImage', $productInfo) and $this->getProductDomainObject()->hasVariants()) {
				//if ($this->getProductDomainObject()->hasVariants()) {
					$defaultVariant = $this->getProductDomainObject()->getDefaultVariant();
					if ($defaultVariant->hasImage()) {
						return $defaultVariant->getImageInfo();
					}
				}
				
				return scProductImage::getImage($productInfo['ProductID']);
			}
			
			$imageData = $this->getProductImageInfo($productInfo);
			
			if (empty($imageData['ProductImage']) and empty($imageData['ProductImageThumb'])) {
				return new scProductNoImageImageInfo();
			}
			
			$imageInfo = new scProductImageInfo($imageData['ProductImage'], $imageData['ProductImageWidth'], $imageData['ProductImageHeight'], $imageData['ProductImageSize']);
			$imageInfo->setLargeImage(new scImageInfo($imageData['ProductImageLarge'], $imageData['ProductImageLargeWidth'], $imageData['ProductImageLargeHeight'], $imageData['ProductImageLargeSize']));
			$imageInfo->setThumbnail(new scImageInfo($imageData['ProductImageThumb'], $imageData['ProductImageThumbWidth'], $imageData['ProductImageThumbHeight'], $imageData['ProductImageThumbSize']));
			$imageInfo->setBoxImage(new scImageInfo($imageData['ProductImageBox'], $imageData['ProductImageBoxWidth'], $imageData['ProductImageBoxHeight'], $imageData['ProductImageBoxSize']));
			$imageInfo->setIsDefault($imageData['ProductImageIsDefault']);
			$imageInfo->setProductImageId($imageData['ProductImageID']);
			
			return $imageInfo;
		}
		
		public function getCreateTimestamp()
		{
			return $this->get('ProductCreatedTimestamp');
		}
		
		public function getSkuCode()
		{
			return $this->get('ProductCode');
		}
		
		/**
		 * @return scWeight
		 */
		public function getWeight()
		{
			return new scWeight($this->get('ProductWeight'), scSettings::getSettingValue(scSettings::SETTING_LOCALIZATION_WEIGHT_UNIT));
		}
		
		/**
		 * @return int or null
		 */
		public function getManufacturerId()
		{
			return $this->getProductDomainObject()->getManufacturerId();
		}
		
		/**
		 * @return scManufacturerDomainObject or null
		 */
		public function getManufacturer()
		{
			return $this->getProductDomainObject()->getManufacturer();
		}
		
		/**
		 * @return bool
		 */
		public function isEgood()
		{
			return $this->getProductDomainObject()->isEgood();
		}
		
		public function checkBundleRecursion($unwantedIds)
		{
			return $this->getProductDomainObject()->checkBundleRecursion($unwantedIds);
		}
		
		public function getBundleProducts()
		{
			return $this->getProductDomainObject()->getBundleProducts();
		}
		
		public function retrieveBundleData()
		{
			return $this->getProductDomainObject()->retrieveBundleData();
		}
		
		/**
		 * @return scProductImageInfo[]
		 */
		public function getImages()
		{
			if (!is_null($this->_productImages)) {
				return $this->_productImages;
			}
			
			$this->_productImages = array();
			// Load the product images information (multiple non-existent images will be filtered)
			$sql = 'SELECT * FROM ProductImages WHERE ProductID = ? ORDER BY ProductImageIsDefault DESC';
			$productImages = miStaticDBUtil::execSelect($sql, array($this->getProductId()));
			// Get the image files locations
			if (count($productImages) > 0) {
				// Get the first image no matter what
				$this->_productImages[] = scProductImage::getImageInfo(array_shift($productImages));
				
				// Filter the remaining images if they are non-existent
				foreach ($productImages as $imageArray) {
					$image = scProductImage::getImageInfo($imageArray);
					if (!$image->exists()) {
						continue;
					}
					
					$this->_productImages[] = $image;
				}
			}
			
			return $this->_productImages;
		}
		
		public function getOptionsData()
		{
			if (is_null($this->_options)) {
				$this->_options = array();
				$options = scProductService::getProductsOptions(array($this->getProductId()));
				if (isset($options[$this->getProductId()])) {
					$this->_options = $options[$this->getProductId()];
				}
				if ($this->get('ProductIsBundle')) {
					$this->_options = array_merge_recursive($this->_options, $this->getBundleProductsOptions());
				}
			}
			
			return $this->_options;
		}
		
		protected function getBundleProductsOptions()
		{
			if (!$this->get('ProductIsBundle')) {
				return array();
			}
			
			$bundledProductsOptions = array();
			$bundledProducts = $this->getBundleProducts();
			foreach ($bundledProducts as $bundledProduct) {
				$subProductOptions = $bundledProduct->getOptionsData();
				foreach ($subProductOptions as $optionGroupType => $optionGroups) {
					foreach ($optionGroups as $optionGroupId => $optionGroup) {
						if (!$bundledProduct->get('ProductIsBundle')) {
							$optionGroup['OptionGroupName'] = $bundledProduct->getName() . ' - ' . $optionGroup['OptionGroupName'];
						}
						$bundledProductsOptions[$optionGroupType][$bundledProduct->getProductId() . '-' . $optionGroupId] = $optionGroup;
					}
				}
			}
			
			return $bundledProductsOptions;
		}
		
		protected function getProductImageInfo($productInfo)
		{
			// Some the fields might be missing - put an empty string for them
			$imageInfoFields = array('ProductImageID', 'ProductID', 'ProductImage', 'ProductImageWidth',
				'ProductImageHeight', 'ProductImageSize', 'ProductImageThumb', 'ProductImageThumbWidth',
				'ProductImageThumbHeight', 'ProductImageThumbSize', 'ProductImageBox', 'ProductImageBoxWidth',
				'ProductImageBoxHeight', 'ProductImageBoxSize', 'ProductImageIsDefault', 'ProductImageExists',
				'ProductImageLarge', 'ProductImageLargeWidth', 'ProductImageLargeHeight', 'ProductImageLargeSize');
			
			$imageData = array();
			foreach ($imageInfoFields as $field) {
				$imageData[$field] = '';
				if (isset($productInfo[$field])) {
					$imageData[$field] = $productInfo[$field];
				}
			}
			return $imageData;
		}
		
		public function update()
		{
			return $this->getProductDomainObject()->update();
		}
		
		public function insert()
		{
			return $this->getProductDomainObject()->insert();
		}
		
		public function delete($pkValue = null)
		{
			return $this->getProductDomainObject()->delete($pkValue);
		}
	}
	
	class scNoExecProductDomainObject extends scProductDomainObject {
		
		public function update()
		{
			throw new miException(_i('Not implemented. Use scProductDomainObject instead.'));
		}
		
		public function insert()
		{
			throw new miException(_i('Not implemented. Use scProductDomainObject instead.'));
		}
		
		public function delete($pkValue = null)
		{
			throw new miException(_i('Not implemented. Use scProductDomainObject instead.'));
		}
	}
?>