<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSearchData {
		
		// Simple search fields
		const KEY_PHRASE = 'phrase';
		
		// Advanced search fields
		const KEY_PRODUCT_NAME = 'phrase';
		const KEY_PRODUCT_DETAILS = 'ProductDetails';
		const KEY_PRODUCT_CODE = 'ProductCode';
		const KEY_CATEGORY_ID = 'CategoryID';
		const KEY_SUBCATEGORY_SEARCH = 'SearchInSubcategory';
		const KEY_MANUFACTURER_ID = 'ManufacturerID';
		const KEY_PRODUCT_PRICE_FROM = 'ProductPriceFrom';
		const KEY_PRODUCT_PRICE_TO = 'ProductPriceTo';
		
		public static function getSearchData()
		{
			return array(
				self::KEY_PHRASE => miGetParamDefault('phrase', '')
			);
		}
		
		public static function getAdvancedSearchData()
		{
			$data = array(
				self::KEY_PRODUCT_NAME => miGetParamDefault('phrase', ''),
				self::KEY_PRODUCT_DETAILS => miGetParamDefault('ProductDetails', 1),
				self::KEY_PRODUCT_CODE => miGetParamDefault('ProductCode', ''),
				self::KEY_CATEGORY_ID => miGetParamDefault('CategoryID', 0),
				self::KEY_SUBCATEGORY_SEARCH => miGetParamDefault('SearchInSubcategory', 1),
				self::KEY_MANUFACTURER_ID => miGetParamDefault('ManufacturerID', 0),
				self::KEY_PRODUCT_PRICE_FROM => miGetParamDefault('ProductPriceFrom', ''),
				self::KEY_PRODUCT_PRICE_TO => miGetParamDefault('ProductPriceTo', '')
			);
			
			// Normalize checkbox data
			$data[self::KEY_PRODUCT_DETAILS] = ($data[self::KEY_PRODUCT_DETAILS]) ? 1 : 0;
			$data[self::KEY_SUBCATEGORY_SEARCH] = ($data[self::KEY_SUBCATEGORY_SEARCH]) ? 1 : 0;

			return $data;
		}
	}
	
	/**
	 * Gives the basic functionality for extensible search fields
	 */
	abstract class scSearchEngine {
		const FIELD_CHECKED_KEY = 'fieldchecked';
		const FIELD_NAME_KEY = 'fieldname';
		const FIELD_ID = 'fieldid';
		
		/**
		 * All possible search fields
		 * 
		 * @var array $_fields
		 */
		protected $_fields = array();
			
		/**
		 * The search fields text description
		 * 
		 * @var array $_fieldsText
		 */
		protected $_fieldsText = array();
		
		/**
		 * Fields which will be selected from the database
		 * 
		 * @var array $_selectFields
		 */
		protected $_selectFields = array();
		
		/**
		 * Selected search fields (initialized to default)
		 * 
		 * @var array $_searchFields
		 */
		protected $_searchFields = array();
		
		/**
		 * Fields information prepared for the user interface
		 * 
		 * @var array
		 */
		protected $_fieldsInfo = array();
		
		
		/**
		 * Prohibits explicit object creation
		 */
		protected function __construct()
		{
			$this->initFieldsText();
			$this->initSearchFields();
			
			$this->initFieldsInfo();
			
			$this->_selectFields = array_flip(array_intersect(array_flip($this->_fields), $this->_selectFields));
			$this->_searchFields = array_flip(array_intersect(array_flip($this->_fields), $this->_searchFields));
		}

		/**
		 * Initializes the search fields info
		 */
		protected function initFieldsInfo()
		{
			foreach ($this->_fields as $fieldId => $fieldName) {
				$row[self::FIELD_CHECKED_KEY] = in_array($fieldId, $this->_searchFields) ? 'checked="checked"' : '';
				$row[self::FIELD_NAME_KEY] = $this->_fieldsText[$fieldId];
				$row[self::FIELD_ID] = $fieldId;
				
				$this->_fieldsInfo[] = $row;
			}
		}
		
		/**
		 * Returns all the fields data used for parsing
		 */
		public function getFieldsInfo()
		{
			return $this->_fieldsInfo;
		}
		
		/**
		 * Initializes the search fields
		 */
		abstract protected function initSearchFields();
		
		/**
		 * Initializes the text for each search field
		 */
		abstract protected function initFieldsText();
		
		/**
		 * Performs the search operation
		 */
		abstract public function find($phrase, $condition);
	}

	/**
	 * Defines the search fields for products
	 */
	class scProductSearchEngine extends scSearchEngine {
		/**
		 * The only instance of the object
		 * 
		 * @var scProductSearchFields $_instance
		 */
		protected static $_instance = null;
		
		/**
		 * Definition of the product fields
		 */
		const PRODUCT_ID = 0;
		const PRODUCT_NAME = 1;
		const PRODUCT_DESCRIPTION = 2;
		const PRODUCT_DETAILED_DESCRIPTION = 3;
		const PRODUCT_META_KEYWORDS = 4;
		const PRODUCT_META_DESCRIPTION = 5;
		const PRODUCT_CODE = 6;
		const PRODUCT_MANUFACTURER_NAME = 7;
		const PRODUCT_WEIGHT = 8;
		const PRODUCT_QUANTITY = 9;
		
		/**
		 * All possible search fields for products
		 * 
		 * @var array $_fields
		 */
		protected $_fields = array(
			// Default fields
			self::PRODUCT_ID => 'Products.ProductID',
			self::PRODUCT_NAME => 'ProductName',
			self::PRODUCT_DESCRIPTION => 'ProductDescription',
			self::PRODUCT_DETAILED_DESCRIPTION => 'ProductDetailedDescription',
			self::PRODUCT_META_KEYWORDS => 'ProductMetaKeywords',
			self::PRODUCT_META_DESCRIPTION => 'ProductMetaDescription',
			self::PRODUCT_CODE => 'ProductCode',
			self::PRODUCT_MANUFACTURER_NAME => 'ManufacturerName',
			// Additional fields
			self::PRODUCT_WEIGHT => 'ProductWeight',
			self::PRODUCT_QUANTITY => 'ProductQuantity'
		);
			
		/**
		 * Product fields which will be selected from the database
		 * 
		 * @var array $_selectFields
		 */
		protected $_selectFields = array(
			self::PRODUCT_ID,
			self::PRODUCT_NAME
		);
		
		/**
		 * Selected product search fields (initialized to default)
		 * 
		 * @var array $_searchFields
		 */
		protected $_searchFields = array(
			self::PRODUCT_ID,
			self::PRODUCT_NAME,
			self::PRODUCT_DESCRIPTION,
			self::PRODUCT_DETAILED_DESCRIPTION,
			self::PRODUCT_META_KEYWORDS,
			self::PRODUCT_META_DESCRIPTION,
			self::PRODUCT_CODE,
			self::PRODUCT_MANUFACTURER_NAME
		);			

		/**
		 * Initializes the search fields
		 */
		protected function initSearchFields()
		{
			$this->_searchFields = array_map('intval', miGetParamDefault('ProductSearchFields', $this->_searchFields));
		}
		
		/**
		 * Initializes the text for each search field
		 */
		protected function initFieldsText()
		{
			// Populate the $_productFieldsText variable
			$this->_fieldsText = array(
				self::PRODUCT_ID => _t('ID'),
				self::PRODUCT_NAME => _t('Name'),
				self::PRODUCT_DESCRIPTION => _t('Description'),
				self::PRODUCT_DETAILED_DESCRIPTION => _t('Detailed Description'),
				self::PRODUCT_META_KEYWORDS => _t('SEO Keywords'),
				self::PRODUCT_META_DESCRIPTION => _t('SEO Description'),
				self::PRODUCT_CODE => _t('SKU'),
				self::PRODUCT_MANUFACTURER_NAME => _t('Manufacturer'),
				// Additional fields
				self::PRODUCT_WEIGHT => _t('Weight'),
				self::PRODUCT_QUANTITY => _t('Quantity')
			);
		}
		
		/**
		 * Ensures that the object cannot be cloned
		 */
		protected function __clone()
		{
		}
		
		/**
		 * Returns the only instance of the object
		 * 
		 * @return scProductSearchEngine
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scProductSearchEngine();
				
			return self::$_instance;
		}
		
		/**
		 * Performs the searching operation
		 * 
		 * @param string $phrase The search phrase entered by the user
		 * @param string $condition The prepared condition to search in the database
		 * 
		 * @return array
		 */
		public function find($phrase, $condition)
		{
			if (empty($condition))
				return array();			// Do not allow empty searches

			$recordset = new miSqlRecordset('Products', 'ProductID');

			// Prepare join conditions
			$currentLangId = scMultiLanguage::getCurrentLanguageId();
			$recordset->addJoinCondition('INNER', 'ProductsML', 'ON (Products.ProductID = ProductsML.ProductID AND ProductsML.LanguageID = ' . $currentLangId . ')');
			$recordset->addJoinCondition('LEFT', 'Manufacturers', 'ON (Products.ManufacturerID = Manufacturers.ManufacturerID)');
			$recordset->addJoinCondition('LEFT', 'ManufacturersML', 'ON (Manufacturers.ManufacturerID = ManufacturersML.ManufacturerID AND ManufacturersML.LanguageID = ' . $currentLangId . ')');
			
			// Set the desired select fields
			$recordset->setSelectFields($this->_selectFields);
			
			// Prepare & apply search filters
			$filters = array();
			foreach ($this->_searchFields as $field)
				$filters[] = new miSqlFilterCustom($field, sql_escape_string($field) . ' LIKE "%' . sql_escape_string($condition) . '%"');
			
			$recordset->addFilter(new scSqlFilterOr('ProductID', $filters));

			// Return the result records
			return $recordset->getRecords();
		}
	}

	/**
	 * Defines the search fields for orders
	 */
	class scOrderSearchEngine extends scSearchEngine {
		/**
		 * The only instance of the object
		 * 
		 * @var scOrderSearchFields $_instance
		 */
		protected static $_instance = null;
		
		/**
		 * Definition of the order fields
		 */
		const ORDER_ID = 0;
		const ORDER_CUSTOMER_EMAIL = 1;
		const ORDER_BILLING_FIRST_NAME = 2;
		const ORDER_BILLING_LAST_NAME = 3;
		const ORDER_BILLING_COMPANY = 4;
		const ORDER_SHIPPING_FIRST_NAME = 5;
		const ORDER_SHIPPING_LAST_NAME = 6;
		const ORDER_SHIPPING_COMPANY = 7;
		const ORDER_ITEM_PRODUCT_CODE = 8;
		const ORDER_ITEM_PRODUCT_NAME = 9;
		const ORDER_STATUS = 10;
		
		/**
		 * All possible search fields for orders
		 * 
		 * @var array $_fields
		 */
		protected $_fields = array(
			self::ORDER_ID => 'Orders.OrderID',
			self::ORDER_CUSTOMER_EMAIL => 'OrderCustomerEmail',
			self::ORDER_BILLING_FIRST_NAME => 'OrderBillingFirstName',
			self::ORDER_BILLING_LAST_NAME => 'OrderBillingLastName',
			self::ORDER_BILLING_COMPANY => 'OrderBillingCompany',
			self::ORDER_SHIPPING_FIRST_NAME => 'OrderShippingFirstName',
			self::ORDER_SHIPPING_LAST_NAME => 'OrderShippingLastName',
			self::ORDER_SHIPPING_COMPANY => 'OrderShippingCompany',
			self::ORDER_ITEM_PRODUCT_CODE => 'OrderItemProductCode',
			self::ORDER_ITEM_PRODUCT_NAME => 'OrderItemProductName',
			self::ORDER_STATUS => 'OrderStatusName'
		);
			
		/**
		 * Order fields which will be selected from the database
		 * 
		 * @var array $_selectFields
		 */
		protected $_selectFields = array(
			self::ORDER_ID,
			self::ORDER_CUSTOMER_EMAIL,
			self::ORDER_STATUS
		);
		
		/**
		 * Selected order search fields (initialized to default)
		 * 
		 * @var array $_searchFields
		 */
		protected $_searchFields = array(
			self::ORDER_ID,
			self::ORDER_CUSTOMER_EMAIL,
			self::ORDER_BILLING_FIRST_NAME,
			self::ORDER_BILLING_LAST_NAME,
			self::ORDER_BILLING_COMPANY,
			self::ORDER_SHIPPING_FIRST_NAME,
			self::ORDER_SHIPPING_LAST_NAME,
			self::ORDER_SHIPPING_COMPANY,
			self::ORDER_ITEM_PRODUCT_CODE,
			self::ORDER_ITEM_PRODUCT_NAME
		);			

		/**
		 * Initializes the search fields
		 */
		protected function initSearchFields()
		{
			$this->_searchFields = array_map('intval', miGetParamDefault('OrderSearchFields', $this->_searchFields));
		}
		
		/**
		 * Initializes the text for each search field
		 */
		protected function initFieldsText()
		{
			// Populate the $_productFieldsText variable
			$this->_fieldsText = array(
				self::ORDER_ID => _t('ID'),
				self::ORDER_CUSTOMER_EMAIL => _t('Customer Email'),
				self::ORDER_BILLING_FIRST_NAME => _t('Billing first name'),
				self::ORDER_BILLING_LAST_NAME => _t('Billing last name'),
				self::ORDER_BILLING_COMPANY => _t('Billing company'),
				self::ORDER_SHIPPING_FIRST_NAME => _t('Shipping first name'),
				self::ORDER_SHIPPING_LAST_NAME => _t('Shipping last name'),
				self::ORDER_SHIPPING_COMPANY => _t('Shipping company'),
				self::ORDER_ITEM_PRODUCT_CODE => _t('Product code'),
				self::ORDER_ITEM_PRODUCT_NAME => _t('Product name'),
				self::ORDER_STATUS => _t('Order Status')
			);
		}
		
		/**
		 * Ensures that the object cannot be cloned
		 */
		protected function __clone()
		{
		}
		
		/**
		 * Returns the only instance of the object
		 * 
		 * @return scOrderSearchEngine
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scOrderSearchEngine();
				
			return self::$_instance;
		}
		
		public function find($phrase, $condition)
		{
			if (empty($condition))
				return array();			// Do not allow empty searches
			
			$recordset = new miSqlRecordset('Orders', 'OrderID');

			// Prepare join conditions
			$currentLangId = scMultiLanguage::getCurrentLanguageId();
			$recordset->addJoinCondition('LEFT', 'OrderItems', 'USING (OrderID)');
			$recordset->addJoinCondition('LEFT', 'OrderStatusesML', 'ON (Orders.OrderStatus = OrderStatusesML.OrderStatusID AND LanguageID=' . $currentLangId . ')');
			$recordset->setGroupBy('OrderID');
			
			// Set the desired select fields
			$recordset->setSelectFields($this->_selectFields);
			
			// Prepare & apply search filters
			$filters = array();
			foreach ($this->_searchFields as $field)
				$filters[] = new miSqlFilterSubstring($field, $condition);
			
			$recordset->addFilter(new scSqlFilterOr('OrderID', $filters));
			
			// Return the result records
			return $recordset->getRecords();
		}
	}
	
	/**
	 * The search engine class
	 */
	class scSearch {
		/**
		 * The original search phrase
		 * 
		 * @var string $_phrase
		 */
		protected $_phrase;
		
		/**
		 * Condition prepared for MySQL LIKE
		 * 
		 * @var string $_condition
		 */
		protected $_condition;
		
		/**
		 * Word delimiters (excluding the space character)
		 * 
		 * @var array $_delimiters
		 */
		protected $_delimiters = array(
			'/', '-', '+', '_', '=' , "\n",
			'<', '>', '*', '(', ')', '	'
		);

		/**
		 * Wildcard patterns (used to prepare the original search phrase for MySQL matching) 
		 * 
		 * @var array $_wildcards
		 */
		protected $_wildcards = array(
			'/(?<!\\\\)%/' => '\\%',
			'/[ ]+/' => '%',
			'/(?<!\\\\)[%]+/' => '%',
		);
		
		/**
		 * The only instance of this object
		 * 
		 * @var scSearch $_instance
		 */
		protected static $_instance = null;

		/**
		 * The class constructor, initializes the search phrase and prepares it for MySQL 
		 */
		protected function __construct()
		{
			$this->_phrase = trim(miGetParamDefault('phrase', ''));
			$this->_condition = $this->getCondition();
		}
		
		/**
		 * Return the only instance of the object
		 * 
		 * @return scSearch
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scSearch();
			
			return self::$_instance;
		}
		
		/**
		 * Prepares the search phrase for MySQL
		 * 
		 * @return string
		 */
		protected function getCondition()
		{
			$condition = str_replace($this->_delimiters, ' ', $this->_phrase);
			$condition = trim($condition);

			return preg_replace(array_keys($this->_wildcards), array_values($this->_wildcards), $condition);
		}
		
		/**
		 * Returns the search phrase
		 * 
		 * @return string
		 */
		public function getSearchPhrase()
		{
			return $this->_phrase;
		}
		
		// - Delegated requests -------------------------------------------------------------------------- //
		
		/**
		 * Finds products matching search criteria and returns them
		 * 
		 * @return array
		 */
		public function findProducts()
		{
			return scProductSearchEngine::singleton()->find($this->_phrase, $this->_condition);
		}
		
		/**
		 * Finds orders matching search criteria and returns them
		 * 
		 * @return array
		 */
		public function findOrders()
		{
			return scOrderSearchEngine::singleton()->find($this->_phrase, $this->_condition);
		}

		
		/**
		 * Returns the fields information (used for user interface)
		 * 
		 * @return array
		 */
		public function getProductFields()
		{
			return scProductSearchEngine::singleton()->getFieldsInfo();
		}
		
		/**
		 * Returns the fields information (used for user interface)
		 * 
		 * @return array
		 */
		public function getOrderFields()
		{
			return scOrderSearchEngine::singleton()->getFieldsInfo();
		}
	}
?>