<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
		
	interface IUserSearchFinder {
		
		public function setSearchData(array $data);
		
		public function getFindQuery();
	}
	
	interface IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data);
	}
	
	class scUserSearch {
		
		const PRODUCT_ID = 'ProductID';
		const PRODUCT_NAME = 'ProductName';
		const PRODUCT_DESCRIPTION = 'ProductDescription';
		const PRODUCT_DETAILED_DESCRIPTION = 'ProductDetailedDescription';
		const MANUFACTURER_NAME = 'ManufacturerName';
		const PRODUCT_IMAGE_ID = 'ProductImageID';
		const PRODUCT_IMAGE_THUMB_X = 'ProductImageThumbWidth';
		const PRODUCT_IMAGE_THUMB_Y = 'ProductImageThumbWidth';
		const PRODUCT_IMAGE_IS_DEFAULT = 'ProductImageIsDefault';
		const PRODUCT_CODE = 'ProductCode';
		const CATEGORY_ID = 'CategoryID';
		const MANUFACTURER_ID = 'ManufacturerID';
		const PRODUCT_PRICE = 'ProductPrice';
		const PRODUCT_CLASS_ID = 'ProductClassID';
		const PRODUCT_ATTRIBUTES = 'ProductAttributes';
		const PRODUCT_OPTIONS = 'ProductOptions';
		
		const VARIANTS_MODE_LIST_DEFAULT = 1;
		const VARIANTS_MODE_LIST_PRODUCTS = 2;
		const VARIANTS_MODE_LIST_ALL = 3;
		
		/**
		 * The one and only instance of the object
		 *
		 * @var scUserSearch
		 */
		protected static $_instance = null;
		
		/**
		 * @var array
		 */
		protected $_finders = array();
		
		/**
		 * The search phrases
		 */
		protected $_conditions = array();
		
		/**
		 * Order by clauses
		 */
		protected $_orderBy = array();
		
		/**
		 * @var array
		 */
		protected $_finderQueries = null;

		/**
		 * Basic object initialization
		 */
		protected function __construct()
		{			
			// Init finders
			$this->_finders['scUserSearchProductsFinder'] = new scUserSearchProductsFinder();
			$this->_finders['scUserSearchProductVariantsFinder'] = new scUserSearchProductVariantsFinder();
		}
		
		/**
		 * Prevent object cloning
		 */
		protected function __clone () {}
		
		/**
		 * Returns the one and only instance of the object
		 *
		 * @return scUserSearch
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scUserSearch();
			
			return self::$_instance;
		}
		
		/**
		 * Add search criteria to the search
		 *
		 * @param string $criteria
		 */
		public function addCriteria($searchField, $phrase)
		{
			// Save the search information
			$this->_conditions[$searchField] = $phrase;
		}
		
		/**
		 * Add order by clauses
		 * 
		 * @param string $orderBy
		 */
		public function addOrderBy($orderBy)
		{
			$this->_orderBy[] = $orderBy;
		}
		
		public function setVariantsMode($mode)
		{
			foreach ($this->_finders as $finder) {
				$finder->setVariantsMode($mode);
			}
		}
		
		/**
		 * Return the matched records count
		 *
		 *  @return int
		 */
		public function getRecordsCount()
		{
			// Construct the query
			$query = 'SELECT COUNT(DISTINCT ProductIDSort) AS Count FROM (' . $this->getFindersSubquery() . ') AS Products';
			$result = miStaticDBUtil::execSelect($query);
			return intval($result[0]['Count']);
		}

		/**
		 * Returns the records matching the search criteria
		 * 
		 * @return array
		 */
		public function getRecords($from, $numRecords)
		{
			// Build the query and get all matches
			$query = 'SELECT * FROM (' . $this->getFindersSubquery() . ') AS Products' .
				' GROUP BY ProductIDSort' .
				' ORDER BY ' . implode(', ', $this->_orderBy) .
				' LIMIT ' . intval($from) . ', ' . intval($numRecords);
			$data = miStaticDBUtil::execSelect($query);
			
			// Tell the finders which items have been found.
			foreach ($data as $match) {
				$this->_finders[$match['ResultFinder']]->addResultId($match['ResultID']);
			}
			
			// Get each finder's result data.
			$resultData = array();
			foreach ($this->_finders as $finder) {
				$resultData[get_class($finder)] = $finder->getResultData();
			}
			
			// Build the output array.
			$records = array();
			foreach ($data as $match) {
				$records[] = $resultData[$match['ResultFinder']][$match['ResultID']];
			}
			
			return $records;
		}
		
		protected function getFindersSubquery()
		{
			if (is_null($this->_finderQueries)) {
				$this->setFindersSearchData();
				$this->_finderQueries = array();
				foreach ($this->_finders as $finder) {
					// Ask each finder for its subquery
					$this->_finderQueries[] = $finder->getFindQuery();
				}
				// Remove possible empty queries
				$this->_finderQueries = array_filter($this->_finderQueries);
				
				if (empty($this->_finderQueries)) {
					throw new miException(_i('No finder queries.'));
				}
			}
			return '(' . implode(') UNION (', $this->_finderQueries) . ')';
		}
		
		protected function setFindersSearchData()
		{
			foreach ($this->_finders as $finder) {
				$finder->setSearchData($this->_conditions);
			}
		}
	}
	
	abstract class scUserSearchFinder implements IUserSearchFinder {
		
		/**
		 * @var array
		 */
		protected $_searchData = array();
		
		/**
		 * @var array
		 */
		protected $_resultIds = array();
		
		/**
		 * @var int
		 */
		protected $_variantsMode = scUserSearch::VARIANTS_MODE_LIST_DEFAULT;
		
		/**
		 * @return miSqlRecordset
		 */
		abstract protected function getRecordset();
		
		/**
		 * @param string $field
		 * @return IFinderRecordsetModifier
		 */
		abstract protected function getRecordsetModifierForField($field);
		
		/**
		 * @return array
		 */
		abstract public function getResultData();
		
		public function setSearchData(array $data)
		{
			$this->_searchData = $data;
		}
		
		/**
		 * @return string
		 */
		public function getFindQuery()
		{
			$recordset = $this->getRecordset();
			$this->modifyRecordset($recordset);
			return $recordset->getSelectQuery();
		}
		
		public function setVariantsMode($mode)
		{
			$this->_variantsMode = $mode;
		}
		
		public function addResultId($id)
		{
			$this->_resultIds[] = (int)$id;
		}
		
		protected function modifyRecordset(miSqlRecordset $recordset)
		{
			foreach ($this->_searchData as $field => $data) {
				$modifier = $this->getRecordsetModifierForField($field);
				if ($modifier instanceof IFinderRecordsetModifier) {
					$modifier->modifyRecordset($recordset, $field, $data);
				}
			}
		}
	}
	
	class scUserSearchProductsFinder extends scUserSearchFinder {
		
		public function getResultData()
		{
			if (empty($this->_resultIds)) {
				return array();
			}
			
			$recordset = new miSqlRecordset('Products');
			$recordset->addJoinCondition('LEFT', 'ProductVariants', 'ON Products.ProductID = ProductVariants.ProductID');
			$recordset->addJoinCondition('LEFT', 'ProductsML', 'ON Products.ProductID = ProductsML.ProductID AND LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->addJoinCondition('LEFT', 'ProductImages', 'ON ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault');
			$recordset->addFilter(new miSqlFilterIn('Products.ProductID', $this->_resultIds));
			$recordset->setSelectFields(array('*', 'Products.ProductID AS ProductID'));
			$recordset->setGroupBy('Products.ProductID');
			
			$products = $recordset->getAllRecords();
			
			$listingItems = array();
			foreach ($products as &$product) {
				$listingItem = new scProductSL();
				$listingItem->setRow($product);
				
				$listingItems[$product['ProductID']] = $listingItem;
			}
			unset($product);
			
			return $listingItems;
		}
		
		protected function getRecordset()
		{
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->addJoinCondition('RIGHT', 'Products', 'USING(ProductID)', 'SearchProducts');
			$recordset->addJoinCondition('LEFT', 'ProductsML', 'ON SearchProducts.ProductID = ProductsML.ProductID AND ProductsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->setSelectFields($this->getSelectFields());
			// Check "out of stock" setting
			if (scProductService::hideOutOfStock()) {
				$recordset->addFilter(new miSqlFilterBiggerThan('ProductQuantity', 0));
			}
			$recordset->addFilter(new miSqlFilterEqual('ProductIsActive', 1));
			
			if ($this->_variantsMode != scUserSearch::VARIANTS_MODE_LIST_PRODUCTS) {
				// Do not select products that have variants if variants are handled by another finder.
				$recordset->addFilter(new scSqlFilterIsNull('ProductVariantID'));
			}
			
			return $recordset;
		}
		
		protected function getSelectFields()
		{
			$productPriceFieldName = scCurrencyUtil::getProductPriceColumnName();
			return array(
				'"' . sql_escape_string(get_class($this)) . '" AS ResultFinder',
				'SearchProducts.ProductID AS ResultID',
				'SearchProducts.ProductID AS ProductIDSort',
				'ProductsML.ProductName',
				$productPriceFieldName
			);
		}
		
		protected function getRecordsetModifierForField($field)
		{
			switch ($field)
			{
				case scUserSearch::PRODUCT_NAME:
					return new scRecordsetModifierProductName();
				
				case scUserSearch::CATEGORY_ID:
					return new scRecordsetModifierCategory();
				
				case scUserSearch::MANUFACTURER_ID:
					return new scRecordsetModifierManufacturer();
				
				case scUserSearch::PRODUCT_CLASS_ID:
					return new scRecordsetModifierProductClass();
				
				case scUserSearch::PRODUCT_PRICE:
					return new scRecordsetModifierProductPrice();
				
				case scUserSearch::PRODUCT_CODE:
					return new scRecordsetModifierProductCode();
				
				case scUserSearch::PRODUCT_ATTRIBUTES:
					return new scRecordsetModifierPredefined();
				
				case scUserSearch::PRODUCT_OPTIONS:
					return new scRecordsetModifierProductOptions();
			}
		}
	}
	
	class scUserSearchProductVariantsFinder extends scUserSearchFinder {
		
		public function getFindQuery()
		{
			switch ($this->_variantsMode) {
				case scUserSearch::VARIANTS_MODE_LIST_PRODUCTS:
					return '';
				
				case scUserSearch::VARIANTS_MODE_LIST_ALL:
					return parent::getFindQuery();
				
				case scUserSearch::VARIANTS_MODE_LIST_DEFAULT:
					return 'SELECT * FROM (' . parent::getFindQuery() . ') AS ProductVariantsFinderTable GROUP BY ProductID';
				
				default:
					throw new miException(_i('Unknown variants listing mode.'));
			}
		}
		
		public function getResultData()
		{
			if (empty($this->_resultIds)) {
				return array();
			}
			
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->addJoinCondition('INNER', 'Products', 'USING(ProductID)');
			$recordset->addJoinCondition('LEFT', 'ProductsML', 'ON Products.ProductID = ProductsML.ProductID AND LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->addJoinCondition('LEFT', 'ProductImages', 'ON ProductImages.ProductID = Products.ProductID AND ProductImageIsDefault');
			$recordset->addFilter(new miSqlFilterIn('ProductVariantID', $this->_resultIds));
			$recordset->setSelectFields(array('*', 'Products.ProductID AS ProductID'));
			
            $variants = $recordset->getAllRecords();
			
			$listingItems = array();
            foreach ($variants as &$variant) {
            	$listingItem = new scSearchListingProductVariant();
            	$listingItem->setRow($variant);
            	$listingItems[$variant['ProductVariantID']] = $listingItem;
            }
            unset($variant);
            
			return $listingItems;
		}
		
		protected function getRecordset()
		{
			$recordset = new miSqlRecordset('ProductVariants');
			$recordset->addJoinCondition('INNER', 'Products', 'USING(ProductID)', 'SearchProducts');
			$recordset->addJoinCondition('LEFT', 'ProductsML', 'ON ProductVariants.ProductID = ProductsML.ProductID AND ProductsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->setSelectFields($this->getSelectFields());
			$recordset->setOrder('ProductVariantIsDefault', 'DESC');
			$recordset->setRecordsLimit(0, 1);
			
			// Check "out of stock" setting
			if (scProductService::hideOutOfStock()) {
				$recordset->addFilter(new miSqlFilterBiggerThan('ProductVariantQuantity', 0));
			}
			$recordset->addFilter(new miSqlFilterEqual('ProductIsActive', 1));
			
			return $recordset;
		}
		
		protected function getSelectFields()
		{
			$productPriceFieldName = scCurrencyUtil::getProductPriceColumnName();
			return array(
				'"' . sql_escape_string(get_class($this)) . '" AS ResultFinder',
				'ProductVariants.ProductVariantID AS ResultID',
				'ProductVariants.ProductID',
				'ProductsML.ProductName',
				'ProductVariantPrice AS ' . $productPriceFieldName
			);
		}
		
		protected function getRecordsetModifierForField($field)
		{
			switch ($field)
			{
				case scUserSearch::PRODUCT_NAME:
					return new scRecordsetModifierProductVariantName();
				
				case scUserSearch::CATEGORY_ID:
					return new scRecordsetModifierCategory();
				
				case scUserSearch::MANUFACTURER_ID:
					return new scRecordsetModifierManufacturer();
				
				case scUserSearch::PRODUCT_CLASS_ID:
					return new scRecordsetModifierProductClass();
				
				case scUserSearch::PRODUCT_PRICE:
					return new scRecordsetModifierProductVariantPrice();
				
				case scUserSearch::PRODUCT_CODE:
					return new scRecordsetModifierProductVariantCode();
				
				case scUserSearch::PRODUCT_ATTRIBUTES:
					return new scRecordsetModifierPredefined();
				
				case scUserSearch::PRODUCT_OPTIONS:
					return new scRecordsetModifierProductVariantOptions();
			}
		}
	}
	
	class scRecordsetModifierProductCode implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			$recordset->addFilter(new miSqlFilterSubstring('ProductCode', $data));
		}
	}
	
	class scRecordsetModifierProductVariantCode implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			$recordset->addFilter(new miSqlFilterSubstring('ProductVariantCode', $data));
		}
	}
	
	class scRecordsetModifierProductPrice implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			$from = (float)$data[0];
			$to = (float)$data[1];
			
			$currentCurrencyCode = scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			$baseCurrency = scCurrencyUtil::getBaseCurrency();
			
			$convertPrices = false;
			$productPriceFieldName = scCurrencyUtil::getProductPriceColumnName();
			if ($productPriceFieldName == 'ProductPrice') { // we should convert the currency only if we use ProductPrice
				$convertPrices = true;
			}
			$productPriceFieldName = scProductPrice::getProductPriceFieldForQuery($productPriceFieldName);
			if (!empty($to)) {
				if ($convertPrices) {
					$to = scCurrencyUtil::convert($to, $currentCurrencyCode, $baseCurrency);
				}
				$recordset->addFilter(new miSqlFilterCustom($productPriceFieldName, '0+'.$productPriceFieldName.' <= 0+"' . sql_escape_string($to) . '"'));
			}
			if (!empty($from)) {
				if ($convertPrices) {
					$from = scCurrencyUtil::convert($from, $currentCurrencyCode, $baseCurrency);
				}
				$recordset->addFilter(new miSqlFilterCustom($productPriceFieldName, '0+'.$productPriceFieldName.' >= 0+"' . sql_escape_string($from) . '"'));
			}
		}
	}
	
	class scRecordsetModifierProductVariantPrice implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			$from = (float)$data[0];
			$to = (float)$data[1];
			
			$currentCurrencyCode = scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			$baseCurrency = scCurrencyUtil::getBaseCurrency();
			
			if (!empty($to)) {
				$to = scCurrencyUtil::convert($to, $currentCurrencyCode, $baseCurrency);
				$recordset->addFilter(new miSqlFilterCustom('ProductVariantPrice', '0+ProductVariantPrice <= 0+"' . sql_escape_string($to) . '"'));
			}
			if (!empty($from)) {
				$from = scCurrencyUtil::convert($from, $currentCurrencyCode, $baseCurrency);
				$recordset->addFilter(new miSqlFilterCustom('ProductVariantPrice', '0+ProductVariantPrice >= 0+"' . sql_escape_string($from) . '"'));
			}
		}
	}
	
	class scRecordsetModifierProductName implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			$filters = $this->getFilters($field, $data);
			
			if (!empty($filters)) {
				foreach ($filters as $filter) {
					$recordset->addFilter(new miSqlFilterCustom($field, $filter));
				}
			}
		}
		
		protected function getFilters($field, $data)
		{
			$f = array();
			
			list($productName, $searchInDetails) = $data;
			$phrases = $this->parsePhrase($productName);
			if ($phrases === false)
				return $f;
			
			foreach ($phrases as $phrase) {
				$filter = $this->getPhraseFilter($field, $phrase, $searchInDetails);
				if (!empty($filter)) {
					$f[] = $filter;
				}
			}
			return $f;
		}
			
		/**
		 * Parses the phrase for text search
		 */
		protected function parsePhrase($phrase)
		{
			// Prepare the phrase corresponding to this field		
			$phrase = explode(' ', $phrase);
			if (count($phrase) == 0)
				return false;
			
			// If some space is forgotten
			$phrase = array_map('trim', $phrase);
			$phrase = array_filter($phrase);
			if (count($phrase) == 0)
				return false;
			
			return $phrase;
		}
		
		protected function getPhraseFilter($field, $phrase, $searchInDetails)
		{
			$phrase = miStaticDBUtil::escapeStringForLike($phrase);
			if ($searchInDetails) {
				return '(' . $field . ' LIKE "%' . $phrase . '%" OR ' .
					scUserSearch::PRODUCT_DESCRIPTION . ' LIKE "%' . $phrase . '%" OR ' .
					scUserSearch::PRODUCT_DETAILED_DESCRIPTION . ' LIKE "%' . $phrase . '%" OR ' .
					scUserSearch::PRODUCT_CODE . ' LIKE "%' . $phrase . '%")';
			}
			return $field . ' LIKE "%' . $phrase . '%"';
		}
	}
	
	class scRecordsetModifierProductVariantName extends scRecordsetModifierProductName {
		
		protected function getPhraseFilter($field, $phrase, $searchInDetails)
		{
			$filter = parent::getPhraseFilter($field, $phrase, $searchInDetails);
			
			if (!$searchInDetails) {
				return $filter;
			}
			
			if (!empty($filter)) {
				$filter .= ' OR ';
			}
			$phrase = miStaticDBUtil::escapeStringForLike($phrase);
			$filter .= 'ProductVariantCode LIKE "%' . $phrase . '%"';
			return '(' . $filter . ')';
		}
	}
	
	class scRecordsetModifierCategory implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			
			list($categoryId, $subcategories) = array_map('intval', $data);
			if ($categoryId == 0) {
				return;
			}
			
			if (!$subcategories) {
				$recordset->addJoinCondition('LEFT', 'Categories', 'ON SearchProducts.CategoryID = Categories.CategoryID');
				$recordset->addFilter(new scSqlFilterOr('CategoryID', array(
					new miSqlFilterEqual('SearchProducts.CategoryID', $categoryId),
					new miSqlFilterCustom('SearchProducts.CategoryID', 'SearchProducts.ProductID IN (SELECT ProductID FROM ProductXCategories WHERE CategoryID = ' . sql_escape_string($categoryId) . ')'),
				)));
				return;
			}
			
			// Get category left and right column
			$category = miStaticDBUtil::execSelect('SELECT CategoryLeft, CategoryRight FROM Categories WHERE CategoryID = ?', array($categoryId));
			if (count($category) == 0) {
				return;
			}
			
			$category = reset($category);
			$preFilter = 'CategoryLeft >= ' . $category['CategoryLeft'] . ' AND CategoryRight <= ' . $category['CategoryRight'];
			
			// search additional categories
			$subCategoryIds = miUtil::getDBArray('SELECT CategoryID FROM Categories WHERE ' . $preFilter, 'CategoryID', 'CategoryID');
			if (count($subCategoryIds) > 0) {
				$preFilter = '((' . $preFilter . ') OR (SearchProducts.ProductID IN (SELECT ProductID FROM ProductXCategories WHERE CategoryID IN(' . join(',', $subCategoryIds) . '))))';	
			}
			
			$recordset->addJoinCondition('LEFT', 'Categories', 'ON SearchProducts.CategoryID = Categories.CategoryID');
			$recordset->addFilter(new miSqlFilterCustom('Products.CategoryID', $preFilter));			
		}
	}
	
	class scRecordsetModifierManufacturer implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			$recordset->addJoinCondition('LEFT', 'ManufacturersML', 'ON SearchProducts.ManufacturerID = ManufacturersML.ManufacturerID AND ManufacturersML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId());
			$recordset->addFilter(new miSqlFilterEqual('ManufacturersML.ManufacturerID', $data));
		}
	}
	
	class scRecordsetModifierProductClass implements IFinderRecordsetModifier {
		
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			$recordset->addFilter(new miSqlFilterEqual('ProductClassID', $data));
		}
	}
	
	class scRecordsetModifierPredefined implements IFinderRecordsetModifier {
		
		/**
		 * @param miSqlRecordset $recordset
		 * @param string $field
		 * @param array $data
		 */
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			
			if (!is_array($data)) {
				throw new miException(_i('Invalid data passed to scRecordsetModifierPredefined.'));
			}
			
			foreach ($data as $subquery) {
				$recordset->addFilter(new miSqlFilterCustom('ProductID', 'SearchProducts.ProductID IN (' . $subquery . ')'));
			}
		}
	}
	
	class scRecordsetModifierProductOptions implements IFinderRecordsetModifier {
		
		/**
		 * @param miSqlRecordset $recordset
		 * @param string $field
		 * @param array $data
		 */
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			
			foreach ($data as $optionId) {
				if ($optionId != '') {
					$recordset->addFilter(new miSqlFilterCustom('ProductID', 'FALSE'));
					return;
				}
			}
		}
	}
	
	class scRecordsetModifierProductVariantOptions implements IFinderRecordsetModifier {
		
		/**
		 * @param miSqlRecordset $recordset
		 * @param string $field
		 * @param array $data
		 */
		public function modifyRecordset(miSqlRecordset $recordset, $field, $data)
		{
			if (empty($data)) {
				return;
			}
			
			foreach ($data as $optionGroupId => $optionId) {
				if ($optionId == '') {
					continue;
				}
				
				$filter = 'SELECT ProductVariantID' .
				' FROM ProductVariantOptions' .
				' INNER JOIN Options ON ProductVariantOptions.OptionID = Options.OptionID' .
				' WHERE OptionGroupID = ' . intval($optionGroupId) .
				' AND ProductVariantOptions.OptionID = ' . intval($optionId);
				
				$recordset->addFilter(new miSqlFilterCustom('ProductVariantID', 'ProductVariantID IN (' . $filter . ')'));
			}
		}
	}
?>