<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Generates WSDL 1.1 with SOAP binding
	 */
	class scSoapWsdl {
		/**
		 * 
		 * @var string
		 */
		protected $_className;
		
		/**
		 * 
		 * @var string
		 */
		protected $_serviceName;
		
		/**
		 * The URI of the service
		 * 
		 * @var string
		 */
		protected $_uri;
		
		/**
		 * The namespace of the service
		 * 
		 * @var string
		 */
		protected $_namespace;
		
		/**
		 * 
		 * @var scXsd
		 */
		protected $_xsd;
		
		/**
		 * hash with the names of all faults
		 * @var array
		 */
		protected $_faultNames;
		
		protected $_requestHeaders = array();
		protected $_responseHeaders = array();
		
		/**
		 * Create the SOAP WSDL class
		 * 
		 * @param string $className
		 * @param string $serviceName
		 * @param string $uri service uri
		 * @param scXsd $xsd
		 */
		public function __construct($className, $serviceName, $uri, $namespace, scXsd $xsd)
		{
			$this->_className = $className;
			$this->_serviceName = $serviceName;
			$this->_uri = $uri;
			$this->_namespace = $namespace;
			$this->_xsd = $xsd;
		}
		
		/**
		 * Return WSDL web service description 
		 * 
		 * @return string the wsdl
		 */
		public function getWsdl()
		{
			$class = new ReflectionClass($this->_className);
			if (!$class->isInstantiable())
				throw new miException(sprintf('The class "%s" cannot be instantiated.', $this->_className));
			$methods = $class->getMethods(256);	// 256 = public methods
			
			$portTypes = $bindings = $messages = array();
			$bindings[] = array(
				scXml::TAG => 'soap:binding',
				scXml::ATTRIBUTES => array('style' => 'document', 'transport' => 'http://schemas.xmlsoap.org/soap/http'),
			);
			
			foreach ($methods as $method) {
				$docComment = $method->getDocComment();
				$parsedDocComment = self::parseDocComment($docComment);
				
				if (isset($parsedDocComment['xsd'][0]) and ($parsedDocComment['xsd'][0]['name'] == 'skip'))
					continue;
				
				if (count($parsedDocComment['params']) != 1)
					throw new miException(sprintf(_i('Soap action method "%s" must accept one param with the request struct.'), $method));
				
				$classNames = $this->getClassesFromReflection($method, $parsedDocComment);
				foreach ($classNames as $className)
					$this->_xsd->addClassAsElement($className);
				
				$portTypes[] = $this->getPortTypeXmlDefinition($method, $parsedDocComment);
				$bindings[] = $this->getBindingXmlDefinition($method, $parsedDocComment);
				$messages = array_merge($messages, $this->getMessagesXmlDefinition($method->name, $parsedDocComment));
			}
			
			// Add request headers as messages
			$messages = array_merge($messages, $this->getRequestHeadersMessagesXmlDefinition());
			
			// Add response headers as messages
			$messages = array_merge($messages, $this->getResponseHeadersMessagesXmlDefinition());
			
			// Add the messages for the faults
			$messages = array_merge($messages, $this->getFaultMessagesXmlDefinition());
			
			// Create the set of definitions
			$mainElements = array();
			$mainElements[] = $this->getTypesXmlDefinition($this->_namespace);
			foreach ($messages as $message)
				$mainElements[] = $message;
			$mainElements[] = array(
							scXml::TAG => 'wsdl:portType',
							scXml::ATTRIBUTES => array('name' => $this->_serviceName . 'Interface'),
							scXml::CHILDREN => $portTypes
						);
			$mainElements[] = array(
							scXml::TAG => 'wsdl:binding',
							scXml::ATTRIBUTES => array('name' => $this->_serviceName . 'Binding', 'type' => 'tns:' . $this->_serviceName . 'Interface'),
							scXml::CHILDREN => $bindings
						);
			$mainElements[] = array(
							scXml::TAG => 'wsdl:service',
							scXml::ATTRIBUTES => array('name' => $this->_serviceName),
							scXml::CHILDREN => array(
								array(
									scXml::TAG => 'wsdl:port',
									scXml::ATTRIBUTES => array('name' => $this->_serviceName . 'Interface', 'binding' => 'tns:' . $this->_serviceName . 'Binding'),
									scXml::CHILDREN => array(
										array(
											scXml::TAG => 'soap:address',
											scXml::ATTRIBUTES => array('location' => $this->_uri),
										),
									)
								),
							)
						);
			
			
			// Build the WSDL
			$definitions = array(
				array(
					scXml::TAG => 'definitions',
					scXml::ATTRIBUTES => array(
						'xmlns:wsdl' => 'http://schemas.xmlsoap.org/wsdl/',
						'xmlns:soap' => 'http://schemas.xmlsoap.org/wsdl/soap/',
						'xmlns:xsd' => 'http://www.w3.org/2001/XMLSchema',
						'xmlns:tns' => $this->_namespace,
						'xmlns' => 'http://schemas.xmlsoap.org/wsdl/',
						'targetNamespace' => $this->_namespace
					),
					scXml::CHILDREN => $mainElements
				),
			);
			return '<?xml version="1.0" encoding="UTF-8"?>' . "\n" . scXml::xmlWriter($definitions);
		}
		
		protected function getFaultMessagesXmlDefinition()
		{
			$messages = array();
			foreach ($this->_faultNames as $faultMessage) {
				$messages[] = array(
					scXml::TAG => 'wsdl:message',
					scXml::ATTRIBUTES => array('name' => $faultMessage),
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'wsdl:part',
							scXml::ATTRIBUTES => array('name' => 'fault', 'element' => 'tns:' . $faultMessage)
						)
					)
				);
			}
			$this->_xsd->addClassAsElement($faultMessage);
			return $messages;
		}
		
		/**
		 * Returns XML definition for wsdl:types
		 * 
		 * @param string $namespace
		 * @return array
		 */
		protected function getTypesXmlDefinition($namespace)
		{
			return array(
				scXml::TAG => 'wsdl:types',
				scXml::CHILDREN => array($this->_xsd->getSchemaXmlDefinition($namespace))
			);
		}
		
		/**
		 * Returns the class names which are parameters to the method and return value
		 * 
		 * @return array
		 */
		protected function getClassesFromReflection($method, $parsedDocComment)
		{
			$classNames = array();
			
			$params = $method->getParameters();
			foreach ($params as $param)
				if ($param->getClass() != null)
					$classNames[] = $param->getClass()->getName();
			
			foreach ($parsedDocComment['params'] as $param) {
				$classNames[] = $param['type'];
			}
			
			$returnType = $parsedDocComment['return']['type'];
			$builtIn = array('boolean', 'bool', 'string', 'int', 'integer', 'float', 'double', 'real');
			if (($returnType != '') and !in_array($returnType, $builtIn)) {
				if (class_exists($returnType)) {
					$classNames[] = $returnType;
				}
			}
			
			return array_unique($classNames);
		}
		
		/**
		 * Return the port type xml definition
		 * 
		 * @param array $method
		 * @param array $parsedDocComment
		 * @return array
		 */
		protected function getPortTypeXmlDefinition($method, $parsedDocComment)
		{
			$faults = array();
			foreach ($parsedDocComment['xsd'] as $xsd) {
				if ($xsd['name'] == 'fault') {
					$faults[] = array(
						scXml::TAG => 'wsdl:fault',
						scXml::ATTRIBUTES => array('message' => 'tns:' . $xsd['value'], 'name' => $xsd['value'])
					);
				}
			}
			
			$portTypeDef = array(
				scXml::TAG => 'wsdl:operation',
				scXml::ATTRIBUTES => array('name' => $method->name),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'wsdl:documentation',
						scXml::DATA => $parsedDocComment['description'],
					),
					array(
						scXml::TAG => 'wsdl:input',
						scXml::ATTRIBUTES => array('message' => 'tns:' . $method->name . 'Request', 'name' => $method->name . 'Request')
					),
					array(
						scXml::TAG => 'wsdl:output',
						scXml::ATTRIBUTES => array('message' => 'tns:' . $method->name . 'Response', 'name' => $method->name . 'Response')
					),
				)
			);
			$portTypeDef[scXml::CHILDREN] = array_merge($portTypeDef[scXml::CHILDREN], $faults);
			return $portTypeDef;
		}
		
		/**
		 * Return soap binding xml definition
		 * 
		 * @param $method
		 * @param array $parsedDocComment
		 * @return array
		 */
		protected function getBindingXmlDefinition($method, $parsedDocComment)
		{
			$requestHeaders = array();
			$responseHeaders = array();
			foreach ($parsedDocComment['xsd'] as $xsd) {
				if ($xsd['name'] == 'soaprequestheader') {
					$this->_requestHeaders[$xsd['value']] = $xsd['value'];
					$requestHeaders[] = $xsd['value'];
				}
				if ($xsd['name'] == 'soapresponseheader') {
					$this->_responseHeaders[$xsd['value']] = $xsd['value'];
					$responseHeaders[] = $xsd['value'];
				}
			}
			
			$inputChildren = array();
			foreach ($requestHeaders as $requestHeader) {
				$inputChildren[] = array(
					scXml::TAG => 'soap:header',
					scXml::ATTRIBUTES => array('message' => 'tns:' . $requestHeader, 'part' => $requestHeader, 'use' => 'literal'),
				);
			}
			$inputChildren[] = array(
				scXml::TAG => 'soap:body',
				scXml::ATTRIBUTES => array('use' => 'literal'),
			);
			
			$outputChildren = array();
			foreach ($responseHeaders as $responseHeader) {
				$outputChildren[] = array(
					scXml::TAG => 'soap:header',
					scXml::ATTRIBUTES => array('message' => 'tns:' . $responseHeader, 'part' => $responseHeader, 'use' => 'literal'),
				);
			}
			$outputChildren[] = array(
				scXml::TAG => 'soap:body',
				scXml::ATTRIBUTES => array('use' => 'literal'),
			);
			
			
			$faults = array();
			foreach ($parsedDocComment['xsd'] as $xsd) {
				if ($xsd['name'] == 'fault') {
					$faults[] = array(
						scXml::TAG => 'wsdl:fault',
						scXml::ATTRIBUTES => array('name' => $xsd['value']),
						scXml::CHILDREN => array(
							array(
								scXml::TAG => 'soap:fault',
								scXml::ATTRIBUTES => array('name' => $xsd['value'], 'use' => 'literal'),
							),
						)
					);
					$this->_faultNames[$xsd['value']] = $xsd['value'];
				}
			}
			
			$bindingDefs =  array(
				scXml::TAG => 'wsdl:operation',
				scXml::ATTRIBUTES => array('name' => $method->name),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'soap:operation',
						scXml::ATTRIBUTES => array('soapAction' => ''),
					),
					array(
						scXml::TAG => 'wsdl:input',
						scXml::ATTRIBUTES => array('name' => $method->name . 'Request'),
						scXml::CHILDREN => $inputChildren
					),
					array(
						scXml::TAG => 'wsdl:output',
						scXml::ATTRIBUTES => array('name' => $method->name . 'Response'),
						scXml::CHILDREN => $outputChildren
					),
				)
			);
			$bindingDefs[scXml::CHILDREN] = array_merge($bindingDefs[scXml::CHILDREN], $faults);
			return $bindingDefs;
		}
		
		/**
		 * Return two WSDL messages sections - for the request and for the response
		 * 
		 * @param string $methodName the name of the method that uses these messages
		 * @param array $parsedDocComment
		 * @return array array of XML definitions
		 */
		protected function getMessagesXmlDefinition($methodName, $parsedDocComment)
		{
			$messages = array();
			
			$requestElement = $parsedDocComment['params'][0]['type'];
			$messages[] = array(
				scXml::TAG => 'wsdl:message',
				scXml::ATTRIBUTES => array('name' => $methodName . 'Request'),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'wsdl:part',
						scXml::ATTRIBUTES => array('name' => 'request', 'element' => 'tns:' . $requestElement),
					)
				)
			);
			
			$responseElement = $parsedDocComment['return']['type'];
			$messages[] = array(
				scXml::TAG => 'wsdl:message',
				scXml::ATTRIBUTES => array('name' => $methodName . 'Response'),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'wsdl:part',
						scXml::ATTRIBUTES => array('name' => 'response', 'element' => 'tns:' . $responseElement),
					)
				)
			);
			
			return $messages;
		}
		
		protected function getRequestHeadersMessagesXmlDefinition()
		{
			$messages = array();
			foreach ($this->_requestHeaders as $requestHeader) {
				$this->_xsd->addClassAsElement($requestHeader);
				
				$messages[] = array(
					scXml::TAG => 'wsdl:message',
					scXml::ATTRIBUTES => array('name' => $requestHeader),
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'wsdl:part',
							scXml::ATTRIBUTES => array('name' => $requestHeader, 'element' => 'tns:' . $requestHeader),
						)
					)
				);
			}
			return $messages;
		}
		
		protected function getResponseHeadersMessagesXmlDefinition()
		{
			$messages = array();
			foreach ($this->_responseHeaders as $responseHeader) {
				$this->_xsd->addClassAsElement($responseHeader);
				
				$messages[] = array(
					scXml::TAG => 'wsdl:message',
					scXml::ATTRIBUTES => array('name' => $responseHeader),
					scXml::CHILDREN => array(
						array(
							scXml::TAG => 'wsdl:part',
							scXml::ATTRIBUTES => array('name' => $responseHeader, 'element' => 'tns:' . $responseHeader),
						)
					)
				);
			}
			return $messages;
		}
		
		/**
		 * Parses doc comment
		 * 
		 * @param string $docComment
		 * @return array parsed doc comment
		 */
		public static function parseDocComment($docComment)
		{
			$params = array();
			$xsd = array();
			$vars = array();
			$returnType = '';
			$returnDescription = '';
			$description = '';
			
			$docComment = trim($docComment);
			if ($docComment == '')	// No doc comment ?
				return array('description' => '', 'return' => array('type' => '', 'description' => ''), 'params' => array(), 'xsd' => array(), 'vars' => array());
			
			if ((substr_compare($docComment, '/**', 0, 3) !== 0) or (substr_compare($docComment, '*/', strlen($docComment)-2, 2) !== 0))
				throw new miException(_i('Invalid doc comment.'));
			
			// Remove the prolog (/**) and epilog (*/)
			$docComment = trim(substr($docComment, 3, -2));
			
			$lines = preg_split("/[\r\n]/", $docComment);
			foreach ($lines as $line) {
				$line = trim($line);
				if ($line == '')
					continue;
				
				// Remove optional * on each line of the doc comment
				if ($line[0] == '*') {
					$line = trim(substr($line, 1));
					if ($line == '')
						continue;
				}
				
				if ($line[0] != '@') {
					$description .= "\n" . $line;
					continue;
				}
				
				$parts = explode(' ', $line, 2);
				if ($parts[0] == '@return') {
					$returnInfo = explode(' ', trim($parts[1]), 2);
					$returnType = trim($returnInfo[0]);
					$returnDescription = trim(isset($returnInfo[1]) ? $returnInfo[1] : '');
				}
				
				if ($parts[0] == '@xsd') {
					$xsdInfo = explode(' ', trim($parts[1]), 2);
					$xsd[] = array('name' => trim($xsdInfo[0]), 'value' => trim(isset($xsdInfo[1]) ? $xsdInfo[1] : ''));
				}
				
				if ($parts[0] == '@var') {
					$varInfo = explode(' ', trim($parts[1]), 2);
					$varType = $varInfo[0];
					$varDescription = isset($varInfo[1]) ? $varInfo[1] : '';
					$vars[] = array('type' => trim($varType), 'description' => trim($varDescription));
				}
				
				if ($parts[0] == '@param') {
					$paramInfo = explode(' ', trim($parts[1]), 3);
					$paramType = $paramInfo[0];
					$paramName = isset($paramInfo[1]) ? $paramInfo[1] : '';
					$paramDescription = isset($paramInfo[2]) ? $paramInfo[2] : '';
					$params[] = array('type' => trim($paramType), 'name' => trim($paramName), 'description' => trim($paramDescription));
				}
			}
			
			return array(
				'description' => trim($description),
				'return' => array('type' => $returnType, 'description' => $returnDescription),
				'params' => $params,
				'xsd' => $xsd,
				'vars' => $vars
			);
		}
	}
?>