<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * XML Schema 1.0 class
	 */
	class scXsd {
		
		/**
		 * Array of class names to be defined as types
		 * @var array
		 */
		protected $_classesAsTypes = array();
		
		/**
		 * Array of scXsdType objects
		 * @var array
		 */
		protected $_externalTypes = array();
		
		/**
		 * Defined types
		 * 
		 * @var array array of type names
		 */
		protected $_definedTypes = array();
		
		/**
		 * 
		 * @var array array of strings with class names to be defined as elements
		 */
		protected $_classesAsElements = array();
		
		/**
		 * Maps between doc comment var types and XSD types
		 * @var array
		 */
		protected $_builtinXsdTypes = array(
			// Built-in PHP types
			'bool' => 'boolean',
			'boolean' => 'boolean',
			'int' => 'int',		// 32-bit signed int
			'float' => 'float',	// 32-bit float
			'double' => 'double',	// 64-bit float
			'real' => 'float',	// 32-bit float
			'string' => 'string',
			
			// Non built-in types
			'integer' => 'integer',	// This is integer with infite size
			'nonPositiveInteger' => 'nonPositiveInteger',
			'negativeInteger' => 'negativeInteger',
			'positiveInteger' => 'positiveInteger',
			'nonNegativeInteger' => 'nonNegativeInteger',
			'unsignedLong' => 'unsignedLong',	// 64-bit unsigned long
			'unsignedInt' => 'unsignedInt',		// 32-bit unsigned int
			'unsignedShort' => 'unsignedShort',	// 16-bit unsigned short
			'unsignedByte' => 'unsignedByte',	// 8-bit unsigned byte
			'long' => 'long',	// 64-bit signed long
			'short' => 'short',	// 32-bit signed short
			'byte' => 'byte',	// 8-bit signed byte
			
			'decimal' => 'decimal',
			'duration' => 'duration',
			'dateTime' => 'dateTime',
			'time' => 'time',
			'date' => 'date',
			'gYearMonth' => 'gYearMonth',
			'gYear' => 'gYear',
			'gMonthDay' => 'gMonthDay',
			'gDay' => 'gDay',
			'gMonth' => 'gMonth',
			'hexBinary' => 'hexBinary',
			'base64Binary' => 'base64Binary',
			'anyURI' => 'anyURI',
		);
		
		/**
		 * Add class as element. Will use reflection to define the class.
		 * 
		 * @param string $className
		 */
		public function addClassAsElement($className)
		{
			$this->_classesAsElements[] = $className;
		}
		
		/**
		 * Add class as type. Will use reflection to define the class.
		 * 
		 * @param string $className
		 */
		protected function addClassAsType($className)
		{
			$this->_classesAsTypes[] = $className; // Add classes to be defined at the end
		}
		
		/**
		 * Returns true if this type is defined
		 * 
		 * @param string $varType
		 * @return boolean
		 */
		protected function isXsdTypeDefined($varType)
		{
			return isset($this->_builtinXsdTypes[$varType]) or isset($this->_externalTypes[$varType]);
		}
		
		/**
		 * Returns the XSD type from a doc comment var type
		 * 
		 * @param string $varType the internal php type
		 * @return string the XSD type
		 */
		protected function getXsdType($varType)
		{
			if (isset($this->_builtinXsdTypes[$varType]))
				return 'xsd:' . $this->_builtinXsdTypes[$varType];
			
			if (isset($this->_externalTypes[$varType]))
				return 'tns:' . $varType;
			
			throw new miException(sprintf(_i('Unknown var type "%s".'), $varType));
		}
		
		/**
		 * Adds externally created XSD type
		 * 
		 * @param scXsdType $xsdTypeObj
		 */
		public function addExternalType(scXsdType $xsdTypeObj)
		{
			$this->_externalTypes[$xsdTypeObj->getName()] = $xsdTypeObj;
		}
		
		/**
		 * Adds class as enum
		 * @param string $className
		 */
		public function addEnumClass($className)
		{
			$reflectionClass = new ReflectionClass($className);
			$constants = $reflectionClass->getConstants();
			$this->_externalTypes[$className] = scXsdSimpleTypeRestriction::createEnumeration($className, $constants);
		}
		
		/**
		 * Returns XML definition for the XSD schema
		 * 
		 * @param string $namespace
		 * @return array xml definition of the schema
		 */
		public function getSchemaXmlDefinition($namespace)
		{
			$schema = array();
			
			foreach ($this->_classesAsElements as $className) {
				$schema[] = $this->getXsdElementXmlDefinition($className);
			}
			
			foreach ($this->_externalTypes as $simpleType) {
				$schema[] = $simpleType->getXmlDefinition();
			}
			
			while (!empty($this->_classesAsTypes)) {
				$className = array_shift($this->_classesAsTypes);	// Remove elements from the begining
				if (isset($this->_definedTypes[$className]) or isset($this->_externalTypes[$className]))
					continue;
				
				$this->_definedTypes[$className] = $className;
				$schema[] = $this->getXsdTypeXmlDefinition($className);
			}
			
			return array(
				scXml::TAG => 'xsd:schema',
				scXml::ATTRIBUTES => array('xmlns' => 'http://www.w3.org/2001/XMLSchema', 'elementFormDefault' => 'qualified', 'targetNamespace' => $namespace),
				scXml::CHILDREN => $schema
			);
		}
		
		/**
		 * Returns type WSDL section
		 * 
		 * @param string $className
		 * @return array XML definition
		 */
		protected function getXsdTypeXmlDefinition($className)
		{
			$xsdModelGroup = $this->getXsdModelGroup($className);
			
			$complexType = new scXsdComplexType($className);
			$complexType->setModelGroup($xsdModelGroup);
			return $complexType->getXmlDefinition();
		}
		
		/**
		 * Returns XML definition for xsd:element, to be used:
		 * definitions -> wsdl:types -> xsd:schema -> "xsd:element"
		 * 
		 * @param string $className
		 * @return array
		 */
		protected function getXsdElementXmlDefinition($className)
		{
			$xsdModelGroup = $this->getXsdModelGroup($className);
			
			$complexType = new scXsdComplexType();
			$complexType->setModelGroup($xsdModelGroup);
			
			$typeDef = array(
				scXml::TAG => 'xsd:element',
				scXml::ATTRIBUTES => array('name' => $className),
				scXml::CHILDREN => array($complexType->getXmlDefinition())
			);
			
			return $typeDef;
		}
		
		/**
		 * Returns XSD model group object
		 * 
		 * @param string $className
		 * @return scXsdModelGroup
		 */
		protected function getXsdModelGroup($className)
		{
			$xsdOrderIndicators = array('sequence', 'all', 'choice');
			
			$class = new ReflectionClass($className);
			$classDocComment = $class->getDocComment();
			$parsedClassDocComment = scSoapWsdl::parseDocComment($classDocComment);
			
			$orderIndicator = 'sequence';
			foreach ($parsedClassDocComment['xsd'] as $xsd) {
				if ($xsd['name'] == 'order') {
					if (!in_array($xsd['value'], $xsdOrderIndicators))
						throw new miException(sprintf(_i('Invalid XSD order indicator "%s".'), $xsd['value']));
					$orderIndicator = $xsd['value'];
				}
			}
			
			$properties = $class->getProperties(256);	// Public properties
			$particles = array();
			foreach ($properties as $property) {
				$propertyName = $property->getName();
				$propertyDocComment = $property->getDocComment();
				$parsedPropertyDocComment = scSoapWsdl::parseDocComment($propertyDocComment);
				
				foreach ($parsedPropertyDocComment['vars'] as $var) {
					$particles[] = $this->getXsdParticle($parsedPropertyDocComment['xsd'], $var, $propertyName, $parsedPropertyDocComment['description']);
				}
			}
			
			return new scXsdModelGroup($orderIndicator, $particles);
		}
		
		/**
		 * Returns xsd:element for a class member.
		 * Used: definitions -> wsdl:types -> xsd:schema -> xsd:element -> xsd:complexType -> xsd:sequence -> "xsd:element"
		 * 
		 * @param array $xsdParams parsed doc comment xsd params info
		 * @param array $var parsed doc comment var info
		 * @param string $propertyName the name of the element
		 * @param string $description particle description
		 * @return array xml definition
		 */
		protected function getXsdParticle($xsdParams, $var, $propertyName, $description)
		{
			$attributes = $this->getXsdElementAttributes($xsdParams);
			$type = $var['type'];
			
			// Check if array
			if (substr($type, -2) == '[]') {
				$type = substr($type, 0, -2);
				
				// Set minOccurs=0 and maxOccurs=unbounded for arrays, unless override in var doc comment
				if (!isset($attributes['minOccurs']))
					$attributes['minOccurs'] = 0;
				if (!isset($attributes['maxOccurs']))
					$attributes['maxOccurs'] = 'unbounded';
			}
			
			$annotation = null;
			if ($description != '')
				$annotation = new scXsdAnnotation($description);
			
			if ($this->isXsdTypeDefined($type)) {
				$xsdType = $this->getXsdType($type);
				
				return new scXsdParticle($propertyName, $xsdType, $attributes, $annotation);
			}
			
			if (class_exists($type, false)) {
				$this->addClassAsType($type);
				
				return new scXsdParticle($propertyName, 'tns:' . $type, $attributes, $annotation);
			}
			
			throw new miException(sprintf(_i('Unknown type "%s".'), $type));
		}
		
		/**
		 * Convert info returned from parsed doc comment into hash with attribute name => attribute value
		 * 
		 * @param array $xsdParams
		 * @return array a hash with attributes
		 */
		protected function getXsdElementAttributes($xsdParams)
		{
			$attributes = array();
			foreach ($xsdParams as $xsdParam) {
				if ($xsdParam['name'] == 'minOccurs')
					$attributes['minOccurs'] = $xsdParam['value'];
				if ($xsdParam['name'] == 'maxOccurs')
					$attributes['maxOccurs'] = $xsdParam['value'];
			}
			return $attributes;
		}
	}
	
	
	/**
	 * Defines a XSD type
	 */
	class scXsdType {
		protected $_name;
		
		public function setName($name)
		{
			$this->_name = $name;
		}
		
		public function getName()
		{
			return $this->_name;
		}
		
		public function getXmlDefinition()
		{
			throw new miException(_i('Not implemented.'));
		}
	}
	
	class scXsdSimpleType extends scXsdType {
		
	}
	
	/**
	 * Simple type XSD which is a restriction
	 */
	class scXsdSimpleTypeRestriction extends scXsdSimpleType {
		const TYPE_ENUMERATION = 1;
		protected $_type;
		
		protected $_enumeration = array();
		
		protected function __construct($name, $type)
		{
			$this->_name = $name;
			$this->_type = $type;
		}
		
		public function getType()
		{
			return $this->_type;
		}
		
		public function getEnumeration()
		{
			return $this->_enumeration;
		}
		
		public static function createEnumeration($name, $enumeration)
		{
			$xsd = new scXsdSimpleTypeRestriction($name, self::TYPE_ENUMERATION);
			$xsd->_enumeration = $enumeration;
			return $xsd;
		}
		
		/**
		 * Returns XML definition for simple type
		 * 
		 * @param scXsdSimpleType $simpleType
		 * @return array
		 */
		public function getXmlDefinition()
		{
			if ($this->getType() != self::TYPE_ENUMERATION)
				throw new miException(_i('Only enumeration simple type is supported.'));
			
			$enumDefs = array();
			foreach ($this->getEnumeration() as $enumElem)
				$enumDefs[] = array(
					scXml::TAG => 'xsd:enumeration',
					scXml::ATTRIBUTES => array('value' => $enumElem),
				);
				
			$typeDef = array(
				scXml::TAG => 'xsd:simpleType',
				scXml::ATTRIBUTES => array('name' => $this->getName()),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'xsd:restriction',
						scXml::ATTRIBUTES => array('base' => 'xsd:string'),
						scXml::CHILDREN => $enumDefs
					)
				)
			);
			
			return $typeDef;
		}
	}
	
	/**
	 * Complex type XSD
	 */
	class scXsdComplexType extends scXsdType {
		const SIMPLE_CONTENT = 1;
		const COMPLEX_CONTENT = 2;
		const MODEL_GROUP = 3;
		
		protected $_contentType = 0;
		
		protected $_name = '';
		
		/**
		 * 
		 * @var scXsdComplexTypeSimpleContent
		 */
		protected $_simpleContent;
		
		/**
		 * 
		 * @var scXsdComplexTypeComplexContent
		 */
		protected $_complexContent;
		
		/**
		 * 
		 * @var scXsdModelGroup
		 */
		protected $_modelGroup;
		
		public function __construct($name = '')
		{
			$this->_name = $name;
		}
		
		public function setSimpleContent(scXsdComplexTypeSimpleContent $simpleContent)
		{
			$this->_contentType = self::SIMPLE_CONTENT;
			$this->_simpleContent = $simpleContent;
		}
		
		public function setComplexContent(scXsdComplexTypeComplexContent $complexContent)
		{
			$this->_contentType = self::COMPLEX_CONTENT;
			$this->_complexContent = $complexContent;
		}
		
		public function setModelGroup(scXsdModelGroup $modelGroup)
		{
			$this->_contentType = self::MODEL_GROUP;
			$this->_modelGroup = $modelGroup;
		}
		
		public function getXmlDefinition()
		{
			switch ($this->_contentType) {
				case self::SIMPLE_CONTENT:
					$children = $this->_simpleContent->getXmlDefinition();
					break;
				
				case self::COMPLEX_CONTENT:
					throw new miException(_i('Not implemented'));
					break;
				
				case self::MODEL_GROUP:
					$children = $this->_modelGroup->getXmlDefinition();
					break;
				
				default:
					throw new miException(_i('Content not set.'));
			}
			
			$typeDef = array(
				scXml::TAG => 'xsd:complexType',
				scXml::CHILDREN => array($children)
			);
			if ($this->_name != '')
				$typeDef[scXml::ATTRIBUTES] = array('name' => $this->_name);
			return $typeDef;
		}
	}
	
	class scXsdComplexTypeSimpleContent {
		/**
		 * Simple content extension element
		 * @var scXsdSimpleContentExtension
		 */
		protected $_extension = null;
		
		public function setExtension(scXsdSimpleContentExtension $extension)
		{
			$this->_extension = $extension;
		}
		
		public function getXmlDefinition()
		{
			return array(
				scXml::TAG => 'xsd:simpleContent',
				scXml::CHILDREN => array($this->_extension->getXmlDefinition())
			);
		}
	}
	
	/**
	 * Simple content extension
	 */
	class scXsdSimpleContentExtension {
		protected $_base;
		protected $_attributes;
		
		/**
		 * Contrsuct simple content extension
		 * @param string $base
		 * @param array $attributes array of scXsdAttribute objects
		 */
		public function __construct($base, $attributes)
		{
			$this->_base = $base;
			$this->_attributes = $attributes;
		}
		
		public function getXmlDefinition()
		{
			$children = array();
			foreach ($this->_attributes as $attribute)
				$children[] = $attribute->getXmlDefinition();
			
			return array(
				scXml::TAG => 'xsd:extension',
				scXml::ATTRIBUTES => array('base' => $this->_base),
				scXml::CHILDREN => $children
			);
		}
	}
	
	/**
	 * Defines XSD attribute
	 */
	class scXsdAttribute {
		protected $_name;
		protected $_type;
		protected $_use;
		
		public function __construct($name, $type, $use = 'optional')
		{
			$this->_name = $name;
			$this->_type = $type;
			$this->_use = $use;
		}
		
		public function getXmlDefinition()
		{
			$attributes = array('name' => $this->_name, 'type' => $this->_type);
			if ($this->_use != 'optional')
				$attributes['use'] = $this->_use;
			
			return array(
				scXml::TAG => 'xsd:attribute',
				scXml::ATTRIBUTES => $attributes
			);
		}
	}
	
	class scXsdComplexTypeComplexContent {
		
	}
	
	/**
	 * XSD module group - one of the all, choice or seqence elements
	 */
	class scXsdModelGroup {
		protected $_validGroupTags = array('all', 'choice', 'sequence');

		/**
		 * 
		 * @var string
		 */
		protected $_groupTag;
		
		/**
		 * @var array
		 */
		protected $_particles;
		
		/**
		 * Constructs the object
		 * @param string $groupTag
		 * @param array $particles array of scXsdParticle objects
		 */
		public function __construct($groupTag, $particles)
		{
			if (!in_array($groupTag, $this->_validGroupTags))
				throw new miException(sprintf(_i('Unknown group tag "%s".'), $groupTag));
			
			$this->_groupTag = $groupTag;
			$this->_particles = $particles;
		}
		
		public function setGroupTag($groupTag)
		{
			$this->_groupTag = $groupTag;
		}
		
		public function getGroupTag()
		{
			return $this->_groupTag;
		}
		
		public function getXmlDefinition()
		{
			$varsTypeDef = array();
			foreach ($this->_particles as $particle) {
				$varsTypeDef[] = $particle->getXmlDefinition();
			}
			
			return array(
				scXml::TAG => 'xsd:' . $this->_groupTag,
				scXml::CHILDREN => $varsTypeDef
			);
		}
	}
	
	/**
	 * XSD schema particle
	 * Currently supports only element declration 
	 */
	class scXsdParticle {
		protected $_name;
		protected $_type;
		protected $_attributes;
		
		/**
		 * @var scXsdAnnotation
		 */
		protected $_annotation;
		
		/**
		 * Constructs the particle object
		 * 
		 * @param string $name
		 * @param string $type
		 * @param array $attributes
		 * @param scXsdAnnotation|null $annotation
		 */
		public function __construct($name, $type, $attributes, scXsdAnnotation $annotation = null)
		{
			$this->_name = $name;
			$this->_type = $type;
			$this->_attributes = $attributes;
			$this->_annotation = $annotation;
		}
				
		public function getXmlDefinition()
		{
			$attributes = array('name' => $this->_name, 'type' => $this->_type);
			$attributes = array_merge($attributes, $this->_attributes);
			
			$result = array(
				scXml::TAG => 'xsd:element',
				scXml::ATTRIBUTES => $attributes,
			);
			if ($this->_annotation != null)
				$result[scXml::CHILDREN] = array($this->_annotation->getXmlDefinition());
			return $result;
		}
	}
	
	/**
	 * XSD annotation
	 */
	class scXsdAnnotation {
		/**
		 * Human-readable description
		 * @var string
		 */
		protected $_description;
		
		/**
		 * Creates the object
		 * 
		 * @param string $description
		 */
		public function __construct($description)
		{
			$this->_description = $description;
		}
		
		public function getXmlDefinition()
		{
			return array(
				scXml::TAG => 'xsd:annotation',
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'xsd:documentation',
						scXml::DATA => $this->_description
						)
				)
			);
		}
	}
?>