<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scTax {
		
		/**
		 * If the store works in this display mode then the customer sees only one price.
		 * This price could be with or without taxes (for a single product) depending on the following:
		 * 1) Whether the customer did select the 'Include with Display Price' checkbox for a single Tax and;
		 * 2) Whether the customer did assign the Tax to the product;
		 */
		const DISPLAY_PRICES_TAX_MODE_WITH_TAX = 0;
		
		/**
		 * If the store works in this display mode then the customer sees both prices (with and without taxes)
		 */
		const DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX = 1;
		
		protected $_taxId;
		protected $_taxInfo;
		
		protected static $_cache = array();
		protected static $_prepared = array();
		
		/**
		 * Constructs scTax object
		 *
		 * @param int $taxId
		 */
		public function __construct($taxId)
		{
			$this->_taxId = $taxId;
			
			$sql = 'SELECT * FROM Taxes JOIN TaxesML USING(TaxID) WHERE Taxes.TaxID=? AND LanguageID=?';
			 $rows = miStaticDBUtil::execSelect($sql, array($taxId, scMultiLanguage::getCurrentLanguageId()));
			 if (count($rows) == 0)
			 	throw new miException(_t('Cannot find tax record.'));
			 $this->_taxInfo = $rows[0];
		}
		
		/**
		 * Returns if the shipping is taxed
		 * 
		 * @return boolean true if the shipping is taxed
		 */
		public function isShippingTaxed()
		{
			return $this->_taxInfo['TaxApplyToShipping'] ? true : false;
		}
		
		/**
		 * Returns a string describing the tax
		 * 
		 * @return string the tax label
		 */
		public function getTaxLabel()
		{
			return $this->_taxInfo['TaxName'];
		}
		
		/**
		 * Find the tax rate
		 * 
		 * @static 
		 * @param int $taxZoneId
		 * @param int $taxClassId
		 * @return float the tax rate, or 0 if the tax rate has not been set
		 */
		public static function getTaxRate($taxZoneId, $taxClassId)
		{
			$sql = 'SELECT TaxRate FROM TaxRates WHERE TaxZoneID=? AND TaxClassID=?';
			$rows = miStaticDBUtil::execSelect($sql, array($taxZoneId, $taxClassId));
			if (count($rows) == 0)
				return 0;
			
			return $rows[0]['TaxRate'];
		}
		
		public static function getTaxInfoByTaxClassId($taxClassId)
		{
			static $taxInfosByClass = array();
			
			if (empty($taxInfosByClass)) {
				$results = miStaticDBUtil::execSelect('SELECT Taxes.*, TaxClasses.TaxClassID FROM Taxes INNER JOIN TaxClasses USING(TaxID)');
				if (count($results) == 0) {
					throw new miDBException(_i('Tax Classes not defined.'), miDBException::EXCEPTION_RECORD_NOT_FOUND);
				}
				foreach ($results as $result) {
					$taxInfosByClass[$result['TaxClassID']] = $result;
				}
			}
			
			if (!isset($taxInfosByClass[$taxClassId])) {
				throw new miDBException(sprintf(_i('Tax Class ID "%s" not found.'), $taxClassId), miDBException::EXCEPTION_RECORD_NOT_FOUND);
			}
			return $taxInfosByClass[$taxClassId];
		}
		
		/**
		 * Returns array with keys price modes and values mode names
		 *
		 * @return array
		 */
		public static function getDisplayPricesTaxModes()
		{
			return array(
				self::DISPLAY_PRICES_TAX_MODE_WITH_TAX => _t('Include tax with price'),
				self::DISPLAY_PRICES_TAX_MODE_WITH_AND_WITHOUT_TAX => _t('Display with and without tax')
			);
		}
		
		public static function getAllTaxClassIds()
		{
			$query = 'SELECT TaxID, TaxClassID FROM TaxClasses';
			return miUtil::getDBArray($query, 'TaxClassID', 'TaxID');
		}
		
		/**
		 * Returns all tax classes, zones and rates
		 *
		 * @return array
		 */
		public static function getAllTaxRates()
		{
			$taxClasses = new miSqlRecordset('TaxClasses');
			$taxClasses->addJoinCondition('INNER', 'TaxRates', 'USING(TaxClassID)');
			$taxClasses->addJoinCondition('INNER', 'TaxZones', 'USING(TaxZoneID)');
			return $taxClasses->getAllRecords();
		}
		
		public static function getAllTaxNames()
		{
			$query = 'SELECT TaxID, TaxName FROM TaxesML WHERE LanguageID=?';
			return miStaticDBUtil::execSelect($query, array(scMultiLanguage::getCurrentLanguageId()));
		}
		
		public static function getTaxName($taxId)
		{
			return scUtil::getMLDbValue('TaxesML', 'TaxName', 'TaxID', $taxId);
		}
		
		/**
		 * Returns the tax class ids of a product
		 * 
		 * @static
		 * @param int $productId
		 * @return array array('TaxClassID' => 'TaxClassID')
		 */
		public static function getProductTaxClasses($productId)
		{
			// Check if there is data in the cache and use it.
			if (isset(self::$_cache[$productId])) {
				return self::$_cache[$productId];
			}
			
			// Check if the requested item is already prepared.
			// Add it to the prepare list if it is not.
			if (!in_array($productId, self::$_prepared)) {
				self::$_prepared[] = intval($productId);
			}
			
			// Cache the Product TaxClass data
			$recordset = new miSqlRecordset('ProductTaxClasses');
			$recordset->addFilter(new miSqlFilterIn('ProductID', self::$_prepared));
			$result = $recordset->getAllRecords();
			
			// Store empty array for all the items that were just fetched.
			// Those values are later (a few lines later) replaced with real values where available.
			foreach (self::$_prepared as $preparedId) {
				self::$_cache[$preparedId] = array();
			}
			self::$_prepared = array();
			
			if (empty($result)) {
				// Nothing to cache.
				return array();
			}
			
			// Add the results to the cache.
			$productsTaxClasses = array();
			foreach ($result as $row) {
				if (!isset($productsTaxClasses[$row['ProductID']])) {
					$productsTaxClasses[$row['ProductID']] = array();
				}
				$productsTaxClasses[$row['ProductID']][] = $row['TaxClassID'];
			}
			self::$_cache = $productsTaxClasses + self::$_cache;
			return self::$_cache[$productId];
		}
		
		/**
		 * Computes the tax for a cart item
		 * 
		 * @static 
		 * @param float $taxRate the tax rate
		 * @param scCartItem $cartItem the cart item
		 * @return scMultiCurrencyAmount the tax
		 */
		public static function getTax($taxRate, scCartItem $cartItem)
		{
			return scMultiCurrencyAmountMath::multiply($cartItem->getTotal(), ($taxRate / 100));
		}
		
		/**
		 * Add items that will be used later in the script
		 * @param array $productIds
		 */
		public static function prepareProducts($productIds)
		{
			$productIds = array_map('intval', $productIds);
			
			self::$_prepared = array_merge(self::$_prepared, $productIds);
			self::$_prepared = array_unique(self::$_prepared);
		}
	}
?>