<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scSeoUrlTypes {
		
		const URL_LANGUAGE = 'urlLang';
		const URL_PAGE = 'urlPage';
		const URL_CURRENCY = 'urlCurrency';
		const URL_QUERY = 'urlQuery';
		
		const PAGE_TYPE = 'pageType';
		const PAGE_ID = 'pageId';
		
		const LANGUAGE_NAME_HTML = 1;
		const NAME_HTML_LANGUAGE = 2;
		const NAME_LANGUAGE_HTML = 3;
		
		protected static $_urlTypes = array(
			self::LANGUAGE_NAME_HTML => 'en/page.html',
			self::NAME_HTML_LANGUAGE => 'page.html.en',
			self::NAME_LANGUAGE_HTML => 'page.en.html',
		);
		
		protected static $_expressions = array(
			self::LANGUAGE_NAME_HTML => '/^((?P<language>[a-z]{2})\\/)?(?P<page>.+)\.html$/',
			self::NAME_HTML_LANGUAGE => '/^(?P<page>.+)\.html(\.(?P<language>[a-z]{2}))?$/',
			self::NAME_LANGUAGE_HTML => '/^(?P<page>.+?)(\.(?P<language>[a-z]{2}))?\.html$/',
		);
		
		protected static $_prefixes = array(
			scFriendlyUrl::TYPE_MANUFACTURER => 'manufacturer',
			scFriendlyUrl::TYPE_PRODUCT => 'product',
			scFriendlyUrl::TYPE_PRODUCT_VARIANT => 'productv',
			scFriendlyUrl::TYPE_RICH_TEXT_PAGE => 'page',
			scFriendlyUrl::TYPE_DYNAMIC_FORM => 'form',
			scFriendlyUrl::TYPE_CATEGORY => 'category',
			scFriendlyUrl::TYPE_GALLERY => 'gallery',
			scFriendlyUrl::TYPE_NEWS => 'news',
			scFriendlyUrl::TYPE_WISHLIST => 'wishlist',
			scFriendlyUrl::TYPE_PRODUCT_COMPARISON => 'product_comparison',
		);
		
		public static function getAllTypes()
		{
			return self::$_urlTypes;
		}
		
		/**
		 * @param string $url
		 * @return array|false
		 */
		public static function splitUrl($url)
		{
			$type = scSettings::getSettingValue(scSettings::SETTING_SEO_URLS_TYPE);
			return self::splitUrlOfType($url, $type);
		}
		
		/**
		 * Returns split URL
		 * 
		 * For /bg/productv/2217/puma-panigale-50-mega.html?Query=String will return
		 * $split = array(
		 * 	self::URL_PAGE => 'productv/2217/puma-panigale-50-mega',
		 *  self::URL_QUERY => 'Query=String',
		 *  self::URL_LANGUAGE => 'BG',
		 *  self::URL_CURRENCY => 'BGN'
		 * );
		 * 
		 * Note: Will always return the base currency.
		 * 
		 * @param string $url
		 * @param int $type LANGUAGE_NAME_HTML, NAME_HTML_LANGUAGE or NAME_LANGUAGE_HTML
		 * @return array $split
		 * @throws miException
		 */
		public static function splitUrlOfType($url, $type)
		{
			if (!isset(self::$_expressions[$type])) {
				throw new miException(sprintf(_i('Unknown SEO URL type "%s".'), $type));
			}
			
			$parsed = scUrlUtil::parse($url);
			
			$path = substr($parsed[scUrlUtil::PATH], 1); // Remove the starting slash ('/')
			
			$regex = self::$_expressions[$type];
			if (!preg_match($regex, $path, $matches)) {
				// Complete mismatch.
				return false;
			}
			if (!isset($matches['page']) && empty($matches['page'])) {
				// No page matched.
				return false;
			}
			
			$split = array(
				self::URL_PAGE => $matches['page'],
				self::URL_QUERY => '',
				self::URL_LANGUAGE => scMultiLanguage::getDefaultLanguageCode(),
				self::URL_CURRENCY => scCurrencyUtil::getBaseCurrencyCode(),
			);
			if (isset($parsed[scUrlUtil::QUERY])) {
				$split[self::URL_QUERY] = $parsed[scUrlUtil::QUERY];
			}
			
			if (isset($matches['language']) && !empty($matches['language'])) {
				$split[self::URL_LANGUAGE] = strtoupper($matches['language']);
			}
			
			// TODO: Handle currency
			
			return $split;
		}
		
		/**
		 * Return page code and page id
		 * 
		 * Example:
		 * For /bg/productv/2217/puma-panigale-50-mega.html?Query=String will return
		 * $split = array(
		 *  self::PAGE_TYPE => scFriendlyUrl::TYPE_PRODUCT_VARIANT,
		 *  self::PAGE_ID => '2217/puma-panigale-50-mega'
		 * );
		 * 
		 * @param array $page Page as matched by splitUrl()
		 * @return $split info
		 */
		public static function splitPage($page)
		{
			$split = array();
			
			if ($page[self::URL_PAGE] == 'manufacturers') {
				$split[self::PAGE_TYPE] = scFriendlyUrl::TYPE_MANUFACTURER;
				return $split;
			}
			if ($page[self::URL_PAGE] == 'news') {
				$split[self::PAGE_TYPE] = scFriendlyUrl::TYPE_NEWS;
				return $split;
			}
			if ($page[self::URL_PAGE] == 'wishlist') {
				$split[self::PAGE_TYPE] = scFriendlyUrl::TYPE_WISHLIST;
				return $split;
			}
			if ($page[self::URL_PAGE] == 'product_comparison') {
				$split[self::PAGE_TYPE] = scFriendlyUrl::TYPE_PRODUCT_COMPARISON;
				return $split;
			}
			
			// For URL_PAGE 'productv/2217/puma-panigale-50-mega' matches['id'] will be '2217/puma-panigale-50-mega'
			if (preg_match('/^(?P<type>[^\\/]+)\\/(?P<id>.+)$/', $page[self::URL_PAGE], $matches)) {
				$split[self::PAGE_TYPE] = self::getPageTypeForPrefix($matches['type']);
				if ($split[self::PAGE_TYPE] !== false) {
					$split[self::PAGE_ID] = $matches['id'];
					return $split;
				}
			}
			
			$linkablePages = scFriendlyUrl::getLinkableDynamicPages();
			unset($linkablePages[scFriendlyUrl::DYN_PAGE_MANUFACTURERS], $linkablePages[scFriendlyUrl::DYN_PAGE_NEWS], $linkablePages[scFriendlyUrl::DYN_PAGE_WISHLIST]);
			foreach ($linkablePages as $linkablePageType => $linkablePage) {
				if ($page[self::URL_PAGE] == basename($linkablePage, '.php')) {
					$split[self::PAGE_TYPE] = scFriendlyUrl::TYPE_DYNAMIC_PAGE;
					$split[self::PAGE_ID] = $linkablePageType;
					return $split;
				}
			}
			
			return false;
		}
		
		protected static function getPageTypeForPrefix($prefix)
		{
			return array_search($prefix, self::$_prefixes);
		}
	}
?>