<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scUrlResolver {
		
		/**
		 * @return scUrlResolver
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scUrlResolver();
			}
			return $instance;
		}
		
		protected function __construct()
		{
		}
		
		/**
		 * @return scUrlResolverResult or false
		 */
		public function resolve($uri)
		{
			if (empty($uri)) {
				return false;
			}
			
			$defaultUrlType = scSettings::getSettingValue(scSettings::SETTING_SEO_URLS_TYPE);
			
			$result = $this->resolveTempletizedUrl($uri, $defaultUrlType);
			$types = array_keys(scSeoUrlTypes::getAllTypes());
			
			if ($result === false) {
				// The url does not match the type as set in Settings. Try the other types
				foreach ($types as $type) {
					if ($defaultUrlType == $type)
						continue;
					
					$result = $this->resolveTempletizedUrl($uri, $type);
					if ($result !== false) {
						return $result;
					}
				}
			}
			
			$parsedUri = scUrlUtil::parse($uri);
			$uri = $parsedUri[scUrlUtil::PATH];
			if (isset($parsedUri[scUrlUtil::QUERY]))
				$uri .= '?' . $parsedUri[scUrlUtil::QUERY];
			$uri = trim(urldecode($uri));
			// Remove the starting slash ('/')
			$uri = substr($uri, 1);
			
			// Try to find custom url
			if ($result === false) {
				$result = $this->resolveCustomUrl($uri);
			}
			
			// Still no match. Try with simple urls (cart.php, product.php?ProductID=2, etc.)
			if ($result === false) {
				$result = $this->resolveSimpleUrl($uri, $parsedUri);
			}
			
			if ($result === false) {
				return false;
			}
			
			return $result;
		}
		
		/**
		 * Checks if a $uri is supposed to go to the page specified by $pageType and $pageId.
		 * @return bool
		 */
		public function is($uri, $pageType, $pageId)
		{
			$uri = '/' . scUrlUtil::convertUrl($uri, scUrlUtil::URL_TYPE_RELATIVE);
			$result = $this->resolve($uri);
			if ($result === false) {
				return false;
			}
			return (($result->getPageType() == $pageType) && ($result->getIdValue() == $pageId));
		}
		
		/**
		 * @return scUrlResolverResult or false
		 */
		protected function resolveTempletizedUrl($uri, $urlType)
		{
			$split = scSeoUrlTypes::splitUrlOfType($uri, $urlType);
			if ($split === false) {
				return false;
			}
			
			$pageData = scSeoUrlTypes::splitPage($split);
			if ($pageData === false) {
				return false;
			}
			$split[scSeoUrlTypes::URL_PAGE] = $pageData;
			
			$resolver = scUrlResolverFactory::getUrlResolverForPageType($pageData[scSeoUrlTypes::PAGE_TYPE]);
			return $resolver->resolve($split);
		}
		
		/**
		 * @return scUrlResolverResult or false
		 */
		protected function resolveCustomUrl($requestedUri)
		{
			$matchedUrl = $this->getBestMatch($requestedUri);
			if ($matchedUrl === false) {
				return false;
			}
			
			$resolver = scUrlResolverFactory::getUrlResolverForPageType($matchedUrl['CustomUrlEntityCode']);
			$resolverResult = $resolver->resolveCustomUrl($matchedUrl);
			$resolverResult->setLanguageCode(scMultiLanguage::singleton()->getLanguageCodeById($matchedUrl['LanguageID']));
			if (!$matchedUrl['CustomUrlIsCurrent']) {
				// If it is not a primary URL check if a primary one exists.
				$primaryUrl = scUrlUtil::convertUrl($resolverResult->getPrimaryUrl(), scUrlUtil::URL_TYPE_ABSOLUTE);
				$resolvedUrl = scUrlUtil::convertUrl($requestedUri, scUrlUtil::URL_TYPE_ABSOLUTE);
				if ($primaryUrl != $resolvedUrl) {
					$resolverResult->setIsPrimary(false);
				}
			}
			return $resolverResult;
		}
		
		/**
		 * Finds the custom URL that matches the best.
		 * Returns the record the way it would be returned with a SELECT to the DB.
		 * 
		 * The admin should be able to specify:
		 * /sad/blah.php
		 * (which should match ignoring query string)
		 * 
		 * /sad/blah.php?something=2
		 * (whcih should match including query string, but still allow other query variables, meaning that /sad/blah.php?something=2&x=5 and /sad/blah.php?x=5&something=2) should also match.
		 * If the admin has configured the same base url, but with different query params, which all match, we should use the one which has the highest number of query params:
		 * Both of these will match /sad/blah.php?something=2&x=5&z=8:
		 * /sad/blah.php?something=2
		 * /sad/blah.php?something=2&x=5
		 * but we should use the second, as it is more specific (more params).
		 * 
		 * Return FALSE if no match was found.
		 */
		protected function getBestMatch($requestedUri)
		{
			$parsedUri = scUrlUtil::parse($requestedUri);
			
			$requestedBaseUri = $requestedUri;
			$requestedQuery = '';
			if (isset($parsedUri[scUrlUtil::QUERY])) {
				// If there is a query string remove it from the base URI.
				// Store it for further processing. 
				$requestedQuery = $parsedUri[scUrlUtil::QUERY];
				unset($parsedUri[scUrlUtil::QUERY]);
				$requestedBaseUri = scUrlUtil::glue($parsedUri);
			}
			
			// Try to match the base URI
			$sql = 'SELECT * FROM CustomUrls INNER JOIN CustomUrlsML USING(CustomUrlID) WHERE CustomUrlHash=?';
			$results = miStaticDBUtil::execSelect($sql, array(sprintf('%u', crc32($requestedBaseUri))));
			if (!count($results)) {
				return false;
			}
			
			$matchedUrl = false;
			
			// If the query strings are processed they need to be sorted first.
			$requestedQuery = scUrlUtil::parseStr($requestedQuery);
			ksort($requestedQuery);
			
			foreach ($results as $testUrlData) {
				// First check for an exact match
				if ($testUrlData['CustomUrlText'] == $requestedUri) {
					$matchedUrl = $testUrlData;
					break;
				}
				
				// It is not an exact match so the query strings need to be compared.
				if (empty($requestedQuery)) {
					// If the requested URL has no query string
					// it has to be an exact match.
					continue;
				}
				
				$parsedTestUrl = scUrlUtil::parse($testUrlData['CustomUrlText']);
				
				if (!isset($parsedTestUrl[scUrlUtil::QUERY])) {
					// If the test URL does not have a query string
					// then it's a partial match. There may be a better match than it.
					// We'll be storing number of matching query parameters in 'matchedQueryParams'
					// for each URL. All we'll need to do is check if the one we've found
					// is better than the one stored.
					if ($matchedUrl === false) {
						$testUrlData['matchedQueryParams'] = 0;
						$matchedUrl = $testUrlData;
					}
					continue;
				}
				
				$testQuery = scUrlUtil::parseStr($parsedTestUrl[scUrlUtil::QUERY]);
				ksort($testQuery);
				
				// All the query parameters in the test URL are considered REQUIRED.
				// If any one of them is missing the URL is not a match.
				if (array_diff_key($testQuery, $requestedQuery)) {
					// Some of the parameters are of the test URL are not set in the requested one.
					continue;
				}
				if (array_diff($testQuery, $requestedQuery)) {
					// Some of the values in the requested URL differ from those in the test one.
					continue;
				}
				
				$testUrlData['matchedQueryParams'] = count($testQuery);
				if (($matchedUrl === false) || $matchedUrl['matchedQueryParams'] < $testUrlData['matchedQueryParams']) {
					$matchedUrl = $testUrlData;
				}
			}
			
			return $matchedUrl;
		}
		
		/**
		 * Resolves URLs of the type cart.php, product.php?ProductID=2, etc.
		 * @return scUrlResolverResult or false
		 */
		protected function resolveSimpleUrl($url, $parsedUrl)
		{
			if (!isset($parsedUrl[scUrlUtil::PATH])) {
				// Nothing can be done without path
				return false;
			}
			$path = $parsedUrl[scUrlUtil::PATH];
			$path = substr($path, 1);
			
			// Get linkable dynamic pages
			// Try to match the type (e.g. addresses.php, cart.php, etc.)
			$linkableDynPages = scFriendlyUrl::getLinkableDynamicPages();
			$pageId = array_search($path, $linkableDynPages);
			if ($pageId !== false) {
				// Matched a page type. Get the resolver for it and use it to make the ResolverResult
				$resolver = scUrlResolverFactory::getUrlResolverForPageType(scFriendlyUrl::TYPE_DYNAMIC_PAGE);
				$languageCode = miGetParamDefault('UILanguage', scMultiLanguage::getCurrentLanguageCode());
				$customUrlData = array(
					'CustomUrlEntityID' => $pageId,
					'LanguageID' => scMultiLanguage::singleton()->getLanguageIdByCode($languageCode),
				);
				return $resolver->resolveCustomUrl($customUrlData);
			}
			
			// Did not match a Dynamic page. Try to match other pages (e.g. product.php, categories.php, etc.)
			$prefixes = scFriendlyUrl::getUrlPrefixes();
			foreach ($prefixes as $type => $prefix) {
				// Match the begining of the url against the prefixes.
				$pos = strpos($url, $prefix);
				if (($pos === false) || ($pos > 0)) {
					continue;
				}
				
				// Consider the first integer after the prefix the ID.
				// That's the way URLs are generated by the system currently.
				$queryString = substr($url, $pos);
				if (!preg_match('/[0-9]+/', $queryString, $match) || empty($match)) {
					return false;
				}
				
				$resolver = scUrlResolverFactory::getUrlResolverForPageType($type);
				$languageCode = miGetParamDefault('UILanguage', scMultiLanguage::getCurrentLanguageCode());
				$customUrlData = array(
					'CustomUrlEntityID' => $match[0],
					'LanguageID' => scMultiLanguage::singleton()->getLanguageIdByCode($languageCode),
				);
				return $resolver->resolveCustomUrl($customUrlData);
			}
			
			return false;
		}
	}
	
	interface IUrlResolver {
		
		/**
		 * Resolves which script should handle the request, and all info needed to properly decode which page to display
		 * 
		 * @param array $urlData
		 * @todo $urlData may be object
		 * @return scUrlResolverResult
		 */
		public function resolve($urlData);
		
		/**
		 * Resolves which script should handle the request, and all info needed to properly decode which page to display
		 * To be used when a custom url is detected
		 * 
		 * @param array $customUrlData
		 * @todo $customUrlData may be object
		 * @return scUrlResolverResult
		 */
		public function resolveCustomUrl($customUrlData);
		
		/**
		 * Return page name in the requested language
		 * 
		 * @param int $id
		 * @param string $languageCode (optional)
		 * @return string the page name
		 */
		public function getName($id, $languageCode = null);
		
		/**
		 * Get the page names in all languages
		 * @param int $id
		 * @return array array(languageId => pageName)
		 */
		public function getNames($id);
	}
	
	/**
	 * Object to store data on friendly url resolving.
	 */
	class scUrlResolverResult {
		
		/**
		 * The query params
		 * This will hold the query params, and additional query params that needs to be set for the handling script 
		 * @var array
		 */
		protected $_params = array();
		
		/**
		 * The script name, which will display the page
		 * E.g: product.php
		 * @var string
		 */
		protected $_handlingScript = null;
		
		/**
		 * 2-character language code
		 * @var string
		 */
		protected $_languageCode = null;
		
		/**
		 * One of scFriendlyUrl::TYPE_* constants
		 * @var int
		 */
		protected $_pageType = null;
		
		/**
		 * The ID of the object (e.g. ProductID, ManufacturerID, etc)
		 * @var int
		 */
		protected $_idValue = null;
		
		/**
		 * The SEO part of the url
		 * e.g. in url '/bg/productv/2217/puma-panigale-50-mega.html' it will be 'puma-panigale-50-mega'
		 * @var string
		 */
		protected $_pageNameForUrl = null;
		
		/**
		 * Is this URL the current one for this page.
		 * @var bool
		 */
		protected $_isPrimary = true;
		
		public function getPrimaryUrl()
		{
			return scFriendlyUrl::get($this->_pageType, $this->_idValue, $this->_pageNameForUrl, $this->_languageCode);
		}
		
		public function getPageNameForUrl()
		{
			return $this->_pageNameForUrl;
		}
		
		public function getPageType()
		{
			return $this->_pageType;
		}
		
		public function getIdValue()
		{
			return $this->_idValue;
		}
		
		public function getIsPrimary()
		{
			return $this->_isPrimary;
		}
		
		public function getParams()
		{
			return $this->_params;
		}
		
		public function getLanguageCode()
		{
			return $this->_languageCode;
		}
		
		public function getHandlingScript()
		{
			return $this->_handlingScript;
		}
		
		
		public function setPageNameForUrl($name)
		{
			$this->_pageNameForUrl = $name;
		}
		
		public function setParam($paramName, $paramValue)
		{
			$this->_params[$paramName] = $paramValue;
		}
		
		public function setIdValue($idValue)
		{
			$this->_idValue = $idValue;
		}
		
		public function setHandlingScript($script)
		{
			$this->_handlingScript = $script;
		}
		
		public function setLanguageCode($languageCode)
		{
			$this->_languageCode = $languageCode;
		}
		
		public function setPageType($type)
		{
			$this->_pageType = $type;
		}
		
		public function setIsPrimary($isPrimary)
		{
			$this->_isPrimary = $isPrimary;
		}
	}
	
	abstract class scBaseUrlResolver implements IUrlResolver {
		
		protected static $_names = array();
		
		protected $_pageType;
		
		protected $_idField;
		
		protected $_handlingScript;
		
		protected $_tableName;
		
		protected $_nameField;
		
		public function resolve($urlData)
		{
			if (!$this->isUrlDataValid($urlData)) {
				return false;
			}
			
			$pageData = &$urlData[scSeoUrlTypes::URL_PAGE];
			
			$pageData[scSeoUrlTypes::PAGE_ID] = explode('/', $pageData[scSeoUrlTypes::PAGE_ID]);
			$id = $pageData[scSeoUrlTypes::PAGE_ID][0];
			if ($id <= 0) {
				return false;
			}
			
			$customUrlData = array(
				'CustomUrlEntityID' => $id,
				'LanguageID' => scMultiLanguage::singleton()->getLanguageIdByCode($urlData[scSeoUrlTypes::URL_LANGUAGE]),
			);
			$result = $this->resolveCustomUrl($customUrlData);
			$result->setPageNameForUrl($pageData[scSeoUrlTypes::PAGE_ID][1]);
			return $result;
		}
		
		/**
		 * @return scUrlResolverResult
		 */
		public function resolveCustomUrl($customUrlData)
		{
			$result = new scUrlResolverResult();
			$result->setHandlingScript($this->getHandlingScript());
			$result->setPageType($this->_pageType);
			if (isset($customUrlData['CustomUrlEntityID'])) {
				$result->setIdValue($customUrlData['CustomUrlEntityID']);
				$result->setParam($this->_idField, $customUrlData['CustomUrlEntityID']);
			}
			if (isset($customUrlData['LanguageID'])) {
				$result->setLanguageCode(scMultiLanguage::singleton()->getLanguageCodeById($customUrlData['LanguageID']));
			}
			return $result;
		}
		
		public function getHandlingScript()
		{
			return $this->_handlingScript;
		}
		
		public function getNames($id)
		{
			if (isset(self::$_names[$id])) {
				return self::$_names[$id];
			}
			
			$recordset = new miSqlRecordset($this->_tableName . 'ML');
			$recordset->addFilter(new miSqlFilterEqual($this->_idField, $id));
			$recordset->setSelectFields(array(
				'LanguageID',
				$this->_nameField,
			));
			$recordset = $recordset->getAllRecords();
			self::$_names[$id] = miUtil::getKeyValueArray($recordset, 'LanguageID', $this->_nameField);
			
			return self::$_names[$id];
		}
		
		public function getName($id, $languageCode = null)
		{
			$names = $this->getNames($id);
			if (is_null($languageCode)) {
				$languageId = scMultiLanguage::getCurrentLanguageId();
			} else {
				$languageId = scMultiLanguage::singleton()->getLanguageIdByCode($languageCode);
			}
			if (!isset($names[$languageId])) {
				return '';
			}
			return $names[$languageId];
		}
		
		protected function isUrlDataValid($urlData)
		{
			$pageData = $urlData[scSeoUrlTypes::URL_PAGE];
			if ($pageData[scSeoUrlTypes::PAGE_TYPE] != $this->_pageType) {
				return false;
			}
			if (empty($pageData[scSeoUrlTypes::PAGE_ID])) {
				return false;
			}
			return true;
		}
	}
	
	class scProductsUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_PRODUCT;
		
		protected $_idField = 'ProductID';
		
		protected $_handlingScript = 'product.php';
		
		protected $_tableName = 'Products';
		
		protected $_nameField = 'ProductName';
	}
	
	class scCategoryUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_CATEGORY;
		
		protected $_idField = 'CategoryID';
		
		protected $_handlingScript = 'categories.php';
		
		protected $_tableName = 'Categories';
		
		protected $_nameField = 'CategoryName';
	}
	
	class scGalleryUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_GALLERY;
		
		protected $_idField = 'GalleryID';
		
		protected $_handlingScript = 'gallery.php';
		
		protected $_tableName = 'Galleries';
		
		protected $_nameField = 'GalleryTitle';
	}
	
	class scProductVariantsUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_PRODUCT_VARIANT;
		
		protected $_idField = 'ProductVariantID';
		
		protected $_handlingScript = 'product.php';
		
		public function getNames($id)
		{
			if (isset(self::$_names[$id])) {
				return self::$_names[$id];
			}
			
			$productId = scUtil::getDbValue('ProductVariants', 'ProductID', 'ProductVariantID', $id);
			$resolver = new scProductsUrlResolver();
			self::$_names[$id] = $resolver->getNames($productId);
			return self::$_names[$id];
		}
	}
	
	class scRichTextPageUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_RICH_TEXT_PAGE;
		
		protected $_idField = 'PageID';
		
		protected $_handlingScript = 'page.php';
		
		public function getNames($id)
		{
			if (isset(self::$_names[$id])) {
				return self::$_names[$id];
			}
			
			$recordset = new miSqlRecordset('CMSRichTextPagesML');
			$recordset->addFilter(new miSqlFilterEqual('CMSRichTextPageID', $id));
			$recordset->setSelectFields(array(
				'LanguageID',
				'CMSRichTextPageTitle',
			));
			$recordset = $recordset->getAllRecords();
			self::$_names[$id] = miUtil::getKeyValueArray($recordset, 'LanguageID', 'CMSRichTextPageTitle');
			
			return self::$_names[$id];
		}
	}
	
	class scFormUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_DYNAMIC_FORM;
		
		protected $_idField = 'FormID';
		
		protected $_handlingScript = 'form.php';
		
		public function getNames($id)
		{
			if (isset(self::$_names[$id])) {
				return self::$_names[$id];
			}
			
			$recordset = new miSqlRecordset('CMSFormsML');
			$recordset->addFilter(new miSqlFilterEqual('CMSFormID', $id));
			$recordset->setSelectFields(array(
				'LanguageID',
				'CMSFormTitle',
			));
			$recordset = $recordset->getAllRecords();
			self::$_names[$id] = miUtil::getKeyValueArray($recordset, 'LanguageID', 'CMSFormTitle');
			
			return self::$_names[$id];
		}
	}
	
	class scManufacturersUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_MANUFACTURER;
		
		protected $_idField = 'ManufacturerID';
		
		protected $_handlingScript = 'manufacturers.php';
		
		protected $_tableName = 'Manufacturers';
		
		protected $_nameField = 'ManufacturerName';
		
		public function resolve($urlData)
		{
			$pageData = $urlData[scSeoUrlTypes::URL_PAGE];
			if ($pageData[scSeoUrlTypes::PAGE_TYPE] != scFriendlyUrl::TYPE_MANUFACTURER) {
				return false;
			}
			if (!isset($pageData[scSeoUrlTypes::PAGE_ID])) {
				scMultiLanguage::singleton()->setCurrentLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
				return $this->resolveCustomUrl(array());
			}
			
			if (empty($pageData[scSeoUrlTypes::PAGE_ID])) {
				return false;
			}
			
			list($manufacturerId) = explode('/', $pageData[scSeoUrlTypes::PAGE_ID]);
			if ($manufacturerId <= 0) {
				return false;
			}
			
			$result = $this->resolveCustomUrl(array('CustomUrlEntityID' => $manufacturerId));
			$result->setLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
			return $result;
		}
	}
	
	class scNewsUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_NEWS;
		
		protected $_idField = 'NewsID';
		
		protected $_handlingScript = 'module.php';
		
		protected $_tableName = 'News';
		
		protected $_nameField = 'NewsTitle';
		
		public function resolve($urlData)
		{
			$pageData = $urlData[scSeoUrlTypes::URL_PAGE];
			if ($pageData[scSeoUrlTypes::PAGE_TYPE] != scFriendlyUrl::TYPE_NEWS) {
				return false;
			}
			if (!isset($pageData[scSeoUrlTypes::PAGE_ID])) {
				scMultiLanguage::singleton()->setCurrentLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
				return $this->resolveCustomUrl(array());
			}
			
			if (empty($pageData[scSeoUrlTypes::PAGE_ID])) {
				return false;
			}
			
			list($id) = explode('/', $pageData[scSeoUrlTypes::PAGE_ID]);
			if ($id <= 0) {
				return false;
			}
			
			$result = $this->resolveCustomUrl(array('CustomUrlEntityID' => $id));
			$result->setLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
			return $result;
		}
		
		public function resolveCustomUrl($customUrlData)
		{
			$result = parent::resolveCustomUrl($customUrlData);
			$result->setParam('ModuleName', 'com.summercart.news');
			return $result;
		}
	}
	
	class scWishlistUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_WISHLIST;
		
		protected $_handlingScript = 'module.php';
		
		public function resolve($urlData)
		{
			$pageData = $urlData[scSeoUrlTypes::URL_PAGE];
			if ($pageData[scSeoUrlTypes::PAGE_TYPE] != scFriendlyUrl::TYPE_WISHLIST) {
				return false;
			}
			if (isset($pageData[scSeoUrlTypes::PAGE_ID])) {
				return false;
			}
			
			$result = $this->resolveCustomUrl(array());
			$result->setLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
			return $result;
		}
		
		public function resolveCustomUrl($customUrlData)
		{
			$result = new scUrlResolverResult();
			$result->setPageType($this->_pageType);
			$result->setParam('ModuleName', 'com.summercart.wishlist');
			$result->setHandlingScript($this->getHandlingScript());
			return $result;
		}
	}
	
	class scProductComparisonUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_PRODUCT_COMPARISON;
		
		protected $_handlingScript = 'module.php';
		
		public function resolve($urlData)
		{
			$pageData = $urlData[scSeoUrlTypes::URL_PAGE];
			if ($pageData[scSeoUrlTypes::PAGE_TYPE] != scFriendlyUrl::TYPE_PRODUCT_COMPARISON) {
				return false;
			}
			if (isset($pageData[scSeoUrlTypes::PAGE_ID])) {
				return false;
			}
			
			$result = $this->resolveCustomUrl(array());
			$result->setLanguageCode($urlData[scSeoUrlTypes::URL_LANGUAGE]);
			return $result;
		}
		
		public function resolveCustomUrl($customUrlData)
		{
			$result = new scUrlResolverResult();
			$result->setPageType($this->_pageType);
			$result->setParam('ModuleName', 'com.summercart.productcomparison');
			$result->setHandlingScript($this->getHandlingScript());
			return $result;
		}
		
		public function getName($id, $languageCode = null)
		{
			return '';
		}
	}
	
	class scDynamicPageUrlResolver extends scBaseUrlResolver {
		
		protected $_pageType = scFriendlyUrl::TYPE_DYNAMIC_PAGE;
		
		public function resolve($urlData)
		{
			$customUrlData = array(
				'CustomUrlEntityID' => $urlData[scSeoUrlTypes::URL_PAGE][scSeoUrlTypes::PAGE_ID],
				'LanguageID' => scMultiLanguage::singleton()->getLanguageIdByCode($urlData[scSeoUrlTypes::URL_LANGUAGE]),
			);
			return $this->resolveCustomUrl($customUrlData);
		}
		
		public function resolveCustomUrl($customUrlData)
		{
			$linkablePages = scFriendlyUrl::getLinkableDynamicPages();
			if (empty($customUrlData['CustomUrlEntityID'])) {
				// Cannot recognize the page without a reference.
				return false;
			}
			if (!isset($linkablePages[$customUrlData['CustomUrlEntityID']])) {
				// The page is not defined as LINKABLE.
				return false;
			}
			
			$result = new scUrlResolverResult();
			$result->setPageType($this->_pageType);
			$result->setIdValue($customUrlData['CustomUrlEntityID']);
			
			$target = $linkablePages[$customUrlData['CustomUrlEntityID']];
			$parsed = scUrlUtil::parse($target);
			
			$result->setHandlingScript($parsed[scUrlUtil::PATH]);
			
			if (isset($parsed[scUrlUtil::QUERY])) {
				$params = scUrlUtil::parseStr($parsed[scUrlUtil::QUERY]);
				foreach ($params as $paramName => $paramValue) {
					$result->setParam($paramName, $paramValue);
				}
			}
			
			$result->setLanguageCode(scMultiLanguage::singleton()->getLanguageCodeById($customUrlData['LanguageID']));
			
			return $result;
		}
		
		public function getNames($id)
		{
			if (isset(self::$_names[$id])) {
				return self::$_names[$id];
			}
			if (!empty(self::$_names)) {
				return array();
			}
			
			$linkablePages = scFriendlyUrl::getLinkableDynamicPages();
			$recordset = new miSqlRecordset('CMSDynamicPagesML');
			$recordset->addJoinCondition('INNER', 'CMSDynamicPages', 'USING(CMSDynamicPageID)');
			$recordset->addFilter(new miSqlFilterIn('CMSDynamicPagePath', $linkablePages));
			$recordset->setSelectFields(array(
				'CMSDynamicPagePath',
				'LanguageID',
				'CMSDynamicPageTitle',
			));
			$recordset = $recordset->getAllRecords();
			
			foreach ($recordset as $row) {
				$pageId = array_search($row['CMSDynamicPagePath'], $linkablePages);
				self::$_names[$pageId][$row['LanguageID']] = $row['CMSDynamicPageTitle'];
			}
			
			if (!isset(self::$_names[$id])) {
				// Unable to find a name for the page
				return array();
			}
			
			return self::$_names[$id];
		}
	}
?>