<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scUrlUtil {
		
		const SCHEME = 'scheme';
		const USER = 'user';
		const PASS = 'pass';
		const HOST = 'host';
		const PORT = 'port';
		const PATH = 'path';
		const QUERY = 'query';
		const FRAGMENT = 'fragment';
		
		const URL_TYPE_ABSOLUTE = 1;
		const URL_TYPE_HTTPS = 2;
		const URL_TYPE_DOCROOT = 3;
		const URL_TYPE_RELATIVE = 4;
		
		/**
		 * @param string $url
		 * @return array
		 */
		public static function parse($url)
		{
			return parse_url($url);
		}
		
		/**
		 * @param array $parsedUrlUrl An array generated by php's built-in parse_url() 
		 * @return string|false
		 */
		public static function glue($parsedUrl)
		{
			if (!is_array($parsedUrl))
				return false;
			
			$url = '';
			if (isset($parsedUrl[self::SCHEME])) {
				$url .= $parsedUrl[self::SCHEME] . ':';
				if (strtolower($parsedUrl[self::SCHEME]) != 'mailto') {
					$url .= '//';
				}
			}
			if (isset($parsedUrl[self::USER])) {
				$url .= $parsedUrl[self::USER];
				if (isset($parsedUrl[self::PASS])) {
					$url .= ':' . $parsedUrl[self::PASS];
				}
				$url .= '@';
			}
			if (isset($parsedUrl[self::HOST])) {
				$url .= $parsedUrl[self::HOST];
			}
			if (isset($parsedUrl[self::PORT])) {
				$url .= ':'.$parsedUrl[self::PORT];
			}
			if (isset($parsedUrl[self::PATH])) {
				$url .= $parsedUrl[self::PATH];
			}
			if (isset($parsedUrl[self::QUERY])) {
				$url .= '?' . $parsedUrl[self::QUERY];
			}
			if (isset($parsedUrl[self::FRAGMENT])) {
				$url .= '#' . $parsedUrl[self::FRAGMENT];
			}
			
			return $url;
		}
		
		public static function addQueryParams($url, $param)
		{
			$url = self::parse($url);
			if (!isset($url[self::QUERY]))
				$url[self::QUERY] = '';
			
			// Convert the added parameters to array
			$param = self::parseStr($param);
			// Convert the URL's query string to array
			$url[self::QUERY] = self::parseStr($url[self::QUERY]);
			
			foreach ($param as $key => $value) {
				self::addQuerySubParam($url[self::QUERY], $key, $value);
			}
			$url[self::QUERY] = http_build_query($url[self::QUERY]);
			
			return self::glue($url);
		}
		
		public static function parseStr($queryString)
		{
			parse_str($queryString, $return);
			
			if (get_magic_quotes_gpc()) {
				$return = stripslashes_recursive($return);
			}
			
			return $return;
		}
		
		public static function getUrlType($url)
		{
			$parsed = self::parse($url);
			return self::getParsedUrlType($parsed);
		}
		
		public static function convertUrl($url, $convertTo)
		{
			$parsed = self::parse($url);
			$urlType = self::getParsedUrlType($parsed);
			if ($urlType == $convertTo) {
				return $url;
			}
			
			switch ($convertTo) {
				case self::URL_TYPE_ABSOLUTE:
					$parsed[self::SCHEME] = 'http';
					$parsed[self::HOST] = scPaths::getHttpHost();
					if ($urlType == self::URL_TYPE_RELATIVE) {
						$parsed[self::PATH] = self::relativeToDocrootPath($parsed[self::PATH]);
					}
					break;
				case self::URL_TYPE_HTTPS:
					$parsed[self::SCHEME] = 'https';
					$parsed[self::HOST] = scPaths::getHttpsHost();
					if ($urlType == self::URL_TYPE_RELATIVE) {
						$parsed[self::PATH] = self::relativeToDocrootPath($parsed[self::PATH]);
					}
					break;
				case self::URL_TYPE_RELATIVE:
					$parsed[self::PATH] = self::docrootToRelativePath($parsed[self::PATH]);
					if (isset($parsed[self::SCHEME])) {
						unset($parsed[self::SCHEME]);
					}
					if (isset($parsed[self::HOST])) {
						unset($parsed[self::HOST]);
					}
					break;
				case self::URL_TYPE_DOCROOT:
					if ($urlType == self::URL_TYPE_RELATIVE) {
						$parsed[self::PATH] = self::relativeToDocrootPath($parsed[self::PATH]);
					}
					if (isset($parsed[self::SCHEME])) {
						unset($parsed[self::SCHEME]);
					}
					if (isset($parsed[self::HOST])) {
						unset($parsed[self::HOST]);
					}
					break;
			}
			
			return self::glue($parsed);
		}
		
		public static function getParsedUrlType($parsedUrl)
		{
			if (isset($parsedUrl[self::SCHEME])) {
				if ($parsedUrl[self::SCHEME] == 'https') {
					return self::URL_TYPE_HTTPS;
				}
				if ($parsedUrl[self::SCHEME] == 'http') {
					return self::URL_TYPE_ABSOLUTE;
				}
				throw new miException(sprintf(_i('Unsupported url scheme "%s".'), $parsedUrl[self::SCHEME]));
			}
			if (isset($parsedUrl[self::PATH])) {
				if (substr($parsedUrl[self::PATH], 0, 1) == '/') {
					return self::URL_TYPE_DOCROOT;
				}
				return self::URL_TYPE_RELATIVE;
			}
			throw new miException(_i('Unsupported url - no scheme or path.'));
		}
		
		/**
		 * Removes all ".." in a path/URL.
		 * e.g. "path/to/../../file" will become "file"
		 * @param string $path
		 */
		public static function normalizePath($path)
		{
			$result = preg_replace('/\w+\/\.\.(?:\/|$)/', '', $path);
			if ($result == $path) {
				return $result;
			}
			return self::normalizePath($result);
		}
		
		protected static function relativeToDocrootPath($path)
		{
			return scPaths::getStoreWebPath() . $path;
		}
		
		protected static function docrootToRelativePath($path)
		{
			return substr($path, strlen(scPaths::getStoreWebPath()));
		}
		
		protected static function addQuerySubParam(&$toArray, $key, $value)
		{
			if (!is_array($value)) {
				$toArray[$key] = $value;
				return;
			}
			
			$toArray[$key] = array();
			foreach ($value as $subkey => $subvalue) {
				self::addQuerySubParam($toArray[$key], $subkey, $subvalue);
			}
		}
	}
?>