<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Validates that the value is no less than minValue
	 * minValue is property
	 */
	class scValidatorMinValue extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			$fieldProperties = $this->_webForm->getWidget($this->_fieldName)->getProperties();
			if (!isset($fieldProperties['minValue'])) {
				throw new miException(sprintf(_t('No minValue property is specified for field %s.'), $this->_fieldName));
			}

			if ($fieldValue < $fieldProperties['minValue']) {
				$errors->addError($this->_fieldName, sprintf(_t('The value must be no smaller than %d.'), $fieldProperties['minValue']));
			}
		}
	}

	/**
	 * Validates that the value is no larger than maxValue
	 * maxValue is property
	 */
	class scValidatorMaxValue extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			$fieldProperties = $this->_webForm->getWidget($this->_fieldName)->getProperties();
			if (!isset($fieldProperties['maxValue'])) {
				throw new miException(sprintf(_t('No maxValue property is specified for field %s.'), $this->_fieldName));
			}

			if ($fieldValue > $fieldProperties['maxValue']) {
				$errors->addError($this->_fieldName, sprintf(_t('The value must be no bigger than %d.'), $fieldProperties['maxValue']));
			}
		}
	}

	/**
	 * Validates email.
	 * Allows an empty string. Use miValidatorEmail for required e-mail fields.
	 */
	class scValidatorEmail extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (((string)$fieldValue === '') || preg_match('/^[A-Za-z0-9]+([_\.-A-Za-z0-9]+)*@[A-Za-z0-9]+([_\.-][A-Za-z0-9]+)*\.([A-Za-z]){2,4}$/iU', $fieldValue))
				return;
			$errors->addError($this->_fieldName, _t('Invalid email format.'));
		}
	}
	
	/**
	 * Validates that the field is not empty
	 * Works correctly with multi-language fields
	 */
	class scValidatorNoEmpty extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			
			// Check if multilanguage text field
			if (is_array($fieldValue)) {
				foreach ($fieldValue as $fieldValueLng) {
					if ($fieldValueLng != '')
						return;
				}
				
				$errors->addError($this->_fieldName, _t('Field is empty.'));
				return;
			}
			
			// Non-multilanguage field
			if ($fieldValue != '')
				return;
			
			$errors->addError($this->_fieldName, _t('Field is empty.'));
		}
	}
	
	/**
	 * Validates integer value
	 */
	class scValidatorInt extends miValidator {
		
		protected $_regEx = '#^[0-9]+$#';
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (preg_match($this->_regEx, $fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('Invalid integer.'));
		}
	}
	
	/**
	 * Validates signed integer value
	 */
	class scValidatorSignedInt extends scValidatorInt {
		
		protected $_regEx = '#^\-?[0-9]+$#';
	}
	
	/**
	 * Validates demimal value
	 * Empty values are allowed
	 */
	class scValidatorDecimal extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (trim($fieldValue) == '')
				return;
			
			$widget = $this->_webForm->getWidget($this->_fieldName);
			$fieldProperties = $widget ? $widget->getProperties() : array();
			$maxDecimals = isset($fieldProperties['maxDecimals']) ? intval($fieldProperties['maxDecimals']) : 2;
			
			if (preg_match('#^[0-9]{1,10}\.?([0-9]){0,' . $maxDecimals . '}$#', $fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('Invalid decimal.'));
		}
	}
	
	/**
	 * Unique validator
	 */
	class scValidatorUniqueFields extends miValidator {
		/**
		 * The actual record used to read the data
		 * 
		 * @access protected
		 */
		protected $_record;
		
		/**
		 * The actual record used to read the data
		 * 
		 * @access protected
		 */
		protected $_fieldNames;
		
		/**
		 * Alternative error message
		 */
		protected $_errorMessage;
		
		/**
		 * Constructs the unique validator class
		 * 
		 * @access public
		 * @param miWebForm $webForm
		 * @param array $fieldNames
		 * @param miSqlRecord $record
		 */
		public function __construct(miWebForm &$webForm, $fieldNames, miSqlRecord $record, $errorMessage = false)
		{
			$this->_record = $record;
			$this->_fieldNames = $fieldNames;
			$this->_errorMessage = $errorMessage;
			parent::__construct($webForm, $fieldNames[0]);
		}
		
		/*
		 * Check if value has changed
		 * 
		 * @access public
		 * @return boolean
		 */
		public function hasValueChanged()
		{
			// Check if is creating - then it is a change
			if (count($this->_record->getRow()) == 0)
				return true;
			
			// We are editing - check if there are any changes
			foreach ($this->_fieldNames as $fieldName) {
				if (!$this->_record->has($fieldName))
					return true;
				
				$fieldValue = $this->_webForm->getSubmittedData($fieldName);
				$oldValue = $this->_record->get($fieldName);
				
				// If there is change - return so
				if ($fieldValue != $oldValue)
					return true;
			}
			
			// There is no change
			return false;
		}
		
		/*
		 * Get filters objects
		 * 
		 * @access public
		 * @return arrey of miSqlFilterEqual objects
		 */
		public function getFilters()
		{
			$filterObects = array();
			foreach ($this->_fieldNames as $fieldName) {
				$fieldValue = $this->_webForm->getSubmittedData($fieldName);				
				$filterObects[] = new miSqlFilterCustom($fieldName, $fieldName . '="' . sql_escape_string($fieldValue) .'"');	
			}
			return $filterObects;
		}
		
		/**
		 * Validate
		 */
		public function validate(miWebFormErrorsHandler &$errors)
		{
			// In case such record already exists, i.e. editing
			if (!$this->hasValueChanged())
				return;
				
			// Get fields
			$filters = $this->getFilters();
			
			// Check if there is another record with the value
			$recordset = new miSqlRecordset($this->_record->getTableName());
			$recordset->addFilters($filters);
			
			if ($recordset->getRecordsCount()) {
				// The value is not unique
				$message = ($this->_errorMessage != false) ? $this->_errorMessage : _t('Not unique.');
				$errors->addError($this->_fieldName, $message);
			}
			// Record doesn't exists. The value is unqiue
		}
	}
	

	/**
	 * Validates http/https url
	 */
	class scValidatorHttp extends miValidator {
		
		/**
		 * @param string $string
		 * @return bool
		 */
		public static function isValid($string)
		{
			if (substr($string, 0, 7) == 'http://' || substr($string, 0, 8) == 'https://') {
				return true;
			}
			return false;
		}
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			
			if ($fieldValue == '')
				return;
				
			if (self::isValid($fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('Invalid http url.'));
		}
	}

	/**
	 * Validates if the field value exists in a specified table
	 */
	class scValidatorValueExists extends miValidator {
		
		/**
		 * Validate
		 */
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			$fieldProperties = $this->_webForm->getWidget($this->_fieldName)->getProperties();
			if (!@is_array($fieldProperties['existsIn']) || count($fieldProperties['existsIn']) != 2) {
				throw new miException(sprintf(_t('"existsIn" property must be specified for field %s.'), $this->_fieldName));
			}

			$record = new miSqlRecord($fieldProperties['existsIn'][0], $fieldProperties['existsIn'][1]);
			try {
				$record->readPK($fieldValue);
			} catch (miDBException $exception) {
				// Record doesn't exists.
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					$errors->addError($this->_fieldName, _t('Record does not exist.'));
					return;
				}
				// Different exception. Rethrow
				throw $exception;
			}
		}
	}
	
	/**
	 * Validates if the selected currency exists
	 */
	class scCurrencyExistsValidator extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (scCurrencyUtil::isCurrencyCodeExists($fieldValue))
				return;
			
			$errors->addError($this->_fieldName, _t('The selected currency is not defined. You can define new currencies using the Currencies management screen.'));
		}
	}
	
	/**
	 * Validates discount/coupon value by type
	 */
	class scDiscountValueValidator extends miValidator {
		
		protected $_valueField;
		protected $_shippingField;
		
		public function __construct(miWebForm &$webForm, $typeField, $valueField, $shippingField)
		{
			parent::__construct($webForm, $typeField);
			$this->_valueField = $valueField;
			$this->_shippingField = $shippingField;
		}
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$type = $this->_webForm->getSubmittedData($this->_fieldName);
			$value = $this->_webForm->getSubmittedData($this->_valueField);
			$shipping = $this->_webForm->getSubmittedData($this->_shippingField);
			
			if (($type == scDiscount::TYPE_FREE_SHIPPING)) {
				if (empty($shipping)) {
					$errors->addError($this->_shippingField, _t('A shipping method has to be selected.'));
				}
			} else {
				if (empty($value)) {
					$errors->addError($this->_valueField, _t('Value cannot be empty.'));
				}
			}
		}
	}
	
	/**
	 * Validates City depending on country
	 */
	class scAddressValidator extends miValidator {
		protected $_countryIdField;
		
		public function __construct(miWebForm &$webForm, $cityIdField, $countryIdField)
		{
			parent::__construct($webForm, $cityIdField);
			$this->_countryIdField = $countryIdField;
		}
		
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$countryId = (int)$this->_webForm->getSubmittedData($this->_countryIdField);
			$cityId = (int)$this->_webForm->getSubmittedData($this->_fieldName);
			if (!scCountry::getCountryRequireValidCity($countryId))
				return;
			
			if ((scCountry::getStatesCount($countryId) > 0) && ($cityId == 0)) {
				$errors->addError($this->_fieldName, _t('Please enter a valid city name.'));
			}
		}
	}
	
	/**
	 * Validates unique CustomUrl
	 */
	class scValidatorUniqueCustomUrl extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$useCustomUrls = miGetParamDefault('UseCustomUrls', 0);
			if (!$useCustomUrls)
				return;
			
			$index = scDynRowsWidget::getRowIndexByFieldName($this->_fieldName);
			
			if (!$index)
				return;
			
			$postData = scDynRowsWidget::getSubmittedData(array('CustomUrlText', 'CustomUrlID'), miGetParamDefault('CustomUrls__rowCount', 0));
			$urls = $postData[$index]['CustomUrlText'];
			$customUrlId = $postData[$index]['CustomUrlID'];
			
			foreach ($urls as $langId => $url) {
				if ($url == '') {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url cannot be empty.'));
					continue;
				}
				
				$parsed = @scUrlUtil::parse($url);
				if ($parsed === false) {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Invalid url.'));
					continue;
				}
				if (isset($parsed[scUrlUtil::SCHEME]) || isset($parsed[scUrlUtil::HOST])) {
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url cannot be outside store root.'));
					continue;
				}
				
				// Check if there is another record with the value.
				// Check CustomUrlText and not CustomUrlHash because the hash
				// represents the the path of the URL without the query string.
				$checkRecord = miStaticDBUtil::execSelect('SELECT CustomUrlID FROM CustomUrlsML' .
					' WHERE (CustomUrlID != ? || LanguageID != ?) AND CustomUrlText = ?',
					array($customUrlId, $langId, $url));
				if (!empty($checkRecord))
					$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url already exists.'));
				
				// Check if there is another post with the value
				foreach ($postData as $pData)
					foreach ($pData['CustomUrlText'] as $languageId => $postUrl)
						if (($url == $postUrl) && (($customUrlId != $pData['CustomUrlID']) || ($langId != $languageId)) && !$errors->hasFieldErrors($this->_fieldName . '[' . $langId . ']'))
							$errors->addError($this->_fieldName . '[' . $langId . ']', _t('Url already exists.'));
			}
		}
	}
	
	/**
	 * Used to validate scImageUploadWidget data 
	 */
	class scImageUploadedValidator extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$properties = $this->_webForm->getWidget($this->_fieldName)->getProperties();
			if (empty($properties['required']))
				return;
				
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (empty($fieldValue['imageWebPath'])) {
				$errors->addError($this->_fieldName, _t('Please upload an image.'));
			}
		}	
	}
?>