<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scCheckoutShippingMethodWidget extends miRadioWidget {
		
		/**
		 * @var scCart
		 */
		protected $_cart = null;
		
		/**
		 * @var array
		 */
		protected $_messages = array();
		
		/**
		 * @var bool
		 */
		protected $_areOptionsSet = false;
		
		/**
		 * @var array
		 */
		protected $_methodIdsToCodes = array();
		
		public function setCart(scCart $cart)
		{
			$this->_cart = $cart;
		}
		
		protected function setupShippingOptions()
		{
			if (!isset($this->_cart) || !($this->_cart instanceof scCart)) {
				throw new miException(_i('An instance of scCart is needed.'));
			}
			
			// Order total lines are calculated the first time they are requested after checkout info has changed.
			// This widget is normally called on the "payment and shipping" step of the checkout.
			// The order's total lines are not needed at this step (normally) as they are not shown or used in any other way.
			// But the user just entered their address on the previous step. That would actually affect the shippings.
			// The shipping addresses used by the shipping modules are correct but the order totals have to be
			// recalculated in order for the free shipping discount to be applied if needed.
			// If nothing has changed since the last time getOrderTotalLines() was called it is just going to return the cached values.
			// If, however, something has changed the getOrderTotalLines() would trigger recalcualtion.
			$this->_cart->getOrderTotal()->getOrderTotalLines();
			
			$shippingModules = scModulesFactory::getActiveShippingModulesForZone($this->_properties['countryId'], $this->_properties['stateId']);
			$cartIsAllIntangible = $this->_cart->isAllIntangible();
			
			$options = array();
			foreach ($shippingModules as $shippingModule) {
				$moduleId = $shippingModule['ModuleID'];
				$shippingModuleId = $shippingModule['ShippingModuleID'];
				if (!isset($shippingModule['object']))
					continue;
				
				// If all items in the cart are INTANGIBLE the only shipping module we should enable
				// is a Digital Delivery.
				// If there is at least one TANGIBLE item in the cart then the Digital Delivery module cannot be used.
				if ($cartIsAllIntangible XOR ($shippingModule['object'] instanceof scDigitalDeliveryShipping)) {
					// Cart is NOT all intangible AND Shipping module is Digital Delivery = Skip module
					// Cart is all intangible AND Shipping module is NOT Digital Delivery = Skip module
					// Cart is all intangible AND Shipping module is Digital Delivery = Use module
					// Cart is NOT all intangible AND Shipping module is NOT Digital Delivery = Use module
					continue;
				}
				
				$module = $shippingModule['object'];
				
				try {
					$shippingMethodRates = $module->getShippingRates($this->_cart, array_keys($shippingModule['ShippingModuleActiveMethods']));
					if (isset($shippingMethodRates[IShippingModule::SHIPPING_RATES_MESSAGE])) {
						$this->_messages[] = sprintf(_t('From %s: %s.'), $shippingModule['ShippingModuleDisplayName'], $shippingMethodRates[IShippingModule::SHIPPING_RATES_MESSAGE]);
					}
					
					if (!isset($shippingMethodRates[IShippingModule::SHIPPING_RATES]))
						continue;
					
					$rates = $shippingMethodRates[IShippingModule::SHIPPING_RATES];
					$shippingMethods = $module->getShippingMethods();
					foreach ($shippingMethods as $shippingMethodId => $shippingMethod)
						$shippingMethods[$shippingMethodId] = scInterfaceTexts::getTranslatedText($shippingMethod);
					
					$currencyCode = isset($shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY]) ? $shippingMethodRates[IShippingModule::SHIPPING_RATES_CURRENCY] : null;

					$options = array_merge($options, $this->getShippingOptions($rates, $shippingMethods, $shippingModuleId, $shippingModule['ShippingModuleActiveMethods'], $currencyCode));
					
					$this->_methodIdsToCodes = $this->_methodIdsToCodes + $this->getModuleMethodIdsToCodes($shippingModule);
				} catch (Exception $exception) {
					$this->_messages[] = sprintf(_t('From %s: %s.'), $shippingModule['ShippingModuleDisplayName'],
						sprintf(_t('Exception "%s".'), $exception->getMessage()));
				}
			}
			
			$this->setRadioButtons($options);
		}
		
		/**
		 * Return the shipping options and their rates
		 * The options values format is: ShippingModuleID-ShippingMethodID
		 * 
		 * @return array
		 */
		protected function getShippingOptions($rates, $shippingMethods, $shippingModuleId, $activeMethods, $currencyCode)
		{
			$options = array();
			foreach ($shippingMethods as $shippingMethodId => $shippingMethod) {
				if (!isset($rates[$shippingMethodId]))
					continue;
				
				if (!isset($activeMethods[$shippingMethodId])) {
					continue;
				}
				
				if ($this->_cart->getFreeShippingMethodId() == $activeMethods[$shippingMethodId]['ShippingMethodID']) {
					$price = ' (' . _t('Free') . ')';
				} else if ($rates[$shippingMethodId] == 0) {
					$price = '';
				} else {
					if (is_null($currencyCode)) {
						$currency = scCurrencyUtil::getBaseCurrency();
					} else {
						$currency = new scCurrency($currencyCode);
					}
					$mcAmount = scMultiCurrencyAmount::factory();
					$mcAmount->set($currency, $rates[$shippingMethodId]);
					
					$priceObj = scPrice::factory($mcAmount);
					
					$price = ' (' . $priceObj->getDisplayPrice(scCurrencyUtil::getCurrentCurrency()) . ')';
				}
				$options[$shippingModuleId . '-' . $shippingMethodId] =
					(!empty($activeMethods[$shippingMethodId]['ShippingMethodDisplayName']) ? $activeMethods[$shippingMethodId]['ShippingMethodDisplayName'] : $shippingMethod) . $price;
			}
			return $options;
		}
		
		public function getEditableControl()
		{
			if (!$this->_areOptionsSet) {
				$this->setupShippingOptions();
				$this->_areOptionsSet = true;
			}
			
			$value = $this->getValue();
			
			$widget = '';
			$idIndex = 0;
			foreach ($this->_radioButtons as $key => $option) {
				$checked = ($value == $key) ? ' checked="checked"' : '';
				$id = $this->_fieldName . '--' . ($idIndex++);
				$widget .= '<label for="' . $id . '">';
				$widget .= '<input type="radio" name="' . $this->_fieldName . '" id="' . $id . '" value="' . miI18N::htmlEscape($key) . '"' . $checked . '/>';
				$widget .= miI18N::htmlEscape($option);
				$widget .= '</label>';
			}
			
			$result = array($this->_fieldName => $widget);
			if (count($this->_messages))
				$result['SHIPPINGMESSAGES'] = _t('Notices:<br/>') . implode('<br/> ', $this->_messages);
			else
				$result['SHIPPINGMESSAGES'] = '';
			return $result;
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			if (miGetParamDefault($this->_fieldName, '') == '')
				$errors->addError($this->_fieldName, _t('There are no available shipping methods for your order!'));
		}
		
		protected function getValue()
		{
			$value = $this->_webForm->getFormData($this->_fieldName);
			if (!empty($this->_radioButtons[$value]))
				return $value;
			
			$freeShippingMethodId = $this->_cart->getFreeShippingMethodId();
			if (isset($this->_methodIdsToCodes[$freeShippingMethodId])) {
				$value = $this->_methodIdsToCodes[$freeShippingMethodId];
				if (!empty($this->_radioButtons[$value]))
					return $value;
			}
			
			// If we cannot find the $value in the radioButtons group use the default
			if ($this->_defaultRadioIndex !== null) {
				return $this->_defaultRadioIndex;
			}
			
			reset($this->_radioButtons);
			return key($this->_radioButtons);
		}
		
		protected function getModuleMethodIdsToCodes(&$shippingModule)
		{
			$methodIdsToCodes = array();
			foreach ($shippingModule['ShippingModuleActiveMethods'] as $method) {
				$methodIdsToCodes[$method['ShippingMethodID']] = $shippingModule['ShippingModuleID'] . '-' . $method['ShippingMethodCode'];
			}
			return $methodIdsToCodes;
		}
	}
?>