<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scDynRowsWidget extends miBaseWidget {

		/**
		 * The foreign key of the primary table to join by
		 */
		protected $_foreignKeyName = null;
		
		/**
		 * The repeating section template
		 */
		protected $_template = null;
		
		/**
		 * The declaration array of the RS fields
		 */
		protected $_rsDataFields;
		
		/**
		 * Array with field objects
		 */
		protected $_rsFormFields = array();
		
		/**
		 * The number of repeating sections
		 */
		protected $_rowCount;
		
		/**
		 * The validators
		 * 
		 * @access protected
		 * @var array
		 */
		protected $_rsValidators = array();
		
		/**
		 * Constructs the repeating section
		 */
		public function __construct(miWebForm &$webForm, $properties)
		{
			parent::__construct($webForm, $properties);
			
			$this->validateDynRowsProperties($properties);
		}
		
		protected function validateDynRowsProperties($properties)
		{
			foreach (array('table', 'template', 'foreignKey', 'fields') as $propertyName) {
				if (!isset($properties[$propertyName])) {
					throw new miException('You must specify a "'.$propertyName.'" property for a repeating section!');
				}
			}
			
			$this->_foreignKeyName = $properties['foreignKey'];
			$this->_template = $properties['template'];
			$this->_rsDataFields = $properties['fields'];
		}
		
		/**
		 * Subclasses can modify the properties passed to the fields upon their instantiation
		 * 
		 * @param array $properties
		 * @param int $rowNum
		 * @param string $suffix
		 * @param string $fieldName
		 */
		protected function modifyFieldProperties(&$properties, $rowNum, $suffix, $fieldName)
		{
		}
		
		/**
		 * Subclasses may implement initialization of widgets.
		 *
		 * @param miWidget $widget
		 * @param string $fieldName The fieldname the way it was defined in the datafields array.
		 */
		protected function initField(miBaseWidget $field, $fieldName)
		{
		}
		
		/**
		 * Populates the webform with the data
		 * The data is read from the field when they are displayed
		 */
		protected function setWebFormData()
		{
			$records = $this->_webForm->getFormData($this->_fieldName);
			if ($records == null) {
				$this->_rowCount = 0;
				return;
			}
			$this->_rowCount = count($records);

			foreach ($records as $rowIndex => $record) {
				foreach ($record as $fieldName => $value)
					$this->_webForm->setFormData($fieldName . $this->getSuffix($rowIndex+1), $value);
			}
		}
		
		/**
		 * Creates the field objects
		 */
		protected function createFieldObjects()
		{
			$this->_rsFormFields = array();
			for ($i = 0; $i <= $this->_rowCount; $i++) {
				$suffix = $this->getSuffix($i);
				
				foreach ($this->_rsDataFields as $key => $rsDataField) {
					$dataFieldName = $rsDataField['data'] . $suffix;
					
					$properties = isset($rsDataField['properties']) ? $rsDataField['properties'] : array();
					$this->modifyFieldProperties($properties, $i, $suffix, $dataFieldName);
					
					$field = new $rsDataField['field']($this->_webForm, $properties);
					$field->setFieldName($dataFieldName);
					$this->initField($field, $rsDataField['data']);
					$this->_rsFormFields[$i][$key] = $field;
					
					if (isset($rsDataField['validator'])) {
						// Multiple validators per field
						if (is_array($rsDataField['validator'])) {
							$this->_rsValidators[$i][$key] = array();
							foreach ($rsDataField['validator'] as $validator)
								$this->_rsValidators[$i][$key][] = new $validator($this->_webForm, $rsDataField['data'] . $suffix);
							
							continue;
						}
						
						// Single validator
						$this->_rsValidators[$i][$key] = new $rsDataField['validator']($this->_webForm, $rsDataField['data'] . $suffix);
					}
				}
			}
		}
		
		/**
		 * Return array with the formField object with name $fieldName for each row in the repeating section
		 * 
		 * @return array field objects
		 */
		public function getRsFieldObjs($fieldName)
		{
			$result = array();
			foreach ($this->_rsFormFields as $rowIndex => &$rowFormFields) {
				$fullFieldName = $fieldName . $this->getSuffix($rowIndex);
				foreach ($rowFormFields as &$rsFormField) {
					if ($rsFormField->getFieldName() == $fullFieldName)
						$result[] = $rsFormField;
				}
			}
			return $result;
		}
		
		/**
		 * Returns the field suffix for row index $index
		 * 
		 * @param int $index
		 * @return string the suffix
		 */
		static public function getSuffix($index)
		{
			return '::' . ($index == 0 ? '' : $index) . '::';
		}
		
		/**
		 * Returns the sections used to control hiding of the first row
		 * The first section is displayed once, and the subsecequent sections are not displayed at all
		 * 
		 * @return array array of miTemplateParserSectionInfo objects
		 */
		protected function getFirstRowSections()
		{
			$firstRowSections = array();
			$firstRowSections[] = new miTemplateParserSectionInfo($this->_fieldName . '_FirstRow', 1);
			for ($i = 0; $i < $this->_rowCount; $i++)
				$firstRowSections[] = new miTemplateParserSectionInfo($this->_fieldName . '_FirstRow', 0);
			return $firstRowSections;
		}
		
		/**
		 * Returns the field values
		 * 
		 * @return array with field name => array of field values
		 */
		protected function getFieldValues()
		{
			// Build repeating section for each data entry
			$sectionFieldValues = array();
			foreach ($this->_rsFormFields as $rowIndex => $rowFormFields) {
				$suffix = $this->getSuffix($rowIndex);
				
				$sectionFieldValues['%%ROWNUM%%'][] = ($rowIndex == 0 ? '' : $rowIndex);
				foreach ($rowFormFields as $rsFormField) {
					$fieldTmplVars = $rsFormField->getEditableControl();
					foreach ($fieldTmplVars as $templateVarName => $templateVarValue) {
						$sectionFieldValues['%%' . strtoupper(preg_replace('/::[0-9]*::/', '', $templateVarName)) . '%%'][] = $templateVarValue;
					}
				}
			}
			return $sectionFieldValues;
		}
		
		/**
		 * Return section info object with the field values
		 * 
		 * @param array the field values
		 * @return miTemplateParserSectionInfo the section info object
		 */
		protected function getSectionInfo($fieldValues)
		{
			$sectionInfo = new miTemplateParserSectionInfo();
			$sectionInfo->setSectionInfo($this->_fieldName.'_Rows', $this->_rowCount + 1, $fieldValues);
			$sectionInfo->addSubsectionsArray($this->_fieldName.'_FirstRow', $this->getFirstRowSections());
			return $sectionInfo;
		}
		
		/**
		 * Parse the repeating section template
		 * 
		 * @param miTemplateParserSectionInfo the section info object
		 * @return string the html
		 */
		protected function parseTemplate($sectionInfo)
		{
			$rsTpl = new scTemplateParser();
			$rsTpl->readTemplate($this->_template);
			$this->assignTemplateVars($rsTpl);
			$rsTpl->setSectionInfos(array($sectionInfo));
			return $rsTpl->templateParse();
		}
		
		/**
		 * @param miTemplateParser $templateParser
		 */
		protected function assignTemplateVars(miTemplateParser &$templateParser)
		{
			$templateParser->assign('%%ROWCOUNT%%', $this->_rowCount);
		}
		
		/**
		 * Returns the field contents to be displayed
		 * 
		 * @return array
		 */
		public function getEditableControl()
		{
			$this->setWebFormData();
			$this->createFieldObjects();
			
			$sectionInfo = $this->getSectionInfo($this->getFieldValues());
			return array($this->_fieldName => $this->parseTemplate($sectionInfo));
		}
		
		/**
		 * Returns the repeating section data for all fields
		 */
		public function getData()
		{
			if (!miHasParam($this->_fieldName . '__rowCount'))
				return array();
			$this->_rowCount = miGetParamDefault($this->_fieldName . '__rowCount', 0);
			$this->createFieldObjects();
			
			$result = array();
			
			foreach ($this->_rsFormFields as $rowIndex => $rowFormFields) {
				if ($rowIndex == 0)
					continue;
				
				$data = array();
				foreach ($rowFormFields as $rsFormField)
					$data = array_merge($data, $rsFormField->getData());
				foreach ($data as $key => $value)
					$result[$rowIndex-1][preg_replace('/::[0-9]*::/', '', $key)] = $value;
				
			}
			return array($this->_fieldName => $result);
		}
		
		public function validateData(miWebFormErrorsHandler &$errors)
		{
			foreach ($this->_rsFormFields as $rowIndex => $rowFormFields) {
				if ($rowIndex == 0)
					continue;
					
				// Validate the rsFields
				foreach ($rowFormFields as $key => $rsFormField) {
					$rsFormField->validateData($errors);
					
					if (empty($this->_rsValidators[$rowIndex][$key]))
						continue;
					
					// Multiple validators per field
					if (is_array($this->_rsValidators[$rowIndex][$key])) {
						foreach ($this->_rsValidators[$rowIndex][$key] as $validator)
							$validator->validate($errors);
						
						continue;
					}
					
					// Single validator
					$this->_rsValidators[$rowIndex][$key]->validate($errors);
				}
			}
		}
		
		/**
		 * Returns row index by field name
		 */
		public static function getRowIndexByFieldName($fieldName)
		{
			preg_match('/\:\:([0-9]+)\:\:/', $fieldName, $matches);
			return end($matches);
		}
		
		/**
		 * Returns submitted post data
		 */
		public static function getSubmittedData($fieldNames = array(), $rowsCount)
		{
			$postData = array();
			
			for ($n = 1; $n <= $rowsCount; $n++)
				foreach ($fieldNames as $fieldName)
					$postData[$n][$fieldName] = miGetParamDefault($fieldName . '::' . $n . '::', '');
			
			return $postData;
		}
	}
?>