<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Collect On Delivery Order Total module
	 */
	class scOrderTotalCollectOnDelivery implements IOrderTotalModule, IModuleOptions {
		protected $_moduleId;
		
		/**
		 * Order total module constructor
		 */
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		/**
		 * Returns order total lines
		 * Each order total line array has the following keys:
		 * description - the order total description, shown to the user
		 * amount - the amount shown on the line
		 * type - order total line type
		 * 
		 * @param scCart $cart the shopping cart object
		 * @return array the order total lines info
		 */
		public function getOrderTotalLines(scCart $cart)
		{
			if ($cart->getItemsCount() == 0) {
				return array();
			}
			
			if ($cart->getPaymentMethod() == 0)
				return array();
			
			// Consider only if the payment module is COD
			$moduleId = scUtil::getDbValue('PaymentModules', 'ModuleID', 'PaymentModuleID', $cart->getPaymentMethod(), 0);
			if ($moduleId != scModulesFactory::getModuleIdByName('com.summercart.payment.cod'))
				return array();
			
			$codFee = scMultiCurrencyAmount::getZero();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId);
			
			// check if the shipping module has COD fee implementation
			// and return it if applicable
			if (!empty($options['UseShippingModuleFee'])) {
				$shippingModuleId = $cart->getShippingModuleId();
				$moduleId = scUtil::getDbValue('ShippingModules', 'ModuleID', 'ShippingModuleID', $shippingModuleId, 0);
				if ($moduleId) {
					try {
						$className = scModulesFactory::getModuleClassName($moduleId, 'IShippingModuleCodInfo');
						$shipping = new $className($moduleId, $shippingModuleId);
						$codFee = $shipping->getCodFee($cart);
					} catch (miException $e) {
						scLog::singleton()->warning(_i('Exception retrieving C.O.D. fee: ') . $e->getMessage(), scLog::CATEGORY_SHIPPING);
						// do nothing
					}
					if (!($codFee instanceof scMultiCurrencyAmount))
						throw new miException(_i('Invalid C.O.D. fee.'));
				}
			} else {
				if (isset($options['FixedFee'])) {
					$codFee = scMultiCurrencyAmountMath::add($codFee, scMultiCurrencyAmount::factoryBaseCurrency($options['FixedFee']));
				}
				if (isset($options['PercentFee'])) {
					$percentFee = scMultiCurrencyAmountMath::multiply($cart->getSubtotal(), ($options['PercentFee'] / 100));
					$codFee = scMultiCurrencyAmountMath::add($codFee, $percentFee);
				}
			}
			
			if ($codFee->get() <= 0) {
				return array();
			}
			
			if (isset($options['FeeIsNotInvoiced']) && $options['FeeIsNotInvoiced']) {
				// _t('C.O.D. fee of %s is paid upon delivery')
				$description = 'C.O.D. fee of %s is paid upon delivery';
				$param = scPrice::factory($codFee)->getDisplayPrice();
				$line = new scOrderTotalLineWithParam($description, scMultiCurrencyAmount::getZero(), self::ORDER_TOTAL_TYPE_SURCHARGE, $param);
			} else {
				// _t('C.O.D. fee')
				$line = new scOrderTotalLine('C.O.D. fee', $codFee, self::ORDER_TOTAL_TYPE_SURCHARGE);
			}
			return array($line);
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return scModuleOptionsInfo the option fields
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'scAmountWidget',
					'data' => 'FixedFee',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'PercentFee',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'UseShippingModuleFee',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'FeeIsNotInvoiced',
					'properties' => array('defaultValue' => false)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('UseShippingModuleFee')->setRadioButtons(array(1 => _t('Yes'), 0 => _t('No')), 0);
			$options->setTemplateFile('modules/ordertotal/cod/options.tmpl');
			return $options;
		}
	}
?>