<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Coupon Order Total module
	 */
	class scOrderTotalCoupon implements IOrderTotalModule, IModuleOptions {
		protected $_moduleId;
		
		/**
		 * Order total module constructor
		 */
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		/**
		 * Returns order total lines
		 * Each order total line array has the following keys:
		 * description - the order total description, shown to the user
		 * amount - the amount shown on the line
		 * type - order total line type
		 * 
		 * @param scCart $cart the shopping cart object
		 * @return array the order total lines info
		 */
		public function getOrderTotalLines(scCart $cart)
		{
			$couponCalculator = new scCouponCalculator();
			
			$coupon = $cart->getCoupon();
			if (is_null($coupon)) {
				return array();
			}
			
			if ($coupon[scCouponsContainer::KEY_IMPLEMENTATION] != scDiscount::IMPLEMENTATION_ORDER_TOTAL) {
				// Not IOrderTotalModule's responsibility
				return array();
			}
			
			$subTotal = $cart->getSubtotal();
			
			// Check for free shipping coupon. If available for all products set it in OrderTotal and return no lines.
			$freeShippings = scDiscount::getFreeShippingMethods($cart, $couponCalculator, $subTotal, array($coupon));
			if (count($freeShippings) > 0) {
				$cart->setFreeShippingMethodId(reset($freeShippings));
				return array();
			}
			
			$cartItems = $cart->getItems();
			
			$couponApplied = false;
			$couponDiscount = scMultiCurrencyAmount::getZero();
			foreach ($cartItems as $item) {
				$listingItem = $item->getProductListingItem();
				
				$price = $item->getPriceObj()->getAmount();
				$productId = $item->getProductId();
				
				$customerId = scCustomerAuth::singleton()->getCustomerId();
				$customerGroupId = scCustomerAuth::singleton()->getCustomerGroupId();
				
				$manufacturer = $listingItem->getManufacturer();
				if ($manufacturer instanceof scManufacturerDomainObject) {
					$manufacturerId = $manufacturer->get('ManufacturerID');
				} else {
					$manufacturerId = null;
				}
				
				if ($couponCalculator->checkConditions($coupon, $productId, $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal)) {
					if ($coupon[scCouponsContainer::KEY_APPLY_MODE] == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
						list($discount) = $couponCalculator->getDiscount($coupon, $subTotal);
						$couponDiscount = scMultiCurrencyAmountMath::add($couponDiscount, $discount);
						$couponApplied = true;
						// If this coupon apply mode is subtotal, then we cannot apply it more then once
						break;
					}
					
					list($discount) = $couponCalculator->getDiscount($coupon, $price);
					
					// Do not allow discount larger than product price
					if ($discount->get() > $price->get()) {
						$discount = clone $price;
					}
					
					$discount = scMultiCurrencyAmountMath::multiply($discount, $item->getQuantity());
					$couponDiscount = scMultiCurrencyAmountMath::add($couponDiscount, $discount);
					$couponApplied = true;
				}
			}
			
			if (!$couponApplied)
				$cart->removeCoupon();
			
			// _t('Coupon %s')
			if ($couponDiscount->get() != 0) {
				$couponDiscount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $couponDiscount);
				return array(new scOrderTotalLineWithParam('Coupon %s', $couponDiscount, self::ORDER_TOTAL_TYPE_COUPON, $coupon[scCouponsContainer::KEY_CODE]));
			}
			return array();
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return scModuleOptionsInfo the option fields
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			return null;
		}
		
		/**
		 * Checks if the coupon module is active
		 * 
		 * @return bool
		 */
		public function isActive()
		{
			$sql = 	'SELECT Modules.ModuleID FROM OrderTotalModules INNER JOIN Modules USING (ModuleID) ' .
					'WHERE OrderTotalModuleIsActive <> 0 AND ModuleIsActive <> 0 ' .
					'AND ModuleName = "com.summercart.ordertotal.coupon"';
			$result = miStaticDBUtil::execSelect($sql);
			
			return count($result) > 0;
		}
	}
	
	class scProductPriceCoupon implements IProductPriceModifier {
		
		/**
		 * Coupons are only available in the cart.
		 */
		public function getPriceModifications($products)
		{
			return array();
		}
		
		/**
		 * @param scCartItem $cartItem
		 * @return scProductPriceModification[]
		 */
		public function getCartPriceModifications(scCartItem $cartItem)
		{
			if (!($cartItem instanceof scCartItemProduct)) {
				return array();
			}
			
			$couponCalculator = new scCouponCalculator();
			
			$coupon = $this->getCart()->getCoupon();
			if (is_null($coupon)) {
				// No coupon applied
				return array();
			}
			
			if ($coupon[scCouponsContainer::KEY_IMPLEMENTATION] != scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER) {
				// Not IProductPriceModifier's responsibility
				return array();
			}
			
			$subTotal = $this->getCart()->getSubtotal();
			$listingItem = $cartItem->getProductListingItem();
			$price = $cartItem->getPriceObj()->getAmount();
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			$customerGroupId = scCustomerAuth::singleton()->getCustomerGroupId();
			$manufacturer = $listingItem->getManufacturer();
			if ($manufacturer instanceof scManufacturerDomainObject) {
				$manufacturerId = $manufacturer->get('ManufacturerID');
			} else {
				$manufacturerId = null;
			}
			
			$couponDiscount = scMultiCurrencyAmount::getZero();
			$couponPercent = 0;
			
			if ($couponCalculator->checkConditions($coupon, $listingItem->getProductId(), $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal)) {
				list($couponDiscount, $couponPercent) = $couponCalculator->getDiscount($coupon, $price);
				
				// Do not allow discount larger than product price
				if ($couponDiscount->get() > $price->get()) {
					$couponDiscount = clone $price;
					$couponPercent = 100;
				}
			}
			
			$discounts = array();
			if ($couponDiscount->get() != 0) {
				$couponDiscount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $couponDiscount);
				$discounts[] = new scProductPriceModification($listingItem->getProductId(), $couponDiscount, $couponPercent, _t('Coupon %s'), true);
			}
			return $discounts;
		}
		
		/**
		 * @return scCart
		 */
		protected function getCart()
		{
			return scCart::getCart();
		}
	}
	
	/**
	 * Contains the applyable coupons
	 */
	class scCouponsContainer
	{
		/**
		 * Associative array keys
		 */
		const KEY_CODE = 'CouponCode';
		const KEY_TYPE = 'CouponType';
		const KEY_VALUE = 'CouponValue';
		const KEY_SHIPPING_METHOD = 'ShippingMethod';
		const KEY_CONDITIONS = 'Conditions';
		const KEY_APPLY_MODE = 'CouponApplyMode';
		const KEY_IMPLEMENTATION = 'CouponImplementation';
		
		const KEY_CONDITIONS_ORDER_EXCEED = 'OrderExceeds';
		const KEY_CONDITIONS_CUSTOMER = 'Customer';
		const KEY_CONDITIONS_CUSTOMER_GROUP = 'CustomerGroup';
		const KEY_CONDITIONS_CATEGORY = 'Category';
		const KEY_CONDITIONS_PRODUCT = 'Product';
		const KEY_CONDITIONS_MANUFACTURER = 'Manufacturer';
		
		/**
		 * The coupons table
		 */
		protected $_coupons = array();
		
		/**
		 * The one and only instance of the object
		 */
		protected static $_instance = null;

		/**
		 * Initializes the coupons table
		 */
		protected function __construct()
		{			
		}
		
		/**
		 * Prevent explicit object creation
		 */
		protected function __clone()
		{
		}
		
		/**
		 * Returns the one and only instance of the object
		 * 
		 * @return scCouponsContainer
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scCouponsContainer();
				
			return self::$_instance;
		}
		
		/**
		 * Returns the coupon information
		 * 
		 * @param string $couponCode
		 * @return array
		 * @throws miException
		 */
		public function getCoupon($couponCode)
		{
			$sql =	'SELECT *, Coupons.CouponID as CouponID FROM Coupons ' . 
					'LEFT JOIN CouponProducts USING(CouponID) ' .
					'LEFT JOIN CouponCustomers ON Coupons.CouponID = CouponCustomers.CouponID ' .
					'LEFT JOIN CouponCustomerGroups ON Coupons.CouponID = CouponCustomerGroups.CouponID ' .
					'LEFT JOIN CouponManufacturers ON Coupons.CouponID = CouponManufacturers.CouponID ' .
					'LEFT JOIN CouponCategories ON Coupons.CouponID = CouponCategories.CouponID ' .
					'WHERE CouponCode = "' . sql_escape_string($couponCode) . '" AND CouponIsActive <> 0 ' .
					'AND CouponValidFrom <= CURDATE() AND CouponValidTo >= CURDATE()' . 
					'AND CouponUsageCount < CouponAllowedUsageCount';
					
			$couponRates = miStaticDBUtil::execSelect($sql);

			if (count($couponRates) == 0)
				throw new miException(_t('Invalid coupon.'));
			
			$coupon = array();
			foreach ($couponRates as $couponRate) {
				// Save the coupon basic information
				$coupon[self::KEY_CODE] = $couponRate['CouponCode'];
				$coupon[self::KEY_TYPE] = $couponRate['CouponType'];
				$coupon[self::KEY_VALUE] = $couponRate['CouponValue'];
				$coupon[self::KEY_APPLY_MODE] = $couponRate['CouponApplyMode'];
				
				//$coupon[self::KEY_IMPLEMENTATION] = $couponRate['CouponImplementation'];
				if ($couponRate['CouponApplyMode'] == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
					$coupon[self::KEY_IMPLEMENTATION] = scDiscount::IMPLEMENTATION_ORDER_TOTAL;
				} else {
					$coupon[self::KEY_IMPLEMENTATION] = scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER;
				}
				
				$coupon[self::KEY_SHIPPING_METHOD] = $couponRate['CouponFreeShippingMethodID'];
				if (!isset($coupon[self::KEY_CONDITIONS]))
					$coupon[self::KEY_CONDITIONS] = array();
				
				// Save the conditions
				if ($couponRate['CouponConditionOrderExceeds'] > 0)				
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_ORDER_EXCEED] = $couponRate['CouponConditionOrderExceeds'];
				if (!empty($couponRate['CustomerID']))
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_CUSTOMER][$couponRate['CustomerID']] = $couponRate['CustomerID'];
				if (!empty($couponRate['CustomerGroupID']))
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_CUSTOMER_GROUP][$couponRate['CustomerGroupID']] = $couponRate['CustomerGroupID'];
				if (!empty($couponRate['CategoryID']))
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_CATEGORY][$couponRate['CategoryID']] = $couponRate['CategoryID'];
				if (!empty($couponRate['ProductID']))
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_PRODUCT][$couponRate['ProductID']] = $couponRate['ProductID'];
				if (!empty($couponRate['ManufacturerID']))
					$coupon[self::KEY_CONDITIONS][self::KEY_CONDITIONS_MANUFACTURER][$couponRate['ManufacturerID']] = $couponRate['ManufacturerID'];
			}
			
			return $coupon;
		}
		
		/**
		 * Validates the coupon
		 *
		 * @param array $coupon
		 * @param scCart $cart
		 * @return bool
		 */
		public function isCouponValid($coupon, scCart $cart)
		{
			$couponValid = false;
			
			$subTotal = $cart->getSubtotal();
			$cartItems = $cart->getItems();
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			$customerGroupId = scCustomerAuth::singleton()->getCustomerGroupId();
			
			$couponCalculator = new scCouponCalculator();
			foreach ($cartItems as $item) {
				$listingItem = $item->getProductListingItem();
				
				$price = $item->getPriceObj()->getAmount();
				$productId = $item->getProductId();
				
				$manufacturer = $listingItem->getManufacturer();
				if ($manufacturer instanceof scManufacturerDomainObject) {
					$manufacturerId = $manufacturer->get('ManufacturerID');
				} else {
					$manufacturerId = null;
				}
				
				if ($couponCalculator->checkConditions($coupon, $productId, $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal))
					$couponValid = true;
			}
			
			return $couponValid;
		}
	}
	
	/**
	 * Class for calculating the coupon amounts
	 */
	class scCouponCalculator
	{	
		/**
		 * Checks if coupon is valid
		 * 
		 * @param string $couponCode
		 * @param int $productId
		 * @param int $categoryId
		 * @param int $customerId
		 * @param float $subTotal
		 * 
		 * @return bool
		 */
		public function checkCoupon($couponCode, $productId, $categoryId, $customerId, $customerGroupId, $manufacturerId, $subTotal)
		{
			// Get applyable coupons
			$coupon = scCouponsContainer::singleton()->getCoupon($couponCode);
			return $this->checkConditions($coupon, $productId, $categoryId, $customerId, $customerGroupId, $manufacturerId, $subTotal);
		}
		
		public function getApplicableFreeShippings(IProductListingItem $listingItem, $subTotal = 0, $discounts)
		{
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			$customerGroupId = scCustomerAuth::singleton()->getCustomerGroupId();
			
			$applicableFreeShippings = array();
			
			$manufacturer = $listingItem->getManufacturer();
			if ($manufacturer instanceof scManufacturerDomainObject) {
				$manufacturerId = $manufacturer->get('ManufacturerID');
			} else {
				$manufacturerId = null;
			}
			
			foreach ($discounts as $discount) {
				// Check if $discount is a Free Shipping
				if ($discount[scCouponsContainer::KEY_TYPE] != scDiscount::TYPE_FREE_SHIPPING)
					continue;
				
				// Check if all the conditions are met
				if (!$this->checkConditions($discount, $listingItem->getProductId(), $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal))
					continue;
				
				$applicableFreeShippings[] = $discount[scCouponsContainer::KEY_SHIPPING_METHOD];
			}
			
			return $applicableFreeShippings;
		}
		
		/**
		 * Return the coupon discount amount
		 * 
		 * @param array $coupon
		 * @param float $productPrice
		 * @return array
		 */
		public function getDiscount($coupon, $productPrice)
		{
			switch ($coupon[scCouponsContainer::KEY_TYPE]) {
				case scDiscount::TYPE_AMOUNT:
					$amount = scMultiCurrencyAmount::factoryBaseCurrency($coupon[scCouponsContainer::KEY_VALUE]);
					$percent = $coupon[scCouponsContainer::KEY_VALUE] * 100 / $productPrice->get(scCurrencyUtil::getBaseCurrency());
					break;
				case scDiscount::TYPE_PERCENTAGE:
					$amount = scMultiCurrencyAmountMath::multiply($productPrice, $coupon[scCouponsContainer::KEY_VALUE]);
					$amount = scMultiCurrencyAmountMath::divide($amount, 100);
					$percent = $coupon[scCouponsContainer::KEY_VALUE];
					break;
				default:
					throw new miException(_i('Unknown discount type.'));
			}
			return array($amount, $percent);
		}		
		
		/**
		 * Check if all conditions for coupon are met
		 * 
		 * @param array $coupon
		 * @param int $productId
		 * @param int $categoryId
		 * @param int $customerId
		 * @param int $subtotal
		 * 
		 * @return bool
		 */
		public function checkConditions($coupon, $productId, $categoryIds, $customerId, $customerGroupId, $manufacturerId, $subTotal)
		{
			foreach ($coupon[scCouponsContainer::KEY_CONDITIONS] as $condition => $parameter) {
				switch ($condition) {
					case scCouponsContainer::KEY_CONDITIONS_ORDER_EXCEED:
						if ($subTotal->get() < $parameter)
							return false;
						break;
					case scCouponsContainer::KEY_CONDITIONS_PRODUCT:
						if (!in_array($productId, $parameter, false))
							return false;
						break;
					case scCouponsContainer::KEY_CONDITIONS_CATEGORY:
						$matchingCategories = array_intersect($categoryIds, $parameter);
						if (empty($matchingCategories))
							return false;
						break;
					case scCouponsContainer::KEY_CONDITIONS_CUSTOMER:
						if (!$customerId || !in_array($customerId, $parameter))
							return false;
						break;
					case scCouponsContainer::KEY_CONDITIONS_CUSTOMER_GROUP:
						if (!$customerGroupId || !in_array($customerGroupId, $parameter))
							return false;
						break;
					case scCouponsContainer::KEY_CONDITIONS_MANUFACTURER:
						if (empty($manufacturerId) || !in_array($manufacturerId, $parameter))
							return false;
						break;
				}
			}
			
			return true;
		}
	}
?>