<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Discount Order Total module
	 */
	class scOrderTotalDiscount implements IOrderTotalModule, IModuleOptions {
		protected $_moduleId;
		
		/**
		 * Order total module constructor
		 */
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		/**
		 * Returns order total lines
		 * Each order total line array has the following keys:
		 * description - the order total description, shown to the user
		 * amount - the amount shown on the line
		 * type - order total line type
		 * 
		 * @param scCart $cart the shopping cart object
		 * @return array the order total lines info
		 */
		public function getOrderTotalLines(scCart $cart)
		{
			$discounts = $this->createDiscountCalculator();
			
			// Reset the free shipping. It may be unavailable anymore.
			$cart->setFreeShippingMethodId(null);
			
			$subTotal = $cart->getSubtotal();
			
			// Check for free shipping discount. If available for all products do not apply any other discount.
			$freeShippings = scDiscount::getFreeShippingMethods($cart, $discounts, $subTotal, scDiscountsContainer::singleton()->getDiscounts());
			
			// Set free shipping method only if there are such and there are any products in the cart
			if ((count($freeShippings) > 0) && ($cart->getItemsCount() > 0)) {
				$cart->setFreeShippingMethodId(reset($freeShippings));
			}
			
			$cartItems = $cart->getItems();
			
			$discount = scMultiCurrencyAmount::getZero();
			foreach ($cartItems as $item) {
				$listingItem = $item->getProductListingItem();
				
				// It is possible to have a discount that is applied to a product, but is shown as order total line
				$productDiscount = $discounts->getProductDiscount($listingItem, $subTotal, $item, $item->getQuantity(), scDiscount::IMPLEMENTATION_ORDER_TOTAL);
				$discount = scMultiCurrencyAmountMath::add($discount, $productDiscount->getAmount());
			}
			
			// The discount that is applied to the subtotal, and shown on the order total line
			$subtotalDiscount = $discounts->getSubtotalDiscount($subTotal, $cartItems, scDiscount::IMPLEMENTATION_ORDER_TOTAL);
			$discount = scMultiCurrencyAmountMath::add($discount, $subtotalDiscount);
			
			// _t('Discounts')
			if ($discount->get() != 0) {
				return array(new scOrderTotalLine('Discounts', $discount, self::ORDER_TOTAL_TYPE_DISCOUNT));
			}
			return array();
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return scModuleOptionsInfo the option fields
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			return null;
		}
		
		/**
		 * @return scDiscountCalculator
		 */
		protected function createDiscountCalculator()
		{
			return new scDiscountCalculator(scDiscountCalculator::MODE_CART);
		}
	}
	
	/**
	 * Calculates the discounts on pages that shows the product price - product details page, product listing page 
	 */
	class scProductPriceDiscount implements IProductPriceModifier, IModuleOptions {
		/**
		 * Return the price modifications (discounts)
		 * 
		 * @param array $products array of scProductSL objects
		 * @return scProductPriceModification[]
		 */
		public function getPriceModifications($products)
		{
			$discounts = array();
			$discountsCalculator = $this->createDiscountCalculator(scDiscountCalculator::MODE_LISTING);
			foreach ($products as $product) {
				$productPriceModifications = $discountsCalculator->getProductDiscount($product);
				if (!is_array($productPriceModifications)) {
					$productPriceModifications = array($productPriceModifications);
				}
				
				$productPriceModificationsByMode = array();
				foreach ($productPriceModifications as $productPriceModification) {
					if ($productPriceModification->getAmount()->get() != 0) {
						$displayMode = $productPriceModification->getDisplayMode();
						$productPriceModificationsByMode[$displayMode][] = $productPriceModification;
					}
				}
				
				// Show the regular discounts first
				if (!empty($productPriceModificationsByMode[scProductPriceModification::DISPLAY_MODE_DISCOUNT])) {
					$discounts = array_merge($discounts, $productPriceModificationsByMode[scProductPriceModification::DISPLAY_MODE_DISCOUNT]);
				}
				// Show all discounts with display mode set to product price after
				if (!empty($productPriceModificationsByMode[scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE])) {
					$discounts = array_merge($discounts, $productPriceModificationsByMode[scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE]);
				}
			}
			return $discounts;
		}
		
		/**
		 * Return the price modifications that are discount applied to a product
		 *
		 * @param scCartItem $cartItem
		 * @return scProductPriceModification[]
		 */
		public function getCartPriceModifications(scCartItem $cartItem)
		{
			if (!($cartItem instanceof scCartItemProduct)) {
				return array();
			}
			$discountsCalculator = $this->createDiscountCalculator(scDiscountCalculator::MODE_CART);
			$subTotal = $this->getCart()->getSubtotal();
			$listingItem = $cartItem->getProductListingItem();
			
			$productPriceModifications = $discountsCalculator->getProductDiscount($listingItem, $subTotal, $cartItem, $cartItem->getQuantity(), scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER);
			if (!is_array($productPriceModifications)) {
				$productPriceModifications = array($productPriceModifications);
			}
			
			$discounts = array();
			foreach ($productPriceModifications as $productPriceModification) {
				if ($productPriceModification->getAmount()->get() == 0)
					continue;
				
				$discounts[] = $productPriceModification;
			}
			
			return $discounts;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return scModuleOptionsInfo the option fields
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			return null;
		}
		
		/**
		 * @return scCart
		 */
		protected function getCart()
		{
			return scCart::getCart();
		}
		
		/**
		 * @param int $mode
		 * @return scDiscountCalculator
		 */
		protected function createDiscountCalculator($mode)
		{
			return new scDiscountCalculator($mode);
		}
	}
	
	/**
	 * Contains the applyable discounts
	 */
	class scDiscountsContainer
	{
		/**
		 * Associative array keys
		 */
		const KEY_TYPE = 'DiscountType';
		const KEY_VALUE = 'DiscountValue';
		const KEY_CONDITIONS = 'Conditions';
		const KEY_SHIPPING_METHOD = 'ShippingMethod';
		const KEY_APPLY_MODE = 'DiscountApplyMode';
		const KEY_IMPLEMENTATION = 'DiscountImplementation';
		const KEY_PAYMENT_MODULE = 'PaymentModuleID';
		
		const KEY_CONDITIONS_ORDER_EXCEED = 'OrderExceeds';
		const KEY_CONDITIONS_CUSTOMER = 'Customer';
		const KEY_CONDITIONS_CUSTOMER_GROUP = 'CustomerGroup';
		const KEY_CONDITIONS_CATEGORY = 'Category';
		const KEY_CONDITIONS_PRODUCT = 'Product';
		const KEY_CONDITIONS_MANUFACTURER = 'Manufacturer';
		const KEY_CONDITIONS_PAYMENT_MODULE = 'PaymentModuleID';
		
		/**
		 * The discounts table
		 */
		protected $_discounts = array();
		
		/**
		 * The one and only instance of the object
		 */
		protected static $_instance = null;

		/**
		 * Initializes the discounts table
		 */
		protected function __construct()
		{			
			$this->initDiscounts();
		}
		
		/**
		 * Prevent explicit object creation
		 */
		protected function __clone()
		{
		}
		
		/**
		 * Returns the one and only instance of the object
		 * 
		 * @return scDiscountsContainer
		 */
		public static function singleton()
		{
			if (is_null(self::$_instance))
				self::$_instance = new scDiscountsContainer();
				
			return self::$_instance;
		}
		
		/**
		 * Returns the discounts table
		 */
		public function getDiscounts()
		{
			return $this->_discounts;
		}
		
		/**
		 * Remove the discount. Used when a discount is allowed single use only
		 * @param int $index the discount index
		 */
		public function deleteDiscount($key)
		{
			unset($this->_discounts[$key]);
		}
		
		/**
		 * Caches up the available discounts
		 * 
		 * @return void
		 */
		protected function initDiscounts()
		{
			if (count($this->_discounts) > 0)
				return;

			$sql =	'SELECT *, Discounts.DiscountID as DiscountID FROM Discounts ' . 
					'LEFT JOIN DiscountProducts USING(DiscountID) ' .
					'LEFT JOIN DiscountCustomers ON Discounts.DiscountID = DiscountCustomers.DiscountID ' .
					'LEFT JOIN DiscountCustomerGroups ON Discounts.DiscountID = DiscountCustomerGroups.DiscountID ' .
					'LEFT JOIN DiscountManufacturers ON Discounts.DiscountID = DiscountManufacturers.DiscountID ' .
					'LEFT JOIN DiscountCategories ON Discounts.DiscountID = DiscountCategories.DiscountID ' .
					'WHERE DiscountIsActive = 1 AND DiscountValidFrom <= CURDATE() AND DiscountValidTo >= CURDATE()' .
					'ORDER BY FIELD(DiscountImplementation, ?, ?)';
			
			$discountRates = miStaticDBUtil::execSelect($sql, array(scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER, scDiscount::IMPLEMENTATION_ORDER_TOTAL));
			
			// Refacture the discount rates array
			foreach ($discountRates as $discountRate) {
				// Save the discount basic information
				$this->_discounts[$discountRate['DiscountID']][self::KEY_TYPE] = $discountRate['DiscountType'];
				$this->_discounts[$discountRate['DiscountID']][self::KEY_VALUE] = $discountRate['DiscountValue'];
				$this->_discounts[$discountRate['DiscountID']][self::KEY_SHIPPING_METHOD] = $discountRate['DiscountFreeShippingMethodID'];
				$this->_discounts[$discountRate['DiscountID']][self::KEY_APPLY_MODE] = $discountRate['DiscountApplyMode'];
				$this->_discounts[$discountRate['DiscountID']][self::KEY_PAYMENT_MODULE] = $discountRate['PaymentModuleID'];
				
				//$this->_discounts[$discountRate['DiscountID']][self::KEY_IMPLEMENTATION] = $discountRate['DiscountImplementation'];
				if ($discountRate['DiscountApplyMode'] == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
					$this->_discounts[$discountRate['DiscountID']][self::KEY_IMPLEMENTATION] = scDiscount::IMPLEMENTATION_ORDER_TOTAL;
				} else {
					$this->_discounts[$discountRate['DiscountID']][self::KEY_IMPLEMENTATION] = scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER;
				}
				
				if (!isset($this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS]))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS] = array();
				
				// Save the conditions
				if ($discountRate['DiscountConditionOrderExceeds'] > 0)				
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_ORDER_EXCEED] = $discountRate['DiscountConditionOrderExceeds'];
				if (!empty($discountRate['CustomerID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_CUSTOMER][$discountRate['CustomerID']] = $discountRate['CustomerID'];
				if (!empty($discountRate['CustomerGroupID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_CUSTOMER_GROUP][$discountRate['CustomerGroupID']] = $discountRate['CustomerGroupID'];
				if (!empty($discountRate['CategoryID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_CATEGORY][$discountRate['CategoryID']] = $discountRate['CategoryID'];
				if (!empty($discountRate['ProductID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_PRODUCT][$discountRate['ProductID']] = $discountRate['ProductID'];
				if (!empty($discountRate['ManufacturerID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_MANUFACTURER][$discountRate['ManufacturerID']] = $discountRate['ManufacturerID'];
				if (!empty($discountRate['PaymentModuleID']))
					$this->_discounts[$discountRate['DiscountID']][self::KEY_CONDITIONS][self::KEY_CONDITIONS_PAYMENT_MODULE][$discountRate['PaymentModuleID']] = $discountRate['PaymentModuleID'];
			}
		}
	}
	
	/**
	 * Class for calculating the discount amounts
	 */
	class scDiscountCalculator
	{
		const MODE_LISTING = 1;
		const MODE_CART = 2;
		const MODE_FEED = 3;
		
		protected $_mode;
		
		protected static $_paymentModules = array();
		
		public function __construct($mode)
		{
			$this->_mode = $mode;
		}
		
		/**
		 * Gives the calculated discount for given item
		 * 
		 * @param IProductListingItem $listingItem
		 * @param float $subTotal
		 * @param scCartItem $cartItem
		 * @param int $quantity
		 * @param int $requestedImplementation NULL or one of scDiscount::IMPLEMENTATION_*
		 * @return mixed array/scProductPriceModification
		 */
		public function getProductDiscount(IProductListingItem $listingItem, scMultiCurrencyAmount $subTotal = null, scCartItem $cartItem = null, $quantity = 1, $requestedImplementation = null)
		{
			$customerId = $this->getCustomerAuth()->getCustomerId();
			$customerGroupId = $this->getCustomerAuth()->getCustomerGroupId();
			
			// If the $cartItem is not null, and is not scCartItemProduct return zero
			if (!is_null($cartItem) && !($cartItem instanceof scCartItemProduct)) {
				// Make sure discounts can be calculated.
				return new scProductPriceModification($listingItem->getProductId(), scMultiCurrencyAmount::getZero(), 0, _t('Discount'), true);
			}
			
			// Get the price from the best source available.
			if (!is_null($cartItem)) {
				$price = $cartItem->getPrice()->getAmount();
			} else {
				$price = scWholesalePrices::getListingItemWholesalePrice($listingItem, $quantity)->getAmount();
			}
			
			$discountAmount = scMultiCurrencyAmount::getZero();
			$discountPercent = 0;
			$discounts = scDiscountsContainer::singleton()->getDiscounts();
			
			$manufacturer = $listingItem->getManufacturer();
			if ($manufacturer instanceof scManufacturerDomainObject) {
				$manufacturerId = $manufacturer->get('ManufacturerID');
			} else {
				$manufacturerId = null;
			}
			
			$productPriceModifications = array();
			foreach ($discounts as $key => $discount) {
				if ($discount[scDiscountsContainer::KEY_TYPE] == scDiscount::TYPE_FREE_SHIPPING) {
					continue;
				}
				
				// If the $discount is not applied per product quantity
				// it should be applied elsewhere (e.g. the getSubtotalDiscount() method)
				if ($discount[scDiscountsContainer::KEY_APPLY_MODE] != scDiscount::APPLY_MODE_PER_QUANTITY) {
					continue;
				}
				
				// If a specific implementation of the discount is required
				// check if the current $discount is of the same implemantaion.
				if (!is_null($requestedImplementation) && ($requestedImplementation != $discount[scDiscountsContainer::KEY_IMPLEMENTATION])) {
					continue;
				}
				
				// Check if all the conditions are met
				if (!$this->checkConditions($discount, $listingItem->getProductId(), $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal))
					continue;
				
				if ($discount[scDiscountsContainer::KEY_APPLY_MODE] == scDiscount::APPLY_MODE_TO_SUBTOTAL) {
					// Apply to subtotal.
					list($amount, $percent) = $this->getDiscount($discount, $subTotal);
					if ($discount[scDiscountsContainer::KEY_IMPLEMENTATION] == scDiscount::IMPLEMENTATION_PRODUCT_PRICE_MODIFIER) {
						// Only apply it once, since otherwise we would duplicate the subtotal discount for each product
						scDiscountsContainer::singleton()->deleteDiscount($key);
					}
				} else {
					// Apply to product.
					list($amount, $percent) = $this->getDiscount($discount, $price);
				}
				
				// Do not allow discount larger than product price
				if ($amount->get() > $price->get()) {
					$amount = clone $price;
					$percent = 100;
				}
				
				if (($discount['PaymentModuleID']) && ($requestedImplementation !== scDiscount::IMPLEMENTATION_ORDER_TOTAL) && ($this->_mode != self::MODE_FEED)) {
					if (empty(self::$_paymentModules)) {
						self::$_paymentModules = miUtil::getKeyValueArray(scCheckoutPaymentMethodWidget::getActivePaymentModules(), 'PaymentModuleID', 'PaymentModuleDisplayName');
					}
					
					$priceModAmount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $amount);
					$priceModPercent = - $percent;
					$priceModDescription = sprintf(_t('%s price'), self::$_paymentModules[$discount['PaymentModuleID']]);
					$priceModDisplayMode = scProductPriceModification::DISPLAY_MODE_PRODUCT_PRICE;
					$productPriceModifications[] = new scProductPriceModification($listingItem->getProductId(), $priceModAmount, $priceModPercent, $priceModDescription, true, $priceModDisplayMode);
					continue;
				}
				
				// Apply the largest available discount for this product
				if ($amount->get() > $discountAmount->get()) {
					$discountAmount = $amount;
					$discountPercent = $percent;
				}
			}
			
			$priceModAmount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $discountAmount);
			$priceModPercent = - $discountPercent;
			
			$largestProductPriceModification = new scProductPriceModification($listingItem->getProductId(), $priceModAmount, $priceModPercent, _t('Discount'), true, scProductPriceModification::DISPLAY_MODE_DISCOUNT);
			if (empty($productPriceModifications)) {
				return $largestProductPriceModification;
			}
			
			$productPriceModifications[] = $largestProductPriceModification;
			return $productPriceModifications;
		}
		
		/**
		 * @return scMultiCurrencyAmount
		 */
		public function getSubtotalDiscount(scMultiCurrencyAmount $subTotal, $cartItems, $requestedImplementation = null)
		{
			$customerId = $this->getCustomerAuth()->getCustomerId();
			$customerGroupId = $this->getCustomerAuth()->getCustomerGroupId();
			
			$discountAmount = scMultiCurrencyAmount::getZero();
			$discounts = scDiscountsContainer::singleton()->getDiscounts();
			
			foreach ($discounts as $key => $discount) {
				if ($discount[scDiscountsContainer::KEY_TYPE] == scDiscount::TYPE_FREE_SHIPPING) {
					continue;
				}
				
				// If the $discount is not applied to the subtotal skip it.
				if (($discount[scDiscountsContainer::KEY_APPLY_MODE] != scDiscount::APPLY_MODE_TO_SUBTOTAL)) {
					continue;
				}
				
				if (!is_null($requestedImplementation) && ($requestedImplementation != $discount[scDiscountsContainer::KEY_IMPLEMENTATION])) {
					continue;
				}
				
				if ($discount['PaymentModuleID']) { // Do not apply discounts by payment module as subtotal
					continue;
				}
				
				$matchedConditions = false;
				foreach ($cartItems as $cartItem) {
					if (!($cartItem instanceof scCartItemProduct)) {
						continue;
					}
					
					$listingItem = $cartItem->getProductListingItem();
					$manufacturer = $listingItem->getManufacturer();
					if ($manufacturer instanceof scManufacturerDomainObject) {
						$manufacturerId = $manufacturer->get('ManufacturerID');
					} else {
						$manufacturerId = null;
					}
					
					if ($this->checkConditions($discount, $listingItem->getProductId(), $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal)) {
						$matchedConditions = true;
						break;
					}
				}
				
				if (!$matchedConditions) {
					// The conditions did not match to any of the cart items. Skip this $discount.
					continue;
				}
				
				list($amount) = $this->getDiscount($discount, $subTotal);
				
				// Apply the largest available discount
				if ($amount->get() > $discountAmount->get()) {
					$discountAmount = $amount;
				}
			}
			
			if ($discountAmount->get() > $subTotal->get()) {
				$discountAmount = clone $subTotal;
			}
			
			return scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $discountAmount);
		}
		
		public function getApplicableFreeShippings(IProductListingItem $listingItem, scMultiCurrencyAmount $subTotal, $discounts)
		{
			$customerId = $this->getCustomerAuth()->getCustomerId();
			$customerGroupId = $this->getCustomerAuth()->getCustomerGroupId();
			
			$applicableFreeShippings = array();
			
			$manufacturer = $listingItem->getManufacturer();
			if ($manufacturer instanceof scManufacturerDomainObject) {
				$manufacturerId = $manufacturer->get('ManufacturerID');
			} else {
				$manufacturerId = null;
			}
			
			foreach ($discounts as $discount) {
				// Check if $discount is a Free Shipping
				if ($discount[scDiscountsContainer::KEY_TYPE] != scDiscount::TYPE_FREE_SHIPPING)
					continue;
				
				// Check if all the conditions are met
				if (!$this->checkConditions($discount, $listingItem->getProductId(), $listingItem->getAllCategoryIds(), $customerId, $customerGroupId, $manufacturerId, $subTotal))
					continue;
				
				$applicableFreeShippings[] = $discount[scDiscountsContainer::KEY_SHIPPING_METHOD];
			}
			
			return $applicableFreeShippings;
		}
		
		/**
		 * Return the discount amount
		 * 
		 * @param array $discount
		 * @param scMultiCurrencyAmount $productPrice
		 * @return scMultiCurrencyAmount
		 */
		protected function getDiscount($discount, $productPrice)
		{
			switch ($discount[scDiscountsContainer::KEY_TYPE]) {
				case scDiscount::TYPE_AMOUNT:
					$amount = scMultiCurrencyAmount::factoryBaseCurrency($discount[scDiscountsContainer::KEY_VALUE]);
					$percent = $discount[scDiscountsContainer::KEY_VALUE] * 100 / $productPrice->get(scCurrencyUtil::getBaseCurrency());
					break;
					
				case scDiscount::TYPE_PERCENTAGE:
					$amount = scMultiCurrencyAmountMath::multiply($productPrice, $discount[scDiscountsContainer::KEY_VALUE]);
					$amount = scMultiCurrencyAmountMath::divide($amount, 100);
					$percent = $discount[scDiscountsContainer::KEY_VALUE];
					break;
					
				default:
					throw new miException(_i('Unknown discount type.'));
			}
			return array($amount, $percent);
		}
		
		/**
		 * Check if all conditions for discount are met
		 * 
		 * @param array $discount
		 * @param int $productId
		 * @param int $categoryId
		 * @param int $customerId
		 * @param int $subtotal
		 * 
		 * @return bool
		 */
		protected function checkConditions($discount, $productId, $categoryIds, $customerId, $customerGroupId, $manufacturerId, scMultiCurrencyAmount $subTotal = null)
		{
			foreach ($discount[scDiscountsContainer::KEY_CONDITIONS] as $condition => $parameter) {
				switch ($condition) {
					case scDiscountsContainer::KEY_CONDITIONS_ORDER_EXCEED:
						if (is_null($subTotal) || ($subTotal->get() < $parameter))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_PRODUCT:
						if (!in_array($productId, $parameter, false))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_CATEGORY:
						$matchingCategories = array_intersect($categoryIds, $parameter);
						if (empty($matchingCategories))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_CUSTOMER:
						if (!$customerId || !in_array($customerId, $parameter))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_CUSTOMER_GROUP:
						if (!$customerGroupId || !in_array($customerGroupId, $parameter))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_MANUFACTURER:
						if (empty($manufacturerId) || !in_array($manufacturerId, $parameter))
							return false;
						break;
					case scDiscountsContainer::KEY_CONDITIONS_PAYMENT_MODULE:
						if ($this->_mode == self::MODE_CART) {
							// Check whether the customer has selected a payment module
							$customerPaymentModuleId = $this->getCartPaymentMethodId();
							if ((!$customerPaymentModuleId) || (!in_array($customerPaymentModuleId, $parameter)))
								return false;
						}
						
						$paymentModuleIsActive = scUtil::getDbValue('PaymentModules', 'PaymentModuleIsActive', 'PaymentModuleID', array_pop($parameter), false);
						if (!$paymentModuleIsActive)
							return false;
						break;
				}
			}
			
			return true;
		}
		
		/**
		 * @return scCustomerAuth
		 */
		protected function getCustomerAuth()
		{
			return scCustomerAuth::singleton();
		}
		
		/**
		 * @return int
		 */
		protected function getCartPaymentMethodId()
		{
			return scCart::getCart()->getPaymentMethod();
		}
	}
?>