<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Bonus Points Order Total module
	 */
	class scOrderTotalPoints implements IOrderTotalModule {
		
		protected $_moduleId;
		
		/**
		 * @var int
		 */
		protected $_bonusPoints;
		
		/**
		 * @var scMultiCurrencyAmount
		 */
		protected $_totalDiscount;
		
		/**
		 * @var int
		 */
		protected $_usedPoints;
		
		/**
		 * Order total module constructor
		 */
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		/**
		 * Returns order total lines
		 * Each order total line array has the following keys:
		 * description - the order total description, shown to the user
		 * amount - the amount shown on the line
		 * type - order total line type
		 * 
		 * @param scCart $cart the shopping cart object
		 * @return array the order total lines info
		 */
		public function getOrderTotalLines(scCart $cart)
		{
			// first don't allow a guest checkout of products that are buyable only with points
			$customer = $this->getCustomer();
			if ($customer === false) {
				return array();
			}
			
			$this->_bonusPoints = $this->getCustomerAllowedPoints($cart);
			
			$this->_totalDiscount = scMultiCurrencyAmount::getZero(); // Store the amount that is being paid using points.
			$this->_usedPoints = 0;
			
			$cartItems = $cart->getItems();
			
			$cartItemsByHash = array();
			$cartItemsPointRates = array();
			foreach ($cartItems as $cartItem) {
				// Reset the order item's bonus points.
				// They will be recalculated.
				$cartItem->setOrderItemBonusPoints(0);
				
				// Check if points are even applicable
				$cartItemPointsMode = $cartItem->getBonusPointsMode();
				if ($cartItemPointsMode == scBonusPoints::PRODUCT_MONEY_ONLY) {
					continue;
				}
				$hash = $cartItem->getHash();
				$cartItemsByHash[$hash] = $cartItem;
				$cartItemsPointRates[$cartItemPointsMode][$hash] = $cartItem->getPointsRate();
			}
			
			if (empty($cartItemsPointRates)) {
				return array();
			}
			
			// Sort hashes by their ratio placing those with better ratio first.
			foreach ($cartItemsPointRates as &$hashes) {
				arsort($hashes);
			}
			
			if (isset($cartItemsPointRates[scBonusPoints::PRODUCT_POINTS_ONLY])) {
				foreach ($cartItemsPointRates[scBonusPoints::PRODUCT_POINTS_ONLY] as $hash => $ratio) {
					// Check if there are more points left
					if ($this->_bonusPoints <= 0) {
						break;
					}
					
					$cartItem = $cartItemsByHash[$hash];
					$paidQty = $this->payWithPointsFullItems($cartItem);
					if (is_null($paidQty)) {
						$cart->removeItem($cartItem->getHash());
						throw new miException(_t('You have insufficient bonus points.'));
					}
				}
			}
			
			if (isset($cartItemsPointRates[scBonusPoints::PRODUCT_POINTS_AND_MONEY])) {
				foreach ($cartItemsPointRates[scBonusPoints::PRODUCT_POINTS_AND_MONEY] as $hash => $ratio) {
					// Check if there are more points left
					if ($this->_bonusPoints <= 0) {
						break;
					}
					
					$cartItem = $cartItemsByHash[$hash];
					$priceInPoints = $cartItem->getPriceInPoints();
					if ($priceInPoints == 0) {
						continue;
					}
					
					// Check if the points can cover the full price of the item.
					if ($this->_bonusPoints >= $priceInPoints) {
						
						$paidQty = $this->payWithPointsFullItems($cartItem);
						
						// If the complete quantity is paid with points go for the next item
						if ($cartItem->getQuantity() == $paidQty) {
							continue;
						}
					}
					
					//  When an item is partialy paid in points, a proportional part is be discounted.
					$this->payWithPointsPartialItems($cartItem);
				}
			}
			
			if ($this->_totalDiscount->get() == 0) {
				return array();
			}
			
			$discount = scMultiCurrencyAmountMath::subtract(scMultiCurrencyAmount::getZero(), $this->_totalDiscount);
			
			// _t('%s bonus points')
			return array(new scOrderTotalLineWithParam('%s bonus points', $discount, IOrderTotalModule::ORDER_TOTAL_TYPE_POINTS, $this->_usedPoints));
		}
		
		protected function getCustomerAllowedPoints(scCart $cart)
		{
			if (!scSettings::getSettingValue(scSettings::SETTING_BONUS_POINTS_MANUAL_SPENDING)) {
				// If no manual approval is required
				return $this->getCustomer()->get('CustomerBonusPoints');
			}
			
			$customerInfo = $cart->getCustomerInfo();
			if (!isset($customerInfo['CustomerAllowedPoints'])) {
				// If the customer has not yet made his descission (probably not reached the page where he does that)
				return $this->getCustomer()->get('CustomerBonusPoints');
			}
			if ($customerInfo['CustomerAllowedPoints']) {
				return $this->getCustomer()->get('CustomerBonusPoints');
			}
			return 0;
		}
		
		/**
		 * Attempt to pay a cart item with points
		 * 
		 * @param scCartItem $cartItem
		 * @return int the paid quantity
		 */
		protected function payWithPointsFullItems(scCartItem $cartItem)
		{
			$priceInPoints = $cartItem->getPriceInPoints();
			
			// Check if the earned points are sufficient
			if ($priceInPoints > $this->_bonusPoints) {
				// Just skip the item if there are not enough points for it.
				return;
			}
			
			$paidQty = min(floor($this->_bonusPoints / $priceInPoints), $cartItem->getQuantity());
			
			$this->_bonusPoints -= $priceInPoints * $paidQty;
			$this->_usedPoints += $priceInPoints * $paidQty;
			$cartItem->setOrderItemBonusPoints($priceInPoints * $paidQty);
			
			// Add the amount to the total discount.
			$paidAmount = scMultiCurrencyAmountMath::multiply($cartItem->getPriceObj()->getAmount(), $paidQty);
			$this->_totalDiscount = scMultiCurrencyAmountMath::add($this->_totalDiscount, $paidAmount);
			
			if (($cartItem->getBonusPointsMode() == scBonusPoints::PRODUCT_POINTS_ONLY) && ($cartItem->getQuantity() != $paidQty)) {
				throw new miException(_t('You have insufficient bonus points.'));
			}
			
			return $paidQty;
		}
		
		protected function payWithPointsPartialItems(scCartItem $cartItem)
		{
			if ($this->_bonusPoints <= 0)
				return;
			
			// Calculate discount amount
			$discountAmount = scMultiCurrencyAmount::factoryBaseCurrency($cartItem->getPointsRate() * $this->_bonusPoints);
			
			if ($discountAmount->get() >= $cartItem->getPriceObj()->getAmount()->get())
				throw new miException(_i('Partial points payment when the item can be paid with points in full.'));
			
			// Add it to the total
			$this->_totalDiscount = scMultiCurrencyAmountMath::add($this->_totalDiscount, $discountAmount);
			$this->_usedPoints += $this->_bonusPoints;
			
			// Tell the item what part of it is being paid using points.
			$cartItem->setOrderItemBonusPoints($this->_bonusPoints);
						
			// Mark all points used
			$this->_bonusPoints = 0;
		}
		
		/**
		 * @return scCustomerDomainObject
		 */
		protected function getCustomer()
		{
			return scCustomerAuth::singleton()->getCustomer();
		}
	}
?>