<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Subtotal Order Total module
	 *
	 */
	class scOrderTotalShipping implements IOrderTotalModule, IModuleOptions {
		protected $_moduleId;

		const OPTION_NAME_PRICE_IN_TOTAL = 'ShipmentPriceInGrandTotal';
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getOrderTotalLines(scCart $cart)
		{
			if ($cart->getItemsCount() == 0) {
				return array();
			}
			
			$shippingModuleId = $cart->getShippingModuleId();
			if ($shippingModuleId == 0)
				return array();
			
			$moduleId = scUtil::getDbValue('ShippingModules', 'ModuleID', 'ShippingModuleID', $shippingModuleId, false);
			if ($moduleId === false)
				throw new miException(_t('Invalid ShippingModuleID.'));
			
			$shippingMethod = $cart->getShippingMethod();
			
			$className = scModulesFactory::getModuleClassName($moduleId, 'IShippingModule');
			$shippingModule =  new $className($moduleId, $shippingModuleId);
			try {
				$shippingPrices = $shippingModule->getShippingRates($cart, array($shippingMethod));
			} catch (miException $exception) {
				$shippingModuleDisplayName = scUtil::getMLDbValue('ShippingModulesML', 'ShippingModuleDisplayName', 'ShippingModuleID', $shippingModuleId);
				throw new miException(sprintf(_t('Exception from shipping module %s: %s.'), $shippingModuleDisplayName, $exception->getMessage()));
			}
			
			// check that we have the price
			if (!isset($shippingPrices[IShippingModule::SHIPPING_RATES]))
				return array();
//				throw new miException(sprintf(_t('No shipping rates for the selected shipping module %s.'), $className));
			
			if (!isset($shippingPrices[IShippingModule::SHIPPING_RATES][$shippingMethod]))
				return array();
//				throw new miException(sprintf(_t('No shipping rate for the selected shipping method "%s". Shipping module "%s".'), $shippingMethod, $className));
			
			$shippingModuleAndMethod = $shippingModuleId . '-' . $shippingMethod;
			if ($this->getDiscountFreeShippingMethod($cart) == $shippingModuleAndMethod) {
				$amount = 0;
				$description = 'Free Shipping'; // _t('Free Shipping')
				$isFreeShipping = true;
			} else {
				$amount = $shippingPrices[IShippingModule::SHIPPING_RATES][$shippingMethod];
				$description = 'Shipping'; // _t('Shipping')
				$isFreeShipping = false;
			}
			
			// Convert to base currency, if needed
			if (isset($shippingPrices[IShippingModule::SHIPPING_RATES_CURRENCY]) and
				($shippingPrices[IShippingModule::SHIPPING_RATES_CURRENCY] != scCurrencyUtil::getBaseCurrencyCode()))
			{
				// Set the amount in both current and base currency
				$orderTotalLineAmount = scMultiCurrencyAmount::factory();
				
				$currency = scCurrencyUtil::getBaseCurrency();
				$baseCurrencyAmount = scCurrencyUtil::convert($amount, $shippingPrices[IShippingModule::SHIPPING_RATES_CURRENCY], $currency);
				$orderTotalLineAmount->set($currency, $baseCurrencyAmount);
				
				if (scCurrencyUtil::getCurrentCurrency()->getCurrencyCode() != scCurrencyUtil::getBaseCurrency()->getCurrencyCode()) {
					// Do the conversion only if the current currency is not the default one.
					$currency = scCurrencyUtil::getCurrentCurrency();
					$currentCurrencyAmount = scCurrencyUtil::convert($amount, $shippingPrices[IShippingModule::SHIPPING_RATES_CURRENCY], scCurrencyUtil::getCurrentCurrency());
					$orderTotalLineAmount->set($currency, $currentCurrencyAmount);
				}
			} else {
				$orderTotalLineAmount = scMultiCurrencyAmount::factoryBaseCurrency($amount);
			}
			
			// Get the payment method to check if shiping is included in the total amount.
			$paymentMethodId = $this->getCart()->getPaymentMethod();
			// Check if the shipping is covered by the store (free shipping) or by the buyer.
			// If it is covered by the buyer, check if it is included in the order total (it is invoiced).
			if ($isFreeShipping || $this->shippingIncludedInTotal($paymentMethodId)) {
				return array(new scOrderTotalLine($description, $orderTotalLineAmount, self::ORDER_TOTAL_TYPE_SHIPPING));
			}
			
			// The shipping amount is going to be paid to the shipper upon delivery.
			// The store does not pay to the shipper, the buyer does.
			
			// Get the display price to let the customer know what they'll have to pay to the shipper.
			$displayPrice = scPrice::factory($orderTotalLineAmount)->getDisplayPrice();
			// Set the line's amount to zero so it is not included in the total.
			$orderTotalLineAmount = scMultiCurrencyAmount::getZero();
			
			// _t('Shipping cost of %s is paid upon delivery')
			return array(new scOrderTotalLineWithParam('Shipping cost of %s is paid upon delivery', $orderTotalLineAmount, self::ORDER_TOTAL_TYPE_SHIPPING, $displayPrice));
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$optionFields = array(
				array(
					'field' => 'scOrderTotalShippingOptionsWidget',
					'data' => 'ShipmentPriceInGrandTotal',
					'properties' => array(
						'template' => 'modules/ordertotal/shipping/priceintotal/widget.tmpl'
					)
				)
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($optionFields, $webForm);
			$options->getOptionField('ShipmentPriceInGrandTotal')->setRadioButtons($this->getPaymentModuleOptions());
			$options->setTemplateFile('modules/ordertotal/shipping/options.tmpl');
			return $options;
		}
		
		protected function getPaymentModuleOptions()
		{
			$query = 'SELECT PaymentModules.PaymentModuleID, PaymentModulesML.PaymentModuleDisplayName'.
			' FROM PaymentModules LEFT JOIN PaymentModulesML ON'.
			' (PaymentModules.PaymentModuleID = PaymentModulesML.PaymentModuleID AND PaymentModulesML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId() .')' .
			' WHERE PaymentModules.PaymentModuleIsActive = 1'.
			' ORDER BY PaymentModules.PaymentModuleSort';
				
			 return miUtil::getDBArray($query, 'PaymentModuleID', 'PaymentModuleDisplayName');
		}
		
		protected function getDiscountFreeShippingMethod(scCart $cart)
		{
			$freeShippingMethodId = $cart->getFreeShippingMethodId();
			return scUtil::getDbValue('ShippingMethods', 'CONCAT_WS("-", ShippingModuleID, ShippingMethodCode)', 'ShippingMethodID', $freeShippingMethodId, '');
		}
		
		/**
		 * @return scCart
		 */
		protected function getCart()
		{
			return scCart::getCart();
		}
		
		public function shippingIncludedInTotal($paymentModuleId)
		{
			$includedInTotalOptionValue = scModuleOptions::getModuleOption($this->_moduleId, self::OPTION_NAME_PRICE_IN_TOTAL);
			if ($includedInTotalOptionValue === false) {
				return true;
			}
			if (!in_array($paymentModuleId, explode(',', $includedInTotalOptionValue))) {
				return true;
			}
			return false;
		}
	}
	
		
	class scOrderTotalShippingOptionsWidget extends miRadioWidget 
	{	
		protected $_template;
		
		public function __construct(miWebForm &$webForm, $properties = array())
		{
			parent::__construct($webForm, $properties);
			$this->_template = empty($properties['template']) ? '' : $properties['template'];
		}
		
		public function getEditableControl()
		{
			$values = explode(',', $this->_webForm->getFormData($this->_fieldName));
			$t = new scTemplateParser();	
			$checkboxes = new miTemplateParserSectionInfo('checkboxes');
			$templateData = array();
			$checkedSubSections = array();
			foreach ($this->_radioButtons as $key => $paymentModuleName) {
				$checked = (int)in_array($key, $values);
				$checkedSubSections[] = new miTemplateParserSectionInfo('checked', $checked);
				$templateData[] = array(
					'id' => $this->_fieldName . '_' . $key,
					'name' => $name =  $this->_fieldName . '[]',
					'value' => $key,
					'label' => $paymentModuleName
				);
			}
			$checkboxes->setIterationsCount(count($templateData));
			$checkboxes->addValues(miUIUtil::templetizeRows(miUIUtil::transposeRows($templateData)));
			$checkboxes->addSubsectionsArray('checked', $checkedSubSections);
			
			$t->addSectionInfo($checkboxes);
			$t->readTemplate($this->_template);
			return array($this->_fieldName => $t->templateParse());
		}
		
		public function getData()
		{
			$value = miGetParamDefault($this->_fieldName, '');
			if (!is_array($value)) {
				return array($this->_fieldName => '');
			}
			return array($this->_fieldName => implode(',', $value));
		}
	}
?>