<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Tax Order Total module
	 *
	 */
	class scOrderTotalTax implements IOrderTotalModule, IModuleOptions {
		protected $_moduleId;

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getOrderTotalLines(scCart $cart)
		{
			$taxRates = array();
			$taxAmounts = array();
			$taxCartItems = array();
			
			$customerVatCountry = '';
			$customerInfo = $cart->getCustomerInfo();
			if (isset($customerInfo['CustomerShippingVatNumber']) && !empty($customerInfo['CustomerShippingVatNumber'])) {
				$customerVatCountry = scVatNumbersUtil::getCountryFromNumber($customerInfo['CustomerShippingVatNumber']);
			}
			
			$cartItems = $cart->getItems();
			foreach ($cartItems as $cartItem) {
				$taxClasses = scTax::getProductTaxClasses($cartItem->getProductId());
				foreach ($taxClasses as $taxClassId) {
					$taxInfo = scTax::getTaxInfoByTaxClassId($taxClassId);
					$taxId = $taxInfo['TaxID'];
					
					// Find the product tax rate
					$taxZoneId = scZones::getTaxZoneId($taxId, $cart->getCustomerInfo());
					$taxRate = scTax::getTaxRate($taxZoneId, $taxClassId);
					
					// If store owner is VAT registered, and the customer has provided VAT number, and the customer is in a different country
					// then this is Intra-Community delivery - zero the rate. The VAT will be paid by the receiving party to it's government
					if (!empty($taxInfo['TaxRegNumber'])) {
						$taxNumberCountry = scVatNumbersUtil::getCountryFromNumber($taxInfo['TaxRegNumber']);
						if (!empty($customerVatCountry) && !empty($taxNumberCountry)) {
							if ($customerVatCountry != $taxNumberCountry) {
								$taxRate = 0;
							}
						}
					}
					
					// Collect info on the different applicable taxes by tax id
					$taxAmounts[$taxId][] = scTax::getTax($taxRate, $cartItem);
					$taxRates[$taxId][] = $taxRate;
					$taxCartItems[$taxId][] = $cartItem;
				}
			}
			
			// Create the tax description lines
			// Also show the tax rate for a tax if all cart items taxed with it have the same tax rate
			$taxLines = array();
			foreach ($taxRates as $taxId => $taxRatesArray) {
				$taxObject = $this->getTaxObject($taxId);
				
				// _t('Tax %s')
				$taxDescription = $taxObject->getTaxLabel();
				$taxRatesArray = array_unique($taxRatesArray);
				if (count($taxRatesArray) == 1)
					$taxDescription .= ' (' . $taxRatesArray[0] . '%)';
				
				$taxableAmountChange = $this->getTotalsTaxableAmount($cart, $taxObject->isShippingTaxed());
				$averageTaxPercentage = $this->getAverageTaxRate($taxRates[$taxId], $taxCartItems[$taxId]) / 100;
				$taxableAmountChange = scMultiCurrencyAmountMath::multiply($taxableAmountChange, $averageTaxPercentage);
				
				$taxAmount = scMultiCurrencyAmount::getZero();
				foreach ($taxAmounts[$taxId] as $amount) {
					$taxAmount = scMultiCurrencyAmountMath::add($taxAmount, $amount);
				}
				$taxAmount = scMultiCurrencyAmountMath::add($taxAmount, $taxableAmountChange);
				
				$taxLines[] = new scOrderTotalLineWithParam('Tax %s', $taxAmount, self::ORDER_TOTAL_TYPE_TAX, $taxDescription);
			}
			
			return $taxLines;
		}
		
		
		/**
		 * Factory method for tax objects
		 * 
		 * @param int $taxId
		 * @return scTax tax object
		 */
		protected function getTaxObject($taxId)
		{
			static $taxObjects = array();
			if (!isset($taxObjects[$taxId]))
				$taxObjects[$taxId] = new scTax($taxId);
			return $taxObjects[$taxId];
		}
		
		/**
		 * Computes the average tax rate
		 * Computes weighted average using the cart item total 
		 * Used when we have have items in the cart with different tax rates for a tax
		 * 
		 * @param array the tax rate for each cart item
		 * @param array the cart items
		 * @return float the average tax rate
		 */
		protected function getAverageTaxRate($taxRates, $cartItems)
		{
			$totals = array();
			$subtotal = 0;
			foreach ($cartItems as $key => $cartItem) {
				$cartItemTotal = $cartItem->getTotal()->get();
				$subtotal += $cartItemTotal;
				$totals[] = $cartItemTotal * $taxRates[$key];
			}
			return array_sum($totals) / $subtotal;
		}
		
		/**
		 * Returns the amount by which the taxable amount should be changed
		 * The result can be positive or negative.
		 * 
		 * @param scCart $cart
		 * @param boolean $isShippingTaxed true if the shipping should be taxed
		 * @return scMultiCurrencyAmount how much the taxable amount should be changed
		 */
		protected function getTotalsTaxableAmount(scCart $cart, $isShippingTaxed)
		{
			$orderTotalLines = $cart->getOrderTotal()->getCurrentOrderTotalLines();
			
			$grandTotal = scMultiCurrencyAmount::getZero();
			foreach ($orderTotalLines as $orderTotalLine) {
				if ($orderTotalLine->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL)
					continue;
				if (($orderTotalLine->getType() == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) and !$isShippingTaxed)
					continue;
				$grandTotal = scMultiCurrencyAmountMath::add($grandTotal, $orderTotalLine->getAmount());
			}
			
			return $grandTotal;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			return null;
		}
	}
?>