<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Authorize.Net AIM payment processing module
	 */
	class scPaymentModuleAuthorizeAIM implements IPaymentModule, IModuleOptions {
		const AUTHORIZE_NET_URL = 'https://secure.authorize.net/gateway/transact.dll';
		const AUTHORIZE_NET_DEVELOPER_URL = 'https://test.authorize.net/gateway/transact.dll';
		
		const MODE_LIVE = 'Live';
		const MODE_DEMO = 'Demo';
		const MODE_DEVELOPER = 'Developer';
		
		protected $_moduleId;

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) 
				return scPaymentForms::getCreditCardForm();
		}
		
		public function hasCheckoutStep()
		{
			return false;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'USD';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('LoginID', 'Mode', 'TransactionKey', 'MD5Hash'));
			
			$params = array(
				'x_login' => $options['LoginID'],
				'x_tran_key' => $options['TransactionKey'],
				'x_version' => '3.1',
				'x_test_request' => ($options['Mode'] != self::MODE_LIVE),
				
				'x_delim_data' => 'TRUE',
				'x_delim_char' => '*',
				'x_encap_char' => '\\',
				
				'x_amount' => $payAmount,
				'x_currency_code' => 'USD',
				'x_method' => 'CC',
				'x_type' => 'AUTH_CAPTURE',
				'x_invoice_num' => scOrdersInProgress::singleton()->getLastOrderId(),
				
				'x_card_num' => $paramsPost['CardNumber'],
				'x_exp_date' => sprintf('%02d/%4d', $paramsPost['CardExpirationMonth'], $paramsPost['CardExpirationYear']),
				'x_card_code' => $paramsPost['CardCVV'],
				
				'x_relay_response' => 'FALSE',
				'x_customer_ip' => $_SERVER['REMOTE_ADDR'],
				
				'x_first_name' => $customerInfo['CustomerBillingFirstName'],
				'x_last_name' => $customerInfo['CustomerBillingLastName'],
				'x_company' => $customerInfo['CustomerBillingCompany'],
				'x_address' => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'x_city' => $customerInfo['CustomerBillingCity'],
				'x_state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'x_zip' => $customerInfo['CustomerBillingZip'],
				'x_country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'x_email' => $customerInfo['CustomerEmail'],
				'x_phone' => $customerInfo['CustomerBillingPhone'],
				
				'x_ship_to_first_name' => $customerInfo['CustomerShippingFirstName'],
				'x_ship_to_last_name' => $customerInfo['CustomerShippingLastName'],
				'x_ship_to_address' => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2'],
				'x_ship_to_city' => $customerInfo['CustomerShippingCity'],
				'x_ship_to_state' => scCountry::getStateCode($customerInfo['CustomerShippingState']),
				'x_ship_to_zip' => $customerInfo['CustomerShippingZip'],
				'x_ship_to_country' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
			);
			
			$url = ($options['Mode'] == self::MODE_DEVELOPER) ? self::AUTHORIZE_NET_DEVELOPER_URL : self::AUTHORIZE_NET_URL;
			$result = scHttp::post($url, http_build_query($params));
			$response = explode('\\*\\', trim($result, '\\'));
			array_unshift($response, '');
			
			// If the response is not 1, 2 or 4 then it is either 3 (error) or unknown
			if (($response[1] != 1) and ($response[1] != 2) and ($response[1] != 4)) {
				throw new scPaymentProcessException(_t('Processing error.'),
					array(_t('OrderID') => $response[8],
					_t('Response Code') => $response[1], _t('Response Subcode') => $response[2],
					_t('Response Reason Text') => $response[4], _t('Response Reason Code') => $response[3]));
			}
			
			$key = md5($options['MD5Hash'] . $options['LoginID'] . $response[7] . $response[10]);
			if (strtoupper($key) != $response[38])
				throw new scPaymentProcessException(_t('MD5 Hash does not match.'), array(_t('OrderID') => $response[8]));
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($response[8], $response[10], 'USD');
			
			// Return the result
			$result = new scPaymentProcessResult($response[8], $response[10], 'USD', $response[7]);
			$result->setTransactionNote($response[4] . ' (' . $response[3] . ')');
			$result->setPaymentNotes($result->getTransactionNote());
			
			if ($response[1] == 1)
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			elseif ($response[1] == 2)
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			elseif ($response[1] == 4)
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'LoginID',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'TransactionKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MD5Hash',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);

			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_DEMO => _t('Demo'), self::MODE_DEVELOPER => _t('Developer'));
			$options->getOptionField('Mode')->setOptions($modes);
			$options->setTemplateFile('modules/payment/authorize/aim_options.tmpl');
			return $options;
		}
	}
?>