<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/cp1251.php');
	
	/**
	 * Borica.bg payment processing module
	 */
	class scPaymentModuleBoricaGate implements IPaymentModule, IModuleOptions {
		const BORICA_APGW_URL = 'https://gate.borica.bg/boreps/registerTransaction';
		const BORICA_APGW_TEST_URL = 'https://88.203.239.27/boreps/registerTransaction';
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		protected $_supportedCurrencies = array('BGN', 'USD', 'EUR');
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('AccountCurrencyCode'));
			$currencyCode = $options['AccountCurrencyCode'];
			if (!in_array($currencyCode, $this->_supportedCurrencies))
				$currencyCode = 'BGN';	// fallback to BGN
			return $currencyCode;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$isoLangCode = strtoupper(scMultiLanguage::getCurrentLanguageCode());
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('TID', 'OrderDescription', 'Mode', 'AccountCurrencyCode', 'ApgwPrivateKey', 'ApgwTestPrivateKey'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$api = new scPaymentModuleBoricaApi($options['TID'], $options['Mode'], $options['ApgwPrivateKey'], $options['ApgwTestPrivateKey'], '', '');
			$data = $api->buildBoricaMessage(scPaymentModuleBoricaApi::TRANSACTION_TYPE_AUTH, $payAmount, $this->getPaymentCurrencyCode(), scOrdersInProgress::singleton()->getLastOrderId(), $options['OrderDescription'], $isoLangCode);
			
			$url = ($options['Mode'] == self::MODE_TEST) ? self::BORICA_APGW_TEST_URL : self::BORICA_APGW_URL;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, array('eBorica' => $data));
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('eBorica'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('TID', 'Mode'));
			
			$data = base64_decode(strtr($params['eBorica'], ' ', '+'));
			$dataArray = unpack('A2transactionType/A14timestamp/A12amount/A8terminal/A15orderId/A2respCode/A3version/A128signature', $data);
			
			// Verify the signature. Everything before the signature (56 bytes) is the data
			$certificate = $this->getBoricaCertificate($options['Mode']);
			if (!$this->verifySignature(substr($data, 0, 56), $dataArray['signature'], $certificate)) {
				throw new miException(_t('Cannot verify Borica signature. The message was corrupted or the store certificate is invalid.'));
			}
			
			// Verify order id and amount
			$currencyCode = $this->getPaymentCurrencyCode();
			scPaymentProcess::checkPaymentToOrder($dataArray['orderId'], $dataArray['amount'] / 100, $currencyCode);
			
			// Verify the terminal id
			if ($dataArray['terminal'] != $options['TID']) {
				throw new miException(_t('Terminal ID (TID) does not match.'));
			}
			
			$result = new scPaymentProcessResult($dataArray['orderId'], $dataArray['amount'] / 100, $currencyCode);
			$result->setPaymentStatus(($dataArray['respCode'] == scPaymentModuleBoricaApi::RESP_CODE_SUCCESS) ? scPaymentStatus::PAYMENT_STATUS_CAPTURE : scPaymentStatus::PAYMENT_STATUS_DECLINED);
			
			$responseCodes = scPaymentModuleBoricaApi::getResponseCodes();
			if (isset($responseCodes[$dataArray['respCode']]))
				$result->setPaymentNotes(sprintf(_i('%s (%s)'), $responseCodes[$dataArray['respCode']], $dataArray['respCode']));
			else
				$result->setPaymentNotes(sprintf(_t('Unknown response code: %s.'), $dataArray['respCode']));
			
			return $result;
		}
		
		/**
		 * Returns the Borica public key certificate
		 * 
		 * @param int $mode
		 * @return string
		 */
		protected function getBoricaCertificate($mode)
		{
			$certificateFile = ($mode == self::MODE_TEST) ? 'borica_certificate_test.pem' : 'borica_certificate.pem';
			return file_get_contents(dirname(__FILE__) . '/' . $certificateFile);
		}
		
		/**
		 * Calculates 128-bytes signature of $data using the private key
		 * 
		 * @param string $data
		 * @param string $privateKey
		 * @return string the signature
		 */
		protected function calcSignature($data, $privateKey)
		{
			$pkeyid = openssl_pkey_get_private($privateKey);
			openssl_sign($data, $signature, $pkeyid);
			openssl_free_key($pkeyid);
			
			return $signature;
		}
		
		/**
		 * Verifies the signature
		 * 
		 * @param string $data
		 * @param string $signature
		 * @param string $publicKey
		 * @return boolean
		 */
		protected function verifySignature($data, $signature, $publicKey)
		{
			$pubkeyid = openssl_pkey_get_public($publicKey);
			$ok = openssl_verify($data, $signature, $pubkeyid);
			openssl_free_key($pubkeyid);
			
			return ($ok == 1) ? true : false;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'TID',
					'properties' => array('minLength' => 8, 'maxLength' => 8)
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'AccountCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'ApgwPrivateKey',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApgwCertificateSignRequest',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'ApgwCertificate',
				),
				
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'ApgwTestPrivateKey',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'ApgwTestCertificateSignRequest',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'ApgwTestCertificate',
				),
				
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'EtlogCertificate',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'EtlogPrivateKey',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'EtlogCertificateSignRequest',
				),
				
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 125),
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('AccountCurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test')), self::MODE_TEST);
			$options->setTemplateFile('modules/payment/borica/options.tmpl');
			return $options;
		}
	}
	
	class scPaymentModuleBoricaApi {
		const BORICA_APGW_URL = 'https://gate.borica.bg/boreps/registerTransaction';
		const BORICA_APGW_TEST_URL = 'https://gatet.borica.bg/boreps/registerTransaction';
		
		const BORICA_MANAGE_URL = 'https://gate.borica.bg/boreps/manageTransaction';
		const BORICA_MANAGE_TEST_URL = 'https://gatet.borica.bg/boreps/manageTransaction';
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		const TRANSACTION_TYPE_AUTH = '10';	// Direct capture
		const TRANSACTION_TYPE_DELAYED_AUTH = '20';	// Authorise only
		const TRANSACTION_TYPE_EXECUTE_DELAYED_AUTH = '21';	// Capture
		const TRANSACTION_TYPE_CANCEL_DELAYED_AUTH = '22'; // Void authorisation 
		const TRANSACTION_TYPE_RECURRING_REQUEST = '30';
		const TRANSACTION_TYPE_RECURRING_REQUEST_AUTH = '31';
		const TRANSACTION_TYPE_RECURRING_REQUEST_CANCEL = '32';
		const TRANSACTION_TYPE_REVERSAL = '40';
		
		const RESP_CODE_SUCCESS = '00';
		
		protected $_tid;
		protected $_mode;
		protected $_privateKey;
		protected $_testPrivateKey;
		protected $_certificate;
		protected $_testCertificate;
		
		public function __construct($tid, $mode, $privateKey, $testPrivateKey, $certificate, $testCertificate)
		{
			$this->_tid = $tid;
			$this->_mode = $mode;
			$this->_privateKey = $privateKey;
			$this->_testPrivateKey = $testPrivateKey;
			$this->_certificate = $certificate;
			$this->_testCertificate = $testCertificate;
		}
		
		public function doReversal($orderId, $amount, $currencyCode)
		{
			$eBorica = $this->buildBoricaMessage(self::TRANSACTION_TYPE_REVERSAL, $amount, $currencyCode, $orderId, 'Reversal', 'BG');
			
			$url = ($this->_mode == self::MODE_TEST) ? self::BORICA_MANAGE_TEST_URL : self::BORICA_MANAGE_URL;
			
			try {
				$certFilename = tempnam('/tmp/', 'borica.cer');
				file_put_contents($certFilename, ($this->_mode == self::MODE_TEST) ? $this->_testCertificate : $this->_certificate);
				
				$keyFilename = tempnam('/tmp/', 'borica.key');
				file_put_contents($keyFilename, ($this->_mode == self::MODE_TEST) ? $this->_testPrivateKey : $this->_privateKey);
				
				$curlOptions = array(
					'sslVersion' => 3,
					'sslCert' => $certFilename,
					'sslKey' => $keyFilename
				);
				if ($this->_mode == self::MODE_TEST)
					$curlOptions['sslVerifyPeer'] = false;
				$result = scHttp::get($url . '?eBorica=' . urlencode($eBorica), $curlOptions);
			} catch (Exception $e) {
				unlink($certFilename);
				unlink($keyFilename);
				throw $e;
			}
			unlink($certFilename);
			unlink($keyFilename);
			
			$data = base64_decode(strtr($result, ' ', '+'));
			$dataArray = unpack('A2transactionType/A14timestamp/A12amount/A8terminal/A15orderId/A2respCode/A3version/A128signature', $data);
			
			// Check for error condition
			if ($dataArray['respCode'] != self::RESP_CODE_SUCCESS) {
				$responseCodes = self::getResponseCodes();
				if (isset($responseCodes[$dataArray['respCode']]))
					throw new scPaymentProcessException(sprintf(_i('%s (%s)'), $responseCodes[$dataArray['respCode']], $dataArray['respCode']));
				else
					throw new scPaymentProcessException(sprintf(_t('Unknown response code: %s.'), $dataArray['respCode']));
			}
		}
		
		public static function getResponseCodes()
		{
			return array(
				'00' => _t('Success.'),
				'03' => _i('Invalid merchant.'),
				'04' => _i('Pick up.'),
				'05' => _i('Do not honour.'),
				'06' => _i('Error.'),
				'12' => _i('Invalid transaction.'),
				'13' => _i('Invalid amount.'),
				'14' => _i('Invalid card number.'),
				'15' => _i('Unable to route at IEM.'),
				'17' => _i('Customer cancellation.'),
				'30' => _i('Format error.'),
				'31' => _i('Issuer sign-off.'),
				'33' => _i('Expired card.'),
				'36' => _i('Restricted card.'),
				'37' => _i('Card acceptor call acquirer security.'),
				'41' => _i('Lost card.'),
				'43' => _i('Stolen card.'),
				'51' => _i('Not sufficient funds.'),
				'54' => _i('Expired card.'),
				'55' => _i('Incorrect secret code.'),
				'57' => _i('Transaction not permitted to cardholder.'),
				'58' => _i('Transaction not permitted to terminal.'),
				'61' => _i('Exceeds withdrawal amount limit.'),
				'62' => _i('Restricted card.'),
				'65' => _i('Exceeds withdrawal frequency limit.'),
				'66' => _i('Card acceptor call acquirer’s security department.'),
				'75' => _i('3 times incorrect secret code.'),
				'80' => _t('Network error.'),
				'81' => _t('Foreign network failure.'),
				'82' => _t('Time-out at IEM.'),
				'83' => _t('Transaction failed.'),
				'85' => _t('Reversal already placed.'),
				'86' => _t('Duplicate transaction.'),
				'87' => _i('Protocol version mismatch.'),
				'88' => _i('Missing BOReq parameter.'),
				'89' => _i('Cannot find original transaction.'),
				'90' => _i('Card not registered in directory server.'),
				'91' => _i('Authorisation system timeout.'),
				'92' => _i('Invalid format of eBorica param.'),
				'93' => _t('Unsuccessful 3D auth.'),
				'94' => _t('Canceled transaction.'),
				'95' => _t('Invalid message signature.'),
				'96' => _t('Internal error.'),
				'97' => _t('Reversal failed.'),
				'98' => _i('Cannot find matching transaction.'),
				'99' => _i('Authorisation denied from TPSS.'),
			);
		}
		
		public function buildBoricaMessage($transactionType, $amount, $currencyCode, $orderId, $orderDescription, $isoLangCode)
		{
			$transactionType = $transactionType;
			$timestamp = date('YmdHis');
			$amountInStotinki = sprintf('%012d', round($amount * 100));
			$terminalId = sprintf('%-8.8s', $this->_tid);
			$orderId = sprintf('%-15.15s', $orderId);
			$description = sprintf('%-125.125s', scCp1251::utf8ToCp1251($orderDescription));
			$languageCode = ($isoLangCode == 'BG') ? 'BG' : 'EN';
			$version = '1.1';
			$currencyCode = $currencyCode;
			
			$data = $transactionType . $timestamp . $amountInStotinki . $terminalId . $orderId . $description . $languageCode . $version . $currencyCode;
//			var_dump($data);
			$privateKey = ($this->_mode == self::MODE_TEST) ? $this->_testPrivateKey : $this->_privateKey;
			$signature = $this->calcSignature($data, $privateKey);
//			var_dump($data);
//			var_dump(base64_encode($data . $signature));
			return base64_encode($data . $signature);
		}
		
		/**
		 * Calculates 128-bytes signature of $data using the private key
		 * 
		 * @param string $data
		 * @param string $privateKey
		 * @return string the signature
		 */
		protected function calcSignature($data, $privateKey)
		{
			$pkeyid = openssl_pkey_get_private($privateKey);
			openssl_sign($data, $signature, $pkeyid);
			openssl_free_key($pkeyid);
			
			return $signature;
		}
	}
?>