<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Check payment processing module
	 */
	class scPaymentModuleCheck implements IPaymentModule, IModuleOptions {
		protected $_moduleId;

		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) {
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('AcceptedCheckType'));
				return scPaymentForms::getCheckForm($options['AcceptedCheckType']);
			}
		}
		
		public function hasCheckoutStep()
		{
			return false;
		}
		
		/**
		 * Returns true if the routing number is valid
		 * 
		 * @param string $routingNumber
		 * @return boolean
		 */
		protected function isABARoutingValid($routingNumber)
		{
			if (strlen($routingNumber) != 9)
				return false;
			
			if (!ctype_digit($routingNumber))
				return false;
				
			// Compute checksum
			$checksum = 0;
			for ($i = 0; $i < 9; $i += 3)
				$checksum += ($routingNumber[$i] * 3) + ($routingNumber[$i+1] * 7) + $routingNumber[$i+2];
			
			if ($checksum == 0)
				return false;
			
			if (($checksum % 10) != 0)
				return false;
				
			return true;
		}
		
		/**
		 * Returns if the sort code is valid - that it consists of 6 digits
		 * 
		 * @param string $sortCode
		 * @return boolean
		 */
		protected function isValidUkSortCode($sortCode)
		{
			// Remove any hyphens or spaces
			$sortCode = strtr($sortCode, array(' ' => '', '-' => ''));
			
			if (strlen($sortCode) != 6)
				return false;
			
			if (!ctype_digit($sortCode))
				return false;
			
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return null;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('AcceptedCheckType'));
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$result = new scPaymentProcessResult($orderId);
			
			if ($options['AcceptedCheckType'] == scPaymentForms::CHECK_TYPE_US) {
				if (!$this->isABARoutingValid($paramsPost['RoutingNumber']))
					$result->addValidatorError(_t('Invalid routing number.'), 'RoutingNumber');
			} else if ($options['AcceptedCheckType'] == scPaymentForms::CHECK_TYPE_UK) {
				if (!$this->isValidUkSortCode($paramsPost['RoutingNumber']))
					$result->addValidatorError(_t('Invalid sort code.'), 'RoutingNumber');
			}
			
			if (empty($paramsPost['AccountNumber']))
				$result->addValidatorError(_t('Please enter the account number.'), 'AccountNumber');
			if (empty($paramsPost['BankName']))
				$result->addValidatorError(_t('Please enter the bank name.'), 'BankName');
			if (empty($paramsPost['NameOnAccount']))
				$result->addValidatorError(_t('Please enter the name on account.'), 'NameOnAccount');
			
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_OFFLINE);
			
			// Success - save the credit card info
			if (!$result->hasValidatorErrors())
				$this->saveCheckInfo($orderId, $options['AcceptedCheckType'], $paramsPost);
			
			return $result;
		}
		
		protected function saveCheckInfo($orderId, $checkType, $paramsPost)
		{
			$checkAccountTypes = scPaymentForms::getCheckAccountTypes();
			if (!isset($checkAccountTypes[$paramsPost['AccountType']]))
				throw new miException(_i('Unknown check account type.'));
			
			$paymentCheck = new scPaymentCheck($orderId);
			$paymentCheck->set('PaymentCheckType', $checkType);
			$paymentCheck->set('PaymentCheckAccountType', $paramsPost['AccountType']);
			
			// Remove spaces or hyphens from the routing number / sort code
			$paymentCheck->set('PaymentCheckRoutingNumber', strtr($paramsPost['RoutingNumber'], array(' ' => '', '-' => '')));
			$paymentCheck->set('PaymentCheckAccount', $paramsPost['AccountNumber']);
			$paymentCheck->set('PaymentCheckBankName', $paramsPost['BankName']);
			$paymentCheck->set('PaymentCheckNameOnAccount', $paramsPost['NameOnAccount']);
			$paymentCheck->save();
		}
		
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miRadioWidget',
					'data' => 'AcceptedCheckType',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('AcceptedCheckType')->setRadioButtons(scPaymentForms::getCheckTypes(), scPaymentForms::CHECK_TYPE_US);
			$options->setTemplateFile('modules/payment/check/options.tmpl');
			return $options;
		}
	}
?>