<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Chronopay payment processing module
	 */
	class scPaymentModuleChronoPay implements IPaymentModule, IModuleOptions {
		protected $_moduleId;
		
		const CHRONOPAY_URL = 'https://secure.chronopay.com/index_shop.cgi';
		
		/**
		 * Supported languages by Chronopay
		 * @var array
		 */
		protected $_supportedLanguages = array(
			'EN', 'RU', 'NL', 'DE', 'ES'
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ProductCurrencyCode'));
			if (!scCurrencyUtil::isCurrencyCodeExists($options['ProductCurrencyCode'])) {
				throw new scPaymentProcessException(sprintf(_i('Unknown currency code %s.'), $options['ProductCurrencyCode']));
			}
			return $options['ProductCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ProductID', 'OrderDescription'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = in_array($isoLangCode, $this->_supportedLanguages) ? $isoLangCode : 'EN';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$street = $customerInfo['CustomerBillingAddress1'];
			if (trim($customerInfo['CustomerBillingAddress2']) != '')
				$street .= ', ' . $customerInfo['CustomerBillingAddress2'];
			
			$params = array(
				'product_id' => $options['ProductID'],
				'product_name' => $options['OrderDescription'],
				'cs1' => $orderId,
//				'return_url' => scPaths::getUrl('modules/payment/chronopay/return.php?OrderID=' . $orderId),
			
				'cb_url' => scPaths::getSecureUrl('modules/payment/chronopay/callback.php'),
				'cb_type' => 'P',
				
				'decline_url' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
				
				'language' => $lang,
				
				'product_price' => $payAmount,
				
				'f_name' => $customerInfo['CustomerBillingFirstName'],
				's_name' => $customerInfo['CustomerBillingLastName'],
				'street' => $street,
				'city' => $customerInfo['CustomerBillingCity'],
				'state' => scCountry::getStateCode($customerInfo['CustomerBillingState']),
				'zip' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryCode3($customerInfo['CustomerBillingCountry']),
				'phone' => $customerInfo['CustomerBillingPhone'],
				'email' => $customerInfo['CustomerEmail'],
			);
			
			$secret = $this->getSecret();
			$cs2 = scCryptUtil::hmac($secret, $orderId, $params['product_id'] . $params['product_price'] . $payCurrencyCode, scCryptUtil::HASH_SHA1);
			$params['cs2'] = $cs2;
			$cs3 = scCryptUtil::hmac($secret, $orderId, scCryptUtil::HASH_MD5);
			$params['cs3'] = $cs3;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::CHRONOPAY_URL, $params);
			return $result;
		}
		
		protected function getSecret()
		{
			$secret = scModuleOptions::getModuleOption($this->_moduleId, 'Secret');
			if ($secret === false) {
				$secret = getmypid() . uniqid(rand(), true);
				scModuleOptions::setModuleOption($this->_moduleId, 'Secret', $secret);
			}
			return $secret;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('cs1', 'cs3'));
			$orderId = $params['cs1'];
			
			$secret = $this->getSecret();
			$cs3 = scCryptUtil::hmac($secret, $orderId, scCryptUtil::HASH_MD5);
			if ($cs3 != $params['cs3'])
				throw new miException(_i('Incorrect signature received from ChronoPay.'));
			
			$result = new scPaymentProcessResult($orderId);
			return $result;
		}
		
		/**
		 * Handles callback return
		 *
		 * @param array $params
		 */
		function processCallback($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('ProductID'));
			
			scPaymentProcess::checkParamsExist($params, array('transaction_type', 'customer_id', 'site_id', 'product_id', 'date', 'time',
				'transaction_id', 'email', 'country', 'name', 'city', 'street', 'phone', 'state', 'zip',
				'language', 'cs1', 'cs2', 'cs3', 'total', 'currency'));
			
			$orderId = $params['cs1'];
			
			// Check the signature
			$secret = $this->getSecret();
			$cs2 = scCryptUtil::hmac($secret, $orderId, $params['product_id'] . $params['total'] . $params['currency'], scCryptUtil::HASH_SHA1);
			if ($cs2 != $params['cs2'])
				throw new miException(_i('Incorrect signature received from ChronoPay.'));
				
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($orderId, $params['total'], $params['currency']);
			
			// Check that it has been paid to the correct email
			if ($options['ProductID'] != $params['product_id'])
				throw new miException(sprintf(_i('The product id received from ChronoPay (%s) does not match the one on file (%s).'), $params['product_id'], $options['ProductID']));
			
			$result = new scPaymentProcessResult($orderId, $params['total'], $params['currency'], $params['transaction_id']);
			
			switch ($params['transaction_type']) {
				case 'onetime';
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
					
				case 'initial';
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					break;
				
				case 'decline';
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					break;
				
				default:
					$result->setPaymentNotes(sprintf(_i('Unknown transaction_type: %s'), $params['transaction_type']));
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			}
			
			foreach ($params as $key => $value) {
				$result->setTransactionDetail($key, $value);
			}
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'ProductID',
					'properties' => array('minLength' => 1)
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
					'properties' => array('minLength' => 1)
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'ProductCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$currencies = scCurrencyUtil::getActiveCurrencyCodes();
			$options->getOptionField('ProductCurrencyCode')->setOptions(array_combine($currencies, $currencies));
			$options->setTemplateFile('modules/payment/chronopay/options.tmpl');
			return $options;
		}
	}
?>