<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * DSK Direct payment processing module
	 */
	class scPaymentModuleDskDirect implements IPaymentModule, IModuleOptions {
		const DSK_DIRECT_SIMULATOR_URL = 'https://www.dskdirect.bg/page/default.aspx?xml_id=/bg-BG/.CardPayments/test';
		const DSK_DIRECT_URL = 'https://www.dskdirect.bg/e-commerce/default.aspx?xml_id=/bg-BG/.CardPayments/';
		
		const PAYMENT_TIMEOUT_MINUTES = 60; // minutes to complete the payment at DSK Direct
		
		protected $_supportedCurrencies = array('BGN', 'USD', 'EUR');
		
		protected $_moduleId;
		
		/**
		 * ISO 639-2 to DSK Direct language code map
		 * @var array
		 */
		protected $_langCodeMap = array(
			// ISO 639-2 => DSK Direct
			'bg' => 'bg-BG',
			'fr' => 'fr-FR',
			'en' => 'en-US'
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('CurrencyCode'));
			$currencyCode = $options['CurrencyCode'];
			if (!in_array($currencyCode, $this->_supportedCurrencies))
				$currencyCode = 'BGN';	// fallback to BGN
			return $currencyCode;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'OrderDescription', 'CurrencyCode', 'MD5Password'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = isset($this->_langCodeMap[$isoLangCode]) ? $this->_langCodeMap[$isoLangCode] : 'en-US';
			
			$params = array(
				'merchant_id' => $options['MerchantID'],
				'amount' => $payAmount,
				'currency_code' => $payCurrencyCode,
				'track_id' => scOrdersInProgress::singleton()->getLastOrderId(),
				'description' => $options['OrderDescription'],
				'payment_timeout' => self::PAYMENT_TIMEOUT_MINUTES,
				'cancel_return_url' => scPaths::getSecureUrl('modules/payment/dskdirect/return.php'),
				'ok_return_url' => scPaths::getSecureUrl('modules/payment/dskdirect/return.php'),
				'language' => $lang,
			);
			
			list($header, $mac) = $this->calcHeaderAndMac($params, $options['MD5Password']);
			$params['header'] = $header;
			$params['mac'] = $mac;
			
			$result = new scPaymentProcessResult();
			$url = ($options['Mode'] == 'Live') ? self::DSK_DIRECT_URL : self::DSK_DIRECT_SIMULATOR_URL;
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * Calculates and returns the header and the mac as needed for the DSK Direct protocol
		 * 
		 * @param array $params
		 * @param string $password
		 * @return array($header,$mac)
		 */
		protected function calcHeaderAndMac($params, $password)
		{
			$header = sprintf('%02d', count($params));
			$header .= implode(',', array_keys($params)) . ',';
			foreach ($params as $param)
				$header .= sprintf('%03d', scUtf8::strlen($param));
			
			// Calculate the mac
			$mac = $header;
			foreach ($params as $param)
				$mac .= $param;
			$mac = md5($mac . $password);
			
			return array($header, $mac);
		}
		
		/**
		 * Checks that the mac is valid
		 * 
		 * @param array $params
		 */
		protected function checkReturnMac($params, $password)
		{
			$header = $params['return_header'];
			$mac = $params['return_mac'];
			if (($header == '') and ($mac == '')) {
				throw new miException('The MD5 password is not setup at the DSK Bank server.');
			}
			
			$macParams = array();
			
			$numParams = substr($header, 0, 2);
			$pos = $oldpos = 2;
			for ($i = 0; $i < $numParams; $i++) {
				$pos = strpos($header, ',', $oldpos);
				if ($pos === false)
					throw new miException(_i('Invalid "header" param from DSK Direct.'));
				
				$param = substr($header, $oldpos, $pos - $oldpos);
				if (!isset($params[$param]))
					throw new miException(sprintf(_i('Missing param "%s" from DSK Direct.'), $param));
				
				$macParams[] = $param;
				$oldpos = $pos + 1;
			}
			
			// Check if the header has enough paramer lengths (each is 3 digit long)
			if ((strlen($header) - $pos - 1) != ($numParams * 3)) {
				throw new miException(_i('Invalid "header" param from DSK Direct (invalid length).'));
			}
			
			// Check if the parameters length match
			$pos += 1;
			foreach ($macParams as $macParam) {
				$length = substr($header, $pos, 3);
				$pos += 3;
				if (strlen($params[$macParam]) != intval($length))
					throw new miException(sprintf(_i('Param "%s" length does not match header.'), $macParam));
			}
			
			// Now check if the mac matches
			$mac = $header;
			foreach ($macParams as $macParam)
				$mac .= $params[$macParam];
			$mac = md5($mac . $password);
			
			if (strcasecmp($params['return_mac'], $mac) != 0)
				throw new miException(_i('Invalid mac received from DSK Direct. Please check that the MD5 Password is correct.'));
		}
		
		protected function getPaymentStatusMessage($paymentStatus)
		{
			$paymentStatusMessages = array(
				0 => _i('Заявка за плащане'),
				1 => _i('Грешка при 3DS верификация'),
				2 => _i('Издателят не приема верификацията'),
				3 => _i('Успешно картово плащане'),
				4 => _i('Неуспешна авторизация'),
				5 => _i('Отказана авторизация'),
				6 => _i('В обработка'),
				// 7 is missing
				8 => _i('Отказано от потребителя'),
				9 => _i('Просрочена валидност на плащането'),
				10 => _i('Създадено плащане в ДСК Директ; не е изпратено към банковата система'),
				11 => _i('Успешно плащане в ДСК Директ'),
				12 => _i('Неуспешно плащане в ДСК Директ'),
				13 => _i('Заключено плащане в ДСК Директ'),
			);
			
			if (isset($paymentStatusMessages[$paymentStatus]))
				return $paymentStatusMessages[$paymentStatus];
			
			return sprintf(_t('Unknown payment status %s.'), $paymentStatus);
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Mode', 'CurrencyCode', 'MD5Password'));
			
			if (!isset($params['return_header']) or !isset($params['return_mac']))
				throw new scPaymentProcessException(_t('MD5 signature not enabled in your DSK account. Please contact DSK.'));
			
			$this->checkReturnMac($params, $options['MD5Password']);
			
			// Check if cancelled by user
			if ($params['status'] == 8) {
				$result = new scPaymentProcessResult();
				$url = scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE);
				$result->setThirdPartyRedirect($url, array(), 'get');
				return $result;
			}
			
			if (isset($params['PayToMerchant']) and ($params['PayToMerchant'] != $options['MerchantID']))
				throw new scPaymentProcessException(_t('Merchant does not match.'));
			
			$orderId = $params['track_id'];
			$total = isset($params['AmountToPay']) ? $params['AmountToPay'] : 0;
			$currency = isset($params['AmountCurrency']) ? $params['AmountCurrency'] : false;
			
			$result = new scPaymentProcessResult($orderId, $total, $currency, $params['transaction_id']);
			if (($params['status'] == 3) or ($params['status'] == 11)) {
				// Check that the amount matches
//				scPaymentProcess::checkPaymentToOrder($orderId, $total, $currency);
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			}
			$result->setPaymentNotes($this->getPaymentStatusMessage($params['status']));
			$result->setAllTransactionDetails($params);
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'MerchantID',
					'properties' => array('maxLength' => 32),
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
					'properties' => array('minLength' => 1, 'maxLength' => 255),
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MD5Password',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('CurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
			$options->getOptionField('Mode')->setRadioButtons(array('Live' => _t('Live'), 'Simulator' => _t('Simulator')), 'Live');
			$options->setTemplateFile('modules/payment/dskdirect/options.tmpl');
			return $options;
		}
	}
?>