<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Easypay.bg payment processing module
	 */
	 
	 class scPaymentModuleEasypayBg implements IPaymentModule, IModuleOptions {
		
	 	const EASYPAY_LIVE_URL = 'https://www.epay.bg/ezp/reg_bill.cgi';
		const EASYPAY_DEVELOPER_URL = 'https://devep2.datamax.bg/ep2/epay2_demo/ezp/reg_bill.cgi';
		
		const MODE_LIVE = 'Live';
		const MODE_DEVELOPER = 'Developer';
		
		const EASYPAY_RETURN_CODE_OK = 'IDN';
		const EASYPAY_RETURN_CODE_ERROR = 'ERR';
		
		
		protected $_moduleId;
	 	
		protected function getPageParam(){}
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if (($contentType == self::CONTENT_TYPE_CHECKOUT_PENDING) || ($contentType == self::CONTENT_TYPE_VIEW_ORDER)) {
				
				if (!isset($params['OrderID'])) {
					throw new scPaymentProcessException(_t('Required OrderID parameter is missing.'));
				}
				
				$t = new scTemplateParser();
				$t->readTemplate('modules/payment/epay_bg/easypay_note.tmpl');
				
				$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PaymentInstructions'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
				$t->assign('%%PAYMENTINSTRUCTIONS%%', $options['PaymentInstructions']);
				$t->assign('%%IDN%%', $this->getIdnFromOrder($params['OrderID']));
				return $t->templateParse();
			}
		}
		
	 	public function hasCheckoutStep()
		{
			return false;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'BGN';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MIN', 'SecretKey', 'ExpirationTimeHours', 'Mode'));
			
			$loginId = $options['MIN'];
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$expDate = date('d.m.Y H:i', time() + $options['ExpirationTimeHours'] * 60 * 60);
			$secret = $options['SecretKey'];
			
			$data = array(
				'MIN' => $loginId,
				'INVOICE' => $orderId,
				'AMOUNT' => $payAmount,
				'EXP_TIME' => $expDate,
			);
			
			$content = '';
			foreach ($data as $key => $val) {
				$content .= $key . '=' . $val . "\n";
			}
			$encoded = base64_encode($content);
			$checksum = scCryptUtil::hmac($secret, $encoded, scCryptUtil::HASH_SHA1);
			$params = array(
				'ENCODED' => $encoded,
				'CHECKSUM' => $checksum,
			);
			
			if ($options['Mode'] == self::MODE_DEVELOPER)
				$url = self::EASYPAY_DEVELOPER_URL;
			else
				$url = self::EASYPAY_LIVE_URL;
			
			$idn = $this->getIdn($url, $params, $options['Mode']);
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$result = new scPaymentProcessResult($orderId);
			$result->setTransactionDetail('IDN', $idn);
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
			$result->setTransactionAmount($payAmount);
			return $result; 
		}
		
		/**
		 * Handles payment notification
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
			$result = new scPaymentProcessResult($params['OrderId']);
        	switch ($params['Status']) {
				case 'PAID':
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
					$result->setTransactionCode($params['STAN']);
					
					if ($params['BCODE'] != '000000')
						$result->setTransactionNote(sprintf(_t('Pay time: %s, Borica code %s.'), $params['PayTime'], $params['BCODE']));
					else 
						$result->setTransactionNote(sprintf(_t('Pay time: %s.'), $params['PayTime']));
					break;
				
				case 'EXPIRED':
					$result->setPaymentNotes(_t('Payment expired.'));
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote(_t('Notification.'));
					break;
				
				case 'DENIED':
					$result->setPaymentNotes(_t('Payment cancelled by customer.'));
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setTransactionNote(_t('Notification.'));
					break;
			}
			$result->setAllTransactionDetails($params);
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MIN',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'SecretKey',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ExpirationTimeHours',
					'validator' => 'scValidatorInt'
				),
				array(
					'field' => 'scWebFormWidgetTextAreaML',
					'data' => 'PaymentInstructions',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_DEVELOPER => _t('Developer'));
			$options->getOptionField('Mode')->setOptions($modes);
			$options->setTemplateFile('modules/payment/epay_bg/easypay_options.tmpl');
			return $options;
		}
		
		/**
		 * Gets Easypay IDN from Epay.
		 * The IDN is a unique number that identifies the payment
		 * 
		 * @param string $url the Easypay URL 
		 * @param array $params any params to add to the URL 
		 * @return string the IDN
		 * @throws scPaymentProcessException
		 */
		protected function getIdn($url, $params, $mode)
		{
			// add the params to the url since we need to make a get request
			if (!empty($params))
				$url .= '?' . http_build_query($params);
			
			// disable SSL sertificate verification in test mode
			$curlParams = array();
			if ($mode == self::MODE_DEVELOPER) {
				$curlParams['sslVerifyPeer'] = false;
			}
			
			// make the request
			try {
				$result = scHttp::get($url, $curlParams);
			} catch (scHttpException $e) {
				throw new scPaymentProcessException(_i('Could not connect to easypay.bg server.'));
			}
			
			//parse the response
			$error = '';
			$returnCode = substr($result, 0, 3);
			if ($returnCode == self::EASYPAY_RETURN_CODE_OK) {
				$idn = str_replace($returnCode . '=', '', $result);
				$idn = trim($idn);
				if (!preg_match('/^[0-9]{10}&/', $idn)) {
					return $idn;
				}
				$error = _i('Invalid IDN returned: ') . $idn;
			}
			if ($returnCode == self::EASYPAY_RETURN_CODE_ERROR) {
				$error = str_replace($returnCode . '=', '', $result);
			}
			throw new scPaymentProcessException(_t('Error occured.'), array('error' => $error, 'result' => $result));
		}
		
		/**
		 * Retreives the IDN from transaction details
		 *
		 * @param int $orderId
		 * @return string IDN value
		 */
		protected function getIdnFromOrder($orderId)
		{
			$sql = 
				'SELECT TransactionDetailValue ' . 
				'FROM TransactionDetails INNER JOIN Transactions USING(TransactionID) ' .
				'WHERE Transactions.OrderID = ? AND TransactionDetails.TransactionDetailName = ?';
			$res = miStaticDBUtil::execSelect($sql, array($orderId, 'IDN'));
			if (empty($res)) {
				throw new scPaymentProcessException(_t('Could not find IDN.'));
			}
			return $res[0]['TransactionDetailValue'];
		}
	}
?>