<?php
	/**
	 * Copyright (c) 2008, 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * This file is shared by Epay and Easypay payment modules.
	 */

	require_once('../../../include/miphpf/Init.php');
	require_once('../../../include/sc/Init.php');

	// We need to turn error reporting off since we must provide a well formed string to Epay
	error_reporting(0);
	
	class scEpayEasypayNotificationProcessor {
		
		/**
		 * Used only for logging purposes. Will change to epay or
		 * easypay if one of them is determined later.
		 */ 
		public $_activeMethod = 'ePay.bg/Easypay.bg';
		
		/**
		 * Process the notification from Epay/Easypay.
		 * Auto detects if Epay or Easypay payment module should be used for the processing.
		 */
		public function doProcess()
		{
			$encoded  = $_POST['encoded'];
			$checksum = $_POST['checksum'];
			
			// decode data
			$data = base64_decode($encoded);
			$lines = explode("\n", $data);
			
			$paymentModuleId = $this->getPaymentModuleId($lines);
			$paymentModuleName = $this->getPaymentModuleName($paymentModuleId);
			
			// get SecretKey to validate notification
			$moduleId = scModulesFactory::getModuleIdByName($paymentModuleName);
			$options = scModuleOptions::getModuleOptions($moduleId, array('MIN', 'SecretKey', 'ExpirationTimeHours', 'Mode'));
			
			// Validate notification
			$hmac = scCryptUtil::hmac($options['SecretKey'], $encoded, scCryptUtil::HASH_SHA1);
			if ($hmac != $checksum) {
				scLog::singleton()->error(_i($this->_activeMethod . ': Checksum does not match.'), scLog::CATEGORY_PAYMENT);
				die('ERR=Checksum does not match - ' . $this->_activeMethod);
			}
			
			// Possible notifcations:
			// INVOICE=123456:STATUS=PAID:PAY_TIME=YYYYMMDDhhmmss:STAN=[6 числа]:BCODE=[6 числа/букви]
			// INVOICE=123457:STATUS=DENIED
			// INVOICE=123457:STATUS=EXPIRED
			
			foreach ($lines as $line) {
				if (preg_match("/^INVOICE=(\d+):STATUS=(PAID|DENIED|EXPIRED)(:PAY_TIME=(\d+):STAN=(\d+):BCODE=([0-9a-zA-Z]+))?$/", $line, $regs)) {
					$orderId = $regs[1];
					$_POST['OrderId'] = $orderId;
					$_POST['Status'] = $regs[2];
					$_POST['PayTime'] = isset($regs[4]) ? $regs[4] : '';
					$_POST['STAN'] = isset($regs[5]) ? $regs[5] : '';	// Transaction number
					$_POST['BCODE'] = isset($regs[6]) ? $regs[6] : '';	// Borica Code
					
					$result = scPaymentProcess::callPaymentModule($paymentModuleName, 'processNotify', scPaymentProcess::CALL_TYPE_NOTIFY);
					echo sprintf("INVOICE=%d:STATUS=%s\n", $orderId, ($result) ? 'OK' : 'ERR');
				}
			}
		}
		
		/**
		 * Get one unique PaymentModuleID from the orders that are in the notification data.
		 * If there are more that one an exception is thrown.
		 * @param array $lines
		 */
		protected function getPaymentModuleId($lines)
		{
			// get all order ids from the notification
			$orderIds = array();
			foreach ($lines as $line) {
				if (preg_match("/^INVOICE=(\d+):STATUS=(PAID|DENIED|EXPIRED)(:PAY_TIME=(\d+):STAN=(\d+):BCODE=([0-9a-zA-Z]+))?$/", $line, $regs)) {
					$orderIds[] = (int)$regs[1];
				}
			}
			if (empty($orderIds)) {
				throw new miException(_i($this->_activeMethod . ': No order ids in the notification string.'));
			}
			
			// get all payment module ids based on order id
			$sql = 'SELECT PaymentModuleID FROM Orders WHERE OrderID IN (' . implode(',', $orderIds) . ')';
			$res = miStaticDBUtil::execSelect($sql);
			if (empty($res)) {
				throw new miException(_i($this->_activeMethod . ': No PaymentModuleID for orders.'));
			}
			
			// It doesn't make sense to have more than 1 unique PaymentModuleID in a single notification
			// because it is either from epay or from easypay
			$res = miUIUtil::transposeRows($res);
			$res = array_unique($res['PaymentModuleID']);
			if (count($res) > 1) {
				throw new miException(_i($this->_activeMethod . ': Not unique PaymentModuleID.'));
				die('ERR=Internal error: Not unique PaymentModuleID');
			}
			
			return $res[0];
		}
		
		/**
		 * This function tries to get the correct payment module name from the PaymentModuleID 
		 * that we got from getPaymentModuleId() method.
		 * This should be either epay or easypay.
		 *
		 * @param int $paymentModuleId
		 */
		protected function getPaymentModuleName($paymentModuleId)
		{
			// Get payment module ids of epay and easypay modules
			$sql = 'SELECT PaymentModuleID FROM PaymentModules INNER JOIN Modules USING(ModuleID) WHERE ModuleName = ?';
			
			// Get epay moduleId
			$epayModuleName = 'com.summercart.payment.epaybg.paylogin';
			$res = miStaticDBUtil::execSelect($sql, array($epayModuleName));
			$epayPaymentModuleId = $res[0]['PaymentModuleID'];
			
			// Get easypay moduleId
			$easypayModuleName = 'com.summercart.payment.easypaybg';
			$res = miStaticDBUtil::execSelect($sql, array($easypayModuleName));
			$easypayPaymentModuleId = $res[0]['PaymentModuleID'];
			
			// The PaymentModuleID should be one of Epay or Easypay 
			if ($paymentModuleId == $easypayPaymentModuleId) {
				$paymentModuleName = $easypayModuleName;
				$this->_activeMethod = 'Easypay.bg';
			} elseif ($paymentModuleId == $epayPaymentModuleId) {
				$paymentModuleName = $epayModuleName;
				$this->_activeMethod = 'Epay.bg';
			} else {
				throw new miException(_i($this->_activeMethod . ': PaymentModuleID not Epay nor Easypay.'));
			}
			return $paymentModuleName;
		}
		
	}
	
	// try to process the data
	try {
		$processor = new scEpayEasypayNotificationProcessor();
		$processor->doProcess();
	} catch (miException $e) {
		scLog::singleton()->error(sprintf(_i($processor->_activeMethod . ': Notify exception: %s.'), $e->getMessage()), scLog::CATEGORY_PAYMENT);
		die('ERR=Internal error: ' . $e->getMessage());
	}
?>