<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/HsbcHashCalculator.php');
	require_once(dirname(__FILE__) . '/HsbcCountryCodes.php');
	
	/**
	 * HSBC CPI payment processing module
	 */
	class scPaymentModuleHsbcCpi implements IPaymentModule, IModuleOptions {
		const SANDBOX_URL = 'https://www.uat.cpi.netq.hsbc.com/servlet';
		const LIVE_URL = 'https://www.cpi.hsbc.com/servlet';
		
		protected $_moduleId;
		
		protected $_currencyCodeMap = array(
			'AUD' => '036',
			'CAD' => '124',
			'DKK' => '208',	// The reference says 206, which is incorrect
			'EUR' => '978',
			'HKD' => '344',
			'JPY' => '392',
			'NZD' => '554',
			'NOK' => '578',
			'SGD' => '702',
			'SEK' => '752',
			'CHF' => '756',
			'GBP' => '826',
			'USD' => '840',
		);
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('AccountCurrencyCode'));
			return $options['AccountCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('StorefrontId', 'OrderDesc', 'Mode', 'TransactionType', 'AccountCurrencyCode', 'SharedSecret'));
			
			if (!isset($this->_currencyCodeMap[$payCurrencyCode]))
				throw new miException(sprintf(_i('Unsupported currency code "%s" in HSBC CPI.'), $payCurrencyCode));
			
			// Compute the amount by removing the decimals
			if ($payCurrencyCode == 'JPY')
				$amount = round($payAmount);
			else
				$amount = round($payAmount * 100);
			
			if (!scPaths::isSslEnabled())
				throw new miException(_i('HSBC payment module requires SSL to be enabled.'));
			
			$merchantData = implode(':', array($options['Mode'], $options['TransactionType']));
			
			// Note: the encoding is ISO 8859-1
			$params = array(
				'StorefrontId' => $options['StorefrontId'],
				'TimeStamp' => time() . '000',	// Make it in milliseconds
				'TransactionType' => $options['TransactionType'],
				
				'PurchaseAmount' => (string)$amount,
				'PurchaseCurrency'	=> $this->_currencyCodeMap[$payCurrencyCode],
				'OrderId' => (string)scOrdersInProgress::singleton()->getLastOrderId(),
				
				'OrderDesc' => utf8_decode($options['OrderDesc']),
				'MerchantData' => $merchantData,
				'Mode' => $options['Mode'],
				
				'CpiDirectResultUrl' => scPaths::getSecureUrl('modules/payment/hsbc/direct_return.php'),	// Transaction is posted here
				'CpiReturnUrl'  => scPaths::getSecureUrl('modules/payment/hsbc/return.php'),	// Customer is redirected here
			);
			
			// HSBC accepts all or nothing. Thus if it is guest checkout don't pass customer info
			if ($customerInfo['CustomerEmail'] != '') {
				$params = array_merge($params,
					array(
						'BillingFirstName' => utf8_decode($customerInfo['CustomerBillingFirstName']),
						'BillingLastName' => utf8_decode($customerInfo['CustomerBillingLastName']),
						'BillingAddress1' => utf8_decode($customerInfo['CustomerBillingAddress1']),
						'BillingAddress2' => utf8_decode($customerInfo['CustomerBillingAddress2']),
						'BillingCity' => utf8_decode($customerInfo['CustomerBillingCity']),
						'BillingCounty' => utf8_decode(scCountry::getStateCode($customerInfo['CustomerBillingState'])),
						'BillingPostal' => utf8_decode($customerInfo['CustomerBillingZip']),
						'BillingCountry' => scHsbcCountryCodes::getCountryCode(scCountry::getCountryCode2($customerInfo['CustomerBillingCountry'])),
						'ShopperEmail' => $customerInfo['CustomerEmail'],
						
						'ShippingFirstName' => utf8_decode($customerInfo['CustomerShippingFirstName']),
						'ShippingLastName' => utf8_decode($customerInfo['CustomerShippingLastName']),
						'ShippingAddress1' => utf8_decode($customerInfo['CustomerShippingAddress1']),
						'ShippingAddress2' => utf8_decode($customerInfo['CustomerShippingAddress2']),
						'ShippingCity' => utf8_decode($customerInfo['CustomerShippingCity']),
						'ShippingCounty' => utf8_decode(scCountry::getStateCode($customerInfo['CustomerShippingState'])),
						'ShippingPostal' => utf8_decode($customerInfo['CustomerShippingZip']),
						'ShippingCountry' => scHsbcCountryCodes::getCountryCode(scCountry::getCountryCode2($customerInfo['CustomerShippingCountry'])),
					)
				);
			}
			
			$this->limitFieldSizes($params);
//			var_dump($params);die;
			
//			$orderHash = $this->computeHash($params, 'zEeWQNKelqPE2DRFueuDq1QrASjux2lM');
			
			$hsbcHashCalculator = new scHsbcHashCalculator();
			$hash = $hsbcHashCalculator->generateHash(array_values($params), $options['SharedSecret']);
			
			$params['OrderHash'] = $hash;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::LIVE_URL, $params);
			return $result;
		}
		
		protected function limitFieldSizes(&$params)
		{
			$fieldMaxSizes = array(
				'OrderDesc' => 54,
				
				'BillingAddress1' => 60,
				'BillingAddress2' => 60,
				'BillingCity' => 25,
				'BillingCounty' => 25,
				'BillingFirstName' => 32,
				'BillingLastName' => 32,
				'BillingPostal' => 20,
				'ShopperEmail' => 64,
				'ShippingAddress1' => 60,
				'ShippingAddress2' => 60,
				'ShippingCity' => 25,
				'ShippingCounty' => 25,
				'ShippingFirstName' => 32,
				'ShippingLastName' => 32,
				'ShippingPostal' => 20,
			);
			
			foreach ($params as $key => $param) {
				if (!isset($fieldMaxSizes[$key]))
					continue;
				
				if (strlen($param) > $fieldMaxSizes[$key])
					$params[$key] = substr($param, 0, $fieldMaxSizes[$key]);
			}
		}
		
		
		/**
		 * Handle return and generate payment result
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		protected function handleReturnInfo($params)
		{
			if (!isset($params['OrderHash']) and isset($params['CpiResultsCode']))
				throw new scPaymentProcessException(sprintf(_i('Error: CpiResultsCode: %s'), $params['CpiResultsCode']), $params);
			
			scPaymentProcess::checkParamsExist($params, array('StorefrontId', 'OrderId', 'PurchaseAmount', 'PurchaseCurrency', 'PurchaseDate', 'MerchantData', 'CpiResultsCode', 'OrderHash'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('StorefrontId', 'OrderDesc', 'Mode', 'TransactionType', 'AccountCurrencyCode', 'SharedSecret'));
			
			// Check the hash
			$hsbcHashCalculator = new scHsbcHashCalculator();
			$params2 = $params;
			unset($params2['OrderHash']);
			$hash = $hsbcHashCalculator->generateHash(array_values($params2), $options['SharedSecret']);
			
			if ($params['OrderHash'] != $hash)
				throw new scPaymentProcessException(_i('Invalid order hash. Please check you Shared Secret.'), array(_t('OrderID') => $params['OrderId']));
			
			if (!in_array($params['PurchaseCurrency'], $this->_currencyCodeMap))
				throw new scPaymentProcessException(_t('Unknown currency code.'), array(_t('Currency') => $params['PurchaseCurrency']));
			
			$currencyCode = array_search($params['PurchaseCurrency'], $this->_currencyCodeMap);
			$amount = $params['PurchaseAmount'] / 100;
			
			// Check that the amount matches
			scPaymentProcess::checkPaymentToOrder($params['OrderId'], $amount, $currencyCode);
			
			list($mode, $transactionType) = explode(':', $params['MerchantData']);
			$cpiResultsCode = $params['CpiResultsCode'];
			
			$result = new scPaymentProcessResult($params['OrderId'], $amount, $currencyCode);
			if ($cpiResultsCode == 0) {
				if ($transactionType == 'Capture')
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
				elseif ($transactionType == 'Auth')
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
				else
					throw new scPaymentProcessException(_i('Unknown transaction type.'), array(_i('Transaction Type') => $transactionType));
				
				if ($mode == 'T')
					$result->setPaymentNotes(_t('Order in demo mode.'));
			} else {
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
				$result->setPaymentNotes($this->getCpiResultErrorMessage($cpiResultsCode));
			}
			
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		function processReturn($params)
		{
			return $this->handleReturnInfo($params);
		}
		
		/**
		 * Handles direct return - notify
		 * 
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		function processDirectReturn($params)
		{
			echo '<html></html>';	// HSBC expects "data"
			return $this->handleReturnInfo($params);
		}
		
		protected function getCpiResultErrorMessage($cpiResultsCode)
		{
			$messages = array(
				0 => _i('The transaction was approved.'),
				1 => _i('The user cancelled the transaction.'),
				2 => _i('The processor declined the transaction for an unknown reason.'),
				3 => _i('The transaction was declined because of a problem with the card. For example, an invalid card number or expiration date was specified.'),
				4 => _i('The processor did not return a response.'),
				5 => _i('The amount specified in the transaction was either too high or too low for the processor.'),
				6 => _i('The specified currency is not supported by either the processor or the card.'),
				7 => _i('The order is invalid because the order ID is a duplicate.'),
				8 => _i('The transaction was rejected by FraudShield.'),
				9 => _i('The transaction was placed in Review state by FraudShield (see note 1 below).'),
				10 => _i('The transaction failed because of invalid input data.'),
				11 => _i('The transaction failed because the CPI was configured incorrectly.'),
				12 => _i('The transaction failed because the Storefront was configured incorrectly.'),
				13 => _i('The connection timed out.'),
				14 => _i('The transaction failed because the cardholder’s browser refused a cookie.'),
				15 => _i('The customer’s browser does not support 128-bit encryption.'),
				16 => _i('The CPI cannot communicate with the Payment Engine.'),
			);
			
			if (!isset($messages[$cpiResultsCode]))
				throw new scPaymentProcessException(_i('Unknown CpiResultsCode.'), array(_i('CpiResultsCode') => $cpiResultsCode));
			
			return $messages[$cpiResultsCode];
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'StorefrontId',
					'properties' => array('minLength' => 13, 'maxLength' => 13)
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'AccountCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDesc',
					'properties' => array('minLength' => 1, 'maxLength' => 54)
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'SharedSecret',
					'properties' => array('minLength' => 1)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('Mode')->setRadioButtons(array('P' => _t('Production'), 'T' => _t('Test')), 'T');
			$options->getOptionField('AccountCurrencyCode')->setOptions(array_combine(array_keys($this->_currencyCodeMap), array_keys($this->_currencyCodeMap)));
			$options->getOptionField('TransactionType')->setRadioButtons(array('Auth' => _i('Auth'), 'Capture' => _t('Capture')), 'Capture');
			$options->setTemplateFile('modules/payment/hsbc/options.tmpl');
			return $options;
		}
	}
?>