<?php
	/**
	 * Copyright (c) 2011 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * mobilpay.ro romanian payment processing module
	 */
	class scPaymentModuleMobilpayRo implements IPaymentModule, IModuleOptions {
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		const SANDBOX_URL = 'http://sandboxsecure.mobilpay.ro';
		const PRODUCTION_URL = 'https://secure.mobilpay.ro';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'RON';
		}
		
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Signature', 'PublicCertificate', 'PrivateKey', 'OrderDescription', 'Mode'));
			$api = new scPaymentModuleMobilpayApi($options['PublicCertificate'], $options['PrivateKey']);
			
			$requestXmlDefinitions = $this->getRequestXmlDefinitions($options['Signature'], $payAmount, $payCurrencyCode, $options['OrderDescription'], $cart);
			$params = $api->encryptData($requestXmlDefinitions);
			
			$url = ($options['Mode'] == self::MODE_LIVE) ? self::PRODUCTION_URL : self::SANDBOX_URL;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * Handles payment return. 
		 * The customer is being redirected to the payment result page because the transaction status was already set by processNotify
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('orderId'));
			scPaymentProcess::redirectToPaymentResult((int)$params['orderId']);
			exit;
		}
		
		/**
		 * Handles payment notification
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PublicCertificate', 'PrivateKey'));
			$api = new scPaymentModuleMobilpayApi($options['PublicCertificate'], $options['PrivateKey']);
			
			if (empty($params['env_key']) || empty($params['data'])) {
				$message = _i('Invalid parameters received from the payment processor.');
				$errorCode = scPaymentModuleMobilpayApi::CONFIRM_ERROR_CODE_INVALID_PARAMETERS;
				$errorType = scPaymentModuleMobilpayApi::CONFIRM_ERROR_TYPE_PERMANENT;
				$api->showConfirmResponse($message, $errorCode, $errorType);
				return;
			}
			
			$result = new scPaymentProcessResult();
			try {
				$xmlResult = $api->decryptData($params['env_key'], $params['data']);
				
				$xml = new DOMDocument();
				$xml->loadXML($xmlResult);
				
				$order = $xml->getElementsByTagName('order')->item(0);
				$mobilpayResult = $order->getElementsByTagName('mobilpay')->item(0);
				$mobilpayXmlResult = $xml->saveXML($mobilpayResult);
				
				$orderId = $order->getAttribute('id');
				$processedAmount = $mobilpayResult->getElementsByTagName('processed_amount')->item(0)->nodeValue;
				$payCurrencyCode = $order->getElementsByTagName('invoice')->item(0)->getAttribute('currency');
				
				$transactionCode = $mobilpayResult->getElementsByTagName('purchase')->item(0)->nodeValue;
				$transactionAction = $mobilpayResult->getElementsByTagName('action')->item(0)->nodeValue;
				
				$error = $mobilpayResult->getElementsByTagName('error')->item(0);
				$errorCode = $error->getAttribute('code');
				$errorMessage = $error->nodeValue;
				
				$result->setOrderId($orderId);
				$result->setTransactionAmount($processedAmount);
				$result->setTransactionCurrencyCode($payCurrencyCode);
				$result->setTransactionCode($transactionCode);
				$this->setTransactionDetailsFromXml($result, new SimpleXMLElement($mobilpayXmlResult));
				
				// Validate order amount
				$originalAmount = $mobilpayResult->getElementsByTagName('original_amount')->item(0)->nodeValue;
				if ($originalAmount != $processedAmount) {
					throw new miException(_t('Received original_amount cannot be different from the processed_amount.'));
				}
				scPaymentProcess::checkPaymentToOrder($orderId, $processedAmount);
				
				if ($errorCode != 0) {
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
					$result->setPaymentNotes($errorMessage);
					$result->setTransactionDetail('errorCode', $errorCode);
					$result->setTransactionNote($errorMessage);
					
				} else {
					switch ($transactionAction) {
						// Transaction is confirmed.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_CONFIRM:
							$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
							break;
						
						// Transaction is cancelled.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_CANCELED:
							$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
							break;
						
						// Transaction is being processed for fraud. Capture is still pending.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_CONFIRMED_PENDING:
						// Transaction is being processed. Capture is still pending.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_PAID:
						// The transaction is under review. Capture is still pending.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_PAID_PENDING:
							// When the processing is done, a new confirm request will be send to cancel/confirm the transaction.
							$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_PENDING);
							break;
						
						// The transaction amount was refunded to the customer.
						case scPaymentModuleMobilpayApi::TRANSACTION_ACTION_CREDIT:
							$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_REFUND);
							break;
						
						default:
							$message = _i('Invalid action parameter received from the payment processor.');
							$errorCode = scPaymentModuleMobilpayApi::CONFIRM_ERROR_CODE_UNKNOWN_ACTION;
							$errorType = scPaymentModuleMobilpayApi::CONFIRM_ERROR_TYPE_PERMANENT;
							$api->showConfirmResponse($message, $errorCode, $errorType);
							return $result;
					}
				}
				
				$message = $mobilpayResult->getAttribute('crc');
				$api->showConfirmResponse($message);
				
			} catch (miException $exception) {
				$errorCode = scPaymentModuleMobilpayApi::CONFIRM_ERROR_CODE_INTERNAL_EXCEPTION;
				$errorType = scPaymentModuleMobilpayApi::CONFIRM_ERROR_TYPE_TEMPORARY;
				$api->showConfirmResponse($exception->getMessage(), $errorCode, $errorType);
			}
			
			return $result;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'Signature',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'PrivateKey',
					'validator' => 'scMobilPayValidatorRsaPrivateKey'
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'PublicCertificate',
					'validator' => 'scMobilPayValidatorSslCertificate'
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test')), self::MODE_TEST);
			$options->setTemplateFile('modules/payment/mobilpay_ro/mobilpay_options.tmpl');
			return $options;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		protected function getRequestXmlDefinitions($signature, $payAmount, $payCurrencyCode, $orderDescription, scCart $cart)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$currency = new scCurrency($payCurrencyCode);
			$customerInfo = $cart->getCustomerInfo();
			
			$billingAddressType = 'person';
			$billingFirstName = $customerInfo['CustomerBillingFirstName'];
			$billingLastName = $customerInfo['CustomerBillingLastName'];
			if (!empty($customerInfo['CustomerBillingCompany'])) {
				$billingAddressType = 'company';
				$billingFirstName = $customerInfo['CustomerBillingCompany'];
				$billingLastName = '';
			}
			
			$shippingAddressType = 'person';
			$shippingFirstName = $customerInfo['CustomerShippingFirstName'];
			$shippingLastName = $customerInfo['CustomerShippingLastName'];
			if (!empty($customerInfo['CustomerShippingCompany'])) {
				$shippingAddressType = 'company';
				$shippingFirstName = $customerInfo['CustomerShippingCompany'];
				$shippingLastName = '';
			}
			
			#XXX: not yet implemented by the payment processor
//			$cartItems = $cart->getItems();
//			$itemsXmlDefinitions = array();
//			foreach ($cartItems as $cartItem) {
//				$itemsXmlDefinitions[] = array(
//					scXml::TAG => 'item',
//					scXml::CHILDREN => array(
//						array(scXml::TAG => 'code', scXml::DATA => $cartItem->getProductCode()),
//						array(scXml::TAG => 'name', scXml::DATA => $cartItem->getName()),
//						array(scXml::TAG => 'measurment', scXml::DATA => ''),
//						array(scXml::TAG => 'quantity', scXml::DATA => $cartItem->getQuantity()),
//						array(scXml::TAG => 'price', scXml::DATA => $cartItem->getPriceObj()->getAmountInCurrency($currency)),
//						array(scXml::TAG => 'vat', scXml::DATA => $this->getCartItemTaxRate($cartItem, $customerInfo)),
//					),
//				);
//			}
			
			$xmlDefinitions = array(
				array(
					scXml::TAG => 'order',
					scXml::ATTRIBUTES => array('type' => 'card', 'id' => $orderId, 'timestamp' => date('YmdHis')),
					scXml::CHILDREN => array(
						array(scXml::TAG => 'signature', scXml::DATA => $signature),
						array(scXml::TAG => 'url',
							scXml::CHILDREN => array(
								array(scXml::TAG => 'confirm', scXml::DATA => scXml::escape(scPaths::getSecureUrl('modules/payment/mobilpay_ro/notify.php'))),
								array(scXml::TAG => 'return', scXml::DATA => scXml::escape(scPaths::getSecureUrl('modules/payment/mobilpay_ro/return.php'))),
							),
						),
						array(scXml::TAG => 'invoice',
							scXml::ATTRIBUTES => array('currency' => $payCurrencyCode, 'amount' => $payAmount),
							scXml::CHILDREN => array(
								array(scXml::TAG => 'details', scXml::DATA => $orderDescription),
								array(scXml::TAG => 'contact_info',
									scXml::CHILDREN => array(
										array(scXml::TAG => 'billing',
											scXml::ATTRIBUTES => array('type' => $billingAddressType),
											scXml::CHILDREN => array(
												array(scXml::TAG => 'first_name', scXml::DATA => $billingFirstName),
												array(scXml::TAG => 'last_name', scXml::DATA => $billingLastName),
												array(scXml::TAG => 'fiscal_number', scXml::DATA => $customerInfo['CustomerBillingVatNumber']),
												array(scXml::TAG => 'country', scXml::DATA => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry'])),
												array(scXml::TAG => 'county', scXml::DATA => scCountry::getStateName($customerInfo['CustomerBillingState'])),
												array(scXml::TAG => 'city', scXml::DATA => $customerInfo['CustomerBillingCity']),
												array(scXml::TAG => 'zip_code', scXml::DATA => $customerInfo['CustomerBillingZip']),
												array(scXml::TAG => 'address', scXml::DATA => $customerInfo['CustomerBillingAddress1'] . ' ' . $customerInfo['CustomerBillingAddress2']),
												array(scXml::TAG => 'email', scXml::DATA => $customerInfo['CustomerEmail']),
												array(scXml::TAG => 'mobile_phone', scXml::DATA => $customerInfo['CustomerBillingPhone']),
											),
										),
										array(scXml::TAG => 'shipping',
											scXml::ATTRIBUTES => array('type' => $shippingAddressType, 'sameasbilling' => intval($customerInfo['CustomerShipToBillingAddress'])),
											scXml::CHILDREN => array(
												array(scXml::TAG => 'first_name', scXml::DATA => $shippingFirstName),
												array(scXml::TAG => 'last_name', scXml::DATA => $shippingLastName),
												array(scXml::TAG => 'fiscal_number', scXml::DATA => $customerInfo['CustomerShippingVatNumber']),
												array(scXml::TAG => 'country', scXml::DATA => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry'])),
												array(scXml::TAG => 'county', scXml::DATA => scCountry::getStateName($customerInfo['CustomerShippingState'])),
												array(scXml::TAG => 'city', scXml::DATA => $customerInfo['CustomerShippingCity']),
												array(scXml::TAG => 'zip_code', scXml::DATA => $customerInfo['CustomerShippingZip']),
												array(scXml::TAG => 'address', scXml::DATA => $customerInfo['CustomerShippingAddress1'] . ' ' . $customerInfo['CustomerShippingAddress2']),
												array(scXml::TAG => 'email', scXml::DATA => $customerInfo['CustomerEmail']),
												array(scXml::TAG => 'mobile_phone', scXml::DATA => $customerInfo['CustomerShippingPhone']),
											),
										),
									),
								),
								#XXX: not yet implemented by the payment processor
//								array(scXml::TAG => 'items',
//									scXml::CHILDREN => $itemsXmlDefinitions
//								),
							),
						),
					),
				),
			);
			return $xmlDefinitions;
		}
		
		#XXX: not yet implemented by the payment processor
//		protected function getCartItemTaxRate(scCartItem $cartItem, $customerInfo)
//		{
//			$taxRate = 0;
//			$taxClasses = scTax::getProductTaxClasses($cartItem->getProductId());
//			foreach ($taxClasses as $taxClassId) {
//				$taxInfo = scTax::getTaxInfoByTaxClassId($taxClassId);
//				
//				// If store owner is VAT registered, and the customer has provided VAT number, and the customer is in a different country
//				// then this is Intra-Community delivery - zero the rate. The VAT will be paid by the receiving party to it's government
//				if (!empty($taxInfo['TaxRegNumber'])) {
//					$taxNumberCountry = scVatNumbersUtil::getCountryFromNumber($taxInfo['TaxRegNumber']);
//					if (!empty($customerVatCountry) && !empty($taxNumberCountry)) {
//						if ($customerVatCountry != $taxNumberCountry) {
//							continue;
//						}
//					}
//				}
//				
//				$taxZoneId = scZones::getTaxZoneId($taxInfo['TaxID'], $customerInfo);
//				$taxRate += scTax::getTaxRate($taxZoneId, $taxClassId);
//			}
//			return round($taxRate / 100, 3);
//		}
		
		/**
		 * Iterates the xml and sets all values as transaction details
		 * @param scPaymentProcessResult $result
		 * @param string $xml
		 */
		protected function setTransactionDetailsFromXml(scPaymentProcessResult $result, $xml)
		{
			foreach ($xml as $key => $xmlElement) {
				if ($xmlElement->children()) {
					$this->setTransactionDetailsFromXml($result, $xmlElement);
					continue;
				}
				
				$result->setTransactionDetail($key, (string)$xmlElement);
			}
		}
	}
	
	class scPaymentModuleMobilpayApi {
		
		const TRANSACTION_ACTION_CONFIRM = 'confirm';
		const TRANSACTION_ACTION_CONFIRMED_PENDING = 'confirmed_pending';
		const TRANSACTION_ACTION_PAID = 'paid';
		const TRANSACTION_ACTION_PAID_PENDING = 'paid_pending';
		const TRANSACTION_ACTION_CANCELED = 'canceled';
		const TRANSACTION_ACTION_CREDIT = 'credit';
		
		/* Defined by the payment processor */
		const CONFIRM_ERROR_TYPE_NONE = 0;
		const CONFIRM_ERROR_TYPE_TEMPORARY = 1;
		const CONFIRM_ERROR_TYPE_PERMANENT = 2;
		
		/* Defined by us for debugging purposes */
		const CONFIRM_ERROR_CODE_NONE = 0;
		const CONFIRM_ERROR_CODE_UNKNOWN_ACTION = 1;
		const CONFIRM_ERROR_CODE_INVALID_PARAMETERS = 2;
		const CONFIRM_ERROR_CODE_INTERNAL_EXCEPTION = 3;
		
		protected $_publicCertificate;
		protected $_privateKey;
		
		/**
		 * @param string $publicCertificate
		 * @param string $privateKey
		 */
		public function __construct($publicCertificate, $privateKey)
		{
			$this->_publicCertificate = $publicCertificate;
			$this->_privateKey = $privateKey;
		}
		
		/**
		 * Generate encrypted data and envelope key.
		 * Use the PublicCertificate for the encryption process
		 *
		 * @param array $requestXmlDefinitions
		 * @return array
		 */
		public function encryptData($requestXmlDefinitions)
		{
			$publicKey = openssl_pkey_get_public($this->_publicCertificate);
			if ($publicKey === false) {
				$errorMessage = _t('Error while loading X509 public key certificate! Reason: ');
				while ($errorString = openssl_error_string()) {
					$errorMessage .= $errorString . '; ';
				}
				throw new miException($errorMessage);
			}
			
			$srcData = '<?xml version="1.0" encoding="utf-8" ?>' . "\n";
			$srcData .= scXml::xmlWriter($requestXmlDefinitions);
			$encData = null;
			$envKeys = null;
			$result = openssl_seal($srcData, $encData, $envKeys, array($publicKey));
			if ($result === false) {
				$errorMessage = _t('Error while encrypting data! Reason: ');
				while ($errorString = openssl_error_string()) {
					$errorMessage .= $errorString . '; ';
				}
				throw new miException($errorMessage);
			}
			return array('data' => base64_encode($encData), 'env_key' => base64_encode($envKeys[0]));
		}
		
		/**
		 * Descrypt the response from the payment processor.
		 * Use the PrivateKey for the decryption process
		 *
		 * @param string $envKey
		 * @param string $encData
		 * @return string
		 */
		public function decryptData($envKey, $encData)
		{
			$privateKey = openssl_get_privatekey($this->_privateKey);
			if ($privateKey === false) {
				throw new miException(_t('Error while loading the private key.'));
			}
			
			$srcData = base64_decode($encData);
			if ($srcData === false) {
				openssl_free_key($privateKey);
				throw new miException(_t('Failed to decode the data.'));
			}
			
			$srcEnvKey = base64_decode($envKey);
			if ($srcEnvKey === false) {
				throw new miException(_t('Failed the decode the envelope key.'));
			}
			
			$data = null;
			$result = openssl_open($srcData, $data, $srcEnvKey, $privateKey);
			if ($result === false) {
				throw new miException(_t('Failed to decrypt the data.'));
			}
			
			return $data;
		}
		
		/**
		 * This method informs the payment processor whether the notification process was successful.
		 * If no error occured while saving the transaction details then we should display only the crc code.
		 * If an error occured while saving the transaction details then we should supply it's type, code and message for debugging reasons.
		 *
		 * @param string $message
		 * @param int $errorCode
		 * @param int $errorType
		 */
		public function showConfirmResponse($message, $errorCode = self::CONFIRM_ERROR_CODE_NONE, $errorType = self::CONFIRM_ERROR_TYPE_NONE)
		{
			switch ($errorCode) {
				case self::CONFIRM_ERROR_CODE_NONE:
					$responseDefinitions = array(
						array(scXml::TAG => 'crc', scXml::DATA => $message)
					);
					break;
				
				default:
					$responseDefinitions = array(
						array(scXml::TAG => 'crc',
							scXml::ATTRIBUTES => array('error_type' => $errorType, 'error_code' => $errorCode),
							scXml::DATA => $message
						),
					);
			}
			
			header('Content-type: application/xml');
			echo '<?xml version="1.0" encoding="utf-8"?>' . "\n";
			echo scXml::xmlWriter($responseDefinitions);
		}
	}
	
	class scMobilPayValidatorRsaPrivateKey extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (empty($fieldValue))
				return;
			
			$key = openssl_pkey_get_private($fieldValue);
			if ($key === false)
			$errors->addError($this->_fieldName, _t('Invalid private key.'));
		}
	}
	
	class scMobilPayValidatorSslCertificate extends miValidator {
		public function validate(miWebFormErrorsHandler &$errors)
		{
			$fieldValue = $this->_webForm->getSubmittedData($this->_fieldName);
			if (empty($fieldValue))
				return;
			
			$key = openssl_pkey_get_public($fieldValue);
			if ($key === false)
			$errors->addError($this->_fieldName, _t('Invalid SSL certificate.'));
		}
	}
	?>