<?php
	/**
	 * Implements the PayPal NVP API
	 *
	 */
	
	class scPayPalNvpApi {
		CONST PAYPAL_API_VERSION = '61.0';
		
		const MODE_SANDBOX = 'Sandbox';
		const MODE_PRODUCTION = 'Production';
		
		const SANDBOX_URL = 'https://api-3t.sandbox.paypal.com/nvp';
		const PRODUCTION_URL = 'https://api-3t.paypal.com/nvp';
		
		const CHECKOUT_BUTTON_TYPE_STATIC = 'Static';
		const CHECKOUT_BUTTON_TYPE_DYNAMIC = 'Dynamic';
		
		const MAX_SHIPPING_OPTIONS = 10;
		
		protected $_url;
		protected $_username;
		protected $_password;
		protected $_signature;
		
		/**
		 * Constructs the PayPal API object
		 *
		 * @param string $mode
		 */
		public function __construct($mode, $username = '', $password = '', $signature = '')
		{
			$this->_url = (($mode == self::MODE_PRODUCTION) || $mode == 'Live') ? self::PRODUCTION_URL : self::SANDBOX_URL;
			$this->_username = $username;
			$this->_password = $password;
			$this->_signature = $signature;
		}
		
		/**
		 * Call PayPal SetExpressCheckout
		 * @param scCart $cart
		 * @param string $currencyCode 3-character currency code
		 * @param string $paymentAction Sale, Order or Authorization
		 * @param int $orderId
		 * @param string $custom
		 * @param array $shippingMethods
		 * @param boolean $allowNote
		 * @param boolean $requireConfirmedShippingAddress
		 * @param string $localeCode 2-letter country code of the locale
		 * @param string $landingPage "Billing" or "Login"
		 * @return array nvp result
		 */
		public function doSetExpressCheckout(scCart $cart, $currencyCode, $paymentAction, $orderId, $custom, $shippingMethods, $allowNote, $requireConfirmedShippingAddress, $localeCode, $landingPage)
		{
			$currency = new scCurrency($currencyCode);
			
			$customerInfo = $cart->getCustomerInfo();
			
			$params = array(
				'METHOD' => 'SetExpressCheckout',
				'PAYMENTACTION' => $paymentAction,
				
				'RETURNURL' => scPaths::getSecureUrl('modules/checkout/paypal/return.php'),
				'CANCELURL' => scUrlUtil::convertUrl(scFriendlyUrl::get(scFriendlyUrl::TYPE_DYNAMIC_PAGE, scFriendlyUrl::DYN_PAGE_CART), scUrlUtil::URL_TYPE_ABSOLUTE),
				'LOCALECODE' => $localeCode,
				'CURRENCYCODE' => $currencyCode,
				'INVNUM' => $orderId,
				'CUSTOM' => $custom,
				
				'BUTTONSOURCE' => 'MirchevIdeas_SummerCart_EC',
				
				'ALLOWNOTE' => $allowNote ? 1 : 0,
				'REQCONFIRMSHIPPING' => $requireConfirmedShippingAddress ? 1 : 0,
				'LANDINGPAGE' => $landingPage,
				
				// Prefill information (in case the customer will signup for PayPal)
				'EMAIL' => $customerInfo['CustomerEmail'],
				'SHIPTONAME' => $customerInfo['CustomerShippingFirstName'] . ' ' . $customerInfo['CustomerShippingLastName'],
				'SHIPTOSTREET' => $customerInfo['CustomerShippingAddress1'],
				'SHIPTOSTREET2' => $customerInfo['CustomerShippingAddress2'],
				'SHIPTOCITY' => $customerInfo['CustomerShippingCity'],
				'SHIPTOSTATE' => scCountry::getStateName($customerInfo['CustomerShippingState']),
				'SHIPTOZIP' => $customerInfo['CustomerShippingZip'],
				'SHIPTOCOUNTRY' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
				'SHIPTOPHONENUM' => $customerInfo['CustomerShippingPhone'],
			);
			
			// Add Order Items
			$n = 0;
			$params = array_merge($params, $this->getCartItems($cart, $currency, $n));
			
			// Add order line items
			$params = array_merge($params, $this->getLineItems($cart, $currency, $n));
			
			// Add Shipping Methods
			$params = array_merge($params, $this->getShippingMethods($shippingMethods, $currency));
			if (isset($params['L_SHIPPINGOPTIONISDEFAULT0'])) {
				$params['CALLBACK'] = scPaths::getSecureUrl('modules/checkout/paypal/callback.php?SessionID=' . scSession::singleton()->getSessionId());
				$params['CALLBACKTIMEOUT'] = 6;
			}
			
			// Add totals
			$params = array_merge($params, $this->getTotals($cart, $currency, $params['L_SHIPPINGOPTIONAMOUNT0'], $this->getSubtotalFromParams($params)));
			
			$params += $this->getCommonParams();
			return $this->doOperation($params);
		}
		
		public function getCallbackResponse($shippingMethods, scCurrency $currency)
		{
			$params = $this->getShippingMethods($shippingMethods, $currency);
			
			$params['METHOD'] = 'CallbackResponse';
			$params['OFFERINSURANCEOPTION'] = 'false';
			return $params;
		}
		
		public function doGetExpressCheckoutDetails($token)
		{
			$params = array(
				'METHOD' => 'GetExpressCheckoutDetails',
				'TOKEN' => $token
			);
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		public function doDoExpressCheckoutPayment(scCart $cart, $currencyCode, $paymentAction, $token, $payerId, $shippingAmount)
		{
			$currency = new scCurrency($currencyCode);
			
			$params = array(
				'METHOD' => 'DoExpressCheckoutPayment',
				'TOKEN' => $token,
				'PAYMENTACTION' => $paymentAction,
				'PAYERID' => $payerId,
				'RETURNFMFDETAILS' => 1,
				
				'CURRENCYCODE' => $currencyCode,
				
				'NOTIFYURL' => scPaths::getSecureUrl('modules/checkout/paypal/notify.php'),
			);
			$params += $this->getCommonParams();
			
			// Add Order Items
			$n = 0;
			$params = array_merge($params, $this->getCartItems($cart, $currency, $n));
			
			// Add order line items
			$params = array_merge($params, $this->getLineItems($cart, $currency, $n));
			
			// Add totals
			$params = array_merge($params, $this->getTotals($cart, $currency, $shippingAmount, $this->getSubtotalFromParams($params)));
			
			return $this->doOperation($params);
		}
		
		/**
		 * Perform operation capture
		 * @param string $authId
		 * @param float $amount
		 * @param string $currencyCode 3-character currency code
		 * @param string $completeType "Complete" or "NotComplete"
		 * @param string $note optional note
		 * @return array
		 */
		public function doDoCapture($authId, $amount, $currencyCode, $completeType, $note = '')
		{
			$params = array(
				'METHOD' => 'DoCapture',
				'AUTHORIZATIONID' => $authId,
				'AMT' => number_format($amount, 2, '.', ''),
				'CURRENCYCODE' => $currencyCode,
				'COMPLETETYPE' => $completeType,
			);
			if ($note != '')
				$params['NOTE'] = $note;
			
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		/**
		 * Perform operation refund
		 * 
		 * @param string $transactionId
		 * @param float $amount
		 * @param string $refundType one of "Other", "Full", "Partial"
		 * @param string $note optional note
		 * @return array
		 */
		public function doDoRefund($transactionId, $amount, $refundType, $note = '')
		{
			$params = array(
				'METHOD' => 'RefundTransaction',
				'TRANSACTIONID' => $transactionId,
				'REFUNDTYPE' => $refundType,
			);
			if ($refundType == 'Partial')
				$params['AMT'] = number_format($amount, 2, '.', '');
			if ($note != '')
				$params['NOTE'] = $note;
			
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		/**
		 * Perform operation void
		 * 
		 * @param string $authId
		 * @param string $note optional note
		 * @return array
		 */
		public function doDoVoid($authId, $note = '')
		{
			$params = array(
				'METHOD' => 'DoVoid',
				'AUTHORIZATIONID' => $authId,
			);
			if ($note != '')
				$params['NOTE'] = $note;
			
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		/**
		 * Perform operation reauthorization
		 * 
		 * @param string $authId
		 * @param float $amount
		 * @param string $currencyCode 3-character currency code
		 * @return array
		 */
		public function doDoReauthorization($authId, $amount, $currencyCode)
		{
			$params = array(
				'METHOD' => 'DoReauthorization',
				'AUTHORIZATIONID' => $authId,
				'AMT' => number_format($amount, 2, '.', ''),
				'CURRENCYCODE' => $currencyCode
			);
			
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		/**
		 * Retrieve Pal details
		 * 
		 * @return array
		 */
		public function doGetPalDetails()
		{
			$params = array(
				'METHOD' => 'GetPalDetails',
			);
			
			$params += $this->getCommonParams();
			
			return $this->doOperation($params);
		}
		
		protected function getCommonParams()
		{
			return array(
				'VERSION' => self::PAYPAL_API_VERSION,
				'USER' => $this->_username,
				'PWD' => $this->_password,
				'SIGNATURE' => $this->_signature,
			);
		}
		
		protected function doOperation($params)
		{
			$nvpResult = array();
			$result = scHttp::post($this->_url, http_build_query($params));
			$nvpResult = scUrlUtil::parseStr($result);
			
			if ($nvpResult['ACK'] == 'SuccessWithWarning') {
				list($errorMessage, $details) = $this->getErrorMessage($nvpResult);
				$message = 'PayPal Express: ' . $errorMessage;
				foreach ($details as $key => $value)
					$message .= '; ' . $key . '=' . $value;
				scLog::singleton()->warning($message, scLog::CATEGORY_PAYMENT);
			}
			
			if (($nvpResult['ACK'] == 'Success') or ($nvpResult['ACK'] == 'SuccessWithWarning'))
				return $nvpResult;
			
			$this->handleError($nvpResult);
		}
		
		protected function getErrorMessage($nvpResult)
		{
			$errorMessage = array();
			$details = array();
			
			$i = 0;
			while (isset($nvpResult['L_ERRORCODE' . $i])) {
				$details['L_ERRORCODE' . $i] = $nvpResult['L_ERRORCODE' . $i];
				$details['L_SHORTMESSAGE' . $i] = $nvpResult['L_SHORTMESSAGE' . $i];
				$details['L_LONGMESSAGE' . $i] = $nvpResult['L_LONGMESSAGE' . $i];
				$details['L_SEVERITYCODE' . $i] = $nvpResult['L_SEVERITYCODE' . $i];
				$errorMessage[] = $nvpResult['L_SHORTMESSAGE' . $i];
				$i++;
			}
			
			return array(implode(', ', $errorMessage), $details);
		}
		
		protected function handleError($nvpResult)
		{
			list($errorMessage, $details) = $this->getErrorMessage($nvpResult);
			
			throw new scPaymentProcessException($errorMessage, $details);
		}
		
		protected function getSubtotalFromParams($params)
		{
			// Calculate subtotal
			$subtotal = 0;
			$n = 0;
			while (isset($params['L_AMT' . $n])) {
				$subtotal += $params['L_AMT' . $n] * $params['L_QTY' . $n];
				$n++;
			}
			return $subtotal;
		}
		
		protected function getTotals(scCart $cart, $currency, $shippingAmount, $subtotal)
		{
			$params = array();
			$params['SHIPPINGAMT'] = number_format($shippingAmount, 2, '.', '');
			
			$tax = $cart->getOrderTotal()->getSummedOrderTotalLinesAmount(IOrderTotalModule::ORDER_TOTAL_TYPE_TAX);
			$params['TAXAMT'] = number_format(scPrice::factory($tax)->getAmountInCurrency($currency), 2, '.', '');
			
			$params['ITEMAMT'] = number_format($subtotal, 2, '.', '');
			
			$params['AMT'] = number_format($params['SHIPPINGAMT'] + $params['TAXAMT'] + $params['ITEMAMT'], 2, '.', '');
			
			$params['MAXAMT'] = '10000';
			return $params;
		}
		
		protected function getLineItems(scCart $cart, $currency, &$n)
		{
			$params = array();
			
			$orderTotal = $cart->getOrderTotal();
			$orderTotalLines = $orderTotal->getOrderTotalLines();
			
			foreach ($orderTotalLines as $orderTotalLine) {
				$type = $orderTotalLine->getType();
				if ($type == IOrderTotalModule::ORDER_TOTAL_TYPE_SUBTOTAL)
					continue;
				if ($type == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING)
					continue;
				
				if ($type == IOrderTotalModule::ORDER_TOTAL_TYPE_TAX) {
					continue;
				}
				
				$params['L_NAME' . $n] = $orderTotalLine->getTranslatedDescription();
				$params['L_AMT' . $n] = number_format(scPrice::factory($orderTotalLine->getAmount())->getAmountInCurrency($currency), 2, '.', '');
				$params['L_QTY' . $n] = 1;
				$n++;
			}
			
			return $params;
		}
		
		protected function getCartItems(scCart $cart, $currency, &$n)
		{
			$items = $cart->getItems();
			
			$params = array();
			foreach ($items as $item) {
				$params['L_NAME' . $n] = $item->getName();
				$params['L_NUMBER' . $n] = $item->getCartItemCode();
				$params['L_AMT' . $n] = number_format($item->getPriceObj()->getAmountInCurrency($currency), 2, '.', '');
				$params['L_QTY' . $n] = $item->getQuantity();
				
				// Set the description with the selected option info
				if ($item instanceof scCartItemProduct) {
					$options = array_merge($item->getOptions(), $item->getLocalOptions());
					
					if (count($options)) {
						$description = '';
						$price = $item->getPriceObj(false)->getAmount();
						foreach ($options as $option) {
							$optionInfo = $option->getOptionInfo($price);
							
							$description .= $optionInfo['OptionGroupName'] . ' - ' . $optionInfo['OptionName'] . ' (' . $optionInfo['OptionPrice'] . ')';
							$params['L_DESC' . $n] = $description;
						}
					}
				}
				$n++;
			}
			
			return $params;
		}
		
		protected function getShippingMethods($shippingMethods, scCurrency $currency)
		{
			$params = array();
			
			// PayPal limits the number of shipping options
			$shippingMethods = array_slice($shippingMethods, 0, self::MAX_SHIPPING_OPTIONS);
			
			$n = 0;
			foreach ($shippingMethods as $shippingMethod) {
				$params['L_SHIPPINGOPTIONISDEFAULT' . $n] = ($n == 0) ? 'true' : 'false';
				$params['L_SHIPPINGOPTIONNAME' . $n] = $shippingMethod['ShippingMethodDisplayName']; //$shippingMethod['ShippingMethodID'];
				$params['L_SHIPPINGOPTIONLABEL' . $n] = ':';	// This must have a value in order to work
				$params['L_SHIPPINGOPTIONAMOUNT' . $n] = number_format($shippingMethod['Rate']->get($currency), 2, '.', '');
				
				if (isset($shippingMethod['Tax']))
					$params['L_TAXAMT' . $n] = number_format($shippingMethod['Tax']->get($currency), 2, '.', '');
				$n++;
			}
			return $params;
		}
	}
?>