<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * PayPal PayFlow Pro payment processing module
	 */
	class scPaymentModulePayPalPayflowPro implements IPaymentModule, IModuleOptions {
		const PAYFLOW_PRO_URL = 'https://payflowpro.paypal.com/';
		const PAYFLOW_PRO_TEST_URL = 'https://pilot-payflowpro.paypal.com/';
		
		const MODE_LIVE = 'Live';
		const MODE_TEST = 'Test';
		
		protected $_moduleId;
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array())
		{
			if ($contentType == self::CONTENT_TYPE_PAYMENT_FORM) 
				return scPaymentForms::getCreditCardForm();
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			return 'USD';
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'BuyerAuthEnabled', 'Vendor', 'Partner', 'User', 'Password'));
			$url = ($options['Mode'] == self::MODE_TEST) ? self::PAYFLOW_PRO_TEST_URL : self::PAYFLOW_PRO_URL;
			
			if (!scPaymentForms::isCardExpirationValid($paramsPost['CardExpirationYear'], $paramsPost['CardExpirationMonth']))
				throw new scPaymentProcessException(_t('Invalid expiration date.'));
			
			// If buyer auth is not enabled - directly process the transaction
			if ($options['BuyerAuthEnabled'] == 0) {
				$orderId = scOrdersInProgress::singleton()->getLastOrderId();
				return $this->doProcessPayment($orderId, $payAmount, $payCurrencyCode, $cart, $paramsPost, $customerInfo, $options, $url, null);
			}
			
			return $this->verifyEnrollment($url, $customerInfo, $options, $payAmount, $payCurrencyCode, $cart, $paramsPost);
		}
		
		/**
		 * Handle the payment
		 * 
		 * @param $payAmount
		 * @param $payCurrencyCode
		 * @param $cart
		 * @param $paramsPost
		 * @param $customerInfo
		 * @param $options
		 * @param $url
		 * @param SimpleXMLElement $buyerAuthResult
		 * @return scPaymentProcessResult
		 */
		protected function doProcessPayment($orderId, $payAmount, $payCurrencyCode, scCart $cart, $paramsPost, $customerInfo, $options, $url, $buyerAuthResult)
		{
			$transactionDef = $this->getTransactionXmlDef('Sale', $cart, $customerInfo, $payAmount, $paramsPost, $buyerAuthResult);
			$xml = $this->prepareRequestXml($transactionDef, $options);
			$result = $this->doRequest($xml, $url);
			
			// Handle response
			$xmlResult = new SimpleXMLElement($result);
			$transactionResult = $xmlResult->ResponseData->TransactionResults->TransactionResult;
			
			$result = new scPaymentProcessResult($orderId, $payAmount, $payCurrencyCode, $transactionResult->PNRef);
			
			$resultCode = (string)$transactionResult->Result;
			if ($resultCode != '0') {
				
//				if (in_array($resultCode, array(12, 24, 50, 114))) {
//					$result->addValidatorError($transactionResult->Message);
//					return $result;
//				}
				
				// Error
				throw new scPaymentProcessException(_t('Processing error.'),
					array(_i('OrderID') => $orderId,
					_i('Result') => $resultCode,
					_i('Message') => $transactionResult->Message,
					_i('PNRef') => $transactionResult->PNRef));
			}
			
			
			if (isset($transactionResult->FraudPreprocessResult)) {
				if (isset($transactionResult->FraudPreprocessResult->Message))
					$result->setTransactionDetail('FraudPreprocessResult:Message', $transactionResult->FraudPreprocessResult->Message);
			}
			if (isset($transactionResult->FraudPostprocessResult)) {
				if (isset($transactionResult->FraudPostprocessResult->Message))
					$result->setTransactionDetail('FraudPostprocessResult:Message', $transactionResult->FraudPostprocessResult->Message);
			}
			
			if (isset($transactionResult->IAVSResult))
				$result->setTransactionDetail('IAVSResult', $transactionResult->IAVSResult);
			if (isset($transactionResult->CVResult))
				$result->setTransactionDetail('CVResult', $transactionResult->CVResult);
			if (isset($transactionResult->CardSecure))
				$result->setTransactionDetail('CardSecure', $transactionResult->CardSecure);
			if (isset($transactionResult->AuthCode))
				$result->setTransactionDetail('AuthCode', $transactionResult->AuthCode);
			if (isset($transactionResult->AVSResult)) {
				if (isset($transactionResult->AVSResult->StreetMatch))
					$result->setTransactionDetail('AVSResult:StreetMatch', $transactionResult->AVSResult->StreetMatch);
				if (isset($transactionResult->AVSResult->ZipMatch))
					$result->setTransactionDetail('AVSResult:ZipMatch', $transactionResult->AVSResult->ZipMatch);
			}
			
			// Add BuyerAuth info
			if ($buyerAuthResult != null) {
				foreach ($buyerAuthResult->children() as $key => $value) {
					$result->setTransactionDetail(_i('BuyerAuth: ') . $key, $value);
				}
			}
			
			$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
			return $result;
		}
		
		/**
		 * Check that the user is enrolled in buyer authentication (3D secure)
		 * Will call process payment if the buyer is not enrolled
		 * 
		 * @param $url
		 * @param $customerInfo
		 * @param $options
		 * @param $payAmount
		 * @param $payCurrencyCode
		 * @param $cart
		 * @param $paramsPost
		 * @return scPaymentProcessResult
		 */
		protected function verifyEnrollment($url, $customerInfo, $options, $payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			$transactionDef = $this->getTransactionXmlDef('VerifyEnrollment', $cart, $customerInfo, $payAmount, $paramsPost, null);
			$xml = $this->prepareRequestXml($transactionDef, $options);
			$result = $this->doRequest($xml, $url);
			
			$xmlResult = new SimpleXMLElement($result);
			$transactionResult = $xmlResult->ResponseData->TransactionResults->TransactionResult;
			if ($transactionResult->Result != '0')
				throw new scPaymentProcessException(sprintf(_i('VerifyEnrollment failed. Result: %d'), $transactionResult->Result));
			
			// E == Enrolled
			if ($transactionResult->BuyerAuthResult->Status != 'E') {
				// If not enrolled go ahead without 3D secure
				return $this->doProcessPayment($orderId, $payAmount, $payCurrencyCode, $cart, $paramsPost, $customerInfo, $options, $url, $transactionResult->BuyerAuthResult);
			}
			
			// Encrypt the MD
			$authenticationId = (string)$transactionResult->BuyerAuthResult->AuthenticationId;
			$md = serialize(array('OrderID' => $orderId, 'params' => $paramsPost, 'payAmount' => $payAmount, 'payCurrencyCode' => $payCurrencyCode));
			$md = scCryptUtil::encrypt($md, pack('H*', scCryptUtil::hmac($options['Password'], $authenticationId, scCryptUtil::HASH_MD5)));
			
			// Do a redirect to 3D secure ACS
			$result = new scPaymentProcessResult($orderId, $payAmount, $payCurrencyCode);
			$result->setThirdPartyRedirect((string)$transactionResult->BuyerAuthResult->ACSUrl,
				array(
					'MD' => base64_encode($md),
					'PaReq' => (string)$transactionResult->BuyerAuthResult->PAReq,
					'TermUrl' => scPaths::getSecureUrl('modules/payment/paypal/payflow_pro_3dauth.php')), 'iframe', array('iframe_height' => 500));
			
			// Save AuthenticationId
//			$transaction->setTransactionDetail('AuthenticationId', $responseFields['MD']);
//			$transaction->insertTransactionDetails($transactionId);
			
			return $result;
		}
		
		/**
		 * Handles 3D AUTH return
		 *
		 * @param array $params
		 */
		function process3dAuth($params)
		{
			scPaymentProcess::checkParamsExist($params, array('MD', 'PaRes'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Mode', 'Vendor', 'Partner', 'User', 'Password'));
			$url = ($options['Mode'] == self::MODE_TEST) ? self::PAYFLOW_PRO_TEST_URL : self::PAYFLOW_PRO_URL;
			
			$transactionDef = array(
				scXml::TAG => 'Transaction',
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'ValidateAuthentication',
						scXml::CHILDREN => array(
							array(
								scXml::TAG => 'PARes',
								scXml::DATA => $params['PaRes']
							)
						)
					),
				)
			);
			$xml = $this->prepareRequestXml($transactionDef, $options);
			$result = $this->doRequest($xml, $url);
			$xmlResult = new SimpleXMLElement($result);
			$transactionResult = $xmlResult->ResponseData->TransactionResults->TransactionResult;
			
			// Decrypt the MD
			$authenticationId = (string)$transactionResult->BuyerAuthResult->AuthenticationId;
			$md = base64_decode($params['MD']);
			$md = scCryptUtil::decrypt($md, pack('H*', scCryptUtil::hmac($options['Password'], $authenticationId, scCryptUtil::HASH_MD5)));
			$md = unserialize($md);
			if (!is_array($md))
				throw new scPaymentProcessException(sprintf(_i('Invalid MD.')), array('MD' => $params['MD']));
			
			$orderId = $md['OrderID'];
			$paramsPost = $md['params'];
			$payAmount = $md['payAmount'];
			$payCurrencyCode = $md['payCurrencyCode'];
//			var_dump($md);die;
			
			if ($transactionResult->Result != '0')
				throw new scPaymentProcessException(sprintf(_i('ValidateAuthentication failed. Result: %d'), $transactionResult->Result));
			
			$cart = scCart::getCart();
			$customerInfo = $cart->getCustomerInfo();
			return $this->doProcessPayment($orderId, $payAmount, $payCurrencyCode, $cart, $paramsPost, $customerInfo, $options, $url, $transactionResult->BuyerAuthResult);
		}
		
		protected function doRequest($xml, $url)
		{
			$headers = array(
				'Content-Type: text/xml',
				'X-VPS-REQUEST-ID: ' . md5(microtime()),
				'X-VPS-CLIENT-TIMEOUT: 45',
				'X-VPS-VIT-INTEGRATION-PRODUCT: Summer Cart',
//				'X-VPS-VIT-INTEGRATION-VERSION: ',
				'X-VPS-VIT-OS-NAME: ' . PHP_OS,
			);
			
			return scHttp::post($url, $xml, array('timeout' => 45, 'httpHeaders' => $headers));
		}
		
		protected function getTransactionXmlDef($requestType, $cart, $customerInfo, $payAmount, $paramsPost, $buyerAuthResult)
		{
			$currencyCode = 'USD'; // In spite of the documentaion saying that this should 3-digit numeric code, paypal needs alphabetical currency code instead
			
			$invoice = array(
				scXml::TAG => 'Invoice',
				scXml::CHILDREN => array(
					array(scXml::TAG => 'CustIP', scXml::DATA => $_SERVER['REMOTE_ADDR']),
					array(scXml::TAG => 'InvNum', scXml::DATA => scOrdersInProgress::singleton()->getLastOrderId()),
					array(
						scXml::TAG => 'BillTo',
						scXml::CHILDREN => array(
							array(scXml::TAG => 'Name', scXml::DATA => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName']),
							array(
								scXml::TAG => 'Address',
								scXml::CHILDREN => array(
									array(scXml::TAG => 'Company', scXml::DATA => $customerInfo['CustomerBillingCompany']),
									array(scXml::TAG => 'Street', scXml::DATA => $customerInfo['CustomerBillingAddress1']),
									array(scXml::TAG => 'Street2', scXml::DATA => $customerInfo['CustomerBillingAddress2']),
									array(scXml::TAG => 'City', scXml::DATA => $customerInfo['CustomerBillingCity']),
									array(scXml::TAG => 'State', scXml::DATA => scCountry::getStateCode($customerInfo['CustomerBillingState'])),
									array(scXml::TAG => 'Zip', scXml::DATA => $customerInfo['CustomerBillingZip']),
									array(scXml::TAG => 'Country', scXml::DATA => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry'])),
								)
							),
							array(scXml::TAG => 'EMail', scXml::DATA => $customerInfo['CustomerEmail']),
							array(scXml::TAG => 'Phone', scXml::DATA => $customerInfo['CustomerBillingPhone']),
							array(scXml::TAG => 'Fax', scXml::DATA => $customerInfo['CustomerBillingFax']),
						),
					),
					array(
						scXml::TAG => 'ShipTo',
						scXml::CHILDREN => array(
							array(scXml::TAG => 'Name', scXml::DATA => $customerInfo['CustomerShippingFirstName'] . ' ' . $customerInfo['CustomerShippingLastName']),
							array(
								scXml::TAG => 'Address',
								scXml::CHILDREN => array(
									array(scXml::TAG => 'Company', scXml::DATA => $customerInfo['CustomerShippingCompany']),
									array(scXml::TAG => 'Street', scXml::DATA => $customerInfo['CustomerShippingAddress1']),
									array(scXml::TAG => 'Street2', $customerInfo['CustomerBillingAddress2']),
									array(scXml::TAG => 'City', scXml::DATA => $customerInfo['CustomerShippingCity']),
									array(scXml::TAG => 'State', scXml::DATA => scCountry::getStateCode($customerInfo['CustomerShippingState'])),
									array(scXml::TAG => 'Zip', scXml::DATA => $customerInfo['CustomerShippingZip']),
									array(scXml::TAG => 'Country', scXml::DATA => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry'])),
								)
							)
						),
					),
					array(scXml::TAG => 'Items', scXml::CHILDREN => $this->getLineItemsXmlDef($cart)),
					array(scXml::TAG => 'TotalAmt',	scXml::ATTRIBUTES => array('Currency' => $currencyCode), scXml::DATA => $payAmount)
				),
			);
			
			
			$tender = array(
				scXml::TAG => 'Tender',
				scXml::CHILDREN => array(
				array(
					scXml::TAG => 'Card',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'CardNum', scXml::DATA => $paramsPost['CardNumber']),
						array(scXml::TAG => 'ExpDate', scXml::DATA => sprintf('%4d%02d', $paramsPost['CardExpirationYear'], $paramsPost['CardExpirationMonth'])),
						array(scXml::TAG => 'CVNum', scXml::DATA => $paramsPost['CardCVV']),
						array(scXml::TAG => 'NameOnCard', scXml::DATA => $paramsPost['NameOnCard']),
					),
				)
			));
			
			// If we have buyer auth info - add it as child of tender element
			if ($buyerAuthResult != null) {
				$tender[scXml::CHILDREN][0][scXml::CHILDREN][] = array(scXml::XML => $buyerAuthResult->asXml());
//				$tender[scXml::CHILDREN][0][scXml::CHILDREN][] = array(
//					scXml::TAG => 'BuyerAuthResult',
//					scXml::CHILDREN => array(
//						array(scXml::TAG => 'Status', scXml::DATA => $buyerAuthResult->Status),
//						array(scXml::TAG => 'AuthenticationId', scXml::DATA => $buyerAuthResult->AuthenticationId),
//						array(scXml::TAG => 'ECI', scXml::DATA => $buyerAuthResult->ECI),
//						array(scXml::TAG => 'CAVV', scXml::DATA => $buyerAuthResult->CAVV),
//						array(scXml::TAG => 'XID', scXml::DATA => $buyerAuthResult->XID),
//					)
//				);
			}
			
			$transactionDef = array(
				scXml::TAG => 'Transaction',
				scXml::CHILDREN => array(
					array(
						scXml::TAG => $requestType,
						scXml::CHILDREN => array(
							array(
								scXml::TAG => 'PayData',
								scXml::CHILDREN => array($invoice, $tender)
							)
						)
					),
					array(scXml::TAG => 'BrowserUserAgent', scXml::DATA => $_SERVER['HTTP_USER_AGENT']),
				)
			);
			
			return $transactionDef;
		}
		
		protected function prepareRequestXml($transactionDef, $options)
		{
			$xmlDef = array(
				array(
				scXml::TAG => 'XMLPayRequest',
				scXml::ATTRIBUTES => array('Timeout' => '30', 'version' => '2.0', 'xmlns' => 'http://www.paypal.com/XMLPay'),
				scXml::CHILDREN => array(
					array(
						scXml::TAG => 'RequestData',
						scXml::CHILDREN => array(
							array(scXml::TAG => 'Vendor', scXml::DATA => $options['Vendor']),
							array(scXml::TAG => 'Partner', scXml::DATA => $options['Partner']),
							array(scXml::TAG => 'Transactions', scXml::CHILDREN => array($transactionDef)
							),
						)
					),
					array(
						scXml::TAG => 'RequestAuth',
						scXml::CHILDREN => array(
							array(
								scXml::TAG => 'UserPass',
								scXml::CHILDREN => array(
									array(scXml::TAG => 'User',	scXml::DATA => $options['User']),
									array(scXml::TAG => 'Password', scXml::DATA => $options['Password'])
								)
							)
						)
					),
				)
			));
			
			return '<?xml version="1.0" encoding="UTF-8"?>' . "\n" . scXml::xmlWriter($xmlDef);
		}
		
		protected function getLineItemsXmlDef(scCart $cart)
		{
			$xmlDef = array();
			
			$currency = new scCurrency('USD');
			$items = $cart->getItems();
			$n = 0;
			foreach ($items as $item) {
				$n++;
				$xmlDef[] = array(
					scXml::TAG => 'Item',
					scXml::ATTRIBUTES => array('Number' => $n),
					scXml::CHILDREN => array(
						array(scXml::TAG => 'SKU',	scXml::DATA => $item->getCartItemCode()),
						array(scXml::TAG => 'Description', scXml::DATA => $item->getName()),
						array(scXml::TAG => 'Quantity', scXml::DATA => $item->getQuantity()),
						array(scXml::TAG => 'UnitPrice', scXml::DATA => $item->getPriceObj()->getAmountInCurrency($currency)),
						array(scXml::TAG => 'TotalAmt', scXml::DATA => scPrice::factory($item->getTotal())->getAmountInCurrency($currency)),
					)
				);
			}
			
			return $xmlDef;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miSelectWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'BuyerAuthEnabled',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Vendor',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Partner',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'User',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Password',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$modes = array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test'));
			$options->getOptionField('Mode')->setOptions($modes);
			$options->setTemplateFile('modules/payment/paypal/payflow_pro_options.tmpl');
			return $options;
		}
	}
?>