<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * RBS WorldPay payment processing module
	 */
	class scPaymentModuleRBSWorldPay implements IPaymentModule, IModuleOptions {
		protected $_moduleId;
		
		const RBS_WORLDPAY_TEST_URL = 'https://select-test.wp3.rbsworldpay.com/wcc/purchase';
		const RBS_WORLDPAY_LIVE_URL = 'https://select.wp3.rbsworldpay.com/wcc/purchase';
		
		const MODE_TEST = 0;
		const MODE_LIVE = 1;
		
		/**
		 * Supported languages by RBS WorldPay
		 * @var array
		 */
		protected $_supportedLanguages = array(
			'JA', 'DE', 'EN', 'ES', 'FR', 'IT', 'NL', 'SV'
		);
		
		protected $_supportedCurrencies = array('ARS', 'AUD', 'BRL', 'BGL', 'CAD', 'CHF', 'CLP', 'CNY', 'COP', 'CZK', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'IDR', 'JPY',
			'KES', 'KRW', 'MYR', 'NOK', 'NZD', 'PHP', 'PLN', 'PTE', 'SEK', 'SGD', 'SKK', 'THB', 'TWD', 'USD', 'VND', 'ZAR',
			'CYP', 'EGP', 'BHD', 'BGN', 'AED', 'HRK', 'JOD', 'KWD', 'EEK', 'ROL', 'MTL', 'LVL', 'LTL', 'MXN', 'MKD', 'ANG',
			'ILS', 'NPR', 'QAR', 'RUB', 'OMR', 'RON', 'PKR', 'INR', 'SAR', 'LKR', 'UAH');
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Return the payment currency
		 */
		public function getPaymentCurrencyCode()
		{
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_supportedCurrencies))
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrencyCode'));
			return $options['DefaultCurrencyCode'];
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('InstID', 'Mode', 'AuthMode', 'OrderDescription', 'AllowContactEdit', 'MD5Secret'), scModuleOptions::MODULE_OPTIONS_INCLUDE_ML);
			
			$isoLangCode = strtolower(scMultiLanguage::getCurrentLanguageCode());
			$lang = in_array($isoLangCode, $this->_supportedLanguages) ? $isoLangCode : 'EN';
			
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			
			// Fix incorrect Bulgarian currency code
//			if ($payCurrencyCode == 'BGN')
//				$payCurrencyCode = 'BGL';
			
			$params = array(
				'instId' => $options['InstID'],
				'desc' => $options['OrderDescription'],
				'cartId' => $orderId,
				'authMode' => $options['AuthMode'],
				'fixContact' => $options['AllowContactEdit'] ? 0 : 1,
				
				'lang' => $lang,
				
				'amount' => $payAmount,
				'currency' => $payCurrencyCode,
				
				'name' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'address' => $this->buildTextAddress($customerInfo['CustomerBillingAddress1'], $customerInfo['CustomerBillingAddress2'], $customerInfo['CustomerBillingCity'], $customerInfo['CustomerBillingState']),
				'postcode' => $customerInfo['CustomerBillingZip'],
				'country' => scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']),
				'email' => $customerInfo['CustomerEmail'],
				'tel' => $customerInfo['CustomerBillingPhone'],
				'fax' => $customerInfo['CustomerBillingFax'],
			);
			if ($options['Mode'] == self::MODE_TEST) {
				$params['testMode'] = 100;
			}
			
			if (!$this->isShippingAddressSame($customerInfo)) {
				$params = array_merge($params, array(
					'withDelivery' => 'true',
					'delvName' => $customerInfo['CustomerShippingFirstName'] . ' ' . $customerInfo['CustomerShippingLastName'],
					'delvAddress' => $this->buildTextAddress($customerInfo['CustomerShippingAddress1'], $customerInfo['CustomerShippingAddress2'], $customerInfo['CustomerShippingCity'], $customerInfo['CustomerShippingState']),
					'delvPostcode' => $customerInfo['CustomerShippingZip'],
					'delvCountry' => scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']),
				));
			}
			
			$this->addSignature($params, $options['MD5Secret']);
			
			$result = new scPaymentProcessResult();
			$url = ($options['Mode'] == self::MODE_TEST) ? self::RBS_WORLDPAY_TEST_URL : self::RBS_WORLDPAY_LIVE_URL;
			
			$result->setThirdPartyRedirect($url, $params);
			return $result;
		}
		
		/**
		 * Format a text address
		 * 
		 * @param string $address1
		 * @param string $address2
		 * @param string $city
		 * @param string $state
		 * @return string
		 */
		protected function buildTextAddress($address1, $address2, $city, $state)
		{
			$text = $address1;
			if (trim($address2) != '')
				$text .= '&#10;' . $address2;
			$text .= '&#10;' . $city;
			if (trim($state) != '')
				$text .= '&#10;' . $state;
			return $text;
		}
		
		/**
		 * Returns true if billing and shipping addresses are the same
		 * 
		 * @param array $customerInfo
		 * @return bool
		 */
		protected function isShippingAddressSame($customerInfo)
		{
			$fields = array(
				'CustomerBillingFirstName' => 'CustomerShippingFirstName',
				'CustomerBillingLastName' => 'CustomerShippingLastName',
				'CustomerBillingAddress1' => 'CustomerShippingAddress1',
				'CustomerBillingAddress2' => 'CustomerShippingAddress2',
				'CustomerBillingCity' => 'CustomerShippingCity',
				'CustomerBillingState' => 'CustomerShippingState',
				'CustomerBillingCountry' => 'CustomerShippingCountry',
				'CustomerBillingZip' => 'CustomerShippingZip'
			);
			foreach ($fields as $field1 => $field2) {
				if ($customerInfo[$field1] != $customerInfo[$field2])
					return false;
			}
			return true;
			
		}
		
		/**
		 * Add the signature fields
		 * 
		 * @param array $params in/out param
		 * @param string $md5Secret
		 */
		protected function addSignature(&$params, $md5Secret)
		{
			$signatureArray = array($md5Secret);
			$signatureFieldsArray = array();
			foreach ($params as $key => $value) {
				$signatureArray[] = $value;
				$signatureFieldsArray[] = $key;
			}
			
			// WorldPay does (brain-dead) conversion to ISO-8859-1 before computing the MD5 signature
			$params['signature'] = md5(utf8_decode(implode(':', $signatureArray)));
			$params['signatureFields'] = implode(':', $signatureFieldsArray);
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			$orderId = (int)miGetParamDefault('OrderID', '');
			
			$result = new scPaymentProcessResult($orderId);
			return $result;
		}
		
		/**
		 * Handles status notify return
		 *
		 * @param array $params
		 */
		function processResponse($params)
		{
			scPaymentProcess::checkParamsExist($params, array('cartId', 'authAmount', 'authCurrency', 'rawAuthMessage', 'callbackPW', 'countryMatch', 'AVS', 'cardType', 'ipAddress', 'authMode'));
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('PaymentResponsePassword'));
			
			$transId = isset($params['transId']) ? $params['transId'] : '';
			$result = new scPaymentProcessResult($params['cartId'], $params['authAmount'], $params['authCurrency'], $transId);
			
			if ($params['callbackPW'] != $options['PaymentResponsePassword']) {
				throw new miException(_i('The Payment Response password does not match.'));
			}
			
			// Check that the amount matches
			$currency = $params['currency'];
//			$currency = ($currency == 'BGL') ? 'BGN' : $currency;	// Fix incorrect Bulgarian currency code
			scPaymentProcess::checkPaymentToOrder($params['cartId'], $params['amount'], $currency);
			
			if ($params['transStatus'] == 'Y') {
				if ($params['authMode'] == 'A')
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_CAPTURE);
				elseif ($params['authMode'] == 'E')
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_AUTH);
				else
					$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);	// Unknown auth mode
			} else
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			
			if (isset($params['testMode']) and ($params['testMode'] > 0)) {
				$result->setPaymentNotes(_t('Order in demo mode.'));
				$result->setPaymentStatus(scPaymentStatus::PAYMENT_STATUS_DECLINED);
			}
			
			$result->setTransactionDetail('countryMatch', $params['countryMatch']);
			$result->setTransactionDetail('AVS', $params['AVS']);
			$result->setTransactionDetail('cardType', $params['cardType']);
			$result->setTransactionDetail('ipAddress', $params['ipAddress']);
			$result->setTransactionDetail('authMode', $params['authMode']);
			if ($params['authAmount'] != '')
				$result->setTransactionDetail('authAmount', $params['authAmount']);
			if ($params['authCurrency'] != '')
				$result->setTransactionDetail('authCurrency', $params['authCurrency']);
			$result->setTransactionDetail('rawAuthMessage', $params['rawAuthMessage']);
			if (isset($params['transTime'])) {
				$result->setTransactionDetail('transTime', $params['transTime']);
				$result->setTransactionDetail('transTimeFormatted', date('r', floatval($params['transTime']) / 1000));
			}
			
			// Return the Shopper response
			$t = new scTemplateParser();
			$t->assign('%%RETURN_MESSAGE%%', sprintf('<a href="%s">' . _t('Return to merchant') . '</a>', miI18N::htmlEscape(scPaths::getSecureUrl('modules/payment/rbsworldpay/return.php?OrderID=' . $params['cartId']))));
			if ($params['transStatus'] == 'Y')
				$t->readTemplate('modules/payment/rbsworldpay/resultY.html');
			else
				$t->readTemplate('modules/payment/rbsworldpay/resultC.html');
			echo $t->templateParse();
			
			return $result;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'InstID',
					'properties' => array('minLength' => 1)
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'AuthMode',
				),
				array(
					'field' => 'scWebFormWidgetTextML',
					'data' => 'OrderDescription',
				),
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'AllowContactEdit',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MD5Secret',
					'properties' => array('minLength' => 1, 'maxLength' => 16)
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'PaymentResponsePassword',
					'properties' => array('minLength' => 1)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('Mode')->setRadioButtons(array(self::MODE_LIVE => _t('Live'), self::MODE_TEST => _t('Test')), self::MODE_TEST);
			$options->getOptionField('AuthMode')->setRadioButtons(array('A' => _i('Full Auth'), 'E' => _i('Pre Auth')), 'A');
			sort($this->_supportedCurrencies);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_supportedCurrencies, $this->_supportedCurrencies));
			$options->setTemplateFile('modules/payment/rbsworldpay/options.tmpl');
			return $options;
		}
	}
?>