<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Realex payment processing module
	 */
	class scPaymentModuleRealexHostedPage implements IPaymentModule, IModuleOptions {
		
		protected $_moduleId;
		
		const URL = 'https://epage.payandshop.com/epage.cgi';
		
		protected $_validCurrencies = array('EUR','GBP','USD','SEK','CHF','HKD','JPY');
		
		public function __construct($moduleId)
		{
			$this->_moduleId = $moduleId;
		}
		
		public function getPaymentContent($contentType, $params = array()) {}

		public function hasCheckoutStep()
		{
			return true;
		}
		
		public function getPaymentCurrencyCode()
		{
			$validCurrencies = array();
			if (in_array(scCurrencyUtil::getCurrentCurrency()->getCurrencyCode(), $this->_validCurrencies)) {
				return scCurrencyUtil::getCurrentCurrency()->getCurrencyCode();
			}
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('DefaultCurrency'));
			return $options['DefaultCurrency']; 
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		public function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$customerInfo = $cart->getCustomerInfo();
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Account', 'Secret', 'AutoCapture'));
			$orderId = scOrdersInProgress::singleton()->getLastOrderId();
			$currencyCode = $this->getPaymentCurrencyCode();
			$amountToPay = $cart->getOrderTotal()->getGrandTotalInCurrency(new scCurrency($currencyCode));
			
			$params = array(
				'MERCHANT_ID' => $options['MerchantID'],
				'ACCOUNT' => $options['Account'],
				'ORDER_ID' => $orderId,
				'AMOUNT' => round($amountToPay * 100), // required integer by payment proccessor
				'CURRENCY' => $currencyCode,
				'TIMESTAMP' => date('YmdHis'),
				'AUTO_SETTLE_FLAG' => $options['AutoCapture'],
			);
			
			$params['SHIPPING_CODE'] = $customerInfo['CustomerShippingZip'];
			$params['SHIPPING_CO'] = scCountry::getCountryCode2($customerInfo['CustomerShippingCountry']);
			$params['BILLING_CODE'] = $customerInfo['CustomerBillingZip'];
			$params['BILLING_CO'] = scCountry::getCountryCode2($customerInfo['CustomerBillingCountry']);
			$params['CUST_NUM'] = $customerInfo['CustomerID'];
			
			if (empty($options['Account'])) {
				unset($params['ACCOUNT']);
			}
			
			$params = $this->prepareParams($params, $options['Secret']);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect(self::URL, $params);
			return $result; 
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 * @return scPaymentProcessResult
		 */
		public function processNotify($params)
		{
			scPaymentProcess::checkParamsExist($params, array('TIMESTAMP', 'MERCHANT_ID', 'ORDER_ID', 'RESULT', 'MESSAGE', 'PASREF', 'AUTHCODE'));
			
			// get module options
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('MerchantID', 'Account', 'Secret', 'AutoCapture'));
			
			// verify the hash
			$inputString = implode('.', array($params['TIMESTAMP'], $params['MERCHANT_ID'], $params['ORDER_ID'], $params['RESULT'], $params['MESSAGE'], $params['PASREF'], $params['AUTHCODE']));
			
			$tempHash = sha1($inputString);
			$hash = sha1($tempHash . '.' . $options['Secret']);
			if ($hash != $params['SHA1HASH']) {
				throw new scPaymentProcessException(_t('Invalid response from Realex. Please contact the owner of the store.'));
			}
			
			if (isset($params['MERCHANT_ID']) and ($params['MERCHANT_ID'] != $options['MerchantID']))
				throw new scPaymentProcessException(_i('Merchant does not match.'));
			
			$orderId = $params['ORDER_ID'];
			
			$order = new scOrder();
			$order->read($orderId);
			
			// We are not sure if the amount and the currency are correct
			$result = new scPaymentProcessResult($orderId, ($params['AMOUNT'] / 100), '', $params['PASREF']);
			$result->setPaymentStatus($this->getStatusForSc($params['RESULT']));
			$result->setPaymentNotes($this->getStatusMessage($params['RESULT']));
			$result->setAllTransactionDetails($params);
			
			return $result;
		}
		
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'MerchantID',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'Account',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'Secret',
				),
				
				array(
					'field' => 'miSelectWidget',
					'data' => 'AutoCapture',
				),
				
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrency',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			
			$captureModes = array(1 => _t('Enabled'), 0 => _t('Disabled'));
			$options->getOptionField('AutoCapture')->setOptions($captureModes);
			
			$options->getOptionField('DefaultCurrency')->setOptions(array_combine($this->_validCurrencies, $this->_validCurrencies));
			
			$options->setTemplateFile('modules/payment/realex/options.tmpl');
			return $options;
		}
		
		/**
		 * calculates and adds the checksumm to the params array
		 * @param array $params
		 * @return array $params
		 */
		protected function prepareParams($params, $secret)
		{
			$inputString = implode('.', array($params['TIMESTAMP'], $params['MERCHANT_ID'], $params['ORDER_ID'], $params['AMOUNT'], $params['CURRENCY']));			
			$tempHash = sha1($inputString);
			$hash = sha1($tempHash . '.' . $secret);
			$params['SHA1HASH'] = $hash;
			return $params;
		}
		
		/**
		 * We don't make difference between differen 2xx, 3xx and 5xx statuses
		 * @param string $status
		 * @return string $status
		 */
		protected function normalizeStatusForSc($status)
		{
			$status = (int)$status;
			if ($status >= 200 && $status <= 299) 
				$status = 200;
			if ($status >= 300 && $status <= 399) 
				$status = 300;
			if ($status >= 500 && $status <= 599) 
				$status = 500;
				
			return (string)$status;
		}
		
		protected function getStatusForSc($status) 
		{
			$status = $this->normalizeStatusForSc($status);
			
			switch ($status) {
				case '00': 
					$options = scModuleOptions::getModuleOptions($this->_moduleId, array('AutoCapture'));
					return $options['AutoCapture'] ? scPaymentStatus::PAYMENT_STATUS_CAPTURE : scPaymentStatus::PAYMENT_STATUS_AUTH;
				
				case '101':
				case '102':
				case '103':
				case '200':
				case '300':
				case '500':
				case '666':
				default:
					return scPaymentStatus::PAYMENT_STATUS_DECLINED;
			}
		}
		
		protected function getStatusMessage($status) 
		{
			$status = $this->normalizeStatusForSc($status);
			
			switch($status) {
				case '00': 
					$message = _t('Successful');
					break;
					
				case '101': 
					$message = _t('Declined by Bank');
					break;
					
				case '102': 
					$message = _t('Referral by Bank');
					break;
					
				case '103': 
					$message = _t('Card reported lost or stolen');
					break;
					
				case '200': 
					$message = _t('Error with bank systems');
					break;
					
				case '300': 
					$message = _t('Error with Realex Payments systems');
					break;
					
				case '500': 
					$message = _t('Incorrect XML message formation or content');
					break;
					
				case '666': 
					$message = _t('Client deactivated');
					break;
					
				default:
					$message = null;
			}
			return $message;
		}
	}
?>