<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/vsp.php');
	
	/**
	 * Sage Pay VSP Form payment processing module
	 */
	class scPaymentModuleSagePayVSPForm extends scPaymentModuleSagePayVSP implements IPaymentModule, IModuleOptions {
		
		protected $_sagepayRedirectUrls = array(
			self::VSP_SIMULATOR_MODE => 'https://test.sagepay.com/Simulator/VSPFormGateway.asp',
			self::VSP_TEST_MODE => 'https://test.sagepay.com/gateway/service/vspform-register.vsp',
			self::VSP_LIVE_MODE => 'https://live.sagepay.com/gateway/service/vspform-register.vsp'
		);
		
		/**
		 * Performs XOR with a key
		 * 
		 * @param string $data
		 * @param string $key
		 * @return string
		 */
		function xorEncrypt($data, $key)
		{
			$keylen = strlen($key);
			
			$output = '';
			for ($i = 0; $i < strlen($data); $i++) {
				$output .= $data[$i] ^ $key[$i % $keylen];
			}
			return $output;
		}
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId,
				array('Vendor', 'TransactionType', 'EncryptionPassword', 'Mode', 'OrderDescription', 'VendorEmail', 'SendEMail', 'eMailMessage'));
			
			$customerInfo = $cart->getCustomerInfo();
			
			// These are the VSP Form specific fields
			$data = array(
				'SuccessURL' => scPaths::getSecureUrl('modules/payment/sagepay/vsp_form_return.php'),
				'FailureURL' => scPaths::getSecureUrl('modules/payment/sagepay/vsp_form_return.php'),
				
				'CustomerName' => $customerInfo['CustomerBillingFirstName'] . ' ' . $customerInfo['CustomerBillingLastName'],
				'CustomerEMail' => $customerInfo['CustomerEmail'],
				'VendorEMail' => $options['VendorEmail'],
				'SendEMail' => $options['SendEMail'],
				'eMailMessage' => $options['eMailMessage'],
			);
			
			// Add the common fields
			$data += $this->getCommonDataFields($payAmount, $payCurrencyCode, $cart, $customerInfo, $options);
			
			// Combine the values	
			$values = array();
			foreach ($data as $key => $value)
				$values[] = $key . '=' . $value; 
			
			$encryptionPassword = $options['EncryptionPassword'];
			$params = array(
				'VPSProtocol' => '2.23',
				'TxType' => $options['TransactionType'],
				'Vendor' => $options['Vendor'],
				'Crypt' => base64_encode($this->xorEncrypt(implode('&', $values), $encryptionPassword))
			);
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($this->getProxRedirectUrl($options['Mode']), $params);
			return $result; 
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Vendor', 'TransactionType', 'EncryptionPassword', 'Mode'));
			
			scPaymentProcess::checkParamsExist($params, array('crypt'));
			
			$crypt = str_replace(' ', '+', $params['crypt']);
			$data = $this->xorEncrypt(base64_decode($crypt), $options['EncryptionPassword']);
			
			$tokens = $this->getTokens($data);
			
			if (!isset($tokens['Status']))
				throw new scPaymentProcessException(_t('Invalid data from Sage Pay.'));
			
			$orderId = $tokens['VendorTxCode'];
			$transactionId = isset($tokens['VPSTxId']) ? $tokens['VPSTxId'] : '';
			
			// Check that the amount matches (remove any commas in the number)
			scPaymentProcess::checkPaymentToOrder($orderId, str_replace(',', '', $tokens['Amount']));
			
			$result = new scPaymentProcessResult($tokens['VendorTxCode'], $tokens['Amount'], $this->getOrderCurrencyCode($orderId), $transactionId);
			$result->setPaymentStatus($this->getPaymentStatus($tokens['Status']));
			$result->setPaymentNotes(sprintf(_i('%s (%s)'), $tokens['Status'], $tokens['StatusDetail']));
			
			$this->setTransactionDetails($result, $tokens);
			
			return $result;
		}
		
		/**
		 * Return the pay currency code for the order
		 * 
		 * @param int $orderId
		 * @return string 3-character currency code
		 */
		protected function getOrderCurrencyCode($orderId)
		{
			$order = new scOrder($orderId);
			return $order->get('OrderTotalPayCurrencyCode');
		}
		
		/**
		 * Return the tokens from the result
		 * Attempts to extract the correct info even if there is '&' or '=' in the data
		 * 
		 * @param string $data
		 * @return array
		 */
		protected function getTokens($data)
		{
			$supportedTokens = array('Status', 'StatusDetail', 'VendorTxCode', 'VPSTxId', 'TxAuthNo', 'Amount', 'AVSCV2',
				'AddressResult', 'PostCodeResult', 'CV2Result', 'GiftAid', '3DSecureStatus', 'CAVV', 'AddressStatus',
				'PayerStatus', 'CardType', 'Last4Digits');
			
			$tokens = array();
			
			// Find all token positions
			$positions = array();
			foreach ($supportedTokens as $supportedToken) {
				$pos = strpos($data, $supportedToken . '=');
				if ($pos !== false)
					$positions[$pos] = $supportedToken;
			}
			ksort($positions);
			
			// Extract the tokens
			reset($positions);
			while (list($pos, $token) = each($positions)) {
				$valueStart = $pos + strlen($token) + 1;
				
				$nextpos = key($positions);
				if ($nextpos) {
					$tokens[$token] = substr($data, $valueStart, $nextpos - $valueStart - 1);
				} else {
					$tokens[$token] = substr($data, $valueStart);
				}
			}
			
			return $tokens;
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'Vendor',
					'properties' => array('maxLength' => 15)
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'EncryptionPassword',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 100),
				),
				array(
					'field' => 'scModuleOptionMultiCheckboxWidget',
					'data' => 'SupportedCurrencyCodes',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'VendorEmail',
					'properties' => array('maxLength' => 255)
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'SendEMail',
				),
				array(
					'field' => 'scWebFormWidgetTextarea',
					'data' => 'eMailMessage',
					'properties' => array('maxLength' => 7500)
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('TransactionType')->setOptions(array_combine($this->_transactionTypes, $this->_transactionTypes));
			
			$modes = array(self::VSP_SIMULATOR_MODE => _t('Simulator'), self::VSP_TEST_MODE => _t('Test'), self::VSP_LIVE_MODE => _t('Live'));
			$options->getOptionField('Mode')->setRadioButtons($modes, self::VSP_TEST_MODE);
			
			$options->getOptionField('SupportedCurrencyCodes')->setRadioButtons(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			
			$sendEmailOptions = array(0 => _t('Do not send'), 1 => _t('Send to customer and vendor'), 2 => _t('Send to vendor only'));
			$options->getOptionField('SendEMail')->setOptions($sendEmailOptions, 1);
			$options->setTemplateFile('modules/payment/sagepay/vsp_form_options.tmpl');
			return $options;
		}
	}
?>