<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/vsp.php');
	
	/**
	 * Sage Pay VSP Form payment processing module
	 */
	class scPaymentModuleSagePayVSPServer extends scPaymentModuleSagePayVSP implements IPaymentModule, IModuleOptions {
		
		protected $_sagepayRedirectUrls = array(
			self::VSP_SIMULATOR_MODE => 'https://test.sagepay.com/Simulator/VSPServerGateway.asp?Service=VendorRegisterTx',
			self::VSP_TEST_MODE => 'https://test.sagepay.com/gateway/service/vspserver-register.vsp',
			self::VSP_LIVE_MODE => 'https://live.sagepay.com/gateway/service/vspserver-register.vsp'
		);
		
		public function hasCheckoutStep()
		{
			return true;
		}
		
		/**
		 * Process the payment
		 *
		 * @param float $payAmount
		 * @param string $payCurrencyCode
		 * @param scCart $cart
		 * @param array $paramsPost
		 * @return scPaymentProcessResult the payment processing status
		 */
		function processPayment($payAmount, $payCurrencyCode, scCart $cart, $paramsPost)
		{
			$options = scModuleOptions::getModuleOptions($this->_moduleId, array('Vendor', 'TransactionType', 'Mode', 'OrderDescription'));
			
			$customerInfo = $cart->getCustomerInfo();
			
			// These are the VSP Server specific fields
			$data = array(
				'VPSProtocol' => '2.23',
				'TxType' => $options['TransactionType'],
				'Vendor' => $options['Vendor'],
				
				'NotificationURL' => scPaths::getSecureUrl('modules/payment/sagepay/vsp_server_notify.php'),
				
				'CustomerEMail' => $customerInfo['CustomerEmail'],
				'Profile' => 'NORMAL',	// TODO: or LOW
			);
			
			// Add the common fields
			$data += $this->getCommonDataFields($payAmount, $payCurrencyCode, $cart, $customerInfo, $options);
			
			// Append the transaction id number to the order id so that it gets unique TxCode every time
			$transaction = new scTransaction(scOrdersInProgress::singleton()->getLastOrderId(), $this->_moduleId,
				$payAmount, $payCurrencyCode, $data['VendorTxCode']);
			$transactionId = $transaction->insertTransaction();
			$data['VendorTxCode'] = scOrdersInProgress::singleton()->getLastOrderId() . '-' . $transactionId;
			
			$transactionRegistrationResult = scHttp::post($this->getProxRedirectUrl($options['Mode']), http_build_query($data));
			
			$responseFields = $this->getResponseValues($transactionRegistrationResult);
			if ($responseFields['Status'] != 'OK')
				throw new scPaymentProcessException(sprintf(_t('Error status from Sage Pay: %s (%s).'), $responseFields['Status'], $responseFields['StatusDetail']));
			
			// Save the transaction code
			$transaction->setTransactionCode($responseFields['VPSTxId']);
			$transaction->updateTransaction();
			
			// Save the security key as transaction detail
			$transaction->setTransactionDetail('SecurityKey', $responseFields['SecurityKey']);
			$transaction->insertTransactionDetails($transactionId);
			
//			var_dump($responseFields);
//			var_dump($data);die;
			
			$result = new scPaymentProcessResult();
			$result->setThirdPartyRedirect($responseFields['NextURL'], array());
			return $result;
		}
		
		/**
		 * Handle notification
		 *
		 * @param array $params
		 */
		public function processNotify($params)
		{
			scPaymentProcess::checkParamsExist($params, array('VPSProtocol', 'TxType', 'VendorTxCode', 'VPSTxId', 'Status', 'VPSSignature'));
			
			list($orderId, $transactionId) = explode('-', $params['VendorTxCode']);
			
			$options = scModuleOptions::getModuleOptions($this->_moduleId);
			
			// Validate notification
			$data = $params;
			$data['VendorName'] = $options['Vendor'];
			$data['SecurityKey'] = $this->getSecurityKey($transactionId);
			$signature = $this->computeSignature($data);
			
			if ($signature != $params['VPSSignature'])
				throw new scPaymentProcessException(_i('Sage Pay notify signature does not match.'));
			
			// Process the notification
			$result = new scPaymentProcessResult($orderId, 0, '', $params['VPSTxId']);
			$result->setPaymentStatus($this->getPaymentStatus($params['Status']));
			
			$paymentNotes = $params['Status'];
			if (isset($params['StatusDetail']))
				$paymentNotes .= ' (' . $params['StatusDetail'] . ')';
			$result->setPaymentNotes($paymentNotes);
			
			$this->setTransactionDetails($result, $params);
			
			return $result;
		}
		
		/**
		 * Handles payment return
		 *
		 * @param array $params
		 */
		function processReturn($params)
		{
			scPaymentProcess::checkParamsExist($params, array('OrderID'));
			
			$paymentStatus = isset($params['PaymentStatus']) ? $params['PaymentStatus'] : '';
			$paymentNotes = isset($params['PaymentNotes']) ? $params['PaymentNotes'] : '';
			scPaymentProcess::redirectToPaymentResult((int)$params['OrderID'], $paymentNotes);
			exit;
		}
		
		/**
		 * Returns the security key obtained during transaction registration
		 * 
		 * @param int $transactionId
		 * @return string
		 */
		protected function getSecurityKey($transactionId)
		{
			$sql = 'SELECT TransactionDetailValue FROM TransactionDetails WHERE TransactionID=? AND TransactionDetailName=?';
			$rows = miStaticDBUtil::execSelect($sql, array($transactionId, 'SecurityKey'));
			if (count($rows) == 0)
				throw new scPaymentProcessException(sprintf(_i('Sage Pay SecurityKey for transaction %s not found.'), $transactionId));
			
			return $rows[0]['TransactionDetailValue'];
		}
		
		/**
		 * Computes the signature used to the sign the notification
		 * 
		 * @param array $data
		 * @return string
		 */
		protected function computeSignature($data)
		{
			$fields = array('VPSTxId', 'VendorTxCode', 'Status', 'TxAuthNo', 'VendorName', 'AVSCV2', 'SecurityKey',
				'AddressResult', 'PostCodeResult', 'CV2Result', 'GiftAid', '3DSecureStatus', 'CAVV',
				'AddressStatus', 'PayerStatus', 'CardType', 'Last4Digits');
			
			$text = '';
			foreach ($fields as $field) {
				if (isset($data[$field]))
					$text .= $data[$field];
			}
			
			return strtoupper(md5($text));
		}
		
		/**
		 * Returns the option fields
		 * 
		 * @param miWebForm $webForm the web form where the fields will be displayed
		 * @return array
		 */
		function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'Vendor',
					'properties' => array('maxLength' => 15)
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'TransactionType',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Mode',
				),
				array(
					'field' => 'miBaseTextWidget',
					'data' => 'OrderDescription',
					'properties' => array('maxLength' => 100),
				),
				array(
					'field' => 'scModuleOptionMultiCheckboxWidget',
					'data' => 'SupportedCurrencyCodes',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'DefaultCurrencyCode',
					'validator' => 'scCurrencyExistsValidator'
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->getOptionField('TransactionType')->setOptions(array_combine($this->_transactionTypes, $this->_transactionTypes));
			
			$modes = array(self::VSP_SIMULATOR_MODE => _t('Simulator'), self::VSP_TEST_MODE => _t('Test'), self::VSP_LIVE_MODE => _t('Live'));
			$options->getOptionField('Mode')->setRadioButtons($modes, self::VSP_TEST_MODE);
			
			$options->getOptionField('SupportedCurrencyCodes')->setRadioButtons(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			$options->getOptionField('DefaultCurrencyCode')->setOptions(array_combine($this->_sagepayCurrencies, $this->_sagepayCurrencies), self::DEFAULT_CURRENCY_CODE);
			
			$options->setTemplateFile('modules/payment/sagepay/vsp_server_options.tmpl');
			return $options;
		}
	}
?>