<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/return_codes.php');
	require_once(dirname(__FILE__) . '/shipping_methods.php');
	
	class scCanadaPost implements IShippingModule, IModuleOptions {
		
		protected $_moduleId;
		protected $_shippingModuleId;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
		
		public function getShippingMethods()
		{
			$shippingMethods = array();
			foreach (scCanadaPostShippingMethods::$shippingMethods as $serviceTypeId => $serviceName)
				$shippingMethods[$serviceTypeId] = _i('CANADA POST ') . $serviceName;
			return $shippingMethods;
		}
		
		public function getShippingRates(scCart $cart, $shippingMethods)
		{
			try {
				$options = $this->checkAndGetOptions();
				$shippingAddress = $cart->getShippingAddress();
				
				$apiClient = new scCanadaPostAPI();
				$apiClient->setOptions($options);
				$apiClient->setShippingAddress($shippingAddress);
				$apiClient->setProducts($cart);
				$serviceRates = $apiClient->getRates();

			} catch (scCanadaPostAPIException $exception) {
				// Pass exception message up
				return array(self::SHIPPING_RATES_MESSAGE => $exception->getMessage() . ' (' . $exception->getCode() . ')');
			}
			
			// Check for SoftError
			$result = array();
			
			// Extract rates
			$prices = array();
			
			foreach ($serviceRates['Entries'] as $entry) {
				$prices[$entry['id']] = $entry['rate'];
			}
			$result[self::SHIPPING_RATES] = $prices;
			$result[self::SHIPPING_RATES_CURRENCY] = 'CAD';
			
			return $result;
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/canada_post/logo_canadapost.png';
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'merchantCPCID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'fromPostalCode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'turnAroundTime',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/shipping/canada_post/options.tmpl');
			return $options;
		}
		
		protected function checkAndGetOptions()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (!isset($options['merchantCPCID']))
				throw new scCanadaPostAPIException(_t('Please setup Canada Post merchant ID first.'));

			return $options;
		}
	}
	
	/**
	 * Proxy class to Canada Post web services
	 */
	class scCanadaPostAPI {
		
		const CANADA_POST_API_URL = 'http://sellonline.canadapost.ca:30000';
		
		protected $_totalPrice = array();
		protected $_options = array();
		protected $_shippingItems = array();
		protected $_shippingAddress = array();
		
		/**
		 * 
		 * @param array $options
		 */
		public function setOptions($options)
		{
			$this->_options = $options;	
		}
		
		/**
		 * 
		 * @param scCart $cart
		 */
		public function setProducts(scCart $cart)
		{
			// Canada post expects prices to be in CAD 
			$this->_totalPrice = $cart->getSubtotal()->get(new scCurrency('CAD'));
			
			$products = $cart->getItems();
			foreach ($products as $product) {
				
				$shippingItem = array();
				$shippingItem['quantity'] = $product->getQuantity();
				
				// Canada post expects weights to be in kg
				$shippingItem['weight'] = $product->getWeight()->getWeightInUnits(scWeightUnits::WEIGHT_UNIT_KGS);
				
				// we don't have this data so zeros 
				$shippingItem['length'] = 0;
				$shippingItem['width'] = 0;
				$shippingItem['height'] = 0;
				
				// product description
				$data = $product->getProductInfo();
				$shippingItem['description'] = $data['ProductName'];
				
				// prtoduct image
				$imageInfo = $product->getImageInfo()->getDataArray();
				if ($imageInfo['ImageThumbExists'] == 1) {
					$shippingItem['imageURL'] = $imageInfo['ImageThumb'];
				}
				$this->_shippingItems[] = $shippingItem;
			}
		}
		
		/**
		 * 
		 * @param scAddress $address
		 */
		public function setShippingAddress(scAddress $address)
		{
			$this->_shippingAddress['city'] = $address->getCity();
			$countryCode = scCountry::getCountryCode2($address->getCountryId());
			$this->_shippingAddress['provOrState'] = '';
			if (($countryCode == 'CA') || ($countryCode == 'US')) {
				$englishLangId = scUtil::getDbValue('Languages', 'LanguageID', 'LanguageCode', 'EN');
				$this->_shippingAddress['provOrState'] = scCountry::getStateName($address->getStateId(), $englishLangId);
			}
			$this->_shippingAddress['country'] = $countryCode;
			$this->_shippingAddress['postalCode'] = $address->getPostalCode();
		}
		
		/**
		 * shorthand for generate the XML request, proccess the request and return data
		 */
		public function getRates()
		{
			$xml = $this->generateXmlRequest();
			$response = $this->makeRequest($xml);
			return $this->processResponse($response);
		}
		
		protected function getLanguageCode()
		{
			$currentLanguageCode = scMultiLanguage::singleton()->getCurrentLanguageCode();
			if (strtoupper($currentLanguageCode) == 'FR') {
				return 'fr';
			}
			return 'en';
		}
		
		/**
		 * Generate XML Request
		 *
		 * @param string $request the request operation
		 * @param string|array $values the values of the request
		 * @return string the XML for the request
		 */
		protected function generateXmlRequest()
		{
			// generate shipping items xml
			$shippingItemsXml = '';
	   		foreach ($this->_shippingItems as $item) {
	   			$shippingItemDefinition = array(array(
	   				scXml::TAG => 'item',
	   				scXml::CHILDREN => array(
	   					array(scXml::TAG => 'quantity', scXml::DATA => $item['quantity']),
	   					array(scXml::TAG => 'weight', scXml::DATA => $item['weight']),
	   					
	   					array(scXml::TAG => 'length', scXml::DATA => $item['length']),
	   					array(scXml::TAG => 'width', scXml::DATA => $item['width']),
	   					array(scXml::TAG => 'height', scXml::DATA => $item['height']),
	   					
	   					array(scXml::TAG => 'description', scXml::DATA => $item['description']),
	   					array(scXml::TAG => 'imageURL', scXml::DATA => $item['imageURL']),
	   				),
	   			));
	   			$shippingItemsXml .= scXml::xmlWriter($shippingItemDefinition);
	   		}
	   		
	   		$ratesRequestChildrens = array();
	   		$ratesRequestChildrens[] = array(scXml::TAG => 'merchantCPCID', scXml::DATA => $this->_options['merchantCPCID']);
	   		
	   		// only add this to XML if is set 
	   		if (!empty($this->_options['fromPostalCode'])) {
		   		$ratesRequestChildrens[] = array(scXml::TAG => 'fromPostalCode', scXml::DATA => $this->_options['fromPostalCode']);
	   		}
	   		if (!empty($this->_options['turnAroundTime'])) {
		   		$ratesRequestChildrens[] = array(scXml::TAG => 'turnAroundTime', scXml::DATA => $this->_options['turnAroundTime']);
	   		}
	   		
	   		$ratesRequestChildrens[] = array(scXml::TAG => 'itemsPrice', scXml::DATA => $this->_totalPrice);
	   		$ratesRequestChildrens[] = array(
	   			scXml::TAG => 'lineItems', 
	   			scXml::CHILDREN => array(
					array(scXml::TAG => 'item', scXml::XML => $shippingItemsXml),
				),
			);
	   		
			$ratesRequestChildrens[] = array(scXml::TAG => 'city', scXml::DATA => $this->_shippingAddress['city']);
   			$ratesRequestChildrens[] = array(scXml::TAG => 'provOrState', scXml::DATA => $this->_shippingAddress['provOrState']);
	   		$ratesRequestChildrens[] = array(scXml::TAG => 'country', scXml::DATA => $this->_shippingAddress['country']);
	   		$ratesRequestChildrens[] = array(scXml::TAG => 'postalCode', scXml::DATA => $this->_shippingAddress['postalCode']);
	   		
	   		$definitions = array(
				array(
					scXml::TAG => 'eparcel',
					scXml::CHILDREN => array(
						array(scXml::TAG => 'language', scXml::DATA => $this->getLanguageCode()),
						array(scXml::TAG => 'ratesAndServicesRequest', scXml::CHILDREN => $ratesRequestChildrens),
					),
				),
			);
			return scXml::xmlWriter($definitions);
		}
		
		protected function makeRequest($xml)
		{
			$xmlResponse = scHttp::post(self::CANADA_POST_API_URL, $xml);
			return new scCanadaPostResponse($xmlResponse);
		}
		
		/**
		 * 
		 * @param scCanadaPostResponse $response
		 */
		protected function processResponse(scCanadaPostResponse $response)
		{
			$xml = $response->getSimpleXML()->ratesAndServicesResponse;
			foreach ($xml->product as $xmlEntry) {
				$entry = array();

				$attributes = $xmlEntry->attributes();
				$entry['id'] = (string)$attributes['id'];
				$entry['name'] = (string)$xmlEntry->name;
				$entry['rate'] = (string)$xmlEntry->rate;
				$entry['shippingDate'] = (string)$xmlEntry->shippingDate;
				$entry['deliveryDate'] = (string)$xmlEntry->deliveryDate;
				$entry['deliveryDayOfWeek'] = (string)$xmlEntry->deliveryDayOfWeek;
				$entry['nextDayAM'] = (string)$xmlEntry->nextDayAM;
				$entry['packingID'] = (string)$xmlEntry->packingID;
				$result['Entries'][] = $entry;
			}
			
			return $result;
		}
	}
	
	
	/**
	 * Canada Post response container class
	 */
	class scCanadaPostResponse {
		protected $_responseXml;
		protected $_xml;
		
		public function __construct($responseXml)
		{
			try {
				$this->_responseXml = $responseXml;
				$options = (defined('LIBXML_NOWARNING') and defined('LIBXML_NOERROR')) ? LIBXML_NOWARNING | LIBXML_NOERROR : 0;
				$this->_xml = new SimpleXMLElement($this->_responseXml, $options);
			} catch (Exception $exception) {
				throw new scCanadaPostAPIException($exception->getMessage());
			}
			
			$this->checkReplyForError();
		}
		
		/**
		 * Checks if there is an error in the response
		 * If an error is found scCanadaPostAPIException will be thrown
		 * 
		 * @throws scCanadaPostAPIException if an error is returned in response XML
		 */
		protected function checkReplyForError()
		{
			if (!empty($this->_xml->error)) {
				throw new scCanadaPostAPIException((string)$this->_xml->error->statusMessage, (int)$this->_xml->error->statusCode);
			}
		}
		
		/**
		 * Returns simple XML element object with the response
		 * 
		 * @return SimpleXMLElement the simple xml element object
		 */
		public function getSimpleXML()
		{
			return $this->_xml;
		}
	}
	
	
	/**
	 * CanadaPost API Exception class
	 */
	class scCanadaPostAPIException extends miException {
	}
?>