<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once('econtbg.php');
	
	class scEcontShippingLabelModule implements IShippingLabelModule
	{
		protected $_shippingLabel = null;
		
		public function getShippingLabel($orderId)
		{
			if (is_null($this->_shippingLabel))	{
				$this->_shippingLabel = new scEcontShippingLabel($orderId);
			}
			return $this->_shippingLabel;
		}
		
		public function handlePrintAction($orderId)
		{
			$content = $this->getShippingLabel($orderId)->getContent($orderId);
			$this->outputHeaders($orderId);
			echo $content;
			die;
		}
		
		public function getPrintActionOpenType($orderId)
		{
			return IShippingLabelModule::OPEN_TYPE_BLANK;
		}
		
		protected function outputHeaders($orderId)
		{
			$fileName = $this->getShippingLabel()->getCacheFileName($orderId);
			header('Cache-Control: public'); // needed for i.e.
			header('Content-Type: application/pdf');
			header('Content-Disposition: attachment; filename="' . $fileName . '"');
		}
	}
	
	/**
	 * Econt shipping label
	 */
	class scEcontShippingLabel extends scShippingLabelPdf implements IShippingLabel
	{
		public function getContent($orderId, $params = array())
		{
			$order = new scOrder($orderId);
			
			$fileName = $this->getCacheFileName($orderId);
			
			if ($order->get('OrderShippingTrackingNo')) {
				// check if we have cached content
				if (scShippingLabelCache::checkCacheExists($fileName)) {
					return scShippingLabelCache::getCachedContent($fileName);
				}
			}
			
			// generating new shipping label in econt database and return the tracking number
			$loadingNum = $this->requestLoadingForOrder($orderId);
			
			// get the pdf
			$data = file_get_contents($this->getLoadingPrintURL(array($loadingNum)));
			
			// cache it
			scShippingLabelCache::storeShippingLabelContent($fileName, $data);
			
			// return it
			return $data;
		}
	
		public function getCacheFileName($orderId, $params = array())
		{
			return $orderId . '_' . IShippingLabel::DOCUMENT_TYPE_SLIP . '.' . $this->getExtension();
		}
	
		/**
		 * returns URL for pdf with loadings to print
		 *
		 * @param array $loadingNumbers - loading numbers
		 * @return string
		 */
		protected function getLoadingPrintURL($loadingNumbers)
		{
			$pdfUrl = econtConfig::EECONTURL . '/api/api_pdf_shipment.php?user=' . $this->_options['EEcontUsername'] . '&print_media=double';
			foreach ($loadingNumbers as $loadingNum) {
				$pdfUrl .= '&nums[]=' . $loadingNum;
			}
			return $pdfUrl;
		}
	
		public function getReceiverSharePercent()
		{
			$sharePercent = (float) $this->_options['SenderPayPercent'];
			if ($sharePercent > 100)
				$sharePercent = 100;
			if ($sharePercent < 0)
				$sharePercent = 0;
			return (100 - $sharePercent);
		}
		
		/**
		 * requests a new loading for $orderId and saves the loading number in OrderShippingTrackingNo if successfull
		 *
		 * @param int $orderId
		 * @return string - loading number
		 */
		public function requestLoadingForOrder($orderId, $clientInfo = array())
		{
			$order = new scOrder($orderId);
			$orderRow = $order->getRow();
			if (!empty($orderRow['OrderShippingTrackingNo'])) {
				return $orderRow['OrderShippingTrackingNo']; //throw new miException(_i('Shipment already exists for this order.'));
			}
			$loading = $this->createLoadingForOrder($orderId, $clientInfo);
			$request = $this->buildLoadingsXml(array($loading), false, false);
			
			$response = scEcontBg::doRequest($request, econtConfig::EECONTURL . '/xml_parcel_import.php');
			
			if (strpos($response,'API_ERR_LOGIN') !== false)
				throw new miException(_t('Invalid e-Econt username/password.'));
			
			$xmlResponse = new SimpleXMLElement($response);
			
			if (empty($xmlResponse->result->e->error) && empty($xmlResponse->result->e->error_code) && !empty($xmlResponse->result->e->loading_num)) {
				// Success
				$loadingNum = (string)$xmlResponse->result->e->loading_num;
			} else {
				// Handle error
				if (!empty($xmlResponse->result->e->error_code)) {
					throw new miException(econtConfig::getXmlApiError($xmlResponse));
				} else {
					throw new miException((string)$xmlResponse->result->e->error);
				}
			}
			
			$order->set('OrderShippingTrackingNo', $loadingNum);
			$order->update();
				
			return $loadingNum;
		}
		
		/**
		 * returns a loading array for the order
		 * optional client address - array with keys: Country, City, Zip, Address1, Address2, Company, FirstName, LastName, Phone
		 * @throws miDBException, Exception
		 * @param int $orderId
		 * @param array $clientAddress
		 * @return array - loading
		 */
		public function createLoadingForOrder($orderId, $clientInfo = array())
		{
			$order = new scOrder($orderId);
			$orderRow = $order->getRow();
	
			$requestCurrency = new scCurrency('BGN');
			
			// set client info
			$tmp = array_merge($orderRow,$clientInfo);
				
			if (!empty($clientInfo)) {
				$arrayKeyPrefix = '';
			} else {
				if($orderRow['OrderShipToBillingAddress'])
				$arrayKeyPrefix = 'OrderBilling';
				else
				$arrayKeyPrefix = 'OrderShipping';
			}
			
			// only works for BG
			if (strtoupper($tmp['OrderShippingCountryCode2']) != 'BG')
				throw new miException(_t('Econt shipping labels can be printed only for shipments to Bulgaria.'));
			
			$receiver = array(
					'name' => empty($tmp[$arrayKeyPrefix . 'Company']) ? $tmp[$arrayKeyPrefix . 'FirstName'] . ' ' .  $tmp[$arrayKeyPrefix . 'LastName'] : $tmp[$arrayKeyPrefix . 'Company'],
					'name_person' => (empty($tmp[$arrayKeyPrefix . 'Company'])) ? '' : $tmp[$arrayKeyPrefix . 'FirstName'] . ' ' .  $tmp[$arrayKeyPrefix . 'LastName'],
					'phone_num' => trim($tmp[$arrayKeyPrefix . 'Phone']),
					'city' => trim($tmp[$arrayKeyPrefix . 'City']),
					'post_code' => trim($tmp[$arrayKeyPrefix . 'Zip']),
					'street' => trim($tmp[$arrayKeyPrefix . 'Address1'] . ' ' . $tmp[$arrayKeyPrefix . 'Address2'])
			);
			
			// get oc
			$declaredValueForShipping = $order->getDeclaredValueForShipping($order->getBaseCurrency(), $requestCurrency);
			
			$orderWeight = $order->getTotalWeight(scWeightUnits::WEIGHT_UNIT_KGS);
			
			$loading = array(
				'sender' => array(
					'name' => $this->_options['CompanyName'],
					'name_person' => $this->_options['PersonName'],
					'phone_num' => trim($this->_options['Phone']),
					'city' => trim($this->_options['City']),
					'post_code' => trim($this->_options['Zip']),
					'street' => trim($this->_options['Address1'] . ' ' . $this->_options['Address2'])
				),
				'receiver' => $receiver,
				'shipment' => array(
					'shipment_type' => 'PACK',
					'pack_count' => '1',
					'description' => _t('Order') . ' ' . $order->get('OrderID'),
					'weight' => $orderWeight,
				),
				'payment' => array(
					'side' => 'SENDER',
					'method' => $this->_options['PaymentMethod'],
					'key_word' => $this->_options['CreditNum']
				),
				'services' => array(
					'dc' => ($this->_options['DeliveryReceipt'] == 'YES') ? 'ON' : '',
					'oc' => ($this->_options['OCMinAmount'] <= $declaredValueForShipping) ? $declaredValueForShipping : '',
				)
			);
			
			// CD
			$paymentModuleId = $order->getModuleIdForPayment();
			
			$codModuleName = 'com.summercart.payment.cod';
			$codModuleId = scModulesFactory::getModuleIdByName($codModuleName);
			
			if ($paymentModuleId == $codModuleId) {
				$totalPrice = scCurrencyUtil::convert($order->getBaseCurrencyTotal(),$order->getBaseCurrency()->getCurrencyCode(), $requestCurrency);
				$shippingPrice = 0;
				$CODPrice = 0;
				
				$totalLines = $order->getOrderTotalLines();
				foreach ($totalLines as $totalLine) {
					if ($totalLine->get('OrderTotalLineType') == IOrderTotalModule::ORDER_TOTAL_TYPE_SHIPPING) {
						$shippingPrice += scCurrencyUtil::convert($totalLine->get('OrderTotalLineAmount'), $order->getBaseCurrency()->getCurrencyCode(), $requestCurrency);
					}
					if ($totalLine->get('OrderTotalLineType') == IOrderTotalModule::ORDER_TOTAL_TYPE_SURCHARGE) {
						$CODPrice += scCurrencyUtil::convert($totalLine->get('OrderTotalLineAmount'), $order->getBaseCurrency()->getCurrencyCode(), $requestCurrency);
					}
				}
				$loading['services']['cd'] = $totalPrice - $shippingPrice - $CODPrice;
				
				if (!empty($this->_options['CDAgreementNum']))
					$loading['services']['cd_agreement_num'] = $this->_options['CDAgreementNum'];
				
				// the receiver pays his part of the shipping cost directly when the payment method is CoD
				// if share_percent is 100 the payment side will be converted to RECEIVER and the method to CASH
				$loading['payment']['share_percent'] = $this->getReceiverSharePercent();
			}
			
			// shipment type
			if ($orderRow['OrderShippingMethodCode'] == scEcontBg::ECONT_DELIVERY_TO_OFFICE) {
				if ($this->_options['PickupFrom'] == 'OFFICE') {
					$loading['shipment']['tariff_code'] = 2;
					$loading['shipment']['tariff_sub_code'] = 'OFFICE_OFFICE';
				} else {
					$loading['shipment']['tariff_code'] = 3;
					$loading['shipment']['tariff_sub_code'] = 'DOOR_OFFICE';
				}
			} else {
				if ($this->_options['PickupFrom'] == 'OFFICE') {
					$loading['shipment']['tariff_code'] = 3;
					$loading['shipment']['tariff_sub_code'] = 'OFFICE_DOOR';
				} else {
					if ($loading['sender']['post_code'] == $loading['receiver']['post_code'])
						$tariffCode = 1;
					else
						$tariffCode = 4;
					
					$loading['shipment']['tariff_code'] = $tariffCode;
					$loading['shipment']['tariff_sub_code'] = 'DOOR_DOOR';
				}
			}
			
			return $loading;
		}
	
		/**
		 * Returns loadings request XML
		 *
		 * @param array $loadings
		 * @param bool $validate
		 * @param bool $onlyCalculate
		 * @return string
		 */
		protected function buildLoadingsXml($loadings, $validate = true, $onlyCalculate = true)
		{
			$xmlRequest = new SimpleXMLElement("<?xml version='1.0' encoding='UTF-8'?><parcels></parcels>");
			$xmlRequest->addChild('is_summercart','true');
			$xmlRequest->addChild('system');
			$xmlRequest->system->addChild('validate');
			$xmlRequest->system->validate = ($validate) ? 1 : 0;
			$xmlRequest->system->addChild('response_type');
			$xmlRequest->system->response_type = 'XML';
			$xmlRequest->system->addChild('only_calculate');
			$xmlRequest->system->only_calculate = ($onlyCalculate) ? 1 : 0;
	
			$xmlRequest->addChild('client');
			$xmlRequest->client->addChild('username');
			$xmlRequest->client->username = $this->_options['EEcontUsername'];
			$xmlRequest->client->addChild('password');
			$xmlRequest->client->password = $this->_options['EEcontPassword'];
				
			$xmlRequest->addChild('loadings');
			foreach ($loadings as $loadingInfo) {
				$this->addLoadingToXml($loadingInfo, $xmlRequest->loadings);
			}
				
			return $xmlRequest->asXML();
		}
	
		/**
		 * adds a row to $xmlElement with data from $loading
		 *
		 * @param array $loading
		 * @param SimpleXMLElement $xmlElement
		 */
		protected function addLoadingToXml($loading, SimpleXMLElement $xmlElement)
		{
			if (empty($loading))
				return;
			
			$row = $xmlElement->addChild('row');
			
			// sender
			$row->addChild('sender');
			foreach ($loading['sender'] as $name => $value) {
				$row->sender->addChild($name);
				$row->sender->$name = $value;
			}
			
			// receiver
			$row->addChild('receiver');
			foreach ($loading['receiver'] as $name => $value) {
				$row->receiver->addChild($name);
				$row->receiver->$name = $value;
			}
			
			// shipment
			$row->addChild('shipment');
			foreach ($loading['shipment'] as $name => $value) {
				// truncate package description to 70 chars
				if (($name == 'description') && (strlen(utf8_decode($value)) > 70)) {
					$value = scUtf8::substr($value, 0, 67) . '...';
				}
				$row->shipment->addChild($name);
				$row->shipment->$name = $value;
			}
			
			// payment
			$row->addChild('payment');
			foreach ($loading['payment'] as $name => $value) {
				$row->payment->addChild($name);
				$row->payment->$name = $value;
			}
			
			// services
			$row->addChild('services');
			if (!empty($loading['services']['dc'])) {
				$row->services->addChild('dc');
				$row->services->dc = $loading['services']['dc'];
			}
			if (!empty($loading['services']['oc'])) {
				$row->services->addChild('oc');
				$row->services->oc = $loading['services']['oc'];
			}
			if (!empty($loading['services']['cd'])) {
				$row->services->addChild('cd');
				$row->services->cd = $loading['services']['cd'];
				$row->services->cd->addAttribute('type','GET');
			}
			if (!empty($loading['services']['cd_agreement_num'])) {
				$row->services->addChild('cd_agreement_num');
				$row->services->cd_agreement_num = $loading['services']['cd_agreement_num'];
			}
		}
	}
?>