<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scFedEx implements IShippingModule, IModuleOptions {
		const MIN_WEIGHT = 0.1;
		
		protected $_moduleId;
		protected $_shippingModuleId;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
		
		function getShippingMethods()
		{
			$shippingMethods = array();
			foreach (scFedExAPI::$fedExServices as $serviceTypeId => $serviceName)
				$shippingMethods[$serviceTypeId] = 'FedEx ' . $serviceName;
			return $shippingMethods;
		}
		
		function getShippingRates(scCart $cart, $shippingMethods)
		{
			try {
				$options = $this->checkAndGetOptions();
				$weight = $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_LBS);
				
				$shippingAddress = $cart->getShippingAddress();
				$packagingWeight = new scWeight($options['PackagingWeight'], scWeightUnits::getCurrentWeightUnit());
				$packageInfo = array(
					'OriginCountryCode' => $options['AddressCountryCode'],
					'OriginPostalCode' => $options['AddressPostalCode'],
					'OriginStateOrProvinceCode' => $options['AddressStateOrProvinceCode'],
					
					'DestinationCountryCode' => $shippingAddress->getCountryCode(),
					'DestinationPostalCode' => $shippingAddress->getPostalCode(),
					'DestinationStateOrProvinceCode' => $shippingAddress->getStateCode(),
					
					'Packaging' => 'YOURPACKAGING',
					'WeightUnits' => 'LBS',
					'Weight' => $weight + $packagingWeight->getWeightInUnits(scWeightUnits::WEIGHT_UNIT_LBS)
				);
				
				if ($packageInfo['Weight'] < self::MIN_WEIGHT)
					$packageInfo['Weight'] = self::MIN_WEIGHT;
				
				if (!empty($options['Insurance']))
					$packageInfo['DeclaredValue'] = $cart->getDeclaredValueForShipping(new scCurrency('USD'));
				
				$fedExApi = new scFedExAPI($options['AccountNumber']);
				$fedExApi->setMeterNumber($options['MeterNumber']);
				$serviceRates = $fedExApi->FDXRateAvailableServices($packageInfo);
			} catch (scFedExAPIException $exception) {
				// Pass exception message up
				return array(self::SHIPPING_RATES_MESSAGE => $exception->getMessage() . ' (' . $exception->getCode() . ')');
			}
			
			// Check for SoftError
			$result = array();
			if (isset($serviceRates['SoftError']))
				$result[self::SHIPPING_RATES_MESSAGE] = $serviceRates['SoftError']['Message'] . ' (' . $serviceRates['SoftError']['Code'] . ')';
			
			// Extract rates
			$prices = array();
			foreach ($serviceRates['Entries'] as $entry) {
				$prices[$entry['Service']] = $entry['EstimatedCharges']['DiscountedCharges']['NetCharge'];
			}
			$result[self::SHIPPING_RATES] = $prices;
			$result[self::SHIPPING_RATES_CURRENCY] = 'USD';
			return $result;
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/fedex/logo_fedex.png';
		}
		
		/**
		 * Checks if the module options have been set
		 * Obtains meter number if needed
		 * 
		 * @return array associative array with option values
		 * @throws scFedExAPIException if account number is not set
		 */
		protected function checkAndGetOptions()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (!isset($options['AccountNumber']))
				throw new scFedExAPIException(_t('Please setup FedEx Account Number first.'));
			
			if (!isset($options['MeterNumber']) or strlen($options['MeterNumber'])==0) {
				// Obtain meter number
				$fedExApi = new scFedExAPI($options['AccountNumber']);
				$subscriptionData = array(
					'ContactPersonName' => $options['ContactPersonName'],
					'ContactCompanyName' => $options['ContactCompanyName'],
					'ContactDepartment' => $options['ContactDepartment'],
					'ContactPhoneNumber' => $options['ContactPhoneNumber'],
					'ContactPagerNumber' => $options['ContactPagerNumber'],
					'ContactFaxNumber' => $options['ContactFaxNumber'],
					'ContactE-MailAddress' => $options['ContactE-MailAddress'],
					'AddressLine1' => $options['AddressLine1'],
					'AddressLine2' => $options['AddressLine2'],
					'AddressCity' => $options['AddressCity'],
					'AddressStateOrProvinceCode' => $options['AddressStateOrProvinceCode'],
					'AddressPostalCode' => $options['AddressPostalCode'],
					'AddressCountryCode' => $options['AddressCountryCode'],
				);
				
				$meterNumber = $fedExApi->FDXSubscription($subscriptionData);
				scModuleOptions::setShippingModuleOption($this->_shippingModuleId, 'MeterNumber', $meterNumber);
				$options['MeterNumber'] = $meterNumber;
			}
			return $options;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'AccountNumber',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'MeterNumber',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactPersonName',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactCompanyName',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactDepartment',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactPhoneNumber',
					'validator' => 'scValidatorInt'
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactPagerNumber',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactFaxNumber',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'ContactE-MailAddress',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressLine1',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressLine2',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressCity',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressStateOrProvinceCode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressPostalCode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressCountryCode',
				),
				
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'Insurance',
				),
				array(
					'field' => 'scWeightWidget',
					'data' => 'PackagingWeight',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/shipping/fedex/options.tmpl');
			return $options;
		}
	}
	
	/**
	 * Proxy class to FedEx web services
	 */
	class scFedExAPI {
		const FEDEX_API_URL = 'https://gateway.fedex.com/GatewayDC';
		const FEDEX_API_TEST_URL = 'https://gatewaybeta.fedex.com/GatewayDC';
		
		static public $fedExServices = array(
			'PRIORITYOVERNIGHT' => 'Priority Overnight',
			'STANDARDOVERNIGHT' => 'Standard Overnight',
			'FIRSTOVERNIGHT' => 'First Overnight',
			'FEDEX2DAY' => '2Day',
			'FEDEXEXPRESSSAVER' => 'Express Saver',
			'INTERNATIONALPRIORITY' => 'International Priority',
			'INTERNATIONALECONOMY' => 'International Economy',
			'INTERNATIONALFIRST' => 'International First',
			
			'FEDEX1DAYFREIGHT' => '1Day Freight',
			'FEDEX2DAYFREIGHT' => '2Day Freight',
			'FEDEX3DAYFREIGHT' => '3Day Freight',
			'INTERNATIONALPRIORITYFREIGHT' => 'International Priority Freight',
			'INTERNATIONALECONOMYFREIGHT' => 'International Economy Freight',
//			'EUROPEFIRSTINTERNATIONALPRIORITY' => '',
			
			'FEDEXGROUND' => 'Ground',
			'GROUNDHOMEDELIVERY' => 'Ground Home Delivery',
		);
		
		static public $packageTypes = array('FEDEXENVELOPE', 'FEDEXPAK', 'FEDEXBOX', 'FEDEXTUBE', 'FEDEX10KGBOX', 'FEDEX25KGBOX', 'YOURPACKAGING');
		static public $weightUnits = array('LBS', 'KGS');
		
		protected $_accountNumber;
		protected $_meterNumber;
		
		/**
		 * Constructs the class
		 * 
		 * @param string $accountNumber nine-digit FedEx account number
		 */
		public function __construct($accountNumber)
		{
			$this->_accountNumber = $accountNumber;
		}
		
		/**
		 * Set the FedEx meter number
		 * 
		 * @param string $meterNumber the meter number (up to 10 digits)
		 */
		public function setMeterNumber($meterNumber)
		{
			$this->_meterNumber = $meterNumber;
		}
		
		/**
		 * Makes FDXScubription request
		 * 
		 * @param array $options associative array with request options
		 * @return string meter number
		 * @throws scFedExAPIException if the response is malformed or indicates error
		 */
		public function FDXSubscription($options)
		{
			$xml = $this->getFDXSubscriptionRequest($options);	
			$fedExResponse = $this->sendRequest($xml);
			return $this->processFDXSubscriptionResponse($fedExResponse);
		}
		
		/**
		 * Makes FDXRateAvailableServices request
		 * 
		 * @throws scFedExAPIException if the response is malformed or indicates error
		 * @return
		 */
		public function FDXRateAvailableServices($packageInfo)
		{
			$xml = $this->getFDXRateAvailableServicesRequest($packageInfo);	
			$fedExResponse = $this->sendRequest($xml);
			return $this->processFDXRateAvailableServices($fedExResponse);
		}
		
		/**
		 * Makes FDXRate request
		 * 
		 * @param array $packageInfo associative array with package info
		 * @throws scFedExAPIException if the response is malformed or indicates error
		 */
		public function FDXRate($packageInfo)
		{
			$xml = $this->getFDXRateRequest($packageInfo);	
			$fedExResponse = $this->sendRequest($xml);
			return $this->processFDXRateResponse($fedExResponse);
		}
		
		/**
		 * Processes FDXRate response
		 * 
		 * @param scFedExResponse $fedExResponse
		 * @return array
		 */
		protected function processFDXRateResponse($fedExResponse)
		{
			$xml = $fedExResponse->getSimpleXML();
			
			$estimatedChargesXml = $xml->EstimatedCharges;
			$discountedChargesXml = $xml->EstimatedCharges->DiscountedCharges;
			
			$entry = array();
			$entry['EstimatedCharges'] = array(
				'DimWeightUsed' => (string)$estimatedChargesXml->DimWeightUsed,
				'RateScale' => (string)$estimatedChargesXml->RateScale,
				'RateZone' => (string)$estimatedChargesXml->RateZone,
				'CurrencyCode' => (string)$estimatedChargesXml->CurrencyCode,
				'BilledWeight' => (string)$estimatedChargesXml->BilledWeight,
				'DimWeight' => (string)$estimatedChargesXml->DimWeight,
				'DiscountedCharges' => array(
					'BaseCharge' => (string)$discountedChargesXml->BaseCharge,
					'TotalDiscount' => (string)$discountedChargesXml->TotalDiscount,
					'TotalSurcharge' => (string)$discountedChargesXml->TotalSurcharge,
					'NetCharge' => (string)$discountedChargesXml->NetCharge,
					'TotalRebate' => (string)$discountedChargesXml->TotalRebate,
				)
			);
			$entry['SignatureOption'] = (string)$xml->SignatureOption;
			
			return $entry;
		}
		
		/**
		 * Builds the FDXRate request
		 * 
		 * @param array $packageInfo associative array with package info
		 * @return string the request in xml form
		 */
		protected function getFDXRateRequest($packageInfo)
		{
			$carrierCode = (($packageInfo['Service'] == 'FEDEXGROUND') or ($packageInfo['Service'] == 'GROUNDHOMEDELIVERY')) ? 'FDXG' : 'FDXE';
			$requestValues = array(
				'RequestHeader' => array('AccountNumber' => $this->_accountNumber, 'MeterNumber' => $this->_meterNumber, 'CarrierCode' => $carrierCode),
				'Service' => $packageInfo['Service'],
				'Packaging' => $packageInfo['Packaging'],
				'WeightUnits' => $packageInfo['WeightUnits'],
				'Weight' => number_format($packageInfo['Weight'], 1, '.', ''),
				'OriginAddress' => array(
					'CountryCode' => $packageInfo['OriginCountryCode'],
					'PostalCode' => $packageInfo['OriginPostalCode'],
					'StateOrProvinceCode' => $packageInfo['OriginStateOrProvinceCode']),
				'DestinationAddress' => array(
					'CountryCode' => $packageInfo['DestinationCountryCode'],
					'PostalCode' => $packageInfo['DestinationPostalCode'],
					'StateOrProvinceCode' => $packageInfo['DestinationStateOrProvinceCode']),
				'Payment' => array(),
				'PackageCount' => 1,
				'DeclaredValue' => array('Value' => number_format($packageInfo['DeclaredValue'], 2, '.', ''), 'CurrencyCode' => 'USD'),
			);
			return $this->buildXmlRequest('FDXRateRequest', $requestValues);
		}
		
		/**
		 * Builds the FDXRateAvailableServices
		 * If country is CA or US and if StateOrProvinceCode is passed it will be checked
		 * 
		 * @param array $options associative array with request options
		 * @return string the request in xml form
		 */
		protected function getFDXRateAvailableServicesRequest($packageInfo)
		{
			$requestValues = array(
				'RequestHeader' => array('AccountNumber' => $this->_accountNumber, 'MeterNumber' => $this->_meterNumber),
				'Packaging' => $packageInfo['Packaging'],
				'WeightUnits' => $packageInfo['WeightUnits'],
				'Weight' => number_format($packageInfo['Weight'], 1, '.', ''),
				'OriginAddress' => array(
					'CountryCode' => $packageInfo['OriginCountryCode'],
					'PostalCode' => $packageInfo['OriginPostalCode'],
					'StateOrProvinceCode' => $packageInfo['OriginStateOrProvinceCode']),
				'DestinationAddress' => array(
					'CountryCode' => $packageInfo['DestinationCountryCode'],
					'PostalCode' => $packageInfo['DestinationPostalCode'],
					'StateOrProvinceCode' => $packageInfo['DestinationStateOrProvinceCode']),
				'Payment' => array()
			);
			if (isset($packageInfo['DeclaredValue']))
				$requestValues['DeclaredValue'] = array('Value' => number_format($packageInfo['DeclaredValue'], 2, '.', ''), 'CurrencyCode' => 'USD');
			
			return $this->buildXmlRequest('FDXRateAvailableServicesRequest', $requestValues);
		}
		
		/**
		 * Process the FDXRateAvailableServices response
		 * The result is an associative array:
		 * Entries - array with rate entries
		 * SoftError - if there is was a soft error this will contain Type, Code and Message
		 * 
		 * @param scFedExResponse $fedExResponse
		 * @return array the result with the rate entries and possible SoftError
		 */
		protected function processFDXRateAvailableServices($fedExResponse)
		{
			$xml = $fedExResponse->getSimpleXML();
			
			$result = array('Entries' => array());
			if (isset($xml->SoftError)) {
				$softError['Type'] = (string)$xml->SoftError->Type;
				$softError['Code'] = (string)$xml->SoftError->Code;
				$softError['Message'] = (string)$xml->SoftError->Message;
				$result['SoftError'] = $softError;
			}
			
			foreach ($xml->Entry as $xmlEntry) {
				$estimatedChargesXml = $xmlEntry->EstimatedCharges;
				$discountedChargesXml = $xmlEntry->EstimatedCharges->DiscountedCharges;
				
				$entry = array();
				$entry['Service'] = (string)$xmlEntry->Service;
				$entry['Packaging'] = (string)$xmlEntry->Packaging;
				$entry['DestinationStationID'] = (string)$xmlEntry->DestinationStationID;
				$entry['DeliveryDate'] = (string)$xmlEntry->DeliveryDate;
				$entry['DeliveryDay'] = (string)$xmlEntry->DeliveryDay;
				$entry['EstimatedCharges'] = array(
					'DimWeightUsed' => (string)$estimatedChargesXml->DimWeightUsed,
					'RateScale' => (string)$estimatedChargesXml->RateScale,
					'RateZone' => (string)$estimatedChargesXml->RateZone,
					'CurrencyCode' => (string)$estimatedChargesXml->CurrencyCode,
					'BilledWeight' => (string)$estimatedChargesXml->BilledWeight,
					'DimWeight' => (string)$estimatedChargesXml->DimWeight,
					'DiscountedCharges' => array(
						'BaseCharge' => (string)$discountedChargesXml->BaseCharge,
						'TotalDiscount' => (string)$discountedChargesXml->TotalDiscount,
						'TotalSurcharge' => (string)$discountedChargesXml->TotalSurcharge,
						'NetCharge' => (string)$discountedChargesXml->NetCharge,
						'TotalRebate' => (string)$discountedChargesXml->TotalRebate,
					)
				);
				$entry['SignatureOption'] = (string)$xmlEntry->SignatureOption;
				$result['Entries'][] = $entry;
			}
			return $result;
		}
		
		/**
		 * Process the FDXSubscription response
		 * 
		 * @param scFedExResponse $fedExResponse
		 * @return string meter number
		 */
		protected function processFDXSubscriptionResponse($fedExResponse)
		{
			return (string)$fedExResponse->getSimpleXML()->MeterNumber;
		}
		
		/**
		 * Builds the FDXSubscriptionRequest
		 * 
		 * @param array $options associative array with request options
		 * @return string the request in xml form
		 */
		protected function getFDXSubscriptionRequest($options)
		{
			$requestValues = array(
				'RequestHeader' => array('AccountNumber' => $this->_accountNumber),
				'Contact' => array(),
				'Address' => array(),
			);
			
			$contactOptions = array(
				'ContactPersonName' => 'PersonName',
				'ContactCompanyName' => 'CompanyName',
				'ContactDepartment' => 'Department',
				'ContactPhoneNumber' => 'PhoneNumber',
				'ContactPagerNumber' => 'PagerNumber',
				'ContactFaxNumber' => 'FaxNumber',
				'ContactE-MailAddress' => 'E-MailAddress',
			);
			foreach ($options as $option => $value) {
				if (isset($contactOptions[$option]) and $value)
					$requestValues['Contact'][$contactOptions[$option]] = $value;
			}
			
			$addressOptions = array(
				'AddressLine1' => 'Line1',
				'AddressLine2' => 'Line2',
				'AddressCity' => 'City',
				'AddressStateOrProvinceCode' => 'StateOrProvinceCode',
				'AddressPostalCode' => 'PostalCode',
				'AddressCountryCode' => 'CountryCode',
			);
			foreach ($options as $option => $value) {
				if (isset($addressOptions[$option]) and $value)
					$requestValues['Address'][$addressOptions[$option]] = $value;
			}
			
			return $this->buildXmlRequest('FDXSubscriptionRequest', $requestValues);
		}
		
		/**
		 * Send request to FedEx and receive response
		 * 
		 * @param string $xml the request xml
		 * @return scFedExResponse the response
		 */
		protected function sendRequest($xml)
		{
			$xmlResponse = scHttp::post(self::FEDEX_API_URL, $xml);
			return new scFedExResponse($xmlResponse);
		}
		
		/**
		 * Build XML Request
		 *
		 * @param string $request the request operation
		 * @param string|array $values the values of the request
		 * @return string the XML for the request
		 */
		protected function buildXmlRequest($request, $values)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$xml .= '<' . $request .' xmlns:api="http://www.fedex.com/fsmapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="' . $request . '.xsd">';
			$xml .= scXml::buildXmlValues($values);
			$xml .= '</' . $request . '>';
			return $xml;
		}
	}
	
	/**
	 * FedEx response container class
	 */
	class scFedExResponse {
		protected $_responseXml;
		protected $_xml;
		
		public function __construct($responseXml)
		{
			try {
				$this->_responseXml = $responseXml;
				
				$options = (defined('LIBXML_NOWARNING') and defined('LIBXML_NOERROR')) ? LIBXML_NOWARNING | LIBXML_NOERROR : 0;
				$this->_xml = new SimpleXMLElement($this->_responseXml, $options);
			} catch (Exception $exception) {
				throw new scFedExAPIException($exception->getMessage());
			}
			
			$this->checkReplyForError();
		}
		
		/**
		 * Checks if there is an error in the response
		 * If an error is found scFedExAPIException will be thrown
		 * 
		 * @throws scFedExAPIException if an error is returned in response XML
		 */
		protected function checkReplyForError()
		{
			if ($this->_xml->getName() == 'Error')
				throw new scFedExAPIException((string)$this->_xml->Message, (int)$this->_xml->Code);
			if (isset($this->_xml->Error))
				throw new scFedExAPIException((string)$this->_xml->Error->Message, (int)$this->_xml->Error->Code);
		}
		
		/**
		 * Returns simple XML element object with the response
		 * 
		 * @return SimpleXMLElement the simple xml element object
		 */
		public function getSimpleXML()
		{
			return $this->_xml;
		}
	}
	
	/**
	 * FedEx API Exception class
	 */
	class scFedExAPIException extends miException {
	}
?>