<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * UPS Shipping module
	 */
	class scUPS implements IShippingModule, IModuleOptions {
		const MIN_WEIGHT = 0.1;
		
		protected $_moduleId;
		protected $_shippingModuleId;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
		
		function getShippingMethods()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			$addressCountryCode = isset($options['AddressCountryCode']) ? $options['AddressCountryCode'] : '';
			return scUPSAPI::getServices($addressCountryCode);
		}
		
		function getShippingRates(scCart $cart, $shippingMethods)
		{
			try {
				$options = $this->checkAndGetOptions();
				
				$packagingWeight = new scWeight($options['PackagingWeight'], scWeightUnits::getCurrentWeightUnit());
				if (in_array($options['AddressCountryCode'], array('', 'DO', 'PR', 'US'))) {
					$weight = $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_LBS) + $packagingWeight->getWeightInUnits(scWeightUnits::WEIGHT_UNIT_LBS);
					$weightUnit = 'LBS';
				} else {
					$weight = $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_KGS) + $packagingWeight->getWeightInUnits(scWeightUnits::WEIGHT_UNIT_KGS);
					$weightUnit = 'KGS';
				}
				if ($weight < self::MIN_WEIGHT)
					$weight = self::MIN_WEIGHT;
				
				$upsApi = new scUPSAPI($options['UserID'], $options['Password'], $options['Server']);
				$upsApi->setAccessNumber($options['AccessNumber']);
				
				$shippingAddress = $cart->getShippingAddress();
				$packageInfo = array(
					'OriginCity' => $options['AddressCity'],
					'OriginStateOrProvinceCode' => $options['AddressStateOrProvinceCode'],
					'OriginPostalCode' => $options['AddressPostalCode'],
					'OriginCountryCode' => $options['AddressCountryCode'],
					
					'DestinationCity' => $shippingAddress->getCity(),
					'DestinationCountryCode' => $shippingAddress->getCountryCode(),
					'DestinationPostalCode' => $shippingAddress->getPostalCode(),
					'DestinationStateOrProvinceCode' => $shippingAddress->getStateCode(),
					
					'PickupType' => $options['PickupType'],
					'CustomerClassification' => $options['CustomerClassification'],
					
					'WeightUnits' => $weightUnit,
					'Weight' => $weight,
					
//					'AccountNumber' => $options['AccountNumber']
				);
				if (!empty($options['Insurance']))
					$packageInfo['DeclaredValue'] = $cart->getDeclaredValueForShipping(new scCurrency('USD'));
				$ratedShipments = $upsApi->RatingServiceSelection($packageInfo);
				
				// Extract rates
				$prices = array();
				$currency = '';
				$warnings = array();
				foreach ($ratedShipments['RatedShipment'] as $ratedShipment) {
					$currency = $ratedShipment['TotalChargesCurrencyCode'];
					$prices[$ratedShipment['ServiceCode']] = $ratedShipment['TotalCharges'];
					if (isset($ratedShipment['RatedShipmentWarning'])) {
						// Ignore 'invoice may vary warning'
						if ($ratedShipment['RatedShipmentWarning'] != 'Your invoice may vary from the displayed reference rates')
							$warnings[] = $ratedShipment['RatedShipmentWarning'];
					}
				}
				
				$result = array();
				$result[self::SHIPPING_RATES] = $prices;
				$result[self::SHIPPING_RATES_CURRENCY] = $currency;
				
				$warnings = array_unique($warnings);
				if (count($warnings))
					$result[self::SHIPPING_RATES_MESSAGE] = implode(', ', $warnings);
				
				return $result;
			} catch (scUPSAPIException $exception) {
				// Pass exception message up
				return array(self::SHIPPING_RATES_MESSAGE => $exception->getMessage() . ' (' . $exception->getCode() . ')');
			}
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/ups/logo_ups.png';
		}
		
		protected function checkAndGetOptions()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (empty($options['UserID']) or empty($options['Password']) or empty($options['AccessNumber']))
				throw new scUPSAPIException(_t('Please setup UPS UserID, Password and Access Number first.'));
			return $options;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'UserID',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'Password',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AccessNumber',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Server',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressCity',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressStateOrProvinceCode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressPostalCode',
				),
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressCountryCode',
				),
				
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'Insurance',
				),
				array(
					'field' => 'scWeightWidget',
					'data' => 'PackagingWeight',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'PickupType',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'PackageType',
				),
				array(
					'field' => 'miSelectWidget',
					'data' => 'CustomerClassification',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/shipping/ups/options.tmpl');
			
			$options->getOptionField('Server')->setRadioButtons(
				array(scUPSAPI::UPS_SERVER_PRODUCTION => _t('Production'), scUPSAPI::UPS_SERVER_TEST => _t('Test')), scUPSAPI::UPS_SERVER_TEST);
			
			$options->getOptionField('PickupType')->setOptions(scUPSAPI::getPickupTypes(), scUPSAPI::PICKUP_TYPE_ONE_TIME);
			$options->getOptionField('PackageType')->setOptions(scUPSAPI::getPackageTypes(), scUPSAPI::PACKAGE_TYPE_PACKAGE);
			$customerClassifications = array('' => 'Automatic') + scUPSAPI::getCustomerClassification();
			$options->getOptionField('CustomerClassification')->setOptions($customerClassifications, '');
			return $options;
		}
	}
	
	/**
	 * Proxy class to UPS web services
	 */
	class scUPSAPI {
		const XPCI_VERSION = '1.0001';
		const UPS_SERVER_TEST = 'T';
		const UPS_SERVER_PRODUCTION = 'P';
		const UPS_SERVER_TEST_URL = 'https://wwwcie.ups.com/ups.app/xml/';
		const UPS_SERVER_PRODUCTION_URL = 'https://www.ups.com/ups.app/xml/';
		
		const PICKUP_TYPE_DAILY = '01';
		const PICKUP_TYPE_CUSTOMER_COUNTER = '03';
		const PICKUP_TYPE_ONE_TIME = '06';
		const PICKUP_TYPE_ON_CALL_AIR = '07';
		const PICKUP_TYPE_SUGGESTED_RATAIL_PRICES = '11';
		const PICKUP_TYPE_LETTER_CENTER = '19';
		const PICKUP_TYPE_AIR_SERVICE_CENTER = '20';
		
		const CUSTOMER_CLASSIFICATION_WHOLESALE = '01';
		const CUSTOMER_CLASSIFICATION_OCCASIONAL = '03';
		const CUSTOMER_CLASSIFICATION_RETAIL = '04';
		
		const PACKAGE_TYPE_UPS_LETTER = '01';
		const PACKAGE_TYPE_PACKAGE = '02';
		const PACKAGE_TYPE_UPS_TUBE = '03';
		const PACKAGE_TYPE_UPS_PAK = '04';
		const PACKAGE_TYPE_UPS_EXPRESS_BOX = '21';
		const PACKAGE_TYPE_UPS_25KG_BOX = '24';
		const PACKAGE_TYPE_UPS_10KG_BOX = '25';
		
		protected $_userId;
		protected $_password;
		protected $_accessNumber;
		protected $_server;
		
		/**
		 * Constructs the class
		 * 
		 * @param string $userId UPS user id
		 * @param string $$password UPS password
		 */
		public function __construct($userId, $password, $server = self::UPS_SERVER_TEST)
		{
			$this->_userId = $userId;
			$this->_password = $password;
			$this->_server = $server;
		}
		
		/**
		 * Sets the UPS access number
		 * 
		 * @param string $accessNumber
		 */
		public function setAccessNumber($accessNumber)
		{
			$this->_accessNumber = $accessNumber;
		}
		
		/**
		 * Perform rating service request
		 * 
		 * @param array $packageInfo
		 * @return array
		 */
		public function RatingServiceSelection($packageInfo)
		{
			$xml = $this->buildAccessRequest();
			$xml .= $this->buildRatingServiceSelectionRequest($packageInfo);
			$response = $this->sendRequest($xml, 'Rate');
			return $this->processRatingServiceSelection($response);
		}
		
		/**
		 * Build the access request XML
		 * 
		 * @return string the XML
		 */
		protected function buildAccessRequest()
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$values = array('AccessRequest' => array(
				'AccessLicenseNumber' => $this->_accessNumber,
				'UserId' => $this->_userId,
				'Password' => $this->_password
			));
			$xml .= scXml::buildXmlValues($values);
			return $xml;
		}
		
		/**
		 * Build the rating service selection request XML
		 * 
		 * @param array $packageInfo
		 * @return string the XML
		 */
		protected function buildRatingServiceSelectionRequest($packageInfo)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$values = array('RatingServiceSelectionRequest' => array(
				'Request' => array(
					'TransactionReference' => array('CustomerContext' => self::XPCI_VERSION),
					'RequestAction' => 'Rate',
					'RequestOption' => 'shop',),
				'PickupType' => array('Code' => $packageInfo['PickupType']),
				'Shipment' => array(
					'Shipper' => array('Address' => array(
						'City' => $packageInfo['OriginCity'],
						'StateProvinceCode' => $packageInfo['OriginStateOrProvinceCode'],
						'PostalCode' => $packageInfo['OriginPostalCode'],
						'CountryCode' => $packageInfo['OriginCountryCode'])),
					'ShipTo' => array('Address' => array(
						'City' => $packageInfo['DestinationCity'],
						'StateProvinceCode' => $packageInfo['DestinationStateOrProvinceCode'],
						'PostalCode' => $packageInfo['DestinationPostalCode'],
						'CountryCode' => $packageInfo['DestinationCountryCode'])),
					'Package' => array(
						'PackagingType' => array('Code' => '02', 'Description' => 'Package'),
						'Description' => 'Rate Shopping',
						'PackageWeight' => array('Weight' => $packageInfo['Weight'], 'UnitOfMeasurement' => array('Code' => $packageInfo['WeightUnits']))))
				));
			if (isset($packageInfo['DeclaredValue']))
				$values['RatingServiceSelectionRequest']['Shipment']['Package']['PackageServiceOptions']['InsuredValue'] =
					array('MonetaryValue' => number_format($packageInfo['DeclaredValue'], 2, '.', ''), 'CurrencyCode' => 'USD');
			
			// For negotiated rates
			if (isset($packageInfo['AccountNumber'])) {
				$values['RatingServiceSelectionRequest']['Shipment']['Shipper']['ShipperNumber'] = $packageInfo['AccountNumber'];
				$values['RatingServiceSelectionRequest']['Shipment']['RateInformation']['NegotiatedRatesIndicator'] = '1';
			}
			
			if (!empty($packageInfo['CustomerClassification']))
				$values['RatingServiceSelectionRequest']['CustomerClassification']['Code'] = $packageInfo['CustomerClassification'];
			$xml .= scXml::buildXmlValues($values);
			return $xml;
		}
		
		/**
		 * Processes the rating service selection response
		 * 
		 * @param scUPSResponse the UPS response object
		 * @return array the response
		 */
		protected function processRatingServiceSelection($upsResponse)
		{
			$xml = $upsResponse->getSimpleXML();
			
			$result = array('RatedShipment' => array());
			foreach ($xml->RatedShipment as $xmlEntry) {
				$ratedShipment = array();
				$ratedShipment['ServiceCode'] = (string)$xmlEntry->Service->Code;
				if (isset($xmlEntry->RatedShipmentWarning))
					$ratedShipment['RatedShipmentWarning'] = (string)$xmlEntry->RatedShipmentWarning;
				
				$ratedShipment['Weight'] = (string)$xmlEntry->BillingWeight->Weight;
				if (isset($xmlEntry->BillingWeight->UnitOfMeasurement) and isset($xmlEntry->BillingWeight->UnitOfMeasurement->Code))
					$ratedShipment['WeightUnit'] = (string)$xmlEntry->BillingWeight->UnitOfMeasurement->Code;
				
				$ratedShipment['TransportationCharges'] = (string)$xmlEntry->TransportationCharges->MonetaryValue;
				if (isset($xmlEntry->TransportationCharges->CurrencyCode))
					$ratedShipment['TransportationChargesCurrencyCode'] = (string)$xmlEntry->TransportationCharges->CurrencyCode;
				
				$ratedShipment['ServiceOptionsCharges'] = (string)$xmlEntry->ServiceOptionsCharges->MonetaryValue;
				if (isset($xmlEntry->ServiceOptionsCharges->CurrencyCode))
					$ratedShipment['ServiceOptionsChargesCurrencyCode'] = (string)$xmlEntry->ServiceOptionsCharges->CurrencyCode;
				
				$ratedShipment['TotalCharges'] = (string)$xmlEntry->TotalCharges->MonetaryValue;
				if (isset($xmlEntry->TotalCharges->CurrencyCode))
					$ratedShipment['TotalChargesCurrencyCode'] = (string)$xmlEntry->TotalCharges->CurrencyCode;
				
				$ratedShipment['GuaranteedDaysToDelivery'] = (string)$xmlEntry->GuaranteedDaysToDelivery;
				$result['RatedShipment'][] = $ratedShipment;
			}
			return $result;
		}
		
		/**
		 * Returns array with all services
		 * Several services have different names depending on the origin country
		 * 
		 * @param string the origin country code
		 * @return array service names array(serviceId => serviceName)
		 */
		public static function getServices($originCountryCode)
		{
			$services = array(
				'01' => 'UPS Next Day Air',
				'02' => 'UPS 2nd Day Air',
				'03' => 'UPS Ground',
				'07' => 'UPS Worldwide Express',
				'08' => 'UPS Worldwide Expedited',
				'11' => 'UPS Standard',
				'12' => 'UPS 3-Day Select',
				'13' => 'UPS Next Day Air Saver',
				'14' => 'UPS Next Day Air Early A.M.',
				'54' => 'UPS Worldwide Express Plus',
				'59' => 'UPS 2nd Day Air A.M.',
				'65' => 'UPS Saver',
				
				/*
				// Poland domestic only (Stolica)
				'82' => 'UPS Today Standard',
				'83' => 'UPS Today Dedicated Courrier',
				'84' => 'UPS Today Intercity',
				'85' => 'UPS Today Express',
				'86' => 'UPS Today Express Saver',
				*/
			);
			
			// Exceptions - codes 01, 07, 08, 13, 14 and 54

			// Canada
			if ($originCountryCode == 'CA') {
				$services['01'] = 'UPS Express';
				$services['13'] = 'UPS Saver';
				$services['14'] = 'UPS Express Early A.M.';
			}
			
			// Mexico
			if ($originCountryCode == 'MX') {
				$services['08'] = 'UPS Expedited';
				$services['54'] = 'UPS Express Plus';
			}
			
			if (($originCountryCode != 'US') and ($originCountryCode != 'PR') and ($originCountryCode != 'CA'))
				$services['07'] = 'UPS Express';
			
			return $services;
		}
		
		/**
		 * Get the pickup types
		 * 
		 * @return array returns the pickup types in array(pickupTypeCode => pickupTypeName)
		 */
		static public function getPickupTypes()
		{
			 return array(
			 	self::PICKUP_TYPE_DAILY => 'Daily Pickup',
			 	self::PICKUP_TYPE_CUSTOMER_COUNTER => 'Customer Counter',
			 	self::PICKUP_TYPE_ONE_TIME => 'One Time Pickup',
			 	self::PICKUP_TYPE_ON_CALL_AIR => 'On Call Air Pickup®',
			 	self::PICKUP_TYPE_SUGGESTED_RATAIL_PRICES => 'Suggested Retail Rates (UPS Store)',
			 	self::PICKUP_TYPE_LETTER_CENTER => 'Letter Center',
			 	self::PICKUP_TYPE_AIR_SERVICE_CENTER => 'Air Service Center',
			 );
		}
		
		/**
		 * Get the customer classifications
		 * 
		 * @return array returns the customer classifications in array(customerClassificationCode => customerClassificationName)
		 */
		static public function getCustomerClassification()
		{
			 return array(
			 	self::CUSTOMER_CLASSIFICATION_WHOLESALE => 'Wholesale',
			 	self::CUSTOMER_CLASSIFICATION_OCCASIONAL => 'Occasional',
			 	self::CUSTOMER_CLASSIFICATION_RETAIL => 'Retail',
			 );
		}
		
		/**
		 * Get the package types
		 * 
		 * @return array returns the package types in array(packageTypeCode => packageTypeName)
		 */
		static public function getPackageTypes()
		{
			 return array(
			 	self::PACKAGE_TYPE_PACKAGE => 'Your Packaging',
			 	self::PACKAGE_TYPE_UPS_LETTER => 'UPS Letter',
			 	self::PACKAGE_TYPE_UPS_TUBE => 'UPS Tube',
			 	self::PACKAGE_TYPE_UPS_PAK => 'UPS Pak',
			 	self::PACKAGE_TYPE_UPS_EXPRESS_BOX => 'UPS Express Box',
			 	self::PACKAGE_TYPE_UPS_25KG_BOX => 'UPS 25Kg Box®',
			 	self::PACKAGE_TYPE_UPS_10KG_BOX => 'UPS 10Kg Box®',
			 );
		}
		
		/**
		 * Send request to UPS and receive response
		 * 
		 * @param string $xml the request xml
		 * @return scUPSResponse the response
		 */
		protected function sendRequest($xml, $path)
		{
			$url = $this->_server == self::UPS_SERVER_PRODUCTION ? self::UPS_SERVER_PRODUCTION_URL : self::UPS_SERVER_TEST_URL;
			$xmlResponse = scHttp::post($url . $path, $xml);
			return new scUPSResponse($xmlResponse);
		}
	}
	
	/**
	 * UPS response container class
	 */
	class scUPSResponse {
		protected $_responseXml;
		protected $_xml;
		
		public function __construct($responseXml)
		{
			try {
				$this->_responseXml = $responseXml;
				
				$options = (defined('LIBXML_NOWARNING') and defined('LIBXML_NOERROR')) ? LIBXML_NOWARNING | LIBXML_NOERROR : 0;
				$this->_xml = new SimpleXMLElement($this->_responseXml, $options);
			} catch (Exception $exception) {
				throw new scUPSAPIException($exception->getMessage());
			}
			
			$this->checkReplyForError();
		}
		
		/**
		 * Checks if the request failed
		 * If an error is found scUPSAPIException will be thrown
		 * 
		 * @throws scUPSAPIException if an error is returned in response XML
		 */
		protected function checkReplyForError()
		{
			if ((int)$this->_xml->Response->ResponseStatusCode == 0) {
				if (isset($this->_xml->Response->Error)) {
					$message = sprintf('%s (Error severity: %s)', $this->_xml->Response->Error->ErrorDescription, $this->_xml->Response->Error->ErrorSeverity);
					throw new scUPSAPIException($message, (int)$this->_xml->Response->Error->ErrorCode);
				}
				throw new scUPSAPIException((string)$this->_xml->Response->ResponseStatusDescription, (int)$this->_xml->Response->ResponseStatusCode);
			}
		}
		
		/**
		 * Returns simple XML element object with the response
		 * 
		 * @return SimpleXMLElement the simple xml element object
		 */
		public function getSimpleXML()
		{
			return $this->_xml;
		}
	}
	
	/**
	 * UPS API Exception class
	 */
	class scUPSAPIException extends miException {
	}
?>