<?php
	/**
	 * Copyright (c) 2008 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/usps_countries.php');

	class scUSPS implements IShippingModule, IModuleOptions {
		protected $_moduleId;
		protected $_shippingModuleId;
		
		public function __construct($moduleId, $shippingModuleId)
		{
			$this->_moduleId = $moduleId;
			$this->_shippingModuleId = $shippingModuleId;
		}
		
		function getShippingMethods()
		{
			return scUSPSAPI::getServices();
		}
		
		function getShippingRates(scCart $cart, $shippingMethods)
		{
			try {
				$options = $this->checkAndGetOptions();
				
				$packagingWeight = new scWeight($options['PackagingWeight'], scWeightUnits::getCurrentWeightUnit());
				$weight = $cart->getTotalWeight(scWeightUnits::WEIGHT_UNIT_LBS) +
					$packagingWeight->getWeightInUnits(scWeightUnits::WEIGHT_UNIT_LBS);
				
				$shippingAddress = $cart->getShippingAddress();
				$packageInfo = array(
					'OriginPostalCode' => $options['AddressPostalCode'],
					
					'DestinationCountryCode' => $shippingAddress->getCountryCode(),
					'DestinationPostalCode' => $shippingAddress->getPostalCode(),
					
					'Weight' => $weight,
				);
				if (!empty($options['Insurance']))
					$packageInfo['ValueOfContents'] = $cart->getDeclaredValueForShipping(new scCurrency('USD'));
				
				$uspsApi = new scUSPSAPI($options['UserID'], $options['Server']);
				$postage = $uspsApi->Rate($packageInfo);
				
				// We will select the lowest rate when a single service class has more than one rate (like First-Class)
				$prices = array();
				foreach ($postage['Postage'] as $postageService) {
					if (isset($prices[$postageService['ClassID']]) and $prices[$postageService['ClassID']] < $postageService['Rate'])
						continue;
					
					$prices[$postageService['ClassID']] = $postageService['Rate'];
					
					if (isset($postageService['Insurance']))
						$prices[$postageService['ClassID']] += $postageService['Insurance'];
					
					// NOTE: InsComment might contain info about insurance
				}
				
				$result = array();
				$result[self::SHIPPING_RATES] = $prices;
				$result[self::SHIPPING_RATES_CURRENCY] = 'USD';
				return $result;
			} catch (scUSPSAPIException $exception) {
				// Pass exception message up
				return array(self::SHIPPING_RATES_MESSAGE => $exception->getMessage() . ' (' . $exception->getUSPSCode() . ')');
			}
		}
		
		public function getShippingModuleImage()
		{
			return 'skins/default/modules/shipping/usps/logo_usps.png';
		}
		
		protected function checkAndGetOptions()
		{
			$options = scModuleOptions::getShippingModuleOptions($this->_shippingModuleId);
			if (empty($options['UserID']))
				throw new scUSPSAPIException(_t('Please setup USPS UserID first.'));
			return $options;
		}
		
		public function getOptionFields(miWebForm &$webForm)
		{
			$fieldsDescriptions = array(
				array(
					'field' => 'miTextWidget',
					'data' => 'UserID',
				),
				array(
					'field' => 'miRadioWidget',
					'data' => 'Server',
				),
				
				array(
					'field' => 'miTextWidget',
					'data' => 'AddressPostalCode',
				),
				
				array(
					'field' => 'miCheckboxWidget',
					'data' => 'Insurance',
				),
				array(
					'field' => 'scWeightWidget',
					'data' => 'PackagingWeight',
				),
			);
			
			$options = new scModuleOptionsInfo();
			$options->buildOptionFields($fieldsDescriptions, $webForm);
			$options->setTemplateFile('modules/shipping/usps/options.tmpl');
			
			$options->getOptionField('Server')->setRadioButtons(
				array(scUSPSAPI::USPS_SERVER_PRODUCTION => _t('Production'), scUSPSAPI::USPS_SERVER_TEST => _t('Test')), scUSPSAPI::USPS_SERVER_TEST);
			return $options;
		}
	}
	
	/**
	 * Proxy class to USPS web services
	 */
	class scUSPSAPI {
		const USPS_SERVER_TEST = 'T';
		const USPS_SERVER_PRODUCTION = 'P';
		const USPS_SERVER_TEST_URL = 'http://testing.shippingapis.com/ShippingAPITest.dll';
		const USPS_SERVER_PRODUCTION_URL = 'http://production.shippingapis.com/ShippingAPI.dll';
		
		const USPS_DOMESTIC_RATE_API = 'RateV3';
		const USPS_INTERNATIONAL_RATE_API = 'IntlRate';
		
		protected $_userId;
		protected $_server;
		
		public function __construct($userId, $server = self::USPS_SERVER_TEST)
		{
			$this->_userId = $userId;
			$this->_server = $server;
		}
		
		/**
		 * Perform rate request
		 * There could be more than one rate for each ClassID. Specifcally First-Class mail returns up to 3 rates:
		 * First-Class Mail, First-Class Mail Flat, First-Class Mail Parcel
		 * 
		 * @param array $packageInfo
		 * @return array
		 */
		public function Rate($packageInfo)
		{
			if ($this->isDomestic($packageInfo)) {
				$xml = $this->buildDomesticRateRequest($packageInfo);
				$response = $this->sendRequest($xml, self::USPS_DOMESTIC_RATE_API);
				return $this->processDomesticRate($response);
			} else {
				$xml = $this->buildInternationalRateRequest($packageInfo);
				$response = $this->sendRequest($xml, self::USPS_INTERNATIONAL_RATE_API);
				return $this->processInternationalRate($response);
			}
		}
		
		/**
		 * Returns if the shippment is domestic
		 * 
		 * The following are domestic according to DMM 608:
		 * AS - American Samoa
		 * GU - Guam
		 * MP - Northern Mariana Islands, Commonwealth of the
		 * PR - Puerto Rico, Commonwealth of
		 * VI - U.S. Virgin Islands
		 * 
		 * Mail Treated as Domestic:
		 * PW - Palau, Republic of
		 * FM - Micronesia, Federated States of
		 * MH - Marshall Islands, Republic of the
		 */
		protected function isDomestic($packageInfo)
		{
			$countryCodes = array('US', 'AS', 'GU', 'MP', 'PR', 'VI', 'PW', 'FM', 'MH');
			return in_array($packageInfo['DestinationCountryCode'], $countryCodes);
		}
		
		/**
		 * Build the rate request XML
		 * 
		 * @param array $packageInfo
		 * @return string the XML
		 */
		protected function buildDomesticRateRequest($packageInfo)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$values = array('RateV3Request' => array(
				'<attributes>' => array('USERID' => $this->_userId),
				'Package' => array(
					'<attributes>' => array('ID' => 0),
					'Service' => 'ALL',
					'ZipOrigination' => $packageInfo['OriginPostalCode'],
					'ZipDestination' => $packageInfo['DestinationPostalCode'],
					'Pounds' => '0',
					'Ounces' => intval($packageInfo['Weight'] * 16),	// Weight is in LBS
					'Size' => 'REGULAR',
					'Machinable' => 'false')
				));
			$xml .= scXml::buildXmlValues($values);
			return $xml;
		}
		
		protected function buildInternationalRateRequest($packageInfo)
		{
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>';
			$values = array('IntlRateRequest' => array(
				'<attributes>' => array('USERID' => $this->_userId),
				'Package' => array(
					'<attributes>' => array('ID' => 0),
					'Pounds' => '0',
					'Ounces' => intval($packageInfo['Weight'] * 16),	// Weight is in LBS
					'MailType' => 'Package')
				));
			if (isset($packageInfo['ValueOfContents']))
				$values['IntlRateRequest']['Package']['ValueOfContents'] = $packageInfo['ValueOfContents'];
			
			$countryName = scUSPSCountries::getCountryName($packageInfo['DestinationCountryCode']);
			if ($countryName == '')
				throw new scUSPSAPIException(sprintf(_t('USPS cannot ship to this country. Country code "%s".'), $packageInfo['DestinationCountryCode']));
			$values['IntlRateRequest']['Package']['Country'] = $countryName;
			$xml .= scXml::buildXmlValues($values);
			return $xml;
		}
		
		
		/**
		 * Processes the domestic rate response
		 * 
		 * @param scUSPSResponse the USPS response object
		 * @return array the response
		 */
		protected function processDomesticRate($uspsResponse)
		{
			$xml = $uspsResponse->getSimpleXML();
			
			$result = array('Postage' => array());
			foreach ($xml->Package->Postage as $xmlEntry) {
				$postage = array();
				$postage['ClassID'] = (string)$xmlEntry['CLASSID'];
				$postage['MailService'] = (string)$xmlEntry->MailService;
				$postage['SvcCommitments'] = (string)$xmlEntry->SvcCommitments;
				$postage['Rate'] = (string)$xmlEntry->Rate;
				if (isset($xmlEntry->Restrictions))
					$postage['Restrictions'] = (string)$xmlEntry->Restrictions;
				
				$result['Postage'][] = $postage;
			}
			return $result;
		}
		
		/**
		 * Processes the international rate response
		 * 
		 * @param scUSPSResponse the USPS response object
		 * @return array the response
		 */
		protected function processInternationalRate($uspsResponse)
		{
			$xml = $uspsResponse->getSimpleXML();
			
			$result = array('Postage' => array());
			foreach ($xml->Package->Service as $xmlEntry) {
				$postage = array();
				$postage['ClassID'] = 'I' . (string)$xmlEntry['ID'];
				$postage['MailService'] = (string)$xmlEntry->SvcDescription;
				$postage['Rate'] = (string)$xmlEntry->Postage;
				if (isset($xmlEntry->Insurance))
					$postage['Insurance'] = (string)$xmlEntry->Insurance;
				if (isset($xmlEntry->InsComment))
					$postage['InsComment'] = (string)$xmlEntry->InsComment;
				if (isset($xmlEntry->Restrictions))
					$postage['Restrictions'] = (string)$xmlEntry->Restrictions;
				
				$result['Postage'][] = $postage;
			}
			return $result;
		}
		
		/**
		 * Send request to USPS and receive response
		 * 
		 * @param string $xml the request xml
		 * @return scUSPSResponse the response
		 */
		protected function sendRequest($xml, $api)
		{
			$url = ($this->_server == self::USPS_SERVER_PRODUCTION) ? self::USPS_SERVER_PRODUCTION_URL : self::USPS_SERVER_TEST_URL;
			$url .= '?API=' . urlencode($api) . '&XML=' . urlencode($xml);
			$xmlResponse = scHttp::get($url);
			return new scUSPSResponse($xmlResponse);
		}
		
		/**
		 * Returns USPS services
		 * 
		 * @return array
		 */
		static public function getServices()
		{
			$services = array(
				// Domestic
				0 => 'USPS First-Class',
				1 => 'USPS Priority Mail',
				2 => 'USPS Express Mail PO to PO',
				3 => 'USPS Express Mail',
				4 => 'USPS Parcel Post',
				5 => 'USPS Bound Printed Matter',
				6 => 'USPS Media Mail',
				7 => 'USPS Library',
				12 => 'USPS First-Class Postcard Stamped',
				13 => 'USPS Express Mail Flat-Rate Envelope',
				16 => 'USPS Priority Mail Flat-Rate Envelope',
				17 => 'USPS Priority Mail Flat-Rate Box',

				// International
				'I1' => 'USPS Express Mail International (EMS)',
				'I2' => 'USPS Priority Mail International',
				'I3' => 'USPS First-Class Mail International',
				'I4' => 'USPS Global Express Guaranteed',
				'I5' => 'USPS Global Express Guaranteed Document',
				'I6' => 'USPS Global Express Guaranteed Non-Document Rectangular',
				'I7' => 'USPS Global Express Guaranteed Non-Document Non-Rectangular',
				'I8' => 'USPS Priority Mail International Flat Rate Envelope',
				'I9' => 'USPS Priority Mail International Flat Rate Box',
				'I10' => 'USPS Express Mail International (EMS) Flat Rate Envelope'
			);
			return $services;
		}
	}
	
	
	/**
	 * USPS response container class
	 */
	class scUSPSResponse {
		protected $_responseXml;
		protected $_xml;
		
		public function __construct($responseXml)
		{
			try {
				// Hack: USPS data encoding is ISO-8859-1, but it's not specified in the xml. Make sure that it correctly handled
				$responseXml = str_replace('<?xml version="1.0"?>', '<?xml version="1.0" encoding="ISO-8859-1"?>', $responseXml);
				
				$this->_responseXml = $responseXml;
				
				$options = (defined('LIBXML_NOWARNING') and defined('LIBXML_NOERROR')) ? LIBXML_NOWARNING | LIBXML_NOERROR : 0;
				$this->_xml = new SimpleXMLElement($this->_responseXml, $options);
			} catch (Exception $exception) {
				throw new scUSPSAPIException($exception->getMessage());
			}
			
			$this->checkReplyForError();
		}
		
		/**
		 * Checks if the request failed
		 * If an error is found scUSPSAPIException will be thrown
		 * 
		 * @throws scUSPSAPIException if an error is returned in response XML
		 */
		protected function checkReplyForError()
		{
			if ($this->_xml->getName() == 'Error') {
				$message = sprintf(_t('%s (Source: %s, Code: %s)'), trim((string)$this->_xml->Description), (string)$this->_xml->Source, (string)$this->_xml->Number);
				throw new scUSPSAPIException($message, (string)$this->_xml->Number);
			}
			
			if (isset($this->_xml->Package->Error)) {
				$message = sprintf(_t('%s (Source: %s, Code: %s)'),
					trim((string)$this->_xml->Package->Error->Description), (string)$this->_xml->Package->Error->Source, (string)$this->_xml->Package->Error->Number);
				throw new scUSPSAPIException($message, (string)$this->_xml->Package->Error->Number);
			}
		}
		
		/**
		 * Returns simple XML element object with the response
		 * 
		 * @return SimpleXMLElement the simple xml element object
		 */
		public function getSimpleXML()
		{
			return $this->_xml;
		}
	}
	
	/**
	 * USPS API Exception class
	 */
	class scUSPSAPIException extends miException {
		protected $_uspsCode;
		
		public function __construct($message, $uspsCode = '')
		{
			parent::__construct($message);
			$this->_uspsCode = $uspsCode;
		}
		
		public function getUSPSCode()
		{
			return $this->_uspsCode;
		}
	}
?>