<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scEditOrderCartItemService extends scCartItemService {
		
		const WARNING_NO_ESCAPE = true;
		
		/**
		 * @var scOrder
		 */
		protected $_currentOrder;
		
		/**
		 * @var scOrderItem
		 */
		protected $_currentOrderItem = null;
		
		/**
		 * @var scProductSL[]
		 */
		protected $_orderItemProducts = array();
		
		/**
		 * @var array
		 */
		protected static $_initialWarnings = array();
		
		/**
		 * @return scEditOrderCartItemService
		 */
		public static function singleton()
		{
			static $instance = null;
			if (is_null($instance)) {
				$instance = new scEditOrderCartItemService();
			}
			return $instance;
		}
		
		public static function getInitialWarnings()
		{
			return self::$_initialWarnings;
		}
		
		protected static function addInitialWarning($warning, $dontEscape = false)
		{
			if ($dontEscape === self::WARNING_NO_ESCAPE) {
				self::$_initialWarnings[] = $warning;
			} else {
				self::$_initialWarnings[] = miI18N::htmlEscape($warning);
			}
		}
		
		public function getCartItem(scCartItemInfo $cartItemInfo)
		{
			if ($cartItemInfo->getProductVariant() instanceof scProductVariant) {
				return new scEditOrderCartItemProductVariant($cartItemInfo);
			}
			
			if ($cartItemInfo->getProduct()->get('ProductIsBundle')) {
				return new scEditOrderCartItemProductBundle($cartItemInfo);
			}
			
			return new scEditOrderCartItemProduct($cartItemInfo);
		}
		
		public function getCartItemOptions(scCartItemOptionsRetrieverInfo $cartItemOptionsInfo)
		{
			$info = new scCartItemOptionsInfo();
			
			$selectedOptions = $cartItemOptionsInfo->getSelectedOptions();
			
			$retriever = new scEditOrderCartItemOptionsRetriever($cartItemOptionsInfo);
			$options = $retriever->getCartItemOptions();
			$info->setOptions($options);
			
			$retriever = new scEditOrderCartItemLocalOptionsRetriever($cartItemOptionsInfo);
			$options = $retriever->getCartItemOptions();
			$info->setLocalOptions($options);
			
			return $info;
		}
		
		public function setCurrentOrder(scOrder $order)
		{
			$this->_currentOrder = $order;
		}
		
		public function setCurrentOrderItem(scOrderItem $orderItem)
		{
			$this->_currentOrderItem = $orderItem;
		}
		
		public function unsetCurrentOrderItem()
		{
			$this->_currentOrderItem = null;
		}
		
		/**
		 * @param scOrderItem $orderItemObject
		 * @return scEditOrderCartItemProduct
		 */
		public function createCartItemFromOrderItem(scOrderItem $orderItemObject)
		{
			scEditOrderCartItemService::singleton()->setCurrentOrderItem($orderItemObject);
			
			$order = $orderItemObject->getOrder();
			$languageId = $this->getOrderLanguageId($order);
			scEditOrderCartItemService::singleton()->setCurrentOrder($order);
			
			$productId = intval($orderItemObject->get('ProductID'));
			
			$product = $this->getOrderItemProduct($order, $orderItemObject);
			if (is_null($product)) {
				return null;
			}
			
			// Gather options the way the CartItem would accept and understand them.
			$orderItemOptions = $orderItemObject->getOrderItemOptions();
			$optionsRetriever = new scEditOrderOptionsRetriever();
			$optionsRetriever->setOrderItemLanguage($languageId);
			$optionsRetriever->setProduct($product);
			$optionsRetriever->setOrderItemOptions($orderItemOptions);
			$selectedOptions['ProductOptions'] = $optionsRetriever->getCartItemOptions();
			
			$warnings = $optionsRetriever->getWarnings();
			if (!empty($warnings)) {
				$warnings = miI18N::htmlEscape($product->getName() . ' - ' . _t('Product Options') .': ') .
					scEditOrderUICart::getImplodedListHtml($warnings);
				self::addInitialWarning($warnings, self::WARNING_NO_ESCAPE);
			}
			
			$orderItemOptions = $orderItemObject->getOrderItemLocalOptions();
			$optionsRetriever = new scEditOrderLocalOptionsRetriever();
			$optionsRetriever->setOrderItemLanguage($languageId);
			$optionsRetriever->setProduct($product);
			$optionsRetriever->setOrderItemOptions($orderItemOptions);
			$selectedOptions['LocalOptions'] = $optionsRetriever->getCartItemOptions();
			
			$warnings = $optionsRetriever->getWarnings();
			if (!empty($warnings)) {
				$warnings = miI18N::htmlEscape($product->getName() . ' - ' . _t('Local Options') .': ') .
					scEditOrderUICart::getImplodedListHtml($warnings);
				self::addInitialWarning($warnings, self::WARNING_NO_ESCAPE);
			}
			
			// CartItemProduct is constructed using CartItemInfo. The info is onstructed in the createCartItemProduct() method.
			// This method expects the options in the form they are goinf to be transmitted over GET or POST.
			// We only have the options in the form needed by the CartItem itself and not the way the CartItemInfo needs them.
			// If the product has variants options must be given to the createCartItemProduct() in order to create the correct ProductVariant.
			$requestOptions = $this->getCartItemOptionsForCartItemInfo($selectedOptions);
			
			$cartItem = scEditOrderCartItemService::singleton()->createCartItemProduct($productId, $orderItemObject->get('OrderItemQty'), $requestOptions);
			$cartItem->addOptions($selectedOptions['ProductOptions']);
			$cartItem->addLocalOptions($selectedOptions['LocalOptions']);
			
			$warnings = $cartItem->getWarnings();
			if (!empty($warnings)) {
				$warnings = miI18N::htmlEscape($product->getName() . ': ') .
					scEditOrderUICart::getImplodedListHtml($warnings);
				self::addInitialWarning($warnings, self::WARNING_NO_ESCAPE);
			}
			
			$cartItemBasePrice = $cartItem->getBasePrice(true)->getAmount();
			$cartItemFinalPrice = $cartItem->getPriceObj(true)->getAmount();
			$cartItemDiscount = scMultiCurrencyAmountMath::subtract($cartItemBasePrice, $cartItemFinalPrice);
			if ($orderItemObject->get('OrderItemDiscount') != $cartItemDiscount->get($order->getBaseCurrency())) {
				$amount = scMultiCurrencyAmount::factory();
				$amount->set($order->getBaseCurrency(), $orderItemObject->get('OrderItemDiscount'));
				$orderItemDisplayDiscount = scPrice::factory($amount)->getDisplayPrice();
				$cartItemDisplayDiscount = scPrice::factory($cartItemDiscount)->getDisplayPrice();
				self::addInitialWarning(sprintf(_t('There was a discount of %s on "%s". It is %s now.'),
					$orderItemDisplayDiscount, $product->getName(), $cartItemDisplayDiscount));
			}
			
			scEditOrderCartItemService::singleton()->unsetCurrentOrderItem();
			
			return $cartItem;
		}
		
		/**
		 * Converts the result of using EditOrderOptionsRetrievers to the form the CartItemInfo objects needs it.
		 * @return array Array similar to the one that is taken from the $_REQUEST when adding item to the cart.
		 */
		protected function getCartItemOptionsForCartItemInfo($cartItemOptions)
		{
			$cartItemInfoOptions = array();
			$mcZero = scMultiCurrencyAmount::getZero(); // A price is required by scProductOption::getOptionInfo()
			foreach ($cartItemOptions as $optionType => $options) {
				$cartItemInfoOptions[$optionType] = array();
				foreach ($options as $option) {
					if (!($option instanceof scProductOption)) {
						throw new miException(_i('scProductOption expected.'));
					}
					$optionInfo = $option->getOptionInfo($mcZero);
					switch ($optionInfo['OptionGroupType']) {
						case scProductOptions::GROUP_TYPE_PRICE_MODIFIER:
						case scProductOptions::GROUP_TYPE_VARIANT:
							$cartItemInfoOptions[$optionType][$optionInfo['OptionGroupID']] = array($option->getOptionId());
							break;
						case scProductOptions::GROUP_TYPE_USER_TEXT:
							$cartItemInfoOptions[$optionType][$optionInfo['OptionGroupID']] = array($optionInfo['OptionID'] => $optionInfo['OptionUserText']);
							break;
						case scProductOptions::GROUP_TYPE_USER_IMAGE:
							if (!is_file(scPaths::getStoreRootDir() . $optionInfo['OptionImage'])) {
								$this->addInitialWarning(sprintf(_t('Option image not found at "%s".'), $optionInfo['OptionImage']));
								break;
							}
							$file = new scFile();
							$fileData = array(
								'name' => basename($optionInfo['OptionImage']),
								'type' => '',
								'tmp_name' => scPaths::getStoreRootDir() . $optionInfo['OptionImage'],
								'size' => filesize(scPaths::getStoreRootDir() . $optionInfo['OptionImage'])
							);
							$file->setFileData($fileData);
							$cartItemInfoOptions[$optionType][$optionInfo['OptionGroupID']] = array(
								$optionInfo['OptionID'] => $file,
								$optionInfo['OptionID']
							);
							break;
						default:
							throw new miException(_i('Unsupported option group type.'));
					}
				}
			}
			return $cartItemInfoOptions;
		}
		
		/**
		 * @param scOrder $order
		 * @param scOrderItem[] $orderItems
		 * @return scProductSL
		 */
		protected function getOrderItemProduct(scOrder $order, scOrderItem $orderItem)
		{
			$productId = $orderItem->get('ProductID');
			if (isset($this->_orderItemProducts[$productId])) {
				return $this->_orderItemProducts[$productId];
			}
			
			try {
				$product = new scProductSL();
				$product->read($productId);
			} catch (miDBException $e) {
				if ($e->getCode() != miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					throw $e;
				}
				self::addInitialWarning(sprintf(_t('Product cannot be found (ProductID %d).'), $productId));
				return null;
			}
			
			if ($order->get('OrderQuantitiesInStockReduced')) {
				// TODO: Restore quantity if needed.
				$product->set('ProductQuantity', $product->get('ProductQuantity') + $orderItem->get('OrderItemQty'));
			}
			
			$this->_orderItemProducts[$productId] = $product;
			
			return $this->_orderItemProducts[$productId];
		}
		
		/**
		 * @param scOrder $order
		 * @return int
		 */
		protected function getOrderLanguageId(scOrder $order)
		{
			$code = $order->get('OrderLanguageCode');
			$languageCodes = scMultiLanguage::singleton()->getLanguageCodes();
			
			if (!isset($languageCodes[$code])) {
				self::addInitialWarning(sprintf(
					_t('Unable to find language code "%s". "%s" will be used instead.'),
					$code,
					scMultiLanguage::getCurrentLanguageCode()));
				return scMultiLanguage::getCurrentLanguageId();
			}
			
			return $languageCodes[$code];
		}
		
		/**
		 * @return scEditOrderCartItemInfo
		 */
		protected function createCartItemInfo()
		{
			$info = new scEditOrderCartItemInfo();
			$info->setOrder($this->_currentOrder);
			if (!is_null($this->_currentOrderItem)) {
				$info->setOrderItem($this->_currentOrderItem);
			}
			return $info;
		}
	}
?>