<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * We need to convert text represenations of some values to IDs
	 * as some of the fields are saved in the orders with their textual representations.
	 */
	class scEditOrderDomainObject extends scOrder {
		
		protected $_warnings = array();
		
		protected $_orderLanguageId = null;
		
		protected $_countries = array();
		
		protected $_states = array();
		
		public function update()
		{
			// Always save the language code.
			$languageCodes = scMultiLanguage::singleton()->getLanguageCodes();
			$languageCode = array_search($this->getOrderLanguageId(), $languageCodes);
			$this->set('OrderLanguageCode', $languageCode);
			
			return parent::update();
		}
		
		/**
		 * Uses set() for each element in the $row array.
		 * @param array $row
		 */
		public function setRow($row)
		{
			$setRow = array();
			foreach ($row as $field => $value) {
				$this->set($field, $value);
			}
		}
		
		/**
		 * Uses get() to get all fields one by one.
		 * @return array
		 */
		public function getRow()
		{
			$recordFields = array_keys(parent::getRow());
			
			$returnRow = array();
			foreach ($recordFields as $field) {
				$returnRow[$field] = $this->get($field);
			}
			$returnRow['ShippingModuleAndMethod'] = $this->get('ShippingModuleAndMethod');
			
			return $returnRow;
		}
		
		/**
		 * @param string $fieldName
		 * @return mixed
		 * @throws miException in case of fatal error
		 */
		public function get($fieldName)
		{
			switch ($fieldName) {
				case 'ShippingModuleAndMethod':
					$shippingModules = scModulesFactory::getActiveShippingModules();
					if (count($shippingModules) == 0) {
						throw new miException(_t('There are no active shipping modules.'));
					}
					
					if (!isset($shippingModules[$this->get('ShippingModuleID')])) {
						$this->addWarning(_t('The shipping module that the user has selected could not be found.'));
						return $this->selectFirstAvaialableShippingMethod();
					}
					
					$activeMethods = $shippingModules[$this->get('ShippingModuleID')]['ShippingModuleActiveMethods'];
					
					if (!isset($activeMethods[$this->get('OrderShippingMethodCode')])) {
						$this->addWarning(_t('The shipping method that the user has selected is not active.'));
						return $this->selectFirstAvaialableShippingMethod();
					}
					
					return $this->get('ShippingModuleID') . '-' . $this->get('OrderShippingMethodCode');
				
				case 'PaymentModuleID':
					$paymentModules = scCheckoutPaymentMethodWidget::getActivePaymentModules();
					if (count($paymentModules) == 0) {
						throw new miException(_t('There are no active payment modules.'));
					}
					$paymentModuleId = parent::get('PaymentModuleID');
					
					foreach ($paymentModules as $moduleInfo) {
						if ($moduleInfo['PaymentModuleID'] == $paymentModuleId) {
							return $paymentModuleId;
						}
					}
					// Didn't find the module. Add a warning.
					$this->addWarning(_t('The payment module that the user has selected could not be found.'));
					$firstModule = reset($paymentModules);
					return $firstModule['PaymentModuleID'];
					
				case 'OrderShippingCountry':
				case 'OrderBillingCountry':
					$countryName = parent::get($fieldName);
					$countryId = $this->findCountryByName($countryName);
					
					if ($countryId === false) {
						$msg = ($fieldName == 'OrderShippingCountry') ?
							 _t('The country "%s" that the customer has selected as shipping country could not be found.') :
							 _t('The country "%s" that the customer has selected as billing country could not be found.');
						$this->addWarning(sprintf($msg, $countryName));
						return 0;
					}
					return $countryId;
					
				case 'OrderShippingState':
				case 'OrderBillingState':
					$stateName = parent::get($fieldName);
					if (empty($stateName)) {
						return 0;
					}
					
					$stateId = $this->findStateByName($stateName);
					if ($stateId === false) {
						$this->addWarning(sprintf(
							_t('The state "%s" that the user has selected as his %s could not be found.'),
							$stateName,
							$fieldName
						));
						return 0;
					}
					
					return $stateId;
					
				default:
					return parent::get($fieldName);
			}
		}
		
		/**
		 * @param string $fieldName
		 * @param mixed $fieldValue
		 */
		public function set($fieldName, $fieldValue)
		{
			switch ($fieldName) {
				case 'ShippingModuleAndMethod':
					list($moduleId, $methodCode) = explode('-', $fieldValue);
					parent::set('ShippingModuleID', $moduleId);
					parent::set('OrderShippingMethodCode', $methodCode);
					break;
					
				case 'OrderShippingCountry':
				case 'OrderBillingCountry':
					if (is_numeric($fieldValue)) {
						$countries = $this->getCountries();
						parent::set($fieldName, $countries[$fieldValue]);
					} else {
						parent::set($fieldName, $fieldValue);
					}
					break;
					
				case 'OrderShippingState':
				case 'OrderBillingState':
					if (is_numeric($fieldValue)) {
						$states = $this->getStates();
						parent::set($fieldName, $states[$fieldValue]);
					} else {
						parent::set($fieldName, $fieldValue);
					}
					break;
					
				default:
					parent::set($fieldName, $fieldValue);
					break;
			}
		}
		
		public function getWarnings()
		{
			return $this->_warnings;
		}
		
		protected function selectFirstAvaialableShippingMethod()
		{
			$country = $this->get('OrderShippingCountry');
			$state = $this->get('OrderShippingState');
			
			$shippingModules = scModulesFactory::getActiveShippingModulesForZone($country, $state);
			if (empty($shippingModules)) {
				return null;
			}
			$shippingModule = reset($shippingModules);
			$shippingModuleId = $shippingModule['ShippingModuleID'];
			
			$shippingMethod = reset($shippingModule['ShippingModuleActiveMethods']);
			$shippingMethodCode = $shippingMethod['ShippingMethodCode'];
			
			return $shippingModuleId . '-' . $shippingMethodCode;
		}
		
		protected function addWarning($warning)
		{
			$this->_warnings[] = miI18N::htmlEscape($warning);
		}
		
		/**
		 * Return the country id of the country. Checks all languages
		 * 
		 * @param string $countryName
		 * @return int $countryId
		 */
		protected function findCountryByName($countryName)
		{
			$sql = 'SELECT CountryID FROM CountriesML WHERE CountryName=? LIMIT 1';
			$result = miStaticDBUtil::execSelect($sql, array($countryName));
			if (count($result))
				return $result[0]['CountryID'];
			return false;
		}
		
		/**
		 * Return the state id of the state. Checks all languages
		 * 
		 * @param string $stateName
		 * @return int $stateId
		 */
		protected function findStateByName($stateName)
		{
			$sql = 'SELECT StateID FROM StatesML WHERE StateName=? LIMIT 1';
			$result = miStaticDBUtil::execSelect($sql, array($stateName));
			if (count($result))
				return $result[0]['StateID'];
			return false;
		}
		
		/**
		 * @return array
		 */
		protected function getCountries()
		{
			$languageId = $this->getOrderLanguageId();
			if (!isset($this->_countries[$languageId])) {
				$this->_countries[$languageId] = scCountry::getCountryNames(array(), $languageId);
			}
			
			return $this->_countries[$languageId];
		}
		
		/**
		 * @return array
		 */
		protected function getStates()
		{
			$languageId = $this->getOrderLanguageId();
			if (!isset($this->_states[$languageId])) {
				$this->_states[$languageId] = scCountry::getStateNames(array(), $languageId);
			}
			
			return $this->_states[$languageId];
		}
		
		/**
		 * @return int
		 */
		protected function getOrderLanguageId()
		{
			if (is_null($this->_orderLanguageId)) {
				$code = $this->get('OrderLanguageCode');
				$languageCodes = scMultiLanguage::singleton()->getLanguageCodes();
				
				if (isset($languageCodes[$code])) {
					$this->_orderLanguageId = $languageCodes[$code];
				} else {
					$this->_orderLanguageId = scMultiLanguage::getCurrentLanguageId();
				}
			}
			
			return $this->_orderLanguageId;
		}
		
		/**
		 * When returning quantities it is not absolutely necessary for all the products to exist.
		 * You may be editing an old order that contains some deleted products.
		 */
		protected function returnStockItemQuantities()
		{
			$stockItems = $this->getStockItemsData();
			foreach ($stockItems as $stockItemData) {
				try {
					$stockItem = $this->getStockItemFromData($stockItemData);
				} catch (miDBException $e) {
					if ($e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
						continue;
					}
				}
				$stockItem->setQuantityInStock($stockItem->getQuantityInStock() + $stockItemData['OrderItemQty']);
			}
		}
	}
?>