<?php
	/**
	 * Copyright (c) 2010 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	/**
	 * Get the options for an EditOrderCartItem
	 * given the options stored in an order item and the product corresponding to that item.
	 */
	class scEditOrderOptionsRetriever {
		
		const WARNING_NO_ESCAPE = true;
		
		protected $_orderItemLanguage = null;
		
		/**
		 * @var scProductSL
		 */
		protected $_product = null;
		
		/**
		 * Product options for the product in $_product
		 * as returned by scProductService::getProductsOptions()
		 */
		protected $_productOptions = null;
		protected $_orderItemOptions = null;
		
		protected $_productOptionsKey = 'ProductOptions';
		
		protected $_warnings = array();
		
		public function setOrderItemLanguage($language)
		{
			$this->_orderItemLanguage = $language;
		}
		
		public function setProduct(scProductSL $product)
		{
			$this->_product = $product;
			
			$options = scProductService::getProductsOptions(array($product->get('ProductID')));
			$this->_productOptions = array();
			if (isset($options[$product->get('ProductID')][$this->_productOptionsKey])) {
				$this->_productOptions = $options[$product->get('ProductID')][$this->_productOptionsKey];
			}
		}
		
		public function setOrderItemOptions($orderItemOptions)
		{
			$this->_orderItemOptions = $orderItemOptions;
		}
		
		/**
		 * @return array
		 */
		public function getCartItemOptions()
		{
			$this->checkIsReady();
			$this->checkOptionsCompatability();
			
			$cartItemOptions = $this->getMatchingOptions();
			return $cartItemOptions;
		}
		
		public function getWarnings()
		{
			return $this->_warnings;
		}
		
		/**
		 * @param string $warning
		 */
		protected function addWarning($warning, $dontEscape = false)
		{
			if ($dontEscape === self::WARNING_NO_ESCAPE) {
				$this->_warnings[] = $warning;
			} else {
				$this->_warnings[] = miI18N::htmlEscape($warning);
			}
		}
		
		/**
		 * @throws miException Exceptions thrown by this method are thrown for development purposes.
		 */
		protected function checkIsReady()
		{
			if (is_null($this->_orderItemLanguage)) {
				throw new miException('Order item language needed.');
			}
			if (!($this->_product instanceof scProductSL)) {
				throw new miException('scProductSL needed.');
			}
			if (is_null($this->_orderItemOptions)) {
				throw new miException('Order item options needed.');
			}
		}
		
		/**
		 * Compares the product and order item options to add warnings if needed.
		 */
		protected function checkOptionsCompatability()
		{
			if (count($this->_productOptions) != count($this->_orderItemOptions)) {
				$this->addWarning(sprintf(
					_t('Product "%s" options\' count does not match the order item options\' count.'),
					$this->_product->getName()
				));
			}
			
			$productGroupNames = array();
			$orderItemGroupNames = array();
			
			foreach ($this->_productOptions as $optionGroup) {
				$productGroupNames[] = $optionGroup['OptionGroupName'];
			}
			
			foreach ($this->_orderItemOptions as $optionGroup) {
				$orderItemGroupNames[] = $optionGroup->getOptionGroupName();
			}
			
			$diff = array_diff($productGroupNames, $orderItemGroupNames);
			if (count($diff)) {
				$diff = array_map(array('miI18N', 'htmlEscape'), $diff);
				$this->addWarning(sprintf(
					_t('The order item has missing product options.') . ' ' . _t('These product options will be set to default values: %s'),
					scEditOrderUICart::getImplodedListHtml($diff)
				), self::WARNING_NO_ESCAPE);
			}
			
			$diff = array_diff($orderItemGroupNames, $productGroupNames);
			if (count($diff)) {
				$diff = array_map(array('miI18N', 'htmlEscape'), $diff);
				$this->addWarning(sprintf(
					_t('These product options have been selected for the order item, but are not supported by the product: %s'),
					scEditOrderUICart::getImplodedListHtml($diff)
				), self::WARNING_NO_ESCAPE);
			}
		}
		
		/**
		 * @return array
		 */
		protected function getMatchingOptions()
		{
			$cartItemOptions = array();
			$warnings = array();
			foreach ($this->_productOptions as $productOptionId => $productOption) {
				$matchedProductOptionId = null;
				foreach ($this->_orderItemOptions as $orderItemOption) {
					if ($productOption['OptionGroupName'] != $orderItemOption->getOptionGroupName()) {
						continue;
					}
					$matchedProductOptionId = $productOptionId;
					
					foreach ($productOption['Options'] as $optionValueId => $optionInfo) {
						if ($optionInfo['OptionName'] == $orderItemOption->getName()) {
							$optionInfo['OptionGroupName'] = $productOption['OptionGroupName'];
							$optionInfo['OptionGroupType'] = $productOption['OptionGroupType'];
							$optionInfo['OptionID'] = $optionValueId;
							$optionInfo['OptionGroupID'] = $matchedProductOptionId;
							$optionInfo['LanguageID'] = $this->_orderItemLanguage;
							$optionInfo['ProductID'] = $this->_product->getProductId();
							$optionInfo['OptionUserText'] = $orderItemOption->getUserText();
							$optionInfo['OptionImage'] = $orderItemOption->getImage();
							$cartItemOptions[$matchedProductOptionId] = $this->createOption($optionValueId, $optionInfo);
							break(2);
						}
					}
					
					$warnings[] = $orderItemOption->getOptionGroupName() . ': ' . $orderItemOption->getName();
				}
			}
			
			if (!empty($warnings)) {
				$this->addWarning(sprintf(
					_t('Some values have been selected for the order item\'s options, but are no longer supported by the product.') .
						' ' . _t('These product options will be set to default values: %s'),
					scEditOrderUICart::getImplodedListHtml($warnings)
				), self::WARNING_NO_ESCAPE);
			}
			
			return $cartItemOptions;
		}
		
		/**
		 * @param int $optionId
		 * @param array $optionInfo
		 * @return scProductOption
		 */
		protected function createOption($optionId, $optionInfo)
		{
			return new scProductOption($optionId, $optionInfo);
		}
		
		/**
		 * @param array $optionIds
		 * @return array
		 */
		protected function getOptionsInfo($optionIds)
		{
			return scProductOptions::getOptionsInfo($optionIds, $this->_orderItemLanguage);
		}
	}
		
	class scEditOrderLocalOptionsRetriever extends scEditOrderOptionsRetriever {
		
		protected $_productOptionsKey = 'LocalOptions';
		
		/**
		 * @param int $optionId
		 * @param array $optionInfo
		 * @return scProductOption
		 */
		protected function createOption($optionId, $optionInfo)
		{
			$mapping = scLocalOption::getLocalOptionToProductOptionMapping();
			$mapping = array_flip($mapping);
			
			foreach ($mapping as $optionField => $mappedField) {
				if (($optionField != $mappedField) && isset($optionInfo[$optionField])) {
					$optionInfo[$mappedField] = $optionInfo[$optionField];
					unset($optionInfo[$optionField]);
				}
			}
			
			return new scLocalOption($optionId, $optionInfo);
		}
		
		/**
		 * @return array
		 */
		protected function getDefaultOptions()
		{
			return scEditOrderCartItemService::getDefaultLocalOptionsForProduct($this->_product->getProductId());
		}
		
		/**
		 * @param array $optionIds
		 * @return array
		 */
		protected function getOptionsInfo($optionIds)
		{
			return scLocalOptions::getOptionsInfo($optionIds, $this->_orderItemLanguage);
		}
	}
?>