<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	class scNewsletterSubscribeAction extends miAction {
		
		public function doAction()
		{
			try {
				$cmsBoxId = (int)miGetParamDefault('CMSBoxID', 0);
				$newsletterId = (int)miGetParam('NewsletterID');
				$languageId = scMultiLanguage::getCurrentLanguageId();
				$subscriberEmail = trim(miGetParam('SubscriberEmail'));
				$notifyAdmin = false;
				
				if ($cmsBoxId > 0) {
					$options = scModuleOptions::getCmsBoxOptions($cmsBoxId);
					if (($options['NewsletterID'] == $newsletterId) && (bool)$options['AdminNotification']) {
						$notifyAdmin = true;
					}
				}
				
				if (!miValidatorEmail::isValid($subscriberEmail)) {
					throw new miException(_t('Please enter a valid e-mail address.'));
				}
				
				$newsletter = new scNewsletter();
				$newsletter->read($newsletterId);
				
				$customerId = scUtil::getDbValue('Customers', 'CustomerID', 'CustomerEmail', $subscriberEmail, false);
				if ($customerId !== false && (scCustomerAuth::singleton()->getCustomerId() != $customerId)) {
					// Found a customer with this e-mail. Make him log in.
					throw new miException(sprintf(_t('You need to log-in in order to subscribe for newsletter "%s".'), $newsletter->get('NewsletterName', $languageId)));
				}
				
				// Check if the customer already has a subscription.
				// If he does, we don't need to send the administrator a notification.
				// The only thing done was eventually updating the language and setting him to "ACTIVE".
				$alreadySubscribed = $newsletter->hasSubscriber($subscriberEmail);
				
				$newsletter->subscribe($subscriberEmail);
				if ($alreadySubscribed) {
					return $this->showSuccess($newsletter);
				}
				
				try {
					if (!$notifyAdmin) {
						return $this->showSuccess($newsletter);
					}
					
					$notificationEmail = scSettings::getSettingValue(scSettings::SETTING_EMAIL_REPLY_TO_NEWSLETTER);
					if (empty($notificationEmail)) {
						return $this->showSuccess($newsletter);
					}
					
					$email = new scEmail();
					$subject = sprintf(_t('Newsletter subscription to "%s"'), $newsletter->get('NewsletterName', $languageId));
					$html = sprintf(_t('A user with the email "%s" just subscribed for the "%s" newsletter.'), $subscriberEmail, $newsletter->get('NewsletterName', $languageId));
					$email->setTo(new scEmailAddress($notificationEmail));
					$email->setSubject($subject);
					$email->sendHtmlEmail($html);
				} catch (miException $e) {
					// Allow errors when notifying admin.
				}
				
				return $this->showSuccess($newsletter);
			} catch (miException $exception) {
				return $this->showFailure($exception->getMessage());
			}
		}
		
		protected function showSuccess(scNewsletter $newsletter)
		{
			$t = new scTemplateParser();
			$t->readTemplate('modules/system/newsletter/customer/newsletter_subscribe_success.tmpl');
			$t->assignArray(miUIUtil::templetizeRows($newsletter->getRow(scMultiLanguage::getCurrentLanguageId())));
			$t->assignArray(scDynamicPages::getTemplateVars('module.php?ModuleName=com.summercart.newsletter'));
			$html = $t->templateParse();
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
		
		protected function showFailure($error)
		{
			$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($error);
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
			return false;
		}
	}
	
	/**
	 * Displays a webform for unsubscribing.
	 */
	class scNewsletterUnsubscribeAction extends miAction {
		
		protected $_form = null;
		
		public function doAction()
		{
			try {	
				$newsletterId = (int)miGetParam('NewsletterID');
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$form = $this->getWebForm();
				
				$form->addMainPageElements(miUIUtil::templetizeRows($newsletter->getRow(scMultiLanguage::getCurrentLanguageId())));
				$form->setFormData('NewsletterID', $newsletterId);
				
				$html = $form->parse('modules/system/newsletter/customer/newsletter_unsubscribe.tmpl', true);
				
				$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
				$this->_view->addControllerCommand($cmd);
				return true;
			} catch (miException $exception) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($exception->getMessage());
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return false;
			}
		}
		
		/**
		 * @return miWebForm
		 */
		protected function getWebForm()
		{
			if (!is_null($this->_form)) {
				return $this->_form;
			}
			
			$this->_form = new miWebForm(array());
			$this->_form->setErrorsHandler(new scByFieldErrorsHandler());
			
			$widget = new miBaseTextWidget($this->_form);
			$widget->setFieldName('NewsletterID');
			$this->_form->addWidget('NewsletterID', $widget);
			
			$widget = new miBaseTextWidget($this->_form);
			$widget->setFieldName('UnsubscribeEmail');
			$this->_form->addWidget('UnsubscribeEmail', $widget);
			$this->_form->addValidator('UnsubscribeEmail', new miValidatorEmail($this->_form, 'UnsubscribeEmail'));
			
			$this->_view->initMainPageElements();
			$this->_form->addMainPageElements($this->_view->getMainPageElements());
			
			return $this->_form;
		}
	}
	
	class scNewsletterExecUnsubscribeAction extends scNewsletterUnsubscribeAction {
		
		public function doAction()
		{
			try {
				$form = $this->getWebForm();
				$isOk = $form->processSubmit();
				
				if (!$isOk) {
					parent::doAction();
					return false;
				}
				
				$data = $form->getWidget('NewsletterID')->getData();
				$newsletterId = (int)$data['NewsletterID'];
				
				$newsletter = $this->_view->getDomainObject();
				$newsletter->read($newsletterId);
				
				$data = $form->getWidget('UnsubscribeEmail')->getData();
				$unsubscribeEmail = trim($data['UnsubscribeEmail']);
				
				if (!$newsletter->hasSubscriber($unsubscribeEmail)) {
					return $this->showSuccess($newsletter);
				}
				
				$newsletter->unsubscribe($unsubscribeEmail);
				return $this->showSuccess($newsletter);
			} catch (miException $exception) {
				$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($exception->getMessage());
				$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
				return false;
			}
		}
		
		protected function showSuccess(scNewsletter $newsletter)
		{
			$t = new scTemplateParser();
			$t->readTemplate('modules/system/newsletter/customer/newsletter_unsubscribe_success.tmpl');
			$t->assignArray(miUIUtil::templetizeRows($newsletter->getRow(scMultiLanguage::getCurrentLanguageId())));
			$html = $t->templateParse();
			
			$cmd = new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_HTML, $html);
			$this->_view->addControllerCommand($cmd);
			return true;
		}
	}
	
	class scNewsletterOpenAction extends miAction {
		
		public function doAction()
		{
			try {
				scDateTime::setTimezone();
				$blastEmailId = (int)miGetParamDefault('BlastEmailID', 0);
				
				if ($blastEmailId > 0) {
					$values = array(
						'BlastEmailID' => $blastEmailId,
						'BlastEmailOpenDate' => date(MI_MYSQL_DATE_FORMAT)
					);
					miStaticDBUtil::execInsert('NLBlastEmailOpens', $values, miDBUtilImpl::FLAG_IGNORE);
				}
			} catch (miException $e) {
				// Errors are not critical.
				// The image should be shown no matter tracking was successful or not.
				scLog::log(scLog::SEVERITY_WARNING, sprintf(_i('Newsletter open error: %s'), $e->getMessage()));
			}
			
			$gifImage = base64_decode('R0lGODlhAQABAJEAAAAAAP///////wAAAC' .
				'H5BAEAAAIALAAAAAABAAEAAAICVAEAOw==');
			header('Cache-Control: no-cache');
			header('Content-type: image/gif');
			header('Content-length: ' . strlen($gifImage));
			echo $gifImage;
			flush();
			exit;
		}
	}
	
	class scNewsletterClickAction extends miAction {
		
		public function doAction()
		{
			scDateTime::setTimezone();
			$blastEmailId = (int)miGetParamDefault('BlastEmailID', 0);
			$clickUrl = miGetParamDefault('click', '');
			
			$parsed = scUrlUtil::parse($clickUrl);
			
			if (!isset($parsed[scUrlUtil::SCHEME])) {
				$clickUrl = scPaths::getUrl() . $clickUrl;
			}
			
			try {
				$email = new scNewsletterBlastEmail();
				$email->read($blastEmailId);
				
				$date = date(MI_MYSQL_DATE_FORMAT);
				
				// Mark a click
				$query = 'INSERT INTO NLBlastEmailClicks SET BlastEmailID = ?, BlastEmailClickDate = ?' .
					' ON DUPLICATE KEY UPDATE BlastEmailClicksCount = BlastEmailClicksCount + 1';
				miStaticDBUtil::execSQL($query, array($blastEmailId, $date));
				
				// Mark an open (in order to click you have surely opened)
				$values = array(
					'BlastEmailID' => $blastEmailId,
					'BlastEmailOpenDate' => $date,
				);
				miStaticDBUtil::execInsert('NLBlastEmailOpens', $values, miDBUtilImpl::FLAG_IGNORE);
			} catch (miException $e) {
				if ($e instanceof miDBException && $e->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND) {
					$url = scPaths::getStoreWebPath() . 'error.php?msg=' . urlencode($e->getMessage());
					$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $url));
					return false;
				}
				scLog::log(scLog::SEVERITY_WARNING, sprintf(_i('Newsletter click error: %s'), $e->getMessage()));
			}
			
			$this->_view->addControllerCommand(new miControllerCommand(miControllerCommand::CONTROLLER_COMMAND_REDIRECT, $clickUrl));
			return true;
		}
	}
?>