<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */
	
	require_once(dirname(__FILE__) . '/../RatingActionConsts.php');

	
	/**
	 * Domain object class for RatingActions Table
	 */
	class scRatingActionDomainObject extends miDefaultDomainObject
	{
		const RATING_ACTION_STATUS_FIELD_NAME = 'RatingActionStatus';
		
		public function __construct()
		{
			$record = new miSqlRecord('RatingActions', 'RatingActionID');
			parent::__construct($record);
		}
		
		/**
		 * reads a record by primary key
		 *
		 * @param mixed $pklValue
		 */
		public function read($pklValue)
		{
			parent::read($pklValue);
			$recordRow = $this->_record->getRow();
			$recordRow[self::RATING_ACTION_STATUS_FIELD_NAME] = $this->getRatingActionStatus();
			$this->_record->setRow($recordRow);
		}
		
		/**
		 * Updates the current record
		 *
		 * @return int (primary key)
		 */
		public function update()
		{
			$row = $this->_record->getRow();
			if (isset($row[self::RATING_ACTION_STATUS_FIELD_NAME])) {
				unset($row[self::RATING_ACTION_STATUS_FIELD_NAME]);
				$this->setRow($row);
			}
			
			return parent::update();
		}
		
		/**
		 * Inserts a record
		 *
		 * @return int (primary key of the new record)
		 */
		public function insert()
		{
			$row = $this->_record->getRow();
			if (isset($row[self::RATING_ACTION_STATUS_FIELD_NAME])) {
				unset($row[self::RATING_ACTION_STATUS_FIELD_NAME]);
				$this->setRow($row);
			}
			
			return parent::insert();
		}
		
		/**
		 * Gets single rating actions status, based on the statuses from all the rating actions
		 *
		 * @return int (rating actions status)
		 */
		protected function getRatingActionStatus()
		{
			$productReviewStatus = $productRatingVoteStatus = null;
			$productReviewObj = new scProductReviewDomainObject();
			if ($productReviewObj->readByRatingActionId($this->_record->getPK())) {
				$productReviewStatus = $productReviewObj->getStatus();
			}
			
			$productRatingVoteObj = new scProductRatingVotesDomainObject();
			if ($productRatingVoteObj->readByRatingActionId($this->_record->getPK())) {
				$productRatingVoteStatus = $productRatingVoteObj->getStatus();
			}
			
			return scRatingActionsStatuses::getCombinedRatingActionStatus($productReviewStatus, $productRatingVoteStatus);
		}
	}
	
	/**
	 * Abstract class used for RatingActions Domain Objects
	 */
	abstract class scRatingActionChildTableDomainObject extends miDefaultDomainObject
	{
		/**
		 * Should return the field name for the status field
		 */
		abstract protected function getStatusFieldName();
		
		/**
		 * Should return the record for the domain object
		 */
		abstract protected function getRecord();
		
		public function __construct()
		{
			$record = $this->getRecord();
			parent::__construct($record);
		}
		
		/**
		 * Read a record from Rating Action table based on its RatingActionID
		 *
		 * @param int $ratingActionId
		 */
		public function readByRatingActionId($ratingActionId)
		{
			try {
				$this->_record->read('RatingActionID', $ratingActionId);
			} catch (miException $exception) {
				if ($exception->getCode() == miDBException::EXCEPTION_RECORD_NOT_FOUND)
					return false;
			}

			return true;
		}
		
		/**
		 * Gets the status of the Rating Action
		 *
		 * @return int (Rating action status)
		 */
		public function getStatus()
		{
			return $this->get($this->getStatusFieldName());
		}
		
		/**
		 * Sets new status for the Rating Action 
		 *
		 * @param int $value
		 */
		public function updateStatus($value)
		{
			$this->set($this->getStatusFieldName(), $value);
			$this->update();
		}
		
		public function get($name)
		{
			$row = $this->_record->getRow();
			if (isset($row[$name])) {
				return $row[$name];
			}

			return null;
		}
	}
	
	class scProductReviewDomainObject extends scRatingActionChildTableDomainObject
	{
		/**
		 * Returns the field name for the status field of the domain object's table
		 *
		 * @return string
		 */
		public function getStatusFieldName()
		{
			return 'ProductReviewStatus';
		}
		
		/**
		 * Returns miSqlRecord for the current domain object
		 *
		 * @return miSqlRecord
		 */
		protected function getRecord()
		{
			return new miSqlRecord('ProductReviews', 'ProductReviewID');
		}
	}
	
	class scProductRatingVotesDomainObject extends scRatingActionChildTableDomainObject
	{
		
		/**
		 * Returns the field name for the status field of the domain object's table
		 *
		 * @return string
		 */
		public function getStatusFieldName()
		{
			return 'ProductRatingVoteStatus';
		}
		
		/**
		 * Returns miSqlRecord for the current domain object
		 *
		 * @return miSqlRecord
		 */
		protected function getRecord()
		{
			return new miSqlRecord('ProductRatingVotes', 'ProductRatingVoteID');
		}
	}
	
	/**
	 * ViewMapper for ProductReviews Table fields 
	 */
	class scProductReviewsViewMapper extends miViewMapperDefault
	{
		public function __construct(miViewMapperController $viewMapperController)
		{
			$productReviewDomainObject = new scProductReviewDomainObject();
			parent::__construct($viewMapperController, $productReviewDomainObject, miGetParamDefault($productReviewDomainObject->getPKName(), null));
		}
		
		/**
		 * Reads Rating Action record by it's RatingActionID 
		 */
		public function read()
		{
			$ratingActionId = (int)$this->getViewMapperController()->getDomainObject()->get('RatingActionID');
			$this->_domainObj->readByRatingActionId($ratingActionId);
		}
		
		public function delete()
		{
			$pkValue = $this->_domainObj->get($this->_domainObj->getPKName());
			if (is_null($pkValue))
				return;
				
			parent::delete();
		}
	}
	
	/**
	 * ViewMapper for ProductRatingVotes Table fields 
	 */
	class scProductRatingVotesViewMapper extends miViewMapperDefault
	{
		public function __construct(miViewMapperController $viewMapperController)
		{
			$productRatingVoteDomainObject = new scProductRatingVotesDomainObject();
			parent::__construct($viewMapperController, $productRatingVoteDomainObject, miGetParamDefault($productRatingVoteDomainObject->getPKName(), null));
		}
		
		/**
		 * Reads Rating Action record by it's RatingActionID 
		 */
		public function read()
		{
			$ratingActionId = (int)$this->getViewMapperController()->getDomainObject()->get('RatingActionID');
			$this->_domainObj->readByRatingActionId($ratingActionId);
		}
		
		public function delete()
		{
			$pkValue = $this->_domainObj->get($this->_domainObj->getPKName());
			if (is_null($pkValue))
				return;
				
			parent::delete();
		}
	}
	
	/**
	 * ViewMapper for RatingActions Table fields 
	 */
	class scRatingActionStatusViewMapper extends miViewMapperDefault
	{
		public function __construct(miViewMapperController $viewMapperController)
		{
			$ratingActionDomainObject = $viewMapperController->getView()->getDomainObject();
			parent::__construct($viewMapperController, $ratingActionDomainObject, miGetParamDefault($ratingActionDomainObject->getPKName(), null));
		}
		
		/**
		 * Sets value for specified field name
		 *
		 * @param string $fieldName
		 * @param mixed $fieldValue
		 */
		public function set($fieldName, $fieldValue)
		{
			if ($fieldName == scRatingActionDomainObject::RATING_ACTION_STATUS_FIELD_NAME) {
				$this->_viewMapperController->getViewMapper('scProductRatingVotesViewMapper')->set('ProductRatingVoteStatus', $fieldValue);
				$this->_viewMapperController->getViewMapper('scProductReviewsViewMapper')->set('ProductReviewStatus', $fieldValue);
				return;
			}
			
			parent::set($fieldName, $fieldValue);
		}
	}
	
	/**
	 * Widget for displaying IP addresses from INT database fields 
	 */
	class scIPAddressWidget extends miBaseTextWidget
	{
		public function getControl()
		{
			$control = parent::getControl();
			$control[$this->_fieldName] = long2ip($control[$this->_fieldName]);
			
			return $control;
		}
		
		public function getEditableControl()
		{
			return $this->getControl();
		}
	}
	
	/**
	 * Widget for the ProductReviews.ProductReviewText. Returns text if it is not set
	 */
	class scTextAreaProductReviewTextWidget extends scWebFormWidgetTextarea
	{
		public function getEditableControl()
		{
			if (is_null($this->getValue())) {
				return array($this->_fieldName => _t('No review available.'));
			}
			
			return parent::getEditableControl();
		}
	}
	
	/**
	 * Widget for the ProductRatingVotes.ProductRatingVoteValue. Returns text if it is not set
	 */
	class scSelectProductRatingVoteWidget extends miSelectWidget
	{
		public function getEditableControl()
		{
			$value = $this->_webForm->getFormData($this->_fieldName);
			if (is_null($value)) {
				return array($this->_fieldName => _t('No rating available.'));
			}
			
			return parent::getEditableControl();
		}
	}
	
	/**
	 * Widget for displaying Dates from INT database fields
	 */
	class scRatingActionTimestampWidget extends miBaseTextWidget
	{
		public function getControl()
		{
			return array($this->_fieldName => date('d M Y G:i:s', $this->getValue()));
		}
		
		public function getEditableControl()
		{
			return $this->getControl();
		}
	}

	/**
	 * Rating Actions module
	 */
	class scAdminRatingActionsModule implements IAdminModule
	{		
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->setActionHandler('dmEdit', 'scEditRatingAction');
			$page->getDispatcher()->setActionHandler('dmChangeRatingActionStatus', 'scChangeRatingActionStatus');
		}
		
		public function getView()
		{
			scDateTime::setTimezone();
			
			// By default sort by RatingAction date desc
			if (miGetParamDefault(miTableSorter::PARAM_SORTBY, '') == '') {
				miSetParam(miTableSorter::PARAM_SORTBY, 'RatingActionTimestamp');
				miSetParam(miTableSorter::PARAM_SORTDIR, 'DESC');
			}
			
			$ratingActions = new miSqlRecordset('RatingActions', 'RatingActionID');
			$ratingActions->addJoinCondition('LEFT', 'ProductRatingVotes', 'ON(RatingActions.RatingActionID = ProductRatingVotes.RatingActionID)');
			$ratingActions->addJoinCondition('LEFT', 'ProductReviews', 'ON(RatingActions.RatingActionID = ProductReviews.RatingActionID)');
			$ratingActions->addJoinCondition('LEFT', 'Customers', 'ON(RatingActions.CustomerID = Customers.CustomerID)');
			$ratingActions->addJoinCondition('LEFT', 'ProductsML', 'ON((ProductReviews.ProductID = ProductsML.ProductID OR ProductRatingVotes.ProductID = ProductsML.ProductID) AND ProductsML.LanguageID = ' . scMultiLanguage::getCurrentLanguageId() . ')');
			$ratingActions->setSelectFields(array(
				'RatingActions.RatingActionID AS RatingActionID', 'RatingActionInputName', 'RatingActionTimestamp', 'RatingActionIPAddress',
				'ProductRatingVoteStatus', 'ProductRatingVoteValue',
				'ProductReviewStatus', 'ProductReviewText',
				'ProductName', 'ProductsML.ProductID',
				'RatingActions.CustomerID',
				'(IF(ProductReviewStatus IS NOT NULL, ProductReviewStatus, IF(ProductRatingVoteStatus IS NOT NULL, ProductRatingVoteStatus, 0))) AS RatingActionStatus'));
			$ratingActions->setHavingFilterNames(array('RatingActionStatus'));
			$table = new scAdminRatingActionsTable($ratingActions);
			$moduleParams = new miTableCustomParams($table);
			$moduleParams->setRecordLocatorParams(array('ModuleName' => 'com.summercart.ratingactions'));
			
			$view = new scAdminRatingActionsView();
			$view->setTable($table);
			
			return $view;
		}
	}
	
	class scAdminRatingActionsTable extends scTable
	{
		const MAX_REVIEW_CHARS = 45;
		
		protected function addUIElements(&$rows)
		{
			parent::addUiElements($rows);
			
			$ratingsValues = scRatingVotes::getRatingVotesChoices();
			foreach ($rows as &$row) {
				$customerId = (int)$row['CustomerID'];
				if ($customerId) {
					$row['HTML_RatingActionInputName'] = '<a href="customers.php?action=dmEdit&amp;CustomerID=' . 
						$customerId . '">' . miI18N::htmlEscape($row['RatingActionInputName']) . '</a>';
				} else {
					$row['HTML_RatingActionInputName'] = miI18N::htmlEscape($row['RatingActionInputName']);
				}
				
				$row['RatingActionTimestamp'] = date('d M Y', $row['RatingActionTimestamp']);
				$row['ProductReviewText'] = scUtf8::truncate($row['ProductReviewText'], 0, self::MAX_REVIEW_CHARS);
				$row['RatingActionIPAddress'] = long2ip($row['RatingActionIPAddress']);
				$ratingActionStatus = scRatingActionsStatuses::getCombinedRatingActionStatus($row['ProductReviewStatus'], $row['ProductRatingVoteStatus']);
				$row['RatingActionStatusIconName'] = scRatingActionsStatuses::getIconNameByStatus($ratingActionStatus);
				$row['RatingActionStatusDisapproved'] = scRatingActionsStatuses::RATING_ACTIONS_STATUS_DISAPPROVED;
				$row['RatingActionStatusApproved'] = scRatingActionsStatuses::RATING_ACTIONS_STATUS_APPROVED;
				if (isset($row['ProductRatingVoteValue']))
					$row['ProductRatingVoteValue'] = $ratingsValues[$row['ProductRatingVoteValue']];

				switch ($ratingActionStatus) {
					case scRatingActionsStatuses::RATING_ACTIONS_STATUS_APPROVED:
						$row['RatingActionDisapproveLinkText'] = _t('No');
						$row['RatingActionApproveLinkText'] = '';
						break;
					case scRatingActionsStatuses::RATING_ACTIONS_STATUS_DISAPPROVED:
						$row['RatingActionDisapproveLinkText'] = '';
						$row['RatingActionApproveLinkText'] = _t('Yes');
						break; 
					default:
						$row['RatingActionDisapproveLinkText'] = _t('No');
						$row['RatingActionApproveLinkText'] = _t('Yes');
						break;
				}
			}
		}
		
		/**
		 * Returns written status text for specified status value
		 *
		 * @param int $status
		 * @return string
		 */
		protected function getStatus($status)
		{
			$statuses = scRatingActionsStatuses::getRatingActionStatuses();
			return strtolower($statuses[$status]);
		}
	}
	
	/**
	 * Filters by IP address when it is represented by INT in the database
	 */
	class scIpFilter extends miSqlFilterEqual
	{
		public function getSql()
		{
			if ($this->_value == '')
				return '';
			
			$ipAddress = sprintf('%u', ip2long($this->_value));
			if ($ipAddress)
				return 'RatingActionIPAddress=' . $ipAddress; 
			
			return '';
		}
	}
	
	/**
	 * Filter for the RatingActionStatus field (non-existent in the database)
	 */
	class scRatingActionStatusFilter extends miSqlFilterOneValue
	{
		public function getSql()
		{
			$status = $this->_value;
			$fieldNames = array('ProductReviewStatus', 'ProductRatingVoteStatus');
			$sql = array();
			
			if (isset($status) && ($status !== '')) {
				foreach ($fieldNames as $fieldName) {
					$sql[] = $fieldName . ' = "' . sql_escape_string($status) . '" ';
				}
			}
			
			if (count($sql)) {
				return implode(' OR ', $sql);
			}
			
			return '';
		}
	}
	
	class scAdminRatingActionsView extends scView
	{
		protected $_templates = array(
			self::TEMPLATE_ID_LIST => 'modules/system/rating_actions/admin/rating_actions_mgmt.tmpl',
			self::TEMPLATE_ID_EDIT => 'modules/system/rating_actions/admin/rating_actions_edit.tmpl',
		);
		
		protected $_dataFields = array(
			array (
				'field' => 'scRatingActionTimestampWidget',
				'data' => 'RatingActionTimestamp',
				'properties' => array('readOnly' => true),
				'displayName' => 'Date',
			),
			array(
				'field' => 'scIPAddressWidget',
				'data' => 'RatingActionIPAddress',
				'properties' => array('readOnly' => true),
				'displayName' => 'IP Address',
			),
			array(
				'field' => 'scTextAreaProductReviewTextWidget',
				'data' => 'ProductReviewText',
				'modelStrategy' => 'scProductReviewsViewMapper',
				'displayName' => 'Review Content',
			),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductReviewStatus',
				'modelStrategy' => 'scProductReviewsViewMapper',
			),
			array(
				'field' => 'scSelectProductRatingVoteWidget',
				'data' => 'ProductRatingVoteValue',
				'modelStrategy' => 'scProductRatingVotesViewMapper',
				'displayName' => 'Rating Value',
			),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'ProductRatingVoteStatus',
				'modelStrategy' => 'scProductRatingVotesViewMapper',
			),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'RatingActionID',
				'properties' => array('readOnly' => true),
			),
			array(
				'field' => 'miBaseTextWidget',
				'data' => 'RatingActionInputName',
				'displayName' => 'Customer'
			),
			array(
				'field' => 'miSelectWidget',
				'data' => 'RatingActionStatus',
				'modelStrategy' => 'scRatingActionStatusViewMapper',
				'displayName' => 'Rating Action Status',
			),
		);
		
		protected $_domainObjectDisplayName = 'Rating / review';	// _t('Rating / review');
		
		public function __construct()
		{
			$this->_persistVars[] = 'ModuleName';
			
			parent::__construct();
		}
		
		public function initTable($table)
		{
			// Set default filter value for status: "Waiting for approval"
			$this->_defaultFilterValues = array_merge(array('RatingActionStatus' => scRatingActionsStatuses::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL, 'ProductName' => ''), $this->_defaultFilterValues);
			
			parent::initTable($table);
			
			$ratingStatusesOptions = scRatingActionsStatuses::getRatingActionStatuses();
			
			$filterObj = $this->getTableFilterObj();
			$filterObj->setConditionHandler('RatingActionIPAddressCustomFilter', 'scIpFilter');
			$filterObj->setConditionHandler('RatingActionStatusFilter', 'scRatingActionStatusFilter');
			
			scUIUtil::createDropdownFilter($this, $table, 'RatingActionStatus', array('' => _t('All statuses')) +  $ratingStatusesOptions, 'RatingActionStatusFilter', scRatingActionsStatuses::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL);
			scUIUtil::createDropdownFilter($this, $table, 'ProductRatingVoteValue', array('' => _t('All ratings')) +  scRatingVotes::getRatingVotesChoices(), '=', '');
			new scTableRecordsPerPage($table, $this->getTablePagerObj());
		}
		
		protected function createDomainObject()
		{
			return new scRatingActionDomainObject();
		}
	}
	
	class scEditRatingAction extends miEditAction
	{
		public function doAction()
		{
			$this->getWebForm()->getWidget('RatingActionStatus')->setOptions(scRatingActionsStatuses::getRatingActionStatuses());
			$this->getWebForm()->getWidget('ProductRatingVoteValue')->setOptions(scRatingVotes::getRatingVotesChoices());
			parent::doAction();
		}
	}
	
	class scChangeRatingActionStatus extends miAction
	{
		protected $_fieldName;
		
		/**
		 * Sets the field name
		 *
		 * @access public
		 * @param int $field current field name
		 */
		public function setFieldName($field)
		{
			$this->_fieldName = $field;
		}
		
		public function doAction()
		{
			try {
				$ratingActionsDomainObject = $this->_view->getDomainObject();
				
				$ratingActionId = miGetParam($ratingActionsDomainObject->getPKName()); 
				$newRatingActionStatus = miGetParam('RatingActionStatus');
				
				$ratingActionsDomainObject->read($ratingActionId);
				
				$productReviewsDomainObject = new scProductReviewDomainObject();
				if ($productReviewsDomainObject->readByRatingActionId($ratingActionId)) {
					$productReviewsDomainObject->updateStatus($newRatingActionStatus);
				}
				
				$productRatingVotesDomainObject = new scProductRatingVotesDomainObject();
				if ($productRatingVotesDomainObject->readByRatingActionId($ratingActionId)) {
					$productRatingVotesDomainObject->updateStatus($newRatingActionStatus);
				}
			} catch (miException $exception) {
				$msg = $this->_view->getMessage('MI_RECORD_UPDATE_FAILED_MSG');
				$this->_view->addRedirectToListControllerCommand($msg . $exception->getMessage(), miMessage::MSG_TYPE_ERROR);
				return false;
			}
			$msg = sprintf($this->_view->getMessage('MI_RECORD_UPDATED_SUCCESSFULLY_MSG'));
			$this->_view->addRedirectToListControllerCommand($msg, miMessage::MSG_TYPE_INFO);
			return true;
		}
	}
?>