<?php
	/**
	 * Copyright (c) 2009 Mirchev Ideas Ltd. All rights reserved.
	 */

	require_once(dirname(__FILE__) . '/../RatingActionConsts.php');
	require_once(dirname(__FILE__) . '/../RatingActionsUtils.php');
	
	class scRatingActionDomainObject extends miDefaultDomainObject
	{
		public function __construct()
		{
			parent::__construct(new miSqlRecord('RatingActions', 'RatingActionID'));
		}
		
		public function insert()
		{
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			
			$this->set('RatingActionIPAddress', ip2long($_SERVER['REMOTE_ADDR']));
			$this->set('RatingActionTimestamp', time());
			$this->set('CustomerID', ($customerId === false ? null : $customerId));
			
			return parent::insert();
		}
	}
	
	abstract class scProductRatingActionsDomainObject extends miDefaultDomainObject
	{
		abstract protected function setRecord();
		
		protected $_statusColumnName;
		
		public function __construct()
		{
			$this->setRecord();
			parent::__construct($this->_record);
		}
		
		public function insert()
		{
			$statusToSet = scRatingActionsStatuses::RATING_ACTIONS_STATUS_WAITING_FOR_APPROVAL;
			if (scRatingActionsUtils::isApprovedAutomatically(scCustomerAuth::singleton()->getCustomerId()))
				$statusToSet = scRatingActionsStatuses::RATING_ACTIONS_STATUS_APPROVED;
			
			$this->set($this->_statusColumnName, $statusToSet);
			return parent::insert();
		}
		
		protected function assignBonusPoints($reason, $pointsAssignmentDescription)
		{
			if (!scCustomerAuth::singleton()->isLoggedIn())
				return;
			
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			if ($customerId === ICheckoutModule::GUEST_USER_ID)
				return;
			
			scBonusPoints::assignBonusPoints($customerId, $reason, $pointsAssignmentDescription, $this->get($this->getPKName()));
		}
	}
	
	class scProductReviewDomainObject extends scProductRatingActionsDomainObject
	{
		protected $_statusColumnName = 'ProductReviewStatus';
		
		protected function setRecord()
		{
			$this->_record = new miSqlRecord('ProductReviews', 'ProductReviewID');
		}
		
		public function insert()
		{
			$productReviewId = parent::insert();
			
			$product = new scProductSL();
			$product->read($this->get('ProductID'));
			$pointsAssignmentDescription = sprintf(_t('Customer reviewed product "%s".'), $product->getName());
			
			$this->assignBonusPoints(scBonusPoints::ASSIGNMENT_REASON_REVIEW, $pointsAssignmentDescription);
			
			return $productReviewId;
		}
	}
	
	class scProductRatingVoteDomainObject extends scProductRatingActionsDomainObject
	{
		protected $_statusColumnName = 'ProductRatingVoteStatus';
		
		protected function setRecord()
		{
			$this->_record = new miSqlRecord('ProductRatingVotes', 'ProductRatingVoteID');
		}
		
		public function insert()
		{
			$productReviewId = parent::insert();
			
			$product = new scProductSL();
			$product->read($this->get('ProductID'));
			$pointsAssignmentDescription = sprintf(_t('Customer rated product "%s".'), $product->getName());
			
			$this->assignBonusPoints(scBonusPoints::ASSIGNMENT_REASON_VOTE, $pointsAssignmentDescription);
			
			return $productReviewId;
		}
	}

	class scCustomerRatingActionsModule implements IUserModule, IProductDetails 
	{
		
		/**
		 * Initializes the action handlers for this module
		 * 
		 * @param miPage $page
		 */
		public function initActionHandlers(miPage $page)
		{
			$page->getDispatcher()->clearActionHandlers();
			$page->getDispatcher()->setActionHandler('acRate', 'miCreateAction');
			$page->getDispatcher()->setActionHandler('acExecRate', 'scCustomerRatingActionExecCreateAction');
		}
		
		/**
		 * Returns the view that should be displayed
		 * 
		 * @return miDefaultView
		 */
		public function getView()
		{
			return new scCustomerRatingActionView();
		}
		
		public function getProductDetailsHtml(scProductSL $product, miTemplateParserSectionInfo $productSection)
		{
			scDateTime::setTimezone();
			
			$productId = $product->getProductId();
			
			$productReviews = $this->getProductReviews($productId);
			$productsReviewsAndRatesSections = array();
			foreach($productReviews as &$productReview) {
				// If the user has rated as well as wrote review - show the rating.
				if (isset($productReview['ProductRatingVoteValue']))
					$productsReviewsAndRatesSections[] = new miTemplateParserSectionInfo('customerRated', 1, miUIUtil::templetizeRows(miUIUtil::transposeRows(array(0 => array('RatingIconNumber' => sprintf('%02d', (2 * $productReview['ProductRatingVoteValue'])))))));
				else
					$productsReviewsAndRatesSections[] = new miTemplateParserSectionInfo('customerRated', 0, array());
				
				$productReview['RatingActionTime'] = scDateTime::getLocalFormattedTimestamp($productReview['RatingActionTimestamp']);
				$productReview['RatingActionDate'] = scDateTime::getLocalFormattedTimestamp($productReview['RatingActionTimestamp'], true);
			}
			
			$productReviewsCount = count($productReviews);
			$productReviewsSection = new miTemplateParserSectionInfo('productReviews', $productReviewsCount, miUIUtil::templetizeRows(miUIUtil::transposeRows($productReviews)));
			$productReviewsSection->addSubsectionsArray('customerRated', $productsReviewsAndRatesSections); 
			
			$canRateOrReview = new miTemplateParserSectionInfo('productMayBeRatedAndOrReviewed', 0);
			if (scRatingActionsUtils::getAllowRateSetting() != scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_RATE_NONE)
				$canRateOrReview->setIterationsCount(1);
			else if (scRatingActionsUtils::getAllowReviewSetting() != scRatingActionsSettings::RATING_ACTION_SETTING_ALLOW_REVIEW_NONE)
				$canRateOrReview->setIterationsCount(1);
			$productSection->addSubsection($canRateOrReview);
			
			$productHasReviews = new miTemplateParserSectionInfo('productHasReviews', $productReviewsCount ? 1 : 0);
			$productHasReviews->addSubsection($productReviewsSection);
			$productSection->addSubsection($productHasReviews);
			
			list($productRatingAverage, $numVotes) = $this->getProductRatingAverage($productId);
			$ratingIconNumber = $this->getProductRatingIconNumber($productRatingAverage);
			$showRatingValue = ($productRatingAverage > 0) ? 1 : 0;
			$productSection->addSubsection(new miTemplateParserSectionInfo('productHasRatingValue', $showRatingValue,
				array('%%RATING_ICON_NUMBER%%' => $ratingIconNumber, '%%RATING_AVERAGE%%' => $productRatingAverage, '%%RATING_NUM_VOTES%%' => $numVotes)));
			
			return '';
		}
		
		protected function getProductReviews($productId)
		{
			$productReviewsRs = new miSqlRecordset('ProductReviews');
			$productReviewsRs->addFilter(new miSqlFilterEqual('ProductReviews.ProductID', $productId));
			$productReviewsRs->addFilter(new miSqlFilterEqual('ProductReviewStatus', scRatingActionsStatuses::RATING_ACTIONS_STATUS_APPROVED));
			$productReviewsRs->addJoinCondition('INNER', 'RatingActions', 'USING(RatingActionID)');
			$productReviewsRs->addJoinCondition('LEFT', 'ProductRatingVotes', 'USING(RatingActionID)');
			$productReviewsRs->setOrder('RatingActionTimestamp', 'DESC');
			
			return $productReviewsRs->getAllRecords();
		}
		
		/**
		 * Returns the average product rating
		 * 
		 * @param int $productId
		 * @return float
		 */
		protected function getProductRatingAverage($productId)
		{
			$sql = 'SELECT AVG(ProductRatingVoteValue) AS AverageRating, COUNT(*) AS NumVotes FROM ProductRatingVotes WHERE ProductID = ? AND ProductRatingVoteStatus = ?';
			$result = miStaticDBUtil::execSelect($sql, array($productId, scRatingActionsStatuses::RATING_ACTIONS_STATUS_APPROVED));
			return array($result[0]['AverageRating'], $result[0]['NumVotes']);
		}
		
		/**
		 * Returns the icon number - from 00 to 10
		 * 
		 * @param float $ratingValue
		 * @return string
		 */
		protected function getProductRatingIconNumber($ratingValue)
		{
			$iconNumber = intval(($ratingValue * 2) + 0.5);
			if ($iconNumber < 0)
				$iconNumber = 0;
			if ($iconNumber > 10)
				$iconNumber = 10;
			
			return sprintf('%02d', $iconNumber);
		}
	}
	
	class scCustomerRatingActionView extends scView
	{
		protected $_templates = array(
			self::TEMPLATE_ID_CREATE => 'modules/system/rating_actions/customer/rating_actions_create.tmpl',
		);
		
		public function createDomainObject()
		{
			return new scRatingActionDomainObject();
		}
		
		public function initWebForm(&$form)
		{
			if (!miHasParam('load'))
				miSetParam('load', 'table');

			$productId = miGetParam('ProductID');
			
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			
			$allowedRatingActions = scRatingActionsUtils::getAllowedRatingActionsForCustomer($customerId, $productId);
			
			if (!$allowedRatingActions['hasEnabledRatingAction']) {
				$errorMessages = array(
					'%%ERRORMESSAGERATE%%' => $allowedRatingActions['rate']['error'],
					'%%ERRORMESSAGEREVIEW%%' => $allowedRatingActions['review']['error'],
				);
				$form->addTemplateSectionInfo(new miTemplateParserSectionInfo('noRatingActionsEnabled', 1, $errorMessages));
				return true;
			}
			
			$hasEnabledRatingActionsSection = new miTemplateParserSectionInfo('hasEnabledRatingAction', 1, array());
			
			foreach ($allowedRatingActions as $ratingActionName => $ratingActionValues) {
				if ($ratingActionValues['allowed'])
					$hasEnabledRatingActionsSection->addSubsection(new miTemplateParserSectionInfo($ratingActionName, 1, array()));
				elseif ($ratingActionValues['error'])
					$hasEnabledRatingActionsSection->addSubsection(new miTemplateParserSectionInfo($ratingActionName . 'ErrorMessage', 1, array('%%ERRORMESSAGE' . strtoupper($ratingActionName) . '%%' => miI18N::htmlEscape($ratingActionValues['error']))));
			}
			
			if (!$customerId) {
				$hasEnabledRatingActionsSection->addSubsection(new miTemplateParserSectionInfo('hasCustomerName', 1, array()));
			}
			
			$form->addTemplateSectionInfo($hasEnabledRatingActionsSection);
			
			$this->addMainPageElements(array("%%PRODUCTID%%" => $productId));
			
			$this->addMainPageElements(array(
				"%%PRODUCTRATINGVOTEVALUE%%" => miUIUtil::getConstDropdown('ProductRatingVoteValue', '', array('' => '-') + scRatingVotes::getRatingVotesChoices())
			));
			
			return parent::initWebForm($form);	
		}
	}
	
	class scCustomerRatingActionExecCreateAction extends miExecCreateAction
	{
		public function doAction()
		{
			$productId = (int)miGetParam('ProductID');
			
			$ratingVoteValue = (int)miGetParamDefault('ProductRatingVoteValue', '0');
			if ($ratingVoteValue != 0)
				$ratingVoteValue = min(scRatingVotes::RATING_MAX_VOTE, max(scRatingVotes::RATING_MIN_VOTE, $ratingVoteValue));
			
			$productReviewText = trim(miGetParamDefault('ProductReviewText', ''));
			$ratingActionInputName = trim(miGetParamDefault('RatingActionInputName', ''));
			
			$customerId = scCustomerAuth::singleton()->getCustomerId();
			$allowedRatingActions = scRatingActionsUtils::getAllowedRatingActionsForCustomer($customerId, $productId);
			
			// Check for missing data
			if ($allowedRatingActions['review']['allowed'] && $allowedRatingActions['rate']['allowed'] && ($productReviewText == '') && (!$ratingVoteValue)) {
				$this->returnActionResult('missingData', _t('Please select rating or write review.'));
			}
			if (!$allowedRatingActions['review']['allowed'] && $allowedRatingActions['rate']['allowed'] && (!$ratingVoteValue)) {
				$this->returnActionResult('missingData', _t('Please select rating.'));
			}
			if ($allowedRatingActions['review']['allowed'] && !$allowedRatingActions['rate']['allowed'] && ($productReviewText == '')) {
				$this->returnActionResult('missingData', _t('Please write review.'));
			}
			
			try {
				$ratingActionDomainObj = $this->_view->getDomainObject();
				if ($ratingActionInputName == '') {
					if ($customerId) 
						$ratingActionInputName = scCustomerAuth::singleton()->getCustomerFirstName() . ' ' . scCustomerAuth::singleton()->getCustomerLastName();
					else
						$ratingActionInputName = _t('Guest');
				}
				$ratingActionDomainObj->set('RatingActionInputName', $ratingActionInputName);
				$ratingActionId = $ratingActionDomainObj->insert();
				
				if (($productReviewText != '') && $allowedRatingActions['review']['allowed']) {
					$productReview = new scProductReviewDomainObject();
					$productReview->set('ProductID', $productId);
					$productReview->set('RatingActionID', $ratingActionId);
					$productReview->set('ProductReviewText', $productReviewText);
					$productReview->insert();
				}
				
				if (($ratingVoteValue != 0) && $allowedRatingActions['rate']['allowed']) {
					$productRatingVote = new scProductRatingVoteDomainObject();
					$productRatingVote->set('ProductID', $productId);
					$productRatingVote->set('RatingActionID', $ratingActionId);
					$productRatingVote->set('ProductRatingVoteValue', $ratingVoteValue);
					$productRatingVote->insert();
				}
			} catch (miException $exception) {
				return $this->onFailure($exception);
			}
			
			return $this->onSuccess();
		}
		
		protected function onFailure(miException $exception = null)
		{
			$this->returnActionResult('error', $exception->getMessage());
		}
		
		protected function onSuccess($viewMapperControllerResult = null)
		{
			$this->returnActionResult('success', _t('Thank you for your feedback.'));
		}
		
		protected function returnActionResult($resultType, $message)
		{
			echo scJson::jsonEncode(array('type' => $resultType, 'message' => $message));
			exit;
		}
	}
?>